/***************************************************************
 * Name:      JaffisMain.cpp
 * Purpose:   Code for Application Frame
 * Author:    John Q Metro (johnqmetro@hotmail.com)
 * Created:   2009-03-23
 * Updated:   July 23, 2015
 * Copyright: John Q Metro ()
 * License:
 **************************************************************/

#include "JaffisMain.h"
#include <wx/msgdlg.h>

//(*InternalHeaders(JaffisFrame)
#include <wx/intl.h>
#include <wx/string.h>
//*)
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef UTILS2_H_INCLUDED
  #include "core/utils/utils2.h"
#endif // UTILS2_H_INCLUDED
#ifndef GLOBAL_SKEL_H_INCLUDED
  #include "interface/misc/global_skel.h"
#endif // GLOBAL_SKEL_H_INCLUDED
#ifndef SKELCOLL_H_INCLUDED
  #include "core/structs/skelcoll.h"
#endif // SKELCOLL_H_INCLUDED
#ifndef SPECIALFUNCS_H_INCLUDED
  #include "core/utils/specialfuncs.h"
#endif // SPECIALFUNCS_H_INCLUDED
#ifndef GOOGLEINFO_H_INCLUDED
  #include "google/data/googleinfo.h"
#endif // GOOGLEINFO_H_INCLUDED
#ifndef FFNDOWDISPLAY
  #include "fanficnet/categories/ffn_downdisplay.h"
#endif
#ifndef JF_FFNSEARCH
  #include "fanficnet/data/ffn_search.h"
#endif
#ifndef LOGGING_H_INCLUDED
  #include "core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef INITEND_H_INCLUDED
  #include "initend.h"
#endif // INITEND_H_INCLUDED
#ifndef JFILTERGLOBALS
  #include "core/filters/base/filter_globals.h"
#endif
#ifndef RES_BUNDLE_H_INCLUDED
  #include "ffnupdate/data/res_bundle.h"
#endif // RES_BUNDLE_H_INCLUDED
#ifndef UPD_DISP_H_INCLUDED
  #include "ffnupdate/interface/upd_disp.h"
#endif // UPD_DISP_H_INCLUDED
#ifndef FFNDOWN_DIALOG_H_INCLUDED
  #include "ficdown/interface/ffndown_dialog.h"
#endif // FFNDOWN_DIALOG_H_INCLUDED
#ifndef DEFAULTDIR_DIALOG_H_INCLUDED
  #include "interface/misc/defaultdir_dialog.h"
#endif // DEFAULTDIR_DIALOG_H_INCLUDED
#ifndef FFNDUPD_DIALOG_H_INCLUDED
  #include "ficdown/interface/ffndupd_dialog.h"
#endif // FFNDUPD_DIALOG_H_INCLUDED
#ifndef FIM_MAINPANEL1_H_INCLUDED
  #include "fanfic/interface/fim/fim_mainpanel1.h"
#endif // FIM_MAINPANEL1_H_INCLUDED
#ifndef AO3_CATDIALOG_H_INCLUDED
  #include "fanfic/interface/ao3/ao3_catdialog.h"
#endif // AO3_CATDIALOG_H_INCLUDED
#ifndef AO3_MAINPANEL_H_INCLUDED
  #include "fanfic/interface/ao3/ao3_mainpanel.h"
#endif // AO3_MAINPANEL_H_INCLUDED
#ifndef FIMGROUP_MAINPANEL1_H_INCLUDED
  #include "fanfic/interface/fim/fimgroup_mainpanel1.h"
#endif // FIMGRIUP_MAINPANEL1_H_INCLUDED
#ifndef DIRUPDATE_DIALOG_H_INCLUDED
  #include "interface/misc/dirupdate_dialog.h"
#endif // DIRUPDATE_DIALOG_H_INCLUDED
//===============================================================
//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << wxT("-Windows");
#elif defined(__UNIX__)
        wxbuild << wxT("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << wxT("-Unicode build");
#else
        wxbuild << wxT("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}
//==============================================================================
//(*IdInit(JaffisFrame)
const long JaffisFrame::idMenuQuit = wxNewId();
const long JaffisFrame::idMenuAbout = wxNewId();
const long JaffisFrame::ID_STATUSBAR1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(JaffisFrame,wxFrame)
    //(*EventTable(JaffisFrame)
    //*)
END_EVENT_TABLE()
//====================================================================================
JaffisFrame::JaffisFrame(wxWindow* parent,wxWindowID id)
{
    wxMenu* HelpMenu;
    wxMenuItem* AboutItemM;

    mainarranger = NULL;
    // two step construction
    Create(parent, id,wxT("Jaffis"),wxPoint(50,50),wxSize(1000,750), wxDEFAULT_FRAME_STYLE, wxT("id"));
    MainMenu = NULL;
    // creating the main menu
    MainMenu = new wxMenuBar();
    // the file menu
    SetUpFileMenu();
    // the tools menu
    SetUpToolsMenu();
    // other stuff
    HelpMenu = new wxMenu();
    AboutItemM = new wxMenuItem(HelpMenu, idMenuAbout, _("About\tF1"), _("Show info about this application"), wxITEM_NORMAL);
    HelpMenu->Append(AboutItemM);
    MainMenu->Append(HelpMenu, _("Help"));
    SetMenuBar(MainMenu);
    // status bar setup
    StatusBar1 = new wxStatusBar(this, ID_STATUSBAR1, 0, wxT("ID_STATUSBAR1"));
    int __wxStatusBarWidths_1[4] = { -1,-1,-1,-1, };
    int __wxStatusBarStyles_1[4] = { wxSB_NORMAL,wxSB_NORMAL,wxSB_NORMAL,wxSB_NORMAL };
    StatusBar1->SetFieldsCount(4,__wxStatusBarWidths_1);
    StatusBar1->SetStatusStyles(4,__wxStatusBarStyles_1);
    SetStatusBar(StatusBar1);
    // connecting...
    Connect(idMenuAbout,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnAbout);
    Connect(jfID_TOPNOTEBOOK,wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED,(wxObjectEventFunction)&JaffisFrame::NotebookTabChanged);
    // connecting some more
    Connect(jf_MSG_SRCHNDONE_EVENT,(wxObjectEventFunction)&JaffisFrame::OnSearchStarted);
    Connect(jf_MSG_SRCHDONE_EVENT,(wxObjectEventFunction)&JaffisFrame::OnSearchDone);
    Connect(jf_UPDATE_ALLDONE_EVENT,(wxObjectEventFunction)&JaffisFrame::OnUpdateDone);
    Connect(wxEVT_SIZE,(wxObjectEventFunction)&JaffisFrame::onResize);
    Connect(jf_FFNCDS_SIGNAL_DONE,(wxObjectEventFunction)&JaffisFrame::HandleFFNCatDownloaded);
    Connect(jf_PP_SRCHFAIL_EVENT,(wxObjectEventFunction)&JaffisFrame::HandleSearchFail);
    Connect(jf_DOWNL_DATATOTAL_EVENT,(wxObjectEventFunction)&JaffisFrame::HandleDownloadTotal);
    // finishing gui arrangement
    topsizer = new wxBoxSizer(wxVERTICAL);
    #ifdef __WXGTK__
      mainarranger = new wxNotebook(this,jfID_TOPNOTEBOOK,wxDefaultPosition,wxDefaultSize,wxNB_BOTTOM);
    #else
      mainarranger = new wxNotebook(this,jfID_TOPNOTEBOOK,wxDefaultPosition,wxDefaultSize,wxNB_LEFT);
    #endif
    topsizer->Add(mainarranger,1,wxEXPAND);
    SetSizer(topsizer);
    // some final touches
    closestop = false;
    wxString datasizestr = ByteToSize(globalsettings::totaldown);
    StatusBar1->SetStatusText(datasizestr,3);

    // last stuff
    MaximizeTest();
}
//===========================================================================================
JaffisFrame::~JaffisFrame()
{
    //(*Destroy(JaffisFrame)
    //*)
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void JaffisFrame::OnQuit(wxCommandEvent& event) {
	// constants
	const wxString fname = wxT("JaffisFrame::OnQuit");
	const wxString wmsg = wxT("Please wait while information is saved. This could take a few minutes.");
	const wxString cmsg = wxT("The Application is closing");
	// variables
  bool wafr;
	jfTextStatusDialog* text_msg;
	wxSize dsize(400,80);
	// starting
	/**/jfXLogMessage(fname,1);
  wxBeginBusyCursor();
	text_msg = new jfTextStatusDialog(this, -1, cmsg, wmsg, dsize);
	text_msg->Show(true);
  wxYieldIfNeeded();
  wxMilliSleep(400);
  wxYieldIfNeeded();
  wxMilliSleep(200);
  // writing
  wafr = WriteAtFinish();
	/**/jfXLogMessageB(fname,2,wafr);
  assert(wafr);
	text_msg->Close();
  wxEndBusyCursor();
  Close();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void JaffisFrame::OnAbout(wxCommandEvent& event)
{
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _("Welcome to..."));
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void JaffisFrame::OnEditGlobalFilters(wxCommandEvent& event) {
  // creating the filter editing panel
  jfGlobalFilterEditor* filter_edit = new jfGlobalFilterEditor(mainarranger,-1,filterglobals::global_filters);
  // adding it to the notebook and tracker
  ptracker.AddPage(filter_edit->GetId(),jftl_GLOBFILEDIT);
  mainarranger->AddPage(filter_edit,wxT("Global Filters Editor"),true);
  // enabling the close page menu item
  (MainMenu->GetMenu(0))->Enable(jfID_MENU_CLOSECURR,true);
  // disabling the global filters edit item, since there is no point in editing twice
  (MainMenu->GetMenu(1))->Enable(jfID_MENU_GLOBFILTER,false);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void JaffisFrame::OnEditGlobalSkeletons(wxCommandEvent& event) {
    // creating the skeleton editing panel
  jfGlobalSkeletonEdit* skel_edit = new jfGlobalSkeletonEdit(mainarranger,-1,mainskelc);
  // adding it to the notebook and tracker
  ptracker.AddPage(skel_edit->GetId(),jftl_SKELEDIT);
  mainarranger->AddPage(skel_edit,wxT("Global Skeleton Editor"),true);
  // enabling the close page menu item
  (MainMenu->GetMenu(0))->Enable(jfID_MENU_CLOSECURR,true);
  // disabling the global filters edit item, since there is no point in editing twice
  (MainMenu->GetMenu(1))->Enable(jfID_MENU_GLOBSKEL,false);
}
//---------------------------------------------------------------------
void JaffisFrame::OnDefaultDirs(wxCommandEvent& event) {
  jfDefaultDirsDialog* diredit;
  diredit = new jfDefaultDirsDialog(this,-1);
  diredit->Show();
}
//-------------------------------------------------------------------
void JaffisFrame::OnDirUpdate(wxCommandEvent& event) {
  jfUpdateDirectory_Dialog* dirup_dialog;
  dirup_dialog = new jfUpdateDirectory_Dialog(this,-1,wxT("Directory Sync/Update Tool"));
  dirup_dialog->Show();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void JaffisFrame::OnDownloadFFNCats(wxCommandEvent& event) {
  jfFFNCatDownloadStatus* downloader;
  downloader = new jfFFNCatDownloadStatus(this,jfID_FFNCDOWN_PAGE,ffn_catorganizer::main_manager,wxDefaultPosition,wxDefaultSize);
  downloader->Show();
}
//-------------------------------------------------------------------------
void JaffisFrame::OnDownloadAO3Cats(wxCommandEvent& event) {
  jfAO3Cat_DownDialog* downloader;
  downloader = new jfAO3Cat_DownDialog(this,jfID_AO3_CATDOWNL,wxDefaultPosition,wxDefaultSize);
  downloader->Show();
}
//--------------------------------------------------------------------------
void JaffisFrame::OnDownloadFFNFic(wxCommandEvent& event) {
  jfDown_Dialog2* downloader;
  downloader = new jfDown_Dialog2(this,jfID_FFN_FICDOWNT,wxT("Download a Fic"));
  downloader->Show();
}
//--------------------------------------------------------------------
void JaffisFrame::OnUpdateFFNFic(wxCommandEvent& event) {
  // creating the updater panel
  jfFic_UpdateDisplay* fupdater = new jfFic_UpdateDisplay(mainarranger,-1);
  // adding it to the notebook and tracker
  ptracker.AddPage(fupdater->GetId(),jftl_FICUPDATE);
  mainarranger->AddPage(fupdater,wxT("Fanfics Updater"),true);
  // enabling the close page menu item
  (MainMenu->GetMenu(0))->Enable(jfID_MENU_CLOSECURR,true);
}
//-----------------------------------------------------------------------
void JaffisFrame::OnSearchFIMGroups(wxCommandEvent& event) {
  jfFIMGroupMain* fimg_edit = new jfFIMGroupMain(mainarranger,-1,NULL);
  wxString tabname = wxT("FIM Group Search ") + SInt2Str(mainarranger->GetPageCount());
  PageLoad(fimg_edit,jftl_FIMGSEARCH,tabname);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void JaffisFrame::OnNewGoogleSearch(wxCommandEvent& event) {
  // creating the page
  jfGoogleMain* google_edit = new jfGoogleMain(mainarranger,-1,NULL);
  wxString tabname = wxT("Google Search ") + SInt2Str(mainarranger->GetPageCount());
  PageLoad(google_edit,jftl_GOOGSEARCH,tabname);
}
//--------------------------------------------------------------------
void JaffisFrame::OnNewFFNSearch(wxCommandEvent& event) {
  // creating the page
  jfFFNMain* ffn_edit = new jfFFNMain(mainarranger,-1,NULL);
  // setting the link with the category data
  jfFFNSearch* ffns_temp = dynamic_cast<jfFFNSearch*>(ffn_edit->GetSearchPointer());
  wxInt32 rloc = ffn_catorganizer::main_manager->RegisterSearch(ffn_edit);
  assert(rloc>=0);
  ffns_temp->SetData(ffn_catorganizer::main_manager->GetHolder(),rloc);
  // adding it to the notebook and tracker
  wxString tabname = wxT("FFN Search ") + SInt2Str(mainarranger->GetPageCount());
  PageLoad(ffn_edit,jftl_FFNSEARCH,tabname);
}
//--------------------------------------------------------------------
void JaffisFrame::OnNewFIMSearch(wxCommandEvent& event) {
  // creating the page
  jfFIMMain* fim_edit = new jfFIMMain(mainarranger,-1,NULL);
  // adding it to the notebook and tracker
  wxString tabname = wxT("FIM Search ") + SInt2Str(mainarranger->GetPageCount());
  PageLoad(fim_edit,jftl_FIMSEARCH,tabname);
}
//----------------------------------------------------------------------
void JaffisFrame::OnNewAO3Search(wxCommandEvent& event) {
  // creating the page
  jfAO3Main* ao3_edit = new jfAO3Main(mainarranger,-1,NULL);
  // adding it to the notebook and tracker
  wxString tabname = wxT("AO3 Search ") + SInt2Str(mainarranger->GetPageCount());
  PageLoad(ao3_edit,jftl_AO3SEARCH,tabname);
}
//--------------------------------------------------------------------------
void JaffisFrame::OnCloseTab(wxCommandEvent& event) {
  // local variables
  jfMainSearchGroup* qval = NULL;
  jfUpdateTopLevel* rval = NULL;
  jfFFNMain* zval = NULL;
  jfFic_UpdateDisplay* sval = NULL;
  // getting the selected tab
  int sindex = mainarranger->GetSelection();
  assert(sindex!=-1);
  wxBeginBusyCursor();
  // checking to see if the item we abe about to delete is the Global Filter Editor
  bool rgfilt = ptracker.IsGlobEdit(sindex);
  bool rgskel = ptracker.IsGlobSkel(sindex);
  // checking for stopping a search
  if (ptracker.FromMSG(sindex)) {
     qval = dynamic_cast<jfMainSearchGroup*>(mainarranger->GetCurrentPage());
     closestop = qval->CStop();
  }
  else if (ptracker.IsUpdater(sindex)) {
    rval = dynamic_cast<jfUpdateTopLevel*>(mainarranger->GetCurrentPage());
    closestop = rval->OrderStop();
  }
  else if (ptracker.IsFicUpdate(sindex)) {
    sval = dynamic_cast<jfFic_UpdateDisplay*>(mainarranger->GetCurrentPage());
    closestop = sval->StopUpdate();
  }

  // afterwards if closestop is true, the close is delayed until we get a signal...
  if (!closestop) {
    // what if we *have* removed the global filter editor?
    if (rgfilt) (MainMenu->GetMenu(1))->Enable(jfID_MENU_GLOBFILTER,true);
    // what if we *have* removed the global skeleton editor?
    if (rgskel) (MainMenu->GetMenu(1))->Enable(jfID_MENU_GLOBSKEL,true);
    // close now
    CloseSearch();
    wxEndBusyCursor();
  }

}
//--------------------------------------------------------------------------
// when we change a notebook tab, we much check for some menus
void JaffisFrame::NotebookTabChanged(wxNotebookEvent& event) {
  MenuEnabler();
}
//--------------------------------------------------------------------------
// a search has started, to html output for that page must be disabled
void JaffisFrame::OnSearchStarted(wxCommandEvent& event) {
  // getting the id of the page this event is for
  wxWindowID thepage = (wxWindowID)event.GetInt();
  // setting the html value
  ptracker.SetHtmlReady(thepage,false);
  // checking if the the index is the current page
  int sindex = mainarranger->GetSelection();
  assert(sindex>=0);
  int rindex = ptracker.GetWindowIndex(thepage);
  assert(sindex>=0);
  if (sindex==rindex) HtmlREnable(false);
  // done
}
//--------------------------------------------------------------------------
// a search has ended, so html output is available
void JaffisFrame::OnSearchDone(wxCommandEvent& event) {
  // getting the id of the page this event is for
  wxWindowID thepage = (wxWindowID)event.GetInt();
  // setting the html value
  ptracker.SetHtmlReady(thepage,true);
  // checking if the the index is the current page
  int sindex = mainarranger->GetSelection();
  assert(sindex>=0);
  int rindex = ptracker.GetWindowIndex(thepage);
  assert(sindex>=0);
  if (sindex==rindex) HtmlREnable(true);
  // done, unless we close
  if (closestop) {
    CloseSearch();
    wxEndBusyCursor();
  }
}
//-------------------------------------------------------------------------
void JaffisFrame::OnUpdateDone(wxCommandEvent& event) {
  const wxString fname = wxT("JaffisFrame::OnUpdateDone");
  // getting the id of the page this event is for
  wxWindowID thepage = (wxWindowID)event.GetInt();
  // setting the html value
  /**/jfXLogMessage(fname,1);
  ptracker.SetHtmlReady(thepage,true);
  // checking if the the index is the current page
  int sindex = mainarranger->GetSelection();
  assert(sindex>=0);
  /**/jfXLogMessageI(fname,2,sindex);
  int rindex = ptracker.GetWindowIndex(thepage);
  assert(rindex>=0);
  /**/jfXLogMessageI(fname,3,rindex);
  if (sindex==rindex) HtmlREnable(true);
  // done, unless we close
  if (closestop) {
    CloseSearch();
    /**/jfXLogMessage(fname,4);
    wxEndBusyCursor();
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void JaffisFrame::OnHtmlOutCurr(wxCommandEvent& event) {
  DisplayPanelCore* qval;
  qval = dynamic_cast<DisplayPanelCore*>(mainarranger->GetCurrentPage());
  qval->WriteCurrent2Html();
}
//--------------------------------------------------------------------------
void JaffisFrame::OnHtmlOutAll(wxCommandEvent& event) {
  DisplayPanelCore* qval;
  qval = dynamic_cast<DisplayPanelCore*>(mainarranger->GetCurrentPage());
  qval->WriteAll2Html();
}
//-----------------------------------------------------------------------
void JaffisFrame::OnSaveFile(wxCommandEvent& event) {
  // variables
  jfMainSearchGroup* qval;
  bool rvres;
  // getting the main page
  qval = dynamic_cast<jfMainSearchGroup*>(mainarranger->GetCurrentPage());
  // trying the save
  rvres = qval->SaveToFile();
  assert(rvres);
}
//-----------------------------------------------------------------------
void JaffisFrame::OnSaveAsFile(wxCommandEvent& event) {
  // the objects
  wxString xfilen;
  jfMainSearchGroup* qval;
  bool rvres;
  // doing the operation
  if (GetFileNameJSS(xfilen,true)) {
    qval = dynamic_cast<jfMainSearchGroup*>(mainarranger->GetCurrentPage());
    rvres = qval->SaveToFile(xfilen);
    assert(rvres);
  }
}
//--------------------------------------------------------------------------
void JaffisFrame::OnOpenFile(wxCommandEvent& event) {
  // local variables
	const wxChar* fname = wxT("JaffisFrame::OnOpenFile");
  wxString xfilen, typstr, outerr;
  jfSearchCore* loadres;
  jfGoogleMain* google_edit;
  jfFFNMain* ffn_edit;
  jfFIMMain* fim_edit;
  jfAO3Main* ao3_edit;
  jfFIMGroupMain* fimg_edit;
  wxString tabname;
  // we start with the filename
  if (GetFileNameJSS(xfilen,false)) {
    loadres = MakeSearchFromFile(xfilen,outerr);
    // the load suceeded...
    if (loadres!=NULL) {
      // however, we do have to check the type
      typstr = loadres->GetTypeID();
      if (typstr == wxT("GoogleSearch")) {
        // creating a Google Search tab
        google_edit = new jfGoogleMain(mainarranger,-1,loadres);
        tabname = wxT("Google Search: ") + google_edit->GetSearchName();
        PageLoad(google_edit,jftl_GOOGSEARCH,tabname);
      }
      else if (typstr == wxT("FFNSearch")) {
        // creating a FFNS Search tab
        ffn_edit = new jfFFNMain(mainarranger,-1,loadres);
        tabname = wxT("Fanfiction.Net Search: ") + ffn_edit->GetSearchName();
        PageLoad(ffn_edit,jftl_FFNSEARCH,tabname);
      }
      else if (typstr == wxT("FIMSearch")) {
        // creating a FIM Search tab
        fim_edit = new jfFIMMain(mainarranger,-1,loadres);
        tabname = wxT("Fimfiction.net Search: ") + fim_edit->GetSearchName();
        PageLoad(fim_edit,jftl_FIMSEARCH,tabname);
      }
      else if (typstr == wxT("AO3Search")) {
        // creating a AO3 Search tab
        ao3_edit = new jfAO3Main(mainarranger,-1,loadres);
        tabname = wxT("Archiveofourown.org Search: ") + ao3_edit->GetSearchName();
        PageLoad(ao3_edit,jftl_AO3SEARCH,tabname);
      }
      else if (typstr == wxT("FIMGroupSearch")) {
        fimg_edit = new jfFIMGroupMain(mainarranger,-1,loadres);
        tabname = wxT("Fimfiction.net Group Search: ") + fimg_edit->GetSearchName();
        PageLoad(fimg_edit,jftl_FIMGSEARCH,tabname);
      }
      // unrecognized type, this is a real failure
      else assert(false);
    }
    // the load failed, we show a message before exiting
    else {
      wxMessageBox(wxT("File Open failed!"),outerr, wxICON_ERROR | wxCANCEL,this);
    }
  }
}
//-------------------------------------------------------------------------
void JaffisFrame::OnOpenSavedResults(wxCommandEvent& event) {
    // local variables
	const wxChar* fname = wxT("JaffisFrame::OnOpenSavedResults");
  wxString xfilen, typstr, outerr;
  jfFFN_UpdateBundle* loadres;
  jfFileReader* infile;
  jfUpdateTopLevel* ffn_umain;
  size_t xval;
  bool iores;
  // we start with the filename
	/**/jfXLogMessage(fname,1);
  if (GetFileNameJSR(xfilen,false,false)) {
    wxBeginBusyCursor();
		/**/jfXLogMessage(fname,2);
		// getting the inputfile
		infile = new jfFileReader(xfilen);
		iores = infile->OpenFile();
		if (!iores) {
		  outerr = infile->err;
		  delete infile;
		  /**/jfXLogMessage(fname,3,outerr);
		  wxEndBusyCursor();
		  wxMessageBox(wxT("File Error!"),outerr, wxICON_ERROR | wxCANCEL,this);
		}
		else {
      // loading the data
      /**/jfXLogMessage(fname,4);
      loadres = new jfFFN_UpdateBundle();
      loadres->chlink = ffn_catorganizer::main_manager->GetHolder();
      iores = loadres->GetFromFile(infile);
      /**/jfXLogMessage(fname,5);
      // the load suceeded...
      if (iores) {
        /**/jfXLogMessage(fname,6);
        // creating a FFNS Update tab
        ffn_umain = new jfUpdateTopLevel(mainarranger,loadres,-1);
        /**/jfXLogMessage(fname,7);
        // adding it to the notebook and tracker
        ptracker.AddPage(ffn_umain->GetId(),jftl_FFNUPDATE);
        mainarranger->AddPage(ffn_umain,loadres->search_data->GetName(),true);
        xval = ffn_catorganizer::main_manager->RegisterSearch(ffn_umain);
        loadres->search_data->search_index = xval;
        CallResize();
        // enabling the close page menu item
        (MainMenu->GetMenu(0))->Enable(jfID_MENU_CLOSECURR,true);
        /**/jfXLogMessage(fname,8);
        ffn_umain->LoadOld();
        wxEndBusyCursor();
      }
      // the load failed, we show a message before exiting
      else {
        outerr = infile->err;
        /**/jfXLogMessage(fname,9,outerr);
        wxEndBusyCursor();
        wxMessageBox(wxT("Reading from file failed!"),outerr, wxICON_ERROR | wxCANCEL,this);
      }
      delete infile;
      /**/jfXLogMessage(fname,13);
    }
  }
}
//-------------------------------------------------------------------------
void JaffisFrame::OnSaveResults(wxCommandEvent& event) {
  // local variables
  wxString xfilen;
  jfFFNMain* qval;
  jfUpdateTopLevel* uval;
  bool rval;
  // start with cheking the type
  int sindex = mainarranger->GetSelection();
  rval = ptracker.IsUpdater(sindex);
  // start
  if (GetFileNameJSR(xfilen,true,!rval)) {
    // Option 1: Saving a new FFN Search + Results as a bundle
    if (!rval) {
      qval = dynamic_cast<jfFFNMain*>(mainarranger->GetCurrentPage());
      rval = qval->WriteFFNBundle(xfilen);
      assert(rval);
    }
    // Option 2: We already have a bundle, we save it
    else {
      uval = dynamic_cast<jfUpdateTopLevel*>(mainarranger->GetCurrentPage());
      rval = uval->WriteFFNBundle(xfilen);
      assert(rval);
    }
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// some more handlers
void JaffisFrame::onResize(wxSizeEvent& inevent) {
  // info
  wxSize insize = inevent.GetSize();
  // starting
  ResizeCore(insize);
  inevent.Skip();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void JaffisFrame::ResizeCore(wxSize& inval) {
  const wxString fname = wxT("JaffisFrame::ResizeCore");
  // variables
  size_t pcount, pindex;
  DisplayPanelCore* dpanel;
  // starting
  if (mainarranger!=NULL) {
    pcount = mainarranger->GetPageCount();
    for (pindex=0;pindex<pcount;pindex++) {
      if (!ptracker.NotHtml(pindex)) {
        dpanel = dynamic_cast<DisplayPanelCore*>(mainarranger->GetPage(pindex));
        dpanel->WindowResized(inval.GetWidth(),inval.GetHeight());
      }
    }
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void JaffisFrame::CallResize() {
  // variables
  wxSize wsize = GetSize();
  ResizeCore(wsize);
}
//------------------------------------------------------------------------
void JaffisFrame::HandleFFNCatDownloaded(wxCommandEvent& event) {
  (MainMenu->GetMenu(0))->Enable(jfID_MENU_NEWFFNSEA,true);
}
//------------------------------------------------------------------------
void JaffisFrame::HandleSearchFail(wxCommandEvent& event) {
  if (closestop) {
    CloseSearch();
    wxEndBusyCursor();
  }
}
//----------------------------------------------------------------------
void JaffisFrame::HandleDownloadTotal(wxCommandEvent& event) {
  StatusBar1->SetStatusText(event.GetString(),3);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* Helper methods. this one sets up the file menu, which has a lot of items. I
has to rewrite it because wxGTK seems to have problems with creating menu items
first and then adding them: now they are just creating using the 'Append'
method of wxMenu */
void JaffisFrame::SetUpFileMenu() {
  // menu variables
  wxMenu* FileMenu;
  wxMenuItem *miCloseCurrent, *miSave, *miSaveAs, *miSaveUpdat;
  wxMenuItem* miHtmlAll, *miHtmlCurr, *miFFNSearch;
  wxMenuItem* miAO3Seach;
  // we create the file menu
  FileMenu = new wxMenu();
  // the first item in the file menu is launching a search (Google)
  FileMenu->Append(jfID_MENU_NEWGOOGSEA,wxT("New Google Search..."),wxT("Create a Google Search"),wxITEM_NORMAL);
  // the second item in the file menu is also launching a search (Fanfiction.Net)
  miFFNSearch = FileMenu->Append(jfID_MENU_NEWFFNSEA,wxT("New FFN Search..."),wxT("Create a Fanfiction.Net Search"),wxITEM_NORMAL);
  miFFNSearch->Enable(globalsettings::catobtained);
  // the third item in the file menu is also launching a search (fimfiction.net)
  FileMenu->Append(jfID_MENU_NEWFIMSEA,wxT("New FIM Search..."),wxT("Create a Fimfiction.net Search"),wxITEM_NORMAL);
  // the third item in the file menu is also launching a search (fimfiction.net)
  FileMenu->Append(jfID_MENU_NEWAO3SEA ,wxT("New AO3 Search..."),wxT("Create a Archiveofourown.org Search"),wxITEM_NORMAL);
  // the fourth item is closing a search/tab
  miCloseCurrent = FileMenu->Append(jfID_MENU_CLOSECURR,wxT("Close current tab"),wxT("Closes the current toplevel tab."),wxITEM_NORMAL);
  miCloseCurrent->Enable(false);
  // adding these to a menu
  // saving searches
  miSave  = FileMenu->Append(jfID_MENU_SAVESEARCH,wxT("Save Search"),wxT("Save the search to a file"),wxITEM_NORMAL);
  miSave->Enable(false);
  miSaveAs  = FileMenu->Append(jfID_MENU_SAVEASSEARCH,wxT("Save Search As..."),wxT("Save the search to a file"),wxITEM_NORMAL);
  miSaveAs->Enable(false);
  FileMenu->Append(jfID_MENU_OPENSEARCH,wxT("Open..."),wxT("Open a search"),wxITEM_NORMAL);
  FileMenu->Append(jfID_MENU_OPENBUNDLE,wxT("Open Saved Results..."),wxT("Open an updateable Fanfiction.Net result"),wxITEM_NORMAL);
  // adding a separator
  FileMenu->AppendSeparator();
    // next up, the html output selections
  miHtmlAll  = FileMenu->Append(jfID_MENU_HTMLOUT_ALL,wxT("Write results..."),wxT("Write all results to HTML files"),wxITEM_NORMAL);
  miHtmlAll->Enable(false);
  miHtmlCurr = FileMenu->Append(jfID_MENU_HTMLOUT_CUR,wxT("Write current results..."),wxT("Writes the current result category to an HTML file"),wxITEM_NORMAL);
  miHtmlCurr->Enable(false);
  miSaveUpdat = FileMenu->Append(jfID_MENU_SAVEBUNDLE,wxT("Save updatable results..."),wxT("Produces a result filer that includes the search for updating"),wxITEM_NORMAL);
  miSaveUpdat->Enable(false);
  // another separator
  FileMenu->AppendSeparator();
  // finally, the quit
  FileMenu->Append(wxID_EXIT, _("Quit\tAlt-F4"), _("Quit the application"), wxITEM_NORMAL);
  // adding the menu to the main bar
  MainMenu->Append(FileMenu, _("&File"));
  // event handler connections
  Connect(wxID_EXIT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnQuit);
  Connect(jfID_MENU_NEWGOOGSEA,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnNewGoogleSearch);
  Connect(jfID_MENU_NEWFFNSEA,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnNewFFNSearch);
  Connect(jfID_MENU_NEWFIMSEA,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnNewFIMSearch);
  Connect(jfID_MENU_NEWAO3SEA,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnNewAO3Search);
  Connect(jfID_MENU_CLOSECURR,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnCloseTab);
  Connect(jfID_MENU_SAVESEARCH,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnSaveFile);
  Connect(jfID_MENU_SAVEASSEARCH ,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnSaveAsFile);
  Connect(jfID_MENU_OPENSEARCH,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnOpenFile);
  Connect(jfID_MENU_OPENBUNDLE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnOpenSavedResults);
  Connect(jfID_MENU_SAVEBUNDLE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnSaveResults);
  Connect(jfID_MENU_HTMLOUT_ALL,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnHtmlOutAll);
  Connect(jfID_MENU_HTMLOUT_CUR,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnHtmlOutCurr);
}
//-----------------------------------------------------------------------------------------
// setting up the tools menu is paced here, in a similar manner to the file menu
void JaffisFrame::SetUpToolsMenu() {
  // menu variables
  wxMenu* ToolsMenu;
  wxMenuItem* LoadGFilterM, *LoadSkelEditor;
  wxMenuItem* DownFFNCats, *DownAO3Cats;
  wxMenuItem* DownFFNFic, *UpdFFNFic;
  wxMenuItem* DefaultDirsM, *SearchFimGroups;
  wxMenuItem* UpdateDir;
  // the first part of the tools menu (settings)
  ToolsMenu = new wxMenu();
  LoadGFilterM = new wxMenuItem(ToolsMenu,jfID_MENU_GLOBFILTER,wxT("Global Filters..."),wxT("Load the global filter editor"),wxITEM_NORMAL);
  ToolsMenu->Append(LoadGFilterM);
  LoadSkelEditor = new wxMenuItem(ToolsMenu,jfID_MENU_GLOBSKEL,wxT("Global Skeletons..."),wxT("Load the global skeleton editor"),wxITEM_NORMAL);
  ToolsMenu->Append(LoadSkelEditor);
  DefaultDirsM  = new wxMenuItem(ToolsMenu,jfID_MENU_DEFAULTDIRS,wxT("Default Settings..."),wxT("Edit the default settings."),wxITEM_NORMAL);
  ToolsMenu->Append(DefaultDirsM);
  ToolsMenu->AppendSeparator();
  // the second part of the tools menu (utilities)
  DownFFNCats = new wxMenuItem(ToolsMenu,jfID_MENU_DOWNFFNCATS,wxT("Fanfiction.Net Categories..."),wxT("Update Fanfiction.net category data."),wxITEM_NORMAL);
  ToolsMenu->Append(DownFFNCats);
  DownAO3Cats = new wxMenuItem(ToolsMenu,jfID_MENU_DOWNAO3CATS,wxT("AO3 Categories..."),wxT("Update Archive Of Our Own category data."),wxITEM_NORMAL);
  ToolsMenu->Append(DownAO3Cats);
  DownFFNFic = new wxMenuItem(ToolsMenu,jfID_MENU_DOWNFFNFIC,wxT("Fic Download..."),wxT("Download a fanfic."),wxITEM_NORMAL);
  ToolsMenu->Append(DownFFNFic);
  UpdFFNFic = new wxMenuItem(ToolsMenu,jfID_MENU_UPDFFNFIC,wxT("Update Fics..."),wxT("Update downloaded fanfics."),wxITEM_NORMAL);
  ToolsMenu->Append(UpdFFNFic);
  SearchFimGroups = new wxMenuItem(ToolsMenu,jfID_MENU_FFNGRPSEARCH,wxT("Search FIM Groups..."),wxT("Search the Fimfiction.Net Groups"),wxITEM_NORMAL);
  ToolsMenu->Append(SearchFimGroups);
  UpdateDir = new wxMenuItem(ToolsMenu,jfID_MENU_DIRUPDATE,wxT("Update Directory..."),wxT("Update or sync a directory from another."));
  ToolsMenu->Append(UpdateDir);
  MainMenu->Append(ToolsMenu,wxT("&Tools and Settings"));
  // connecting setting menu items
  Connect(jfID_MENU_GLOBFILTER,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnEditGlobalFilters);
  Connect(jfID_MENU_GLOBSKEL,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnEditGlobalSkeletons);
  Connect(jfID_MENU_DEFAULTDIRS,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnDefaultDirs);
  // connecting utility menu items
  Connect(jfID_MENU_DOWNFFNCATS,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnDownloadFFNCats);
  Connect(jfID_MENU_DOWNAO3CATS,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnDownloadAO3Cats);
  Connect(jfID_MENU_DOWNFFNFIC,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnDownloadFFNFic);
  Connect(jfID_MENU_UPDFFNFIC,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnUpdateFFNFic);
  Connect(jfID_MENU_FFNGRPSEARCH,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnSearchFIMGroups);
  Connect(jfID_MENU_DIRUPDATE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&JaffisFrame::OnDirUpdate);
}
//-----------------------------------------------------------------------------------------
void JaffisFrame::EnableMenus(bool inval) {
  (MainMenu->GetMenu(0))->Enable(jfID_MENU_HTMLOUT_ALL,inval); // html out all
  (MainMenu->GetMenu(0))->Enable(jfID_MENU_HTMLOUT_CUR,inval); // html out selective
  (MainMenu->GetMenu(0))->Enable(jfID_MENU_SAVESEARCH,inval);
  (MainMenu->GetMenu(0))->Enable(jfID_MENU_SAVEASSEARCH,inval);
}
//-----------------------------------------------------------------------------------------
void JaffisFrame::HtmlREnable(bool inval) {
  int sindex;
  sindex = mainarranger->GetSelection();
  // usual saves
  (MainMenu->GetMenu(0))->Enable(jfID_MENU_HTMLOUT_ALL,inval); // html out all
  (MainMenu->GetMenu(0))->Enable(jfID_MENU_HTMLOUT_CUR,inval); // html out selective
  // the save as bundle
  if (ptracker.IsFFNSearch(sindex)) {
    (MainMenu->GetMenu(0))->Enable(jfID_MENU_SAVEBUNDLE,inval);
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// a helper method intended to selectivly enable or disbale menus based on page information
void JaffisFrame::MenuEnabler() {
  // local variables
  jfMainSearchGroup* qval;
  jfUpdateTopLevel* zval;
  bool savedef, hready;
  int sindex;
  // the first option is that there are no pages left
  if (ptracker.GetCount()==0) {
    (MainMenu->GetMenu(0))->Enable(jfID_MENU_CLOSECURR,false); // close
    EnableMenus(false);
  }
  // here, there are some pages left
  else {
    (MainMenu->GetMenu(0))->Enable(jfID_MENU_CLOSECURR,true);
    sindex = mainarranger->GetSelection();
    //n the page is not a search, but an editor
    if (ptracker.NotHtml(sindex)) EnableMenus(false);
    // the page is an updator
    else if (ptracker.IsUpdater(sindex)) {
      zval = dynamic_cast<jfUpdateTopLevel*>(mainarranger->GetCurrentPage());
      // the html outpout menus
      hready = ptracker.IsHtmlReady(sindex);
      if (hready) HtmlREnable(true);
      else HtmlREnable(false);
    }
    // the page is an editor
    else { // start
      qval = dynamic_cast<jfMainSearchGroup*>(mainarranger->GetCurrentPage());
      // the save and save as menus
      (MainMenu->GetMenu(0))->Enable(jfID_MENU_SAVEASSEARCH,true);
      savedef = qval->HasBeenSaved();
      (MainMenu->GetMenu(0))->Enable(jfID_MENU_SAVESEARCH,savedef);
      // the html outpout menus
      hready = ptracker.IsHtmlReady(sindex);
      if (hready) HtmlREnable(true);
      else HtmlREnable(false);
    } // end
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// encapsulates getting a filename via file dialog
bool JaffisFrame::GetFileNameJSS(wxString& fnameout, bool save) {
  // constants
  const wxString wildstr = wxT("Jaffis Saved Search Files (*.jss)|*.jss");
  const wxString savemsg = wxT("Specify the Save filename");
  const wxString openmsg = wxT("Specify the File to open");
  const long savesty = wxFD_SAVE|wxFD_OVERWRITE_PROMPT;
  // variables
  wxFileDialog* filepick;
  wxString xfilen;
  jfMainSearchGroup* qval;
  int sresult;
  bool is_google;
  wxString ddir;
  // starting
  fnameout = wxEmptyString;
  // starting, save version
  if (save) {
    // getting the main page
    qval = dynamic_cast<jfMainSearchGroup*>(mainarranger->GetCurrentPage());
    sresult = mainarranger->GetSelection();
    is_google = (jftl_GOOGSEARCH==(ptracker.GetType(sresult)));
    // specifying the default file name
    xfilen = qval->GetDisplayName() + wxT(".jss");
    // creating the file save dialog
    ddir = globalsettings::GSaveDir(is_google);
    filepick = new wxFileDialog(this,savemsg,ddir,xfilen,wildstr,savesty);
  }
  // open version
  else {
    filepick = new wxFileDialog(this,openmsg,(*globalsettings::globaldirs)[5]);
    filepick->SetWildcard(wildstr);
  }
  // showing and getting filename
  sresult = filepick->ShowModal();
  if (sresult == wxID_OK) fnameout = filepick->GetPath();
  // finishing
  delete filepick;
  return (sresult == wxID_OK);
}
//------------------------------------------------------------------
// encapsulates getting a filename via file dialog
bool JaffisFrame::GetFileNameJSR(wxString& fnameout, bool save, bool newx) {
  // constants
  const wxString wildstr = wxT("Jaffis Saved Result Files (*.jsr)|*.jsr");
  const wxString savemsg = wxT("Specify the Save filename");
  const wxString openmsg = wxT("Specify the File to open");
  const long savesty = wxFD_SAVE|wxFD_OVERWRITE_PROMPT;
  // variables
  wxFileDialog* filepick;
  wxString xfilen;
  jfMainSearchGroup* qval;
  jfUpdateTopLevel* uval;
  int sresult;
  // starting
  fnameout = wxEmptyString;
  // starting, save version
  if (save) {
    // getting the main page getting the default file name
    if (newx) {
      qval = dynamic_cast<jfMainSearchGroup*>(mainarranger->GetCurrentPage());
      xfilen = qval->GetDisplayName() + wxT(".jsr");
    }
    else {
      uval = dynamic_cast<jfUpdateTopLevel*>(mainarranger->GetCurrentPage());
      xfilen = uval->GetSearchName() + wxT(".jsr");
    }
    // creating the file save dialog
    filepick = new wxFileDialog(this,savemsg,(*globalsettings::globaldirs)[3],xfilen,wildstr,savesty);
  }
  // open version
  else {
    filepick = new wxFileDialog(this,openmsg,(*globalsettings::globaldirs)[3]);
    filepick->SetWildcard(wildstr);
  }
  // showing and getting filename
  sresult = filepick->ShowModal();
  if (sresult == wxID_OK) fnameout = filepick->GetPath();
  // finishing
  delete filepick;
  return (sresult == wxID_OK);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// if the screen is too small for the frame by default, we have a maximized version
bool JaffisFrame::MaximizeTest() {
  // getting dimesnions
  wxRect rvalue = ::wxGetClientDisplayRect();
  wxSize gsize = GetSize();
  // checking
  bool wc = rvalue.width <= gsize.GetWidth();
  bool hc = rvalue.height <= gsize.GetHeight();
  // deciding
  if (wc||hc) {
    Maximize(true);
    return true;
  }
  else {
    Centre();
    return false;
  }
}
//-------------------------------------------------------------------------------
/* closing a tab has some common actions, which should be done when we close the tab
directly or when a download is halted after the user tries to close a tab */
void JaffisFrame::CloseSearch() {
  // variables
  int sindex;
  jfFFNMain* zval;
  jfAO3Main* qval;
  size_t scat_index;
  // starting
  sindex = mainarranger->GetSelection();
  assert(sindex!=-1);
  // if we are doing an FFN search, we remove the search from the category organizer
  if (ptracker.GetType(sindex)==jftl_FFNSEARCH) {
    zval = dynamic_cast<jfFFNMain*> (mainarranger->GetCurrentPage());
    scat_index = zval->GetSCatIndex();
    ffn_catorganizer::main_manager->RemoveSearch(scat_index);
  }
  // we also do it for AO3
  else if (ptracker.GetType(sindex)==jftl_AO3SEARCH) {
    qval = dynamic_cast<jfAO3Main*> (mainarranger->GetCurrentPage());
    scat_index = qval->GetSCatIndex();
    ao3_catdata::ao3_catmanager->RemoveSearch(scat_index);
  }
  // removing the page
  ptracker.RemovePage(sindex);
  mainarranger->DeletePage(sindex);
  closestop = false;
  MenuEnabler();
}
//-------------------------------------------------------------------
/* I thought I had implemeted this before : common tab load method */
bool JaffisFrame::PageLoad(DisplayPanelCore* xpanel, jf_tlPageType ptype, wxString page_name) {
  // quick checks
  if (xpanel==NULL) return false;
  if (page_name.IsEmpty()) return false;
  // adding it to the notebook and tracker
  ptracker.AddPage(xpanel->GetId(),ptype);
  mainarranger->AddPage(xpanel,page_name,true);
  CallResize();
  // enabling the close page menu item
  (MainMenu->GetMenu(0))->Enable(jfID_MENU_CLOSECURR,true);
  // done
  return true;
}
/*******************************************************************************************/
