/***************************************************************
 * Name:      JaffisMain.h
 * Purpose:   Defines Application Frame
 * Author:    John Q Metro (johnqmetro@hotmail.com)
 * Created:   2009-03-23
 * Updated:   July 23, 2015
 * Copyright: John Q Metro ()
 * License:
 **************************************************************/

#ifndef JAFFISMAIN_H
#define JAFFISMAIN_H

//(*Headers(JaffisFrame)
#include <wx/menu.h>
#include <wx/frame.h>
#include <wx/statusbr.h>

//*)

#include <wx/notebook.h>
#include <wx/sizer.h>

#ifndef FILTER_EDITOR_H_INCLUDED
  #include "interface/filters/base/filter_editor.h"
#endif // FILTER_EDITOR_H_INCLUDED
#ifndef PAGETRACKER_H_INCLUDED
  #include "core/structs/pagetracker.h"
#endif // PAGETRACKER_H_INCLUDED

#ifndef G_MAINPANEL1_H_INCLUDED
  #include "google/interface/g_mainpanel1.h"
#endif // G_MAINPANEL1_H_INCLUDED

#ifndef FFN_MAINPANEL
  #include "fanficnet/interface/ffn_mainpanel.h"
#endif
//==================================================================

class JaffisFrame: public wxFrame
{
    public:

        JaffisFrame(wxWindow* parent,wxWindowID id = -1);
        virtual ~JaffisFrame();

    private:

        //(*Handlers(JaffisFrame)
        void OnQuit(wxCommandEvent& event);
        void OnAbout(wxCommandEvent& event);
        //*)
        // tool menu handlers
        void OnEditGlobalFilters(wxCommandEvent& event);
        void OnEditGlobalSkeletons(wxCommandEvent& event);
        void OnDefaultDirs(wxCommandEvent& event);
        void OnDirUpdate(wxCommandEvent& event);
        void OnDownloadFFNCats(wxCommandEvent& event);
        void OnDownloadAO3Cats(wxCommandEvent& event);
        void OnDownloadFFNFic(wxCommandEvent& event);
        void OnUpdateFFNFic(wxCommandEvent& event);
        void OnSearchFIMGroups(wxCommandEvent& event);
        // more handlers
        void OnNewGoogleSearch(wxCommandEvent& event);
        void OnNewFFNSearch(wxCommandEvent& event);
        void OnNewFIMSearch(wxCommandEvent& event);
        void OnNewAO3Search(wxCommandEvent& event);
        void OnCloseTab(wxCommandEvent& event);
        void NotebookTabChanged(wxNotebookEvent& event);
        void OnSearchStarted(wxCommandEvent& event);
        void OnSearchDone(wxCommandEvent& event);
        void OnUpdateDone(wxCommandEvent& event);
        // i/o handlers
        void OnHtmlOutCurr(wxCommandEvent& event);
        void OnHtmlOutAll(wxCommandEvent& event);
        void OnSaveFile(wxCommandEvent& event);
        void OnSaveAsFile(wxCommandEvent& event);
        void OnOpenFile(wxCommandEvent& event);
        void OnOpenSavedResults(wxCommandEvent& event);
        void OnSaveResults(wxCommandEvent& event);
        // some more handlers // resize
        void onResize(wxSizeEvent& inevent);
        void ResizeCore(wxSize& inval);
        void CallResize();
        void HandleFFNCatDownloaded(wxCommandEvent& event);
        void HandleSearchFail(wxCommandEvent& event);
        void HandleDownloadTotal(wxCommandEvent& event);
        // helper methods
        void SetUpFileMenu();
        void SetUpToolsMenu();
        void EnableMenus(bool inval);
        void HtmlREnable(bool inval);
        void MenuEnabler();
        bool GetFileNameJSS(wxString& fnameout, bool save);
        bool GetFileNameJSR(wxString& fnameout, bool save, bool newx);
        bool MaximizeTest();
        void CloseSearch();
        bool PageLoad(DisplayPanelCore* xpanel, jf_tlPageType ptype, wxString page_name);

        //(*Identifiers(JaffisFrame)
        static const long idMenuQuit;
        static const long idMenuAbout;
        static const long ID_STATUSBAR1;
        //*)

        //(*Declarations(JaffisFrame)
        wxStatusBar* StatusBar1;
        //*)

        // additional stuff
        wxMenuBar* MainMenu;
        wxNotebook* mainarranger;
        wxBoxSizer* topsizer;
        // pages
        bool closestop;
        jfPageTracker ptracker;

        DECLARE_EVENT_TABLE()
};

#endif // JAFFISMAIN_H
