/******************************************************************************
Name    :   core_threads.cpp
Author  :   John Q Metro
Purpose :   Base threads classes for my rewriting of Jaffis downloaders
Created :   December 7, 2013
Updated :   July 23, 2015
******************************************************************************/
#ifndef CORE_THREADS_H_INCLUDED
  #include "core_threads.h"
#endif // CORE_THREADS_H_INCLUDED
//-----------------------------------
#ifndef LOGGING_H_INCLUDED
  #include "../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef DOWN_DATA_H_INCLUDED
  #include "down_data.h"
#endif // DOWN_DATA_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef CURL_GETTER_H_INCLUDED
  #include "curl_getter.h"
#endif // CURL_GETTER_H_INCLUDED
#ifndef INITEND_H_INCLUDED
  #include "../initend.h"
#endif // INITEND_H_INCLUDED
//-----------------------------------

/*****************************************************************************/
jfCoreThread::jfCoreThread():wxThread() {
  msg_target = NULL;
  haltsent = false;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// messaging
//-------------------------------------------
void jfCoreThread::SendStart() const {
  // creating
  jfThreadEvent eventx(jf_DOWNL_START_EVENT,msg_target->GetId(),soption);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//-------------------------------------------
void jfCoreThread::SendEnd() const {
  // creating
  jfThreadEvent eventx(jf_DOWNL_END_EVENT,msg_target->GetId(),soption);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//-------------------------------------------
void jfCoreThread::SendTotalCount(size_t scount) const {
  // creating
  jfThreadEvent eventx(jf_DOWNL_PCOUNT_EVENT,msg_target->GetId(),soption);
  eventx.SetInt1(scount);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//-------------------------------------------
void jfCoreThread::SendLabels(const wxString& item_label, const wxString& first_action) const {
  // creating
  jfThreadEvent eventx(jf_DOWNL_LABELS_EVENT,msg_target->GetId(),soption);
  eventx.SetStrings(item_label,first_action);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//-------------------------------------------
void jfCoreThread::SendNewItem(int lindex, int ltotal, wxString iname) const {
  // creating
  jfThreadEvent eventx(jf_DOWNL_PSTART_EVENT,msg_target->GetId(),soption);
  eventx.SetInts(lindex,ltotal);
  eventx.SetString1(iname);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//-------------------------------------------
void jfCoreThread::SendAction(const wxString& newaction) const {
  // creating
  jfThreadEvent eventx(jf_DOWNL_PPROC_EVENT,msg_target->GetId(),soption);
  eventx.SetString1(newaction);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//-------------------------------------------
void jfCoreThread::SendHalt() const {
  // creating
  assert(!haltsent);
  jfThreadEvent eventx(jf_DOWNL_PHALT_EVENT,msg_target->GetId(),soption);
  // sending the message
  haltsent = true;
  wxPostEvent(msg_target,eventx);
}
//-------------------------------------------
void jfCoreThread::SendReset() const {
  // creating
  jfThreadEvent eventx(jf_DOWNL_RESET_EVENT,msg_target->GetId(),soption);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// logging
//-------------------------------------------
void jfCoreThread::TLog(const wxString& fname, size_t index) const {
	wxString res_msg = BLogCore(MLog(fname),index,false);
	jfXLogString(res_msg);
}
//-------------------------------------------
void jfCoreThread::TLog(const wxString& fname, size_t index, wxString data) const {
  wxString res_msg = BLogCore(MLog(fname),index,true);
	res_msg += data;
	jfXLogString(res_msg);
}
//-------------------------------------------
void jfCoreThread::TLogS(const wxString& fname, size_t index, size_t oval) const {
	wxString res_msg = BLogCore(MLog(fname),index,true);
	res_msg += SInt2Str(oval);
	jfXLogString(res_msg);
}
//-------------------------------------------
void jfCoreThread::TLogI(const wxString& fname, size_t index, int oval) const {
  wxString res_msg = BLogCore(MLog(fname),index,true);
	res_msg += IInt2Str(oval);
	jfXLogString(res_msg);
}
//-------------------------------------------
void jfCoreThread::TLogB(const wxString& fname, size_t index, bool oval) {
	wxString res_msg = BLogCore(MLog(fname),index,true);
	res_msg += Bool2Str(oval);
	jfXLogString(res_msg);
}
//-------------------------------------------
void jfCoreThread::TLogChar(const wxString& fname, size_t index, const char* outmsg) {
	wxString res_msg = BLogCore(MLog(fname),index,true);
	res_msg += wxString(outmsg,wxConvISO8859_1);
	jfXLogString(res_msg);
}
//-------------------------------------------
void jfCoreThread::TAssertLog(bool condition, const wxString& fname, wxString outmsg) {
  wxString result;
  if (!condition) {
    result = wxT("Assertion failed in ");
    result += MLog(fname);
    result += wxT(" : ");
    result += outmsg;
    jfXLogString(result);
    assert(false);
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfCoreThread::MLog(const wxString& fname) const {
  wxString result = wxT("[");
  result += threadname + wxT("] ");
  result += fname;
  return result;
}
//===============================================================================
jfThreadWithDownload::jfThreadWithDownload():jfCoreThread() {
  dconst = NULL;
  fres = jfDown::FETCHED;
  page_data = NULL;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting data
bool jfThreadWithDownload::SetParams(jfDownloadParams* in_dconst) {
  if (in_dconst==NULL) return false;
  dconst = in_dconst;
  msg_target = dconst->msg_target;
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfThreadWithDownload::GetUrl(wxString inurl) {
  // variables
  jfCurlPageRetriever* page_getter;
  bool result;
  // doing the page fetch
  page_getter = new jfCurlPageRetriever(dconst,inurl,this);
  result = page_getter->DownloadPage();
  // the page was fetched fine
  if (result) {
    // preparing for processing
    fres = jfDown::FETCHED;
    err_string.Clear();
    page_data = page_getter->GetResult();
  }
  // page fetching failed
  else {
    fres = page_getter->GetFaliureReason();
    if (fres==jfDown::REDIRECTED) err_string = page_getter->GetRedirect();
  }
  // post cleanup
  delete page_getter;
  SendDatacount();
  return result;
}
//-----------------------------------------------------------
void jfThreadWithDownload::SendDatacount() {
  wxString data_string;
  globalsettings::down_mutex.Lock();
  // starting
  data_string = ByteToSize(globalsettings::totaldown);
  wxCommandEvent eventx(jf_DOWNL_DATATOTAL_EVENT, globalsettings::topwindow->GetId());
  eventx.SetString(data_string);
  globalsettings::down_mutex.Unlock();
  // sending
  wxPostEvent(globalsettings::topwindow,eventx);
}
//===============================================================================
// constructor
jfDownloadThread::jfDownloadThread():jfThreadWithDownload() {
  pageindex = 0;
  epagecount = 1;
  limitr = false;
  hardskip = false;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++
void jfDownloadThread::SendDone(jfDown::ResultAction done_status) {
  // creating
  jfThreadEvent eventx(jf_DOWNL_PEND_EVENT,msg_target->GetId(),soption);
  // setting data
  eventx.SetResAction(done_status);
  if (done_status!=jfDown::PROCEED) {
    eventx.SetError(fres);
    if ((fres==jfDown::PARSE_ERROR) || (fres==jfDown::REDIRECTED)) {
      eventx.SetString2(err_string);
    }
  }
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++
// the core methods
//----------------------
bool jfDownloadThread::HandlePage() {
  // variables
  bool result;
  size_t npagecount;
  // starting
  PreFetch1();
  SetUrl();
  PreFetch2();
  // special skip
  if (hardskip) {
    SendDone(jfDown::SKIP);
    return true;
  }
  // the page was fetched fine
  if (GetUrl(fetchurl)) {
    // preparing for processing
    SendAction(wxT("Processing"));
    // processing
    if (ProcessPage()) {
      // if the processing is done, we check the pagecount and send done
      npagecount = NewPagecount();
      SendDone(jfDown::PROCEED);
      if (npagecount!=epagecount) {
        SendTotalCount(npagecount);
        epagecount = npagecount;
      }
      result = true;
      PostPageAction();
    }
    else {
      // parsing faliure
      fres = jfDown::PARSE_ERROR;
      result = dconst->pageskip;
      if (result) SendDone(jfDown::SKIP);
      else SendDone(jfDown::FAIL);
    }
    delete page_data;
    page_data = NULL;
  }
  // page fetching failed
  else {
    // user has halted download... we do not go on
    if (fres==jfDown::ABORTED) {
      result = false;
      SendDone(jfDown::HALT);
    }
    // otherwise, a real error has happened, things depend on whether we skip
    else {
      result = dconst->pageskip;
      if (!result) result = ((dconst->skipmissing) && (fres==jfDown::MISSING));
      if (result) SendDone(jfDown::SKIP);
      else SendDone(jfDown::FAIL);
    }
  }
  // post cleanup
  return result;
}
//----------------------
bool jfDownloadThread::LoopGet() {
  bool resval;
  // the fetching loop
  for (pageindex=1;pageindex<=epagecount;pageindex++) {
    if (limitr) break;
    resval = HandlePage();
    if (!resval) break;
    if (pageindex!=epagecount) {
      if (PauseHaltCheck()) {
        resval = false;
        break;
      }
    }
  }
  // done
  return resval;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++
// additional methods
//----------------------------------------
bool jfDownloadThread::RedirectCheck() {
  // variables
  jfCurlPageRetriever* page_getter;
  bool result = false;
  // doing the page fetch
  page_getter = new jfCurlPageRetriever(dconst,fetchurl,this);
  if (page_getter->CheckForRedirection()) {
    fetchurl = page_getter->GetRedirect();
    result = true;
  }
  delete page_getter;
  return result;
}
//----------------------------------------
bool jfDownloadThread::PauseHaltCheck() {
  if (dconst->pauselink->PauseHaltCheck()) {
    fres=jfDown::ABORTED;
    SendHalt();
    return true;
  }
  else return false;
}
/*****************************************************************************/
