/******************************************************************************
Name    :   core_threads.h
Author  :   John Q Metro
Purpose :   Base threads classes for my rewriting of Jaffis downloaders
Created :   December 7, 2013
Updated :   July 23, 2015
******************************************************************************/
#ifndef CORE_THREADS_H_INCLUDED
#define CORE_THREADS_H_INCLUDED
#endif // CORE_THREADS_H_INCLUDED
//-----------------------------------
#ifndef DOWN_DATA_H_INCLUDED
  #include "down_data.h"
#endif // DOWN_DATA_H_INCLUDED
//-----------------------------------
#include <wx/thread.h>
#include <wx/string.h>
#include <wx/window.h>
/*****************************************************************************/
/* Messages that get sent by the threads
Start and End (overall, 2 messages)
total Item count
Item label and start action
new item, with local index, local count, and name
change action
done with item okay
done because we are skipping the item due to an error
halting because of an error
halting because we are asked to stop
reset progress bar

data counter message
*/

/*****************************************************************************/
/* This thread mostly implements logging and messages, because there are some non-download
tasks we can use the 'progress bar with background thread' for */
class jfCoreThread : public wxThread {
  public:
    // default constructor
    jfCoreThread();
    // messaging
    void SendStart() const;
    void SendEnd() const;
    void SendTotalCount(size_t scount) const;
    void SendLabels(const wxString& item_label, const wxString& first_action) const;
    void SendNewItem(int lindex, int ltotal, wxString iname = wxEmptyString) const;
    void SendAction(const wxString& newaction) const;
    virtual void SendDone(jfDown::ResultAction done_status) = 0;
    void SendHalt() const;
    void SendReset() const;
    // logging
    void     TLog(const wxString& fname, size_t index) const;
    void     TLog(const wxString& fname, size_t index, wxString data) const;
    void    TLogS(const wxString& fname, size_t index, size_t oval) const;
    void    TLogI(const wxString& fname, size_t index, int oval) const;
    void    TLogB(const wxString& fname, size_t index, bool oval);
    void TLogChar(const wxString& fname, size_t index, const char* outmsg);
    void TAssertLog(bool condition, const wxString& fname, wxString outmsg);
  protected:
    // messaging
    wxWindow* msg_target;
    bool soption; // used as an extra marker for messages
    mutable bool haltsent;
    // logging
    wxString threadname;
    wxString MLog(const wxString& fname) const;
};
//===================================================================================
// thread that serves as base in which we download, but not for every loop
class jfThreadWithDownload : public jfCoreThread {
  public:
    // constructor
    jfThreadWithDownload();
    // a reminder
    virtual wxThread::ExitCode Entry() = 0;
    // setting data
    bool SetParams(jfDownloadParams* in_dconst);
  protected:
    bool GetUrl(wxString inurl);
    void SendDatacount();
    // result info
    jfDown::FetchResult fres;
    wxString* page_data;
    wxString err_string;
    // thread constants
    jfDownloadParams* dconst;
};
//===================================================================================
/* Thread with iterative download. */
class jfDownloadThread : public jfThreadWithDownload {
  public:
    // constructor
    jfDownloadThread();
    // a reminder
    virtual wxThread::ExitCode Entry() = 0;
    // also
    virtual void SendDone(jfDown::ResultAction done_status);
  protected:
    // the core methods
    bool HandlePage();
    bool LoopGet();
    // additional methods
    bool RedirectCheck();
    bool PauseHaltCheck();
    // virtual plug-in methods to handle some vital bits
    virtual void PreFetch1() = 0;
    virtual void PreFetch2() = 0;
    virtual void SetUrl() = 0;
    virtual bool ProcessPage() = 0;
    virtual size_t NewPagecount() const = 0;
    virtual void PostPageAction() = 0;
    // download data
    wxString fetchurl;
    size_t epagecount;
    size_t pageindex;
    bool limitr;
    bool hardskip;
};

/*****************************************************************************/
