//****************************************************************************
// Name:        curl_getter.cpp
// Author :     John Q Metro
// Purpose :    A replacement for jfPageRetriever that uses libcurl
// Created:     November 21, 2013
// Updated:     August 11, 2015
//****************************************************************************
#ifndef CURL_GETTER_H_INCLUDED
  #include "curl_getter.h"
#endif // CURL_GETTER_H_INCLUDED
//--------------------------------------
#ifndef LOGGING_H_INCLUDED
  #include "../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef CHARCONV_H_INCLUDED
  #include "../core/utils/charconv.h"
#endif // CHARCONV_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef CORE_THREADS_H_INCLUDED
  #include "core_threads.h"
#endif // CORE_THREADS_H_INCLUDED
#ifndef INITEND_H_INCLUDED
  #include "../initend.h"
#endif // INITEND_H_INCLUDED
//--------------------------------------------------------------------------

//****************************************************************************
// copying from the libcurl examples
static size_t WriteMemoryCallback(void *contents, size_t size, size_t nmemb, void *userp) {
  size_t realsize = size * nmemb;
  struct jfCurlMemoryStruct *mem = (struct jfCurlMemoryStruct *)userp;

  mem->memory = static_cast<char*>(realloc(mem->memory, mem->size + realsize + 1));
  if(mem->memory == NULL) {
    /* out of memory! */
    printf("not enough memory (realloc returned NULL)\n");
    return 0;
  }

  memcpy(&(mem->memory[mem->size]), contents, realsize);
  mem->size += realsize;
  mem->memory[mem->size] = 0;

  return realsize;
}
//===========================================================================
// constructors
//-------------------------------------------------------------------------
// the default constructor
jfCurlPageRetriever::jfCurlPageRetriever() {
  Initialize();
}
//-------------------------------------------------------------------------------
// constructs and initializes in 1 step
jfCurlPageRetriever::jfCurlPageRetriever(jfDownloadParams* inval,wxString url_in, jfCoreThread* tparent_in) {
  Initialize();
  SetStartData(inval,url_in,tparent_in);
}
//-------------------------------------------------------------------------------
// initializes url and data
bool jfCurlPageRetriever::SetStartData(jfDownloadParams* inval, wxString url_in, jfCoreThread* tparent_in) {
  // constants
  const wxString fname = wxT("jfCurlPageRetriever::SetStartData");
  // asserts and checks for input
  assert(inval!=NULL);
  assert(tparent_in!=NULL);
  internal_const = inval;
  tparent->TAssertLog(!url_in.IsEmpty(),fname,wxT("The input URL is empty!"));
  // we start
  ClearThings();
  url2get = url_in;
  urlset = true;
  tparent = tparent_in;
  // done
  return urlset;
}
//-------------------------------------------------------------------------------
// returns true if a valid URL has been set
bool jfCurlPageRetriever::UrlIsSet() const {
  return urlset;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the connection
//-------------------------------------------------------------------------------
// we connect and get a stream
bool jfCurlPageRetriever::Connect() {
  // constants
  const wxString fname=wxT("jfCurlPageRetriever::Connect");
  char* charurl;
  // assertions and checks
  tparent->TAssertLog(urlset,fname,wxT("The URL is not set"));
  tparent->TAssertLog(!connected,fname,wxT("Already connected."));
  // we start by setting up the parameters
  /**/tparent->TLog(fname,1,url2get);
  charurl = wxStringToCharString(url2get,false);
  /**/tparent->TLogChar(fname,2,charurl);
  curl_handle = curl_easy_init();
  curl_easy_setopt(curl_handle, CURLOPT_URL,charurl);
  curl_easy_setopt(curl_handle, CURLOPT_NOSIGNAL, 1);
  curl_easy_setopt(curl_handle,CURLOPT_TIMEOUT,jfDown::timeout);
  curl_easy_setopt(curl_handle,CURLOPT_FOLLOWLOCATION,false);
  curl_easy_setopt(curl_handle,CURLOPT_USERAGENT, "jaffis-agent/1.0");
  curl_easy_setopt(curl_handle,CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
  curl_easy_setopt(curl_handle,CURLOPT_WRITEDATA, (void *)&rawdown);
  if (!(internal_const->incookie).IsEmpty()) {
    /**/tparent->TLog(fname,3,internal_const->incookie);
    charurl = wxStringToCharString(internal_const->incookie,false);
    curl_easy_setopt(curl_handle,CURLOPT_COOKIE,charurl);
  }
  /**/tparent->TLog(fname,4);
  // we get the data
  cstatus = jfDown::FETCHING;
  res = curl_easy_perform(curl_handle);
  /**/tparent->TLog(fname,5);
  connected = true;
  /**/tparent->TLog(fname,6);
  free(charurl);
  /**/tparent->TLog(fname,7);
  return connected;
}
//-------------------------------------------------------------------------------
// returns the HTTP response code
int jfCurlPageRetriever::GetResponse() const {
  return result_code;
}
//--------------------------------------------------------------------------------
size_t jfCurlPageRetriever::GetFailcount() const {
  return failcount;
}
//--------------------------------------------------------------------------------
wxString jfCurlPageRetriever::GetRedirect() const {
  return redirect;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// does a connection with redirection checks, that is all.
bool jfCurlPageRetriever::CheckForRedirection() {
  // constants
  const wxString fname = wxT("jfCurlPageRetriever::CheckForRedirection");
  CURLcode xresult;
  char* resptr;
  wxString intresult;
  bool redredult = false;
  char* charurl;
  // assertions and checks
  tparent->TAssertLog(urlset,fname,wxT("The URL is not set"));
  tparent->TAssertLog(!connected,fname,wxT("Already connected."));
  // starting
  /**/tparent->TLog(fname,1);
  charurl = wxStringToCharString(url2get,false);
  /**/tparent->TLogChar(fname,2,charurl);
  curl_handle = curl_easy_init();
  // setting up the options
  /**/tparent->TLog(fname,3);
  curl_easy_setopt(curl_handle, CURLOPT_URL,charurl);
  curl_easy_setopt(curl_handle, CURLOPT_NOSIGNAL, 1);
  curl_easy_setopt(curl_handle,CURLOPT_TIMEOUT,jfDown::timeout);
  curl_easy_setopt(curl_handle,CURLOPT_NOBODY,true);
  curl_easy_setopt(curl_handle,CURLOPT_HEADER, true);
  curl_easy_setopt(curl_handle,CURLOPT_USERAGENT, "jaffis-agent/1.0");
  curl_easy_setopt(curl_handle,CURLOPT_FOLLOWLOCATION,false);
  /**/tparent->TLog(fname,4);
  // doing the operation
  xresult = curl_easy_perform(curl_handle);
  // checking the result
  /**/tparent->TLogChar(fname,5,curl_easy_strerror(xresult));
  if (xresult==CURLE_OK) {
    xresult = curl_easy_getinfo(curl_handle,CURLINFO_REDIRECT_URL,&resptr);
    /**/tparent->TLogChar(fname,6,curl_easy_strerror(xresult));
    if (xresult==CURLE_OK) {
      intresult = wxString(resptr,wxConvISO8859_1);
      /**/tparent->TLog(fname,7,intresult);
      if ((intresult!=url2get) && (!intresult.IsEmpty())) {
        redirect = intresult;
        redredult = true;
      }
    }
  }
  /**/tparent->TLogB(fname,8,redredult);
  curl_easy_cleanup(curl_handle);
  curl_handle = NULL;
  free(charurl);
  return redredult;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting the page
//-------------------------------------------------------------------------------
// fetches the page, making multiple retries
bool jfCurlPageRetriever::DownloadPage() {
  // constants
  const wxString fname=wxT("jfCurlPageRetriever::DownloadPage");
  //local variables
  size_t or_idx, ir_idx, or_max, ir_max;
  bool gotten;
  // checks
  /**/tparent->TAssertLog(urlset,fname,wxT("urlset is false"));
  /**/tparent->TAssertLog(!connected,fname,wxT("Already connected!"));
  // starting
  /**/tparent->TLog(fname,1);
  ir_max = jfDown::minor_retries;
  or_max = jfDown::major_retries;
  // the outer fetching loop
  for (or_idx=0;or_idx<=or_max;or_idx++) {
    /**/tparent->TLogS(fname,2,or_idx);
    // the inner fetching loop
    for (ir_idx=0;ir_idx<=ir_max;ir_idx++) {
      // we try
      /**/tparent->TLogS(fname,3,ir_idx);
      gotten = FetchPage();
      if (gotten) return true; // it worked!
      if (!retryable) return false; // it failed
      // retry preparations
      SendRetry(ir_idx+1);
      ClearThings();
      if (ir_idx<ir_max) wxMilliSleep(jfDown::SleepPause());
      // checking for a halt
      if (PauseHaltCheck()) {
        main_status = jfDown::ABORTED;
        return false;
      }
    }
    if (internal_const->no_major_retry) break;
    // after the inner loop fails ...
    RequestPause();
    /**/tparent->TLog(fname,6);
    tparent->Sleep(10000);
    if (PauseHaltCheck()) {
      main_status = jfDown::ABORTED;
      return false;
    }
    // end of outer loop
  }
  // we have failed!
  return false;
}
//-------------------------------------------------------------------------------
jfDown::FetchResult jfCurlPageRetriever::GetFaliureReason() const {
    return main_status;
}
//-------------------------------------------------------------------------------
wxString* jfCurlPageRetriever::GetResult() {
  return outval;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructor
jfCurlPageRetriever::~jfCurlPageRetriever() {
  ClearThings();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// post-connect helper methods
//-------------------------------------------------
// after calling connect, we use this to look at the result codes
bool jfCurlPageRetriever::CheckResultCodes() {
  // variables
  const wxString fname = wxT("jfCurlPageRetriever::CheckResultCodes");
  CURLcode xresult;
  long respval;
  // initial checks
  /**/tparent->TLog(fname,1);
  /**/tparent->TAssertLog(connected,fname,wxT("Function called while not connected!"));
  /**/tparent->TLogChar(fname,2,curl_easy_strerror(res));
  // distilling the zillion curl error codes down to my more manageable list...
  switch (res) {
    case CURLE_OK :
      main_status = jfDown::FETCHED;
      break;
    case CURLE_TOO_MANY_REDIRECTS :
      main_status = jfDown::REDIRECTED;
      break;
    case CURLE_OPERATION_TIMEDOUT :
      main_status = jfDown::TIMEDOUT;
      break;
    case CURLE_SSL_CONNECT_ERROR        :
    case CURLE_PEER_FAILED_VERIFICATION :
    case CURLE_SSL_ENGINE_NOTFOUND      :
    case CURLE_SSL_ENGINE_SETFAILED     :
    case CURLE_SSL_CERTPROBLEM          :
    case CURLE_SSL_CIPHER               :
    case CURLE_SSL_CACERT               :
    case CURLE_USE_SSL_FAILED           :
    case CURLE_SSL_ENGINE_INITFAILED    :
    case CURLE_SSL_CACERT_BADFILE       :
    case CURLE_SSH                      :
    case CURLE_SSL_SHUTDOWN_FAILED      :
    case CURLE_SSL_CRL_BADFILE          :
    case CURLE_SSL_ISSUER_ERROR         :
      main_status = jfDown::SSL_ERROR;
      break;
    case CURLE_COULDNT_CONNECT        :
    case CURLE_COULDNT_RESOLVE_PROXY  :
    case CURLE_COULDNT_RESOLVE_HOST   :
    case CURLE_REMOTE_ACCESS_DENIED   :
    case CURLE_FTP_COULDNT_SET_TYPE   :
    case CURLE_PARTIAL_FILE           :
    case CURLE_FTP_COULDNT_RETR_FILE  :
    case CURLE_FTP_CANT_GET_HOST      :
    case CURLE_QUOTE_ERROR            :
    case CURLE_HTTP_RETURNED_ERROR    :
    case CURLE_WRITE_ERROR            :
    case CURLE_READ_ERROR             :
    case CURLE_HTTP_POST_ERROR        :
    case CURLE_GOT_NOTHING            :
    case CURLE_SEND_ERROR             :
    case CURLE_RECV_ERROR             :
    case CURLE_BAD_CONTENT_ENCODING   :
    case CURLE_SEND_FAIL_REWIND       :
    case CURLE_LOGIN_DENIED           :
    case CURLE_TFTP_PERM              :
    case CURLE_REMOTE_DISK_FULL       :
    case CURLE_TFTP_UNKNOWNID         :
    case CURLE_AGAIN                  :
    // case CURLE_FTP_BAD_FILE_LIST      :
      main_status = jfDown::FAILED;
      break;
    default:
      main_status = jfDown::BLOCKED;
  }
  /**/tparent->TLog(fname,3,jfDown::FetchResult2String(main_status));
  // we also need to get the http response code
  xresult = curl_easy_getinfo(curl_handle,CURLINFO_RESPONSE_CODE,&respval);
  if (xresult==CURLE_OK) result_code = respval;
  else result_code = 0;
  /**/tparent->TLogI(fname,4,result_code);
  /**/tparent->TLogS(fname,5,rawdown.size);
  /* checking the response code, for problems, since curl has no problem
  with downloading a 404 page missing and declaring a success */
  if ((result_code==404) && fail404) {
    main_status = jfDown::MISSING;
  }
  else if ((result_code!=200) && failnon200) {
    main_status = jfDown::FAILED;
  }
  // clearing up any temporary stuff
  if (main_status!=jfDown::FETCHED) {
    if (rawdown.memory!=NULL) {
      free(rawdown.memory);
      rawdown.memory = NULL;
    }
    rawdown.size = 0;
  }
  // setting some additional values
  retryable = ((main_status==jfDown::FAILED) || (main_status==jfDown::TIMEDOUT) ||
               (main_status==jfDown::SSL_ERROR));
  if (main_status==jfDown::FAILED) failcount++;
  /**/tparent->TLogS(fname,6,failcount);
  // done
  return (main_status==jfDown::FETCHED);
}
//-------------------------------------------------
// trying to get the character set for string conversion
wxString jfCurlPageRetriever::GetCharset() const {
  // variables
  const wxString fname = wxT("jfCurlPageRetriever::GetCharset");
  CURLcode xres;
  char* ct;
  wxString cset, contentype, buffer;
  size_t startpos;
  // assuming the character set is in the html <header>, we try to get it from raw data
  /**/tparent->TLog(fname,1);
  /**/tparent->TAssertLog(connected,fname,wxT("Function called while not connected!"));
  cset = ExtractCharset(reinterpret_cast<const wxByte*>(rawdown.memory),rawdown.size);
  /**/tparent->TLog(fname,2,cset);
  // next try is header data
  if (cset.IsEmpty()) {
    xres = curl_easy_getinfo(curl_handle,CURLINFO_CONTENT_TYPE,&ct);
    /**/tparent->TLog(fname,3);
    if (xres==CURLE_OK) {
      contentype = wxString(ct,wxConvISO8859_1);
      /**/tparent->TLog(fname,4,contentype);
      startpos = contentype.find(wxT("charset="));
      if (startpos!=wxString::npos) {
        buffer = contentype.Right(contentype.Length() - (startpos+8));
        // processing what we get
        buffer.Trim();  buffer.Trim(false);
        if (buffer.GetChar(0) == wxT('\"')) {
          // we remove any leading double quotes
          buffer = buffer.Right(buffer.Length()-1);
        }
        if (buffer.Last() == wxT('\"')) {
          // we remove any leading double quotes
          buffer = buffer.Left(buffer.Length()-1);
        }
        // things okay
        /**/tparent->TLog(fname,5,buffer);
        cset = buffer;
      }
    }
    /**/tparent->TLog(fname,6);
  }
  return cset;
}
//-------------------------------------------------
// rawdata to wxString
bool jfCurlPageRetriever::RawToString(const wxString& charset_in) {
  // variables
  const wxString fname = wxT("jfCurlPageRetriever::RawToString");
  bool isutf8;
  size_t strlen;
  wxCSConv converter(charset_in.c_str());
  wxString encoding;
  wxMutexLocker qlock(mlock);
  // some initial checks
  if (rawdown.memory==NULL) return false;
  if (rawdown.size==0) return false;
  AccumulateCheck();
  // starting the conversion
  isutf8 = ((charset_in==wxT("utf-8")) || (charset_in==wxT("UTF-8")));
  outval = new wxString(rawdown.memory,converter,rawdown.size);
  strlen = outval->length();
  /**/tparent->TLogS(fname,1,strlen);
  if (strlen==0) {
    // our fallback is iso-8859-1
    encoding = wxT("iso8859-1");
    converter = wxCSConv(encoding.c_str());
    delete outval;
    outval = new wxString(rawdown.memory,converter,rawdown.size);
    strlen = outval->length();
    /**/tparent->TLogS(fname,2,strlen);
    if (strlen==0) {
      delete outval;
      outval = NULL;
    }
  }
  /**/tparent->TLog(fname,3);
  // done
  return (strlen!=0);
}
//-------------------------------------------------
bool jfCurlPageRetriever::CheckResultData() {
  const wxString fname = wxT("jfCurlPageRetriever::CheckResultData");
  /**/tparent->TLog(fname,1);
  if (internal_const->CheckMissing!=NULL) {
    if (internal_const->CheckMissing(outval)) {
      main_status=jfDown::MISSING;
      retryable = false;
      /**/tparent->TLog(fname,2);
      return false;
    }
  }
  if (internal_const->CheckTruncated!=NULL) {
    if (internal_const->CheckTruncated(outval)) {
      main_status = jfDown::FAILED;
      failcount++;
      retryable = true;
      /**/tparent->TLog(fname,3);
      return false;
    }
  }
  /**/tparent->TLog(fname,4);
  return true;
}
//-------------------------------------------------
// does a full download
bool jfCurlPageRetriever::FetchPage() {
  // variables
  const wxString fname = wxT("jfCurlPageRetriever::FetchPage");
  wxString charset;
  // calling the sub-functions
  /**/tparent->TLog(fname,1);
  if (!Connect()) return false;
  /**/tparent->TLog(fname,2);
  if (!CheckResultCodes()) return false;
  /**/tparent->TLog(fname,3);
  allfetched = true;
  charset = GetCharset();
  /**/tparent->TLog(fname,4,charset);
  // once here, we do not need the downloader
  curl_easy_cleanup(curl_handle);
  curl_handle = NULL;
  connected = false;
  // prepping for the next step
  /**/tparent->TLog(fname,5);
  if (charset.IsEmpty()) charset = wxT("utf-8");
  if (!RawToString(charset)) return false;
  /**/tparent->TLog(fname,6);
  free(rawdown.memory);
  rawdown.memory = NULL;
  rawdown.size = 0;
  /**/tparent->TLog(fname,7);
  // finally
  return CheckResultData();
}
//----------------------------------------------------------------------------
// used in keeping a running total of the download amount
bool jfCurlPageRetriever::AccumulateCheck() {
  wxMutexLocker rt_lock(globalsettings::down_mutex);
  bool rval = false;
  // checking if we need to reset the value
  if (wxDateTime::Today()>=globalsettings::month_later) {
    globalsettings::totaldown = 0;
    globalsettings::SetZeroDate(globalsettings::reset_day);
    rval = true;
  }
  // accumulating
  globalsettings::totaldown += rawdown.size;
  return rval;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// pausing or halting
bool jfCurlPageRetriever::PauseHaltCheck(){
  return internal_const->pauselink->PauseHaltCheck();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfCurlPageRetriever::Initialize() {
  // externally passed data
  internal_const = NULL;
  tparent = NULL;
  urlset = false;
  // connection stuff
  curl_handle = NULL;
  res = CURLE_OK;
  connected = false;
  result_code = 0;
  // download stuff
  rawdown.size = 0;
  rawdown.memory = NULL;
  allfetched = false;
  retryable = false;
  outval = NULL;
  // status
  cstatus = jfDown::NOTSTARTED;
  failcount = 0;
  fail404 = true;
  failnon200 = false;
}
//-------------------------------------------------------------------------------
// clearing up after a faliure in preparation to try again
void jfCurlPageRetriever::ClearThings() {
  // constants
  const wxString fname = wxT("jfCurlPageRetriever::ClearThings");
  // the curl handle
  if (curl_handle!=NULL) {
    curl_easy_cleanup(curl_handle);
    curl_handle = NULL;
    connected = false;
    result_code = 0;
  }
  if (rawdown.memory!=NULL) {
    free(rawdown.memory);
    rawdown.memory = NULL;
    rawdown.size = 0;
  }
  allfetched = false;
  retryable = false;
  redirect.Clear();
  outval = NULL;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// msg passing
void jfCurlPageRetriever::SendRetry(int rcount) {
  // creating
  wxCommandEvent eventx(jf_DOWNL_RETRY_EVENT, internal_const->msg_target->GetId());
  // setting data
  eventx.SetInt(rcount);
  // sending the message
  wxPostEvent(internal_const->msg_target,eventx);
}
//--------------------------------------------------------------------
// sending a message we are embarking on a long pause
void jfCurlPageRetriever::RequestPause() {
  // creating
  wxCommandEvent eventx(jf_DOWNL_RPAUSE_EVENT, internal_const->msg_target->GetId());
  // sending the message
  wxPostEvent(internal_const->msg_target,eventx);
}
//****************************************************************************
