//****************************************************************************
// Name:        curl_getter.h
// Author :     John Q Metro
// Purpose :    A replacement for jfPageRetriever that uses libcurl
// Created:     November 21, 2013
// Updated:     July 23, 2015 (added AccumulateCheck method)
//****************************************************************************
#ifndef CURL_GETTER_H_INCLUDED
#define CURL_GETTER_H_INCLUDED
#endif // CURL_GETTER_H_INCLUDED
//--------------------------------------
#ifndef DOWN_DATA_H_INCLUDED
  #include "down_data.h"
#endif // DOWN_DATA_H_INCLUDED
//--------------------------------------

#include <curl/curl.h>
#ifdef WIN32
  #include "wx/msw/winundef.h"
#endif
//****************************************************************************


struct jfCurlMemoryStruct {
  char *memory;
  size_t size;
};
// a forward declaration
class jfCoreThread;
//===============================================================
class jfCurlPageRetriever : public wxObject {
    public:
      // constructors
      jfCurlPageRetriever();
      jfCurlPageRetriever(jfDownloadParams* inval, wxString url_in, jfCoreThread* tparent_in);
      bool SetStartData(jfDownloadParams* inval, wxString url_in, jfCoreThread* tparent_in);
      bool UrlIsSet() const;
      // the connection
      bool Connect();
      int GetResponse() const;
      size_t GetFailcount() const;
      wxString GetRedirect() const;
      bool CheckForRedirection();
      // getting the page
      bool DownloadPage();
      jfDown::FetchResult GetFaliureReason() const;
      wxString* GetResult();
      // destructor
      ~jfCurlPageRetriever();
    protected:
      // post-connect helper methods
      bool CheckResultCodes();
      wxString GetCharset() const;
      bool RawToString(const wxString& charset_in);
      bool CheckResultData();
      bool FetchPage();
      bool AccumulateCheck();
      // internal helper
      bool PauseHaltCheck();
      // internal methods
      void Initialize();
      void ClearThings();
      // msg passing
      void SendRetry(int rcount);
      void RequestPause();
      // externally passed data
      jfDownloadParams* internal_const;
      wxString url2get;
      jfCoreThread* tparent;
      bool urlset;
      // connection stuff
      CURL *curl_handle;
      CURLcode res;
      bool connected;
      int result_code;
      // download stuff
      struct jfCurlMemoryStruct rawdown;
      bool allfetched;
      wxString* outval;
      bool retryable;
      wxString redirect;
      // status
      mutable wxMutex mlock;
      jfDown::FetchResult main_status;
      jfDown::DownStatus cstatus;
      bool fail404;
      bool failnon200;
      size_t failcount;

};
//****************************************************************************
