/******************************************************************************
Name    :   multi_threads.h
Author  :   John Q Metro
Purpose :   Multi source-category base thread classes
Created :   December 14, 2013
Updated :   January 10, 2013
******************************************************************************/
#ifndef MULTI_THREADS_H_INCLUDED
#define MULTI_THREADS_H_INCLUDED
#endif // MULTI_THREADS_H_INCLUDED
//----------------------------------------
#ifndef CORE_THREADS_H_INCLUDED
  #include "core_threads.h"
#endif // CORE_THREADS_H_INCLUDED
#ifndef MORE_THREADS_H_INCLUDED
  #include "more_threads.h"
#endif // MORE_THREADS_H_INCLUDED
//------------------------------------------
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
/*****************************************************************************/
// multi-catgegory thread-download mixin, yay!
template <class DT>
class jfMultiThreadMixin : public DT {
  public:
    // constructor
    jfMultiThreadMixin();
    // messages
    void SendCatLabels(wxString cat_label, wxString proc_label);
    void SendNewCat(int lindex, int ltotal);
    void SendCatEnd(jfDown::ResultAction done_status);
    void SendCatCount();
  protected:
    void PostCatSend(bool skip);
    bool PostCatHaltCheck();
    // source category data
    wxString catname;
    size_t catindex;
    size_t max_catcount;
};
//================================================================
// constructor
template <class DT>
jfMultiThreadMixin< DT >::jfMultiThreadMixin():DT() {
  catindex = 0;
  max_catcount = 0;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
template <class DT>
void jfMultiThreadMixin< DT >::SendCatLabels(wxString cat_label, wxString proc_label) {
  jfThreadEvent eventx(jf_DOWNL_CATLABELS_EVENT,(this->jfThreadWithDownload::msg_target)->GetId(),
                       this->jfThreadWithDownload::soption);
  eventx.SetStrings(cat_label,proc_label);
  wxPostEvent(this->jfThreadWithDownload::msg_target,eventx);
}
//--------------------
template <class DT>
void jfMultiThreadMixin< DT >::SendNewCat(int lindex, int ltotal) {
  jfThreadEvent eventx(jf_DOWNL_CATSTART_EVENT,(this->jfThreadWithDownload::msg_target)->GetId(),
                       this->jfThreadWithDownload::soption);
  eventx.SetInts(lindex,ltotal);
  eventx.SetString1(catname);
  wxPostEvent(this->jfThreadWithDownload::msg_target,eventx);
}
//--------------------
template <class DT>
void jfMultiThreadMixin< DT >::SendCatEnd(jfDown::ResultAction done_status) {
  jfThreadEvent eventx(jf_DOWNL_CATEND_EVENT,(this->jfThreadWithDownload::msg_target)->GetId(),
                       this->jfThreadWithDownload::soption);
  eventx.SetResAction(done_status);
  wxPostEvent(this->jfThreadWithDownload::msg_target,eventx);
}
//--------------------
template <class DT>
void jfMultiThreadMixin< DT >::SendCatCount() {
  jfThreadEvent eventx(jf_DOWNL_CATCOUNT_EVENT,(this->jfThreadWithDownload::msg_target)->GetId(),
                       this->jfThreadWithDownload::soption);
  eventx.SetInt1(max_catcount);
  wxPostEvent(this->jfThreadWithDownload::msg_target,eventx);
}
//++++++++++++++++++++++++++++++++++++++++++++++++
// bundles the task of sending the right category-done message
template <class DT>
void jfMultiThreadMixin< DT >::PostCatSend(bool skip) {
  if ((this->jfThreadWithDownload::fres) == jfDown::FETCHED) SendCatEnd(jfDown::PROCEED);
  else if ((this->jfThreadWithDownload::fres) == jfDown::ABORTED) {
    if (!(this->jfThreadWithDownload::haltsent)) SendCatEnd(jfDown::HALT);
  }
  else if (skip) SendCatEnd(jfDown::SKIP);
  else SendCatEnd(jfDown::FAIL);
}
//---------------------------------------------
/* After the category, we check for halt and special stop. Returns false if
   we are to stop. */
template <class DT>
bool jfMultiThreadMixin< DT >::PostCatHaltCheck() {
  bool dohalt1 = (this->jfThreadWithDownload::dconst)->pauselink->PauseHaltCheck();
  bool dohalt2 = (this->jfThreadWithDownload::dconst)->pauselink->SpecialStopCheck();
  if (dohalt1 || dohalt2) {
    (this->jfThreadWithDownload::fres)=jfDown::ABORTED;
    jfThreadWithDownload::SendHalt();
    return false;
  }
  else return true;
}
//**************************************************************************
// == [ Classes that use jfMultiThreadMixin ] ==
//+++++++++++++++++++++++++++++++++++++++++++++++++
class jfMultiDownloadThread : public jfMultiThreadMixin<jfDownloadThread> {
  public:
    // constructor
    jfMultiDownloadThread();
};
//+++++++++++++++++++++++++++++++++++++++++++++++++
class jfMultiSearchThreadBase : public jfMultiThreadMixin<jfSingleSearchThreadBase> {
  public:
    // constructor
    jfMultiSearchThreadBase();
  protected:
    // virtual void PreFetch1() = 0;
    virtual void PreFetch2();
    // virtual void SetUrl() = 0;
    // virtual bool ProcessPage() = 0;
    virtual size_t NewPagecount() const;
    virtual void PostPageAction();
};

/*****************************************************************************/

