/////////////////////////////////////////////////////////////////////////////
// Name:        expr_base.cpp
// Author :     John Q Metro
// Purpose :    Defines some expression element classes
// Created:     26.12.06
// Updated:     Febuary 16, 2012
/////////////////////////////////////////////////////////////////////////////

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
#ifndef JFEXPRBASE
#include "expr_base.h"
#endif
#ifndef JFILTERGLOBALS
#include "../filters/base/filter_globals.h"
#endif
#ifndef JFMOREBASE
#include "../filters/morebase.h"
#endif
#ifndef UTILS1_H_INCLUDED
#include "../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../utils/logging.h"
#endif // LOGGING_H_INCLUDED
//****************************************************************************
const wxString expr_operators = wxT("&|%~");
const wxString expr_operands = wxT("smtqlge");
const wxString expr_scitems = expr_operators + wxT("()");
const wxString expr_precstring = wxT("&3|3%3~5");
//****************************************************************************
// a special function for copying an element
jfExprElem* CopyElement(jfExprElem* source) {
	// global variables
	jfExprElem *rtype;
	jfOperandElem* tempo;
	wxChar gtype;
	// basic checking...
	assert(source!=NULL);
	// the main two way divisiton
	if (source->IsOperand()) {
		tempo = dynamic_cast<jfOperandElem*>(source);
		gtype = tempo->GetType();
		// another two way division
		if (tempo->IsString()) {
			// handling duplication of string elements
			if ((gtype==wxT('s')) || (gtype==wxT('t'))) {
				rtype = new jfSubstringElem(*(dynamic_cast<jfSubstringElem*>(source)));
			}
			else if ((gtype==wxT('m')) || (gtype==wxT('q'))) {
				rtype = new jfTokenMElem(*(dynamic_cast<jfTokenMElem*>(source)));
			}
			else assert(false);

		}
		else {
			// handling duplication of filter elements
			if (gtype == wxT('l')) {
				rtype = new jfLocalFilterElem(*(dynamic_cast<jfLocalFilterElem*>(source)));
			}
			else if (gtype == wxT('g')) {
				rtype = new jfGlobalFilterElem(*(dynamic_cast<jfGlobalFilterElem*>(source)));
			}
			else {
				rtype = new jfLiteralFilterElem(*(dynamic_cast<jfLiteralFilterElem*>(source)));
			}
		}
	}
	else {
		// producing a new copy
		rtype = new jfOperatorElem(*(dynamic_cast<jfOperatorElem*>(source)));
	}
	// done
	return rtype;
}
//============================================================================
// getting precedence
size_t GetPrec(const wxChar& intype) {
	// local variables
	wxString xchar;
	size_t xres;
	unsigned long* xval;
	// looking
	xres = expr_precstring.find(intype,0);
	// checking the result...
	assert(xres!=(wxString::npos));
	assert((xres%2)==0);
	// now we get the precedence char
	xchar = expr_precstring[xres+1];
	xval = new unsigned long;
	// converting...
	if (xchar.ToULong(xval)) {
		// sucessful conversion
		xres = (*xval);
		delete xval;
		return xres;
	}
	// the conversion has failed...
	else {
		delete xval;
		assert(false);
		return 0;
	}
}
//============================================================================
/* Name Verification is used on elements that have links to external filters, or
 may have them. Two possible errors can arise: one, the referred filters may not
 exist, two, the filters may refer to themselves, directly or indirectly. NameVerif
 is a class used in verification to avoid this */
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the default constructor
jfNameVerif::jfNameVerif() {
	local_list=NULL;
	local_count=0;
	global_list=NULL;
	global_count=0;
	inerror=false;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// some special error handling methods
//--------------------------------------------------------------------------
// the chainstring is listing of filters that lead up to this point
bool jfNameVerif::MakeChainString() {
	// variables
	size_t lcounter, gcounter;
	wxString rtotal, cbuf;
	// we loop thru local list
	for (lcounter=0;lcounter<local_count;lcounter++) {
		cbuf = local_list->Item(lcounter);
		rtotal += cbuf;
		if ((lcounter-1)!=local_count) rtotal += wxT("->");
	}
	// adding in a -> if need be
	if ((local_count!=0) && (global_count!=0)) rtotal += wxT("->");
	// we loop thru global list
	for (gcounter=0;gcounter<global_count;gcounter++) {
		// mapname first
		cbuf = global_list->Item(2*gcounter);
		rtotal += cbuf + wxT(';');
		// filter name second
		cbuf = global_list->Item((2*gcounter)+1);
		// arrow if need be
		if ((gcounter-1)!=global_count) rtotal += wxT("->");
	}
	// almost done
	chainstring = rtotal;
	return !rtotal.IsEmpty();
}
//--------------------------------------------------------------------------
wxString jfNameVerif::GetChainString() const {
	return chainstring;
}
//--------------------------------------------------------------------------
bool jfNameVerif::InGlobal() const {
	return (global_count!=0);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// looking for names
//--------------------------------------------------------------------------
// looks thru the list of local filters for names
bool jfNameVerif::FindName(const wxString& fname) {
	// variables
	size_t lcounter;
	bool found=false;
	wxString cbuf;
	// we loop thru local list
	for (lcounter=0;lcounter<local_count;lcounter++) {
		cbuf = local_list->Item(lcounter);
		found = (cbuf==fname);
		if (found) break;
	}
	return found;
}
//--------------------------------------------------------------------------
// looking thru the list of global filters for names
bool jfNameVerif::FindName(const wxString& mapname, const wxString& fname) {
  // constants
  const wxString fxname = wxT("jfNameVerif::FindName");
	// variables
	size_t gcounter;
	bool found=false;
	wxString cbuf;
	// checks
	/**/jfXLogMessageS(fxname,1,global_count);
	/**/jfXLogMessageB(fxname,2,global_list!=NULL);
	/**/jfXLogMessage(fxname,4,mapname);
	// we loop thru global list
	for (gcounter=0;gcounter<global_count;gcounter++) {
		// checking for mapname equivalency
		cbuf = global_list->Item(2*gcounter);
		/**/jfXLogMessage(fxname,5,cbuf);
		found = (cbuf==mapname);
		if (found) {
			// we now check the filter name
			cbuf = global_list->Item(1+(2*gcounter));
			/**/jfXLogMessage(fxname,6,cbuf);
			found = (cbuf==fname);
		}
		if (found) break;
	}
	/**/jfXLogMessage(fxname,7);
	return found;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// name push and pop
//--------------------------------------------------------------------------
// pushes an item on the global list
void jfNameVerif::PushGlobal(const wxString& mapname, const wxString& fname) {
	if (global_list==NULL) global_list = new wxArrayString;
	global_list->Add(mapname);
	global_list->Add(fname);
	global_count++;
}
//--------------------------------------------------------------------------
// pushes an item onto the local list
bool jfNameVerif::PushLocal(const wxString& fname) {
	if (InGlobal()) return false;
	if (local_list==NULL) local_list = new wxArrayString;
	local_list->Add(fname);
	local_count++;
	return true;
}
//--------------------------------------------------------------------------
/* pops a name off the list. given that global follows local, we only pop
 off the local list once the is global is exhaused */
bool jfNameVerif::PopName() {
	// removing from the global list
	if (InGlobal()) {
		global_list->RemoveAt(2*(global_count-1),2);
		global_count--;
		if (global_count==0) {
			delete global_list;
			global_list=NULL;
		}
		return true;
	}
	// removing a name from the local list
	else if (local_count!=0) {
		local_list->RemoveAt(local_count-1);
		local_count--;
		if (local_count==0) {
			delete local_list;
			local_list = NULL;
		}
		return true;
	}
	// truying to pop from a completely empty list
	else return false;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting and getting the error
//--------------------------------------------------------------------------
bool jfNameVerif::InError() const {
	return inerror;
}
//--------------------------------------------------------------------------
void jfNameVerif::SetLocalError(const wxString& lfiltname) {
	errormsg = wxT("Error: local filter reference \"") + lfiltname;
	errormsg += wxT("\" from global filter:\n");
	MakeChainString();
	errormsg += chainstring;
	inerror = true;
}
//--------------------------------------------------------------------------
void jfNameVerif::SetNotFoundError(const wxString& fname) {
	errormsg = wxT("Error: Local referenced filter \"") + fname;
	errormsg += wxT("\" is not found.");
	MakeChainString();
	if (!chainstring.IsEmpty()) {
		errormsg += wxT(" Referenced from:\n");
		errormsg += chainstring;
	}
	inerror = true;
}
//--------------------------------------------------------------------------
void jfNameVerif::SetNotFoundError(const wxString& mapname, const wxString& fname) {
	errormsg = wxT("Error: Global referenced filter \"") + mapname + wxT(';') + fname;
	errormsg += wxT("\" is not found.");
	MakeChainString();
	if (!chainstring.IsEmpty()) {
		errormsg += wxT(" Referenced from:\n");
		errormsg += chainstring;
	}
	inerror = true;
}
//--------------------------------------------------------------------------
void jfNameVerif::SetCircularError(const wxString& fname) {
	errormsg = wxT("Error: Circular reference. Filter \"") + fname;
	errormsg += wxT("\" Referenced from:\n");
	errormsg += chainstring;
	inerror = true;
}
//--------------------------------------------------------------------------
void jfNameVerif::SetCircularError(const wxString& mapname, const wxString& fname) {
	errormsg = wxT("Error: Circular reference. Filter \"") + mapname;
	errormsg += wxT(';') + fname;
	errormsg += wxT("\" Referenced from:\n");
	errormsg += chainstring;
	inerror = true;
}
//--------------------------------------------------------------------------
wxString jfNameVerif::GetError() const {
	return errormsg;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the destructor
jfNameVerif::~jfNameVerif() {
	if (global_list!=NULL) delete global_list;
	if (local_list!=NULL) delete local_list;
}
//============================================================================
wxChar jfExprElem::GetType() const { return type; }
//============================================================================
// the constructor for operators
jfOperatorElem::jfOperatorElem(wxChar typestr) {
	bool xpas;
	xpas = (expr_scitems.find(typestr) != wxString::npos);
	assert(xpas);
	type = typestr;
}
//---------------------------------------------------------------------------
jfOperatorElem::jfOperatorElem(const jfOperatorElem& source) {
	type = source.type;
}
//---------------------------------------------------------------------------
bool jfOperatorElem::IsOperand() const { return false; }
//--------------------------------------------------------------------------
wxString jfOperatorElem::String() const {
	wxString res;
	res = wxT("<");
	if (type==wxT('&')) res+= wxT("&");
	res += wxString(type) + wxT(">");
	return res;
}
//===========================================================================
// the following two methods are pretty simple
bool jfOperandElem::IsOperand() const { return true; }
//---------------------------------------------------------------------------
bool jfOperandElem::IsValid() const { return valid; }
//===========================================================================
// basic extensions for substring matching
wxString jfStrElem::GetPattern() const { return pattern; }
//---------------------------------------------------------------------------
bool jfStrElem::GetCasesen() const { return casesen; }
//---------------------------------------------------------------------------
bool jfStrElem::IsString() const { return true; }
//---------------------------------------------------------------------------
bool jfStrElem::GetTitle() const { return title; }
//--------------------------------------------------------------------------
wxString jfStrElem::String() const {
	wxString res;
	res = wxT("<");
	res+= wxString(type) + wxT(";");
	if (casesen) res+= wxT('t');
	else res+= wxT('f');
	if (title) res+= wxT('t');
	else res+= wxT('f');
	res+= wxT(';') + pattern+wxT(">");
	return res;
}
//===========================================================================
// the substring matching element
// the constructor is below
jfSubstringElem::jfSubstringElem(const wxString& newpattern, bool newcs,bool istitle) {
	// loading the basics
	pattern = newpattern;
	Dtrim(pattern);
	assert(!(pattern.IsEmpty()));
	// setting the values
	casesen = newcs;
	valid = true;
	if (istitle) type = wxT('t');
	else type = wxT('s');
	title = istitle;
}
//---------------------------------------------------------------------------
// the copy constructor
jfSubstringElem::jfSubstringElem(const jfSubstringElem& source) {
	type = source.type;
	pattern = source.pattern;
	casesen = source.casesen;
	title = source.title;
	valid = source.valid;
}
//---------------------------------------------------------------------------
// substring matching
bool jfSubstringElem::Match(const wxString& base) const {
	wxString newbase, newpattern;
	// special checking
	newbase = base;
	Dtrim(newbase);
	if (newbase.IsEmpty()) return false;
	// converting by case sensetivity
	if (!casesen) {
		newpattern = pattern.Lower();
		newbase.MakeLower();
	}
	else newpattern = pattern;
	// performing the check
	return (newbase.find(newpattern)!=(wxString::npos));
}
//============================================================================
// token matching
//---------------------------------------------------------------
jfTokenMElem::jfTokenMElem(const wxString& newpattern, bool newcs,bool istitle) {
	// loading the basics
	pattern = newpattern;
	Dtrim(pattern);
	assert(!(pattern.IsEmpty()));
	// setting the values
	casesen = newcs;
	valid = true;
	if (istitle) type = wxT('q');
	else type = wxT('m');
	title = istitle;
	// tokenizing..
	tdata = TokenizeString(pattern,false);
}
//---------------------------------------------------------------
// the copy constructor
jfTokenMElem::jfTokenMElem(const jfTokenMElem& source) {
	type = source.type;
	pattern = source.pattern;
	casesen = source.casesen;
	title = source.title;
	valid = source.valid;
	if (source.tdata == NULL) tdata = NULL;
	else tdata = new wxArrayString(*(source.tdata));
}
//---------------------------------------------------------------
// token matching, a bit different from substring matching...
bool jfTokenMElem::Match(const wxArrayString* base) const {
  // constants
  const wxString fname = wxT("jfTokenMElem::Match");
  // variables
	wxString newbase,newpattern;
	size_t index1,index2;
	size_t countp,countb;
	bool found;
	// basic checking...
	if (pattern==wxT("*")) return true;
	if (base==NULL) return false;
	countp = tdata->GetCount();
	countb = base->GetCount();
	if (countb<countp) return false;
	// the loop
	for(index1=0;index1<(countb-(countp-1));index1++) {
		// getting the token
		newbase = base->Item(index1);
		if (!casesen) newbase.MakeLower();
    // converting by case sensetivity...
    newpattern = tdata->Item(0);
    if (!casesen) newpattern.MakeLower();
		// checking to see if it matches the pattern
		if (newbase.Matches(newpattern)) {
			if (countp == 1) return true;
			// for multi-item pattern, we match more...
			found = true;
			for (index2=(index1+1);index2<(index1+countp);index2++) {
				// getting the token
				newbase = base->Item(index2);
				if (!casesen) newbase.MakeLower();
        // converting by case sensetivity...
        newpattern = tdata->Item(index2-index1);
        if (!casesen) newpattern.MakeLower();
				// matching
				if (!newbase.Matches(newpattern)) {
					found = false;
					break;
				}
			}
			// we've exited the inner loop, have we found anything?
			if (found) return true;
		}
	}
	return false;
}
//-------------------------------------------------------------------------
// the destructor
jfTokenMElem::~jfTokenMElem() {
	if (tdata!=NULL) {
		delete tdata;
		tdata = NULL;
	}
}
//==========================================================================
/* Operand class for embedded filters literals. We disallow embedded expression
 filters (they are redundant). This simplifies things quite a bit. */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// constructors
//---------------------------------------------------------------------------
jfLiteralFilterElem::jfLiteralFilterElem(const wxString& ftype, const wxString& rawdata) {
	bool testx;
	// basic assignments
	type = wxT('e');
	valid = false;
	filttype = ftype;
	source = rawdata;
	thefilter = NULL;
	// checking
	assert(filttype!=EXP_FIL_TAG);
	// we try to create the filter
	thefilter = MakeFilter(ftype,wxT("tempnamex1"));
	if (thefilter!=NULL) {
		testx = thefilter->FromString(rawdata);
		if (!testx) {
			delete thefilter;
			valid = false;
			thefilter = NULL;
		}
		else valid = true;
	}
	// done
}
//---------------------------------------------------------------------------
jfLiteralFilterElem::jfLiteralFilterElem(const jfLiteralFilterElem& fsource) {
	type = wxT('e');
	valid = fsource.valid;
	filttype = fsource.filttype;
	source = fsource.source;
	thefilter = fsource.thefilter->GenCopy();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// public methods
bool jfLiteralFilterElem::Match(const jfBasePD* testelem) const {
	// basic checks
	assert(valid);
	assert(testelem!=NULL);
	// the match
	return (thefilter->Match(testelem));
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// virtual methods that need to be defined
//---------------------------------------------------------------------------
wxString jfLiteralFilterElem::String() const {
	wxString res;
	res = wxT('<');
	res += wxString(type) + wxT(';');
	res+= filttype;
	res += wxT(';') + source + wxT('>');
	return res;
}
//---------------------------------------------------------------------------
bool jfLiteralFilterElem::IsString() const {
	return false;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destuctor
jfLiteralFilterElem::~jfLiteralFilterElem() {
	if (thefilter!=NULL) delete thefilter;
}
//==========================================================================
// a base class for embedded filter operands
//--------------------------------------------------------------------------
bool jfFilterElem::IsString() const { return false; }
//---------------------------------------------------------------
// does the matching
bool jfFilterElem::Match(const jfBasePD* testelem) const {
	// basic checks
	assert(valid);
	assert(testelem!=NULL);
	// the match
	return (thefilter->Match(testelem));
}
//==========================================================================
// external refernced filter elem constructor !!!
jfGlobalFilterElem::jfGlobalFilterElem(const wxString& inmapname, const wxString& filtername) {
	// setting the basic values
	type = wxT('g');
	mapname = inmapname;
	filter_name = filtername;
	valid = false;
	thefilter = NULL;
	// when copying, we link the pointer as well...
	c_pointer = filterglobals::global_filters->GetPointer(mapname,filter_name);
}
//---------------------------------------------------------------------------
jfGlobalFilterElem::jfGlobalFilterElem(const jfGlobalFilterElem& source) {
	// copying the most basic stuff over
	type = source.type;
	// copying the strings
	mapname = source.mapname;
	filter_name = source.filter_name;
	// copying the filters...
	thefilter = NULL;
	// the pointer may no longer be actually valid...
	c_pointer = NULL;
	valid = false;
}
//----------------------------------------------------------------------
// tries to see if the filter linked to is an expression filter
bool jfGlobalFilterElem::IsExpr() const {
	if (c_pointer==NULL) return false;
	else {
		return ((c_pointer->filter)->MatchTypeid(EXP_FIL_TAG));
	}
}
//-------------------------------------------------------------------------
/* for externally referenced filters, this function is the most critical */
bool jfGlobalFilterElem::VerfyNames(jfNameVerif* nameobj){
  //
  const wxString fname = wxT("jfGlobalFilterElem::VerfyNames");
	// basic asserts
	assert(nameobj!=NULL);
	//local vars
	jfExpressionFilter* temp;
	bool resval;
	// first, we search for loops
	/**/jfXLogMessage(fname,1,mapname);
	/**/jfXLogMessage(fname,2,filter_name);
	resval = nameobj->FindName(mapname, filter_name);
	if (resval) {
		nameobj->SetCircularError(mapname,filter_name);
		valid = false;
		/**/jfXLogMessage(fname,3);
		return false;
	}
	/* we'll leave pushing the current name to the expression filter, if the
	 filter we reference *is* an expression filter */
	// next, we try to link...
	if (c_pointer!=NULL) delete c_pointer;
	c_pointer = filterglobals::global_filters->GetPointer(mapname,filter_name);
	/**/jfXLogMessage(fname,4,mapname);
	/**/jfXLogMessage(fname,5,filter_name);
	// the link has failed, and the global filter is not found!
	if (c_pointer==NULL) {
		// error...
		valid = false;
		/**/jfXLogMessage(fname,6);
		nameobj->SetNotFoundError(mapname,filter_name);
		return false;
	}
	/**/jfXLogMessage(fname,7);
	// the link is a success!
	// we now check to see if what we see should be looked at more...
	if (IsExpr()) {
	  /**/jfXLogMessage(fname,8);
		temp = dynamic_cast<jfExpressionFilter*>(c_pointer->filter);
		// the filter is an expression, so we call its own VerifyNames
		/**/jfXLogMessage(fname,9,temp->ToString());
		resval = temp->VerifyNames(nameobj,mapname);
		valid = resval;
		/**/jfXLogMessageB(fname,10,valid);
		return resval;
	}
	// the linked filter is not an expression, no problem
	else {
	  /**/jfXLogMessage(fname,11);
		valid = true;
		return true;
	}
}
//--------------------------------------------------------------------------
// to avoid problems with a filter changing while searching, the external
// filters are copied before any search is launch -- but not before
bool jfGlobalFilterElem::CopyLoad() {
	// to make sure names are validated first...
	assert(valid);
	assert(c_pointer!=NULL);
	// local variables
	jfExpressionFilter* temp;
	// going ahead and copying
	thefilter = (c_pointer->filter)->GenCopy();
	valid = true;
	// now, send the copying deeper
	if (IsExpr()) {
		temp = dynamic_cast<jfExpressionFilter*>(thefilter);
		if (temp->CopyLoad()) valid = true;
		else {
			// the following undoes copying at an element level
			delete thefilter;
			thefilter = NULL;
			valid = false;
		}
	}
	return valid;
}
//--------------------------------------------------------------------------
wxString jfGlobalFilterElem::String() const {
	wxString res;
	res = wxT('<');
	res += wxString(type) + wxT(';');
	res+= mapname;
	res += wxT(';') + filter_name + wxT('>');
	return res;
}
//---------------------------------------------------------------------------
jfGlobalFilterElem::~jfGlobalFilterElem() {
	if (thefilter!=NULL) delete thefilter;
	if (c_pointer!=NULL) delete c_pointer;
}
//===========================================================================
// the constructor for the referenced filter
jfLocalFilterElem::jfLocalFilterElem(jfFilterMap* inlocalmap, const wxString& filtername) {
	// starting
	assert(inlocalmap!=NULL);
	// settling the basic values
	type = wxT('l');
	filter_name = filtername;
	localmap = inlocalmap;
	thefilter = NULL;
	valid = false;
}
//---------------------------------------------------------------------------
// the copy constructor
jfLocalFilterElem::jfLocalFilterElem(const jfLocalFilterElem& source) {
	// copying the most basic stuff over
	type = source.type;
	valid = source.valid;
	filter_name = source.filter_name;
	localmap = source.localmap;
	thefilter = NULL;
}
//-------------------------------------------------------------------------
// checks if the referenced filter is an expression filter
bool jfLocalFilterElem::IsExpr() const {
	if (thefilter==NULL) return false;
	else {
		return (thefilter->MatchTypeid(EXP_FIL_TAG));
	}
}
//---------------------------------------------------------------------------
/* filter checking is similar to that for the global filter reference, with
 the extra bit checking that we do not go from global to local */
bool jfLocalFilterElem::VerfyNames(jfNameVerif* nameobj) {
  // constants
  const wxString fname = wxT("jfLocalFilterElem::VerfyNames");
	// basic asserts
	assert(nameobj!=NULL);
	//local vars
	jfExpressionFilter* temp;
	bool resval;
	// we check if the filter reference chain is global
	/**/jfXLogMessage(fname,1);
	if (nameobj->InGlobal()) {
		nameobj->SetLocalError(filter_name);
		valid = false;
		return false;
	}
	/**/jfXLogMessage(fname,2);
	// first, we search for loops
	resval = nameobj->FindName(filter_name);
	if (resval) {
		nameobj->SetCircularError(filter_name);
		valid = false;
		return false;
	}
	/**/jfXLogMessage(fname,4);
	/* we'll leave pushing the current name to the expression filter, if the
	 filter we reference *is* an expression filter */
	// next, we try to link...
	if (thefilter!=NULL) delete thefilter;
	thefilter = localmap->GetItem(filter_name);
	/**/jfXLogMessage(fname,5);
	// the link has failed, and the local filter is not found!
	if (thefilter==NULL) {
		// error...
		valid = false;
		nameobj->SetNotFoundError(filter_name);
		return false;
	}
	/**/jfXLogMessage(fname,6);
	// the link is a success!
	// we now check to see if what we see should be looked at more...
	if (IsExpr()) {
		temp = dynamic_cast<jfExpressionFilter*>(thefilter);
		// the filter is an expression, so we call its own VerifyNames
		valid = temp->VerifyNames(nameobj,false);
	}
	// the linked filter is not an expression, no problem
	else valid = true;
	/**/jfXLogMessageB(fname,7,valid);
	// finishing up
	thefilter = NULL;
	return valid;
}
//--------------------------------------------------------------------------
// to avoid problems with a filter changing while searching, the external
// filters are copied before any search is launch -- but not before
bool jfLocalFilterElem::CopyLoad() {
	// to make sure names are validated first...
	assert(valid);
	// local variables
	jfExpressionFilter* temp;
	// the basic copying
	thefilter = (localmap->GetItem(filter_name))->GenCopy();
	// let us check if we need to send the copying deeper...
	if (IsExpr()) {
		temp = dynamic_cast<jfExpressionFilter*>(thefilter);
		if (temp->CopyLoad()) valid = true;
		else {
			delete thefilter;
			thefilter = NULL;
			valid = false;
		}
	}
	else valid = true;
	return valid;
}
//----------------------------------------------------------------------
wxString jfLocalFilterElem::String() const {
	wxString res;
	res = wxT('<');
	res += wxString(type) + wxT(';');
	res+= filter_name + wxT('>');
	return res;
}
//---------------------------------------------------------------------------
jfLocalFilterElem::~jfLocalFilterElem() {
	if (thefilter!=NULL) delete thefilter;
}
//****************************************************************************
