/////////////////////////////////////////////////////////////////////////////
// Name:        expr_match.cpp
// Author :     John Q Metro
// Purpose :    Defines functions that use the expression matches
// Created:     April 1, 2010
// Updated:     April 1, 2010
/////////////////////////////////////////////////////////////////////////////
#ifndef JFEXPRMATCH
	#include "expr_match.h"
#endif
#ifndef JFEXPRTOKEN
#include "expr_token.h"
#endif
//**************************************************************************
// a matching function that uses everything: title, embedded filters, the works
bool FullItemExprMatch(const jfBasePD* testelem, const jfElemArray* parsedinfo) {
  // constants
  const wxString fname = wxT("FullItemExprMatch");
	// the default checks
  assert(testelem!=NULL);
	assert(parsedinfo!=NULL);
  // local variables
  jfTruthStack* tstack;
  jfOperandElem* topnd;
  jfFilterElem* tfilte;
  jfSubstringElem* t_string;
  jfTokenMElem* t_token;
  jfLiteralFilterElem* lf_token;
  wxChar theop;
  size_t eindex;
  bool tresult;
  bool fresult;
  // setting things up for evaluation
  tstack = new jfTruthStack();
  // evaluation
  for (eindex=0;eindex<(parsedinfo->size());eindex++) {
    // if things are an operand, we evaluate and push
    if ((*parsedinfo)[eindex]->IsOperand()) {
      topnd = dynamic_cast<jfOperandElem*>((*parsedinfo)[eindex]);
      // evaluating strings
      if (topnd->IsString()) {
        theop = topnd->GetType();
        // substring evaluation
        if ((theop==wxT('s')) || (theop==wxT('t')) ) {
          t_string = dynamic_cast<jfSubstringElem*>(topnd);
          if (t_string->GetTitle()) {
            tresult = t_string->Match(testelem->GetName());
          }
          else {
            tresult = t_string->Match(testelem->GetDescription());
          }

        }
        // tokenized evaluation
        else if ((theop==wxT('m'))||(theop==wxT('q'))) {
          t_token = dynamic_cast<jfTokenMElem*>(topnd);
          if (t_token->GetTitle()) {
            tresult = t_token->Match(testelem->GetTokTitle());
          }
          else {
            tresult = t_token->Match(testelem->GetTokSummary());
          }
        }
        else assert(false);

      }
      // evaluating an embedded filter
      else if ((topnd->GetType())==wxT('e')) {
        lf_token = dynamic_cast<jfLiteralFilterElem*>(topnd);
        tresult = lf_token->Match(testelem);
      }
      // evaluating linked filter operands
      else {
        tfilte = dynamic_cast<jfFilterElem*>(topnd);
        tresult = tfilte->Match(testelem);
      }
      tstack->push(tresult);
    }
    // otherwise, we do logical ops on the stack
    else {
      theop = (*parsedinfo)[eindex]->GetType();
      if (theop==wxT('~')) tstack->Not();
      else if (theop==wxT('&')) tstack->And();
      else if (theop==wxT('|')) tstack->Or();
      else if (theop==wxT('%')) tstack->Xor();
      else assert(false);
    }
  }
  // done, we now get the result and clear
  assert(tstack->One());
  fresult = tstack->top();
  delete tstack;
  testelem->ClearTok();
  // done
  return fresult;
}
//------------------------------------------------------------------------------
// a more limited matching function that works on strings only
bool StringExprMatch(const wxString& teststr, const jfElemArray* parsedinfo) {
    // constants
    const wxString fname = wxT("@!!!!!! StringExprMatch");

		// the default checks
		assert(parsedinfo!=NULL);
		// local variables
		jfTruthStack* tstack;
		jfOperandElem* topnd;
		jfSubstringElem* t_string;
		jfTokenMElem* t_token;
		wxArrayString* token_str;
		wxChar theop;
		size_t eindex;
		bool tresult;
		bool fresult;
		// setting things up for evaluation
		tstack = new jfTruthStack();
		// evaluation
		for (eindex=0;eindex<(parsedinfo->size());eindex++) {
			// if things are an operand, we evaluate and push
			if ((*parsedinfo)[eindex]->IsOperand()) {
				topnd = dynamic_cast<jfOperandElem*>((*parsedinfo)[eindex]);
				// evaluating strings
				if (topnd->IsString()) {
					theop = topnd->GetType();
					// substring evaluation
					if (theop==wxT('s'))  {
						t_string = dynamic_cast<jfSubstringElem*>(topnd);
						tresult = t_string->Match(teststr);
					}
					// tokenized evaluation
					else if (theop==wxT('m')) {
						t_token = dynamic_cast<jfTokenMElem*>(topnd);
						token_str = TokenizeString(teststr,false);
						tresult = t_token->Match(token_str);
						delete token_str;
					}
					else assert(false);
				}
				// nothing other than strings allowed for this version
				else assert(false);
				tstack->push(tresult);
			}
			// otherwise, we do logical ops on the stack
			else {
				theop = (*parsedinfo)[eindex]->GetType();
				if (theop==wxT('~')) tstack->Not();
				else if (theop==wxT('&')) tstack->And();
				else if (theop==wxT('|')) tstack->Or();
				else if (theop==wxT('%')) tstack->Xor();
				else assert(false);
			}
		}
		// done, we now get the result and clear
		assert(tstack->One());
		fresult = tstack->top();
		delete tstack;
		// done
		return fresult;
}
//**************************************************************************
