/////////////////////////////////////////////////////////////////////////////
// Name:        expr_parse.cpp
// Author :     John Q Metro
// Purpose :    Defines functions and stuff for parsing boolean expressions
// Created:     31.12.06
// Updated:     August 22, 2012 (getting rid of exceptions)
/////////////////////////////////////////////////////////////////////////////

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

//----------------------------------------------------------------------------
#ifndef JFEXPRPARSE
#include "expr_parse.h"
#endif
#ifndef LOGGING_H_INCLUDED
  #include "../utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
#include "../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef JFMOREBASE
#include "../filters/base/morebase.h"
#endif
//*****************************************************************************
const wxString v_startc = wxT("(~");
const wxString v_aftert = wxT(")&|%");
const wxString v_stype = wxT("smtq");
const wxString v_idchars = v_stype + wxT("ic");
//============================================================================
// a simple function for looking for chars
bool IsIn(const wxChar& src, const wxString& place) {
	return (place.find_first_of(src)!=wxString::npos);
}
//============================================================================
// the constructor
jfExpParserClass::jfExpParserClass(const wxString& insource, bool issimple,jfFilterMap* inlocalmap) {
	rawexp = insource;
	started = false;
	done = false;
	gotten= false;
	tokenizer = NULL;
	theresult = NULL;
	simple = issimple;
	if (simple) assert(inlocalmap==NULL);
	else localmap = inlocalmap;
	fname1 = wxT("size_t jfExpParserClass::ParseExpression()");
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// activation
bool jfExpParserClass::ParseExpression(size_t& outsize) {
  // constants
  const wxString fname=wxT("jfExpParserClass::ParseExpression");
	// local variables
	bool wasop;
	wxString data;
	wxChar lastc, currc;
	jfOperandElem* temp;
	// clearing old stuff
	// starting up
	assert(!started);
	wasop = false;
	tokenizer = new jfExprToker(rawexp);
	theresult = new jfElemArray();
	started = true;
	// a special case: everything is empty!
	if (!(tokenizer->TokensLeft())) {
		delete tokenizer;
		tokenizer = NULL;
		done = true;
		outsize = 0;
		return true;
	}
	// we work on the first element
	// we've got an operator or parenthesis
	if (tokenizer->AtOp()) {
		lastc = tokenizer->GetTChar();
		if (!IsIn(lastc,v_startc)) {
			return MakeError(wxT("The expression starts with an invalid op!"),true);
		}
		// things are okay
		theresult->Append(lastc);
		wasop = true;
	}
	// we've got an operand here
	else {
		data = tokenizer->GetToken();
		temp = MakeOperand(data);
		if (temp==NULL) return false;
		theresult->Append(temp);
	}
	// we work on subseqent elements
	while (tokenizer->TokensLeft()) {
		// we've got an operator
		if (tokenizer->AtOp()) {
			currc = tokenizer->GetTChar();
			// if the preceeding token was an operand, we check for syntax errors
			if (!wasop) {
				if (IsIn(currc,v_startc)) {
					return MakeError(wxT("This symbol is not allowed after an operand!"),true);
				}
			}
			// if it was an operator, we check for other errors
			else {
				if (lastc==wxT(')')) {
					if (IsIn(currc,v_startc)) {
						return MakeError(wxT("This symbol is not allowed after a closing parentheis!"),true);
					}
				}
				else {
					if (!IsIn(currc,v_startc)) {
						return MakeError(wxT("This symbol is not allowed after most operators!"),true);
					}
				}
			}
			// everything checks out
			theresult->Append(currc);
			wasop = true;
			lastc = currc;
		}
		// we've got an operand
		else {
			// getting data
			data = tokenizer->GetToken();
			// iff non-empty...
			if (!(data.IsEmpty())) {
				// operands cannot follow operands
				if (!wasop) {
					return MakeError(wxT("Operands cannot follow operands!"),true);
				}
				// otherwise, we do a quick check to see if we follow the correct op
				if (lastc==wxT(')')) {
					return MakeError(wxT("Operands cannot follow right parens!"),true);
				}
				// everything is ok...
				temp = MakeOperand(data);
				if (temp==NULL) return false;
				theresult->Append(temp);
				wasop = false;
			}
		}
		// the loop continues here
	}
	// the if the last token is an operator, the only one allowed is ')'
	if (wasop) {
		if (lastc!=wxT(')')) {
			return MakeError(wxT("The expression must not end this way!"));
		}
	}
	// done!
	delete tokenizer;
	tokenizer = NULL;
	done = true;
	outsize = theresult->size();
	return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting the result
jfElemArray* jfExpParserClass::GetResult() {
	assert(done);
	gotten = true;
	return theresult;
}
//--------------------------------------------------------------------------------------
wxString jfExpParserClass::GetRawExp() const {
	return rawexp;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// done
jfExpParserClass::~jfExpParserClass() {
	if ((theresult==NULL) && (!gotten)) delete theresult;
	if (tokenizer!=NULL) delete tokenizer;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// when the parser his an error, this method is called, it takes care of cleanup
// before throwing an error message
bool jfExpParserClass::MakeError(const wxString& msg, bool back) {
	size_t tokloc;
	assert(started);
	assert(!done);
	// deleting the result
	theresult->DeleteAll();
	delete theresult;
	theresult = NULL;
	// getting info about the loacation
	if (back) tokloc = tokenizer->LastIndex();
	else tokloc = tokenizer->GetIndex();
	// deleting the tokenizer
	delete tokenizer;
	tokenizer = NULL;
	// building the result
	parse_error = wxT("PARSING ERROR in ") + fname1 + wxT(" at position ");
	parse_error += SInt2Str(tokloc) + wxT(" : ") + msg;
	// done
	return false;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfOperandElem* jfExpParserClass::MakeOperand(const wxString& source) {
  // constants
  const wxString fname = wxT("jfExpParserClass::MakeOperand");
	// local varibales
	wxString xsrc = source;
	wxString fpart,spart;
	jfOperandElem* res = NULL;
	size_t semiloc,slen;
	bool stok, istit, iscs;
	// checking to see if this is an embedded filter
	if (FilterTest(xsrc,wxT('#'))) {
		// filters are not allowed in simple expressions
		if (simple) {
		  MakeError(wxT("Filters are not allowed in somple expressions!"),true);
		  return NULL;
		}
		if (xsrc.length()<3) {
		  MakeError(wxT("The filter literal is badly formed!"),true);
		  return NULL;
		}
		// looking for the splitter
		semiloc = FindNonEscaped(xsrc,wxT(";"),0);
		if (semiloc==wxString::npos) {
		  MakeError(wxT("The filter literal is badly formed!"),true);
		  return NULL;
		}
		// we split
		SplitString(xsrc,semiloc,fpart,spart);
		// initial checking for problems
		if (fpart==EXP_FIL_TAG) {
		  MakeError(wxT("Expression filter literals are not allowed!"),true);
		  return NULL;
		}
		if (fpart.IsEmpty() || spart.IsEmpty()) {
      MakeError(wxT("The filter literal is badly formed!"),true);
      return NULL;
		}
		// we now work on creating the embedded filter
		res = new jfLiteralFilterElem(fpart,spart);
		if (!(res->IsValid())) {
			delete res;
			MakeError(wxT("The embedded filter is problematic!"),true);
			return NULL;
		}
	}
	// checking to see if this a filter reference operand
	else if (FilterTest(xsrc,wxT('@'))) {
		if (simple) {
		  MakeError(wxT("Filters are not allowed in somple expressions!"),true);
		  return NULL;
    }
		// check for zero length stuff
		if (xsrc.length()<1) {
		  MakeError(wxT("The filter operand is badly formed!"),true);
		  return NULL;
		}
		// looking for the splitter
		semiloc = FindNonEscaped(xsrc,wxT(";"),0);
		// local vs global
		if (semiloc==wxString::npos) {
			// local
			UnEscape(xsrc,wxT(";"));
			res = new jfLocalFilterElem(localmap,xsrc);
		}
		else {
			// global
			SplitString(xsrc,semiloc,fpart,spart);
			if (fpart.IsEmpty() || spart.IsEmpty()) {
			  MakeError(wxT("The global filter reference is badly formed!"),true);
			  return NULL;
			}
			UnEscape(fpart,wxT(";"));
			UnEscape(spart,wxT(";"));
			// making the filter
			res = new jfGlobalFilterElem(fpart,spart);
		}
	}
	// it is actually a string operand...
	else {
		// we look for a split...
		semiloc = FindNonEscaped(xsrc,wxT(";"),0);
		// if there is no split, we treat it a a case insensetive non-title substring
		if (semiloc == wxString::npos) res = new jfSubstringElem(xsrc);
		// otherwise...
		else {
			// we split things into two parts
			slen = xsrc.length();
			fpart = xsrc.substr(0,semiloc);
			Dtrim(fpart);
			spart = xsrc.substr(semiloc+1,slen-(semiloc+1));
			UnEscape(spart,wxT(";"));
			Dtrim(spart);
			// running a quick error check
			if (spart.IsEmpty()) {
			  MakeError(wxT("The string operand is empty!"),true);
			  return NULL;
			}
			if (!CheckId(fpart,stok,iscs,istit)) {
				MakeError(wxT("The id string is invalid!"),true);
				return NULL;
			}
			// titles are not in simple expressions
			if (simple && istit) {
			  MakeError(wxT("Simple expressions do not have title strings"),true);
			  return NULL;
			}
			// everthing is ok, we now create...
			if (stok) res = new jfSubstringElem(spart,iscs,istit);
			else res = new jfTokenMElem(spart,iscs,istit);
		}
	}
	return res;
}
//-------------------------------------------------------
bool jfExpParserClass::FilterTest(wxString& sourcef, wxChar delimt) const {
	size_t mlen;
	bool arroba;
	// running some basic checks
	mlen = sourcef.length();
	assert(mlen>0);
	if (mlen<2) return false;
	// we now see if things are a filter
	arroba = (sourcef[0]==delimt);
	if (!arroba) return false;
	arroba = (sourcef[mlen-1]==delimt);
	if (!arroba) return false;
	// a final check...
	if (mlen>=3) {
		if (sourcef[mlen-2]==wxT('\\')) {
			sourcef = sourcef.substr(0,mlen-1);
			sourcef[mlen-2] = delimt;
			return false;
		}
	}
	// we now assume things are a filter, so we trim the '@''s off
	sourcef = sourcef.substr(1,mlen-2);
	Dtrim(sourcef);
	// done
	return true;
}

//-------------------------------------------------------------------------
// deciphers the id that can be placed in fron of string tokens
bool jfExpParserClass::CheckId(const wxString& idin, bool& ssout, bool& csout,
							   bool& titout) const {
	// local variables
	wxChar xchar,xchar2;
	size_t spos;
	size_t idlen = idin.length();
	bool chkcs = false;
	// a basic check
	if (idlen>2) return false;
	// checking for invalid chars
	if (wxString::npos != idin.find_first_not_of(v_idchars)) return false;
	// we now move on to a special case: zero length id
	if (idlen==0) {
		ssout = true;
		csout = true;
		titout = false;
		return true;
	}
	// with that out of the way, we look for the type char
	spos = idin.find_first_of(v_stype);
	// case no type id found
	if (spos==wxString::npos) {
		// the id must be length one, ( 'c' or 'i' only )
		if (idlen!=1) return false;
		chkcs = true;
		xchar2 = idin[0];
		// defualts
		ssout = true;
		titout = false;
	}
	// a type id is found, we extract
	else {
		xchar = idin[spos];
		// we prepare for checking the other char, if needed
		if (idlen==2) {
			xchar2 = idin[(spos+1)%2];
			chkcs = true;
		}
		// testing what we extracted
		if (xchar==wxT('s'))      { ssout = true; titout = false;}
		else if (xchar==wxT('m')) { ssout = false; titout = false;}
		else if (xchar==wxT('t')) { ssout = true; titout = true;}
		else                      { ssout = false; titout = true;}
	}
	// moving on to case sensetive processing
	if (chkcs) {
		if (xchar2==wxT('c')) csout = true;
		else if (xchar2==wxT('i')) csout = false;
		else return false;
	}
	else csout = false;
	// done
	return true;

}
//=============================================================================
// following function is designed to take in an element array on produce a postfix
// expression verion
jfElemArray* MakeExprPostfix(jfElemArray* infixsrc, wxString& outerr){
	// local variables
	jfElemArray* theresult;
	jfExprElem* tcopy;
	size_t src_count;
	wxChar tchar,pchar;
	size_t tprec, pprec;
	jfOperatorStack* opstack;
	bool opnot, stop;
	wxString fname1 = wxT("MakeExprPostfix(jfElemArray* infixsrc)");
	// starting....
	assert(infixsrc!=NULL);
	src_count = infixsrc->size();
	// a basic check
	if (src_count==0) return new jfElemArray();
	// setting up for the loop
	opstack = new jfOperatorStack();
	theresult = new jfElemArray();
	// the evaluation loop
	for (size_t eindex = 0; eindex<src_count; eindex++) {
		// here we look for operators and parens
		if (infixsrc->NotOperand(eindex)) {
			// getting the type char
			tchar = ((*infixsrc)[eindex])->GetType();
			// lparens are puched on the stack
			if (tchar==wxT('(')) opstack->push(tchar);
			// rparens, we pop and write everything until a lparen is reached
			else if (tchar==wxT(')')) {
				while (opstack->NotLParen()) {
					// a major error
					if (opstack->empty()) {
						delete opstack;
						delete theresult;
						outerr = wxT("EXPRESSION ERROR in ") + fname1 + wxT(" : ");
						outerr += wxT("Mismatched parenthesis!");
						return NULL;
					}
					// ok!
					theresult->Append(opstack->PopElem());
				}
				// we discard what is left
				opstack->pop();
			}
			// we have encountered an operator, ~ is treated differently
			else {
				// basic checking
				opnot = (tchar==wxT('~'));
				tprec = GetPrec(tchar);
				// the popping loop
				while (opstack->IsOp()) {
					// checking the top of the stack
					pchar = opstack->top();
					pprec = GetPrec(pchar);
					// seeing if we stop popping and push
					if (opnot) stop = (pprec<=tprec);
					else stop = (pprec<tprec);
					// if we don't stop, we pop and write the operator
					if (!stop) {
						theresult->Append(opstack->PopElem());
					}
					else break;
				}
				// pushing the operand
				opstack->push(tchar);
			}
		}
		// we've got operands!
		else {
			// we just copy them over
			tcopy = CopyElement((*infixsrc)[eindex]);
			theresult->Append(tcopy);
		}
	}
	// we're at the end of the scanning loop
	// we now pop anything left on the stack
	while (opstack->IsOp()) {
		theresult->Append(opstack->PopElem());
	}
	// checking for errors
	if (opstack->IsLParen()) {
		delete opstack;
		delete theresult;
    outerr = wxT("EXPRESSION ERROR in ") + fname1 + wxT(" : ");
    outerr += wxT("Mismatched parenthesis!");
    return NULL;
	}
	// clearing up
	delete opstack;
	return theresult;
}
//*****************************************************************************
