/////////////////////////////////////////////////////////////////////////////
// Name:        expr_parse.h
// Author :     John Q Metro
// Purpose :    Declares functions and stuff for parsing boolean expressions
// Created:     31.12.06
// Updated:     August 22, 2012 (removing exceptions)
/////////////////////////////////////////////////////////////////////////////

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

// for all others, include the necessary headers
#ifndef WX_PRECOMP

#endif
// ----------------------------------------------------------------------------

//----------------------------------------------------------------------------
#ifndef JFEXPRSTRUCTS
#include "expr_structs.h"
#endif
#define JFEXPRPARSE
//*****************************************************************************
/* The source format of the tokens is as follows

 FILTER LITERAL TOKENS: start and end with '#'
 - The first part is the filter type, the second a
 string representation of the data contents.

 FILTER REFERENCE TOKENS: start and end with '@'
 - for global reference filter tokens, the first part is the map name, the second the
 filter name. The two parts are separated by ';'
 - for local filter tokens, the contents are just the local filter name.

 STRING TOKENS: by default, they are substring case insensetive
 -However, a sort of 'id' at the start, followed by ';' can change this.
 -This id cannot be more than 2 characters long.
 -character it can contain: 'c' or 'i' to set case sensetivity
 -Also, 's', 'm', 't', or 'q' to set the type
 */
//*****************************************************************************
// because of the complication of parsing, using a class can make parts simpler
class jfExpParserClass : public wxObject {
public:
    wxString parse_error;
    // the constructor
    jfExpParserClass(const wxString& insource,bool issimple,jfFilterMap* inlocalmap);
    // activation
    bool ParseExpression(size_t& outsize);
    // getting the result
    jfElemArray* GetResult();
    wxString GetRawExp() const;
    // done
    ~jfExpParserClass();
protected:
    // constants
    wxString fname1;
    // used in parsing
    wxString rawexp;
    bool started, done, gotten;
		bool simple;
    jfExprToker* tokenizer;
    jfElemArray* theresult;
    jfFilterMap* localmap;
    // private parsing helper
    bool MakeError(const wxString& msg, bool back=false);
    // internal parsing methods
    jfOperandElem* MakeOperand(const wxString& source);
    bool IsLiteralFilter(wxString& sourcef);
    bool IsFilterReference(wxString& sourcef);
    bool FilterTest(wxString& sourcef, wxChar delimt) const;
    bool CheckId(const wxString& idin, bool& ssout, bool& csout, bool& titout) const;
};
//=============================================================================
// following function is designed to take in an element array on produce a postfix
// expression verion
jfElemArray* MakeExprPostfix(jfElemArray* infixsrc, wxString& outerr);
//****************************************************************************
