/******************************************************************************
Name    :   ao3_specials2.cpp
Author  :   John Q Metro
Purpose :   AO3 pairing and warning filters
Created :   October 4, 2012
Updated :   October 12 2012
******************************************************************************/
#ifndef AO3_SPECIALS2_H_INCLUDED
  #include "ao3_specials2.h"
#endif // AO3_SPECIALS2_H_INCLUDED
//--------------------------------
#ifndef UTILS1_H_INCLUDED
  #include "../../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef AO3_FICOBJ_H_INCLUDED
  #include "../../../fanfic/data/ao3/ao3_ficobj.h"
#endif // AO3_FICOBJ_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../utils/logging.h"
#endif // LOGGING_H_INCLUDED
//---------------------------------
const wxString warn3_ac = wxT("VDRPN");
const wxChar* warnlist[5] = { wxT("Violence"),wxT("Character Death"),wxT("Rape"),wxT("Underage Sex"), wxT("Warnings Avoided")};
/*****************************************************************************/
// constructors
//---------------------------
jfAO3PairFilter::jfAO3PairFilter():jfBaseFilter() {
  pnames = NULL;
  alternate = true;
}
//------------------------------
jfAO3PairFilter::jfAO3PairFilter(const jfAO3PairFilter& source):jfBaseFilter() {
  // copying base elements
  description = source.description;
  num_id = source.num_id;
  validdata = source.validdata;
  name = source.name + wxT(", copy");
  if (source.pnames==NULL) pnames = NULL;
  else {
    pnames = new wxArrayString(*source.pnames);
  }
  alternate = source.alternate;
}
//+++++++++++++++++++++++++++++++++++++++++++
// public methods implemented here
//---------------------------
bool jfAO3PairFilter::IsEmpty() const {
  if (pnames==NULL) return true;
  else return (pnames->Count()==0);
}
//++++++++++++++++++++++++++++++++++++++++++++++++
// string conversion
//----------------------------------
bool jfAO3PairFilter::FromString(const wxString& sourcedata) {
  validdata = false;
  if (pnames!=NULL) delete pnames;
  // parsing
  jfLineParse lparse(sourcedata);
  if (lparse.NNotX(2)) return false;
  // getting...
  if (!lparse.BoolVal(0,alternate)) return false;
  wxString srcstuff = lparse.GetT(1);
  return SetNamesData(srcstuff);
}
//-------------------------------
wxString jfAO3PairFilter::ToString() const {
  return Bool2Str(alternate) + wxT(";") + GetNamesData();
}
//---------------------------------
bool jfAO3PairFilter::SetNamesData(const wxString& sourcedata) {
  validdata = false;
  if (pnames!=NULL) delete pnames;
  wxString qnames = sourcedata;
  Dtrim(qnames);
  pnames = ParsePairs(qnames);
  if ((pnames==NULL) && (!qnames.IsEmpty())) return false;
  validdata = true;
  return true;
}
//---------------------------------
wxString jfAO3PairFilter::GetNamesData() const {
  if (pnames==NULL) return wxEmptyString;
  size_t pcount,pindex;
  wxString result;
  // starting
  pcount = GetPairs();
  for (pindex=0;pindex<pcount;pindex++) {
    if (!result.IsEmpty()) result += wxT(",");
    result += pnames->Item(2*pindex);
    result += wxT("/");
    result += pnames->Item(2*pindex+1);
  }
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++
// gets a description
wxString jfAO3PairFilter::GetTypeDescription() const {
  return wxT("Matches against the list of pairings that comes with an \
Archiveofourown.org fic. The filter contains lists of string pairs, like \
'harry' and 'ginny', at least one of the pairings must match one specified \
by the fic (unless the filter is empty).");
}
//-------------------------------
// copy
jfBaseFilter* jfAO3PairFilter::GenCopy() const {
  jfAO3PairFilter* newfilt;
  newfilt = new jfAO3PairFilter();
  CopyOver(newfilt);
  if (pnames!=NULL) {
    newfilt->pnames = new wxArrayString(*pnames);
  }
  return newfilt;
}
//------------------------------
// special meta-information
wxString jfAO3PairFilter::GetTypeID() const {
  return wxT("AO3PairFilter");
}
//------------------------------
// other methods
size_t jfAO3PairFilter::GetPairs() const {
  size_t psize;
  if (pnames==NULL) return 0;
  psize = (pnames->Count()) / 2;
  return psize;
}
//--------------------------------
void jfAO3PairFilter::SetAlternates(bool inval) {
  alternate = inval;
}
//----------------------------------
bool jfAO3PairFilter::GetAlternate() const {
  return alternate;
}
//--------------------------------
// destructor
jfAO3PairFilter::~jfAO3PairFilter() {
  if (pnames!=NULL) delete pnames;
}
//+++++++++++++++++++++++++++++++++++++++++++
bool jfAO3PairFilter::CoreMatch(const jfBasePD* testelem) const {
  const wxString fname = wxT("jfAO3PairFilter::CoreMatch");
  // variables and test
  const jfAO3Fanfic* temp;
  wxArrayString* parsedinput;
  assert(testelem!=NULL);
  size_t oloop,iloop,olmax,ilmax;
  wxString test1,test2,check2;
  bool isodd,found;
  size_t foundcount;
  // starting...
  /**/jfXLogMessage(fname,1);
  temp = dynamic_cast<const jfAO3Fanfic*>(testelem);
  parsedinput = ParsePairs(temp->GetRelationships());
  if (parsedinput==NULL) return false;
  // matching loop prep
  olmax = GetPairs();
  ilmax = parsedinput->Count();
  found = false;
  foundcount = 0;
  // outer matching loop.. using the filter info
  for (oloop=0;oloop<olmax;oloop++) {
    test1 = pnames->Item(2*oloop);
    // the inner loop
    for (iloop = 0;iloop<ilmax;iloop++) {
      if (wxNOT_FOUND!=(parsedinput->Item(iloop)).Find(test1)) {
        // we've found one name, we we check for the other
        isodd = (iloop&1);
        test1 = pnames->Item(2*oloop+1);
        if (isodd) check2 = parsedinput->Item(iloop-1);
        else check2 = parsedinput->Item(iloop+1);
        found = (wxNOT_FOUND!=check2.Find(test1));
        if ((!alternate) && found) foundcount++;
        if (found) break;
      }
    }
    if (alternate && found) break;
  }
  // after the loop, cleaning up and returning
  delete parsedinput;
  if (!alternate) found = (foundcount==olmax);
  return found;
}
//-------------------------------------------
wxArrayString* jfAO3PairFilter::ParsePairs(const wxString& inval) const {
  const wxString fname = wxT("jfAO3PairFilter::ParsePairs");
  // variables
  wxArrayString* result;
  wxString tinval,buffer;
  size_t startpos;
  int fpos;
  bool odd;
  wxChar scar;
  // starting...
  tinval = inval.Lower();
  startpos = 0;
  result = new wxArrayString();
  odd = true;
  /**/jfXLogMessage(fname,1,tinval);
  // extraction loop
  while (true) {
    scar = (odd)?(wxT('/')):(wxT(','));
    /**/jfXLogMessage(fname,2,scar);
    // extracting
    fpos = tinval.find(scar,startpos);
    if (fpos==wxNOT_FOUND) break;
    buffer = tinval.substr(startpos,fpos);
    Dtrim(buffer);
    /**/jfXLogMessage(fname,3,buffer);
    // do not tolerate empty parts
    if (buffer.IsEmpty()) {
      delete result;
      return NULL;
    }
    // finishing
    result->Add(buffer);
    startpos = fpos + 1;
    // next one...
    odd = !odd;
    /**/jfXLogMessageB(fname,4,odd);
  }
  // the last paty
  buffer = tinval.Mid(startpos);
  Dtrim(buffer);
  /**/jfXLogMessage(fname,5,buffer);
  if (buffer.IsEmpty()) {
    delete result;
    return NULL;
  }
  result->Add(buffer);
  // testing the size
  startpos = result->Count();
  /**/jfXLogMessageS(fname,4,startpos);
  if (startpos&1) {
    delete result;
    return NULL;
  }
  /**/jfXLogMessage(fname,5);
  return result;
}
//+++++++++++++++++++++++++++++++++++++++++++
// file i/o
//---------------------------
bool jfAO3PairFilter::AddRestToFile(wxTextFile* outfile) const {
  wxString buffer;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // adding
  outfile->AddLine(ToString());
  return true;
}
//---------------------------
bool jfAO3PairFilter::ReadRestFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfAO3PairFilter::ReadRestFromFile");
  // input data
  wxString cline;
  // starting checks (and reading the line)
  assert(infile!=NULL);
  // parsing the line
  if (!infile->ReadLine(cline,funcname)) return false;
  if (!FromString(cline)) return infile->BuildError(wxT("The characters do not match the filter!"));
  return true;
}
//---------------------------
size_t jfAO3PairFilter::ExtraLines() const { return 1; }
//*************************************************************************
jfAO3WarnFilter::jfAO3WarnFilter():jfSpecialsFilter()  {}
//---------------------------
jfAO3WarnFilter::jfAO3WarnFilter(const wxString& insrc):jfSpecialsFilter() {
  FromString(insrc);
}
//---------------------------
// string conversion
bool jfAO3WarnFilter::FromString(const wxString& sourcedata) {
  if (wxString::npos==sourcedata.find_first_not_of(warn3_ac)) {
    value = sourcedata;
    validdata = true;
  }
  else validdata = false;
  return validdata;
}

//---------------------------
// gets a description
wxString jfAO3WarnFilter::GetTypeDescription() const {
  return wxT("Excludes fics based on whether they have the \
specified archive warnings.");
}
//---------------------------
// copy
jfBaseFilter* jfAO3WarnFilter::GenCopy() const {
  jfAO3WarnFilter* qval = new jfAO3WarnFilter(value);
  return qval;
}
//---------------------------
// special meta-information
wxString jfAO3WarnFilter::GetTypeID() const {
  return wxT("AO3WarnFilter");
}
//---------------------------
// custom methods
void jfAO3WarnFilter::SetToFull() {
 value = warn3_ac;
}
//-----------------------------
bool jfAO3WarnFilter::IsFull() const {
  return value==warn3_ac;
}
//---------------------------
// the core matching method
bool jfAO3WarnFilter::CoreMatch(const jfBasePD* testelem) const {
  const wxString fname = wxT("jfAO3WarnFilter::CoreMatch");
  // starting
  const jfAO3Fanfic* temp = dynamic_cast<const jfAO3Fanfic*>(testelem);
  wxChar warn1 = temp->GetWarnflag();
  /**///jfXLogMessage(fname,1,warn1);
  wxString warn2 = temp->GetWarntag();
  /**///jfXLogMessage(fname,2,warn2);
  /**///jfXLogMessage(fname,3,value);
  // contains warings
  if (warn1==wxT('W')) {
    /**///jfXLogMessage(fname,4);
    if (wxString::npos!=warn2.find_first_of(value)) return false;
    else return true;
  }
  else if (warn1==wxT('N')) {
    /**///jfXLogMessage(fname,5);
    if (wxString::npos!=value.find(warn1)) return false;
    else return true;
  }
  else return true;
}
/*****************************************************************************/
