//***************************************************************************
// Name:        ao3_stringfilters.cpp
// Author :     John Q Metro
// Purpose :    Simple expression filters specific to archiveofourown.org
// Created:     September 8, 2012
// Updated:     March 16, 2014
//***************************************************************************
#ifndef AO3_STRINGFILTERS_H_INCLUDED
  #include "ao3_stringfilters.h"
#endif // AO3_STRINGFILTERS_H_INCLUDED
//----------------------------------------------
#ifndef AO3_FICOBJ_H_INCLUDED
  #include "../../../fanfic/data/ao3/ao3_ficobj.h"
#endif // AO3_FICOBJ_H_INCLUDED
#ifndef JFEXPRMATCH
	#include "../../express/expr_match.h"
#endif
#ifndef JFEXPRPARSE
#include "../../express/expr_parse.h"
#endif
#ifndef LOGGING_H_INCLUDED
  #include "../../utils/logging.h"
#endif // LOGGING_H_INCLUDED
//****************************************************************************
// constructors
//----------------------------------------------------------
jfExtraTagFilter::jfExtraTagFilter():jfSimpleExpFilterCore() {}
//----------------------------------------------------------
jfExtraTagFilter::jfExtraTagFilter(const jfExtraTagFilter& source) {
  CoreCopy(source);
}
//----------------------------------------------------------
jfExtraTagFilter::jfExtraTagFilter(jfSimpleExpr* in_source):jfSimpleExpFilterCore(in_source) {}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfExtraTagFilter::GetTypeDescription() const {
  return wxT("The Extra Tag Filter matches the contents of the extra tags against\
 a boolean expression, the elements of which are themselves to be matched. These \
elements are either strings or substrings.");
}
//------------------------------------------------------------
jfExtraTagFilter* jfExtraTagFilter::Copy() const {
  return new jfExtraTagFilter(*this);
}
//------------------------------------------------------------
jfBaseFilter* jfExtraTagFilter::GenCopy() const {
  return Copy();
}
//------------------------------------------------------------
wxString jfExtraTagFilter::GetTypeID() const {
  return wxT("ExtraTagFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfExtraTagFilter::CoreMatch(const jfBasePD* testelem) const {
  wxString datainfo;
  const jfAO3Fanfic* typed_item;
  // checks and starts
  assert(testelem!=NULL);
  typed_item = dynamic_cast<const jfAO3Fanfic*>(testelem);
  datainfo = typed_item->GetExtraTags();
  return InternalMatch(datainfo);
}
//==========================================================================
/* Intended replacement for jfACategoriesFilter, with crossover boolean, forall
filter (mainly intended to allow excluding intra-universe crossovers) and the
usual expression filter operating on the names of all fandoms */
//+++++++++++++++++++++++++++++++++++++++
// constructors
//--------------------------------
jfAO3FandomFilter::jfAO3FandomFilter():jfBaseFilter() {
  // setting the booleans
  validdata = false;
  loaded = false;
  allmatch = false;
  must_crossover = false;
  // setting parsed stuff
  perfmatch_parsed = NULL;
  mainmatch_parsed = NULL;
}
//--------------------------------
jfAO3FandomFilter::jfAO3FandomFilter(const jfAO3FandomFilter& source):jfBaseFilter() {
    // copying the most basic items
  name = source.name;
  validdata = source.validdata;
  parse_error = source.parse_error;
  // copying the booleans
  must_crossover = source.must_crossover;
  // copying per fandom stuff
  allmatch = source.allmatch;
  perfmatch_src = source.perfmatch_src;
  if (source.perfmatch_parsed!=NULL) {
    perfmatch_parsed = (source.perfmatch_parsed)->Copy();
  }
  else perfmatch_parsed = NULL;
  // copying general match stuff
  mainmatch_src = source.mainmatch_src;
  if (source.mainmatch_parsed!=NULL) {
    mainmatch_parsed = (source.mainmatch_parsed)->Copy();
  }
  else mainmatch_parsed = NULL;
}
//+++++++++++++++++++++++++++++++++++++++
//checks if the filter has no effect
bool jfAO3FandomFilter::IsEmpty() const {
  if (must_crossover) return false;
  // note that the perfmatch stuff only has meaning if must_crossover is true
  return (mainmatch_parsed==NULL)||(mainmatch_parsed->size()==0);
}
//+++++++++++++++++++++++++++++++++++++++
// loading from a string representation
//--------------------------------
bool jfAO3FandomFilter::FromString(const wxString& sourcedata) {
  // constants
  const wxString fname = wxT("jfAO3FandomFilter::FromString");
  // variables
  wxString parse1, parse2, oerr;
  bool altmatch;
  jfElemArray *psd1, *psd2;
  // starting with the parsing
  jfLineParse lparse(sourcedata);
  // we only allow 1 or 3 fields
  size_t fc = lparse.Num();
  if ((fc!=1)&&(fc!=3)) return false;
  // if three fields, then we must have a crossover
  if (fc==3) {
    if (!lparse.BoolVal(0,altmatch)) return false;
    parse1 = lparse.UnEscStrE(2);
    parse2 = lparse.UnEscStrE(1);
  }
  else parse1 = lparse.UnEscStrE(0);
  // we start with parsing the main stuff
  if (!parse1.IsEmpty()) {
    psd1 = ParseInput(parse1,oerr);
    if (psd1==NULL) {
      parse_error = oerr;
      return false;
    }
  }
  else psd1 = NULL;
  // next up, parsing the per fandom stuff...
  if (fc==3) {
    if (!parse2.IsEmpty()) {
      psd2 = ParseInput(parse2,oerr);
      if (psd1==NULL) {
        parse_error = oerr;
        if (psd1!=NULL) delete psd1;
        return false;
      }
    }
    else psd2 = NULL;
  }
  // if we get here, everything is okay. we assign values
  mainmatch_src = parse1;
  if (mainmatch_parsed!=NULL) delete mainmatch_parsed;
  mainmatch_parsed = psd1;
  must_crossover = (fc==3);
  if (perfmatch_parsed!=NULL) delete perfmatch_parsed;
  if (must_crossover) {
    perfmatch_parsed = psd2;
    perfmatch_src = parse2;
    allmatch = altmatch;
  }
  else {
    perfmatch_parsed = NULL;
    perfmatch_src.Clear();
    allmatch = false;
  }
  // done
  validdata = true;
  loaded = true;
  return true;
}
//--------------------------------
wxString jfAO3FandomFilter::ToString() const {
  if (!validdata) return wxEmptyString;
  wxString result;
  if (must_crossover) {
    result = Bool2Str(allmatch) + wxT(";");
    result += EscapeNL(perfmatch_src,EXPR_DELIMS) + wxT(";");
  }
  result += EscapeNL(mainmatch_src,EXPR_DELIMS);
  return result;
}
//+++++++++++++++++++++++++++++++++++++++
// specific data setting methods
//--------------------------------
bool jfAO3FandomFilter::GeneralFromExpr(jfSimpleExpr* in_source) {
  // checking for false things
  if (in_source==NULL) return false;
  if (!in_source->valid) return false;
  // setting the values
  if (mainmatch_parsed!=NULL) delete mainmatch_parsed;
  mainmatch_parsed = in_source->parsedinfo;
  in_source->parsedinfo = NULL;
  in_source->valid = false;
  mainmatch_src = in_source->srcstring;
  if (in_source->isempty) {
    if (mainmatch_parsed!=NULL) delete mainmatch_parsed;
    mainmatch_parsed = NULL;
  }
  return true;
}
//--------------------------------
bool jfAO3FandomFilter::PerFandomFromExpr(jfSimpleExpr* in_source) {
  // checking for false things
  if (in_source==NULL) return false;
  if (!in_source->valid) return false;
  // setting the values
  if (perfmatch_parsed!=NULL) delete perfmatch_parsed;
  perfmatch_parsed = in_source->parsedinfo;
  in_source->parsedinfo = NULL;
  in_source->valid = false;
  perfmatch_src = in_source->srcstring;
  if (in_source->isempty) {
    if (perfmatch_parsed!=NULL) delete perfmatch_parsed;
    perfmatch_parsed = NULL;
  }
  return true;
}
//--------------------------------
void jfAO3FandomFilter::SetCrossover(bool iscross) {
  must_crossover = iscross;
}
//--------------------------------
void jfAO3FandomFilter::SetAllMatch(bool amatch) {
  allmatch = amatch;
}
//+++++++++++++++++++++++++++++++++++++++
// getting data
//--------------------------------
wxString jfAO3FandomFilter::GetMainSrc() const {
  return mainmatch_src;
}
//--------------------------------
wxString jfAO3FandomFilter::GetPerFandomSrc() const {
  return perfmatch_src;
}
//--------------------------------
bool jfAO3FandomFilter::GetCrossover() const {
  return must_crossover;
}
//--------------------------------
bool jfAO3FandomFilter::GetAllMatch() const {
  return allmatch;
}
//+++++++++++++++++++++++++++++++++++++++
// Additional implemented methods
wxString jfAO3FandomFilter::GetTypeDescription() const {
  return wxT("A Filter that uses the AO3 Fandom Names with the fic.\
The General Expression filter works on the list of fandoms as a whole,\
while the other filter passes fics that either one fandom or all fandoms\
match the expression. This can be used, for example, to make sure\
crossovers are not just within a specific 'verse, like Star Trek.");
}
//--------------------------------
jfBaseFilter* jfAO3FandomFilter::GenCopy() const {
  jfAO3FandomFilter* result;
  result = new jfAO3FandomFilter(*this);
  return result;
}
//--------------------------------
wxString jfAO3FandomFilter::GetTypeID() const {
  return wxT("AO3FandomFilter");
}
//+++++++++++++++++++++++++++++++++++++++
// destructor
jfAO3FandomFilter::~jfAO3FandomFilter() {
  if (mainmatch_parsed!=NULL) delete mainmatch_parsed;
  if (perfmatch_parsed!=NULL) delete perfmatch_parsed;
}
//+++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfAO3FandomFilter::CoreMatch(const jfBasePD* testelem) const {
  // variables
  const jfAO3Fanfic* typed_item;
  wxString currfandom;
  bool expr_res;
  size_t findex,fcount;
  // we start
  assert(testelem!=NULL);
  // starting
  typed_item = dynamic_cast<const jfAO3Fanfic*>(testelem);
  // the simplest test is the crossover test
  if (must_crossover) {
    if ((typed_item->FandomCount())==1) return false;
  }
  // nest up, the mainmatch
  if (mainmatch_parsed!=NULL) {
    currfandom = typed_item->GetFandoms();
    expr_res = StringExprMatch(currfandom,mainmatch_parsed);
    if (!expr_res) return false;
  }
  // testing the crossovers!
  if (must_crossover && (perfmatch_parsed!=NULL)) {
    // prepping for the loop
    fcount = typed_item->FandomCount();
    // the matching loop
    for (findex=0;findex<fcount;findex++) {
      // checking for the match
      currfandom = typed_item->GetFandom(findex);
      expr_res = StringExprMatch(currfandom,perfmatch_parsed);
      // we now use allmatch. if allmatch is true, then one faliure spells doom
      if (allmatch && (!expr_res)) return false;
      // otherwise, one success means success
      if ((!allmatch) && expr_res) break;
    }
    // if we get here..
    if (!expr_res) return false;
  }
  // everything has passed
  return true;
}
//+++++++++++++++++++++++++++++++++++++++
// file i/o
//--------------------------------
bool jfAO3FandomFilter::AddRestToFile(wxTextFile* outfile) const {
  wxString buffer;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // adding the lines...
  buffer = Bool2Str(must_crossover) + wxT(";");
  buffer += Bool2Str(allmatch);
  outfile->AddLine(buffer);
  // per fandom line
  buffer = EscapeNL(perfmatch_src,IO_DELIMS);
  outfile->AddLine(buffer);
  // main match line
  buffer = EscapeNL(mainmatch_src,IO_DELIMS);
  outfile->AddLine(buffer);
  // done
  return true;
}
//--------------------------------
bool jfAO3FandomFilter::ReadRestFromFile(jfFileReader* infile) {
  // basic stuff
  const wxString fname = wxT("jfAO3FandomFilter::ReadRestFromFile");
  assert(infile!=NULL);
  wxString oerr,cline;
  // starting the read, boolean values
  if (!infile->ReadParseLine(2,fname)) return false;
  if (!(infile->lp).BoolVal(0,must_crossover)) {
    return infile->BuildError(wxT("The crossover field is not a boolean!"));
  }
  if (!(infile->lp).BoolVal(1,allmatch)) {
    return infile->BuildError(wxT("The all fandoms match field is not a boolean!"));
  }
  // next, the per fandom expression
  if (!infile->ReadUnEsc(cline,fname)) return false;
  perfmatch_src = cline;
  if (perfmatch_src.IsEmpty()) perfmatch_parsed = NULL;
  else {
    perfmatch_parsed = ParseInput(cline,oerr);
    if (perfmatch_parsed==NULL) {
      return infile->BuildError(wxT("PF Parsing Error : ")+oerr);
    }
  }
  // next, the main match expression
  if (!infile->ReadUnEsc(cline,fname)) return false;
  mainmatch_src = cline;
  if (mainmatch_src.IsEmpty()) mainmatch_parsed = NULL;
  else {
    mainmatch_parsed = ParseInput(cline,oerr);
    if (mainmatch_parsed==NULL) {
      return infile->BuildError(wxT("MM Parsing Error : ")+oerr);
    }
  }
  // done
  return true;
}
//+++++++++++++++++++++++++++++++++++++++
/* since different types are stored together, the text file reprentation
may have objects of varying length */
size_t jfAO3FandomFilter::ExtraLines() const {
  return 3;
}
//--------------------------------------------
// extra methods
jfElemArray* jfAO3FandomFilter::ParseInput(wxString srcstring,wxString oerr) {
  // constants
  const wxString fname = wxT("jfAO3FandomFilter::ParseInput");
  // variables
  jfExpParserClass* theparser;
  jfElemArray* pfix, *xfix;
  size_t osize;
  // starting
  /**/jfXLogMessage(fname,1,srcstring);
  Dtrim(srcstring);
  theparser = new jfExpParserClass(srcstring,true,NULL);
  // trying the parse
  if (!theparser->ParseExpression(osize)) {
    // the intial parse has failed
    /**/jfXLogMessage(fname,2,theparser->parse_error);
    oerr = theparser->parse_error;
    delete theparser;
    return NULL;
  }
  // parsed okay
  xfix = theparser->GetResult();
  assert(xfix!=NULL);
  delete theparser;
  // next, we postfixize it
  pfix = MakeExprPostfix(xfix,oerr);
  delete xfix;
  if (pfix==NULL) {
    /**/jfXLogMessage(fname,3,oerr);
    parse_error = oerr;
    return NULL;
  }
  // postfixed okay
  return pfix;
}
//============================================================
// constructors
//---------------------------------------------------
jfAO3KudoFilter::jfAO3KudoFilter():jfMinMaxUFilter() {}
//---------------------------------------------------
jfAO3KudoFilter::jfAO3KudoFilter(size_t inmin, size_t inmax):jfMinMaxUFilter(inmin,inmax) {}
//---------------------------------------------------
jfAO3KudoFilter::jfAO3KudoFilter(const jfAO3KudoFilter& source) {
    // copying the most basic items
  name = source.name;
  validdata = source.validdata;
  description = source.description;
  num_id = source.num_id;
  min = source.min;
  max = source.max;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfAO3KudoFilter::GetTypeID() const {
  return wxT("AO3KudoCountFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// returns the list of element names campatible with this filter
//---------------------------------------------------
wxString jfAO3KudoFilter::GetTypeDescription() const {
    return wxT("For items that have word counts, this filter passes if \
the kudo count is between min and max, inclusive, or if max is zero, the \
kudo count is greater than min.");
}
//---------------------------------------------------
jfBaseFilter* jfAO3KudoFilter::GenCopy() const {
  jfAO3KudoFilter* result;
  result = new jfAO3KudoFilter(min,max);
  CopyOver(result);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfAO3KudoFilter::CoreMatch(const jfBasePD* testelem) const {
  // variables
  const jfAO3Fanfic* rvalue;
  size_t cvalue;
  // checks
  assert(testelem!=NULL);
  // determining the type
  rvalue = dynamic_cast<const jfAO3Fanfic*>(testelem);
  cvalue = rvalue->GetKudos();
  // checking the wordcount
  return TestMatch(cvalue);
}
//****************************************************************************
