//***************************************************************************
// Name:        ao3_stringfilters.h
// Author :     John Q Metro
// Purpose :    More filters specific to archiveofourown.org
// Created:     September 7, 2012
// Updated:     March 16, 2014
//***************************************************************************
#ifndef AO3_STRINGFILTERS_H_INCLUDED
#define AO3_STRINGFILTERS_H_INCLUDED
#endif // AO3_STRINGFILTERS_H_INCLUDED
//--------------------------------------
#ifndef SEXP_FILTERS_H_INCLUDED
  #include "../base/sexp_filters.h"
#endif // SEXP_FILTERS_H_INCLUDED
#ifndef MOREFILTERS1_H_INCLUDED
  #include "../extended/morefilters1.h"
#endif
//***************************************************************************
// Filtering the extra tags
class jfExtraTagFilter : public jfSimpleExpFilterCore {
  public:
    jfExtraTagFilter();
    jfExtraTagFilter(const jfExtraTagFilter& source);
    jfExtraTagFilter(jfSimpleExpr* in_source);
    virtual wxString GetTypeDescription() const;
    virtual jfExtraTagFilter* Copy() const;
    virtual jfBaseFilter* GenCopy() const;
    virtual wxString GetTypeID() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
};
//============================================================
/* Intended replacement for jfACategoriesFilter, with crossover boolean, forall
filter (mainly intended to allow excluding intra-universe crossovers) and the
usual expression filter operating on the names of all fandoms */
class jfAO3FandomFilter : public jfBaseFilter {
  public:
    wxString parse_error;
    // constructors
    jfAO3FandomFilter();
    jfAO3FandomFilter(const jfAO3FandomFilter& source);
    // checks if the filter has no effect
    virtual bool IsEmpty() const;
    // loading from a string representation
    virtual bool FromString(const wxString& sourcedata);
    virtual wxString ToString() const;
    // specific data setting methods
    bool GeneralFromExpr(jfSimpleExpr* in_source);
    bool PerFandomFromExpr(jfSimpleExpr* in_source);
    void SetCrossover(bool iscross);
    void SetAllMatch(bool amatch);
    // getting data
    wxString GetMainSrc() const;
    wxString GetPerFandomSrc() const;
    bool GetCrossover() const;
    bool GetAllMatch() const;
    // Additional implemented methods
    virtual wxString GetTypeDescription() const;
    virtual jfBaseFilter* GenCopy() const;
    virtual wxString GetTypeID() const;
    // destructor
    ~jfAO3FandomFilter();
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
    // file i/o
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    /* since different types are stored together, the text file reprentation
    may have objects of varying length */
    virtual size_t ExtraLines() const;
    // extra methods
    jfElemArray* ParseInput(wxString srcstring,wxString oerr);
    // internal data
    bool must_crossover;
    bool allmatch;
    wxString perfmatch_src;
    jfElemArray* perfmatch_parsed;
    wxString mainmatch_src;
    jfElemArray* mainmatch_parsed;
    bool loaded;
};
//==========================================================================
// numeric filter that works via the number of kudos
class jfAO3KudoFilter : public jfMinMaxUFilter {
  public:
    // constructors
    jfAO3KudoFilter();
    jfAO3KudoFilter(size_t inmin, size_t inmax);
    jfAO3KudoFilter(const jfAO3KudoFilter& source);
    // redefined virtual methods
    virtual wxString GetTypeID() const;
    virtual wxString GetTypeDescription() const;
    virtual jfBaseFilter* GenCopy() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
};
//***************************************************************************
