/////////////////////////////////////////////////////////////////////////////
// Name:        filter_globals.cpp
// Author :     John Q Metro
// Purpose :    Defines global filter constants & stuff
// Created:     30.07.06
// Updated:     July 12, 2015
/////////////////////////////////////////////////////////////////////////////

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers
#ifndef WX_PRECOMP

#endif
// ----------------------------------------------------------------------------
#ifndef JFILTERGLOBALS
  #include "filter_globals.h"
#endif
#ifndef BASECOLLECT_H_INCLUDED
  #include "../../structs/basecollect.h"
#endif // BASECOLLECT_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
//***************************************************************************
namespace filterstrd {
  const wxString newfimfils = wxT("FimThumbsFilter,FIMGenreFilter,FIMCharacterFilter,FimRatingFilter,") +
              wxString(wxT("FimThumbPercentFilter,FIMShortDescFilter,FIMDualDescFilter"));
  const wxString newao3fils = wxT("ExtraTagFilter,AO3FandomFilter,AO3RatingFilter,AO3OrientationFilter,AO3PairFilter,AO3WarnFilter,AO3KudoCountFilter");
  const wxString fimgfils = wxT("FIMGroupSCFilter,FIMGroupMCFilter");
  // filters compatible with jfBasePD
  const wxString basepd_fc = wxT("ExprFilter");
  // filters compatible with jfGoogleItem
  const wxString google_fc = basepd_fc + wxT(",") + wxT("UrlFilter");
  // filters compatible with jfGenericFanfic
  const wxString genfic_fc = basepd_fc + wxT(",") + wxT("AuthorFilter,UpdatedDateFilter") +
      wxT(",PairFilterSingle,PairFilterList,PairFilterMultiple");
  // filters compatible with jfGenericFanfic2
  const wxString genfic2_fc = genfic_fc + wxT(",") + wxT("WordCountFilter,CompletedFilter");
  // filters compatible with jfFFNItem
  const wxString ffnfic_fc = genfic2_fc + wxT(",") + wxT("CharacterFilter,CategoryFilter,") +
    wxT("FFNTagFilter,LanguageFilter,FFNRatingFilter,PublishedDateFilter,FFNFavsFilter");
  // filters compatible with jfFIM_Fanfic
  const wxString fimfic_fc = genfic2_fc + wxT(",") + newfimfils;
  // filters compatible with jfAO3_Fanfic
  const wxString ao3fic_fc = (genfic2_fc + wxT(",")) + wxT("CharacterFilter") + wxT(",")  + newao3fils;
  // filters compatible with jfFIMGroup
  const wxString fimgroup_fc = basepd_fc + wxT(",") + fimgfils;
  // all filters
  const wxString all_filters = ffnfic_fc + wxT(",") + wxT("UrlFilter,") +fimgfils
        + wxT(",") + newfimfils + newao3fils;
}
//===========================================================================

jfMapOfFiltermaps* filterglobals::global_filters = NULL;
wxArrayString* filterglobals::fclist[filterglobals::ITEM_NLIST] = {NULL,NULL,NULL,NULL,
        NULL,NULL,NULL,NULL,NULL};
//----------------------------------------------------------------------------
// initializing globals
void filterglobals::LoadStrings() {
  // constants
  const wxString fname = wxT("filterglobals::LoadStrings");
  // initial cleaning
  size_t loopc;
  for (loopc=0;loopc<filterglobals::ITEM_NLIST;loopc++) {
    if (fclist[loopc]!=NULL) delete fclist[loopc];
  }
  // creating the list of list of filters
  fclist[0] = ListToStrArr(filterstrd::all_filters,wxT(","));
  fclist[1] = ListToStrArr(filterstrd::basepd_fc,wxT(","));
  fclist[2] = ListToStrArr(filterstrd::google_fc,wxT(","));
  fclist[3] = ListToStrArr(filterstrd::genfic_fc,wxT(","));
  fclist[4] = ListToStrArr(filterstrd::genfic2_fc,wxT(","));
  fclist[5] = ListToStrArr(filterstrd::ffnfic_fc,wxT(","));
  fclist[6] = ListToStrArr(filterstrd::fimfic_fc,wxT(","));
  fclist[7] = ListToStrArr(filterstrd::ao3fic_fc,wxT(","));
  fclist[8] = ListToStrArr(filterstrd::fimgroup_fc,wxT(","));
  // emptying the global filters if need be
  if (global_filters!=NULL) {
    global_filters->EmptyMaps();
    delete global_filters;
  }
  // loading up the globals
  global_filters = new jfMapOfFiltermaps();
  global_filters->ReadFromFile(fnamex);

}
//--------------------------------------------------------------------------
void filterglobals::Shutdown() {
  // local variables
  size_t loopc;
  // removing the list of filterable object type names
  for (loopc=0;loopc<filterglobals::ITEM_NLIST;loopc++) {
    if (fclist[loopc]!=NULL) {
      delete fclist[loopc];
      fclist[loopc] = NULL;
    }
  }
  // writing to the outfile
  global_filters->WriteToFile(fnamex);
  global_filters->EmptyMaps();
  delete global_filters;
  global_filters = NULL;
}

/////////////////////////////////////////////////////////////////////////////
