/////////////////////////////////////////////////////////////////////////////
// Name:        filter_groups.cpp
// Author :     John Q Metro
// Purpose :    Defines global filter map groupings and associated stuff
// Created:     01.08.06
// Updated:     July 12, 2105
/////////////////////////////////////////////////////////////////////////////
// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif
// for all others, include the necessary headers
#ifndef WX_PRECOMP

#endif
// ----------------------------------------------------------------------------
#ifndef JFILTERGROUPS
  #include "filter_groups.h"
#endif
#ifndef BASECOLLECT_H_INCLUDED
  #include "../../structs/basecollect.h"
#endif // BASECOLLECT_H_INCLUDED
#ifndef MOREFILTERS1_H_INCLUDED
  #include "../extended/morefilters1.h"
#endif
#ifndef SEXP_FILTERS_H_INCLUDED
  #include "sexp_filters.h"
#endif // SEXP_FILTERS_H_INCLUDED
#ifndef PAIRFILTER_H_INCLUDED
  #include "../extended/pairfilter.h"
#endif // PAIRFILTER_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef MOREFILTERS2_H_INCLUDED
  #include "../extended/morefilters2.h"
#endif // MOREFILTERS2_H_INCLUDED
#ifndef MOREFILTERS3_H_INCLUDED
  #include "../extended/morefilters3.h"
#endif // MOREFILTERS3_H_INCLUDED
#ifndef FIMFILTERS1_H_INCLUDED
  #include "../fim/fimfilters1.h"
#endif // FIMFILTERS1_H_INCLUDED
#ifndef FIMFILTERS2_H_INCLUDED
  #include "../fim/fimfilters2.h"
#endif // FIMFILTERS2_H_INCLUDED
#ifndef AO3_STRINGFILTERS_H_INCLUDED
  #include "../ao3/ao3_stringfilters.h"
#endif // AO3_STRINGFILTERS_H_INCLUDED
#ifndef AO3_SPECIALS1_H_INCLUDED
  #include "../ao3/ao3_specials1.h"
#endif // AO3_SPECIALS1_H_INCLUDED
#ifndef AO3_SPECIALS2_H_INCLUDED
  #include "../ao3/ao3_specials2.h"
#endif // AO3_SPECIALS2_H_INCLUDED

#include <wx/textfile.h>
//*****************************************************************************
jfBaseFilter* MakeFilter(const wxString& typestring, const wxString& newname) {
  // local variables
  const wxString fname = wxT("%%% MakeFilter");
  jfBaseFilter* result;
  jfFFNTagFilter* itag_result;
  jfFIMGenreFilter* gtag_result;
  jfFIMCharacterFilter* ctag_result;
  // starting
  /**/jfXLogMessage(fname,1,typestring);
  // the multiple types
  if (typestring==wxT("UrlFilter")) result = new jfUrlFilter();
  else if (typestring==wxT("ExprFilter")) result = new jfExpressionFilter();
  else if (typestring==wxT("WordCountFilter")) result = new jfWordCountFilter();
  else if (typestring==wxT("FFNTagFilter")) {
    itag_result = new jfFFNTagFilter();
    itag_result->SetToEmpty();
    result = itag_result;
  }
  else if (typestring==wxT("AuthorFilter")) result = new jfAuthExprFilter();
  else if (typestring==wxT("CharacterFilter")) result = new jfCharExprFilter();
  else if (typestring==wxT("CategoryFilter")) result = new jfCategoryExprFilter();
  else if (typestring==wxT("PairFilterSingle")) result = new jfPairFilterSingle();
  else if (typestring==wxT("PairFilterList")) result = new jfPairFilterList();
  else if (typestring==wxT("PairFilterMultiple")) result = new jfPairFilterMultiple();
  else if (typestring==wxT("CompletedFilter")) result = new jfCompletedFilter();
  else if (typestring==wxT("LanguageFilter")) result = new jfLanguageExprFilter();
  else if (typestring==wxT("FFNRatingFilter")) result = new jfFFNRatingFilter();
  else if (typestring==wxT("UpdatedDateFilter")) result = new jfUpdatedDateFilter();
  else if (typestring==wxT("PublishedDateFilter")) result = new jfPublishedDateFilter();
  else if (typestring==wxT("FimThumbsFilter")) result = new jfFimThumbsFilter();
  else if (typestring==wxT("FimThumbPercentFilter")) result = new jfFimThumbPercentFilter();
  else if (typestring==wxT("FIMGenreFilter")) {
    gtag_result = new jfFIMGenreFilter();
    gtag_result->SetToEmpty();
    result = gtag_result;
  }
  else if (typestring==wxT("FIMCharacterFilter")) {
    ctag_result = new jfFIMCharacterFilter();
    ctag_result->SetToEmpty();
    result = ctag_result;
  }
  else if (typestring==wxT("FimRatingFilter")) result = new jfFimRatingFilter();
  else if (typestring==wxT("ExtraTagFilter")) result = new jfExtraTagFilter();
  else if (typestring==wxT("AO3FandomFilter")) result = new jfAO3FandomFilter();
  else if (typestring==wxT("AO3RatingFilter")) result = new jfAO3RatingFilter();
  else if (typestring==wxT("AO3OrientationFilter")) result = new jfAO3OrientationFilter();
  else if (typestring==wxT("AO3PairFilter")) result = new jfAO3PairFilter();
  else if (typestring==wxT("AO3WarnFilter")) result = new jfAO3WarnFilter();
  else if (typestring==wxT("FFNFavsFilter")) result = new jfFFNFavsFilter();
  else if (typestring==wxT("FIMGroupSCFilter")) result = new jfFIMGroupSCFilter();
  else if (typestring==wxT("FIMGroupMCFilter")) result = new jfFIMGroupMCFilter();
  else if (typestring==wxT("AO3KudoCountFilter")) result = new jfAO3KudoFilter();
  else if (typestring==wxT("FIMShortDescFilter")) result = new jfFIM_ShortDesc_ExprFilter();
  else if (typestring==wxT("FIMDualDescFilter")) result = new jfFIM_DualDesc_ExprFilter();
  else result = NULL;
  // setting the name
  /**/jfXLogMessage(fname,2,newname);
  if (result!=NULL) result->SetName(newname);
  /**/jfXLogMessageB(fname,3,(result==NULL));
  // done
  return result;
}
//=============================================================================
// returns the name of the map
wxString jfFilterPointer::GetMapName() const {
  return filtermap->GetName();
}
//----------------------------------------------------------------------------
// checks for filter names
bool jfFilterPointer::CheckForName(const wxString& fname) const {
  return filtermap->NameExists(fname);
}
//--------------------------------------------------------------------------
// renames an element
bool jfFilterPointer::RenameElem(const wxString& newname) {
  wxString oldname;
  bool xtemp;
  oldname = filter->GetName();
  // first, we run the 'do nothing' check
  if (newname == oldname) return false;
  // next, we run the invalud checks
  if (newname.IsEmpty()) return false;
  if (CheckForName(newname)) return false;
  // we go ahead with the renameing
  filtermap->RenameElem(xtemp,oldname,newname);
  assert(xtemp);
  return true;
}
//============================================================================
bool jfMapOfFiltermaps::NameExists(const wxString& mapname) const {
  // local variables
  jfMapOfFiltermaps::const_iterator xindex;
  // starting
  if (!empty()) {
    xindex = find(mapname);
    if (xindex != end()) return true;
  }
  return false;
}
//----------------------------------------------------------------------------
// produces a pointer to a specific filter
jfFilterPointer* jfMapOfFiltermaps::GetPointer(const wxString& mapname, const wxString& filtername) {
  // local variables
  jfFilterPointer* result;
  jfMapOfFiltermaps::const_iterator xindex;
  jfFilterMap* map_ptr;
  // testing for not found conditions, and doing intermediate steps
  if (empty()) return NULL;
  xindex = find(mapname);
  if (xindex == end()) return NULL;
  map_ptr = ((*xindex).second);
  if (!(map_ptr->NameExists(filtername))) return NULL;
  // building the result
  result = new jfFilterPointer();
  result->filter = map_ptr->GetItem(filtername);
  result->filtermap = map_ptr;
  result->mapomaps = this;
  // done
  return result;
}
//-----------------------------------------------------------------------------
wxArrayString* jfMapOfFiltermaps::GetMapNames() const {
  // local varibales
  wxArrayString* res;
  jfMapOfFiltermaps::const_iterator xindex;
  // processing
  res = NULL;
  if (!empty()) {
    res = new wxArrayString();
    xindex = begin();
    // the name extraction loop
    while (xindex!=end()) {
      res->Add((*xindex).first);
      xindex++;
    }
  }
  return res;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfMapOfFiltermaps::FindMapIndex(const wxString& mapname, size_t& newindex) const {
  jfMapOfFiltermaps::const_iterator positem;
  // we start...
  positem = find(mapname);
  if (positem==end()) return false;
  // looping to find the index
  positem = begin();
  newindex = 0;
  while((positem->first)!=mapname) {
    positem++;
    newindex++;
  }
  return true;
}
//----------------------------------------------------------------------------
// some other special methods
bool jfMapOfFiltermaps::RenameMap(const wxString& oldname, const wxString& newname, size_t& newindex) {
  jfMapOfFiltermaps::iterator positem;
  jfFilterMap* themap;
  // a check of the newname
  if (newname.IsEmpty()) return false;
  // first, we have to find the map with oldname
  positem = find(oldname);
  if (positem==end()) return false;
  assert((positem->second)!=NULL);
  // if found, we save it
  themap = positem->second;
  // we only need to do the change if the names are unequal
  if (oldname!=newname) {
    // we check to see if newname already exists
    if (NameExists(newname)) return false;
    // otherwise, we go ahead...
    erase(oldname);           // erasing the old
    themap->SetName(newname); // changing the name
    (*this)[newname] = themap;       // inserting the new
  }
  // done
  return FindMapIndex(newname,newindex);
}
//=============================================================================

void jfMapOfFiltermaps::WriteToFile(const wxString& filename) const {
  // local variables
  wxTextFile* outfile;
  jfMapOfFiltermaps::const_iterator xindex;
  jfFilterMap *cmap;
  bool oores;
  // we start
  outfile = new wxTextFile(filename);
  // if the file is not there, we make it
  if (!outfile->Exists()) outfile->Create();
  // we open the file and empty any pre-existing contents
  oores = outfile->Open();
  assert(oores);
  outfile->Clear();
  /* we now iterate over all of the maps, calling the file i/o function
  for each map */
  xindex = begin();
  while (xindex!=end()) {
    cmap = (*xindex).second;
    cmap->AddToFile(outfile);
    xindex++;
  }
  // now that we are done, we finish writing
  outfile->Write();
  outfile->Close();
  // clearing up
  delete outfile;
}
//-------------------------------------------------------------------------
bool jfMapOfFiltermaps::ReadFromFile(const wxString& filename) {
  // constants
  const wxString fname = wxT("jfMapOfFiltermaps::ReadFromFile");
  // local variables
  jfFileReader* infile;
  jfFilterMap *cmap;
  bool iores;
  size_t mcount;
  // we prepare for loading...
  /**/jfXLogMessage(fname,1,filename);
  infile = new jfFileReader(filename);
  // if the file does not exist, we do nothing
  if (!infile->OpenFile()) {
    delete infile;
    /**/jfXLogMessage(fname,2);
    return false;
  }
  // the infile does exist
  else {
    if (!empty()) EmptyMaps();
    mcount = 0;
    /**/jfXLogMessage(fname,3);
    while (infile->BeforeEnd(1)) {
      // trying the read the map
      cmap = new jfFilterMap();
      iores = cmap->GetFromFile(infile);
      mcount++;
      /**/jfXLogMessageS(fname,4,mcount);
      if (!iores) {
        /**/jfXLogMessage(fname,5,infile->err);
        wxMessageBox(infile->err);
        delete infile;
        delete cmap;
        return false;
      }
      // the read worked!
      /**/jfXLogMessageS(fname,6,mcount);
      (*this)[cmap->GetName()] = cmap;
    }
    delete infile;
    /**/jfXLogMessage(fname,7);
    return true;
  }
}

//==========================================================================
int jfMapOfFiltermaps::EmptyMaps() {
  // local variables
  int count = 0;
  jfMapOfFiltermaps::iterator xindex;
  // starting to check
  if (!empty()) {
    // we prepare for iteration
    xindex = begin();
    // we delete each map
    while (xindex!=end()) {
      delete (*xindex).second;
      (*xindex).second = NULL;
      count++;
      xindex++;
    }
    clear();
  }
  return count;
}
/////////////////////////////////////////////////////////////////////////////
