//***************************************************************************
// Name:        sexp_filters.cpp
// Author :     John Q Metro
// Purpose :    Simple expression filters that work on specific string data
// Created:     November 30, 2010
// Updated:     September 3, 2012
//***************************************************************************
#ifndef SEXP_FILTERS_H_INCLUDED
  #include "sexp_filters.h"
#endif // SEXP_FILTERS_H_INCLUDED
//-------------------------------------------------------------------------
#ifndef JFEXPRPARSE
  #include "../../express/expr_parse.h"
#endif
#ifndef JFILTERGLOBALS
  #include "filter_globals.h"
#endif
#ifndef JFEXPRMATCH
	#include "../../express/expr_match.h"
#endif
#ifndef LOGGING_H_INCLUDED
  #include "../../utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef JFFNFICOBJ
  #include "../../../fanficnet/data/ffn_ficobj.h"
#endif // BASEOBJ_H_INCLUDED

#ifndef AO3_FICOBJ_H_INCLUDED
  #include "../../../fanfic/data/ao3/ao3_ficobj.h"
#endif // AO3_FICOBJ_H_INCLUDED
//***************************************************************************
jfSimpleExpr::jfSimpleExpr() {
  parsedinfo = NULL;
  valid = false;
  isempty = false;
}
//-----------------------------------------------------
jfSimpleExpr::jfSimpleExpr(const wxString& sourcedata) {
  parsedinfo = NULL;
  FromString(sourcedata);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// data setting methods
bool jfSimpleExpr::FromString(const wxString& sourcedata) {
  // constsants
  const wxString fname = wxT("jfSimpleExpr::FromString");
  // local data
  jfExpParserClass* theparser;
  jfElemArray* pfix;
  size_t osize;
  wxString oerr;
  // setting the source
  srcstring = sourcedata;
  Dtrim(srcstring);
  // clearing out the old
  if (parsedinfo!=NULL) delete parsedinfo;
  parsedinfo = NULL;
  parse_error.Clear();
  // producing the new
  /**/jfXLogMessage(fname,1,srcstring);
  theparser = new jfExpParserClass(srcstring,true,NULL);
  if (!theparser->ParseExpression(osize)) {
    /**/jfXLogMessage(fname,2,theparser->parse_error);
    parse_error = theparser->parse_error;
    delete theparser;
    return valid = false;
  }
  // parsed okay
  parsedinfo = theparser->GetResult();
  assert(parsedinfo!=NULL);
  delete theparser;
  // next, we postfixize it
  pfix = MakeExprPostfix(parsedinfo,oerr);
  delete parsedinfo;
  if (pfix==NULL) {
    /**/jfXLogMessage(fname,3,oerr);
    parse_error = oerr;
    return valid = false;
  }
  // okay
  parsedinfo = pfix;
  isempty = srcstring.IsEmpty();
  // done
  valid = true;
  return true;
}
//--------------------------------------------------------------------------
bool jfSimpleExpr::InternalMatch(const wxString& incheck) const {
  return StringExprMatch(incheck,parsedinfo);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// info
//-----------------------------------------------------
bool jfSimpleExpr::IsValid() const { return valid; }
//-----------------------------------------------------
bool jfSimpleExpr::IsEmpty() const { return isempty; }
//-----------------------------------------------------
wxString jfSimpleExpr::GetSrc() const {
  return srcstring;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructors
jfSimpleExpr::~jfSimpleExpr() {
  if (parsedinfo!=NULL) delete parsedinfo;
}
//==========================================================================
// the constructors
//----------------------------------------------------------------------
// the basic constructor
jfSimpleExpFilterCore::jfSimpleExpFilterCore():jfBaseFilter() {
  loaded = false;
  parsedinfo = NULL;
  validdata = false;
}
//----------------------------------------------------------------------
jfSimpleExpFilterCore::jfSimpleExpFilterCore(const wxString& sourcedata) {
  parsedinfo = NULL;
  FromString(sourcedata);
}
//----------------------------------------------------------------------
jfSimpleExpFilterCore::jfSimpleExpFilterCore(jfSimpleExpr* in_source) {
  parsedinfo = NULL;
  FromExpr(in_source);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfSimpleExpFilterCore::IsEmpty() const {
	return (parsedinfo==NULL)||(parsedinfo->size()==0);
}
//-----------------------------------------------------------------------------
// returns a general filter type. The default is 0, unless overriden
size_t jfSimpleExpFilterCore::GetFilType() const { return 1;}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// redefined virtual functions
// loading stuff
bool jfSimpleExpFilterCore::FromString(const wxString& sourcedata) {
  // constants
  const wxString fname = wxT("jfSimpleExpFilterCore::FromString");
  // local data
  jfExpParserClass* theparser;
  jfElemArray* pfix;
  wxString oerr;
  size_t osize;
  // setting the source
  /**/jfXLogMessage(fname,1);
  srcstring = sourcedata;
  Dtrim(srcstring);
  // clearing out the old
  if (parsedinfo!=NULL) delete parsedinfo;
  parsedinfo = NULL;
  loaded = false;
  parse_error.Clear();
  // producing the new
  theparser = new jfExpParserClass(srcstring,true,NULL);
  if (!theparser->ParseExpression(osize)) {
    /**/jfXLogMessage(fname,2,theparser->parse_error);
    parse_error = theparser->parse_error;
    delete theparser;
    return validdata = false;
  }
  // parsed okay
  parsedinfo = theparser->GetResult();
  assert(parsedinfo!=NULL);
  delete theparser;
  // next, we postfixize it
  pfix = MakeExprPostfix(parsedinfo,oerr);
  delete parsedinfo;
  if (pfix==NULL) {
    /**/jfXLogMessage(fname,3,oerr);
    parse_error = oerr;
    return validdata = false;
  }
  // postfixed okay
  parsedinfo = pfix;
  // done
  validdata = true;
  loaded = true;
  return true;
}
//----------------------------------------------------------------------
wxString jfSimpleExpFilterCore::ToString() const {
  return srcstring;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// modifying and checking the contents
//----------------------------------------------------------------------
bool jfSimpleExpFilterCore::FromExpr(jfSimpleExpr* in_source) {
  if (in_source==NULL) return false;
  validdata = in_source->valid;
  if (validdata) {
    parsedinfo = in_source->parsedinfo;
    in_source->parsedinfo = NULL;
    in_source->valid = false;
  }
  srcstring = in_source->srcstring;
  parse_error = in_source->parse_error;
  if (validdata) loaded = true;
  return true;
}
//----------------------------------------------------------------------
// returning a string representation of the parsed contents
wxString jfSimpleExpFilterCore::MakePList() const {
  // local variables
  size_t index,maxc;
  wxString result;
  // starting
  maxc = (parsedinfo->size());
  // the loop
  for (index = 0;index<maxc;index++) {
    result+= ((*parsedinfo)[index])->String();
  }
  // done
  return result;
}
//-----------------------------------------------------------------------
// sort of like load string, except the string is empty
void jfSimpleExpFilterCore::EmptyFilter() {
  // setting the source
  srcstring = wxEmptyString;
  // clearing out the old
  if (parsedinfo!=NULL) delete parsedinfo;
  // creating the empty data
  parsedinfo = new jfElemArray();
  // done
  validdata = true;
  parse_error.Clear();
  loaded = true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the destructor
jfSimpleExpFilterCore::~jfSimpleExpFilterCore() {
  if (parsedinfo!=NULL) delete parsedinfo;
  parsedinfo = NULL;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfSimpleExpFilterCore::InternalMatch(const wxString& incheck) const {
  return StringExprMatch(incheck,parsedinfo);
}
//-----------------------------------------------------------------------
// check for validity
void jfSimpleExpFilterCore::SetValid() {
  validdata = loaded;
}
//-----------------------------------------------------------------------
size_t jfSimpleExpFilterCore::ExtraLines() const {
  return 1;
}
//-----------------------------------------------------------------------
bool jfSimpleExpFilterCore::AddRestToFile(wxTextFile* outfile) const {
  wxString buffer;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // composing line 4
  buffer = EscapeNL(srcstring,IO_DELIMS);
  outfile->AddLine(buffer);
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfSimpleExpFilterCore::ReadRestFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfSimpleExpFilterCore::ReadRestFromFile");
  // input data
  wxString cline;
  bool resx;
  // starting checks (and reading the line)
  assert(infile!=NULL);
  if (!infile->ReadUnEsc(cline,funcname)) return false;
  // there is only one line, and one filed, so this is pretty simple
  resx = FromString(cline);
  if (!resx) return infile->BuildError(wxT("The expression string is invalid!"));
  else return true;
}
//-----------------------------------------------------------------------
void jfSimpleExpFilterCore::CoreCopy(const jfSimpleExpFilterCore& source) {
  // copying the most basic items
  name = source.name;
  validdata = source.validdata;
  parse_error = source.parse_error;
  // copying expression filter specific stuff
  srcstring = source.srcstring;
  loaded = source.loaded;
  if (source.parsedinfo!=NULL) {
    parsedinfo = (source.parsedinfo)->Copy();
  }
  else parsedinfo = NULL;

}
//===========================================================================================
// constructors
//----------------------------------------------------------
jfAuthExprFilter::jfAuthExprFilter():jfSimpleExpFilterCore() {}
//----------------------------------------------------------
jfAuthExprFilter::jfAuthExprFilter(const jfAuthExprFilter& source) {
  CoreCopy(source);
}
//----------------------------------------------------------
jfAuthExprFilter::jfAuthExprFilter(jfSimpleExpr* in_source):jfSimpleExpFilterCore(in_source) {}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfAuthExprFilter::GetTypeDescription() const {
  return wxT("The Author Expression Filter  matches elements against a boolean \
expression, the elements of which are themselves to be matched. These elements \
are either strings or substrings.");
}
//------------------------------------------------------------
jfAuthExprFilter* jfAuthExprFilter::Copy() const {
  return new jfAuthExprFilter(*this);
}
//------------------------------------------------------------
jfBaseFilter* jfAuthExprFilter::GenCopy() const {
  return Copy();
}
//------------------------------------------------------------
wxString jfAuthExprFilter::GetTypeID() const {
  return wxT("AuthorFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfAuthExprFilter::CoreMatch(const jfBasePD* testelem) const {
  wxString datainfo;
  const jfGenericFanfic* typed_item;
  // checks and starts
  assert(testelem!=NULL);
  typed_item = dynamic_cast<const jfGenericFanfic*>(testelem);
  datainfo = typed_item->GetAuthor();
  return InternalMatch(datainfo);
}
//===========================================================================================
// constructors
//----------------------------------------------------------
jfCharExprFilter::jfCharExprFilter():jfSimpleExpFilterCore() {}
//----------------------------------------------------------
jfCharExprFilter::jfCharExprFilter(const jfCharExprFilter& source) {
  CoreCopy(source);
}
//----------------------------------------------------------
jfCharExprFilter::jfCharExprFilter(jfSimpleExpr* in_source):jfSimpleExpFilterCore(in_source) {}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfCharExprFilter::GetTypeDescription() const {
  return wxT("The Character Expression Filter matches the character list against a boolean \
expression, the elements of which are themselves to be matched. These elements \
are either strings or substrings.");
}
//------------------------------------------------------------
jfCharExprFilter* jfCharExprFilter::Copy() const {
  return new jfCharExprFilter(*this);
}
//------------------------------------------------------------
jfBaseFilter* jfCharExprFilter::GenCopy() const {
  return Copy();
}
//------------------------------------------------------------
wxString jfCharExprFilter::GetTypeID() const {
  return wxT("CharacterFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfCharExprFilter::CoreMatch(const jfBasePD* testelem) const {
  wxString datatype,datainfo;
  const jfFFNItem* typed_item1;
  const jfAO3Fanfic* typed_item2;
  // checks and starts
  assert(testelem!=NULL);
  // two types
  datatype = testelem->GetTypeID();
  if (datatype==wxT("FFNItem")) {
    typed_item1 = dynamic_cast<const jfFFNItem*>(testelem);
    datainfo = typed_item1->GetCharacter();
  }
  else if (datatype==wxT("AO3Fanfic")) {
    typed_item2 = dynamic_cast<const jfAO3Fanfic*>(testelem);
    datainfo = typed_item2->GetCharacters();
  }
  else assert(false);
  return InternalMatch(datainfo);
}
//===========================================================================================
// constructors
//----------------------------------------------------------
jfCategoryExprFilter::jfCategoryExprFilter():jfSimpleExpFilterCore() {}
//----------------------------------------------------------
jfCategoryExprFilter::jfCategoryExprFilter(const jfCategoryExprFilter& source) {
  CoreCopy(source);
}
//----------------------------------------------------------
jfCategoryExprFilter::jfCategoryExprFilter(jfSimpleExpr* in_source):jfSimpleExpFilterCore(in_source) {}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfCategoryExprFilter::GetTypeDescription() const {
  return wxT("The Category Expression Filter matches the category name against a boolean \
expression, the elements of which are themselves to be matched. These elements \
are either strings or substrings.");
}
//------------------------------------------------------------
jfCategoryExprFilter* jfCategoryExprFilter::Copy() const {
  return new jfCategoryExprFilter(*this);
}
//------------------------------------------------------------
jfBaseFilter* jfCategoryExprFilter::GenCopy() const {
  return Copy();
}
//------------------------------------------------------------
wxString jfCategoryExprFilter::GetTypeID() const {
  return wxT("CategoryFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfCategoryExprFilter::CoreMatch(const jfBasePD* testelem) const {
  wxString datainfo;
  const jfFFNItem* typed_item;
  // checks and starts
  assert(testelem!=NULL);
  typed_item = dynamic_cast<const jfFFNItem*>(testelem);
  datainfo = typed_item->GetCatString();
  return InternalMatch(datainfo);
}
//===========================================================================================
// constructors
//----------------------------------------------------------
jfLanguageExprFilter::jfLanguageExprFilter():jfSimpleExpFilterCore() {}
//----------------------------------------------------------
jfLanguageExprFilter::jfLanguageExprFilter(const jfLanguageExprFilter& source) {
  CoreCopy(source);
}
//----------------------------------------------------------
jfLanguageExprFilter::jfLanguageExprFilter(jfSimpleExpr* in_source):jfSimpleExpFilterCore(in_source) {}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfLanguageExprFilter::GetTypeDescription() const {
  return wxT("The Language Expression Filter matches the language name against a boolean \
expression, the elements of which are themselves to be matched. These elements \
are either strings or substrings.");
}
//------------------------------------------------------------
jfLanguageExprFilter* jfLanguageExprFilter::Copy() const {
  return new jfLanguageExprFilter(*this);
}
//------------------------------------------------------------
jfBaseFilter* jfLanguageExprFilter::GenCopy() const {
  return Copy();
}
//------------------------------------------------------------
wxString jfLanguageExprFilter::GetTypeID() const {
  return wxT("LanguageFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfLanguageExprFilter::CoreMatch(const jfBasePD* testelem) const {
  wxString datainfo;
  const jfFFNItem* typed_item;
  // checks and starts
  assert(testelem!=NULL);
  typed_item = dynamic_cast<const jfFFNItem*>(testelem);
  datainfo = typed_item->GetLanguage();
  return InternalMatch(datainfo);
}
//*******************************************************************************************
