//***************************************************************************
// Name:        sexp_filters.h
// Author :     John Q Metro
// Purpose :    Simple expression filters that work on specific string data
// Created:     November 30, 2010
// Updated:     August 22, 2012
//***************************************************************************
#ifndef SEXP_FILTERS_H_INCLUDED
#define SEXP_FILTERS_H_INCLUDED
#endif // SEXP_FILTERS_H_INCLUDED
//-------------------------------------------------------------------------
#ifndef JFBASEFILTER
  #include "filterbase.h"
#endif
#ifndef JFEXPRSTRUCTS
  #include "../../express/expr_structs.h"
#endif
//***************************************************************************
/* the result of a simple expression editor, the filter class base is abstract,
this is to avoid having a separate editor for each simple expression filter subtype,
since the only real difference is what they match. */
class jfSimpleExpr : public wxObject {
  friend class jfSimpleExpFilterCore;
  friend class jfAO3FandomFilter;
  public:
    wxString parse_error;
    // constructors
    jfSimpleExpr();
    jfSimpleExpr(const wxString& sourcedata);
    // data setting methods
    bool FromString(const wxString& sourcedata);
    // other methods
    bool InternalMatch(const wxString& incheck) const;
    // info
    bool IsValid() const;
    bool IsEmpty() const;
    wxString GetSrc() const;
    // destructors
    ~jfSimpleExpr();
  protected:
    wxString srcstring;
    jfElemArray* parsedinfo;
    bool valid;
    bool isempty;
};
//==========================================================================
/* The simple expression filter: Holds a boolean expression.
 the operands of the expression, which can evaluate to true or false, are
 for either string matching/searching. */
class jfSimpleExpFilterCore : public jfBaseFilter {
  public:
    wxString parse_error;
    // the constructors
    jfSimpleExpFilterCore();
    jfSimpleExpFilterCore(const wxString& sourcedata);
    jfSimpleExpFilterCore(jfSimpleExpr* in_source);
    // redefined virtual functions
    virtual bool IsEmpty() const;
    virtual bool FromString(const wxString& sourcedata);
    virtual wxString ToString() const;
    // extra methods
    bool FromExpr(jfSimpleExpr* in_source);
    wxString MakePList() const;
    void EmptyFilter();
    // returns a general filter type
    virtual size_t GetFilType() const;
    // the destructor
    ~jfSimpleExpFilterCore();
  protected:
    // private methods
    bool InternalMatch(const wxString& incheck) const;
    virtual void SetValid();
    virtual size_t ExtraLines() const;
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    void CoreCopy(const jfSimpleExpFilterCore& source);
    // private data
    wxString srcstring;
    jfElemArray* parsedinfo;
    bool loaded;
};
//=====================================================================================
// a simple expression filter that matches against author data
class jfAuthExprFilter : public jfSimpleExpFilterCore {
  public:
    jfAuthExprFilter();
    jfAuthExprFilter(const jfAuthExprFilter& source);
    jfAuthExprFilter(jfSimpleExpr* in_source);
    virtual wxString GetTypeDescription() const;
    virtual jfAuthExprFilter* Copy() const;
    virtual jfBaseFilter* GenCopy() const;
    virtual wxString GetTypeID() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
};
//===========================================================================
// a simple expression filter that matches against author data
class jfCharExprFilter : public jfSimpleExpFilterCore {
  public:
    jfCharExprFilter();
    jfCharExprFilter(const jfCharExprFilter& source);
    jfCharExprFilter(jfSimpleExpr* in_source);
    virtual wxString GetTypeDescription() const;
    virtual jfCharExprFilter* Copy() const;
    virtual jfBaseFilter* GenCopy() const;
    virtual wxString GetTypeID() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
};
//===========================================================================
// a simple expression filter that matches against the category name
class jfCategoryExprFilter : public jfSimpleExpFilterCore {
  public:
    jfCategoryExprFilter();
    jfCategoryExprFilter(const jfCategoryExprFilter& source);
    jfCategoryExprFilter(jfSimpleExpr* in_source);
    virtual wxString GetTypeDescription() const;
    virtual jfCategoryExprFilter* Copy() const;
    virtual jfBaseFilter* GenCopy() const;
    virtual wxString GetTypeID() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
};
//===========================================================================
// a simple expression filter that matches against the language name
class jfLanguageExprFilter : public jfSimpleExpFilterCore {
  public:
    jfLanguageExprFilter();
    jfLanguageExprFilter(const jfLanguageExprFilter& source);
    jfLanguageExprFilter(jfSimpleExpr* in_source);
    virtual wxString GetTypeDescription() const;
    virtual jfLanguageExprFilter* Copy() const;
    virtual jfBaseFilter* GenCopy() const;
    virtual wxString GetTypeID() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
};
//***************************************************************************
