/******************************************************************************
Name    :   morefilters1.cpp
Author  :   John Q Metro
Purpose :   More Filters
Created :   December 6, 2010
Updated :   August 10, 2013
******************************************************************************/
#ifndef MOREFILTERS1_H_INCLUDED
  #include "morefilters1.h"
#endif
//------------------------------------------------------------------------
#ifndef UTILS1_H_INCLUDED
  #include "../../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef UTILS2_H_INCLUDED
  #include "../../utils/utils2.h"
#endif // UTILS1_H_INCLUDED
#ifndef JFILTERGLOBALS
  #include "../base/filter_globals.h"
#endif
#ifndef LOGGING_H_INCLUDED
  #include "../../utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef JFFNFICOBJ
  #include "../../../fanficnet/data/ffn_ficobj.h"
#endif // BASEOBJ_H_INCLUDED
/******************************************************************************/
// constructors
//---------------------------------------------------
jfMinMaxUFilter::jfMinMaxUFilter():jfBaseFilter() {
  validdata = false;
  min = 0;
  max = 0;
}
//---------------------------------------------------
jfMinMaxUFilter::jfMinMaxUFilter(size_t inmin, size_t inmax):jfBaseFilter() {
  validdata = SetValues(inmin,inmax);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting values
bool jfMinMaxUFilter::SetValues(size_t inmin, size_t inmax) {
  const wxString fname = wxT("jfMinMaxUFilter::SetValues");
  if ((inmax>=inmin) || (inmax==0)) {
    min = inmin;
    max = inmax;
    validdata = true;
  }
  else validdata = false;
  return validdata;
}
//-----------------------------------------------------------------------
// getting values
//-------------------------------------------
size_t jfMinMaxUFilter::GetMin() const { return min; }
//-------------------------------------------
size_t jfMinMaxUFilter::GetMax() const { return max; }
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// redefined virtual methods
bool jfMinMaxUFilter::IsEmpty() const { return false; }
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading from a string representation
//---------------------------------------------------
bool jfMinMaxUFilter::FromString(const wxString& sourcedata) {
  jfLineParse* lparser;
  lparser = new jfLineParse(sourcedata);
  validdata = false;
  if (lparser->NNotX(2)) delete lparser;
  else {
    if ((lparser->SIntVal(0,min)) && (lparser->SIntVal(1,max))) validdata = true;
    delete lparser;
  }
  return validdata;
}
//---------------------------------------------------
wxString jfMinMaxUFilter::ToString() const {
  wxString result;
  result = UInt2Str(min) + wxT(";");
  result += UInt2Str(max);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfMinMaxUFilter::TestMatch(size_t invalue) const {
  assert(invalue<=999999999);
  if (invalue<min) return false;
  else if (max==0) return true;
  else return (invalue<=max);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o
//-----------------------------------
bool jfMinMaxUFilter::AddRestToFile(wxTextFile* outfile) const {
  wxString buffer;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // composing line 4
  buffer = ToString();
  outfile->AddLine(buffer);
  return true;
}
//-----------------------------------
bool jfMinMaxUFilter::ReadRestFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfMinMaxUFilter::ReadRestFromFile");
  // input data
  wxString cline;
  bool resx;
  // starting checks (and reading the line)
  assert(infile!=NULL);
  if(!infile->ReadLine(cline,funcname)) return false;
  // there is only one line, and one filed, so this is pretty simple
  resx = FromString(cline);
  if (!resx) return infile->BuildError(wxT("The min-max string is invalid!"));
  else return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* since different types are stored together, the text file reprentation
may have objects of varying length */
size_t jfMinMaxUFilter::ExtraLines() const { return 1;}
//***************************************************************************
// constructors
//---------------------------------------------------
jfWordCountFilter::jfWordCountFilter():jfMinMaxUFilter() {}
//---------------------------------------------------
jfWordCountFilter::jfWordCountFilter(size_t inmin, size_t inmax):jfMinMaxUFilter(inmin,inmax) {}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfWordCountFilter::GetTypeID() const {
  return wxT("WordCountFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// returns the list of element names campatible with this filter
//---------------------------------------------------
wxString jfWordCountFilter::GetTypeDescription() const {
    return wxT("For items that have word counts, this filter passes if \
the word count is between min and max, inclusive.");
}
//---------------------------------------------------
jfBaseFilter* jfWordCountFilter::GenCopy() const {
  jfWordCountFilter* result;
  result = new jfWordCountFilter(min,max);
  CopyOver(result);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfWordCountFilter::CoreMatch(const jfBasePD* testelem) const {
  // variables
  const jfGenericFanfic2* rvalue;
  size_t cvalue;
  // checks
  assert(testelem!=NULL);
  // determining the type
  rvalue = dynamic_cast<const jfGenericFanfic2*>(testelem);
  cvalue = rvalue->GetWordcount();
  // checking the wordcount
  return TestMatch(cvalue);
}
//==============================================================================
// constructors
jfTagFilterCore::jfTagFilterCore():jfBaseFilter() {
  thedata = NULL;
  validdata = false;
  nullresult = true;
  emptyc = 0;
}
//-----------------------------------------------------------------------
jfTagFilterCore::jfTagFilterCore(const jfTagFilterCore& insrc) {
  insrc.CopyOver(this);
  if (validdata==false) thedata = NULL;
  else {
    thedata = new jfTagListing(*insrc.thedata);
  }
  nullresult = insrc.nullresult;
  emptyc = insrc.emptyc;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting and setting values
//-----------------------------------
bool jfTagFilterCore::SetTags(const jfTagListing* insource) {
  // constants
  const wxString fname = wxT("jfTagFilterCore::SetTags");
  // special cases
  /**/jfXLogMessage(fname,1);
  if (insource==NULL) return false;
  // clearing the old one
  if (thedata != NULL) delete thedata;
  // getting the new one
  thedata = new jfTagListing(*insource);
  /**/jfXLogMessage(fname,2);
  validdata = DoVerify();
  /**/jfXLogMessageB(fname,3,validdata);
  // we set some flags
  IsEmpty();
  MakeNullResult();
  // done
  return validdata;

}
//-----------------------------------
jfTagListing* jfTagFilterCore::GetTagsCopy() const {
  if (thedata == NULL) return NULL;
  return new jfTagListing(*thedata);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// redefined virtual methods
bool jfTagFilterCore::IsEmpty() const {
  // variables
  bool result = true;
  jfTagListing::const_iterator iter;
  // special case
  if (thedata == NULL ) return false;
  // main checking
  iter = thedata->begin();
  while (result) {
    result = ((iter->second)==jfts_NONE);
    iter++;
    if (iter==(thedata->end())) break;
  }
  emptyc = (result)?(1):(2);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading from a string representation
//-----------------------------------
bool jfTagFilterCore::FromString(const wxString& sourcedata) {
  // local variables
  jfTagListing* newvalue;
  bool conversult;
  // we create and convert
  newvalue = new jfTagListing();
  conversult = newvalue->FromString(sourcedata);
  // the results
  if (conversult) {
    // success!
    if (thedata != NULL) delete thedata;
    thedata = newvalue;
    validdata = DoVerify();
    // we set some flags
    IsEmpty();
    MakeNullResult();
    // done
    return validdata;
  }
  else {
    delete newvalue;
    return false;
  }
}
//-----------------------------------
wxString jfTagFilterCore::ToString() const {
  if (thedata==NULL) return wxEmptyString;
  else return thedata->ToString();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfTagFilterCore::MatchVsData(const wxString& incheck) const {
  // constants
  const wxString fname = wxT("jfTagFilterCore::MatchVsData");
  // local variables
  wxString     tag;
  jfTAG_STATUS todo;
  jfTagListing::const_iterator iter,enditer;
  wxArrayString* delimlist;
  size_t altercount,altermatch;
  bool isfound;
  // special case 1: the filter is actually empty
  assert(emptyc!=0);
  if (emptyc==1) return true;
  // parsing the inout to check against
  delimlist = ListToStrArr(incheck,cm);
  if ((delimlist==NULL)) return nullresult;
  else if (delimlist->IsEmpty()) {
    delete delimlist;
    return nullresult;
  }
  // initializing some values
  iter = thedata->begin();
  enditer = thedata->end();
  altercount = 0;
  altermatch = 0;
  // the main loop
  while (iter!=enditer) {
    todo = iter->second;
    if (todo!=jfts_NONE) {
      // we get some info
      tag = iter->first;
      // search
      isfound = (wxNOT_FOUND!=delimlist->Index(tag));
      // the results
      if (todo==jfts_INCLUDE) {
        if (!isfound) {
          delete delimlist;
          return false;
        }
      }
      else if (todo==jfts_EXCLUDE) {
        if (isfound) {
          delete delimlist;
          return false;
        }
      }
      else {
        altercount++;
        if (isfound) altermatch++;
      }
    }
    iter++;
  }
  // if we get past the loop, things are correct unless there is
  // no match for the alternates
  delete delimlist;
  if (altercount>0) return (altermatch>0);
  else return true;
}
//---------------------------------------------------------------------
// calculates the result if what we check against is empty
void jfTagFilterCore::MakeNullResult() {
  // variables
  jfTAG_STATUS todo;
  jfTagListing::const_iterator iter,enditer;
  // special case
  if (emptyc==1) nullresult = true;
  // more general case
  else {
    nullresult = true;
    iter = thedata->begin();
    enditer = thedata->end();
    while (iter!=enditer) {
      todo = iter->second;
      if ((todo==jfts_INCLUDE) || (todo==jfts_ALTERNATE)) {
        nullresult = false;
        break;
      }
      iter++;
    }
  }
  // done
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// check the tags against a list of pre-approved tags
/* all tags in the filter must be in intags, but tags in intags not
in the list will be added, no error */
bool jfTagFilterCore::VerifyTags(size_t NUMTAGIN, const wxChar* intags[], wxArrayString* replist) {
  // constants
  const wxString fname = wxT("jfTagFilterCore::VerifyTags");
  // variables
  size_t tcount, tfoundc, tnfound, tloop;
  wxString ctag;
  bool tfound;
  // checks
  /**/jfXLogMessage(fname,1);
  assert(intags!=NULL);
  if (thedata==NULL) return false;
  /**/jfXLogMessage(fname,2);
  // we start
  tcount = thedata->size();
  tfoundc = 0;
  tnfound = 0;
  /**/jfXLogMessageS(fname,3,NUMTAGIN);
  // the replacement list
  if (replist!=NULL) {
      thedata->ReplaceTags(replist);
  }
  // the loop
  for (tloop=0; tloop<NUMTAGIN; tloop++) {
    ctag = wxString(intags[tloop]);
    tfound = ((thedata->end())!=(thedata->find(ctag)));
    if (tfound) tfoundc++;
    else {
      tnfound++;
      (*thedata)[ctag] = jfts_NONE;
    }
  }
  // the end of the list, we check now whether things worked out okay
  /**/jfXLogMessageS(fname,4,tcount);
  /**/jfXLogMessageS(fname,5,tfoundc);
  return (tcount == tfoundc);
}
//-----------------------------------
void jfTagFilterCore::SetEmpty(size_t NUMTAGIN, const wxChar* intags[]) {
  // variables
  size_t loopc;
  wxString tag;
  // starting checks
  assert(intags!=NULL);
  if (thedata == NULL) thedata = new jfTagListing();
  else thedata->clear();
  // loop to add items
  for (loopc=0;loopc<NUMTAGIN;loopc++) {
    tag = wxString(intags[loopc]);
    (*thedata)[tag] = jfts_NONE;
  }
  validdata = true;
  emptyc = 1;
  nullresult = true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o
//-----------------------------------
bool jfTagFilterCore::AddRestToFile(wxTextFile* outfile) const {
  wxString buffer;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // composing line 4
  buffer = ToString();
  outfile->AddLine(buffer);
  return true;
}
//-----------------------------------
bool jfTagFilterCore::ReadRestFromFile(jfFileReader* infile) {
  const wxString fname = wxT("jfExpressionFilter::ReadRestFromFile");
  // input data
  wxString cline;
  bool resx;
  // starting checks (and reading the line)
  assert(infile!=NULL);
  if (!infile->ReadLine(cline,fname)) return false;
  // there is only one line, and one filed, so this is pretty simple
  resx = FromString(cline);
  /**/jfXLogMessageB(fname,3,resx);
  if (!resx) return infile->BuildError(wxT("The tag string is invalid!"));
  resx = DoVerify();
  /**/jfXLogMessageB(fname,4,resx);
  if (!resx) return infile->BuildError(wxT("The tags do not match what is defined for this type!"));
  IsEmpty();
  MakeNullResult();
  /**/jfXLogMessage(fname,4);
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* since different types are stored together, the text file reprentation
may have objects of varying length */
size_t jfTagFilterCore::ExtraLines() const { return 1;}
//==========================================================================
const size_t ff_tagfilt_count = 22;
const wxChar* ff_tagfilt_items[ff_tagfilt_count] = {wxT("Adventure"),wxT("Angst"),
wxT("Comfort"),wxT("Crime"),wxT("Drama"),wxT("Family"),wxT("Fantasy"),
wxT("Friendship"),wxT("Horror"),wxT("Humour"),wxT("Hurt"),wxT("General"),
wxT("Romance"),wxT("Mystery"),wxT("Parody"),wxT("Poetry"),wxT("Sci-Fi"),
wxT("Spiritual"),wxT("Supernatural"),wxT("Suspense"),wxT("Tragedy"),wxT("Western")};
//========================================================================================
jfFFNTagFilter::jfFFNTagFilter():jfTagFilterCore() {
  cm = wxT("/");
}
//---------------------------------------------
jfFFNTagFilter::jfFFNTagFilter(const jfFFNTagFilter& insrc):jfTagFilterCore(insrc) {
  cm = wxT("/");
  validdata = DoVerify();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting and setting values
bool jfFFNTagFilter::SetToEmpty() {
  SetEmpty(ff_tagfilt_count,ff_tagfilt_items);
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// redefined virtual methods
wxString jfFFNTagFilter::GetTypeID() const {
  return wxT("FFNTagFilter");
}
//-----------------------------------------------------------------------------
wxString jfFFNTagFilter::GetTypeDescription() const {
    return wxT("Compares the Fanfiction.Net genres of the fic against the \
include/exclude/alternate list specified by the filter.");
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfBaseFilter* jfFFNTagFilter::GenCopy() const {
  return new jfFFNTagFilter(*this);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfFFNTagFilter::CoreMatch(const jfBasePD* testelem) const {
  const jfFFNItem* rvalue;
  // we start
  assert(testelem!=NULL);
  rvalue = dynamic_cast<const jfFFNItem*>(testelem);
  return MatchVsData(rvalue->GetGenres());
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// check the tags against a list of pre-approved tags
bool jfFFNTagFilter::DoVerify() {
  return VerifyTags(ff_tagfilt_count,ff_tagfilt_items,NULL);
}

/******************************************************************************/
