/******************************************************************************
Name    :   morefilters1.h
Author  :   John Q Metro
Purpose :   More Filters
Created :   December 6, 2010
Updated :   August 10, 2013 (add jfMinMaxFilter)
******************************************************************************/
#ifndef MOREFILTERS1_H_INCLUDED
#define MOREFILTERS1_H_INCLUDED
#endif
//---------------------------------------
#ifndef JFBASEFILTER
  #include "../base/filterbase.h"
#endif
#ifndef JFMISCTYPES1
	#include "../../objs/misc_types1.h"
#endif
/******************************************************************************/
class jfMinMaxUFilter : public jfBaseFilter {
  public:
    // constructors
    jfMinMaxUFilter();
    jfMinMaxUFilter(size_t inmin, size_t inmax);
    // setting values
    bool SetValues(size_t inmin, size_t inmax);
    // getting values
    size_t GetMin() const;
    size_t GetMax() const;
    // redefined virtual methods
    virtual bool IsEmpty() const;
    //virtual wxString GetTypeID() const = 0;
    // loading from a string representation
    virtual bool FromString(const wxString& sourcedata);
    virtual wxString ToString() const;
    // additional methods
    // virtual wxString GetTypeDescription() const = 0;
    // virtual jfBaseFilter* GenCopy() const = 0;
  protected:
    bool TestMatch(size_t invalue) const;
    // the core matching method
    // virtual bool CoreMatch(const jfBasePD* testelem) const = -;
    // file i/o
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    /* since different types are stored together, the text file reprentation
    may have objects of varying length */
    virtual size_t ExtraLines() const;
    // internal values
    size_t min, max;
};
//==========================================
class jfWordCountFilter : public jfMinMaxUFilter {
  public:
    // constructors
    jfWordCountFilter();
    jfWordCountFilter(size_t inmin, size_t inmax);
    // redefined virtual methods
    virtual wxString GetTypeID() const;
    virtual wxString GetTypeDescription() const;
    virtual jfBaseFilter* GenCopy() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
};
//=============================================================================
class jfTagFilterCore : public jfBaseFilter {
  public:
    // constructors
    jfTagFilterCore();
    jfTagFilterCore(const jfTagFilterCore& insrc);
    // getting and setting values
    bool SetTags(const jfTagListing* insource);
    jfTagListing* GetTagsCopy() const;
    virtual bool SetToEmpty() = 0;
    // redefined virtual methods
    virtual bool IsEmpty() const;
    virtual wxString GetTypeID() const = 0;
    // loading from a string representation
    virtual bool FromString(const wxString& sourcedata);
    virtual wxString ToString() const;
    // returns the list of element names campatible with this filter
    virtual wxString GetTypeDescription() const = 0;
    virtual jfBaseFilter* GenCopy() const = 0;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const = 0;
    bool MatchVsData(const wxString& incheck) const;
    void MakeNullResult();
    // check the tags against a list of pre-approved tags
    bool VerifyTags(size_t NUMTAGIN, const wxChar* intags[], wxArrayString* replist);
    virtual bool DoVerify() = 0;
    void SetEmpty(size_t NUMTAGIN, const wxChar* intags[]);
    // file i/o
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    /* since different types are stored together, the text file reprentation
    may have objects of varying length */
    virtual size_t ExtraLines() const;
    // internal values
    jfTagListing* thedata;
    wxString cm;
    bool nullresult;
    mutable size_t emptyc;
};
//=============================================================================
extern const size_t ff_tagfilt_count;
extern const wxChar* ff_tagfilt_items[];
//=============================================================================
class jfFFNTagFilter : public jfTagFilterCore {
  public:
    // constructors
    jfFFNTagFilter();
    jfFFNTagFilter(const jfFFNTagFilter& insrc);
    // getting and setting values
    virtual bool SetToEmpty();
    // redefined virtual methods
    virtual wxString GetTypeID() const;
    virtual wxString GetTypeDescription() const;
    virtual jfBaseFilter* GenCopy() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
    // check the tags against a list of pre-approved tags
    virtual bool DoVerify();
};
/******************************************************************************/
