/******************************************************************************
Name    :   morefilters2.cpp
Author  :   John Q Metro
Purpose :   More Filters
Created :   November 9, 2011
Updated :   July 12, 2015
******************************************************************************/
#ifndef MOREFILTERS2_H_INCLUDED
  #include "morefilters2.h"
#endif // MOREFILTERS2_H_INCLUDED
//=============================================================================
#ifndef UTILS1_H_INCLUDED
  #include "../../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef JFILTERGLOBALS
  #include "../base/filter_globals.h"
#endif
#ifndef JFFNFICOBJ
  #include "../../../fanficnet/data/ffn_ficobj.h"
#endif // BASEOBJ_H_INCLUDED
/*****************************************************************************/
// constructors
//-----------------------------------------------------
jfCompletedFilter::jfCompletedFilter():jfBaseFilter() {
  valset = false;
  validdata = true;
}
//-----------------------------------------------------
jfCompletedFilter::jfCompletedFilter(bool incexl_in):jfBaseFilter() {
  SetValue(incexl_in);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// match against the filter
bool jfCompletedFilter::IsEmpty() const {
  return !valset;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading from a string representation
//-----------------------------------------------------
bool jfCompletedFilter::FromString(const wxString& sourcedata) {
  jfLineParse* lparser;
  lparser = new jfLineParse(sourcedata);
  validdata = false;
  if (lparser->NNotX(2)) delete lparser;
  else {
    if ((lparser->BoolVal(0,valset)) && (lparser->BoolVal(1,incexl))) validdata = true;
    delete lparser;
  }
  return validdata;
}
//-----------------------------------------------------
wxString jfCompletedFilter::ToString() const {
  wxString result = Bool2Str(valset) + wxT(";");
  result += Bool2Str(incexl);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// data setting
//-----------------------------------------------------
void jfCompletedFilter::SetValue(bool incexl_in) {
  valset = true;
  incexl = incexl_in;
}
//-----------------------------------------------------
bool jfCompletedFilter::GetValue() const {
  return incexl;
}
//----------------------------------------------------------------
void jfCompletedFilter::SetEmpty() {
  valset = false;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// gets a description
wxString jfCompletedFilter::GetTypeDescription() const {
    return wxT("For items that have a completed flag, this filter passes if \
the filter is empty, or if the flag is either set or not (depending on the \
value).");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// copy
jfBaseFilter* jfCompletedFilter::GenCopy() const {
  jfCompletedFilter* result;
  result = new jfCompletedFilter();
  CopyOver(result);
  if (valset) result->SetValue(incexl);
  return result;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special meta-information
wxString jfCompletedFilter::GetTypeID() const {
  return wxT("CompletedFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfCompletedFilter::CoreMatch(const jfBasePD* testelem) const {
  const jfGenericFanfic2* rvalue;
  bool cvalue;
  assert(testelem!=NULL);
  rvalue = dynamic_cast<const jfGenericFanfic2*>(testelem);
  cvalue = rvalue->IsComplete();
  return (incexl==cvalue);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o
//-----------------------------------------------------
bool jfCompletedFilter::AddRestToFile(wxTextFile* outfile) const {
  wxString buffer;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // composing line 4
  buffer = ToString();
  outfile->AddLine(buffer);
  return true;
}
//-----------------------------------------------------
bool jfCompletedFilter::ReadRestFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfCompletedFilter::ReadRestFromFile");
  // input data
  wxString cline;
  bool resx;
  // starting checks (and reading the line)
  assert(infile!=NULL);
  if (!infile->ReadLine(cline,funcname)) return false;
  // there is only one line, and one filed, so this is pretty simple
  resx = FromString(cline);
  if (!resx) return infile->BuildError(wxT("The completed string is invalid!"));
  else return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* since different types are stored together, the text file reprentation may
have objects of varying length */
size_t jfCompletedFilter::ExtraLines() const { return 1;}
/*****************************************************************************/
const wxString FFN_RATINGS = wxT("K+TM");
//============================================================================
// constructors
//--------------------------------------------------
jfFFNRatingFilter::jfFFNRatingFilter() {
  value = FFN_RATINGS;
}
//--------------------------------------------------
jfFFNRatingFilter::jfFFNRatingFilter(wxString value_in) {
  FromString(value_in);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// match against the filter, note this currently dependant on fanfiction.net
bool jfFFNRatingFilter::IsEmpty() const {
  return (FFN_RATINGS==value);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading from a string representation
//--------------------------------------------------
bool jfFFNRatingFilter::FromString(const wxString& sourcedata) {
  size_t nval;
  nval = sourcedata.find_first_not_of(FFN_RATINGS);
  value = sourcedata;
  return (nval!=wxString::npos);
}
//--------------------------------------------------
wxString jfFFNRatingFilter::ToString() const {
  return value;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// gets a description
wxString jfFFNRatingFilter::GetTypeDescription() const {
    return wxT("Filters items depending on their rating. The default is to \
include all ratings.");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// copy
jfBaseFilter* jfFFNRatingFilter::GenCopy() const {
  jfFFNRatingFilter* result;
  result = new jfFFNRatingFilter();
  CopyOver(result);
  result->FromString(value);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special meta-information
wxString jfFFNRatingFilter::GetTypeID() const {
  return wxT("FFNRatingFilter");
}
//--------------------------------------------------------------
// custom methods
bool jfFFNRatingFilter::Includes(wxChar test) const {
  return (value.find(test)!=wxString::npos);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfFFNRatingFilter::CoreMatch(const jfBasePD* testelem) const {
  const jfFFNItem* rvalue;
  wxChar cvalue;
  assert(testelem!=NULL);
  rvalue = dynamic_cast<const jfFFNItem*>(testelem);
  cvalue = rvalue->GetRating();
  return ((value.find(cvalue)!=wxString::npos));
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o
//--------------------------------------------------
bool jfFFNRatingFilter::AddRestToFile(wxTextFile* outfile) const {
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // composing line 4
  outfile->AddLine(value);
  return true;
}
//--------------------------------------------------
bool jfFFNRatingFilter::ReadRestFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfFFNRatingFilter::ReadRestFromFile");
  // input data
  wxString cline;
  bool resx;
  // starting checks (and reading the line)
  assert(infile!=NULL);
  if (!infile->ReadLine(cline,funcname)) return false;
  // there is only one line, and one filed, so this is pretty simple
  resx = FromString(cline);
  if (!resx) return infile->BuildError(wxT("The completed string is invalid!"));
  else return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* since different types are stored together, the text file reprentation
may have objects of varying length */
size_t jfFFNRatingFilter::ExtraLines() const {return 1;}

/*****************************************************************************/

