/******************************************************************************
Name    :   morefilters2.h
Author  :   John Q Metro
Purpose :   More Filters
Created :   November 9, 2011
Updated :   July 12, 2015
******************************************************************************/
#ifndef MOREFILTERS2_H_INCLUDED
#define MOREFILTERS2_H_INCLUDED
#endif // MOREFILTERS2_H_INCLUDED
//=============================================================================
#ifndef JFBASEFILTER
  #include "../base/filterbase.h"
#endif
#ifndef JF_CORETYPES
  #include "../../objs/coretypes.h"
#endif
//=============================================================================
/*****************************************************************************/
class jfCompletedFilter : public jfBaseFilter {
  public:
    // constructors
    jfCompletedFilter();
    jfCompletedFilter(bool incexl_in);
    // match against the filter
    virtual bool IsEmpty() const;
    // loading from a string representation
    virtual bool FromString(const wxString& sourcedata);
    virtual wxString ToString() const;
    // data setting
    void SetValue(bool incexl_in);
    bool GetValue() const;
    void SetEmpty();
    // gets a description
    virtual wxString GetTypeDescription() const;
    // copy
    virtual jfBaseFilter* GenCopy() const;
    // special meta-information
    virtual wxString GetTypeID() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
    // file i/o
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    /* since different types are stored together, the text file reprentation
    may have objects of varying length */
    virtual size_t ExtraLines() const;
    // internal data
    bool valset, incexl;
};
//=============================================================================
extern const wxString FFN_RATINGS;
//--------------------------------------------------------------------
class jfFFNRatingFilter : public jfBaseFilter {
  public:
    // constructors
    jfFFNRatingFilter();
    jfFFNRatingFilter(wxString value_in);
    // match against the filter
    virtual bool IsEmpty() const;
    // loading from a string representation
    virtual bool FromString(const wxString& sourcedata);
    virtual wxString ToString() const;
    // gets a description
    virtual wxString GetTypeDescription() const;
    // copy
    virtual jfBaseFilter* GenCopy() const;
    // special meta-information
    virtual wxString GetTypeID() const;
    // custom methods
    bool Includes(wxChar test) const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
    // file i/o
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    /* since different types are stored together, the text file reprentation
    may have objects of varying length */
    virtual size_t ExtraLines() const;
    // internal data
    wxString value;
};
/*****************************************************************************/
