/******************************************************************************
Name    :   morefilters3.cpp
Author  :   John Q Metro
Purpose :   More Filters
Created :   November 29, 2011
Updated :   July 12, 2015
******************************************************************************/
#ifndef MOREFILTERS3_H_INCLUDED
  #include "morefilters3.h"
#endif // MOREFILTERS3_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../utils/logging.h"
#endif // LOGGING_H_INCLUDED
//=============================================================================
// constructors
//-------------------
jfDateRangeFilter::jfDateRangeFilter():jfBaseFilter() {
  newdate = wxInvalidDateTime;
  olddate = wxInvalidDateTime;
  validdata = true;
}
//----------------
jfDateRangeFilter::jfDateRangeFilter(wxDateTime instart,wxDateTime inend) {
  validdata = SetDates(instart,inend);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// match against the filter
bool jfDateRangeFilter::IsEmpty() const {
  return (!newdate.IsValid()) && (!olddate.IsValid());
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading from a string representation
bool jfDateRangeFilter::FromString(const wxString& sourcedata) {
  // constants
  const wxString fname = wxT("jfDateRangeFilter::FromString");
  // variables
  const wxChar* npointer;
  wxDateTime testdate;
  size_t divider, leng;
  wxString p1, p2;
  // special case to start
  /**/jfXLogMessage(fname,1,sourcedata);
  if (sourcedata==wxEmptyString) {
    newdate = wxInvalidDateTime;
    olddate = wxInvalidDateTime;
    validdata = true;
    return true;
  }
  /**/jfXLogMessage(fname,2);
  // splitting as per normal as per normal
  divider = sourcedata.find(wxT("to"));
  if (divider==wxString::npos) return false;
  /**/jfXLogMessage(fname,3);
  p1 = sourcedata.substr(0,divider);
  leng = sourcedata.length();
  p2 = sourcedata.substr(divider+2,leng-(divider+2));
  Dtrim(p1);
  Dtrim(p2);
  /**/jfXLogMessage(fname,4);
  // assigning based on values
  if (p1.IsEmpty()) newdate = wxInvalidDateTime;
  else {
    npointer = testdate.ParseFormat(p1,wxT("%m/%d/%Y"));
    /**/jfXLogMessage(fname,5,p1);
    if (npointer==NULL) return false;
    newdate = DayPlus(testdate,false);
  }
  if (p2.IsEmpty()) olddate = wxInvalidDateTime;
  else {
    npointer = testdate.ParseFormat(p2,wxT("%m/%d/%Y"));
    /**/jfXLogMessage(fname,6,p2);
    if (npointer==NULL) return false;
    olddate = testdate;
  }
  /**/jfXLogMessage(fname,7);
  validdata = true;
  return true;
}
//-------------------------------------------------------------
wxString jfDateRangeFilter::ToString() const {
  // variables
  wxDateTime xdate;
  wxString result;
  // new date
  if (newdate.IsValid()) {
    xdate = DayPlus(newdate,true);
    result = xdate.Format(wxT("%m/%d/%Y"));
  }
  result += wxT(" to ");
  // old date
  if (olddate.IsValid()) {
    result += olddate.Format(wxT("%m/%d/%Y"));
  }
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting data
bool jfDateRangeFilter::SetDates(wxDateTime instart,wxDateTime inend) {
  // tester
  if ((instart.IsValid()) && (inend.IsValid())) {
    if (instart<inend) return false;
  }
  // setting the dates
  newdate = instart;
  olddate = inend;
  // removing times for comparison. also, date range will be inclusive
  if (olddate.IsValid()) olddate.ResetTime();
  if (newdate.IsValid()) newdate = DayPlus(newdate,false);
  // finishing
  validdata = true;
  return true;
}
//----------------------------------------------------------------------------
wxDateTime jfDateRangeFilter::GetValue(bool first) const {
  if (first) {
    wxDateTime zvalue = DayPlus(newdate,true);
    return zvalue;
  }
  else return olddate;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special methods
wxDateTime jfDateRangeFilter::DayPlus(wxDateTime inval, bool decrement) const {
  // variables
  wxTimeSpan OneDay = wxTimeSpan::Days(1);
  // special case
  if (!inval.IsValid()) return inval;
  // other cases
  inval.ResetTime();
  if (decrement) inval.Subtract(OneDay);
  else inval.Add(OneDay);
  return inval;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfDateRangeFilter::RealMatchCore(const wxDateTime intest) const {
  // special cases, will ignore
  if (!intest.IsValid()) return true;
  if (IsEmpty()) return true;
  // testing
  if (newdate.IsValid()) {
    if (intest>=newdate) return false;
  }
  if (olddate.IsValid()) {
    if (intest<olddate) return false;
  }
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o
//-------------------------------------------------------------
bool jfDateRangeFilter::AddRestToFile(wxTextFile* outfile) const {
  wxString buffer;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // composing line 4
  buffer = ToString();
  outfile->AddLine(buffer);
  return true;
}
//-------------------------------------------------------------
bool jfDateRangeFilter::ReadRestFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfDateRangeFilter::ReadRestFromFile");
  // input data
  wxString cline;
  bool resx;
  // starting checks (and reading the line)
  assert(infile!=NULL);
  if (!infile->ReadLine(cline,funcname)) return false;
  // there is only one line, and one filed, so this is pretty simple
  resx = FromString(cline);
  if (!resx) return infile->BuildError(wxT("The completed string is invalid!"));
  else return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* since different types are stored together, the text file reprentation may have objects of varying length */
size_t jfDateRangeFilter::ExtraLines() const { return 1;}
//***************************************************************************
// constructors
//----------------------
jfUpdatedDateFilter::jfUpdatedDateFilter():jfDateRangeFilter() {}
//----------------------
jfUpdatedDateFilter::jfUpdatedDateFilter(wxDateTime instart,wxDateTime inend):jfDateRangeFilter(instart,inend) {}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// gets a description
wxString jfUpdatedDateFilter::GetTypeDescription() const {
    return wxT("Filters items depending on their updated date (If the item has \
never been updated, the published date is used instead). The default is to \
include everything.");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// copy
jfBaseFilter* jfUpdatedDateFilter::GenCopy() const {
  jfUpdatedDateFilter* result;
  result = new jfUpdatedDateFilter();
  CopyOver(result);
  result->SetDates(newdate,olddate);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special meta-information
wxString jfUpdatedDateFilter::GetTypeID() const { return wxT("UpdatedDateFilter");}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfUpdatedDateFilter::CoreMatch(const jfBasePD* testelem) const {
  // variables
  const jfGenericFanfic* rvalue;
  wxDateTime gdate;
  // asserts
  assert(testelem!=NULL);
  // doing
  rvalue = dynamic_cast<const jfGenericFanfic*>(testelem);
  gdate = rvalue->GetUpdates();
  return RealMatchCore(gdate);
}
//***************************************************************************
// constructors
//----------------------
jfPublishedDateFilter::jfPublishedDateFilter():jfDateRangeFilter() {}
//----------------------
jfPublishedDateFilter::jfPublishedDateFilter(wxDateTime instart,wxDateTime inend):jfDateRangeFilter(instart,inend) {}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// gets a description
wxString jfPublishedDateFilter::GetTypeDescription() const {
    return wxT("Filters items depending on their Published date. The default is to \
include everything.");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// copy
jfBaseFilter* jfPublishedDateFilter::GenCopy() const {
  jfPublishedDateFilter* result;
  result = new jfPublishedDateFilter();
  CopyOver(result);
  result->SetDates(newdate,olddate);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special meta-information
wxString jfPublishedDateFilter::GetTypeID() const { return wxT("PublishedDateFilter");}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfPublishedDateFilter::CoreMatch(const jfBasePD* testelem) const {
  // variables
  const jfFFNItem* rvalue;
  wxDateTime gdate;
  // asserts
  assert(testelem!=NULL);
  // doing
  rvalue = dynamic_cast<const jfFFNItem*>(testelem);
  gdate = rvalue->GetPublished();
  return RealMatchCore(gdate);
}
/******************************************************************************/
// constructors
//---------------------------------------------------
jfFFNFavsFilter::jfFFNFavsFilter():jfMinMaxUFilter() {}
//---------------------------------------------------
jfFFNFavsFilter::jfFFNFavsFilter(size_t inmin, size_t inmax):jfMinMaxUFilter(inmin,inmax) {}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfFFNFavsFilter::GetTypeID() const {
  return wxT("FFNFavsFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// returns the list of element names campatible with this filter
//---------------------------------------------------
wxString jfFFNFavsFilter::GetTypeDescription() const {
    return wxT("For Fanfiction.Net fics, this filter passes if \
the number of Favs is between min and max, inclusive.");
}
//---------------------------------------------------
jfBaseFilter* jfFFNFavsFilter::GenCopy() const {
  jfFFNFavsFilter* result;
  result = new jfFFNFavsFilter(min,max);
  CopyOver(result);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfFFNFavsFilter::CoreMatch(const jfBasePD* testelem) const {
  // variables
  const jfFFNItemCore* rvalue;
  size_t cvalue;
  // checks
  assert(testelem!=NULL);
  // determining the type
  rvalue = dynamic_cast<const jfFFNItemCore*>(testelem);
  cvalue = rvalue->GetFavs();
  // checking the wordcount
  return TestMatch(cvalue);
}
/*****************************************************************************/
