/******************************************************************************
Name    :   morefilters3.h
Author  :   John Q Metro
Purpose :   More Filters
Created :   November 29, 2011
Updated :   August 19, 2013 (adding jfFIMGroupMCFilter)
******************************************************************************/
#ifndef MOREFILTERS3_H_INCLUDED
#define MOREFILTERS3_H_INCLUDED
#endif // MOREFILTERS3_H_INCLUDED
//=============================================================================
#ifndef UTILS1_H_INCLUDED
  #include "../../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef JFILTERGLOBALS
  #include "../base/filter_globals.h"
#endif
#ifndef JFFNFICOBJ
  #include "../../../fanficnet/data/ffn_ficobj.h"
#endif // BASEOBJ_H_INCLUDED
#ifndef MOREFILTERS1_H_INCLUDED
  #include "morefilters1.h"
#endif
/*****************************************************************************/
class jfDateRangeFilter : public jfBaseFilter {
  public:
    // constructors
    jfDateRangeFilter();
    jfDateRangeFilter(wxDateTime instart,wxDateTime inend);
    // match against the filter
    virtual bool IsEmpty() const;
    // loading from a string representation
    virtual bool FromString(const wxString& sourcedata);
    virtual wxString ToString() const;
    // setting and getting data
    bool SetDates(wxDateTime instart,wxDateTime inend);
    wxDateTime GetValue(bool first) const;
    // gets a description
    virtual wxString GetTypeDescription() const = 0;
    // copy
    virtual jfBaseFilter* GenCopy() const = 0;
    // special meta-information
    virtual wxString GetTypeID() const = 0;
  protected:
    // special methdos
    wxDateTime DayPlus(wxDateTime inval, bool decrement) const;
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const = 0;
    bool RealMatchCore(const wxDateTime intest) const;
    // file i/o
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    /* since different types are stored together, the text file reprentation
    may have objects of varying length */
    virtual size_t ExtraLines() const;
    // internal data
    wxDateTime newdate, olddate;
};
//============================================================================
class jfUpdatedDateFilter : public jfDateRangeFilter {
  public:
    // constructors
    jfUpdatedDateFilter();
    jfUpdatedDateFilter(wxDateTime instart,wxDateTime inend);
    // gets a description
    virtual wxString GetTypeDescription() const;
    // copy
    virtual jfBaseFilter* GenCopy() const;
    // special meta-information
    virtual wxString GetTypeID() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
};
//============================================================================
class jfPublishedDateFilter : public jfDateRangeFilter {
  public:
    // constructors
    jfPublishedDateFilter();
    jfPublishedDateFilter(wxDateTime instart,wxDateTime inend);
    // gets a description
    virtual wxString GetTypeDescription() const;
    // copy
    virtual jfBaseFilter* GenCopy() const;
    // special meta-information
    virtual wxString GetTypeID() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
};
//=============================================================================
class jfFFNFavsFilter : public jfMinMaxUFilter {
  public:
    // constructors
    jfFFNFavsFilter();
    jfFFNFavsFilter(size_t inmin, size_t inmax);
    // redefined virtual methods
    virtual wxString GetTypeID() const;
    virtual wxString GetTypeDescription() const;
    virtual jfBaseFilter* GenCopy() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
};
/*****************************************************************************/
