//***************************************************************************
// Name:        pairfilter_tools.cpp
// Author :     John Q Metro
// Purpose :    Relationship/Pairing filters are complicated enough to make a
//              separate source file for utilities and data
// Created:     December 29, 2010
// Updated:     May 29, 2012
//***************************************************************************
#ifndef PAIRFILTER_TOOLS_H_INCLUDED
  #include "pairfilter_tools.h"
#endif // PAIRFILTER_TOOLS_H_INCLUDED
//---------------------------------------------------------------
#ifndef UTILS1_H_INCLUDED
  #include "../../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef UTILS2_H_INCLUDED
  #include "../../utils/utils2.h"
#endif // UTILS1_H_INCLUDED
#ifndef JFEXPRBASE
  #include "../../express/expr_base.h"
#endif
#ifndef JFEXPRTOKEN
#include "../../express/expr_token.h"
#endif
#ifndef UTILS1_H_INCLUDED
  #include "../../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
//***************************************************************************
// a method to strip duplicate characters from a string
bool StripDuplicateChars(wxString& delm_str) {
  // variables
  wxString result = wxEmptyString;
  size_t str_len, str_loop;
  // start
  Dtrim(delm_str);
  if (delm_str.IsEmpty()) return false;
  else {
    // here, we really start, the technique is a copying loop that checks
    // if a char is already in the result before copying
    str_len = delm_str.length();
    for (str_loop=0;str_loop<str_len;str_loop++) {
      if (wxNOT_FOUND==(result.Find(delm_str[str_loop]))) result+= delm_str[str_loop];
    }
    delm_str = result;
  }
  return true;
}
//==========================================================================
jfPF_flaghold::jfPF_flaghold() {
  usetokens = false;
  sidespaces = true;
  nofriend = true;
  nospace = true;
  curr_char = wxT('/');
}
//-------------------------------------------------------------------------
// to and from string
//--------------------------
wxString jfPF_flaghold::ToString() const {
  jfOutString result;
  result << usetokens << sidespaces << nofriend << nospace;
  return wxString(result);
}
//---------------------------------------------
bool jfPF_flaghold::FromString(wxString inval) {
  jfLineParse parser;
  // basic parsing
  parser.SetString(inval);
  if (parser.NNotX(4)) return false;
  // field extraction
  if (!parser.BoolVal(0,usetokens)) return false;
  if (!parser.BoolVal(1,sidespaces)) return false;
  if (!parser.BoolVal(2,nofriend)) return false;
  if (!parser.BoolVal(3,nospace)) return false;
  // done
  return true;
}
//==========================================================================
/* used in the following function.. very restricted, basically handles
checking a single set, with tokenized vs not, and frienship excluding
*/
bool LookForPatternString(wxString lookin, wxString pattern, const jfPF_flaghold& flags) {
  // variables
  jfSubstringElem* slooker;
  bool result;
  // starting...
  assert(!flags.usetokens);
  slooker = new jfSubstringElem(pattern);
  result = slooker->Match(lookin);
  delete slooker;
  if (result && flags.nofriend) {
    pattern += wxT(" friendship");
    slooker = new jfSubstringElem(pattern);
    result = slooker->Match(lookin);
    delete slooker;
    return (!result);
  }
  else return result;
}
//----------------------------------------------------------------------------------
bool LookForPatternToken(const wxArrayString* lookin, wxString pattern, const jfPF_flaghold& flags) {
  // variables
  jfTokenMElem*    tlooker;
  bool result;
  // starting...
  tlooker = new jfTokenMElem(pattern);
  result = tlooker->Match(lookin);
  delete tlooker;
  if (result && flags.nofriend) {
    pattern += wxT(" friendship");
    tlooker = new jfTokenMElem(pattern);
    result = tlooker->Match(lookin);
    delete tlooker;
    return (!result);
  }
  else return result;

}
//--------------------------------------------------------------------------
// function to look for a name pair using one character
bool LookForPairSingleString(wxString lookin, wxString name1, wxString name2, const jfPF_flaghold& flags) {
  // variables
  wxString lookstring;
  bool rval;
  // special cases
  assert(!name1.IsEmpty());
  assert(!name2.IsEmpty());
  assert(!flags.usetokens);
  if (lookin.IsEmpty()) return false;
  // case 1 : name1 + char + name2
  lookstring = name1 + flags.curr_char + name2;
  rval = LookForPatternString(lookin,lookstring,flags);
  if (rval) return true;
  // case 2 : name2 + char + name1
  lookstring = name2 + flags.curr_char + name1;
  rval = LookForPatternString(lookin,lookstring,flags);
  if (rval) return true;
  // cases 3 and 4 are done if we have sidespaces
  if (flags.sidespaces) {
    // case 3 : name1 + space + char + space + name2
    lookstring = name1 + wxT(" ");
    lookstring += flags.curr_char;
    lookstring += wxT(" ") + name2;
    rval = LookForPatternString(lookin,lookstring,flags);
    if (rval) return true;
    // case 2 : name2 + space + char + space + name1
    lookstring = name2 + wxT(" ");
    lookstring += flags.curr_char;
    lookstring += wxT(" ") + name1;
    rval = LookForPatternString(lookin,lookstring,flags);
    if (rval) return true;
  }
  // done with nothing found!
  return false;
}
//---------------------------------------------------------------------------------------------
// function to look for a name pair using one character
bool LookForPairSingleToken(const wxArrayString* lookin, wxString name1, wxString name2, const jfPF_flaghold& flags) {
  // variables
  wxString lookstring;
  bool rval;
  // special cases
  assert(!name1.IsEmpty());
  assert(!name2.IsEmpty());
  if (lookin==NULL) return false;
  if (lookin->IsEmpty()) return false;
  assert(flags.usetokens);
  // case 1 : name1 + char + name2
  lookstring = name1 + flags.curr_char + name2;
  rval = LookForPatternToken(lookin,lookstring,flags);
  if (rval) return true;
  // case 2 : name2 + char + name1
  lookstring = name2 + flags.curr_char + name1;
  rval = LookForPatternToken(lookin,lookstring,flags);
  if (rval) return true;
  // cases 3 and 4 are done if we have sidespaces
  if (flags.sidespaces) {
    // case 3 : name1 + space + char + space + name2
    lookstring = name1 + wxT(" ");
    lookstring += flags.curr_char;
    lookstring += wxT(" ") + name2;
    rval = LookForPatternToken(lookin,lookstring,flags);
    if (rval) return true;
    // case 2 : name2 + space + char + space + name1
    lookstring = name2 + wxT(" ");
    lookstring += flags.curr_char;
    lookstring += wxT(" ") + name1;
    rval = LookForPatternToken(lookin,lookstring,flags);
    if (rval) return true;
  }
  // done with nothing found!
  return false;
}
//---------------------------------------------------------------------------------------------
// functions to look for a name pair via multiple characters
//---------------------------------------------------------------------------------------------
bool LookForPairString(wxString lookin, wxString name1, wxString name2, wxString inchars, jfPF_flaghold& flags) {
  // variables
  size_t clen, cloop;
  bool rval;
  wxString looker;
  // checks
  assert(!name1.IsEmpty());
  assert(!name2.IsEmpty());
  assert(!flags.usetokens);
  if (lookin.IsEmpty()) return false;
  // starting
  clen = inchars.length();
  // the main loop
  for (cloop=0;cloop<clen;cloop++) {
    flags.curr_char = inchars[cloop];
    rval = LookForPairSingleString(lookin,name1,name2,flags);
    if (rval) return true;
  }
  // at the end, there is another option, no separation at all
  if (flags.nospace) {
    flags.curr_char = wxChar('\0');
    looker = name1+name2;
    rval = LookForPatternString(lookin,looker,flags);
    if (rval) return true;
    looker = name2+name1;
    rval = LookForPatternString(lookin,looker,flags);
    return rval;
  }
  else return false;
}
//---------------------------------------------------------------------------------------------
bool LookForPairToken(const wxArrayString* lookin, wxString name1, wxString name2, wxString inchars, jfPF_flaghold& flags) {
  // variables
  size_t clen, cloop;
  bool rval;
  wxString looker;
  // checks
  assert(!name1.IsEmpty());
  assert(!name2.IsEmpty());
  assert(flags.usetokens);
  assert(lookin!=NULL);
  if (lookin->IsEmpty()) return false;
  // starting
  clen = inchars.length();
  // the main loop
  for (cloop=0;cloop<clen;cloop++) {
    flags.curr_char = inchars[cloop];
    rval = LookForPairSingleToken(lookin,name1,name2,flags);
    if (rval) return true;
  }
  // at the end, there is another option, no separation at all
  if (flags.nospace) {
    flags.curr_char = wxChar('\0');
    looker = name1+name2;
    rval = LookForPatternToken(lookin,looker,flags);
    if (rval) return true;
    looker = name2+name1;
    rval = LookForPatternToken(lookin,looker,flags);
    return rval;
  }
  else return false;
}
//=========================================================================
bool jfPaircount::AddOrIncrement(const wxString& inval) {
  if (find(inval)==end()) {
    (*this)[inval] = 1;
    return true;
  }
  else {
    ((*this)[inval]++);
    return false;
  }
}
//=========================================================================
// constructor
jfPairStats::jfPairStats(wxString inchars) {
  // setting the separator list, we assume it has already been checked
  sepchars = inchars;
  // the first item in charfcount is reserved for nothing at all
  charfcount.push_back(0);
  // next, we initialize the other charfcount slots
  for(size_t cloop=0;cloop<inchars.length();cloop++) charfcount.push_back(0);
  // other stuff
  maxname_length = 0;
  maxspec_length = 0;
  filename = wxT("pairstats_default.txt");
  topname = wxT("Default");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the data gathering method
bool jfPairStats::LogFind(wxString name1, wxString name2, wxChar inchar) {
  // variables
  int findex;
  wxString logname;

  // checking which char was used for the separator
  if (inchar==wxT('\0')) charfcount[0]++;
  else {
    findex = sepchars.Find(inchar);
    if (findex!=wxNOT_FOUND) {
      charfcount[findex+1]++;
    }
    else assert(false);
  }
  // next up, we log the name combo in the map
  logname = name1 + wxT(" / ") + name2;
  if (logname.length()>maxname_length) maxname_length = logname.length();
  return maindata.AddOrIncrement(logname);
}
//------------------------------------------------------------------
bool jfPairStats::LogSpecial(wxString identifier) {
  assert(!identifier.IsEmpty());
  if (identifier.length()>maxspec_length) maxspec_length = identifier.length();
  return specials.AddOrIncrement(identifier);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file name stuff
wxString jfPairStats::GetFilename() const {
  return filename;
}
//---------------------------------------------------------------------
void jfPairStats::SetFilename(wxString infilename, wxString newname) {
  assert(!infilename.IsEmpty());
  filename = infilename;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o
void jfPairStats::WriteResults() const {
  // local variables
  jfPaircount::const_iterator iloop;
  size_t charmax, charloop;
  wxTextFile* outfile;
  wxString curline;
  bool owres;
  // setting the values
  charmax = sepchars.length()+1;
  // we get the output file
  outfile = OpenResult();
  // character stats
  curline = wxT("Separator Stats");
  WriteSecHead(curline,outfile,true);
  for (charloop=0;charloop<charmax;charloop++) {
    curline = MakeCharResult(charloop);
    outfile->AddLine(curline);
  }
  // specials
  curline = wxT("Special Keyword Stats");
  WriteSecHead(curline,outfile,true);
  iloop = specials.begin();
  while (iloop!=specials.end()) {
    curline = MakeNameResult(iloop,maxspec_length);
    outfile->AddLine(curline);
    iloop++;
  }
  // name pairs
  curline = wxT("Name Pair Stats");
  WriteSecHead(curline,outfile,true);
  iloop = maindata.begin();
  while (iloop!=maindata.end()) {
    curline = MakeNameResult(iloop,maxname_length);
    outfile->AddLine(curline);
    iloop++;
  }
  // done
  owres = outfile->Write();
  assert(owres);
  outfile->Close();
  delete outfile;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods
//----------------------------------------------
wxString jfPairStats::MakeCharResult(size_t index) const {
  assert(index<=sepchars.length());
  wxString result;
  // we build the label part first
  if (index==0) result = wxT(" null : ");
  else if (sepchars[index]==wxT(' ')) result = wxT("space : ");
  else {
    result = wxT("  ");
    result += sepchars[index];
    result += wxT("  : ");
  }
  // then we add the number
  result += SInt2Str(charfcount[index]);
  // done
  return result;
}
//----------------------------------------------
wxString jfPairStats::MakeNameResult(const jfPaircount::const_iterator& inval, size_t padlength) const {
  // result
  wxString result;
  wxString cval;
  size_t cvlen;
  // we will assume here that inval is valid and correct
  cval = inval->first;
  cvlen = cval.length();
  // padding
  cval.Pad(padlength-cvlen);
  // building the label
  result = cval + wxT(" : ");
  // we add the number
  result += UInt2Str(inval->second);
  // done
  return result;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o helper methods
wxTextFile* jfPairStats::OpenResult() const {
  // variables
  wxTextFile* result;
  wxString bufline;
  // creating or opening the file
  result = new wxTextFile(filename);
  if (result->Exists()) {
    result->Open();
    result->Clear();
  }
  else result->Create();
  // we add the header
  bufline = wxT("Statistics for : ") + topname;
  WriteSecHead(bufline,result);
  // done
  return result;
}
//--------------------------------------------------------------------------
void jfPairStats::WriteSecHead(wxString inname, wxTextFile* outfile_in, bool spacebefore) const {
  // variables
  wxString bufline = wxEmptyString;
  // we do the usual checks
  assert(outfile_in!=NULL);
  assert(outfile_in->IsOpened());
  assert(!inname.IsEmpty());
  // space before, first
  if (spacebefore) outfile_in->AddLine(wxEmptyString);
  // the name
  outfile_in->AddLine(inname);
  // the -------------------
  bufline.Pad(inname.length()+5,wxT('-'));
  outfile_in->AddLine(bufline);
  // done
}

//**************************************************************************
