//***************************************************************************
// Name:        pairfilter_tools.h
// Author :     John Q Metro
// Purpose :    Relationship/Pairing filters are complicated enough to make a
//              separate source file for utilities and data
// Created:     December 29, 2010
// Updated:     January 1, 2011
//***************************************************************************
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <wx/textfile.h>
//-----------------------------------------------------------------
#ifndef PAIRFILTER_TOOLS_H_INCLUDED
#define PAIRFILTER_TOOLS_H_INCLUDED
#endif // PAIRFILTER_TOOLS_H_INCLUDED
//---------------------------------------------------------------
#ifndef BASECOLLECT_H_INCLUDED
  #include "../../structs/basecollect.h"
#endif // BASECOLLECT_H_INCLUDED
#ifndef VECTORS_H_INCLUDED
  #include "../../structs/vectors.h"
#endif // VECTORS_H_INCLUDED

#include <map>
//***************************************************************************
// a method to strip duplicate characters from a string
bool StripDuplicateChars(wxString& delm_str);
//---------------------------------------------------------------------------
// flag holder (to shorten parameters)
class jfPF_flaghold : public wxObject {
  public:
    // the flags
    bool usetokens;
    bool sidespaces;
    bool nofriend;
    bool nospace;
    wxChar curr_char;
    // constructor
    jfPF_flaghold();
    // to and from string
    wxString ToString() const;
    bool FromString(wxString inval);
  protected:
};
//---------------------------------------------------------------------------
// used in the LookForPairSingle  function.. very restricted
bool LookForPatternString(wxString lookin, wxString pattern, const jfPF_flaghold& flags);
bool LookForPatternToken(const wxArrayString* lookin, wxString pattern, const jfPF_flaghold& flags);
// function to look for a name pair using one character
bool LookForPairSingleString(wxString lookin, wxString name1, wxString name2, const jfPF_flaghold& flags);
bool LookForPairSingleToken(const wxArrayString* lookin, wxString name1, wxString name2, const jfPF_flaghold& flags);
// functions to look for a name pair via multiple characters
bool LookForPairString(wxString lookin, wxString name1, wxString name2, wxString inchars,jfPF_flaghold& flags);
bool LookForPairToken(const wxArrayString* lookin, wxString name1, wxString name2, wxString inchars,jfPF_flaghold& flags);
//--------------------------------------------------------------------------
class jfPaircount : public std::map<wxString,size_t,ltstr> {
  public:
    bool AddOrIncrement(const wxString& inval);
};
//----------------------------------------------------------------------------------
// special class used to keep track of statistics
class jfPairStats : public wxObject {
  public:
    // constructor
    jfPairStats(wxString inchars);
    // the data gathering method
    bool LogFind(wxString name1, wxString name2, wxChar inchar);
    bool LogSpecial(wxString identifier);
    // file name stuff
    wxString GetFilename() const;
    void SetFilename(wxString infilename, wxString newname);
    // file i/o
    void WriteResults() const;
  protected:
    // helper methods
    wxString MakeCharResult(size_t index) const;
    wxString MakeNameResult(const jfPaircount::const_iterator& inval, size_t padlength) const;
    // file i/o helper methods
    wxTextFile* OpenResult() const;
    void WriteSecHead(wxString inname, wxTextFile* outfile_in, bool spacebefore=false) const;
    // internal data
    wxString sepchars;
    jfPaircount specials;
    jfPaircount maindata;
    jfIndexVector charfcount;
    wxString filename;
    wxString topname;
    size_t maxname_length;
    size_t maxspec_length;
};
//***************************************************************************
