/******************************************************************************
Name    :   fimfilters1.cpp
Author  :   John Q Metro
Purpose :   Filters for fimfiction.net
Created :   July 4, 2012
Updated :   August 21, 2013
******************************************************************************/
#ifndef FIMFILTERS1_H_INCLUDED
  #include "fimfilters1.h"
#endif // FIMFILTERS1_H_INCLUDED
#ifndef FIM_FICOBJ_H_INCLUDED
  #include "../../../fanfic/data/fim/fim_ficobj.h"
#endif // FIM_FICOBJ_H_INCLUDED
#ifndef FIM_CONSTANTS_H_INCLUDED
  #include "../../../fanfic/data/fim/fim_constants.h"
#endif // FIM_CONSTANTS_H_INCLUDED
//-------------------------------------------------
#ifndef LOGGING_H_INCLUDED
  #include "../../utils/logging.h"
#endif // LOGGING_H_INCLUDED
/*****************************************************************************/
jfFimThumbsFilter::jfFimThumbsFilter():jfBaseFilter() {
  validdata = false;
  min = -9999;
  max =  9999;
}
//---------------------------------------------------
jfFimThumbsFilter::jfFimThumbsFilter(int inmin, int inmax):jfBaseFilter() {
  validdata = SetValues(inmin,inmax);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting values
bool jfFimThumbsFilter::SetValues(int inmin, int inmax) {
  if (inmax>=inmin) {
    min = inmin;
    max = inmax;
    validdata = true;
  }
  else validdata = false;
  return validdata;
}
//-----------------------------------------------------------------------
// getting values
//-------------------------------------------
int jfFimThumbsFilter::GetMin() const { return min; }
//-------------------------------------------
int  jfFimThumbsFilter::GetMax() const { return max; }
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// redefined virtual methods
bool jfFimThumbsFilter::IsEmpty() const { return false; }
//----------------------------------------------------------------------
wxString jfFimThumbsFilter::GetTypeID() const {
  return wxT("FimThumbsFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading from a string representation
//---------------------------------------------------
bool jfFimThumbsFilter::FromString(const wxString& sourcedata) {
  jfLineParse* lparser;
  lparser = new jfLineParse(sourcedata);
  validdata = false;
  if (lparser->NNotX(2)) delete lparser;
  else {
    if ((lparser->IIntVal(0,min)) && (lparser->IIntVal(1,max))) validdata = true;
    delete lparser;
  }
  return validdata;
}
//---------------------------------------------------
wxString jfFimThumbsFilter::ToString() const {
  wxString result;
  result = IInt2Str(min) + wxT(";");
  result += IInt2Str(max);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// returns the list of element names campatible with this filter
//---------------------------------------------------
wxString jfFimThumbsFilter::GetTypeDescription() const {
    return wxT("Fimfiction.net fics have thumbs up and thumbs down rating.\
This filter passes fics for which the thumbsup-thumbsdown lies between the min and max.");
}
//---------------------------------------------------
jfBaseFilter* jfFimThumbsFilter::GenCopy() const {
  jfFimThumbsFilter* result;
  result = new jfFimThumbsFilter(min,max);
  CopyOver(result);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfFimThumbsFilter::CoreMatch(const jfBasePD* testelem) const {
  const wxString fname = wxT("jfFimThumbsFilter::CoreMatch");
  // variables
  const jfFIM_Fanfic* rvalue;
  int tnet;
  // checks
  assert(testelem!=NULL);
  // determining the type
  rvalue = dynamic_cast<const jfFIM_Fanfic*>(testelem);
  /**/jfXLogMessage(fname,1,rvalue->GetShortSummary());
  tnet = (int)(rvalue->GetThumbs(false)) - (int)(rvalue->GetThumbs(true));
  // checking the FimThumbs
  assert(tnet<=999999999);
  if (tnet<min) return false;
  else return (tnet<=max);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o
//-----------------------------------
bool jfFimThumbsFilter::AddRestToFile(wxTextFile* outfile) const {
  wxString buffer;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // composing line 4
  buffer = ToString();
  outfile->AddLine(buffer);
  return true;
}
//-----------------------------------
bool jfFimThumbsFilter::ReadRestFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfFimThumbsFilter::ReadRestFromFile");
  // input data
  wxString cline;
  bool resx;
  // starting checks (and reading the line)
  assert(infile!=NULL);
  if (!infile->ReadLine(cline,funcname)) return false;
  // there is only one line, and one filed, so this is pretty simple
  resx = FromString(cline);
  if (!resx) return infile->BuildError(wxT("The min-max string is invalid!"));
  else return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* since different types are stored together, the text file reprentation
may have objects of varying length */
size_t jfFimThumbsFilter::ExtraLines() const { return 1;}
//=======================================================================
jfFIMGenreFilter::jfFIMGenreFilter():jfTagFilterCore() {
  cm = wxT(",");
}
//---------------------------------------------
jfFIMGenreFilter::jfFIMGenreFilter(const jfFIMGenreFilter& insrc):jfTagFilterCore(insrc) {
  cm = wxT(",");
  validdata = DoVerify();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting and setting values
bool jfFIMGenreFilter::SetToEmpty() {
  SetEmpty(fimcon::genrecount,fimcon::genrelist2);
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// redefined virtual methods
wxString jfFIMGenreFilter::GetTypeID() const {
  return wxT("FIMGenreFilter");
}
//-----------------------------------------------------------------------------
wxString jfFIMGenreFilter::GetTypeDescription() const {
    return wxT("Compares the Fimfiction.net genres of the fic against the \
include/exclude/alternate list specified by the filter.");
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfBaseFilter* jfFIMGenreFilter::GenCopy() const {
  return new jfFIMGenreFilter(*this);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfFIMGenreFilter::CoreMatch(const jfBasePD* testelem) const {
  const jfFIM_Fanfic* rvalue;
  // we start
  assert(testelem!=NULL);
  rvalue = dynamic_cast<const jfFIM_Fanfic*>(testelem);
  return MatchVsData(rvalue->GetGenres());
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// check the tags against a list of pre-approved tags
bool jfFIMGenreFilter::DoVerify() {
  return VerifyTags(fimcon::genrecount,fimcon::genrelist2,NULL);
}
//=======================================================================
wxArrayString* MakeFIMCharRep() {
  wxArrayString* newrep = new wxArrayString();
  newrep->Add(wxT("DJ P0N-3"));
  newrep->Add(wxT("Vinyl Scratch"));
  return newrep;
}
//=====================================================================
jfFIMCharacterFilter::jfFIMCharacterFilter():jfTagFilterCore() {
  cm = wxT(",");
}
//---------------------------------------------
jfFIMCharacterFilter::jfFIMCharacterFilter(const jfFIMCharacterFilter& insrc):jfTagFilterCore(insrc) {
  cm = wxT(",");
  validdata = DoVerify();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting and setting values
bool jfFIMCharacterFilter::SetToEmpty() {
  SetEmpty(fimcon::charcount,fimcon::charnames);
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// redefined virtual methods
wxString jfFIMCharacterFilter::GetTypeID() const {
  return wxT("FIMCharacterFilter");
}
//-----------------------------------------------------------------------------
wxString jfFIMCharacterFilter::GetTypeDescription() const {
    return wxT("Compares the Fimfiction.net Characters of the fic against the \
include/exclude/alternate list specified by the filter.");
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfBaseFilter* jfFIMCharacterFilter::GenCopy() const {
  return new jfFIMCharacterFilter(*this);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfFIMCharacterFilter::CoreMatch(const jfBasePD* testelem) const {
  const jfFIM_Fanfic* rvalue;
  // we start
  assert(testelem!=NULL);
  rvalue = dynamic_cast<const jfFIM_Fanfic*>(testelem);
  return MatchVsData(rvalue->GetCharacters());
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// check the tags against a list of pre-approved tags
bool jfFIMCharacterFilter::DoVerify() {
  wxArrayString* newrep = MakeFIMCharRep();
  bool rval = VerifyTags(fimcon::charcount,fimcon::charnames,newrep);
  delete newrep;
  return rval;
}
//******************************************************************************
const wxString FIM_RATINGS = wxT("ETM");
//============================================================================
// constructors
//--------------------------------------------------
jfFimRatingFilter::jfFimRatingFilter() {
  value = FIM_RATINGS;
}
//--------------------------------------------------
jfFimRatingFilter::jfFimRatingFilter(wxString value_in) {
  FromString(value_in);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// match against the filter, note this currently dependant on fimfiction.net
bool jfFimRatingFilter::IsEmpty() const {
  return (FIM_RATINGS==value);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading from a string representation
//--------------------------------------------------
bool jfFimRatingFilter::FromString(const wxString& sourcedata) {
  size_t nval;
  nval = sourcedata.find_first_not_of(FIM_RATINGS);
  value = sourcedata;
  return (nval==wxString::npos);
}
//--------------------------------------------------
wxString jfFimRatingFilter::ToString() const {
  return value;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// gets a description
wxString jfFimRatingFilter::GetTypeDescription() const {
    return wxT("Filters items depending on their rating. The default is to \
include all ratings.");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// copy
jfBaseFilter* jfFimRatingFilter::GenCopy() const {
  jfFimRatingFilter* result;
  result = new jfFimRatingFilter();
  CopyOver(result);
  result->FromString(value);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special meta-information
wxString jfFimRatingFilter::GetTypeID() const {
  return wxT("FimRatingFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfFimRatingFilter::CoreMatch(const jfBasePD* testelem) const {
  const jfFIM_Fanfic* rvalue;
  wxString cvalue;
  assert(testelem!=NULL);
  rvalue = dynamic_cast<const jfFIM_Fanfic*>(testelem);
  cvalue = rvalue->GetRating();
  return ((value.find(cvalue[0])!=wxString::npos));
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o
//--------------------------------------------------
bool jfFimRatingFilter::AddRestToFile(wxTextFile* outfile) const {
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // composing line 4
  outfile->AddLine(value);
  return true;
}
//============================================================================


//--------------------------------------------------------------
// custom methods
bool jfFimRatingFilter::Includes(wxString test) const {
  return (value.find(test[0])!=wxString::npos);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o
//--------------------------------------------------

//--------------------------------------------------
bool jfFimRatingFilter::ReadRestFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfFimRatingFilter::ReadRestFromFile");
  // input data
  wxString cline;
  bool resx;
  // starting checks (and reading the line)
  assert(infile!=NULL);
  if (!infile->ReadLine(cline,funcname)) return false;
  // there is only one line, and one filed, so this is pretty simple
  resx = FromString(cline);
  if (!resx) return infile->BuildError(wxT("The completed string is invalid!"));
  else return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* since different types are stored together, the text file reprentation
may have objects of varying length */
size_t jfFimRatingFilter::ExtraLines() const {return 1;}
//******************************************************************************
// constructors
//--------------------------------------------------
jfFimThumbPercentFilter::jfFimThumbPercentFilter() {
  value = 85;
}
//--------------------------------------------------
jfFimThumbPercentFilter::jfFimThumbPercentFilter(wxString value_in) {
  FromString(value_in);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// match against the filter, note this currently dependant on fimfiction.net
bool jfFimThumbPercentFilter::IsEmpty() const {
  return (0==value);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading from a string representation
//--------------------------------------------------
bool jfFimThumbPercentFilter::FromString(const wxString& sourcedata) {
  unsigned long tval;
  if (!Str2ULong(sourcedata,tval)) return false;
  if (tval>100) return false;
  value = tval;
  return true;
}
//--------------------------------------------------
wxString jfFimThumbPercentFilter::ToString() const {
  return SInt2Str(value);
}
//---------------------------------------------------
bool jfFimThumbPercentFilter::SetPercent(size_t invalue) {
  if (invalue>100) return false;
  value = invalue;
  return true;
}
//---------------------------------------------------
size_t jfFimThumbPercentFilter::GetPercent() const {
  return value;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// gets a description
wxString jfFimThumbPercentFilter::GetTypeDescription() const {
    return wxT("Filters items depending on what percentage of the sum\
of thumbs up and thumbs down is thumbs up. The value is the minimum\
percentage accepeted.");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// copy
jfBaseFilter* jfFimThumbPercentFilter::GenCopy() const {
  jfFimThumbPercentFilter* result;
  result = new jfFimThumbPercentFilter();
  CopyOver(result);
  result->SetPercent(value);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special meta-information
wxString jfFimThumbPercentFilter::GetTypeID() const {
  return wxT("FimThumbPercentFilter");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the core matching method
bool jfFimThumbPercentFilter::CoreMatch(const jfBasePD* testelem) const {
  // constants and checks
  const jfFIM_Fanfic* rvalue;
  size_t cvalue1,cvalue2;
  float compvalue;
  assert(testelem!=NULL);
  // getting values
  rvalue = dynamic_cast<const jfFIM_Fanfic*>(testelem);
  cvalue1 = rvalue->GetThumbs(false);
  cvalue2 = rvalue->GetThumbs(true);
  compvalue = ((float)cvalue1 + (float)cvalue2);
  compvalue = (cvalue1/compvalue)*100;
  // doing the result
  return (compvalue>=value);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o
//--------------------------------------------------
bool jfFimThumbPercentFilter::AddRestToFile(wxTextFile* outfile) const {
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // composing line 4
  outfile->AddLine(SInt2Str(value));
  return true;
}
//--------------------------------------------------
bool jfFimThumbPercentFilter::ReadRestFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfFimThumbPercentFilter::ReadRestFromFile");
  // input data
  wxString cline;
  bool resx;
  // starting checks (and reading the line)
  assert(infile!=NULL);
  if (!infile->ReadLine(cline,funcname)) return false;
  // there is only one line, and one filed, so this is pretty simple
  resx = FromString(cline);
  if (!resx) return infile->BuildError(wxT("The percentage string is invalid!"));
  else return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* since different types are stored together, the text file reprentation
may have objects of varying length */
size_t jfFimThumbPercentFilter::ExtraLines() const {return 1;}
/*****************************************************************************/
