/******************************************************************************
Name    :   fimfilters1.h
Author  :   John Q Metro
Purpose :   Filters for fimfiction.net
Created :   July 4, 2012
Updated :   August 19, 2013
******************************************************************************/
#ifndef FIMFILTERS1_H_INCLUDED
#define FIMFILTERS1_H_INCLUDED
#endif // FIMFILTERS1_H_INCLUDED
//-----------------------------
#ifndef JFBASEFILTER
  #include "../base/filterbase.h"
#endif
#ifndef MOREFILTERS1_H_INCLUDED
  #include "../extended/morefilters1.h"
#endif
/*****************************************************************************/
class jfFimThumbsFilter : public jfBaseFilter {
  public:
    // constructors
    jfFimThumbsFilter();
    jfFimThumbsFilter(int inmin, int inmax);
    // setting values
    bool SetValues(int inmin, int inmax);
    // getting values
    int GetMin() const;
    int GetMax() const;
    // redefined virtual methods
    virtual bool IsEmpty() const;
    virtual wxString GetTypeID() const;
    // loading from a string representation
    virtual bool FromString(const wxString& sourcedata);
    virtual wxString ToString() const;
    // returns the list of element names campatible with this filter
    virtual wxString GetTypeDescription() const;
    virtual jfBaseFilter* GenCopy() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
    // file i/o
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    /* since different types are stored together, the text file reprentation
    may have objects of varying length */
    virtual size_t ExtraLines() const;
    // internal values
    int min, max;
};
//===========================================================================
class jfFIMGenreFilter : public jfTagFilterCore {
  public:
    // constructors
    jfFIMGenreFilter();
    jfFIMGenreFilter(const jfFIMGenreFilter& insrc);
    // getting and setting values
    virtual bool SetToEmpty();
    // redefined virtual methods
    virtual wxString GetTypeID() const;
    virtual wxString GetTypeDescription() const;
    virtual jfBaseFilter* GenCopy() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
    // check the tags against a list of pre-approved tags
    virtual bool DoVerify();
};
//===========================================================================
// getting replacements
wxArrayString* MakeFIMCharRep();
//+++++++++++++++++++++++++++++++++++++++++++++++++++++
class jfFIMCharacterFilter : public jfTagFilterCore {
  public:
    // constructors
    jfFIMCharacterFilter();
    jfFIMCharacterFilter(const jfFIMCharacterFilter& insrc);
    // getting and setting values
    virtual bool SetToEmpty();
    // redefined virtual methods
    virtual wxString GetTypeID() const;
    virtual wxString GetTypeDescription() const;
    virtual jfBaseFilter* GenCopy() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
    // check the tags against a list of pre-approved tags
    virtual bool DoVerify();
};
//================================================================
class jfFimRatingFilter : public jfBaseFilter {
  public:
    // constructors
    jfFimRatingFilter();
    jfFimRatingFilter(wxString value_in);
    // match against the filter
    virtual bool IsEmpty() const;
    // loading from a string representation
    virtual bool FromString(const wxString& sourcedata);
    virtual wxString ToString() const;
    // gets a description
    virtual wxString GetTypeDescription() const;
    // copy
    virtual jfBaseFilter* GenCopy() const;
    // special meta-information
    virtual wxString GetTypeID() const;
    // custom methods
    bool Includes(wxString test) const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
    // file i/o
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    /* since different types are stored together, the text file reprentation
    may have objects of varying length */
    virtual size_t ExtraLines() const;
    // internal data
    wxString value;
};
//================================================================
class jfFimThumbPercentFilter : public jfBaseFilter {
  public:
    // constructors
    jfFimThumbPercentFilter();
    jfFimThumbPercentFilter(wxString value_in);
    // match against the filter
    virtual bool IsEmpty() const;
    // loading from a string representation
    virtual bool FromString(const wxString& sourcedata);
    virtual wxString ToString() const;
    bool SetPercent(size_t invalue);
    size_t GetPercent() const;
    // gets a description
    virtual wxString GetTypeDescription() const;
    // copy
    virtual jfBaseFilter* GenCopy() const;
    // special meta-information
    virtual wxString GetTypeID() const;
  protected:
    // the core matching method
    virtual bool CoreMatch(const jfBasePD* testelem) const;
    // file i/o
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    /* since different types are stored together, the text file reprentation
    may have objects of varying length */
    virtual size_t ExtraLines() const;
    // internal data
    size_t value;
};

/*****************************************************************************/
