/*****************************************************************************
Name    : baseobj.cpp
Basic   : Defines the most basic data classes
Author  : John Q Metro
Started : March 17, 2009
Updated : January 3, 2015
Notes   :

******************************************************************************/
#ifndef BASEOBJ_H_INCLUDED
    #include "baseobj.h"
#endif // BASEOBJ_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef JFEXPRTOKEN
  #include "../express/expr_token.h"
#endif
#ifndef LOGGING_H_INCLUDED
  #include "../utils/logging.h"
#endif // LOGGING_H_INCLUDED
//***************************************************************************
const wxChar* IO_DELIMS=wxT(";#{}");
//****************************************************************************
jfBaseObj::jfBaseObj() {
  num_id = 0;
  name = wxT("BaseObj Default Name");
  description = wxT("");
  validdata = true;
}
//------------------------------------------------------------------
jfBaseObj::jfBaseObj(size_t src_id, const wxString& nname) {
  num_id = src_id;
  validdata = true;
  description = wxT("");
  SetName(nname);
}
//============================================================================
bool jfBaseObj::IsValid() const {
  return validdata;
}
//--------------------------------------------------------------------------
wxString jfBaseObj::GetName() const {
  return name;
}
//-------------------------------------------------------------------------
bool jfBaseObj::SetName(const wxString& newname) {
  if (newname.length()==0) {
    validdata = false;
    return false;
  }
  else {
    name = newname;
    return true;
  }
}
//------------------------------------------------------------------------
wxString jfBaseObj::GetDescription() const {
  return description;
}
//-------------------------------------------------------------------------
void jfBaseObj::SetDescription(const wxString& newdesc) {
  description = newdesc;
}
//-------------------------------------------------------------------------
size_t jfBaseObj::GetID() const {
  return num_id;
}
//-------------------------------------------------------------------------
void jfBaseObj::SetID(size_t src_id) {
  num_id = src_id;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfBaseObj::AddToFile(wxTextFile* outfile) const {
  // constants and variables
  const wxString fname = wxT("jfBaseObj::AddToFile");
  jfOutString s_out;
  bool rbuf;
  wxString qbuf;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // line 1
  s_out << GetTypeID() << num_id << validdata;
  qbuf = wxT("###") + wxString(s_out);
  outfile->AddLine(qbuf);
  s_out.Clear();
  // line 2
  s_out << name << description;
  outfile->AddLine(s_out);
  // virtual calls
  rbuf = AddMiddleToFile(outfile);
  if (!rbuf) return false;
  rbuf = AddRestToFile(outfile);
  if (!rbuf) return false;
  // adding on the footer
  outfile->AddLine(wxT("###"));
  return true;
}
//-----------------------------------------------------------------------------
bool jfBaseObj::GetFromFile(jfFileReader* infile) {
  const wxString funcname=wxT("jfBaseObj::GetFromFile");
  // input data
  wxString buffer;

  // starting checks
  assert(infile!=NULL);
  // Reading and processing line 1
  if (!infile->ReadParseSLine(3,funcname)) return false;
  buffer = infile->lp.UnEscStr(0);
  if (buffer!=GetTypeID()) {
    return infile->BuildError(wxT("The type (") + buffer + wxT(") does not match!"));
  }
  // field two is a number id
  if (!infile->lp.SIntVal(1,num_id)) {
    return infile->BuildError(wxT("The id is not an integer!"));
  }
  // field number 3 is the validdata attribute
  if (!infile->lp.BoolVal(2,validdata)) {
    return infile->BuildError(wxT("Validdata is not a boolean!"));
  }
  // we get the next line
  if (!infile->ReadParseLine(2,funcname)) return false;
  // the name
  name = infile->lp.UnEscStr(0);
  if (name.IsEmpty()) infile->BuildError(wxT("The name is blank!"));
  // description
  description = infile->lp.UnEscStr(1);
  // we do the virtual methods
  if (!ReadMiddleFromFile(infile)) return false;
  if (!ReadRestFromFile(infile)) return false;
  // we should be left just before a final line...
  if (!infile->ReadLast(funcname)) return false;
  return true;
  // done
}
//****************************************************************************
jfUrlBase::jfUrlBase():jfBaseObj() {
  xparser = NULL;
  validdata = false;
  toktitle = NULL;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfUrlBase::GetUrl() const {
  return primarylink;
}
//-------------------------------------------------------------
// builds a parse error string
wxString jfUrlBase::GetParseError() {
  if (xparser==NULL) return wxEmptyString;
  // builing the result
  wxString result = wxT("Parsing error in ") + func_string;
  result += wxT(" at position ") + SInt2Str(xparser->GetIndex());
  result += wxT(" : ") + perr_string;
  // retruning the result
  return result;
}
//---------------------------------------------------------
wxString jfUrlBase::GetPError() const {
  return perr_string;
}
//----------------------------------------------------------------
void jfUrlBase::ClearParse() {
  if (xparser!=NULL) delete xparser;
  xparser = NULL;
}
//-------------------------------------------------------------
const wxArrayString* jfUrlBase::GetTokTitle() const {
  if (toktitle==NULL) {
    toktitle = TokenizeString(name,true);
  }
  return toktitle;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal methods
//-------------------------------------------------------------
void jfUrlBase::SetParserString(const wxString& indata) {
  if (xparser==NULL) xparser = new jfStringParser(indata);
  else xparser->ChangeData(indata);
}
//-------------------------------------------------------------
void jfUrlBase::CopyFrom(jfUrlBase* src) {
  assert(src!=NULL);
  num_id = src->num_id;
  name = src->name;
  description = src->description;
  validdata = src->validdata;
  primarylink = src->primarylink;
  ClearTokParser();
}
//-----------------------------------------------------------
void jfUrlBase::CopyTo(jfUrlBase* dest) const {
  assert(dest!=NULL);
  dest->num_id = num_id;
  dest->name = name;
  dest->description = description;
  dest->validdata = validdata;
  dest->ClearTokParser();
}
//-------------------------------------------------------------
void jfUrlBase::ClearTokParser() {
  if (xparser!=NULL) {
    delete xparser;
    xparser = NULL;
  }
  if (toktitle!=NULL) {
    delete toktitle;
    toktitle = NULL;
  }
}
//---------------------------------------------------------------------
void jfUrlBase::SetValid() {
  validdata = (!name.IsEmpty()) && (!primarylink.IsEmpty());
}
//****************************************************************************
// getting the string data
wxString jfResultUnit::GetUrl() const {
  return itemurl;
}
//--------------------------------------------------------------------------
wxString jfResultUnit::GetHTML() const {
  return htmldata;
}
//--------------------------------------------------------------------------
wxString jfResultUnit::GetName() const {
  return name;
}
//----------------------------------------------
size_t jfResultUnit::GetID() const {
  return idval;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// marking data
//--------------------------------------------------------------------------
bool jfResultUnit::HasMark()const {
  return (marked!=0);
}
//--------------------------------------------------------------------------
bool jfResultUnit::Marked(const size_t& catindex) const {
  assert(catindex<32);
  size_t mask, res;
  // constructing the mask
  mask = size_t(1)<<catindex;
  // doing a bitwise and to get the result
  res = mask & marked;
  if (res==0) return false;
  else return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// specials
bool jfResultUnit::SameID(const size_t& cval) const {
  return (cval==idval);
}
//-------------------------------------------------------
void jfResultUnit::SetHTML(const wxString& invalue) {
  htmldata = invalue;
}
//*****************************************************************************
// --- jfBasePD Methods
// ==========================================================================
// the default constructor
jfBasePD::jfBasePD():jfUrlBase() {
  included = true;
  toksummary = NULL;
  rank = 0;
  discard = false;
}
//---------------------------------------------------------------------------
// the default constructor
jfBasePD::jfBasePD(const size_t& newrank):jfUrlBase() {
  included = true;
  toksummary = NULL;
  toktitle = NULL;
  rank = newrank;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// gived the order in which the item appaeared in the source
size_t jfBasePD::GetRank() const {
  return rank;
}
//---------------------------------------------------------------------------
// returns some info for displaying the item is a gui result list
jfResultUnit* jfBasePD::GetResultUnit() const {
  const wxString fname = wxT("jfBasePD::GetResultUnit");
  jfResultUnit* result;
  result = new jfResultUnit();
  result->name = name;
  result->itemurl = primarylink;
  result->idval = num_id;
  result->marked = mark.GetNum();
  result->htmldata = ToDisplayHTML();
  return result;
}
//-----------------------------------------------------------------------
/* full fimfanfic.net descriptions are sometimes so long that HTML listview boxes
are messed up. This method is intended to get a truncated version, limited in number
of lines and total characters. Moved here because other sites can have this problem */
wxString jfBasePD::GetDescExtract(size_t lines, size_t maxlen) const {
  // special cases
  if ((lines==0) || (maxlen==0)) return wxEmptyString;
  if (description.length()==0) return wxEmptyString;
  // variables
  wxString buffer, runres;
  size_t runline, runmax, linelen, templine;
  jfStringParser* dparse;
  bool truncated;
  int lendif;
  size_t rpos1, rpos2;
  // initializing
  dparse = new jfStringParser(description);
  runline = 0;
  runmax = 0;
  truncated = false;
  // the parsing loop
  while (dparse->MovePastOrEnd(wxT("<br />"),buffer)) {
    // line check
    runline++;
    if (runline>lines) {
      truncated = true;
      break;
    }
    // line length check
    linelen = buffer.length();
    templine = runmax + 4 + linelen;
    if (templine>maxlen) {
      truncated = true;
      lendif = templine - maxlen;
      lendif = linelen -lendif - 4;
      if (lendif<0) break;
      buffer.Truncate(lendif);
      linelen = lendif;
    }
    // appending the value
    if (!runres.IsEmpty()) runres += wxT("<br>");
    runres += buffer;
    runmax += linelen + 4;
    if (truncated) break;
  }
  // post processing
  delete dparse;
  // finally, we make sure we do not end inside a tag
  rpos1 = runres.rfind('<');
  if (rpos1!=wxString::npos) {
    rpos2 = runres.rfind('>');
    if ((rpos2==wxString::npos) || (rpos2<rpos1)) {
      runres.Truncate(rpos1);
      truncated = true;
    }
  }
  if (truncated) runres += wxT(" ...");
  return runres;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
const wxArrayString* jfBasePD::GetTokSummary() const {
  if (toksummary==NULL) {
    toksummary = TokenizeString(description,true);
  }
  return toksummary;
}
// ----------------------------------------------------------------------------
void jfBasePD::ClearTok() const{
  if (toktitle!=NULL) {
    delete toktitle;
    toktitle = NULL;
  }
  if (toksummary!=NULL) {
    delete toksummary;
    toksummary = NULL;
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// changing the rank
void jfBasePD::ChangeRank(const size_t& newrank) {
  rank = newrank;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfBasePD::~jfBasePD() {
  ClearTokParser();
  if (toksummary!=NULL) {
    delete toksummary;
    toksummary = NULL;
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfBasePD::CopyOver(jfBasePD* destination) const {
  assert(destination!=NULL);
  CopyTo(dynamic_cast<jfUrlBase*>(destination));
  destination->rank = rank;
  destination->included = included;
  destination->mark = mark;
}
//-----------------------------------------------------------------------------
void jfBasePD::LoadCoreValues(jfSkeletonParser* inparser) const {
  // runs some checks first
  assert(inparser!=NULL);
  // adding the values
  inparser->AddText(wxT("ITEM_NAME"),name);
  inparser->AddText(wxT("ITEM_DESCRIPTION"),description);
  inparser->AddText(wxT("ITEM_URL"),primarylink);
  inparser->AddUInt(wxT("ITEM_ID"),num_id);
  inparser->AddUInt(wxT("ITEM_RANK"),rank);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfBasePD::AddMiddleToFile(wxTextFile* outfile) const {
  wxString result;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // line 3
  result = UInt2Str(mark.GetNum()) + wxT(';');
  result += SInt2Str(rank) + wxT(';');
  result +=  EscapeNL(primarylink,IO_DELIMS);
  outfile->AddLine(result);
  return true;
}
//--------------------------------------------------------------------------
// reading from the file
bool jfBasePD::ReadMiddleFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfBasePD::ReadMiddleFromFile");
  // input data
  wxString buffer;
  wxUint32 marked;
  // reading the line
  if (!infile->ReadParseLine(3,funcname)) return false;
  if (!infile->lp.UIntVal(0,marked)) {
    return infile->BuildError(wxT("The marked field is not an integer!"));
  }
  mark.SetAll(marked);
  // field two is the rank, implemented as an integer
  if (!infile->lp.SIntVal(0,rank)) {
    return infile->BuildError(wxT("The rank is not an integer!"));
  }
  // field three is the url
  primarylink = infile->lp.UnEscStr(2);
  return true;
}
//===========================================================================




//****************************************************************************
