/////////////////////////////////////////////////////////////////////////////
// Name:        baseobj.h
// Author :     John Q Metro
// Purpose :    Declares the most basic data classes. Based on the old
//              jaffis_base.h
// Created:     March 17, 2009
// Updated:     January 3, 2015
/////////////////////////////////////////////////////////////////////////////
// For compilers that support precompilation, includes "wx.h".

//----------------------------------------------------------------------
#ifndef BASEOBJ_H_INCLUDED
    #define BASEOBJ_H_INCLUDED
#endif // BASEOBJ_H_INCLUDED

#ifndef STRINGPARSER_H_INCLUDED
  #include "../utils/stringparser.h"
#endif // STRINGPARSER_H_INCLUDED

#ifndef UTILS2_H_INCLUDED
  #include "../utils/utils2.h"
#endif // UTILS2_H_INCLUDED

#ifndef HTMLSKELTON_H_INCLUDED
  #include "../utils/htmlskelton.h"
#endif // HTMLSKELTON_H_INCLUDED
#ifndef JF_CORETYPES
  #include "coretypes.h"
#endif

#include <wx/textfile.h>
//*****************************************************************************
/* Descendants of jfBaseObj are stored in text files the following way:
Line 1: "###" <TypeID String> ";" <num_id> ";" <validdata>
Line 2: <name> ";" <description> (with ; , #, and newlines escaped)
subsequent lines
*/
//==========================================================================
extern const wxChar* IO_DELIMS;
//===========================================================================
class jfBaseObj : public wxObject {
  public:
    jfBaseObj();
    jfBaseObj(size_t src_id, const wxString& nname);
    // accessing the internals
    bool IsValid() const;
    wxString GetName() const;
    bool SetName(const wxString& newname);
    wxString GetDescription() const;
    void SetDescription(const wxString& newdesc);
    size_t GetID() const;
    void SetID(size_t src_id);
    // special meta-information
    virtual wxString GetTypeID() const=0;
    // file i/o
    virtual bool AddToFile(wxTextFile* outfile) const;
    bool GetFromFile(jfFileReader* infile);
  protected:
    // file i/o output
    virtual bool AddMiddleToFile(wxTextFile* outfile) const =0;
    virtual bool AddRestToFile(wxTextFile* outfile) const =0;
    // file i/o input
    virtual bool ReadMiddleFromFile(jfFileReader* infile) =0;
    virtual bool ReadRestFromFile(jfFileReader* infile) =0;
    // internal data
    size_t num_id;          // stores an assigned id number
    bool validdata;           // whenever a data setting method fails, is false
    wxString name,description; // stories item name and description
};
//==========================================================================
/* a class intermediate between jfBaseObj and jfBasePD, some things have urls but
are not search results */
class jfUrlBase : public jfBaseObj {
  public:
    Set32 mark;
    // the constructor
    jfUrlBase();
    // getting data methods
    wxString GetUrl() const;
    wxString GetParseError();
    wxString GetPError() const;
    void ClearParse();
    const wxArrayString* GetTokTitle() const;
  protected:
    // internal methods
    void SetParserString(const wxString& indata);
    void CopyFrom(jfUrlBase* src);
    void CopyTo(jfUrlBase*dest) const;
    void ClearTokParser();
    virtual void SetValid();
    // internal data
    wxString primarylink;
    // parsing and parse error handling
    jfStringParser* xparser;
    wxString perr_string;
    wxString func_string;

    // special stuff used in tokenized matching
    mutable wxArrayString *toktitle;
};
//============================================================================
/* The following class is basically a compact extract of jfBasePD used in
displaying results */
class jfResultUnit : public wxObject {
  friend class jfBasePD;
  public:
    // getting the string data
    wxString GetUrl() const;
    wxString GetHTML() const;
    wxString GetName() const;
    size_t GetID() const;
    // marking data
    bool HasMark()const;
    bool Marked(const size_t& catindex) const;
    // specials
    bool SameID(const size_t& cval) const;
    void SetHTML(const wxString& invalue);
  protected:
    // internal data
    size_t marked;
    wxString itemurl, name;
    wxString htmldata;
    size_t idval;
};

//============================================================================
// a very basal data class for searched items such as fics and pages
class jfBasePD : public jfUrlBase {
  public:
    // basic search data
    bool included;
    /* if the following is true while validdata is false, we should
    delete the item and go on, instead of causing a fatal error */
    bool discard;
    // the base constructor
    jfBasePD();
    jfBasePD(const size_t& newrank);
    // getting data methods
    size_t GetRank() const;
    jfResultUnit* GetResultUnit() const;
    wxString GetDescExtract(size_t lines, size_t maxlen) const;
    // to string functions
    virtual wxString ToText() const =0;
    virtual wxString ToDisplayHTML() const=0;
    // tokenized matching stuff
    const wxArrayString* GetTokSummary() const;
    void ClearTok() const;
    // added stuff
    void ChangeRank(const size_t& newrank);
    virtual bool LoadValues(jfSkeletonParser* inparser) const = 0;
    virtual void ProcessDescription() = 0;
    // the destructor
    ~jfBasePD();
  protected:
    // other stuff
    void CopyOver(jfBasePD* destination) const;
    void LoadCoreValues(jfSkeletonParser* inparser) const;
    // i/o methods
    virtual bool AddMiddleToFile(wxTextFile* outfile) const;
    virtual bool ReadMiddleFromFile(jfFileReader* infile);
    size_t rank;
    // special stuff used in tokenized matching
    mutable wxArrayString *toksummary;
};


//*****************************************************************************
