/*****************************************************************************
 Name:        misc_types1.cpp
 Author :     John Q Metro
 Purpose :    Some types
 Created:     April 2, 1010
 Updated:     December 16, 2012
 ******************************************************************************/
#ifndef JFMISCTYPES1
	#include "misc_types1.h"
#endif
#ifndef UTILS1_H_INCLUDED
#include "../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef BASEOBJ_H_INCLUDED
#include "baseobj.h"
#endif // BASEOBJ_H_INCLUDED
#ifndef UTILS2_H_INCLUDED
#include "../utils/utils2.h"
#endif // UTILS2_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../utils/logging.h"
#endif // LOGGING_H_INCLUDED
//*****************************************************************************
wxString jfTagListing::ToString() const {
	// local variables
	wxString result = wxEmptyString;
	wxString buf;
	jfTagListing::const_iterator indexer;
	// we start...
	if (!empty()) {
		// the loop
		for (indexer = begin(); indexer != end(); indexer++) {
			buf = EscapeString(indexer->first,IO_DELIMS);
			result += buf + wxT(";");
			result += SInt2Str((size_t)(indexer->second));
			result += wxT(";");
		}
		// removing the trailing ;
		result.Truncate(result.length()-1);
	}
	return result;
}
//--------------------------------------------------------------------
bool jfTagListing::FromString(const wxString& instr) {
    // constants
    const wxString fname = wxT("jfTagListing::FromString");
	// local variables
	jfLineParse* parser;
	size_t numtags, modc,ncount;
	wxString intag;
	size_t incat;
	jfTAG_STATUS intags;
	// starting
	clear();
	// a special case...
	if (instr.IsEmpty()) return true;
	// we use the parser
	parser = new jfLineParse(instr);
	modc = (parser->Num()) % 2;
	if (modc!=0) {
		delete parser;
		return false;
	}
	// things seem to be okay
	numtags = (parser->Num())/2;
	// the loop
	for (ncount = 0; ncount <numtags; ncount++ ) {
		// get and check the tag string
		intag = parser->UnEscStr(ncount*2);
		if (intag.IsEmpty()) {
			delete parser;
			return false;
		}
		// get and convert the tag category
		if (!parser->SIntVal(ncount*2+1,incat)) {
			delete parser;
			return false;
		}
		intags = (jfTAG_STATUS)incat;
		// inserting
		(*this)[intag] = intags;
	}
	delete parser;
	return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// initializing
//----------------------------------------
bool jfTagListing::FromArray(const wxChar* insrc[], size_t incount) {
  // basic check
  if (incount==0) return false;
  // variables
  size_t qloop;
  wxString ctag;
  // main loop
  for (qloop=0;qloop<incount;qloop++) {
    ctag = wxString(insrc[qloop]);
    if (ctag.IsEmpty()) return false;
    (*this)[ctag] = jfts_NONE;
  }
  return true;
}
//---------------------------------------
bool jfTagListing::FromStrArray(const wxArrayString* insrc) {
  // checks
  if (insrc==NULL) return false;
  if (insrc->IsEmpty()) return false;
  // variables
  size_t qloop,qmax;
  wxString ctag;
  // start
  qmax = insrc->Count();
  // main loop
  for (qloop=0;qloop<qmax;qloop++) {
    ctag = insrc->Item(qloop);
    if (ctag.IsEmpty()) return false;
    (*this)[ctag] = jfts_NONE;
  }
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special
bool jfTagListing::Valset() const {
  if (empty()) return false;
  // variables
  jfTagListing::const_iterator it;
  // checking
  it = begin();
  while (it!=end()) {
    if ((*it).second!=jfts_NONE) return true;
    it++;
  }
  return false;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// tag replacement
//-----------------------------------------------------
bool jfTagListing::ReplaceTag(wxString oldt, wxString newt) {
    // variables
    jfTagListing::iterator tpointer;
    jfTAG_STATUS temp;
    // special case
    if (oldt==newt) return false;
    // getting the old value
    tpointer = find(oldt);
    if (tpointer==end()) return false;
    temp = tpointer->second;
    // deleting and reinserting
    erase(tpointer);
    (*this)[newt] = temp;
    return true;
}
//--------------------------------------------------------------------
bool jfTagListing::ReplaceTags(wxArrayString* replist) {
    // bad numbers of items
    if (replist==NULL) return false;
    if (replist->Count()==0) return false;
    if ((replist->Count())&1) return false;
    // replacement loop
    size_t rcount = 0;
    size_t nindex;
    size_t nmax = (size_t)((replist->Count()) / 2);
    for (nindex = 0; nindex < nmax; nindex++) {
        if (ReplaceTag(replist->Item(2*nindex),replist->Item(2*nindex+1))) rcount++;
    }
    // done
    return (rcount==nmax);

}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special, returns true if the tags match the given list
bool jfTagListing::MatchTags(const wxChar* inlist[], size_t inlistcount) {
  // basic check
  if (inlistcount!=size()) return false;
  // variables
  wxString ctag;
  size_t lindex;
  // the main loop
  for (lindex=0;lindex<inlistcount;lindex++) {
    ctag = wxString(inlist[lindex]);
    if (find(ctag)==end()) return false;
  }
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* special, compares inputted tags to tags already in the list. Returns FALSE if
there are tags in the list not in input. Tags in input that are not in the list
are added. falsechoice controls what we do if we return false. options are
 0:ignore, 1:remove, and 2:abort */
bool jfTagListing::Reconcile(const wxChar* inlist[], size_t inlistcount, size_t falsechoice) {
  // basic check
  assert(inlistcount==size());
  assert(falsechoice<3);
  // variables
  size_t inccount = 0;
  size_t incindex;
  std::vector<bool> inc_list;
  iterator riter;
  bool resval = true;
  bool found;
  // checking the input against what is on the list
  for (incindex=0;incindex<inlistcount;incindex++) {
    if (find(wxString(inlist[incindex]))!=end()) {
      inc_list.push_back(true);
      inccount++;
    }
    else inc_list.push_back(false);
  }
  // with the results...
  if (inccount<size()) {
    if (falsechoice==2) return false;
    else resval = false;
  }
  // removing things can be pretty complicated
  if (falsechoice==1) {
    riter = begin();
    while (riter!=end()) {
      for (incindex=0;incindex<inlistcount;incindex++) {
        found = ((riter->first)==wxString(inlist[incindex]));
        if (found) break;
      }
      if (!found) erase(riter++);
      else riter++;
    }
  }
  // by contrast, adding theing is simple
  for (incindex=0;incindex<inlistcount;incindex++) {
    if (!inc_list[incindex]) (*this)[inlist[incindex]] = jfts_NONE;
  }
  // done
  return resval;
}
//*****************************************************************************


