/*****************************************************************************
Name:        skelstore.h
Author :     John Q Metro
Purpose :    holding html skeletons
Created:     November 16, 2009
Updated:     August 21, 2012
******************************************************************************/
#ifndef SKELSTORE_H_INCLUDED
#define SKELSTORE_H_INCLUDED
#endif // SKELSTORE_H_INCLUDED
#ifndef BASEOBJ_H_INCLUDED
    #include "baseobj.h"
#endif // BASEOBJ_H_INCLUDED
/*****************************************************************************/
enum jfs_Skeltype {jfs_SINGLE, jfs_MULTIPLE, jfs_FICPART};
//=============================================================================
class jfSkeletonCore : public jfBaseObj {
  public:
    // public data
    wxString header_skel;
    wxString item_skel;
    wxString item_separator;
    wxString footer_skel;
    // the constructor
    jfSkeletonCore();
    // getting data
    wxString GetGuide() const;
    virtual jfs_Skeltype GetSkelType() const = 0;
    // setting data
    void SetGuide(const wxString& inguide);
  protected:
    // internal helper methods
    void CopyOver(jfSkeletonCore& src);
    // overloaded methods
    virtual bool AddMiddleToFile(wxTextFile* outfile) const;
    virtual bool ReadMiddleFromFile(jfFileReader* infile);
    // some extra stuff
    virtual bool AddMoreToFile(wxTextFile* outfile) const = 0;
    virtual bool ReadMoreFromFile(jfFileReader* infile) = 0;
    // internal data
    wxString skelguide;
};
//============================================================================
class jfSkeletonBase : public jfSkeletonCore {
  public:
    // public data
    wxString cat_header;
    wxString cat_footer;
    // the constructor
    jfSkeletonBase();
  protected:
    // internal helper methods
    void CopyOver(jfSkeletonBase& src);
    // overloaded methods
    virtual bool AddMoreToFile(wxTextFile* outfile) const;
    virtual bool ReadMoreFromFile(jfFileReader* infile);
    // internal data
    wxString skelguide;
};
//==========================================================================
class jfSkeletonSingle : public jfSkeletonBase {
  public:
    // the copy constructor
    jfSkeletonSingle();
    jfSkeletonSingle(jfSkeletonSingle* src);
    // created methods
    virtual wxString GetTypeID() const;
    virtual jfs_Skeltype GetSkelType() const;
  protected:
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
};
//==========================================================================
class jfSkeletonMultiple : public jfSkeletonBase {
  public:
    // the copy constructor
    jfSkeletonMultiple();
    jfSkeletonMultiple(jfSkeletonMultiple* src);
    // additional skeletons
    wxString catlink;
    wxString catlink_separator;
    wxString top_footer;
    wxString cat_separator;
    // methods
    virtual wxString GetTypeID() const;
    virtual jfs_Skeltype GetSkelType() const;
  protected:
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
};
//====================================================================
/* Skeleton for handling downloading fics fron Fanfiction.Net. We have 4 extra fields for handling
groups links and part links. One of each for the link and separator. */
class jfSkeletonFicPart : public jfSkeletonCore {
  public:
    // constructors
    jfSkeletonFicPart();
    jfSkeletonFicPart(jfSkeletonFicPart* src);
    // additional skeletons
    wxString grouplink;
    wxString grouplink_separator;
    wxString partlink;
    wxString partlink_separator;
    // methods
    virtual wxString GetTypeID() const;
    virtual jfs_Skeltype GetSkelType() const;
  protected:
    // file i/o
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    // some extra stuff
    virtual bool AddMoreToFile(wxTextFile* outfile) const;
    virtual bool ReadMoreFromFile(jfFileReader* infile);
};
/*****************************************************************************/
// a class uses to store multiple classes that go together
class jfHtmlParams : public wxObject {
  public:
    wxTextFile *outfile;
    jfSkeletonParser* parse;
    jfSkeletonBase* base;
    mutable wxString errmsg;
    // the constructor
    jfHtmlParams();
    // info
    bool CheckOk() const;
    wxString MakeResult(wxString inskel);
    // data access
    jfSkeletonSingle* Single();
    jfSkeletonMultiple* Multiple();
    size_t GetResCat() const;
    bool SetResCat(size_t inval);
  private:
    size_t rescindex;
};
/*****************************************************************************/
