/******************************************************************************
// Name:        basecollect.h
// Author :     John Q Metro
// Purpose :    Declares core item collection classes
// Created:     March 19, 2009
// Updated:     August 21, 2012
latest change : changing to use jfFileReader
******************************************************************************/
#ifndef BASECOLLECT_H_INCLUDED
  #define BASECOLLECT_H_INCLUDED
#endif // BASECOLLECT_H_INCLUDED
#ifndef BASEOBJ_H_INCLUDED
    #include "../objs/baseobj.h"
#endif // BASEOBJ_H_INCLUDED
#ifndef JFBASEFILTER
  #include "../filters/base/filterbase.h"
#endif

#include <map>
//*****************************************************************************
// defines some common things for an ancestor collection class
class jfBaseCollection : public jfBaseObj {
  public:
    // default constructors
    jfBaseCollection();
    // implementable data methods
    wxString GetName() const;
    wxString GetDescription() const;
    bool SetName(const wxString& in_name);
    void SetDescription(const wxString& in_desc);
    size_t GetCount() const;
    bool IsEmpty() const;
    // general index info
    wxInt32 GetIndex() const;
    bool ItemV() const;
    // stuff for the current index
    wxString CurrName() const;
    size_t CurrID() const;
    wxString CurDesc() const;
    // index manipulation
    virtual void SaveIndex();
    virtual bool RestoreIndex();
    virtual bool NextIndex() =0;
    virtual bool BackIndex() =0;
    virtual void ResetIndex() =0;
    virtual bool ToIndex(const wxInt32& newindex) = 0;
    // some more abstract methods
    virtual wxArrayString* GetNameList() const = 0;
    virtual wxArrayString* GetDescriptionList() const = 0;
    virtual wxArrayString* GetTypeList() const = 0;
    virtual bool DeleteAtIndex(const size_t& dindex) = 0;
  protected:
    // a private Read helper method
    size_t CalcNewIndex(size_t oindex) const;
    // file I/O
    virtual bool AddMiddleToFile(wxTextFile* outfile) const;
    virtual bool AddMid2ToFile(wxTextFile* outfile) const = 0;
    virtual bool ReadMiddleFromFile(jfFileReader* infile);
    virtual bool ReadMid2FromFile(jfFileReader* infile) = 0;
    // common data
    size_t item_count;
    // indexes
    wxInt32 item_index;
    jfBaseObj* curr_item;
    wxInt32 backup_index;
    jfBaseObj* backup_item;
};
//*****************************************************************************
// how the map template will calculate it's comaparisons
struct ltstr {
  bool operator()(const wxString& s1, const wxString& s2) const {
    int ri = s1.Cmp(s2);
    return (ri<0);
  }
};
//*****************************************************************************
class stl_FilterMap : public std::map<wxString,jfBaseFilter*,ltstr> {
};
//*****************************************************************************
// the filter map uses the filter names as keys, store multiple types of filters
class jfFilterMap : public jfBaseCollection {
  public:
    // the default constructor
    jfFilterMap();
    bool SetMapPtr(jfFilterMap* nval);
    // simple filter map specific methods
    bool NameExists(const wxString& lookfor) const;
    stl_FilterMap::iterator RenameElem(bool& ok, const wxString& oldname, const wxString& newname);
    jfBaseFilter* GetItem(const wxString& itemname);
    bool MatchAll(const jfBasePD* inval) const;
    // some additonal methods
    bool AddFilter(jfBaseFilter* infilter, size_t& outindex);
    bool GetIndexByName(const wxString& findname, size_t& outindex) const;
    jfBaseFilter* GetItem(const size_t& findex);
    bool DeleteByName(const wxString& findname);
    bool ReplaceByName(wxString nameold, jfBaseFilter* newfilter, size_t& outindex);
    bool ReplaceSame(jfBaseFilter* newfilter,size_t& outindex);
    size_t CopyLoad();
    bool VerifyNames(wxString& omsg, bool notinmap);
    size_t PairFilterLogging(bool write);
    // overriding the older classes works better
    virtual void SaveIndex();
    virtual bool RestoreIndex();
    // implemented abstract methods
    virtual bool NextIndex();
    virtual bool BackIndex();
    virtual void ResetIndex();
    virtual bool ToIndex(const wxInt32& newindex);
    virtual wxString GetTypeID() const;
    // some more abstract methods
    virtual wxArrayString* GetNameList() const;
    virtual wxArrayString* GetDescriptionList() const;
    virtual wxArrayString* GetTypeList() const;
    virtual bool DeleteAtIndex(const size_t& dindex);
    // extra methods
    void DeleteAll();
  protected:
    // implemented virtual methods
    virtual bool AddMid2ToFile(wxTextFile* outfile) const;
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadMid2FromFile(jfFileReader* infile);
    virtual bool ReadRestFromFile(jfFileReader* infile);
    // internal data
    stl_FilterMap coredata;
    // indexes
    stl_FilterMap::iterator curr_iterat;
    stl_FilterMap::iterator backup_iterat;
    // external links
    jfFilterMap* exlocmap;
};
//*****************************************************************************
