/******************************************************************************
// Name:        ficmarking.cpp
// Author :     John Q Metro
// Purpose :    Defines data types for storing user flags for fics
// Created:     September 21, 2014
// Updated:     September 23, 2014

******************************************************************************/
#ifndef FICMARKING_H_INCLUDED
  #include "ficmarking.h"
#endif // FICMARKING_H_INCLUDED
//----------------------------------------------
#ifndef UTILS3_H_INCLUDED
  #include "../utils/utils3.h"
#endif // UTILS3_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../utils/logging.h"
#endif // LOGGING_H_INCLUDED
/*****************************************************************************/
bool jf_FMStore::Contains(size_t key) const {
  return (count(key)>0);
}
//=============================================================================
// constructors
//-------------------------------------
jfFicMarkStore::jfFicMarkStore():jfBaseObj() {
  mcount = 0;
}
//-------------------------------------
jfFicMarkStore::jfFicMarkStore(const wxString& in_name):jfBaseObj() {
  SetName(in_name);
  mcount = 0;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special meta-information
wxString jfFicMarkStore::GetTypeID() const {
  return wxT("FicMarkStore");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting info
//-------------------------------------
bool jfFicMarkStore::ContainsID(size_t idin) const {
  return maindata.Contains(idin);
}
//-------------------------------------
jf_FicMark jfFicMarkStore::GetValue(size_t inid) const {
  if (!maindata.Contains(inid)) return jffm_NONE;
  else return maindata[inid];
}
//-------------------------------------
bool jfFicMarkStore::IsEmpty() const {
  return (mcount==0);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting info
bool jfFicMarkStore::SetMark(size_t ididx, jf_FicMark themark) {
  // if NONE, we erase the id, if there
  if (themark==jffm_NONE) {
    if (!ContainsID(ididx)) return false;
    else {
      maindata.erase(ididx);
      mcount--;
    }
  }
  // otherwise, we add or replace, no problem
  else {
    if (!ContainsID(ididx)) mcount++;
    maindata[ididx] = themark;
  }
  // done
  return true;
}
//-------------------------------------
void jfFicMarkStore::ClearData() {
  maindata.clear();
  mcount = 0;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o output
//-------------------------------------
bool jfFicMarkStore::AddMiddleToFile(wxTextFile* outfile) const {
  // special checks and conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // adding
  outfile->AddLine(SInt2Str(mcount));
  return true;
}
//-------------------------------------
bool jfFicMarkStore::AddRestToFile(wxTextFile* outfile) const {
  // special checks and conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // variables
  jfOutString xval;
  jf_FMStore::const_iterator cptr;
  // starting
  cptr = maindata.begin();
  // the loop
  while (cptr != maindata.end()) {
    xval << cptr->first << cptr->second;
    cptr++;
  }
  // done
  outfile->AddLine(xval);
  return true;

}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o input
//-------------------------------------
bool jfFicMarkStore::ReadMiddleFromFile(jfFileReader* infile) {
  // constants, variables and checks
  const wxString fname = wxT("jfFicMarkStore::ReadMiddleFromFile");
  assert(infile!=NULL);
  unsigned long xcount;
  // reading
  if (!infile->ReadULong(xcount,fname) ) {
    return infile->BuildError(wxT("The item count is not an integer!"));
  }
  mcount = xcount;
  return true;
}
//-------------------------------------
bool jfFicMarkStore::ReadRestFromFile(jfFileReader* infile) {
  // constants, variables and checks
  const wxString fname = wxT("jfFicMarkStore::ReadRestFromFile");
  assert(infile!=NULL);
  size_t mindex, value1, value2;
  wxString berr;
  jf_FicMark cfmv;
  // special case
  if (mcount==0) {
    infile->SkipLine();
    return true;
  }
  // reading
  if (!infile->ReadParseLine(2*mcount,fname)) return false;
  // getting the value in a loop
  for (mindex = 0 ; mindex < mcount ; mindex++) {
    // reading
    if (!infile->lp.SIntVal(2*mindex,value1)) {
      return infile->BuildError(wxT("The fic id is not a positive integer!"));
    }
    if (!infile->lp.SBoundVal(2*mindex+1,5,value2,berr)) {
      return infile->BuildError(wxT("Error in getting mark value : ") +  berr);
    }
    // assigning
    cfmv = (jf_FicMark)(value2);
    maindata[value1] = cfmv;
  }
  // done
  return true;
}
//=========================================================================
// constructors
jfFicMarkGroupStore::jfFicMarkGroupStore():jfBaseObj() {
  name = wxT("The ") + GetTypeID();
  fd[0] = new jfFicMarkStore(wxT("Fanfiction.Net FicMark Store"));
  fd[1] = new jfFicMarkStore(wxT("ArchiveOfOurOwn.org FicMark Store"));
  fd[2] = new jfFicMarkStore(wxT("Fimfiction.Net FicMark Store"));
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special meta-information
wxString jfFicMarkGroupStore::GetTypeID() const {
  return wxT("FicMarkGroupStore");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructor
jfFicMarkGroupStore::~jfFicMarkGroupStore() {
  for (size_t fix = 0; fix < fms_count ; fix++ ) {
    if (fd[fix]!=NULL) {
      delete fd[fix];
      fd[fix] = NULL;
    }
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o output
//-------------------------------------
bool jfFicMarkGroupStore::AddMiddleToFile(wxTextFile* outfile) const {
  // special checks and conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // adding
  outfile->AddLine(SInt2Str(fms_count));
  return true;
}
//-------------------------------------
bool jfFicMarkGroupStore::AddRestToFile(wxTextFile* outfile) const {
  // special checks and conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // variables
  size_t tidx;
  bool xwr;
  // writing the output data
  for (tidx=0;tidx<fms_count;tidx++) {
    fd[tidx]->AddToFile(outfile);
  }
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o input
//-------------------------------------
bool jfFicMarkGroupStore::ReadMiddleFromFile(jfFileReader* infile) {
  // constants, variables and checks
  const wxString fname = wxT("jfFicMarkGroupStore::ReadMiddleFromFile");
  assert(infile!=NULL);
  unsigned long xcount;
  // reading
  if (!infile->ReadULong(xcount,fname) ) {
    return infile->BuildError(wxT("The groups count is not an integer!"));
  }
  if (xcount>fms_count) return infile->BuildError(wxT("The number of groups is too large!"));
  pstore = xcount;
  return true;
}
//-------------------------------------
bool jfFicMarkGroupStore::ReadRestFromFile(jfFileReader* infile) {
  // constants, variables and checks
  const wxString fname = wxT("jfFicMarkGroupStore::ReadRestFromFile");
  assert(infile!=NULL);
  size_t mindex;
  bool rokay;
  // reading
  for (mindex = 0 ; mindex < pstore ; mindex++ ) {
    if (!(fd[mindex]->IsEmpty())) fd[mindex]->ClearData();
    rokay = fd[mindex]->GetFromFile(infile);
    if (!rokay) return infile->BuildError(wxT("Faliure in reading group ") +  SInt2Str(mindex));
  }
  // done
  return true;
}
//============================================================================
const wxString ficm::filename = wxT("ficmark.dat");
jfFicMarkGroupStore* ficm::data = NULL;
//---------------------------------------------
bool ficm::Initialize() {
  const wxString fname = wxT("ficm::Initialize");
  jfFileReader* infile;
  bool xresult;
  wxString outerr;
  // creating and opening
  data = new jfFicMarkGroupStore();
  infile = new jfFileReader(filename);
  if (!infile->OpenFile()) {
    outerr = infile->err;
    jfXLogMessage(fname,1,outerr);
    delete infile;
    return false;
  }
  // reading
  xresult = data->GetFromFile(infile);
  if (!xresult) {
    outerr = infile->err;
    jfXLogMessage(fname,2,outerr);
  }
  infile->CloseFile();
  delete infile;
  return xresult;
}
//---------------------------------------------
void ficm::StoreToFile() {
  wxTextFile* outfile;
  outfile = GetOpenFile(filename, true);
  assert(outfile!=NULL);
  data->AddToFile(outfile);
  outfile->Write();
  outfile->Close();
  delete outfile;
}

/*****************************************************************************/
