/******************************************************************************
// Name:        ficmarking.h
// Author :     John Q Metro
// Purpose :    Declares data types for storing user flags for fics
// Created:     September 21, 2014
// Updated:     July 10, 2015

******************************************************************************/
#ifndef FICMARKING_H_INCLUDED
#define FICMARKING_H_INCLUDED
#endif // FICMARKING_H_INCLUDED
//----------------------------------------------
#ifndef BASEOBJ_H_INCLUDED
    #include "../objs/baseobj.h"
#endif // BASEOBJ_H_INCLUDED

#include <map>
/*****************************************************************************/
// basic flag
enum jf_FicMark {jffm_NONE =0, jffm_READ =1, jffm_IGNORE =2, jffm_INTEREST =3, jffm_DOWNLOADED =4};
/* white for none, pale green for read, pale red for ignore, pale blue for interest,
 pale yellow for downloaded */
const wxString fmcolors[5] = {wxT("FFFFFF"),wxT("E0FFE0"),wxT("FFE0E0"),wxT("E0E0FF"),wxT("FFFFD0")};
//-----------------------------------------------
// core ficmark map type
class jf_FMStore : public std::map<size_t,jf_FicMark> {
  public:
    // info
    bool Contains(size_t key) const;
  protected:
};
//------------------------------------------------
// ficmark storage class, with i/o an convenience functions
class jfFicMarkStore : public jfBaseObj {
  public:
    // constructors
    jfFicMarkStore();
    jfFicMarkStore(const wxString& in_name);
    // special meta-information
    virtual wxString GetTypeID() const;
    // getting info
    bool ContainsID(size_t idin) const;
    jf_FicMark GetValue(size_t inid) const;
    bool IsEmpty() const;
    // setting info
    bool SetMark(size_t ididx, jf_FicMark themark);
    void ClearData();
  protected:
    // file i/o output
    virtual bool AddMiddleToFile(wxTextFile* outfile) const;
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    // file i/o input
    virtual bool ReadMiddleFromFile(jfFileReader* infile);
    virtual bool ReadRestFromFile(jfFileReader* infile);
    // internal data
    mutable jf_FMStore maindata;
    size_t mcount;
};
//------------------------------------------------
const size_t fms_count = 3;

// type to store the fic mark stores for all the sites
class jfFicMarkGroupStore : public jfBaseObj {
  public:
    // the data, which is public to simplify access
    jfFicMarkStore* fd[fms_count];
    // constructors
    jfFicMarkGroupStore();
    // special meta-information
    virtual wxString GetTypeID() const;
    // destructor
    ~jfFicMarkGroupStore();
  protected:
    // file i/o output
    virtual bool AddMiddleToFile(wxTextFile* outfile) const;
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    // file i/o input
    virtual bool ReadMiddleFromFile(jfFileReader* infile);
    virtual bool ReadRestFromFile(jfFileReader* infile);
    // internal data
    size_t pstore;
};
//=======================================================
namespace ficm {
  extern const wxString filename;
  extern jfFicMarkGroupStore* data;
  bool Initialize();
  void StoreToFile();
}
/*****************************************************************************/
