/*****************************************************************************
Name:        itemcollect.cpp
Author :     John Q Metro
Purpose :    a vector collection base for jfBasePD that is used for storing
              and processitng items.
Created:     April 17, 2009
Updated:     October 24, 2013
******************************************************************************/
#ifndef ITEMCOLLECT_H_INCLUDED
  #include "itemcollect.h"
#endif // ITEMCOLLECT_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef UTILS2_H_INCLUDED
  #include "../utils/utils2.h"
#endif // UTILS2_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../utils/logging.h"
#endif // LOGGING_H_INCLUDED
/*****************************************************************************/

//=======================================================================
// constructor to initialize values
jfUrlItemCollection::jfUrlItemCollection():jfBaseCollection() {
  // constants
  const wxString fname = wxT("jfUrlItemCollection::jfUrlItemCollection");
  // initializing counts
  /**/jfXLogMessage(fname,1);
  pageindex = 0;
  newpage_items = 0;
  for (int q=0;q<3;q++) {
    pagecount[q]=0;
    grand_total[q]=0;
  }
  /**/jfXLogMessage(fname,2);
  // intermediate data
  page_results = NULL;
  pr_size = 0;
  pr_index = 0;
  /**/jfXLogMessage(fname,3);
  // other
  sdata = NULL;
  idstore = NULL;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting id filtering
bool jfUrlItemCollection::SetIDFiltering(jfIDStore* instore) {
  idstore = instore;
  return (idstore!=NULL);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting some basic data
size_t jfUrlItemCollection::PageIndex() const {
  return pageindex;
}
//-----------------------------------------------------------------------
size_t jfUrlItemCollection::PageTotal() const {
  return pagecount[0];
}
//-----------------------------------------------------------------------
bool jfUrlItemCollection::MorePages() const {
  return (pageindex<pagecount[0]);
}
//-----------------------------------------------------------------------
size_t jfUrlItemCollection::GetGrandTotal() const {
  return grand_total[0];
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfUrlItemCollection::WriteToHTML(jfHtmlParams* indata, bool single) const {
  // local variables
  wxString result,intermed;
  wxString buffer;
  bool lvres,thlres;
  // checks
  assert(indata!=NULL);
  assert(indata->CheckOk());
  // we now load the parser with values
  lvres = LoadValues(indata->parse,indata->GetResCat());
  assert(lvres);
  // doing the parsing
  buffer = (single)?(indata->Single()->cat_header):(indata->Multiple()->cat_header);
  result = indata->MakeResult(buffer);
  // writing
  indata->outfile->AddLine(result);
  // producing the results
  thlres = ToHTMLList(indata,single);
  assert(thlres);
  // doing the parsing
  buffer = (single)?(indata->Single()->cat_footer):(indata->Multiple()->cat_footer);
  result = indata->MakeResult(buffer);
  // writing
  indata->outfile->AddLine(result);
  // done
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// calls DispatchCategory in the params
void jfUrlItemCollection::RegisterCategory() {
  const wxString fname = wxT("jfUrlItemCollection::RegisterCategory");
  /**/jfXAssertLog(sdata!=NULL,fname,wxT("sdata is NULL!"));
  /**/jfXLogMessage(fname,1);
  sdata->DispatchCategory();
  /**/jfXLogMessage(fname,2);
}
//-----------------------------------------------------------------------
// gets the results from page_results and sends them
void jfUrlItemCollection::DispatchPageResults() {
  const wxString fname = wxT("jfUrlItemCollection::DispatchPageResults");
  // some checks
  assert(sdata!=NULL);
  assert(sdata->Check());
  assert(page_results!=NULL);
  // local variables
  jfResultUnitVector* result;
  jfResultUnit* tresult;
  // we start
  if (pr_size>0) {
    result = new jfResultUnitVector();
    for (pr_index=0;pr_index<pr_size;pr_index++) {
      tresult = (*page_results)[pr_index]->GetResultUnit();
      result->push_back(tresult);
    }
    sdata->DispatchResults(newpage_items,result);
  }
  else {
    sdata->DispatchCount(newpage_items);
  }
}
//--------------------------------------------------------------
// saves old grand total and page count
void jfUrlItemCollection::SaveOld() {
  pagecount[2] = pagecount[1];
  pagecount[1] = pagecount[0];
  grand_total[2] = grand_total[1];
  grand_total[1] = grand_total[0];
}
//-----------------------------------------------------------
void jfUrlItemCollection::RestoreOld() {
  pagecount[0] = pagecount[1];
  pagecount[1] = pagecount[2];
  grand_total[0] = grand_total[1];
  grand_total[1] = grand_total[2];
}
//------------------------------------------------------------
/* while processing a page, but before adding things to the main list, we may
realize it is incomplete or defective. This method reverts some values for a
retry */
void jfUrlItemCollection::UndoPage() {
  pageindex--;
  if (page_results!=NULL) {
    delete page_results;
    page_results = NULL;
  }
  pr_size = 0;
  pr_index = 0;
  RestoreOld();
}
//----------------------------------------------------------------------
bool jfUrlItemCollection::MakeError(wxString inerr, wxString infunc, wxString inblock) {
  parse_error = wxT("PARSING ERROR : ") + inerr + wxT(" in ") + infunc;
  if (!inblock.IsEmpty()) {
    parse_error += wxT("\nSOURCE EXTRACT :\n");
    parse_error += inblock;
  }
  /**/jfXLogString(parse_error);
  return false;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// filtering and removing from the current page
//-----------------------------------------------------------------------
/* we go over the page results and mark those not matching the filter
list for removal. */
size_t jfUrlItemCollection::FilterByList() {
  // constants
  const wxString fname = wxT("jfUrlItemCollection::FilterByList");
  // variables and checks
  size_t fsize,rcount;
  assert(page_results!=NULL);
  assert(sdata->Check());
  jfBasePD* tvector;
	wxArrayString* scheck;
  // the constructor of the items should have them be included by default
  // we use a special cases first
  if ((sdata->def_filtermap)==NULL) return pr_size;
  fsize = (sdata->def_filtermap)->GetCount();
  if (fsize==0) return pr_size;
  rcount = 0;
	scheck = sdata->def_filtermap->GetNameList();
	delete scheck;
  // we assume pr_size is correct
  for (pr_index = 0;pr_index<pr_size;pr_index++) {
    // and then we test against the filter list
    tvector = (*page_results)[pr_index];
    tvector->included = (sdata->def_filtermap)->MatchAll(tvector);
    if (tvector->included) rcount++;
  }
  return rcount;
}
//-----------------------------------------------------------------------
/* does the catgeory sort. some items may be marked as not included. I've
 also added ID filtering to this */
size_t jfUrlItemCollection::CategorySort() {
  jfBasePD* tvector;
  size_t rcount;
  assert(sdata->Check());
  bool idfilon;
  // the main checksing loop
  rcount = pr_size;
  idfilon = (idstore!=NULL);
  for (pr_index = 0;pr_index<pr_size;pr_index++) {
    // and then we test against the filter list
    tvector = (*page_results)[pr_index];
    (sdata->categories)->Sort(tvector);
    if (sdata->NoCatTest(tvector)) tvector->included = false;
    if ((idfilon) && (tvector->included)) {
      tvector->included = !(idstore->DiscardTest(tvector));
    }
    if (!(tvector->included)) rcount--;
  }
  return rcount;

}
//-----------------------------------------------------------------------
/* replaces page_results with a new version in which all objects that do
not have included as TRUE are removed (and deleted) */
size_t jfUrlItemCollection::Condense() {
  // the result list
  jfPDVector* newlist;
  jfBasePD* tvector;
  // we start
  newlist = new jfPDVector();
  // the loop
  for (pr_index = 0;pr_index<pr_size;pr_index++) {
    // and then we check
    tvector = (*page_results)[pr_index];
    if (tvector->included) newlist->push_back(tvector);
    else {
      delete tvector;
      (*page_results)[pr_index] = NULL;
    }
  }
  // at the end of the loop, we get rid of the old
  delete page_results;
  // and usher in the new
  page_results = newlist;
  pr_size = newlist->size();
  return pr_size;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual i/o methods
//----------------------------------------------------------------------------
/* What we write here is page index, page count, and grand total. I've decided
*not* to include search core, old index, or intermediate info here */
bool jfUrlItemCollection::AddMid2ToFile(wxTextFile* outfile) const {
  // local varibales
  jfOutString buffer;
  // the usual checks
  if (outfile==NULL) return false;
  if (!outfile->IsOpened()) return false;
  // starting
  buffer << pageindex << pagecount[0] << grand_total[0];
  // doing
  outfile->AddLine(wxString(buffer));
  return true;
}
//-----------------------------------------------------------------------------
bool jfUrlItemCollection::ReadMid2FromFile(jfFileReader* infile) {
  const wxString fname=wxT("jfUrlItemCollection::ReadMid2FromFile");
  const wxString inai=wxT(" value is not an integer!");
  // local variables
  wxString cline;
  // starting...
  assert(infile!=NULL);
  if (!infile->ReadParseLine(3,fname)) return false;
  // now that we have the parser line, we get
  if (!infile->lp.SIntVal(0,pageindex)) return infile->BuildError(wxT("The page index")+inai);
  if (!infile->lp.SIntVal(1,pagecount[0])) return infile->BuildError(wxT("The pagecount")+inai);
  if (!infile->lp.SIntVal(2,grand_total[0])) return infile->BuildError(wxT("The grand item total")+inai);
  /* note that the older values for pagecount and grand_total should be 0 if we are loading
  to a newly-constructed object, but just in case the object is used, we'll re-init */
  pagecount[1]=0;
  pagecount[2]=0;
  grand_total[1]=0;
  grand_total[2]=0;
  // some final stuff
  if (pageindex>pagecount[0]) return infile->BuildError(wxT("The pageindex value is more than pagecount!"));
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading values
void jfUrlItemCollection::LoadCoreValues(jfSkeletonParser* inparser,size_t which) const {
  // a basic checks
  assert(inparser!=NULL);
  // adding items
  inparser->AddText(wxT("CAT_NAME"),name);
  inparser->AddText(wxT("CAT_DESC"),name);
  inparser->AddUInt(wxT("CAT_ID"),num_id);
  inparser->AddUInt(wxT("ITEM_COUNT"),item_count);
  inparser->AddUInt(wxT("ITEM_RCOUNT"),CountRCategory(which));
  inparser->AddUInt(wxT("CAT_TOTALCOUNT"),grand_total[0]);
  inparser->AddUInt(wxT("CAT_PAGECOUNT"),pagecount[0]);
}
/*****************************************************************************/
