/*****************************************************************************
Name:        itemcollect.h
Author :     John Q Metro
Purpose :    a vector collection base for jfBasePD that is used for storing
              and processitng items.
Created:     April 16, 2009
Updated:     July 1, 2015
******************************************************************************/
#ifndef ITEMCOLLECT_H_INCLUDED
#define ITEMCOLLECT_H_INCLUDED
#endif // ITEMCOLLECT_H_INCLUDED
//-----------------------------------------------------------------------------
#ifndef BASECOLLECT_H_INCLUDED
  #include "basecollect.h"
#endif // BASECOLLECT_H_INCLUDED
#ifndef BASEOBJ_H_INCLUDED
    #include "../objs/baseobj.h"
#endif // BASEOBJ_H_INCLUDED
#ifndef SEARCHCORE_H_INCLUDED
    #include "../objs/searchcore.h"
#endif // SEARCHCORE_H_INCLUDED
#ifndef VECTORS_H_INCLUDED
  #include "vectors.h"
#endif // VECTORS_H_INCLUDED
#ifndef SKELSTORE_H_INCLUDED
  #include "../objs/skelstore.h"
#endif // SKELSTORE_H_INCLUDED
//-------------------------------------------------------------------------------

//=============================================================================
/*
The general process when fecthing and filtering is thus:
 1: Create the collections, passing to it category and params
 2: Get the first URL and use it to fecth the first page (external)
 3: Pass this page to the collection
 4: Dispatch the new category event
 5: Parse the header of this page, getting info on how many pages left there are.
 6: Parse the rest into a new vector of objects (the vector is jfBasePD)
 7: Filter this vector (using def_filtermap)
 8: Get rid of any items that do not pass
 9: Use categories to sort the results into destination categories
10: Get rid of items that do not pass
11: Get a result unit vector form the vector
12: dispatch an event with this vector
13: concatenate the items from the vector onto the main one, getting rid of the temp vector
14: Get the next url (external)
15: Goto 3

Some of this stuff does not need special info on the exact item type, so
we implement a base class here.

*/
class jfUrlItemCollection : public jfBaseCollection {
  public:
    wxString parse_error;
    // constructor to initialize values
    jfUrlItemCollection();
    // adding info
    virtual bool AddFirstPage(const wxString& inpage, size_t& outcount) = 0;
    virtual bool AddPage(const wxString& inpage, size_t& outcount) = 0;
    bool SetIDFiltering(jfIDStore* instore);
    // getting some basic data
    size_t PageIndex() const;
    size_t PageTotal() const;
    bool MorePages() const;
    size_t GetGrandTotal() const;
    virtual size_t CountRCategory(size_t which) const = 0;
    virtual wxString PercRCategory(size_t which) const = 0;
    // extracting info at the specified index
    virtual wxString NameAtIndex(size_t i_index) const = 0;
    virtual wxString DescriptionAtIndex(size_t i_index) const = 0;
    virtual wxString UrlAtIndex(size_t i_index) const = 0;
    virtual size_t IdAtIndex(size_t i_index) const = 0;
    // setting info at the specified index
    virtual bool SetDiscardAtIndex(size_t i_index, bool setval) = 0;
    // i/o
    virtual bool LoadValues(jfSkeletonParser* inparser,size_t which) const = 0;
    bool WriteToHTML(jfHtmlParams* indata, bool single) const;
  protected:
    // private methods
      void RegisterCategory(); // calls DispatchCategory in the params
      void DispatchPageResults(); // gets the results from page_results and sends them
      void SaveOld();
      void RestoreOld();
      void UndoPage();
      bool MakeError(wxString inerr, wxString infunc, wxString inblock=wxEmptyString);
      // filtering and removing from the current page
      size_t FilterByList();
      size_t CategorySort();
      size_t Condense();
    // implemented virtual i/o methods
      virtual bool AddMid2ToFile(wxTextFile* outfile) const;
      virtual bool ReadMid2FromFile(jfFileReader* infile);
      // loading values
      void LoadCoreValues(jfSkeletonParser* inparser,size_t which) const;
    // some more virtual i/o methods
      virtual bool AddDelta(wxTextFile* outfile) const = 0;
      virtual bool ReadDelta(jfFileReader* infile) = 0;
      // more i/o
      virtual bool ToHTMLList(jfHtmlParams* indata, bool single) const = 0;
    // private data
      // page count info
      size_t pageindex;
      size_t newpage_items;
      /* page and item totals. we have 3 copies because we keep the old and the one
      before that. 0: current, 1: old, 2: older */
      size_t pagecount[3];
      size_t grand_total[3];
      // vector holding intermediate result data
      jfPDVector* page_results;
      size_t pr_size, pr_index;
      // where we store the param and category
      jfSearchCore* sdata;
      // extra
      jfIDStore* idstore;

};
/******************************************************************************/

