/*****************************************************************************
Name:        resultcollect.cpp
Author :     John Q Metro
Purpose :    A structure for holding one or more item collections
Created:     May 27, 2009
Updated:     July 1, 2015
******************************************************************************/
#ifndef RESULTCOLLECT_H_INCLUDED
  #include "resultcollect.h"
#endif // RESULTCOLLECT_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../utils/logging.h"
#endif // LOGGING_H_INCLUDED
//*****************************************************************************
// the defult constructor... just helps with the nulls
jfResultCollection::jfResultCollection(jfSearchCore* insearch, bool idfilter):jfBaseObj() {
  // initializations
  assert(insearch!=NULL);
  collections = NULL;
  cprotect = NULL;
  itemcount = 0;
  search = insearch;
  cindex = 0;
  if (idfilter) idstore = new jfIDStore();
  else idstore = NULL;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// public methods
jfSearchCore* jfResultCollection::GetSearch() const {
  return search;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// wrapper to access the collection at the current index
//-----------------------------------------------------------------------
bool jfResultCollection::MorePages() const {
  if (itemcount==0) return false;
  assert(cindex<itemcount);
  return (*collections)[cindex]->MorePages();
}
//----------------------------------------------------------------------
bool jfResultCollection::NotLastPage() const {
  // checks
  if (itemcount==0) return false;
  assert(cindex<itemcount);
  // pagindex & limits
  size_t pageindexl = (*collections)[cindex]->PageIndex();
  if (pageindexl>=(search->GetPagelimit())) return false;
  return MorePages();
}
//-----------------------------------------------------------------------
size_t jfResultCollection::PageTotal() const {
  if (itemcount==0) return 0;
  assert(cindex<itemcount);
  return (*collections)[cindex]->PageTotal();
}
//-----------------------------------------------------------------------
size_t jfResultCollection::ItemCount() const {
  if (itemcount==0) return 0;
  assert(cindex<itemcount);
  return (*collections)[cindex]->GetCount();
}
//-----------------------------------------------------------------------
size_t jfResultCollection::MaxPages() const {
  // local varibles
  size_t pt,pm;
  // checks
  if (itemcount==0) return 0;
  assert(cindex<itemcount);
  // getting the values
  pt = (*collections)[cindex]->PageTotal();
  pm = search->GetPagelimit();
  // returning the result
  return (pm<=pt)?pm:pt;
}
//----------------------------------------------------------------------
jfUrlItemCollection* jfResultCollection::GetCurrPointer() {
  // checks
  if (itemcount==0) return NULL;
  assert(cindex<itemcount);
  // getting the values
  return (*collections)[cindex];
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// extracting information from a particular item
wxString jfResultCollection::GetItemDescription(size_t c_index, size_t i_index) const {
  // checking the collections...
  assert(c_index<itemcount);
  (*cprotect)[c_index]->Lock();
  // getting the string
  wxString xres = (*collections)[c_index]->DescriptionAtIndex(i_index);
  (*cprotect)[c_index]->Unlock();
  // done
  return xres;
}
//--------------------------------------------------------------------------------
// extracting information from a particular item
wxString jfResultCollection::GetItemUrl(size_t c_index, size_t i_index) const {
  // checking the collections...
  assert(c_index<itemcount);
  (*cprotect)[c_index]->Lock();
  // getting the string
  wxString xres = (*collections)[c_index]->UrlAtIndex(i_index);
  (*cprotect)[c_index]->Unlock();
  // done
  return xres;
}
//-----------------------------------------------------------------------------
size_t jfResultCollection::GetItemID(size_t c_index, size_t i_index) const {
  // checking the collections...
  assert(c_index<itemcount);
  (*cprotect)[c_index]->Lock();
  // getting the string
  size_t xres = (*collections)[c_index]->IdAtIndex(i_index);
  (*cprotect)[c_index]->Unlock();
  // done
  return xres;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting info
bool jfResultCollection::SetDiscardValue(size_t c_index, size_t i_index, bool setval) {
  // checking the collections...
  bool rval;
  if (c_index>=itemcount) return false;
  (*cprotect)[c_index]->Lock();
  // getting the string
  rval = (*collections)[c_index]->SetDiscardAtIndex(i_index,setval);
  (*cprotect)[c_index]->Unlock();
  // done
  return rval;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal helper methods
//-----------------------------------------------------------------------------
// checking an index
bool jfResultCollection::CheckIndex(size_t index) const {
  if (itemcount==0) return false;
  if (index>=itemcount) return false;
  if (collections==NULL) return false;
  if (cprotect==NULL) return false;
  if (index>=(collections->size())) return false;
  if (index>=(cprotect->size())) return false;
  if ((*collections)[index]==NULL) return false;
  return true;
}
//-------------------------------------------------------------------------------
bool jfResultCollection::DestroyAll() {
  size_t loopc;
  if (itemcount>0) {
    for (loopc=0;loopc<itemcount;loopc++) {
      delete (*collections)[loopc];
      delete (*cprotect)[loopc];
      (*collections)[loopc] = NULL;
      (*cprotect)[loopc] = NULL;
    }
    delete collections;
    delete cprotect;
    collections = NULL;
    cprotect = NULL;
    itemcount = 0;
    cindex = 0;
  }
  if (idstore!=NULL) {
    delete idstore;
    idstore = NULL;
  }
  return true;
}
//-------------------------------------------------------------------------------
bool jfResultCollection::AddItem(jfUrlItemCollection* newitem) {
  const wxString fname = wxT("jfResultCollection::AddItem");
  assert (newitem!=NULL);
  // if we are empty, we add the vectors
  /**/jfXLogMessageS(fname,1,itemcount);
  if (itemcount==0) {
    collections = new jfItemVector();
    cprotect = new jfMutexVector();
    /**/jfXLogMessage(fname,2);
  }
  // adding
  itemcount++;
  /**/jfXLogMessageS(fname,3,itemcount);
  collections->push_back(newitem);
  /**/jfXLogMessage(fname,4,newitem->GetName());
  cprotect->push_back(new wxMutex);
  /**/jfXLogMessage(fname,5);
  // done
  return true;
}
//------------------------------------------------------------------------------
// inserts an item at the current pointer
bool jfResultCollection::InsertItem(jfUrlItemCollection* newitem) {
  // checks for special conditions
  if (newitem==NULL) return false;
  // variables
  jfItemVector::iterator main_insert;
  jfMutexVector::iterator lock_insert;
  /* special case 1 and 2, the collection is empty or the index is path the
   end. We call Add Item.  */
  if ((itemcount==0) || (cindex==itemcount)) {
    bool aires = AddItem(newitem);
    assert(aires);
  }
  // general case, insertion
  else {
    // making the insertion point
    main_insert = (collections->begin())+cindex;
    lock_insert = (cprotect->begin())+cindex;
    // inserting
    collections->insert(main_insert,newitem);
    cprotect->insert(lock_insert,new wxMutex);
    itemcount++;
  }
  // we update the index to point the the item after
  cindex++;
  // done
  return true;
}
//------------------------------------------------------------------------------
bool jfResultCollection::DeleteAtIndex(size_t dindex) {
  // checks
  if (dindex>=itemcount) return false;
  // variables
  jfItemVector::iterator c_index;
  jfMutexVector::iterator m_index;
  bool dlast;
  // starting...
  dlast = (dindex==(itemcount-1));
  c_index = (collections->begin()) + dindex;
  m_index = (cprotect->begin()) + dindex;
  collections->erase(c_index);
  cprotect->erase(m_index);
  itemcount--;
  if (dlast) cindex--;
  else if (cindex>dindex) cindex--;
  // done
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal file i/o methods
// creates/reopens a file using a name for a specific result category
jfHtmlParams* jfResultCollection::OpenHTMLOutfile(size_t index) {
  // local variables
  jfHtmlParams* result = NULL;
  wxString path;
  // some basic checks
  assert(index<33);
  assert(search!=NULL);
  // category checks
  assert((search->categories)!=NULL);
  if (index==32) assert((search->categories)->UsesDefault());
  else assert(index<((search->categories)->GetCount()));
  // now that the checks are over, we get the path an location for the output file
  path = (search->categories)->GetFullPath(index);
  // creating the output object
  result = new jfHtmlParams();
  // creating the file
  result->outfile = new wxTextFile(path);
  // opening and clearing
  if (result->outfile->Exists()) {
    result->outfile->Open();
    result->outfile->Clear();
  }
  else {
    result->outfile->Create();
    result->outfile->Open();
  }
  // filling in some additional data
  result->SetResCat(index);
  result->parse = new jfSkeletonParser();
  // done
  return result;
}
//----------------------------------------------------------------------------
void jfResultCollection::WriteFooterClose(jfHtmlParams* indata) {
  // local variables
  wxString buffer;
  wxDateTime xnow;
  // some checks
  assert(indata!=NULL);
  assert(indata->CheckOk());
  // loading some time and date data
  xnow = wxDateTime::Now();
  indata->parse->AddText(wxT("TIME"),xnow.FormatTime());
  indata->parse->AddText(wxT("DATE"),xnow.FormatDate());
  // next up, we produce the output from the skeleton
  buffer = indata->MakeResult(indata->base->footer_skel);
  indata->outfile->AddLine(buffer);
  // done, we write and close
  indata->outfile->Write();
  indata->outfile->Close();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfResultCollection::AddMiddleToFile(wxTextFile* outfile) const {
    // local variables
  wxString result;
  size_t windex;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // we convert the count first
  result = SInt2Str(itemcount);
  outfile->AddLine(result);
  // writing the rest
  for (windex=0;windex<itemcount;windex++) {
    (*collections)[windex]->AddToFile(outfile);
  }
  // done
  return true;
}
//-------------------------------------------------------------------------
bool jfResultCollection::ReadMiddleFromFile(jfFileReader* infile) {
  // constants
  const wxString fname = wxT("jfResultCollection::ReadMiddleFromFile");
  // local variables
  jfUrlItemCollection* tcoll;
  unsigned long oval;
  size_t iloop;
  // starting
  /**/jfXLogMessage(fname,1);
  assert(infile!=NULL);
  // we read in the item count
  if (!infile->ReadULong(oval,fname)) return false;
  DestroyAll();
  itemcount = 0;
  /**/jfXLogMessage(fname,2);
  // the collection reading loop
  for (iloop=0;iloop<oval;iloop++) {
    /**/jfXLogMessageS(fname,3,iloop);
    tcoll = MakeEmptyCollection();
    if (!tcoll->GetFromFile(infile)) {
      /**/jfXLogMessage(fname,4,infile->err);
      delete tcoll;
      return false;
    }
    /**/jfXLogMessageS(fname,5,iloop);
    AddItem(tcoll);
  }
  /**/jfXLogMessage(fname,6);
  // done
  return true;
}
//*****************************************************************************
