/*****************************************************************************
Name:        resultcollect.h
Author :     John Q Metro
Purpose :    A structure for holding one or more item collections
Created:     May 27, 2009
Updated:     July 1, 2015 (SetDiscardValue)
******************************************************************************/

#ifndef RESULTCOLLECT_H_INCLUDED
#define RESULTCOLLECT_H_INCLUDED
#endif // RESULTCOLLECT_H_INCLUDED
#ifndef ITEMCOLLECT_H_INCLUDED
  #include "itemcollect.h"
#endif // ITEMCOLLECT_H_INCLUDED
#ifndef SEARCHCORE_H_INCLUDED
  #include "../structs/searchcore.h"
#endif // SEARCHCORE_H_INCLUDED
#include <vector>
//*******************************************************************************
// some vector classes
class jfItemVector : public std::vector<jfUrlItemCollection*> {};
class jfMutexVector : public std::vector<wxMutex*> {};
//-------------------------------------------------------------------------------

class jfResultCollection : public jfBaseObj {
  public:
    // core constructor
    jfResultCollection(jfSearchCore* insearch, bool idfilter);
    // public methods
    jfSearchCore* GetSearch() const;
    // virtual bool CreateItemCollection(const wxString& src_in, wxString& outerr) = 0;
    virtual bool AddPage(const wxString& src_in, wxString& outerr) = 0;
    virtual bool WriteToHTML(size_t result_category) = 0;
    virtual wxString GetTypeID() const=0;
    // wrapper to access the collection at the current index
    bool MorePages() const;
    bool NotLastPage() const;
    size_t PageTotal() const;
    size_t ItemCount() const;
    size_t MaxPages() const;
    jfUrlItemCollection* GetCurrPointer();
    // extracting information from a particular item
    wxString GetItemDescription(size_t c_index, size_t i_index) const;
    wxString GetItemUrl(size_t c_index, size_t i_index) const;
    size_t GetItemID(size_t c_index, size_t i_index) const;
    // setting info
    bool SetDiscardValue(size_t c_index, size_t i_index, bool setval);
  protected:
    // internal helper methods
    bool CheckIndex(size_t index) const;
    bool DestroyAll();
    bool AddItem(jfUrlItemCollection* newitem);
    bool InsertItem(jfUrlItemCollection* newitem);
    bool DeleteAtIndex(size_t dindex);
    // internal file i/o methods
    jfHtmlParams* OpenHTMLOutfile(size_t index);
    void WriteFooterClose(jfHtmlParams* indata);
    virtual void WriteHtmlHeader(jfHtmlParams* indata) = 0;
    // regular file i/o methods
    // file i/o output
    virtual bool AddMiddleToFile(wxTextFile* outfile) const;
    virtual bool AddRestToFile(wxTextFile* outfile) const =0;
    // file i/o input
    virtual jfUrlItemCollection* MakeEmptyCollection() const = 0;
    virtual bool ReadMiddleFromFile(jfFileReader* infile);
    virtual bool ReadRestFromFile(jfFileReader* infile) =0;
    // internal data
    jfItemVector* collections;
    jfMutexVector* cprotect;
    size_t itemcount;
    jfSearchCore* search;
    size_t cindex;
    jfIDStore* idstore;
};




//*******************************************************************************
