/*****************************************************************************
Name    : altlog.cpp
Basic   : Alternate logging (using wxFile)
Author  : John Q Metro
Started : March 3, 2013
Updated : March 3, 2013
Notes   :

******************************************************************************/
#ifndef ALTLOG_H_INCLUDED
  #include "altlog.h"
#endif // ALTLOG_H_INCLUDED
//--------------------------------------------------
#include <wx/datetime.h>
//**********************************************************************************
/* logging class, created because I cannot get wxLogMessage to work on FreeBSD */
//=====================================================================
// constructors
//------------------------------------
jfAltLogger::jfAltLogger() {
  outfile = NULL;
}
//------------------------------------
// constructor which also opens file
jfAltLogger::jfAltLogger(wxString fnamein) {
  outfile = NULL;
  MakeLog(fnamein);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file manipulations
//------------------------------------
// creates the logile and opens it. any previous version gets overwrittem
bool jfAltLogger::MakeLog(wxString fnamein) {
  // special checks and handling
  if (fnamein.IsEmpty()) return false;
  if (outfile!=NULL) CloseLog();
  // opening the file
  outfile = new wxFile();
  if (!outfile->Open(fnamein,wxFile::write)) {
    delete outfile;
    outfile = NULL;
    return false;
  }
  // final stuff
  logname = fnamein;
  return true;
}
//------------------------------------
// closes the lofile, if open
bool jfAltLogger::CloseLog() {
  if (outfile==NULL) return false;
  bool rval = outfile->Flush();
  if (!rval) return false;
  outfile->Close();
  delete outfile;
  outfile = NULL;
  return true;
}
//------------------------------------
bool jfAltLogger::IsOpen() const {
  return (outfile!=NULL);
}
//------------------------------------
wxString jfAltLogger::GetLogname() const {
  return logname;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// logging
bool jfAltLogger::LogString(wxString in_msg) {
  // variables
  wxString composed;
  wxDateTime unow;
  bool tval;
  // check if open
  if (!IsOpen()) return false;
  // date stamp and making the message
  unow = wxDateTime::Now();
  composed = unow.Format() + wxT(" ");
  composed += in_msg + wxT("\n");
  // writing
  tval = outfile->Write(composed);
  if (!tval) return false;
  tval = outfile->Flush();
  if (!tval) return false;
  // donme
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructor
jfAltLogger::~jfAltLogger() {
  CloseLog();
}
//=================================================================================
// initializing altlog variables
jfAltLogger* altlog::logvar = NULL;
wxMutex altlog::loglock;
bool altlog::doaltlog = false;
//**********************************************************************************
