/////////////////////////////////////////////////////////////////////////////
// Name:        charconv.cpp
// Author :     John Q Metro
// Purpose :    Character conversion stuff
// Created:     24.10.05
// Updated:     November 24, 2013
// (originally from jaffis_pagegetter.cpp)
/////////////////////////////////////////////////////////////////////////////

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
// For compilers that support precompilation, includes "wx/wx.h".
#ifndef CHARCONV_H_INCLUDED
  #include "charconv.h"
#endif // CHARCONV_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "utils1.h"
#endif // UTILS1_H_INCLUDED
#include "wx/strconv.h"
#include <string.h>
#include <ctype.h>
#ifndef LOGGING_H_INCLUDED
  #include "logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef UTILS3_H_INCLUDED
  #include "utils3.h"
#endif // UTILS3_H_INCLUDED
//***************************************************************************
// a function that scans the returned data for the charset info
// the assumed maximum size of the rawdata is size+1
// assumed to be html or xhtml, non-multiple bytes per ascii char
wxString ExtractCharset(const wxByte* rawdata, size_t size) {
  // misc stuff
  const wxString dtype = wxT("!doctype html");
  // bool compval;
  // a local copy of temporary data for case insensetive checking
  char *lowerdata;
  size_t buflength;
  // raw data pointers
  const wxByte* endptr = rawdata + size;
  const wxByte* index;
  wxByte* bufferptr;  wxByte* start; const wxByte* thelast;
  size_t sizeleft;
  size_t sizeastart;
  // found data pointers
  // char *ecsetptr;
  wxInt32 csetpos, ecsetpos;
  // size_t reslength;
  wxString theresult;
  wxString tempbuffer;
  // the result
  // bool found;
  // if rawdata is NULL, a fatal error
  assert(rawdata!=NULL);
  assert(size>0);
  // starting
  index = rawdata;
  sizeleft = size;
  // the main loop, looking for tags and examining them for the charset attribute
  while (sizeleft>0) {
    // now we scan for a < character (starting a tag)
    bufferptr = (wxByte*) memchr(index,wxByte('<'),sizeleft);
    if ((bufferptr == NULL) || (bufferptr>=endptr)) return theresult;
    start = bufferptr + 1;
    sizeastart = endptr-start;
    // next, we look for the terminating > (otherwise we use the terminating null)
    bufferptr = (wxByte*) memchr(start,wxByte('>'),sizeastart);
    if ((bufferptr == NULL) || (bufferptr>=endptr)) {
      thelast = endptr;
    }
    else thelast = bufferptr;
    // then, we produce a lowercase copy of the section to examine
    buflength = (thelast-start);
    // first, we check for section of too small a length
    if (buflength < 10) {
      // preparing the indexes for the next loop
      index = thelast;
      sizeleft = endptr-index;
      // proceeding
      continue;
    }
    // then we make the buffer, and copy characters over, converting to lowercase
    lowerdata = new char[buflength+1];
    lowerdata[buflength] = '\0';
    for (size_t i=0;i<buflength;i++) lowerdata[i] = tolower((char)(start[i]));
    tempbuffer = wxString::FromAscii(lowerdata);
    // we can now get rid of the temporary char buffer
    delete[] lowerdata;
    // now that we have gotten here, we look for 'charset='
    csetpos = tempbuffer.Find(wxT("charset="));
    // if nothing is found, or we are at the end, we clear out the buffer
    // and then we prepare for the next loop
    if (csetpos<0) {
      // if (tempbuffer==dtype) theresult = wxT("utf-8");
      // preparing the indexes for the next loop
      index = thelast;
      sizeleft = endptr-index;
      // proceeding
      continue;
    }
    // we then check to see if the charset= is followed by a quote
    csetpos += 8;
    if (tempbuffer.length() <= csetpos) break;
    if (tempbuffer.at(csetpos)==wxT('\"')) (csetpos++);
    // looking for a terminator
    ecsetpos = tempbuffer.find_first_of(wxT(" /\"\'"),csetpos);
    // then, we set the terminating position (the end, or the number we just found)
    ecsetpos = (ecsetpos<0)?(tempbuffer.length()):ecsetpos;
    // building the result
    theresult = tempbuffer.substr(csetpos,ecsetpos-csetpos);
    // checking to see if have a valid string
    if (!theresult.IsEmpty()) break;
    // otherwise...we try to look for the charset again
    // preparing the indexes for the next loop
    index = thelast;
    sizeleft = endptr-index;
  }
  /**/jfXLogMessage(wxT("ExtractCharset"),1,theresult);
  // now that we are at the end, we return...
  return theresult;
}
//---------------------------------------------------------------------------
// converts raw data to a wxstring
// we assume an unsigned char is one byte long
wxString* RawToString(wxByte* indata, size_t length, wxString encoding) {
  const wxString fname = wxT("RawToString");
  // as set of asserts
  jfXAssertLog(indata!=NULL,fname,wxT("indata is null"));
  // variables
  size_t strlen;
  bool isutf8;
  wxCSConv converter(encoding.c_str());
  wxString* result = NULL;
  wxString testerr;
  // starting
  isutf8 = ((encoding==wxT("utf-8")) || (encoding==wxT("UTF-8")));
  // doing the conversion
  /**/jfXLogMessageS(fname,1,length);
  /**/jfXLogMessage(fname,2,encoding);
  result = new wxString((const char*)indata,converter,length);
  /**/jfXLogMessageB(fname,4,result==NULL);
  strlen = result->length();
  /**/jfXLogMessageS(fname,5,strlen);
  if (strlen==0) {
    delete result;
    // our fallback is iso-8859-1
    encoding = wxT("iso8859-1");
    converter = wxCSConv(encoding.c_str());
    /**/jfXLogMessageBytes(fname,6,indata,0,4);
    result = new wxString((const char*)indata,converter,length);
    assert(result!=NULL);
    /**/jfXLogMessageS(fname,7,result->length());
  }

  return result;
}
//---------------------------------------------------------------------------
// Concatenates the second bytestring onto the end of the first
wxInt32 ConcatRaw(wxByte*& first, wxByte* second, size_t maxsize,const size_t& flen, const size_t& slen) {
  // the usual logging name
  const wxString fname = wxT("ConcatRaw");
  // local variables
  wxByte* newbuffer;  // just in case we need a bigger buffer
  // basic error checking
  jfXLogMessageU(fname,1,flen);
  jfXLogMessageB(fname,2,first==NULL);
  assert(second!=NULL);
  assert((first==NULL)?(flen==0):(flen>0));
  // if first is NULL...
  if (first == NULL) {
    newbuffer = new wxByte[maxsize+1];
    first = newbuffer;
  }
  // null terminating the byte strings to make sure strcat works properly
  second[slen] = wxByte(0);
  first[flen] = wxByte(0);
  // checking the size of the result
  if ((flen + slen) > maxsize) {
      // we need to replace first with a bigger buffer
      if (flen == 0) maxsize = slen*2;
      else maxsize = flen*2 + slen;
      // so we make it, and copy first to it's start
      newbuffer = new wxByte[maxsize+1];
      strcpy((char *)newbuffer,(char *)first);
      // we get rid of the data for the old first, and assign the new
      delete[] first;
      first = newbuffer;
  }
  // now that we are here, we do the concatenation using str cat
  strcat((char *)first,(char *)second);
  // we return the result
  return (flen+slen);
}
//---------------------------------------------------------------------------
// when given two byte strings, picks the biggest, and deletes the other one
bool PickBiggest(wxByte*& biggest, size_t& biglen, wxByte*& contender,
   size_t& clength) {
  // some asserts to check for errors
  assert(contender!=NULL);
  assert(clength!=0);
  if (biggest!=NULL) assert(biglen!=0);
  else assert(biglen==0);
  // if the current biggest is null, we just move the pointers
  // and the associated lengths
  if (biglen==0) {
    biggest = contender;
    biglen = clength;
    contender = NULL;
    clength = 0;
  }
  // otherwise, we check to see which is bigger, if the second is bigger than the
  // first, we move it to the first (the designated 'biggest' bytestring pointer).
  if (clength>biglen) {
    delete biggest;
    biggest = contender;
    biglen = clength;
    contender = NULL;
    clength = 0;
    return true;
  }
  else {
    delete contender;
    contender = NULL;
    clength = 0;
    return false;
  }
}
//---------------------------------------------------------------------------
// a function that scans the data passed to it for redirection info
wxString GetRedirect(wxByte* rawdata, size_t size) {
  // consatnts
  const size_t minblen = 14;
  const size_t maxblen = 500;
  // misc stuff
  // bool compval;
  // a local copy of temporary data for case insensetive checking
  char *lowerdata;
  char *initialdata;
  size_t buflength;
  // raw data pointers
  wxByte* endptr = rawdata + size;
  wxByte* index;
  wxByte* bufferptr;  wxByte* start; wxByte* thelast;
  wxInt32 sizeleft;
  size_t sizeastart;
  // found data pointers
  // char *ecsetptr;
  wxInt32 csetpos, ecsetpos;
  wxInt32 npos;
  // size_t reslength;
  wxString theresult;
  wxString tempbuffer;
  wxString inbuffer;
  // the result
  // bool found;
  // if rawdata is NULL, a fatal error
  assert(rawdata!=NULL);
  assert(size>0);
  // starting
  index = rawdata;
  sizeleft = size;
  // the main loop, looking for tags and examining them for the charset attribute
  while (sizeleft>0) {
    // now we scan for a < character (starting a tag)
    bufferptr = (wxByte*) memchr(index,wxByte('<'),sizeleft);
    if ((bufferptr == NULL) || (bufferptr>endptr)) return theresult;
    start = bufferptr;
    sizeastart = endptr-(start+1);
    // next, we look for the terminating > (otherwise we use the terminating null)
    bufferptr = (wxByte*) memchr(start,wxByte('>'),sizeastart);
    if ((bufferptr == NULL) || (bufferptr>endptr)) {
      thelast = endptr;
    }
    else thelast = bufferptr;
    // then, we produce copies (lowercase and as before) to examine
    buflength = (thelast-start)-1;
    // first, we check for section of too small a length
    if ((buflength < minblen) || (buflength > maxblen)) {
      // preparing the indexes for the next loop
      index = thelast+1;
      sizeleft = endptr-index;
      // proceeding
      continue;
    }
    // then we make the buffers,
    lowerdata = new char[buflength+1];
    lowerdata[buflength] = '\0';
    initialdata = new char[buflength+1];
    initialdata[buflength] = '\0';
    // and copy characters over, converting to lowercase for one of the buffers
    for (int i=0;i<buflength;i++) {
      lowerdata[i] = tolower((char)(start[1+i]));
      initialdata[i] = (char)(start[1+i]);
    }
    tempbuffer = wxString::FromAscii(lowerdata);
    inbuffer = wxString::FromAscii(initialdata);
    // we can now get rid of the temporary char buffers
    delete[] lowerdata;
    delete[] initialdata;
    // now that we have gotten here, we look for 'charset='
    csetpos = tempbuffer.Find(wxT("http-equiv=\"refresh\""));
    // if nothing is found, we try again *without* the quote marks
    // (some places do without them)
    if (csetpos<0) csetpos = tempbuffer.Find(wxT("http-equiv=refresh"));
    // if nothing is found, or we are at the end, we prepare for the next loop
    if (csetpos<0) {
      // preparing the indexes for the next loop
      index = thelast+1;
      sizeleft = endptr-index;
      // proceeding
      continue;
    }
    // when this is found, we look for the content tag!
    ecsetpos = csetpos+18;
    npos = tempbuffer.find(wxT("content="),ecsetpos);
    // if nothing is found, or we are at the end, we prepare for the next loop
    if (npos<0) {
      // preparing the indexes for the next loop
      index = thelast+1;
      sizeleft = endptr-index;
      // proceeding
      continue;
    }
    // we then check to see if the content= is followed by a quote
    csetpos = npos+8;
    if (tempbuffer.length() <= csetpos) break;
    if (tempbuffer.at(csetpos)==wxT('\"')) (csetpos++);
    // looking for a terminator
    ecsetpos = tempbuffer.find_first_of(wxT(" /\""),csetpos);
    // then, we set the terminating position (the end, or the number we just found)
    ecsetpos = (ecsetpos<0)?(tempbuffer.length()):ecsetpos;
    // grabbing the result ( from the non-lowercase copy!)
    theresult = inbuffer.substr(csetpos,ecsetpos-csetpos);
    Dtrim(theresult);
    // checking to see if have a valid string
    if (theresult.IsEmpty()) break;
    // we have to separate the url from the time
    theresult = theresult.AfterFirst(wxT(';'));
    Dtrim(theresult);
    // whatever is in theresult, we return
    break;
  }
  // now that we are at the end, we return...
  return theresult;
}
//==================================================================
char* wxStringToCharString(const wxString& inval, bool unicode) {
  // local variables
  size_t ilen;
  char* firstcopy;
  // getting the length
  ilen = inval.length();
  ilen = ilen*4+1;
  // allocating the first buffer
  firstcopy = (char*)(malloc(sizeof(char)*ilen));
  // making the first version
  if (unicode) strcpy(firstcopy, (const char*)inval.mb_str(wxConvUTF8) );
  else strcpy(firstcopy, (const char*)inval.mb_str(wxConvISO8859_1) );
  // because I do not want my return to be too long...
  ilen = strlen(firstcopy) + 1;
  firstcopy = (char*)(realloc(firstcopy,sizeof(char*)*ilen));
  // done
  return firstcopy;
}
/////////////////////////////////////////////////////////////////////////////
