/////////////////////////////////////////////////////////////////////////////
// Name:        dirupdate.cpp
// Author :     John Q Metro
// Purpose :    A Tool for updating directory contants to match a source
// Created:     June 11, 2015
// Updated:     July 6, 2015
/////////////////////////////////////////////////////////////////////////////
#ifndef DIRUPDATE_H_INCLUDED
  #include "dirupdate.h"
#endif // DIRUPDATE_H_INCLUDED
//-----------------------------------------
#ifndef UTILS3_H_INCLUDED
  #include "utils3.h"
#endif // UTILS3_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "utils1.h"
#endif // UTILS1_H_INCLUDED
//***************************************************************************

wxString UpdateToString(const jfe_UpdateAction &inact) {
  if (inact == jfe_NOTCHECKED) return wxT("NOTCHECKED ERROR");
  if (inact == jfe_DONOTHING) return wxT("Skipping");
  if (inact == jfe_NEW) return wxT("Copying Over");
  if (inact == jfe_UPDATE) return wxT("Updating");
  if (inact == jfe_DELETE) return wxT("Deleting");
  else {
    assert(false);
    return wxEmptyString;
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfFileInfo::jfFileInfo() {
  xup = jfe_NOTCHECKED;
  fileptr = NULL;
}
//----------------------------------------
jfFileInfo::~jfFileInfo() {
  if (fileptr!=NULL) delete fileptr;
  fileptr = NULL;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// methods
wxFileName* jfFileInfo::MakeFilePointer(wxString basedir) const {
  const wxString fname = wxT("jfFileInfo::MakeFilePointer");
  // special cases
  if (pathname.IsEmpty()) return NULL;
  if (basedir.IsEmpty()) return NULL;
  // creation
  wxString qstro = basedir + wxFileName::GetPathSeparator() + pathname;
  /**/jfXLogMessage(fname,1,qstro);
  wxFileName* result = new wxFileName(qstro);
  result->Normalize();
  return result;
}
//-------------------------------------------------------------------
jfFileInfo* jfFileInfo::MakeNewCopy(wxString basedir, bool withptr) const {
  const wxString fname = wxT("jfFileInfo::MakeNewCopy");
  jfFileInfo* result;
  result = new jfFileInfo();
  result->isdir = isdir;
  result->pathname = pathname;
  /**/jfXLogMessage(fname,1,pathname);
  result->xup = jfe_NEW;
  if (!isdir) {
    result->moddate = moddate;
    if (withptr) result->fileptr = MakeFilePointer(basedir);
  }
  return result;
}
//----------------------------------------------------------------------
wxString jfFileInfo::GetStringRep() const {
  wxString result = pathname + wxT(" ;; ");
  result += UpdateToString(xup) + wxT(" ;; ");
  if (isdir) result += wxT("DIRECTORY ;;");
  else result += wxT("FILE ;; ");
  if (fileptr!=NULL) result += wxT("Has File Pointer");
  result += wxT("\n");
  return result;
}
//***************************************************************************
// constructor
jfDirectoryUpdater::jfDirectoryUpdater() {
  sourceloaded = false;
  targetloaded = false;
  target_index = -1;
  casesen = wxFileName::IsCaseSensitive();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading the dirctory info
//--------------------------------------
bool jfDirectoryUpdater::LoadSource(wxString ldir) {
  ClearList(false);
  if (LoadList(ldir,sourcelist)) {
    sourceloaded = true;
    sourcedir = ldir;
    return true;
  }
  else return false;
}
//--------------------------------------
bool jfDirectoryUpdater::LoadTarget(wxString ldir) {
  ClearList(true);
  if (LoadList(ldir,targetlist)) {
    targetloaded = true;
    targetdir = ldir;
    return true;
  }
  else return false;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// processing
//--------------------------------------
bool jfDirectoryUpdater::CompareSourceAndTarget() {
  const wxString fname = wxT("jfDirectoryUpdater::CompareSourceAndTarget");
  /**/jfXLogMessage(fname,1);
  if ((!sourceloaded)||(!targetloaded)) return false;
  // variables
  std::vector<jfFileInfo*> *newlist;
  jfFileInfo* tcopy;
  size_t scount;
  // preparing before we start
  scount = sourcelist.size();
  /**/jfXLogMessageS(fname,2,scount);
  tindex = 0;
  newlist = new std::vector<jfFileInfo*>();
  /**/jfXLogMessage(fname,3);
  // the outer loop
  for (sindex=0;sindex<scount;sindex++) {
    // the lists are sorted
    while (true) {
      /**/jfXLogMessageS(fname,4,sindex);
      /**/jfXLogMessage(fname,5,sourcelist[sindex]->pathname);
      // targets exhaused, copy everything
      if (tindex>=targetlist.size()) {
        tcopy = sourcelist[sindex]->MakeNewCopy(sourcedir,true);
        /**/jfXLogMessageS(fname,6,tindex);
        newlist->push_back(tcopy);
        break;
      }
      // here, a file or directory is in the source but not the target
      else if (CompareItems()<0) {
        tcopy = sourcelist[sindex]->MakeNewCopy(sourcedir,true);
        /**/jfXLogMessage(fname,7,targetlist[tindex]->pathname);
        newlist->push_back(tcopy);
        break;
      }
      // file or directory present in both
      else if (CompareItems()==0) {
        /**/jfXLogMessage(fname,8,sourcelist[sindex]->pathname);
        if (sourcelist[sindex]->isdir) {
          /**/jfXLogMessageS(fname,9,sindex);
          tcopy = sourcelist[sindex]->MakeNewCopy(sourcedir,false);
          tcopy->xup = jfe_DONOTHING;
        }
        else if ((sourcelist[sindex]->moddate) != (targetlist[tindex]->moddate)) {
          /**/jfXLogMessageS(fname,10,sindex);
          tcopy = sourcelist[sindex]->MakeNewCopy(sourcedir,true);
          tcopy->xup = jfe_UPDATE;
        }
        else {
          /**/jfXLogMessageS(fname,11,sindex);
          tcopy = sourcelist[sindex]->MakeNewCopy(sourcedir,false);
          tcopy->xup = jfe_DONOTHING;
        }
        newlist->push_back(tcopy);
        /**/jfXLogMessageS(fname,12,sindex);
        tindex++;
        /**/jfXLogMessageS(fname,13,tindex);
        break;
      }
      // file in target but not source : delete!
      else {
        /**/jfXLogMessageS(fname,14,sindex);
        SettingUpDelete(newlist);
      }
      /**/jfXLogMessageS(fname,15,sindex);
    }
    // end of the for loop
  }
  // done
  SettingUpDelete(newlist);
  // finishing
  ClearList(true);
  targetlist = (*newlist);
  targetloaded = true;
  // return true;
  return true;
}
//--------------------------------------
void jfDirectoryUpdater::ResetTargetIndex() {
  target_index = -1;
}
//--------------------------------------
bool jfDirectoryUpdater::NextTarget() {
  if (target_index==targetlist.size()) return false;
  target_index++;
  return true;
}
//--------------------------------------
bool jfDirectoryUpdater::DoCurrentTargetAction() {
  const wxString fname = wxT("jfDirectoryUpdater::DoCurrentTargetAction");
  // variables
  jfFileInfo* ctag;
  jfe_UpdateAction qup;
  wxFileName* resfile;
  wxString sourcepath, destpath;
  bool reval;
  // special cases
  if (target_index<0) return false;
  if (target_index>=targetlist.size()) return false;
  // starting
  ctag = targetlist[target_index];
  qup = ctag->xup;
  // do nothing case
  if ((qup==jfe_NOTCHECKED)||(qup==jfe_DONOTHING)) {
  }
  // do stuff
  else {
    // building target info
    /**/jfXLogMessage(fname,1,targetlist[target_index]->pathname);
    destpath= targetdir + wxFileName::GetPathSeparator() + (targetlist[target_index]->pathname);
    // we are dealing with a directory
    if (ctag->isdir) {
      if (qup==jfe_NEW) {
        /**/jfXLogMessage(fname,2,destpath);
        wxMkdir(destpath);
      }
      else if (qup==jfe_DELETE) {
        reval = DeleteDirectoryTree(destpath);
      }
      else assert(false);

    }
    // we are dealing with a file
    else {
      /**/jfXLogMessage(fname,3);
      // overwrite or copy file
      if ((qup==jfe_NEW) || (qup==jfe_UPDATE)) {
        sourcepath = ctag->fileptr->GetFullPath();
        wxCopyFile(sourcepath,destpath);
      }
      // deleting file in target
      else {
        wxRemoveFile(destpath);
      }
    }
  }
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Target / Writable info
//-----------------------------------------------
size_t jfDirectoryUpdater::TargetCount() const {
  return targetlist.size();
}
//-------------------------------------
wxString jfDirectoryUpdater::CurrentTargetPath() const {
  if (target_index<0) return wxEmptyString;
  else if (targetlist[target_index]==NULL) return wxEmptyString;
  else return targetlist[target_index]->pathname;
}
//--------------------------------------
bool jfDirectoryUpdater::CurrentTargetOkay() const {
  wxString xfilen;
  // basic bad conditions
  if (target_index<0) return false;
  if (targetlist[target_index]==NULL) return false;
  // checking the file pointer
  jfe_UpdateAction qxup = targetlist[target_index]->xup;
  if ((qxup==jfe_NEW)||(qxup==jfe_UPDATE)) {
    if (!(targetlist[target_index]->isdir)) {
      if ((targetlist[target_index]->fileptr)==NULL) return false;
      xfilen = (targetlist[target_index]->fileptr)->GetFullPath();
      if (!wxFileName::IsFileReadable(xfilen)) return false;
    }
  }
  // checking writable (skip)
  return true;
}
//--------------------------------------
wxString jfDirectoryUpdater::GetCurrentTargetAction() const {
  if (target_index<0) return wxEmptyString;
  if (target_index>=targetlist.size()) return wxEmptyString;
  return UpdateToString(targetlist[target_index]->xup);
}
//-------------------------------------
wxString jfDirectoryUpdater::GetCurrentTargetInfo() const {
  if (target_index<0) return wxEmptyString;
  if (target_index>=targetlist.size()) return wxEmptyString;
  return targetlist[target_index]->GetStringRep();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructor
jfDirectoryUpdater::~jfDirectoryUpdater() {
  ClearList(true);
  ClearList(false);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods
//--------------------------------------
bool jfDirectoryUpdater::ClearList(bool target) {
  size_t idxer, llen;
  if (target) llen = targetlist.size();
  else llen = sourcelist.size();
  for (idxer=0;idxer<llen;idxer++) {
    if (target) delete targetlist[idxer];
    else delete sourcelist[idxer];
  }
  if (target) {
    targetlist.clear();
    targetloaded = false;
    target_index = -1;
  }
  else {
    sourcelist.clear();
    sourceloaded = false;
  }
  return true;
}
//--------------------------------------
bool jfDirectoryUpdater::LoadList(wxString loaddir, std::vector<jfFileInfo*> &thelist) {
  const wxString fname = wxT("jfDirectoryUpdater::LoadList");
  // initial check
  if (!wxDir::Exists(loaddir)) return false;
  // base variables
  jfFileInfo* cfile;
  wxFileName* cfilex;
  wxArrayString* flist;
  bool doload;
  wxString cname;
  size_t findex, fcount;
  // setting things up
  /**/jfXLogMessage(fname,1);
  flist = new wxArrayString();
  // getting the list of files
  GetDirectoryTreeList(loaddir,*flist);
  if (!casesen) flist->Sort(jfCI_CompareString);
  fcount = flist->Count();
  /**/jfXLogMessageS(fname,2,fcount);
  // going over the list
  for (findex=0;findex<fcount;findex++) {
    /**/jfXLogMessageS(fname,3,findex);
    cname = flist->Item(findex);
    /**/jfXLogMessage(fname,4,cname);
    cfilex = new wxFileName(cname);
    // loading the file info
    cfile = new jfFileInfo();
    cfile->isdir = cfilex->IsDir();
    if (!cfile->isdir) {
      cfile->moddate = cfilex->GetModificationTime();
    }
    doload = cfilex->MakeRelativeTo(loaddir);
    assert(doload);
    cfile->pathname = cfilex->GetFullPath();
    // clearing
    delete cfilex;
    // adding
    thelist.push_back(cfile);
  }
  // done
  delete flist;
  return true;
}
//--------------------------------------
bool jfDirectoryUpdater::SettingUpDelete(std::vector<jfFileInfo*>* &xnewlist) {
  const wxString fname = wxT("jfDirectoryUpdater::SettingUpDelete");
  // variables
  bool dirdelete;
  wxString deletedir;
  jfFileInfo* tcopy;
  // starting
  dirdelete = false;
  // special case
  if (tindex>=targetlist.size()) return true;
  // deletion loop
  while (true) {
    // directory deleting mode
    if (dirdelete) {
      // anything covered by a dir delete should be deleted already
      if ((targetlist[tindex]->pathname).StartsWith(deletedir)) {
        tcopy = targetlist[tindex]->MakeNewCopy(targetdir,false);
        tcopy->xup = jfe_DONOTHING;
        xnewlist->push_back(tcopy);
        tindex++;
      }
      // breaking the dir delete
      else dirdelete = false;
    }
    // not in directory deleting mode
    else {
      // starting directory deleting mode
      if (targetlist[tindex]->isdir) {
        deletedir = targetlist[tindex]->pathname;
        dirdelete = true;
        tcopy = targetlist[tindex]->MakeNewCopy(targetdir,false);
      }
      // deleting isolated files
      else {
        tcopy = targetlist[tindex]->MakeNewCopy(targetdir,false);
      }
      tcopy->xup = jfe_DELETE;
      xnewlist->push_back(tcopy);
      tindex++;
    }
    // checking before the next while loop
    if (tindex>=targetlist.size()) break;
    if (sindex>=sourcelist.size()) continue;
    if (CompareItems() <= 0) break;
  }
  // done
  return true;
}
//-----------------------------------------------------
int jfDirectoryUpdater::CompareItems() const {
  // ensuring indexes are within range
  assert((sindex>=0) && (sindex<sourcelist.size()));
  assert((tindex>=0) && (tindex<targetlist.size()));
  // getting the strings
  wxString spath = sourcelist[sindex]->pathname;
  wxString tpath = targetlist[tindex]->pathname;
  // case insensetive
  if (!casesen) return jfCI_CompareString(spath,tpath);
  else if (spath<tpath) return -1;
  else if (spath==tpath) return 0;
  else return 1;
}

//***************************************************************************
