/////////////////////////////////////////////////////////////////////////////
// Name:        dirupdate.h
// Author :     John Q Metro
// Purpose :    A Tool for updating directory contents to match a source
// Created:     June 9, 2015
// Updated:     July 6, 2015
/////////////////////////////////////////////////////////////////////////////
#ifndef DIRUPDATE_H_INCLUDED
#define DIRUPDATE_H_INCLUDED
#endif // DIRUPDATE_H_INCLUDED
//-----------------------------------------
#include <wx/filename.h>
#include <wx/dir.h>
#include <vector>
//***************************************************************************
// an enumerated type to identify what to do
enum jfe_UpdateAction {jfe_NOTCHECKED, jfe_DONOTHING, jfe_NEW, jfe_UPDATE, jfe_DELETE};
wxString UpdateToString(const jfe_UpdateAction &inact);
//---------------------------------------------------
// file data class
class jfFileInfo {
  public:
    // core data
    bool isdir;
    wxString pathname;
    jfe_UpdateAction xup;
    wxDateTime moddate;
    wxFileName* fileptr;
    // constructor and destructor
    jfFileInfo();
    ~jfFileInfo();
    // methods
    wxFileName* MakeFilePointer(wxString basedir) const;
    jfFileInfo* MakeNewCopy(wxString basedir, bool withptr) const;
    wxString GetStringRep() const;
  protected:
};
//---------------------------------------------------------------------------
// the full sync class
class jfDirectoryUpdater {
  public:
  // constructor
    jfDirectoryUpdater();
    // loading the dirctory info
    bool LoadSource(wxString ldir);
    bool LoadTarget(wxString ldir);
    // processing
    bool CompareSourceAndTarget();
    void ResetTargetIndex();
    bool NextTarget();
    bool DoCurrentTargetAction();
    // target info
    size_t TargetCount() const;
    wxString CurrentTargetPath() const;
    bool CurrentTargetOkay() const;
    wxString GetCurrentTargetAction() const;
    wxString GetCurrentTargetInfo() const;
    // destructor
    ~jfDirectoryUpdater();
  protected:
    // helper methods
    bool ClearList(bool target);
    bool LoadList(wxString loaddir, std::vector<jfFileInfo*> &thelist);
    bool SettingUpDelete(std::vector<jfFileInfo*>* &xnewlist);
    int CompareItems() const;
    // source
    bool sourceloaded;
    wxString sourcedir;
    std::vector<jfFileInfo*> sourcelist;
    // target
    bool targetloaded;
    wxString targetdir;
    std::vector<jfFileInfo*> targetlist;
    int target_index;
    // sync
    size_t tindex, sindex;
    // other
    bool casesen;
};
//***************************************************************************
