/*****************************************************************************
Name    : htmlparse.cpp
Basic   : Declares some utility functions and things used in manipulating html
Author  : John Q Metro
Started : September 13, 2009
Updated : June 24, 2015

******************************************************************************/
#ifndef HTMLPARSE_H_INCLUDED
  #include "htmlparse.h"
#endif // HTMLPARSE_H_INCLUDED
#ifndef STRINGPARSER_H_INCLUDED
  #include "stringparser.h"
#endif // STRINGPARSER_H_INCLUDED

/*****************************************************************************/
// the namespace variables
wxArrayString*  htmlparse::entity_names = NULL;
wxString        htmlparse::entity_chars;
bool            htmlparse::loaded       = false;
//---------------------------------------------------------------------------
/* the purpose of this function is to load data from files about HTML entities */
bool htmlparse::LoadEntities() {
  // variables
  wxString cline,buffer;
  wxTextFile* infile;
  size_t lcount,lindex;
  jfStringParser* qparser;
  unsigned long *tval;
  bool hpres;
  // we start
  infile = new wxTextFile(entity_fname);
  if (!infile->Exists()) {
    delete infile;
    return false;
  }
  infile->Open(wxCSConv(wxT("iso-8859-1")));
  // preparing for the loop
  lcount = infile->GetLineCount();
  if (htmlparse::loaded) {
    assert(htmlparse::entity_names!=NULL);
    delete htmlparse::entity_names;
    htmlparse::loaded = false;
    htmlparse::entity_chars.Empty();
  }
  htmlparse::entity_names = new wxArrayString();
  qparser = new jfStringParser();
  // looping thru the data file, line by line
  for (lindex=0;lindex<lcount;lindex++) {
    // getting the line and checking it
    cline = infile->GetLine(lindex);
    qparser->ChangeData(cline);
    if (!(qparser->StartsWith(wxT("<!ENTITY")))) continue;
    // we have a line starting with entity! extract the name!
    hpres = qparser->GetDelimited(wxT("<!ENTITY"),wxT("CDATA"),buffer);
    assert(hpres);
    Dtrim(buffer);
    htmlparse::entity_names->Add(buffer);
    // next up is the numeric value
    hpres = qparser->GetDelimited(wxT("\"&#"),wxT(";\""),buffer);
    assert(hpres);
    tval = new unsigned long;
    hpres = buffer.ToULong(tval);
    assert(hpres);
    htmlparse::entity_chars+= wxChar(*tval);
    delete tval;
  }
  // clearing up some stuff
  delete qparser;
  infile->Close();
  delete infile;
  // now that we are done, we check the results
  if (htmlparse::entity_names->Count()==0) {
    delete htmlparse::entity_names;
    htmlparse::entity_names = NULL;
		return false;
  }
  else htmlparse::loaded = true;
	return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool htmlparse::EntityToChar(wxString inval, wxChar& outval) {
  int xfound;
  unsigned long tval;
  // a special case
  if (!htmlparse::loaded) return false;
  // we try to look up inval
  xfound = htmlparse::entity_names->Index(inval);
  // if found, we lookup the character
  if (xfound!=wxNOT_FOUND) {
    outval = htmlparse::entity_chars[xfound];
    return true;
  }
  // otherwise, we try to convert as a number
  else {
    // the character is supposed to start with #, but we'll ignore violations
    if (inval[0]==wxT('#')) inval = inval.Mid(1);
    // hex or decimal?
    if (inval[0]==wxT('x')) {
      inval = inval.Mid(1);
      if (!Str2ULong(inval,tval,true)) return false;
    }
    else if (!Str2ULong(inval,tval,false)) return false;
    outval = wxChar(tval);
  }
  return true;
}
//-----------------------------------------------------------------------------
wxString htmlparse::StripTags(const wxString& inval) {
  // variables
  jfStringParser* qparser;
  wxString result=wxEmptyString,buf;
  size_t qindex;
  size_t which;
  bool sbfound = true;
  // starting
  qparser = new jfStringParser(inval);
  // there is a posibility we start in a tag
  if (!qparser->Find1stPair(wxT("<"),wxT(">"),which,qindex)) {
    delete qparser;
    return inval;
  }
  // if which = 2, we start in a tag
  if (which==2) qparser->MovePast(wxT(">"));
  // now, the loop
  while (qparser->GetMovePast(wxT("<"),buf)) {
    result += buf;
    sbfound = qparser->MovePast(wxT(">"));
  }
  // now, we might have to tack on the end...
  if (sbfound) {
    if (qparser->GetToEnd(buf)) result +=buf;
  }
  delete qparser;
  return result;
}
//------------------------------------------------------
wxString htmlparse::ConvertEntities(const wxString& inval, bool keephtml) {
  // variables
  jfStringParser* qparser;
  wxString result=wxEmptyString,buf,ebuf;
  wxChar tent;
  // starting
  qparser = new jfStringParser(inval);
  /* unlike tags, ends of entities are no escaped, so we
  ignore the possibility of starting and ending in an entity */
  while (qparser->GetMovePast(wxT("&"),buf)) {
    result += buf;
    if (!qparser->GetMovePast(wxT(";"),ebuf)) {
      // here, we just add the rest
      result += wxT("&");
      break;
    }
		if (keephtml && TestEntity(ebuf)) {
			result += wxT("&") + ebuf + wxT(";");
		}
		else {
			// if entity extraction works, we convert
			if (EntityToChar(ebuf,tent)) result += tent;
			else result += wxT("?");
		}
  }
  // done
  if (qparser->GetToEnd(buf)) result += buf;
  delete qparser;
  return result;
}
//------------------------------------------------------------------
wxString htmlparse::HTML2Text(const wxString& inval) {
  wxString resval;
  resval = StripTags(inval);
  resval = ConvertEntities(resval,false);
  return resval;
}
//------------------------------------------------------------------
/* manipulates html input by converting all entities except the minimum
 to characters, basically to UTF-8 */
wxString htmlparse::ToUTFHtml(const wxString& inval) {
  // variables
  wxString result;
  result = StripTags(inval);
  result = ConvertEntities(result,true);
	// finishing...
  return result;
}
//-----------------------------------------------------------------
/* in converting to utf8 html, most entities can be converted to characters,
 but some never should be. This function test if the inval is the latter */
bool htmlparse::TestEntity(const wxString& inval) {
	if ((inval==wxT("lt")) || (inval==wxT("lt")) || (inval==wxT("amp"))) return true;
	if ((inval==wxT("38")) || (inval==wxT("60")) || (inval==wxT("62"))) return true;
	else return false;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString htmlparse::UrlEscape(const wxString& indata, bool esc_tilde) {
  // the constants
  const wxString ch_tr  = wxT("% <>#{}|\\^~[]`;/?:@=&$");
  const wxString ch_rep = wxT("25203C3E237B7D7C5C5E7E5B5D603B2F3F3A403D2624");
  // local variables
  wxString result, cchr, chrep;
  wxString testout;
  size_t loopc, esc_len, chrep_len;
  // basic checks
  esc_len = ch_tr.length();
  chrep_len = ch_rep.length();
  assert((esc_len*2) == chrep_len);
  // we start
  result = indata;
  // the replacement loop
  for (loopc = 0; loopc<esc_len; loopc++) {
    cchr = ch_tr.substr(loopc,1);
    chrep = ch_rep.substr(2*loopc,2);
    result.Replace(cchr,chrep);
  }
  // done
  return result;
}
//--------------------------------------------------------------------------
wxString htmlparse::GoogleEscape(const wxString& instring) {
  wxString Result;
  Result = instring;
  Result.Replace(wxT(" "),wxT("+"));
  Result = UrlEscape(Result,true);
  return Result;
}
/*****************************************************************************/
