/******************************************************************************
Name    :   htmlparse2.cpp
Author  :   John Q Metro
Purpose :   For simplyfying over-complex or bad html
Created :   August 15, 2015
Updated :   October 25, 2015
******************************************************************************/
#ifndef HTMLPARSE2_H_INCLUDED
  #include "htmlparse2.h"
#endif // HTMLPARSE2_H_INCLUDED
/*
#ifndef LOGGING_H_INCLUDED
  #include "logging.h"
#endif // LOGGING_H_INCLUDED
*/
//----------------------------------------
#include <assert.h>
#include <set>

#ifndef UTILS1_H_INCLUDED
  #include "utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef UTILS3_H_INCLUDED
  #include "utils3.h"
#endif // UTILS3_H_INCLUDED


/*****************************************************************************/
/* --- BASIC TAG INFO ---  */
const wxString tags_to_wipe_closed = wxString(wxT("applet;audio;")) +
      wxT("button;canvas;datalist;dialog;fieldset;form;iframe;label;legend;") +
      wxString(wxT("menu;menuitem;object;option;output;optgroup;progress;")) +
      wxT("script;select;style;textarea;video");

const wxString tags_to_wipe_non_closed = wxT("embed;input;keygen;") +
        wxString(wxT("param;source;track;bgsound"));
const wxString cdata_tags = wxT("script;style;textarea;xmp;listing");
const wxString inline_track_tags = wxT("a;abbr;b;bdo;cite;code;dfn;em;i;kbd;q;") +
          wxString(wxT("samp;span;strong;sub;sup;var"));
const wxString block_track_tags = wxT("address;article;aside;blockquote;dd;div;") +
          wxString(wxT("dl;ol;pre;h1;h2;h3;h4;h5;h6;hgroup;figure;footer;")) +
          wxT("figcaption;main;nav;noscript;ul;section;table");

// depreciated : acronym, big, center, dir, font, frame, frameset, noframes, strike, blink, xmp
// tt, plaintext, spacer, listing, image, marquee
// ?? noscript, nav, noembed, isindex
//------------------------------
// sets of the above tags
std::set<wxString>* wclosed_tags;
std::set<wxString>* wopen_tags;
std::set<wxString>* xcdata_tags;

// converting the raw lists into sets of wxStrings
void htmlconsts::Setup() {
  wxArrayString* temp = ListToStrArr(tags_to_wipe_closed,wxT(";"),false);
  wclosed_tags = new std::set<wxString>();
  for (int x = 0; x < temp->Count();x++) {
    wclosed_tags->insert((*temp)[x]);
  }
  delete temp;
  temp = ListToStrArr(tags_to_wipe_non_closed,wxT(";"),false);
  wopen_tags = new std::set<wxString>();
  for (int x = 0; x < temp->Count();x++) {
    wopen_tags->insert((*temp)[x]);
  }
  delete temp;
  temp = ListToStrArr(cdata_tags,wxT(";"),false);
  xcdata_tags = new std::set<wxString>();
  for (int x = 0; x < temp->Count();x++) {
    xcdata_tags->insert((*temp)[x]);
  }
  delete temp;
}
//--------------------------
// set testing
bool htmlconsts::IsWipeClosed(const wxString& tocheck) {
  return (0<(wclosed_tags->count(tocheck)));
}
bool htmlconsts::IsWipeNonClosed(const wxString tocheck) {
  return (0<(wopen_tags->count(tocheck)));
}
bool htmlconsts::IsCDATA(const wxString tocheck) {
  return (0<(xcdata_tags->count(tocheck)));
}
//===============================================================================
// === [ METHODS for jfTagNest_Tracker ] ===
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// default constructor (do not use)
jfTagNest_Tracker::jfTagNest_Tracker():std::map<wxString,int>() {}
//-------------------------------
// the real constructor (the input is a semicolon delimited list of keys)
jfTagNest_Tracker::jfTagNest_Tracker(const wxString& keysource):std::map<wxString,int>() {
  wxArrayString* temp = ListToStrArr(tags_to_wipe_closed,wxT(";"),false);
  for (int x = 0; x<(temp->size()); x++) {
    (*this)[(*temp)[x]] = 0;
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// resets the numeric count of each key to 0
void jfTagNest_Tracker::SetZeroValues() {
  jfTagNest_Tracker::iterator zx = begin();
  while (zx!=end()) {
    (zx->second) = 0;
    zx++;
  }
}
//-------------------------------
/* Takes a tag input. If the tag is not a key, returns false. If
the tag is a key but has / before, the tag count for that key is
decremented, otherwise it is incremented. */
bool jfTagNest_Tracker::LogTag(const wxString& inrectag) {
  bool decrement = false; wxString ztag;
  if (inrectag.IsEmpty()) return false;
  if (inrectag[0]==wxT('/')) {
    decrement = true;
    ztag = inrectag.Mid(1);
  }
  else ztag = inrectag;
  if (count(ztag)==0) return false;
  if (decrement) --((*this)[ztag]);
  else ++((*this)[ztag]);
  return true;
}
//+++++++++++++++++++++++++++++++++++++++
// getting data
//-------------------------------
// is the input a key tag?
bool jfTagNest_Tracker::HasTag(const wxString& totest) const {
  wxString thekey;
  if (totest[0]==wxT('/')) thekey = totest.Mid(1);
  else thekey = totest;
  return (count(thekey)>0);
}
//----------------------------------------
/* returns a list of closing tags for every tag key with a positive value */
wxString jfTagNest_Tracker::CloseTags() const {
  // setting up for the loop
  wxString result = wxEmptyString;
  jfTagNest_Tracker::const_iterator wi = begin();
  // the loop
  while(wi!=end()) {
    if ((wi->second)>0) result += wxT("</") + (wi->first) + wxT('>');
  }
  // done
  return result;
}
//-------------------------------
// returns true if any of the keys have negative values
bool jfTagNest_Tracker::HasOverClosed() const {
  jfTagNest_Tracker::const_iterator qx = begin();
  while (qx!=end()) {
    if ((qx->second)<0) return true;
    qx++;
  }
  return false;
}
//================================================================================
// --- [ jfHTML_Scrubber Methods ] ---
//++++++++++++++++++++++++++++++++++++++++++++++++++
jfHTML_Scrubber::jfHTML_Scrubber(const wxString& html_to_scrub) {
  data = html_to_scrub;
  lower_data = data.Lower();
  datalen = data.length();
  workindex = 0;
  inpara = false;
  incdata = false;
  curcentre = nexcentre = false;
  blocktag_t = inlinetag_t = NULL;
}
//---------------------------------------------------
jfHTML_Scrubber::~jfHTML_Scrubber() {
  delete blocktag_t;
  delete inlinetag_t;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfHTML_Scrubber::ScrubHTML() {
  wxString beforetag;
  wxString outdata;
  bool rvale;
  // insert check to see if we are in a tag
  // setting up the tag trackers
  blocktag_t = new jfTagNest_Tracker(block_track_tags);
  inlinetag_t = new jfTagNest_Tracker(inline_track_tags);
  // the main loop
  while (ToNextTag(outdata)) {
    // adding the text we have moved past
    AppendText(outdata);
    // end tags only get special treatment if they end paragraphs or centering
    if (TagIsEnd()) {
        /* Closing a paragraph has a special routine (the 'if' condition allows
           ignoring stray unpaired /p's). */
        if (ctag==wxT("/p")) {
          if (inpara) CloseParagraph();
        }
        // center tags are used in strange ways sometimes
        else if (ctag==wxT("/center")) {
          if (!inpara) AppendTag(); // no special processing, this has to be changed later
          else if (tagless_pstore.IsEmpty()) { // at the start of a paragraph,
            curcentre = false;
          }
          // if not at the start of a paragraph, we assume this only applies to the next paragraph
          else {
            nexcentre = false;
          }
        }
        else {
          // for end blocks, we make sure inline tags are closed
          if (blocktag_t->HasTag(ctag)) {
            AppendText(inlinetag_t->CloseTags());
            inlinetag_t->SetZeroValues();
            blocktag_t->LogTag(ctag);
          }
          else inlinetag_t->LogTag(ctag);
          AppendTag();
        }
    }
    // start tags get checked
    else {
      // some tags are wiped out, contents included
      if (htmlconsts::IsWipeClosed(ctag)) {
        if (!ToEndTag(ctag)) {
          /* what to do when the end of data is reached
           before the end of the tag we are getting rid of */
          workindex = datalen;
          break;
        }
      }
      // some tags are ignored (and thus are wiped out, these are content-free)
      else if (htmlconsts::IsWipeNonClosed(ctag)) {}
      // some cdata tags
      else if (htmlconsts::IsCDATA(ctag)) {
        // the only cdata tags which escape wiping should be obsolete xmp and listing
        /* insert special code */
      }
      else if (ctag==wxT("p")) {
      }
      else if (ctag==wxT("center")) {
      }
      // for all other tags, no special handling
      else {
        // however, block tags implicity close paragraphs
        if ((blocktag_t->HasTag(ctag)) && inpara) CloseParagraph();
        AppendTag();
      }
    }
  }
  // after to loop, we are either
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++
/* After calling this, the index is after the next tag, text before the next
tag is placed in outdata, ctag is set to the tag string (lowercase, rawtag
hold the tag without lowercasing), and attributesmfor that tag are placed in
tagconents. */
bool jfHTML_Scrubber::ToNextTag(wxString& outdata) {
  // special bad cases
  const wxString fname = wxT("jfHTML_Scrubber::ToNextTag");
  if (data.IsEmpty()) return false;
  if (workindex>=datalen) return false;
  // variables
  size_t spos, mindex2, epos1, epos2;
  wxString buildcdata;
  // this assumes we are not chopped off in the middle of a tag
  if (incdata) {
    buildcdata = wxT("</") + rawtag;
    spos = lower_data.find(buildcdata,workindex);
  }
  else spos = data.find(wxT('<'),workindex);
  if (spos==wxString::npos) return false;
  // here, we have found a tag... find the next space or end
  mindex2 = spos+1;
  epos1 = data.find(wxT('>'),mindex2);
  if (epos1==wxString::npos) return false;  // we also assume the end is not in a tag
  if (incdata) incdata = false;
  epos2 = data.find(wxT(' '),mindex2); // can only handle space
  outdata = data.Mid(workindex,spos-workindex);
  // there is something inside the tag
  if (epos2<epos1) {
    rawtag = data.Mid(mindex2,epos2-mindex2);
    tagcontents = data.Mid(epos2,epos1-epos2);
    Dtrim(rawtag);
  }
  // the tag has no extra stuff
  else {
    rawtag = data.Mid(mindex2,epos1-mindex2);
    tagcontents = wxEmptyString;
  }
  // after
  ctag = rawtag.Lower();
  workindex = epos1+1;
  return true;
}
//-----------------------------
bool jfHTML_Scrubber::ToEndTag(const wxString& stag) {
  // looking for the end
  wxString comptag = wxT("</") + stag;
  size_t spos, epos;
  spos = lower_data.find(comptag,workindex);
  if (spos==wxString::npos) return false;
  // once found, we look for the end of the end
  epos = data.find(wxT(">"),spos);
  if (epos==wxString::npos) return false;
  // moving past that
  workindex = epos+1;
}
//------------------------------
bool jfHTML_Scrubber::TagIsEnd() const {
  return (ctag[0]==wxT('/'));
}
//------------------------------
void jfHTML_Scrubber::AppendTag() {
  wxString endadd = wxT("<") + ctag;
  if (!tagcontents.IsEmpty()) endadd += wxT(" ") + tagcontents;
  endadd += wxT(">");
  if (inpara) pstore += endadd;
  else result += endadd;
}
//------------------------------
void jfHTML_Scrubber::AppendText(const wxString& apptext) {
  if (inpara) {
    pstore += apptext;
    tagless_pstore += apptext;
  }
  else result +=apptext;
}

void jfHTML_Scrubber::CloseParagraph() {
}

//======================================================================================
// --- [ jfHTMTag_ContentCounter Methods ] ---
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// basic constructors
jfHTMTag_ContentCounter::jfHTMTag_ContentCounter(const wxString& intag_string) {
  assert(!intag_string.IsEmpty());
  tag_string = intag_string;
  total_count = non_empty_count = 0;
  max_content_len = 0;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// adding data
size_t jfHTMTag_ContentCounter::AddTagContents(const wxString& tag_contents) {
  total_count++;
  // no contents
  if (tag_contents.IsEmpty()) return (total_count-non_empty_count);
  // new tag
  else if (count(tag_contents)==0) {
    if (tag_contents.length()>max_content_len) max_content_len = tag_contents.length();
    (*this)[tag_contents] = 1;
    non_empty_count++;
    return 1;
  }
  // non-new tag
  else {
    size_t txcount = (*this)[tag_contents];
    txcount++;
    (*this)[tag_contents] = txcount;
    non_empty_count++;
    return txcount;
  }
}
//--------------------------------------------------------------------
size_t jfHTMTag_ContentCounter::Reduce(size_t min_size) {
  jfHTMTag_ContentCounter::iterator iter,citer;
  // perparing for the loop
  iter = begin();
  size_t rc = 0;
  // the loop
  while (iter!=end()) {
    citer = iter;
    iter++;
    if ((citer->second)<min_size) {
      erase(citer);
      rc++;
    }
  }
  // done
  return rc;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting data
//-------------------------------
wxString jfHTMTag_ContentCounter::GetTag() const {
  return tag_string;
}
//-------------------------------
size_t jfHTMTag_ContentCounter::GetTotalCount() const {
  return total_count;
}
//-------------------------------
size_t jfHTMTag_ContentCounter::GetNonEmptyCount() const {
  return non_empty_count;
}
//-------------------------------
size_t jfHTMTag_ContentCounter::GetCountForContents(const wxString& tag_contents) const {
  jfHTMTag_ContentCounter::const_iterator qxval;
  qxval = find(tag_string);
  if (qxval==end()) return 0;
  else return qxval->second;
}
//-------------------------------
bool jfHTMTag_ContentCounter::ContentsExist(const wxString& tag_contents) const {
  return (count(tag_contents)!=0);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// i/o
wxString jfHTMTag_ContentCounter::AllToString() const {
  // variables
  wxString result, curritem;
  size_t clen;
  jfHTMTag_ContentCounter::const_iterator citem;
  // header
  result = wxT("TAG : ") + tag_string + wxT(" : ");
  result += SInt2Str(total_count) + wxT(" occurances.\n");
  // individual contents items
  citem = begin();
  while (citem!=end()) {
    curritem = citem->first;
    clen = curritem.length();
    curritem.Pad(max_content_len-clen,wxT(' '));
    curritem += wxT(" : ") + SInt2Str(citem->second);
    result += curritem + wxT("\n");
    citem++;
  }
  // done
  return result;
}

//========================================================================
// --- [ jfHTMLTags_CounterStore Methods ] ---
// adding data
bool jfHTMLTags_CounterStore::RecordTagData(const wxString& intag,const wxString& in_tagcontents) {
  // specials and variables
  const wxString fname = wxT("jfHTMLTags_CounterStore::RecordTagData");
  assert(!intag.IsEmpty());
  jfHTMTag_ContentCounter* thetag;
  jfHTMLTags_CounterStore::iterator itmptr;
  // checking, case the tag is already in
  if (count(intag)!=0) {
    itmptr = find(intag);
    thetag = itmptr->second;
    thetag->AddTagContents(in_tagcontents);
    return false;
  }
  // new tag
  else {
    thetag = new jfHTMTag_ContentCounter(intag);
    thetag->AddTagContents(in_tagcontents);
    (*this)[intag] = thetag;
    return true;
  }
}
//-----------------------------------------------
size_t jfHTMLTags_CounterStore::Reduce(size_t min_size) {
  jfHTMLTags_CounterStore::iterator xit;
  // reducing all
  size_t xitt = 0;
  xit = begin();
  while (xit!=end()) {
    xitt += (xit->second)->Reduce(min_size);
    xit++;
  }
  return xitt;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// i/o
//------------------------------
wxString jfHTMLTags_CounterStore::AllToString() const {
  // variables
  wxString result;
  jfHTMLTags_CounterStore::const_iterator bokx;
  // starting the header
  result  = wxT("TAG and TAG CONTENTS COUNT\n");
  result += wxT("===========================\n\n\n");
  // starting with the loop
  bokx = begin();
  while (bokx!=end()) {
    result += (bokx->second)->AllToString();
    result += wxT("\n\n");
    bokx++;
  }
  // done
  return result;
}
//------------------------------
bool jfHTMLTags_CounterStore::AllToFile(const wxString& filename) const {
  if (filename.IsEmpty()) return false;
  wxString rdata = AllToString();
  return ToFile(rdata,filename);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructor
jfHTMLTags_CounterStore::~jfHTMLTags_CounterStore() {
  jfHTMLTags_CounterStore::iterator bokx;
  bokx = begin();
  while (bokx!=end()) {
    delete (bokx->second);
    (bokx->second) = NULL;
  }
  clear();
}
//================================================================
/*
jfHTML_Studier::jfHTML_Studier() {
  ResetWorkData();
  tagdata = new jfHTMLTags_CounterStore();
  p_content_store = new jfHTMLTags_CounterStore();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfHTML_Studier::AnalyseHTML(const wxString& indata) {
  const wxString fname = wxT("jfHTML_Studier::AnalyseHTML");
  wxString outstr;
  // starting
  if (indata.IsEmpty()) return false;
  ResetWorkData();
  maindata = indata;
  // the loop. We track paragraphs, assuming we do not start inside a <p>
  while (ToNextTag(outstr)) {
    tagdata->RecordTagData(ctag,tagc);
    // checking for cdata tags
    incdata = ((ctag==wxT("script")) || (ctag==wxT("style")) || (ctag==wxT("xmp")) || (ctag==wxT("textarea")));
    // inside a paragraph
    if (inpara) {
      pcontent += outstr;
      if (ctag == wxT("/p")) {
        // closing
        inpara = false;
        if (pcontent.length()<40) p_content_store->RecordTagData(wxT("p"),pcontent);
        pcontent.clear();
      }
      // should not happen, but closes and reopends paragraph!
      else if (ctag == wxT("p")) {
        if (pcontent.length()<40) p_content_store->RecordTagData(wxT("p"),pcontent);
        else p_content_store->RecordTagData(wxT("p"),wxT("p"));
        pcontent.clear();
      }
      else AddTag();
    }
    // not inside a paragraph
    else {
      if (ctag== wxT("p")) inpara = true;
      else if (ctag == wxT("/p")) {
        p_content_store->RecordTagData(wxT("/p"),wxT("p"));
      }
    }
    // end of loop
  }
  // done
  return true;
}
//--------------------------------
void jfHTML_Studier::ReduceContent() {
  if (p_content_store!=NULL) p_content_store->Reduce(3);
}
//--------------------------------
bool jfHTML_Studier::WriteFiles() const {
  tagdata->AllToFile(wxT("tagdata1.txt"));
  p_content_store->AllToFile(wxT("tagdata2.txt"));
  return true;

}
//++++++++++++++++++++++++++++++++++++++++++++++++++
jfHTML_Studier::~jfHTML_Studier() {
  if (tagdata!=NULL) delete tagdata;
  if (p_content_store!=NULL) delete p_content_store;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfHTML_Studier::ResetWorkData() {
  maindata.clear();
  mindex = 0;
  inpara = false;
  incdata = false;
  pcontent.clear();
  tagc.clear();
  ctag.clear();
}
//--------------------------------
bool jfHTML_Studier::ToNextTag(wxString& outdata) {
  // special bad cases
  const wxString fname = wxT("jfHTML_Studier::ToNextTag");
  if (maindata.IsEmpty()) return false;
  if (mindex>=maindata.length()) return false;
  // variables
  size_t spos, mindex2, epos1, epos2;
  wxString buildcdata;
  // this assumes we are not chopped off in the middle of a tag
  if (incdata) {
    buildcdata = wxT("</") + octag;
    spos = maindata.find(buildcdata,mindex);
  }
  else spos = maindata.find(wxT('<'),mindex);
  if (spos==wxString::npos) return false;
  // here, we have found a tag... find the next space or end
  mindex2 = spos+1;
  epos1 = maindata.find(wxT('>'),mindex2);
  if (epos1==wxString::npos) return false;  // we also assume the end is not in a tag
  if (incdata) incdata = false;
  epos2 = maindata.find(wxT(' '),mindex2); // can only handle space
  outdata = maindata.Mid(mindex,spos-mindex);
  // there is something inside the tag
  if (epos2<epos1) {
    ctag = maindata.Mid(mindex2,epos2-mindex2);
    tagc = maindata.Mid(epos2,epos1-epos2);
    Dtrim(tagc);
  }
  // the tag has no extra stuff
  else {
    ctag = maindata.Mid(mindex2,epos1-mindex2);
    tagc = wxEmptyString;
  }
  // after
  octag = ctag;
  ctag.MakeLower();
  mindex = epos1+1;
  return true;
}
//--------------------------------
bool jfHTML_Studier::TagIsEnd() const {
  if (ctag.IsEmpty()) return false;
  else return (ctag[0]==wxT('/'));
}
//--------------------------------
bool jfHTML_Studier::AddTag() {
  // bad cases
  if (!inpara) return false;
  if (ctag.IsEmpty()) return false;
  // doing it
  pcontent += wxT("<") + ctag;
  if (!tagc.IsEmpty()) pcontent += wxT(" ") + tagc;
  pcontent += wxT(">");
  return true;
}
*/
/*****************************************************************************/
