/******************************************************************************
Name    :   htmlparse2.h
Author  :   John Q Metro
Purpose :   For simplyfying over-complex or bad html
Created :   August 15, 2015
Updated :   October 1, 2015
******************************************************************************/
#ifndef HTMLPARSE2_H_INCLUDED
  #define HTMLPARSE2_H_INCLUDED
#endif // HTMLPARSE2_H_INCLUDED
//----------------------------------------
#include <map>
#include <wx/string.h>
/*****************************************************************************/
namespace htmlconsts {
  void Setup();
  bool IsWipeClosed(const wxString& tocheck);
  bool IsWipeNonClosed(const wxString tocheck);
  bool IsCDATA(const wxString tocheck);
};
//======================================================================================
// map class intended to track opening and closing tag counts
class jfTagNest_Tracker : public std::map<wxString,int> {
  public:
    jfTagNest_Tracker();
    jfTagNest_Tracker(const wxString& keysource);
    // setting values
    void SetZeroValues();
    bool LogTag(const wxString& inrectag);
    // getting data
    bool HasTag(const wxString& totest) const;
    wxString CloseTags() const;
    bool HasOverClosed() const;
  protected:
};
//======================================================================================
// class for cleaning fic HTML
class jfHTML_Scrubber {
  public:
    // constructor and destructor
    jfHTML_Scrubber(const wxString& html_to_scrub);
    ~jfHTML_Scrubber();
    // the main function
    bool ScrubHTML();
  protected:
    bool ToNextTag(wxString& outdata);
    bool ToEndTag(const wxString& stag);
    bool TagIsEnd() const;
    void AppendTag();
    void AppendText(const wxString& apptext);
    void CloseParagraph();
    // internal data
    wxString data, lower_data;
    size_t datalen;
    size_t workindex;
    wxString result;
    bool curcentre, nexcentre;
    // current tag data
    wxString ctag, rawtag;
    wxString tagcontents;
    bool inpara,incdata;
    wxString pstore,tagless_pstore;
    // tag tracking
    jfTagNest_Tracker* blocktag_t;
    jfTagNest_Tracker* inlinetag_t;

};






// --- [ Stuff for getting statistics ] ---

// keeps counts of the contents of particular HTML tags
class jfHTMTag_ContentCounter : public std::map<wxString,size_t> {
  public:
    // basic constructors
    jfHTMTag_ContentCounter(const wxString& intag_string);
    // adding data
    size_t AddTagContents(const wxString& tag_contents);
    size_t Reduce(size_t min_size);
    // getting data
    wxString GetTag() const;
    size_t GetTotalCount() const;
    size_t GetNonEmptyCount() const;
    size_t GetCountForContents(const wxString& tag_contents) const;
    bool ContentsExist(const wxString& tag_contents) const;
    // i/o
    wxString AllToString() const;
  protected:
    // tag info
    wxString tag_string;
    size_t total_count,non_empty_count;
    size_t max_content_len;
};
//-------------------------------
// Set of the above, using the tag as key
class jfHTMLTags_CounterStore : public std::map<wxString,jfHTMTag_ContentCounter*> {
  public:
    // adding data
    bool RecordTagData(const wxString& intag,const wxString& in_tagcontents);
    size_t Reduce(size_t min_size);
    // i/o
    wxString AllToString() const;
    bool AllToFile(const wxString& filename) const;
    // destructor
    ~jfHTMLTags_CounterStore();
  protected:

};

//============================================================
/*
class jfHTML_Studier {
  public:
    // constructor
    jfHTML_Studier();
    // methods
    bool AnalyseHTML(const wxString& indata);
    void ReduceContent();
    bool WriteFiles() const;
    // destructor
    ~jfHTML_Studier();
  protected:
    // helper methods
      void ResetWorkData();
      bool ToNextTag(wxString& outdata);
      bool TagIsEnd() const;
      bool AddTag();
    // working data
      wxString maindata;
      size_t mindex;
      wxString pcontent, ctag, tagc, octag;
      bool inpara, incdata;
    // stores
      jfHTMLTags_CounterStore* tagdata;
      jfHTMLTags_CounterStore* p_content_store;
};
*/


/*****************************************************************************/

