/******************************************************************************
// Name:        htmlskelton.cpp
// Author :     John Q Metro
// Purpose :    For filling in html i/o skeltons with output info
// Created:     November 15, 2009
// Updated:     November 15, 2009
******************************************************************************/
#ifndef HTMLSKELTON_H_INCLUDED
  #include "htmlskelton.h"
#endif // HTMLSKELTON_H_INCLUDED
#ifndef UTILS2_H_INCLUDED
  #include "utils2.h"
#endif // UTILS2_H_INCLUDED
#ifndef STRINGPARSER_H_INCLUDED
  #include "stringparser.h"
#endif // STRINGPARSER_H_INCLUDED
//****************************************************************************/
// the constructor
jfSkeletonParser::jfSkeletonParser() {
  nametags_text = NULL;
  values_text = NULL;
  text_count = 0;
  // boolean items
  nametags_bool = NULL;
  values_bool = NULL;
  bool_count = 0;
  // integer items
  nametags_uint = NULL;
  values_uint = NULL;
  uint_count = 0;;
  // parsing stuff
  done = false;
  total_count = 0;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// data and parsing
//---------------------------------------------------------------------------
bool jfSkeletonParser::SetSkeleton(wxString inskel) {
  src_skelton = inskel;
  done = false;
  return !inskel.IsEmpty();
}
//---------------------------------------------------------------------------
bool jfSkeletonParser::ProduceIntermediate() {
  // consts
  const wxString csn_st = wxT("<<<&");
  const wxString csn_ed = wxT("&>>>");
  const wxString cs_end = wxT("<<&>>");
  // local variables
  jfStringParser* qparser;
  wxString buffer,namebuffer;
  jfsp_DATATYPE ftype;
  size_t fpos;
  // starting
  qparser = new jfStringParser(src_skelton, true);
  int_result = wxEmptyString;
  // the main loop
  while (qparser->GetMovePast(csn_st,buffer)) {
    int_result += buffer;
    // we find the nametag
    if (!qparser->GetMovePast(csn_ed,namebuffer)) {
      delete qparser;
      return false;
    }
    // we get the stuff within the conditional
    if (!qparser->GetMovePast(cs_end,buffer)) {
      delete qparser;
      return false;
    }
    // we, we check if we do add the conditional stuff
    if (!FindTag(namebuffer,ftype,fpos)) {
      delete qparser;
      return false;
    }
    // we've got the conditional, do we include?
    if (NotEmpty(ftype,fpos)) {
      int_result += buffer;
    }
  }
  // at the end of the loop, we add the non-conditional stuff at the end
  qparser->GetToEnd(buffer);
  int_result += buffer;
  // finishing off
  delete qparser;
  return true;
}
//---------------------------------------------------------------------------
/* we replace things with their values. Since logically, most tags will be
used, we use search and replace */
bool jfSkeletonParser::MakeResult() {
  // constants
  wxString nstart = wxT("<<#");
  wxString ntend = wxT("#>>");
  // local varibales
  size_t loopc;
  wxString nametag,replacement;
  // starting
  dne_result = int_result;
  // the text loop
  for (loopc=0;loopc<text_count;loopc++) {
    // getting the tag and its replacemnt
    nametag = nstart + (nametags_text->Item(loopc) + ntend);
    replacement = values_text->Item(loopc);
    // doing search and replace
    dne_result.Replace(nametag,replacement);
  }
  // the boolen loop
  for (loopc=0;loopc<bool_count;loopc++) {
    // getting the tag and its replacemnt
    nametag = nstart + (nametags_bool->Item(loopc) + ntend);
    replacement = Bool2Str((*values_bool)[loopc]);
    // doing search and replace
    dne_result.Replace(nametag,replacement);
  }
  // the unsigned integer loop
  for (loopc=0;loopc<uint_count;loopc++) {
    // getting the tag and its replacemnt
    nametag = nstart + (nametags_uint->Item(loopc) + ntend);
    replacement = SInt2Str((*values_uint)[loopc]);
    // doing search and replace
    dne_result.Replace(nametag,replacement);
  }
  // done
  return true;
}
//---------------------------------------------------------------------------
wxString jfSkeletonParser::GetResult() const {
  return dne_result;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// adding items
//---------------------------------------------------------------------------
bool jfSkeletonParser::AddText(wxString nametag, const wxString& value) {
  // local variables
  jfsp_DATATYPE ores;
  size_t oloc;
  // we create the arrays if we previously had none...
  if (text_count==0) {
    nametags_text = new wxArrayString();
    values_text = new wxArrayString();
  }
  // if we do, we have some checking to do.
  else {
    // we check if the name exists...
    if (FindTag(nametag,ores,oloc)) {
      // if it dowes, but it is not a text tag, fail!
      if (ores!=jfsk_TEXT) return false;
      // if it does, and is text, we change the value
      else {
        values_text->Item(oloc) = value;
        return true;
      }
    }
    // finding the tag failed, so we add... after the next closing brace
  }
  // adding
  nametags_text->Add(nametag);
  values_text->Add(value);
  text_count++;
  total_count++;
  return true;
}
//---------------------------------------------------------------------------
bool jfSkeletonParser::AddBool(wxString nametag, const bool& value) {
  // local variables
  jfsp_DATATYPE ores;
  size_t oloc;
  // we create the arrays if we previously had none...
  if (bool_count==0) {
    nametags_bool = new wxArrayString();
    values_bool = new jfBoolVector();
  }
  // if we do, we have some checking to do.
  else {
    // we check if the name exists...
    if (FindTag(nametag,ores,oloc)) {
      // if it dowes, but it is not a text tag, fail!
      if (ores!=jfsk_BOOL) return false;
      // if it does, and is text, we change the value
      else {
        (*values_bool)[oloc] = value;
        return true;
      }
    }
    // finding the tag failed, so we add... after the next closing brace
  }
  // adding
  nametags_bool->Add(nametag);
  values_bool->push_back(value);
  bool_count++;
  total_count++;
  return true;
}
//---------------------------------------------------------------------------
bool jfSkeletonParser::AddUInt(wxString nametag, const size_t& value) {
  // local variables
  jfsp_DATATYPE ores;
  size_t oloc;
  // we create the arrays if we previously had none...
  if (uint_count==0) {
    nametags_uint = new wxArrayString();
    values_uint = new jfIndexVector();
  }
  // if we do, we have some checking to do.
  else {
    // we check if the name exists...
    if (FindTag(nametag,ores,oloc)) {
      // if it dowes, but it is not a text tag, fail!
      if (ores!=jfsk_UINT) return false;
      // if it does, and is text, we change the value
      else {
        (*values_uint)[oloc] = value;
        return true;
      }
    }
    // finding the tag failed, so we add... after the next closing brace
  }
  // adding
  nametags_uint->Add(nametag);
  values_uint->push_back(value);
  uint_count++;
  total_count++;
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the destructor
jfSkeletonParser::~jfSkeletonParser() {
  if (text_count!=0) {
    delete  nametags_text;
    delete values_text;
  }
  if (bool_count!=0) {
    delete nametags_bool;
    delete values_bool;
  }
  if (uint_count!=0) {
    delete nametags_uint;
    delete values_uint;
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfSkeletonParser::FindTag(wxString nametag, jfsp_DATATYPE& ftype, size_t& fpos) const {
  // local variables
  size_t loopc;
  // searching text
  for (loopc=0;loopc<text_count;loopc++) {
    if ((nametags_text->Item(loopc))==nametag) {
      ftype = jfsk_TEXT;
      fpos = loopc;
      return true;
    }
  }
  // searching bool
  for (loopc=0;loopc<bool_count;loopc++) {
    if ((nametags_bool->Item(loopc))==nametag) {
      ftype = jfsk_BOOL;
      fpos = loopc;
      return true;
    }
  }
  // searching uint
  for (loopc=0;loopc<uint_count;loopc++) {
    if ((nametags_uint->Item(loopc))==nametag) {
      ftype = jfsk_UINT;
      fpos = loopc;
      return true;
    }
  }
  // all failed
  return false;
}
//---------------------------------------------------------------------------
bool jfSkeletonParser::NotEmpty(jfsp_DATATYPE ftype, size_t fpos) const {
  if (ftype==jfsk_TEXT) {
    assert(fpos<text_count);
    wxString examine = values_text->Item(fpos);
    return !(examine.IsEmpty());
  }
  else if (ftype==jfsk_BOOL) {
    assert(fpos<bool_count);
    return (*values_bool)[fpos];
  }
  else if (ftype==jfsk_UINT) {
    assert(fpos<uint_count);
    return (0!=((*values_uint)[fpos]));
  }
  else assert(false);
}
//****************************************************************************/
