/*****************************************************************************
Name    : logging.cpp
Basic   : Separates out logging from the other stuff
Author  : John Q Metro
Started : January 13, 2011
Updated : March 23, 2013
Notes   :

******************************************************************************/
#ifndef LOGGING_H_INCLUDED
  #include "logging.h"
#endif // LOGGING_H_INCLUDED
//------------------------------------------
#ifndef UTILS3_H_INCLUDED
  #include "utils3.h"
#endif // UTILS3_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef INITEND_H_INCLUDED
  #include "../../initend.h"
#endif // INITEND_H_INCLUDED
#ifndef ALTLOG_H_INCLUDED
  #include "altlog.h"
#endif // ALTLOG_H_INCLUDED

//*****************************************************************************
// logging
wxLogStderr* logtarget0 = NULL;
wxString LOGNAME1 = wxT("trace0.txt");

//========================================================
// creates the output log
wxLogStderr* MakeOutLog(bool append) {
  wxLogStderr* rval;
  wxString fname = LOGNAME1;
  // we start
  if (!append) rval = new wxLogStderr(fopen(fname.char_str(),"wt"));
  else rval = new wxLogStderr(fopen(fname.char_str(),"at"));
  // done
  return rval;
}
//--------------------------------------------------------
// top level starting the logging
void InitMainLog() {
  // altlog version
  if (altlog::doaltlog) {
    altlog::logvar = new jfAltLogger();
    altlog::logvar->MakeLog(altlog::logname);
  }
  // wxLogMessage version
  else {
    logtarget0 = MakeOutLog();
    wxLog::SetActiveTarget(logtarget0);
  }
}
//========================================================
void jfXLogMessage(const wxChar* funcname,size_t index, wxString info) {
	assert(funcname!=NULL);
  wxString res_msg = BLogCore(funcname,index,true);
	res_msg += info;
	jfXLogString(res_msg);
}
//---------------------------------------------------------------------------
void jfXLogMessageS(const wxChar* funcname,size_t index, size_t data) {
	assert(funcname!=NULL);
  wxString res_msg = BLogCore(funcname,index,true);
  res_msg += SInt2Str(data);
	jfXLogString(res_msg);
}
//-----------------------------------------------------------------------
void jfXLogMessageI(const wxChar* funcname,size_t index, wxInt32 data) {
	assert(funcname!=NULL);
  wxString res_msg = BLogCore(funcname,index,true);
	res_msg += IInt2Str(data);
	jfXLogString(res_msg);
}
//-----------------------------------------------------------------------
void jfXLogMessageU(const wxChar* funcname,size_t index, size_t data) {
	assert(funcname!=NULL);
  wxString res_msg = BLogCore(funcname,index,true);
	res_msg += UInt2Str(data);
	jfXLogString(res_msg);
}
//--------------------------------------------------------------------
void jfXLogMessageB(const wxChar* funcname,size_t index, bool data) {
	assert(funcname!=NULL);
	wxString res_msg = BLogCore(funcname,index,true);
	res_msg += Bool2Str(data);
	jfXLogString(res_msg);
}
//--------------------------------------------------------------------
void jfXLogMessage(const wxChar* funcname,size_t index) {
	assert(funcname!=NULL);
	wxString res_msg = BLogCore(funcname,index,false);
	jfXLogString(res_msg);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfXLogMessage2S(const wxChar* funcname,size_t index, size_t data1, size_t data2) {
	assert(funcname!=NULL);
	wxString res_msg = BLogCore(funcname,index,true);
	res_msg += SInt2Str(data1) + wxT(" : ");
	res_msg += SInt2Str(data2);
	jfXLogString(res_msg);
}
//-----------------------------------------------------------------------------------
void jfXLogMessage2U(const wxChar* funcname,size_t index, size_t data1, size_t data2) {
	assert(funcname!=NULL);
	wxString res_msg = BLogCore(funcname,index,true);
	res_msg += UInt2Str(data1) + wxT(" : ");
	res_msg += UInt2Str(data2);
	jfXLogString(res_msg);
}
//-----------------------------------------------------------------------------
void jfXLogMessageBytes(const wxChar* funcname, size_t index, const wxByte* indata, size_t iindex, size_t length) {
	assert(funcname!=NULL);
	wxString res_msg = BLogCore(funcname,index,true);
	for (size_t offset=0;offset<length;offset++) {
	  res_msg += UInt2Str(indata[iindex+offset]) + wxT(" ");
	}
	jfXLogString(res_msg);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString BLogCore(const wxChar* funcname, size_t index, bool more) {
  // check
  assert(funcname!=NULL);
  // make the result
	wxString res_msg = wxString(funcname) + wxT(" ");
	wxString buffer = SInt2Str(index);
	if (index<10) buffer = wxT("0") + buffer;
	res_msg+= buffer;
	if (more) res_msg += wxT(" : ");
	// done
	return res_msg;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* The actual Logging functions, which hides the various ways of doing it */
void jfXLogString(wxString in_msg) {
  if (globalsettings::dologging) {
    // alt logging write
    if (altlog::doaltlog) {
      altlog::loglock.Lock();
      altlog::logvar->LogString(in_msg);
      altlog::loglock.Unlock();
    }
    // using wxLogMessage instead
    else {
      // the section...
      #ifdef __WXMSW__
        wxLogMessage(wxT("%s"),in_msg);
      #else
        wxLogMessage(wxT("%s"),in_msg.c_str());
      #endif
    }
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// assert with logging
void jfXAssertLog(bool condition, wxString funcname, const wxChar* outmsg) {
  wxString result;
  if (!condition) {
    result = wxT("Assertion failed in ");
    result += funcname;
    result += wxT(" : ");
    result += outmsg;
    jfXLogString(result);
    assert(false);
  }
}
//*****************************************************************************
