/*****************************************************************************
Name    : stringparser.cpp
Basic   : String parsing class
Author  : John Q Metro
Started : August 21, 2012 (split from utils2.cpp)
Updated : July 19, 2017 (added MovePastLimit)
Notes   :

******************************************************************************/
#ifndef STRINGPARSER_H_INCLUDED
  #include "stringparser.h"
#endif // STRINGPARSER_H_INCLUDED
//----------------------------------------
#ifndef UTILS1_H_INCLUDED
  #include "utils1.h"
#endif // UTILS1_H_INCLUDED
/*****************************************************************************/
// the constructors
jfStringParser::jfStringParser() {
  ready = false;
  mainindex = 0;
  oldindex = 0;
}
//--------------------------------------------------------------------------
jfStringParser::jfStringParser(wxString indata, bool incasesen) {
  ChangeData(indata,incasesen);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// change indata
void jfStringParser::ChangeData(wxString indata, bool incasesen) {
  rawdata = indata;
  mainindex = 0;
  oldindex = 0;
  casesen = incasesen;
  lowdata = indata;
  skipped_data = wxEmptyString;
  // do we go for case sensentivity
  if (!casesen) lowdata.MakeLower();
  ready = !rawdata.IsEmpty();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting basic info
//--------------------------------------------------------------------------
bool jfStringParser::IsReady() const {
  return ready;
}
//--------------------------------------------------------------------------
bool jfStringParser::IsCaseSensetive() const {
  return casesen;
}
//--------------------------------------------------------------------------
size_t jfStringParser::GetIndex() const {
  return mainindex;
}
//--------------------------------------------------------------------------
size_t jfStringParser::GetLength() const {
  return rawdata.length();
}
//--------------------------------------------------------------------------
wxString jfStringParser::GetData() const {
  return rawdata;
}
//--------------------------------------------------------------------------
wxString jfStringParser::GetBlock(size_t innlen) const {
  // special cases
  if ((innlen==0) || (mainindex>=rawdata.length())) return wxEmptyString;
  // getting the length
  size_t extlen;
  if ((mainindex+innlen)>=rawdata.length()) extlen = rawdata.length()-mainindex;
  else extlen = innlen;
  // extrating
  return rawdata.substr(mainindex,extlen);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// changing some controllers
//--------------------------------------------------------------------------
void jfStringParser::SetCaseSen(bool incasesen) {
  if (incasesen!=casesen) {
    if (!incasesen) {
      lowdata.MakeLower();
      casesen = false;
    }
    else {
      lowdata = rawdata;
      casesen = true;
    }
  }
}
//--------------------------------------------------------------------------
bool jfStringParser::SetIndex(size_t innindex) {
  oldindex = mainindex;
  mainindex = innindex;
  if (innindex>=rawdata.length()) return false;
  else return true;
}
//------------------------------------------------------------------
bool jfStringParser::GoBack() {
  if (oldindex==mainindex) return false;
  mainindex = oldindex;
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// looks for <find>, if found, returns true and moves the index past the substring
bool jfStringParser::MovePast(wxString find) {
  // local variables
  size_t fpos;
  // basic check
  if (!ready) return false;
  // case sensetivity
  if (!casesen) find.MakeLower();
  // looking...
  fpos = lowdata.find(find,mainindex);
  if (fpos == wxString::npos) return false;
  else {
    skipped_data = rawdata.substr(mainindex,fpos-mainindex);
    SetIndex(fpos + find.length());
    return true;
  }
}
//------------------------------------------------------------------------
/* looks for the second instance of <find>. if found, trurns true and moves
the index past the substring */
bool jfStringParser::MovePastTwice(wxString find) {
  //constants
  wxString fname = wxT("jfStringParser::MovePastTwice");
  // local variables
  size_t fpos, oindex;
  // starting and checks
  if (!ready) return false;
  if (!casesen) find.MakeLower();
  oindex = mainindex;
  // looking once...
  fpos = lowdata.find(find,mainindex);
  if (fpos == wxString::npos) return false;
  // we've found, so we look again
  mainindex = fpos + find.length();
  fpos = lowdata.find(find,mainindex);
  if (fpos == wxString::npos) {
    mainindex = oindex;
    return false;
  }
  // we've found the item
  else {
    skipped_data = rawdata.substr(oindex,fpos-oindex);
    SetIndex(fpos + find.length());
    return true;
  }
}
//-------------------------------------------------------------------------
/* Similar to MovePast, but we look for limit first, find must be before limit
for true to be returned. */
bool jfStringParser::MovePastLimit(wxString find, wxString limit) {
  // local variables
  size_t limpos,fpos;
  // basic check
  if (!ready) return false;
  // case sensetivity
  if (!casesen) limit.MakeLower();
  // looking for the limit...
  limpos = lowdata.find(limit,mainindex);
  if (limpos == wxString::npos) return false;
  // looking for find...
  if (!casesen) find.MakeLower();
  fpos = lowdata.find(find,mainindex);
  if (fpos == wxString::npos) return false;
  if (fpos >= limpos) return false;
  // success, setting the skipped data and altering mainindex
  skipped_data = rawdata.substr(mainindex,fpos-mainindex);
  SetIndex(fpos + find.length());
  return true;
}
//-------------------------------------------------------------------------
/* Sort of like using MovePast twice, but we keep the original mainindex
if we cannot find both substrings */
bool jfStringParser::MovePastTwo(wxString first, wxString second) {
  // local variables
  size_t fpos, secindex;
  // basic check
  if (!ready) return false;
  // case sensetivity
  if (!casesen) first.MakeLower();
  // looking...
  fpos = lowdata.find(first,mainindex);
  if (fpos == wxString::npos) return false;
  else {
    // looking again
    secindex = fpos + first.length();
    fpos = lowdata.find(second,secindex);
    if (fpos == wxString::npos) return false;
    // handling a positive result
    skipped_data = rawdata.substr(mainindex,fpos-mainindex);
    SetIndex(fpos + second.length());
    return true;
  }
}
//-------------------------------------------------------------------------
// similar to MovePast, but uses Find1stPair to move past the first
bool jfStringParser::MovePastAlt(wxString find1, wxString find2) {
  // local variables
  size_t fpos,which;
  size_t whichl;
  bool locres;
  // we start
  locres = Find1stPair(find1,find2,which,fpos);
  if (!locres) return false;
  if (which==1) whichl = find1.length();
  else whichl = find2.length();
  skipped_data = rawdata.substr(mainindex,fpos-mainindex);
  SetIndex(fpos + whichl);
  return true;
}
//--------------------------------------------------------------------------
// uses the StartsWithAtIndex method to skip a substring if there
bool jfStringParser::MovePastStart(wxString start) {
  // a special case
  if (!ready) return false;
  // the main finding...
  if (StartsWithAtIndex(start)) {
    SetIndex(mainindex + start.length());
    return true;
  }
  else return false;
};
//-------------------------------------------------------------------
bool jfStringParser::MovePastStartAlt(wxString start1, wxString start2) {
  // a special case
  if (!ready) return false;
  // checking...
  bool found1, found2;
  found1 = StartsWithAtIndex(start1);
  if (found1 && (start1.length()>=start2.length())) {
    SetIndex(mainindex+start1.length());
    return true;
  }
  found2 = StartsWithAtIndex(start2);
  if (found2) {
    SetIndex(mainindex+start2.length());
    return true;
  }
  else if (found1) {
    SetIndex(mainindex+start1.length());
    return true;
  }
  else return false;
}
//--------------------------------------------------------------------------
wxString jfStringParser::GetSkipped() const {
  return skipped_data;
}
//--------------------------------------------------------------------------
/* looks for <find>, if found, returns true, returns everything between the index
and the start of <find> in outbuf, and moves the index past the substring */
bool jfStringParser::GetMovePast(wxString end, wxString& outbuf) {
  size_t fpos;
  if (!ready) return false;
  if (!casesen) end.MakeLower();
  fpos = lowdata.find(end,mainindex);
  if (fpos == wxString::npos) return false;
  else {
    outbuf = rawdata.substr(mainindex,fpos-mainindex);
    SetIndex(fpos + end.length());
    return true;
  }
}
//------------------------------------------------------------------------
bool jfStringParser::MovePastOrEnd(wxString endval, wxString& outbuf) {
  // returning false
  if (!ready) return false;
  if (mainindex>=lowdata.length()) return false;
  // variables
  bool qval = GetMovePast(endval,outbuf);
  if (!qval) {
    qval = GetToEnd(outbuf);
    if (qval) mainindex = lowdata.length();
  }
  return qval;
}
//-------------------------------------------------------------------------
bool jfStringParser::GetMovePastPair(wxString end1, wxString end2, wxString& outbuf) {
  // local variables
  size_t wres, pres;
  // starting
  if (Find1stPair(end1,end2,wres,pres)) {
    outbuf = rawdata.substr(mainindex,pres-mainindex);
    if (wres==1) SetIndex(pres + end1.length());
    else SetIndex(pres + end2.length());
    return true;
  }
  else return false;
}
//--------------------------------------------------------------------------------------
bool jfStringParser::GetMovePastPairI(wxString end1, wxString end2, wxString& outbuf, bool& owhich) {
  // local variables
  size_t wres, pres;
  // starting
  if (Find1stPair(end1,end2,wres,pres)) {
    outbuf = rawdata.substr(mainindex,pres-mainindex);
    if (wres==1) {
      SetIndex(pres + end1.length());
      owhich = true;
    }
    else {
      SetIndex(pres + end2.length());
      owhich = false;
    }
    return true;
  }
  else return false;

}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special move upto methods
bool jfStringParser::MoveUpto(wxString ifind) {
  // local variables
  size_t fpos;
  // basic check
  if (!ready) return false;
  // case sensetivity
  if (!casesen) ifind.MakeLower();
  // looking...
  fpos = lowdata.find(ifind,mainindex);
  if (fpos == wxString::npos) return false;
  else {
    skipped_data = rawdata.substr(mainindex,fpos-mainindex);
    SetIndex(fpos);
    return true;
  }
}
//------------------------------------------------------------
bool jfStringParser::GetUpto(wxString end, wxString& outres) {
  size_t fpos;
  if (!ready) return false;
  if (!casesen) end.MakeLower();
  fpos = lowdata.find(end,mainindex);
  if (fpos == wxString::npos) return false;
  else {
    outres = rawdata.substr(mainindex,fpos-mainindex);
    SetIndex(fpos);
    return true;
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// similar to get move past, but tries to get a number, returning false if things succeed.
bool jfStringParser::GetMovePastULong(wxString end, unsigned long& outval, wxString& outerr) {
  // constants
  wxString err_endnotfound = wxT("The specified end of the field was not found!");
  wxString err_notaninteger = wxT("The field was found, but is not an integer!");
  // local variables
  wxString buffer;
  size_t oindex;
  // we start...
  oindex = mainindex;
  if (!GetMovePast(end,buffer)) {
    outerr = err_endnotfound;
    return false;
  }
  Dtrim(buffer);
  // numeric conversion
  if (!Str2ULongC(buffer,outval)) {
    outerr = err_notaninteger;
    mainindex = oindex;
    return false;
  }
  return true;
}
//--------------------------------------------------------------------------
/* a combination of MovePast and GetMovePast, return the value between <start> and
<end> (if found), and move the end past <end> */
bool jfStringParser::GetDelimited(wxString start, wxString end, wxString& outbuf) {
  if (!MovePast(start)) return false;
  return GetMovePast(end,outbuf);
}
//------------------------------------------------------------------------------------
// similar to GetDelimited, but tries to get a number.
bool jfStringParser::GetDelimitedULong(wxString start, wxString end, unsigned long& outval, wxString& outerr) {
  // constants
  wxString err_startnotfound = wxT("The specified start of the field was not found!");
  wxString err_endnotfound = wxT("The specified end of the field was not found!");
  wxString err_notaninteger = wxT("The field was found, but is not an integer!");
  // local variables
  wxString buffer;
  size_t oindex;
  // we start...
  oindex = mainindex;
  if (!MovePast(start)) {
    outerr = err_startnotfound;
    return false;
  }
  if (!GetMovePast(end,buffer)) {
    outerr = err_endnotfound;
    mainindex = oindex;
    return false;
  }
  // we trim spaces and remove commas
  buffer.Replace(wxT(","), wxT(""), true);
  Dtrim(buffer);
  // numeric conversion
  if (!Str2ULongC(buffer,outval)) {
    outerr = err_notaninteger;
    mainindex = oindex;
    return false;
  }
  return true;
}
//--------------------------------------------------------------------------------------
/* another version of get delimited which uses a pair of possible ends */
bool jfStringParser::GetDelimitedEndPair(wxString start, wxString end1,wxString end2, wxString& outbuf) {
  // moving past the start
  if (!MovePast(start)) return false;
  // using find first pair
  return GetMovePastPair(end1,end2,outbuf);
}
//----------------------------------------------------------------------------
/* return strue if a line starts with a certain string */
bool jfStringParser::StartsWith(wxString start) const {
  if (!casesen) start.MakeLower();
  return lowdata.StartsWith(start);
}
//------------------------------------------------------------------------------
// returns true if the given substring is at the current index
bool jfStringParser::StartsWithAtIndex(wxString start) const {
  size_t fpos;
  if (!ready) return false;
  if (!casesen) start.MakeLower();
  fpos = lowdata.find(start,mainindex);
  if (fpos == wxString::npos) return false;
  return (fpos==mainindex);
}
//-----------------------------------------------------------------------------
/* locates the first of a pair */
bool jfStringParser::Find1stPair(wxString astr, wxString bstr, size_t& which, size_t& outloc) {
  // variables
  size_t fpos1,fpos2;
  size_t slen1,slen2;
  bool npos1,npos2;
  // we search
  if (!casesen) astr.MakeLower();
  if (!casesen) bstr.MakeLower();
  fpos1 = lowdata.find(astr,mainindex);
  fpos2 = lowdata.find(bstr,mainindex);
  npos1 = (fpos1==wxString::npos);
  npos2 = (fpos2==wxString::npos);
  // going thru the cases
  // both are not found
  if (npos1 && npos2) return false;
  // 1 is not found
  if (npos1) {
    which = 2;
    outloc = fpos2;
  }
  // 2 is not found
  else if (npos2) {
    which = 1;
    outloc = fpos1;
  }
  // both are found
  else {
    if (fpos1<fpos2) {
      which = 1;
      outloc = fpos1;
    }
    else if (fpos1==fpos2) {
      slen1 = astr.length();
      slen2 = bstr.length();
      outloc = fpos1;
      if (slen1<slen2) which = 2;
      else which = 1;
    }
    else {
      which = 2;
      outloc = fpos2;
    }
  }
  return true;

}
//----------------------------------------------------------------------------
/* locates the first of a multiple set */
bool jfStringParser::Find1stSet(const wxArrayString* instring, size_t& which, size_t& outloc) {
  // local variables
  bool found = false;
  size_t lowpos,lowindex;
  size_t loopc,numstr;
  size_t fpos;
  wxString cstring;
  // special cases
  if (instring==NULL) return false;
  // preparing for the loop
  numstr = instring->Count();
  lowpos = GetLength()+50;
  // the loop
  for (loopc=0;loopc<numstr;loopc++) {
    cstring = (*instring)[loopc];
    if (!casesen) cstring.MakeLower();
    fpos = lowdata.find(cstring,mainindex);
    if (fpos!=wxString::npos) {
      found = true;
      if (fpos<lowpos) {
        lowpos = fpos;
        lowindex = loopc;
      }
    }
  }
  // after the loop, we produce the results
  if (found) {
    which = lowindex;
    outloc = lowpos;
  }
  return found;
}
//----------------------------------------------------------------------------
bool jfStringParser::GetMovePastSet(const wxArrayString* instring, size_t& which, wxString& outbuf) {
  // variables
  size_t outpos,endlen;
  bool isfound;
  // starting
  if (!ready) return false;
  isfound = Find1stSet(instring,which,outpos);
  if (!isfound) return false;
  // extracting
  outbuf = rawdata.substr(mainindex,outpos-mainindex);
  endlen = (instring->Item(which)).length();
  SetIndex(outpos + endlen);
  Dtrim(outbuf);
  return true;
}
//----------------------------------------------------------------------------
// * returns everything up to the end
bool jfStringParser::GetToEnd(wxString& out) {
  if (!ready) return false;
  if (mainindex>=lowdata.length()) return false;
  out = rawdata.substr(mainindex,lowdata.length()-mainindex);
  return true;
}
//--------------------------------------------------------------------------------
bool jfStringParser::GetToEndULong(unsigned long& outval, wxString& outerr) {
  // constants
  wxString err_endfailed = wxT("Getting to the end failed!");
  wxString err_notaninteger = wxT("The field was found, but is not an integer!");
  // local variables
  wxString buffer;
  size_t oindex = mainindex;
  // we start...
  if (!GetToEnd(buffer)) {
    outerr = err_endfailed;
    mainindex = oindex;
    return false;
  }
  // we trim spaces and remove commas
  buffer.Replace(wxT(","), wxT(""), true);
  Dtrim(buffer);
  // numeric conversion
  if (!Str2ULongC(buffer,outval)) {
    outerr = err_notaninteger;
    mainindex = oindex;
    return false;
  }

  return true;
}
//=====================================================================================
bool jfStringParser::GetDelimitedFromEnd(wxString start, wxString end, wxString& outbuf,
        wxString endmarker) {
  //local varibales
  size_t endmark_index,fpos;
  size_t nindex,startdex;
  // the endmarker
  if (endmarker.IsEmpty()) endmark_index = lowdata.length()-1;
  else {
    if (!casesen) endmarker.MakeLower();
    fpos = lowdata.find(endmarker,mainindex);
    if (fpos==wxString::npos) return false;
    else endmark_index = fpos-1;
  }
  // searching back from the endmarker
  if (!casesen) end.MakeLower();
  fpos = lowdata.rfind(end,endmark_index);
  if (fpos==wxString::npos) return false;
  // we've found the last delimiter
  nindex = fpos + end.length();
  endmark_index = fpos;
  // looking for the first delimiter
  if (!casesen) start.MakeLower();
  fpos = lowdata.rfind(start,endmark_index-1);
  if (fpos==wxString::npos) return false;
  // extracting
  startdex = fpos+start.length();
  outbuf = rawdata.substr(startdex,endmark_index-startdex);
  skipped_data = rawdata.substr(mainindex,fpos-mainindex);
  // done
  oldindex = mainindex;
  mainindex = nindex;
  return true;
}
//------------------------------------------------------------------
bool jfStringParser::GetDelimitedFromEndULong(wxString start, wxString end,
        unsigned long& outbuf, wxString& outerr, wxString endmarker) {
  // constants
  wxString err_notfound = wxT("The specified field was not found!");
  wxString err_notaninteger = wxT("The field was found, but is not an integer!");
  // variables
  wxString thevalue;
  // saving the old data
  wxString old_skip = skipped_data;
  size_t ooindex = oldindex;
  size_t omindex = mainindex;
  // trying to extract
  if (!GetDelimitedFromEnd(start,end,thevalue,endmarker)) {
    outerr = err_notfound;
    return false;
  }
  Dtrim(thevalue);
  // trying to convert
  if (!Str2ULongC(thevalue,outbuf)) {
    skipped_data = old_skip;
    oldindex = ooindex;
    mainindex = omindex;
    outerr = err_notaninteger;
    return false;
  }
  else return true;
}
//--------------------------------------------------------------------------------
bool jfStringParser::GetAfterLast(wxString start, wxString& outbuf, wxString endmarker) {
  //local varibales
  size_t endmark_index,fpos;
  size_t startdex;
  // the endmarker
  if (endmarker.IsEmpty()) endmark_index = lowdata.length()-1;
  else {
    if (!casesen) endmarker.MakeLower();
    fpos = lowdata.find(endmarker,mainindex);
    if (fpos==wxString::npos) return false;
    else endmark_index = fpos-1;
  }
  // searching back from the endmarker
  if (!casesen) start.MakeLower();
  fpos = lowdata.rfind(start,endmark_index);
  if (fpos==wxString::npos) return false;
  if (fpos<mainindex) return false;
  // we've found the first delimiter
  startdex = fpos + start.length();
  // extracting
  outbuf = rawdata.substr(startdex,endmark_index-startdex+1);
  skipped_data = rawdata.substr(mainindex,fpos-mainindex);
  // done
  oldindex = mainindex;
  mainindex = endmark_index+1;
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// another custom method, truncates the search after an endmarker to simplify things
bool jfStringParser::ChopAfter(wxString endmark, bool chopmarker) {
  // local variables
  size_t endlen, endindex, fpos;
  // special case
  if (endmark.IsEmpty()) return false;
  // starting
  if (!casesen) endmark.MakeLower();
  fpos = lowdata.find(endmark,mainindex);
  if (fpos==wxString::npos) return false;
  // we have found, we truncate
  if (chopmarker) endindex = fpos;
  else {
    endlen = endmark.length();
    endindex = fpos+endlen;
  }
  rawdata.Truncate(endindex);
  lowdata.Truncate(endindex);
  return true;
}
/*****************************************************************************/
