/*****************************************************************************
Name    : stringparser.h
Basic   : String parsing class
Author  : John Q Metro
Started : August 21, 2012 (spit from utils2.h)
Updated : July 19, 2017 (added MovePastLimit)
Notes   :

******************************************************************************/
#ifndef STRINGPARSER_H_INCLUDED
#define STRINGPARSER_H_INCLUDED
#endif // STRINGPARSER_H_INCLUDED
//----------------------------------------
#ifndef WX_PRECOMP
  #include <wx/wx.h>
#else
#include "wx/wxprec.h"
#endif

#ifdef __BORLANDC__
    #pragma hdrstop
#endif
/*****************************************************************************/
/* another class used in parsing... this one is for more complicated things than delimited
lists, it is mainly there to simplify keeping track of a few things */
class jfStringParser : public wxObject {
  public:
    // the constructors
    jfStringParser();
    jfStringParser(wxString indata, bool incasesen=false);
    // change indata
    void ChangeData(wxString indata, bool incasesen=false);
    // getting basic info
    bool IsReady() const;
    bool IsCaseSensetive() const;
    size_t GetIndex() const;
    size_t GetLength() const;
    wxString GetData() const;
    wxString GetBlock(size_t innlen) const;
    // changing some controllers
    void SetCaseSen(bool incasesen);
    bool SetIndex(size_t innindex);
    bool GoBack();
    // parsing methods
    // looks for <find>, if found, returns true and moves the index past the substring
    bool MovePast(wxString find);
    bool MovePastTwice(wxString find);
    bool MovePastLimit(wxString find, wxString limit);
    bool MovePastTwo(wxString first, wxString second);
    bool MovePastAlt(wxString find1, wxString find2);
    bool MovePastStart(wxString start);
    bool MovePastStartAlt(wxString start1, wxString start2);
    wxString GetSkipped() const;
    /* looks for <find>, if found, returns true, returns everything between the index
    and the start of <find> in outbuf, amd moves the index past the substring */
    bool GetMovePast(wxString end, wxString& outbuf);
    bool MovePastOrEnd(wxString endval, wxString& outbuf);
    bool GetMovePastPair(wxString end1, wxString end2, wxString& outbuf);
    bool GetMovePastPairI(wxString end1, wxString end2, wxString& outbuf, bool& owhich);
    // special move upto methods
    bool MoveUpto(wxString ifind);
    bool GetUpto(wxString end, wxString& outres);
    // similar to get move past, but tries to get a number
    bool GetMovePastULong(wxString end, unsigned long& outval, wxString& outerr);
    /* a combination of MovePast and GetMovePast, return the value between <start> and
    <end> (if found), and move the end past <end> */
    bool GetDelimited(wxString start, wxString end, wxString& outbuf);
    // similar to GetDelimited, but tries to get a number.
    bool GetDelimitedULong(wxString start, wxString end, unsigned long& outval, wxString& outerr);
    /* another version of get delimited which uses a pair of possible ends */
    bool GetDelimitedEndPair(wxString start, wxString end1,wxString end2, wxString& outbuf);
    /* returns true if the string or string at the index starts with a substring */
    bool StartsWith(wxString start) const;
    bool StartsWithAtIndex(wxString start) const;
    /* locates the first of a pair */
    bool Find1stPair(wxString astr, wxString bstr, size_t& which, size_t& outloc);
    /* locates the first of a multiple set */
    bool Find1stSet(const wxArrayString* instring, size_t& which, size_t& outloc);
    bool GetMovePastSet(const wxArrayString* instring, size_t& which, wxString& outbuf);
    // * returns everything up to the end
    bool GetToEnd(wxString& out);
    bool GetToEndULong(unsigned long& outval, wxString& outerr);
    // special methods looking back
    bool GetDelimitedFromEnd(wxString start, wxString end, wxString& outbuf, wxString endmarker=wxEmptyString);
    bool GetDelimitedFromEndULong(wxString start, wxString end, unsigned long& outbuf, wxString& outerr, wxString endmarker=wxEmptyString);
    bool GetAfterLast(wxString start, wxString& outbuf, wxString endmarker=wxEmptyString);
    // another custom method, truncates the search after an endmarker to simplify things
    bool ChopAfter(wxString endmark, bool chopmarker);
  private:
    // internal data...
    wxString rawdata;
    wxString lowdata;
    wxString skipped_data;
    size_t oldindex, mainindex;
    bool ready;
    bool casesen;
};
/*****************************************************************************/
