/*****************************************************************************
Name    : utils1.cpp
Basic   : Defines some utility functions and things used in the other files
Author  : John Q Metro
Started : March 17, 2009
Updated : July 23, 2015
Notes   : mostly taken from the earlier jaffis_utils.cpp and jaffis_utils2.cpp

******************************************************************************/
#define __STDC_FORMAT_MACROS 1
#ifndef UTILS1_H_INCLUDED
  #include "utils1.h"
#endif // UTILS1_H_INCLUDED

#include <wx/tokenzr.h>
//****************************************************************************
const wxString EXPR_DELIMS=wxT("&|~%();@#");
//============================================================================
// two way trimming
void Dtrim(wxString& input) {
  input.Trim();
  input.Trim(false);
}
//===========================================================================
// converts an integer to a string
// why does this not have an already defined function?
wxString IInt2Str(const wxInt32& input){
  // local variables
  wxString result;
  // making the result
  result.Printf(wxT("%d"),input);
  return result;
}
//--------------------------------------------------------------------------
// converts an unsigned integer to a string
// why does this not have an already defined function?
wxString UInt2Str(const wxUint32& input){
  // local variables
  wxString result;
  // making the result
  result.Printf(wxT("%u"),input);
  return result;
}
//-------------------------------------------------------------------------
// converts size_t to a string
wxString SInt2Str(const size_t& input) {
	// local variables
	wxString result;
	/* making the result. note that if size_t is 64bit, wxWidgets-2.9 wants %lu
	as the format specifier */
	#if wxCHECK_VERSION(2,9,0)
    if (sizeof(size_t)==8) result.Printf(wxT("%lu"),input);
    else result.Printf(wxT("%u"),input);
	#else
    result.Printf(wxT("%u"),input);
	#endif
	return result;
}
//--------------------------------------------------------------------------
// converts a double to a string
wxString Doubl2Str(const double& input) {
  // local variables
  wxString result;
  // making the result
  result.Printf(wxT("%f"),input);
  return result;
}
//--------------------------------------------------------------------------
// converts a boolean to a string
wxString Bool2Str(const bool& inval) {
  // local variables
  wxString result;
  result = (inval)?(wxT("true")):(wxT("false"));
  return result;
}
//-----------------------------------------------------------------------
// converts a 64bit unsigned value to a string
wxString UInt64_2Str(const uint64_t& inval) {
  // local variables
  wxString result, format_str;
  // cross platform blues
  #ifdef  _MSC_VER
    result.Printf(wxT("%I64u"),inval);
  #else
    result.Printf(wxT("%"PRIu64),inval);
  #endif
  return result;
}
//------------------------------------------------------------------------
/* the wxString conversion is not bad, except it uses a pointer, which you have
to allocate and also perhaps deallocate. Here is a single line function. */
bool Str2ULong(const wxString& inval, unsigned long& outval, bool hex) {
  // variables
  unsigned long* oval;
  int base = (hex)?(16):(10);
  oval = new unsigned long;
  if (!inval.ToULong(oval,base)) {
    delete oval;
    return false;
  }
  else {
    outval = *oval;
    delete oval;
    return true;
  }
}
//--------------------------------------------------------------------
// a version of the above that handles commas
bool Str2ULongC(const wxString& inval, unsigned long& outval) {
  wxString ninval = inval;
  ninval.Replace(wxT(","), wxT(""));
  return Str2ULong(ninval,outval);
}
//-------------------------------------------------------------------
// handles values like '1.9K'
bool Count2ULong(const wxString& inval, unsigned long& outval) {
  //local variables
  double* xval;
  size_t slen;
  bool resval;
  wxString zpval;
  // we start
  slen = inval.length();
  outval = 0;
  zpval = inval;
  // counts in the thousands
  if (inval[slen-1]==wxT('K')) {
    zpval.RemoveLast();
    xval = new double;
    resval = zpval.ToDouble(xval);
    if (resval) outval = floor((*xval)*1000);
    delete xval;
    return resval;
  }
  // more precise under 1000 count
  else return  Str2ULongC(inval,outval);
}
//==========================================================================
/* takes an input string. if the string is "true" or "false", returns TRUE
and also passes back which in the reference parameter */
bool BoolFromStr(const wxString& in_src, bool& result) {
    if (in_src==wxT("true")) {
      result = true;
      return true;
    }
    else if (in_src==wxT("false")) {
      result = false;
      return true;
    }
    else return false;
}
//================================================================================
// to shorten things, returns true if buffer conatins f1 or f2
bool ContainsAlt(const wxString& buffer, wxString f1, wxString f2) {
  bool f1e,f2e;
  bool r1,r2;
  // special cases
  if (buffer.IsEmpty()) return false;
  f1e = f1.IsEmpty();
  f2e = f2.IsEmpty();
  assert(!(f1e&&f2e));
  // starting
  if (f1e) r1 = false;
  else {
    r1 = buffer.find(f1)!=wxString::npos;
  }
  if (f2e) r2 = false;
  else {
    r2 = buffer.find(f2)!=wxString::npos;
  }
  // done
  return (r1 || r2);
}
//=================================================================================
/* Takes a string and returns the result with the specified chars in delim
preceeded with a backslash. Backslashes are also escapsed. Do not repeat
characters in delims. */
wxString EscapeString(const wxString& in_base, const wxString& delims) {
  // variables
  size_t delim_len,delim_index;
  wxString xreplace,resval=wxT("");
  // we skip a special case
  if (in_base.length()!=0) {
    // escaping backslashes first
    resval = in_base;
    resval.Replace(wxT("\\"),wxT("\\\\"));
    // now, if applicable, we go on to the other delimiters
    delim_len = delims.length();
    if (delim_len!=0) {
      for (delim_index=0;delim_index<delim_len;delim_index++) {
        xreplace = delims[delim_index];
        resval.Replace(xreplace,wxT('\\')+xreplace);
      }
    }
  }
  // done
  return resval;
}
//==================================================================================
// this function finds the first character in delims that is not preceeded by
// a backslash
size_t FindNonEscaped(const wxString& base, const wxString& delims, size_t pos) {
  // local variables
  size_t rpos;
  wxChar testc;
  // the finding loop
  while(true) {
    // looking for a character
    rpos = base.find_first_of(delims,pos);
    // speacial cases: faliure and first char
    if (rpos == wxString::npos) return rpos;
    if (rpos == 0) return 0;
    // checking to see if it after a backslash
    testc = base[rpos-1];
    if (testc == wxT('\\')) {
      pos = rpos+1;
      continue;
    }
    else return rpos;
  }
}
//==========================================================================
/* using the delim list, replaces any escaped delimiters with the corresponding
character. backslashes are also deleimters by default. they are treated first */
size_t UnEscape(wxString& base, const wxString& delims) {
  // local variables
  size_t rcount,dlen;
  wxString xfind, xreplace;
  // initial stuff
  rcount = 0;
  dlen = delims.length();
  // a special case
  if (dlen==0) return 0;
  // unescaping backslashes
  xfind = wxT("\\\\");
  rcount += base.Replace(xfind,wxT("\\"));
  // the replacement loop
  for (size_t i=0;i<dlen;i++) {
    // setting the search-repace values
    xreplace = delims[i];
    xfind = wxT('\\') + xreplace;
    // replacing
    rcount += base.Replace(xfind,xreplace);
  }
  // done
  return rcount;
}
//============================================================================
/* Versions of EscapeString and UnEscape that also replace newlines with ^.
Thus, ^ is also a default escaped character and should not be in delims */
//---------------------------------------------------------------------------
wxString EscapeNL(const wxString& in_base, const wxString& delims) {
  wxString ndelims,xresult=wxT("");
  if (in_base.length()==0) return xresult;
  // we prep the new delim line
  ndelims = wxT('^') + delims;
  xresult = EscapeString(in_base,ndelims);
  // past the special case, we start replaceing newlines
  xresult.Replace(wxT("\r\n"),wxT("^"));
  xresult.Replace(wxT("\n"),wxT("^"));
  // doing the rest
  return xresult;
}
//---------------------------------------------------------------------------
size_t UnEscapeNL(wxString& base, const wxString& delims) {
    // variables
    wxString ndelims;
    wxString nres=wxT("");
    size_t bas_len,rcount=0;
    size_t fpos, spos=0;
    // a simple case
    bas_len = base.length();
    if (bas_len==0) return 0;
    // we replace ^ with new lines first
    while(true) {
      // looking for ^
      fpos = FindNonEscaped(base,wxT("^"),spos);
      if (fpos == wxString::npos) break;
      // adding before to the result string, then the newline
      nres += base.substr(spos,fpos-spos);
      nres += wxT('\n');
      rcount++;
      // setting the new start
      spos = fpos+1;
      if (spos>=bas_len) break;
    }
    // after all the unescaped ^ have been replaced, we tack on the end
    if (spos<bas_len) nres += base.substr(spos,bas_len-spos);
    // unescaping the other characters
    ndelims = wxT('^') + delims;
    rcount += UnEscape(nres,ndelims);
    base = nres;
    return rcount;
}
//-------------------------------------------------------------------------
// converts a wxArrayString to a semi-colon delimited list
wxString StrArrToList(wxArrayString *source, const wxString& ichar) {
  wxString result;
  wxString buffer;
  // empty result cases
  if (source==NULL) return result;
  if ((source->Count())==0) return result;
  // constructing a non-empty result
  for (size_t i=0;i<(source->Count());i++) {
    buffer = source->Item(i);
    buffer.Replace(ichar,wxT("\\")+ichar);
    if (i!=(source->Count()-1)) result+= buffer + ichar;
    else result+=buffer;
  }
  // done
  return result;
}
//==========================================================================
// converts a delimited list into a wxArrayString
wxArrayString *ListToStrArr(const wxString& source, const wxString& ichar, bool trims) {
  wxString list,buffer;
  wxStringTokenizer *toker;
  wxArrayString *result;
  // checking to see if the source is empty
  list = source;
  Dtrim(list);
  if (list.IsEmpty()) return NULL;
  // tokenizing
  result = new wxArrayString();
  toker = new wxStringTokenizer(list,ichar,wxTOKEN_RET_EMPTY_ALL);
  while(toker->HasMoreTokens()) {
    buffer = toker->GetNextToken();
    if (buffer.length()!=0) {
      // handling escaped items
      while ((buffer[buffer.length()-1]==wxT('\\')) && (toker->HasMoreTokens())) {
        buffer[buffer.length()-1] = ichar.at(0);
        buffer += (toker->GetNextToken());
      }
    }
    if (trims) Dtrim(buffer);
    result->Add(buffer);
  }
  if ((result->Count())==0) {
    delete result;
    result = NULL;
  }
  delete toker;
  return result;
}
//===========================================================================
// splits a string at the location, not including that location
bool SplitString(const wxString& input, size_t point, wxString& out1, wxString& out2) {
  size_t slen;
  slen = input.length();
  if (point>=slen) return false;
  // getting part1
  out1 = input.substr(0,point);
  Dtrim(out1);
  // the second part
  out2 = input.substr(point+1,slen-(point+1));
  Dtrim(out2);
  // done
  return true;
}
//===========================================================================
// takes an input integer, treats it as a number of bytes, and returns a string
// describing this number (for example, 39721 becomes "  38.8 KiB  "
wxString ByteToSize(const uint64_t& numbytes, bool binary) {
  double newvalue;
  wxString res;
  int level = 0;
  // we compute the new value
  if (binary) {
    if (numbytes<1024) newvalue = numbytes;
    else if (numbytes<1048576) {
      newvalue = (numbytes/1024.0);
      level = 1;
    }
    else if (numbytes<1073741824) {
      newvalue = (numbytes/1048576.0);
      level = 2;
    }
    else {
      newvalue = (numbytes/1073741824.0);
      level = 3;
    }
  }
  else {
    if (numbytes<1000) newvalue = numbytes;
    else if (numbytes<1000000) {
      newvalue = (numbytes/1000.0);
      level = 1;
    }
    else if (numbytes<1000000000) {
      newvalue = (numbytes/1000000.00);
      level = 2;
    }
    else {
      newvalue = (numbytes/1000000000.0);
      level = 3;
    }
  }
  // next, we convert it to a string
  res = wxString::Format(wxT("%#6.1f"),newvalue);
  // we then add the units
  if (level==0) res+= wxT(" bytes");
  else if (level==1) {
    if (binary) res+= wxT(" KiB  ");
    else res += wxT(" KB   ");
  }
  else if (level==2) {
    if (binary) res+= wxT(" MiB  ");
    else res += wxT(" MB   ");
  }
  else if (level==3) {
    if (binary) res+= wxT(" GiB  ");
    else res += wxT(" GB   ");
  }
  // done
  return res;
}
//-------------------------------------------------------------------------
// returns a double rounded up (or down) to a certain precision after the decimal
double RoundTo(double inval, size_t prec) {
  double powgirl;
  // sanity check
  assert(prec<15);
  // we start...
  powgirl = pow(double(10),double(prec));
  inval *= powgirl;
  if (inval<0) inval -= 0.5;
  else inval += 0.5;
  if (inval>=0) inval = floor(inval);
  else inval = ceil(inval);
  inval /= powgirl;
  return inval;
}
//-------------------------------------------------------------------------
// takes an input double, and outputs it as a percentage
wxString ToPercent(const double& insrc, const size_t& prec) {
  wxString prec_str, format_str;
  wxString res;
  double pvalue;
  // basic checks
  assert(prec<15);
  assert(insrc>=0);
  assert(insrc<9000000000000.0);
  // taking the input value and preparing and rounding
  pvalue = insrc*100;
  // we prepare the format string
  prec_str = SInt2Str(prec);
  Dtrim(prec_str);
  format_str = wxT("%5.")+prec_str + wxT("f");
  // producing the result
  res = wxString::Format(format_str,pvalue);
  Dtrim(res);
  res += wxT("%");
  return res;
}
//--------------------------------------------------------------------------
// takes a pointer to a stopwatch, returns the to the nearest 10th of a sec.
wxString ElapsedTime(wxStopWatch* inwatch) {
  long militime;
  double tinsecs;
  double *ipart;
  double frac;
  assert(inwatch!=NULL);
  wxString result;
  // we start
  ipart = new double;
  militime = inwatch->Time();
  // we start transforming/ rounding the time
  tinsecs = double(militime)/100;
  frac = modf(tinsecs,ipart);
  tinsecs = (frac<0.5)?(*ipart):((*ipart)+1);
  delete ipart;
  tinsecs /= 100;
  result.Printf(wxT("%f.1"),tinsecs);
  result += wxT("s.");
  return result;
}
//-------------------------------------------------------------------
// case insensetive compare strings
int jfCI_CompareString(const wxString& first, const wxString& second) {
  wxString xf1 = first.Lower();
  wxString xs1 = second.Lower();
  if (xf1<xs1) return -1;
  if (xf1==xs1) return 0;
  else return 1;
}
//****************************************************************************
