/*****************************************************************************
Name    : utils1.h
Basic   : Defclares some utility functions and things used in the other files
Author  : John Q Metro
Started : March 17, 2009
Updated : July 23, 2015
Notes   : mostly taken from the earlier jaffis_utils.h and jaffis_utils2.h

******************************************************************************/
#ifndef WX_PRECOMP
  #include <wx/wx.h>
#else
#include "wx/wxprec.h"
#endif

#ifdef __BORLANDC__
    #pragma hdrstop
#endif
//-------------------------------------------------
#ifndef UTILS1_H_INCLUDED
#define UTILS1_H_INCLUDED
#endif // UTILS1_H_INCLUDED
//------------------------------------------------
#include <wx/stopwatch.h>
#if defined(_MSC_VER)
  #define uint64_t unsigned __int64
#else
  #include <inttypes.h>
#endif
//*****************************************************************************
// constants
extern const wxString EXPR_DELIMS;
//*****************************************************************************
// two way trimming
void Dtrim(wxString& input);
//-------------------------------------------------------------------------
// converts an integer to a string
wxString IInt2Str(const wxInt32& input);
// converts an unsigned integer to a string
wxString UInt2Str(const wxUint32& input);
// converts size_t to a string
wxString SInt2Str(const size_t& input);
// converts a double to a string
wxString Doubl2Str(const double& input);
// converts a boolean to a string
wxString Bool2Str(const bool& inval);
// converts a 64bit unsigned value to a string
wxString UInt64_2Str(const uint64_t& inval);
//------------------------------------------------------------------------
// the wxString conversion is not bad, except it uses a pointer. here is a single line function
bool Str2ULong(const wxString& inval, unsigned long& outval, bool hex=false);
// a version of the above that handles commas
bool Str2ULongC(const wxString& inval, unsigned long& outval);
// handles values like '1.9K'
bool Count2ULong(const wxString& inval, unsigned long& outval);
//-------------------------------------------------------------------------
/* takes an input string. if the string is "true" or "false", returns TRUE
and also passes back which in the reference parameter */
bool BoolFromStr(const wxString& in_src, bool& result);
//------------------------------------------------------------------------
// to shorten things, returns true if buffer conatins f1 or f2
bool ContainsAlt(const wxString& buffer, wxString f1, wxString f2);
//-------------------------------------------------------------------------
/* Takes a string and returns the result with the specified chars in delim
preceeded with a backslash. Backslashes are also escapsed. Do not repeat
characters in delims. */
wxString EscapeString(const wxString& in_base, const wxString& delims);
//-------------------------------------------------------------------------------
// this function finds the first character in delims that is not preceeded by
// a backslash
size_t FindNonEscaped(const wxString& base, const wxString& delims, size_t pos = 0);
//-------------------------------------------------------------------------------
/* using the delim list, replaces any escaped delimiters with the corresponding
character. backslashes are also deleimters by default. they are treated first */
size_t UnEscape(wxString& base, const wxString& delims);
//----------------------------------------------------------------------------
/* Versions of EscapeString and UnEscape that also replace newlines with ^.
Thus, ^ is also a default escaped character and should not be in delims */
wxString EscapeNL(const wxString& in_base, const wxString& delims);
size_t UnEscapeNL(wxString& base, const wxString& delims);
//------------------------------------------------------------------------
// converts a wxArrayString to a delimted list
wxString StrArrToList(wxArrayString *source, const wxString& ichar);
//------------------------------------------------------------------------
// converts a delimited list into a wxArrayString
wxArrayString *ListToStrArr(const wxString& source, const wxString& ichar, bool trims = true);
//------------------------------------------------------------------------
// splits a string at the location, not including that location
bool SplitString(const wxString& input, size_t point, wxString& out1, wxString& out2);
//-------------------------------------------------------------------------
// takes an input integer, treats it as a number of bytes, and returns a string
// describing this number (for example, 39721 becomes "  38.8 KiB  "
wxString ByteToSize(const uint64_t& numbytes, bool binary=true);
//-------------------------------------------------------------------------
// returns a double rounded up (or down) to a certain precision after the decimal
double RoundTo(double inval, size_t prec);
//-------------------------------------------------------------------------
// takes an input double, and outputs it as a percentage
wxString ToPercent(const double& insrc, const size_t& prec);
//-------------------------------------------------------------------------
// takes a pointer to a stopwatch, returns the to the nearest 10th of a sec.
wxString ElapsedTime(wxStopWatch* inwatch);
//-------------------------------------------------------------------------
// case insensetive compare strings
int jfCI_CompareString(const wxString& first, const wxString& second);

//*****************************************************************************
