/*****************************************************************************
Name    : utils2.cpp
Basic   : Defines some more utility functions and things used in the other files
Author  : John Q Metro
Started : May 13, 2009
Updated : July 23, 2015
Notes   :

******************************************************************************/
#ifndef UTILS2_H_INCLUDED
  #include "utils2.h"
#endif // UTILS2_H_INCLUDED
#ifndef BASEOBJ_H_INCLUDED
    #include "../objs/baseobj.h"
#endif // BASEOBJ_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef UTILS3_H_INCLUDED
  #include "utils3.h"
#endif // UTILS3_H_INCLUDED

#include <wx/tokenzr.h>
//*****************************************************************************
// constructors
//---------------------------------------------------------------
jfLineParse::jfLineParse() {
  qres = NULL;
}
//---------------------------------------------------------------
jfLineParse::jfLineParse(const wxString& instr) {
  qres = NULL;
  SetString(instr);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// basic info
//---------------------------------------------------------------
size_t jfLineParse::Num() const {
  if (qres==NULL) return 0;
  else return qres->GetCount();
}
//---------------------------------------------------------------
bool jfLineParse::IsEmpt() const {
  return (qres==NULL);
}
//---------------------------------------------------------------
bool jfLineParse::NNotX(const size_t& X) const {
  return (Num()!=X);
};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting values at index
wxString jfLineParse::UnEscStr(size_t index) const {
  wxString buffer;
  // starting
  assert(index<Num());
  buffer = (*qres)[index];
  UnEscapeNL(buffer,IO_DELIMS);
  return buffer;
}
//---------------------------------------------------------------
// getting values at index
wxString jfLineParse::UnEscStrE(size_t index) const {
  wxString buffer;
  // starting
  assert(index<Num());
  buffer = (*qres)[index];
  UnEscapeNL(buffer,EXPR_DELIMS);
  Dtrim(buffer);
  return buffer;
}
//---------------------------------------------------------------
wxString jfLineParse::Get(size_t index) const {
  assert(index<Num());
  return (*qres)[index];
}
//------------------------------------------------------
wxString jfLineParse::GetT(size_t index) const {
  wxString rval;
  rval = Get(index);
  Dtrim(rval);
  return rval;
}
//---------------------------------------------------------------
// we try and get the boolean value at the index
bool jfLineParse::BoolVal(size_t index, bool& outval) const {
  bool rval;
  //starting...
  assert(index<Num());
  rval = BoolFromStr((*qres)[index],outval);
  return rval;
}
//---------------------------------------------------------------
// tries to get the integer value at the index
bool jfLineParse::SIntVal(size_t index, size_t& outval) const {
  unsigned long *tval;
  bool res;
  wxString buffer;
  // starting
  assert(index<Num());
  buffer = (*qres)[index];
  tval = new unsigned long;
  res = buffer.ToULong(tval);
  if (res) outval = (*tval);
  delete tval;
  return res;
}
//---------------------------------------------------------------
// similar to SIntVal, but also returns an error if the value >= ubound
bool jfLineParse::SBoundVal(size_t index, const size_t& ubound, size_t& outval, wxString& oerr) const {
  unsigned long oval;
  wxString buffer;
  // starting
  assert(index<Num());
  buffer = (*qres)[index];
  if (!Str2ULong(buffer,oval)) {
    oerr = wxT("is not a number!");
    return false;
  }
  else if (oval>=ubound) {
    oerr = wxT("is too high!");
    return false;
  }
  outval = oval;
  return true;
}
//------------------------------------------------------------
bool jfLineParse::UIntVal(size_t index, wxUint32& outval) const {
	unsigned long *tval;
	bool res;
	wxString buffer;
	// starting
	assert(index<Num());
	buffer = (*qres)[index];
	tval = new unsigned long;
	res = buffer.ToULong(tval);
	if (res) outval = (*tval);
	delete tval;
	return res;
}
//------------------------------------------------------------
bool jfLineParse::IIntVal(size_t index, int& outval) const {
	long *tval;
	bool res;
	wxString buffer;
	// starting
	assert(index<Num());
	buffer = (*qres)[index];
	tval = new long;
	res = buffer.ToLong(tval);
	if (res) outval = (*tval);
	delete tval;
	return res;
}
//------------------------------------------------------------
bool jfLineParse::DateVal(size_t index, wxDateTime& outval) const {
  wxString buffer = UnEscStr(index);
  if (NULL==outval.ParseFormat(buffer,wxT("%m-%d-%y"))) return false;
  else return true;
}
//--------------------------------------------------------------
// returns a char in a certain range only
bool jfLineParse::CharVal(size_t index, wxChar& outval, const wxString& oallow) const {
  wxString buffer = UnEscStr(index);
  if (buffer.length()!=1) return false;
  if (!oallow.IsEmpty()) {
    if (!oallow.Contains(buffer[0])) return false;
  }
  outval = buffer[0];
  return true;
}
//--------------------------------------------------------------
bool jfLineParse::UInt64Val(size_t index, uint64_t& outval) const {
	wxULongLong_t *tval;
	bool res;
	wxString buffer;
	// starting
	assert(index<Num());
	buffer = (*qres)[index];
	tval = new (wxULongLong_t);
	res = buffer.ToULongLong(tval);
	if (res) outval = (*tval);
	delete tval;
	return res;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// finding a value after a tag
bool jfLineParse::ReturnByTag(wxString tag, wxString& value) const {
  // special cases
  if (qres==NULL) return false;
  if (qres->Count()<2) return false;
  // variables
  int rval;
  rval = qres->Index(tag);
  // checking for bad results
  if (rval==wxNOT_FOUND) return false;
  if (rval==(qres->Count()-1)) return false;
  // okay, go ahead
  value = (*qres)[rval+1];
  return true;
}
//---------------------------------------------------------------
bool jfLineParse::ReturnByTagS(wxString tag, size_t& value) const {
  // variables
  unsigned long uresult;
  wxString intval;
  // getting the value
  if (!ReturnByTag(tag,intval)) return false;
  // converting
  if (!Str2ULong(intval,uresult)) return false;
  // returning
  value = uresult;
  return true;
}
//---------------------------------------------------------------
bool jfLineParse::ReturnByTagB(wxString tag, bool& value) const {
  wxString intval;
  // getting the value
  if (!ReturnByTag(tag,intval)) return false;
  // converting
  if (!BoolFromStr(intval,value)) return false;
  // returning
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxArrayString* jfLineParse::GetMultiple(size_t index, size_t item_count) const {
  // the checks
  assert(index<Num());
  assert(item_count!=0);
  assert((index+item_count-1)<Num());
  // local variables
  wxArrayString* result;
  // we start
  result = new wxArrayString();
  for (size_t loopc=index;loopc<(index+item_count);loopc++) result->Add((*qres)[loopc]);
  return result;
}
//----------------------------------------------------------------
wxArrayString* jfLineParse::UnEscMultiple(size_t index, size_t item_count) const {
  // the checks
  assert(index<Num());
  assert(item_count!=0);
  assert((index+item_count-1)<Num());
  // local variables
  wxArrayString* result;
  // we start
  result = new wxArrayString();
  for (size_t loopc=index;loopc<(index+item_count);loopc++) result->Add(UnEscStr(loopc));
  return result;
}
//----------------------------------------------------------------
wxArrayString* jfLineParse::GetRest(size_t index) const {
  assert(index<Num());
  wxArrayString* result;
  // we start
  result = new wxArrayString();
  for (size_t loopc=index;loopc<Num();loopc++) result->Add((*qres)[loopc]);
  return result;
}
//----------------------------------------------------------------
wxArrayString* jfLineParse::UnEscRest(size_t index) const {
  assert(index<Num());
  wxArrayString* result;
  // we start
  result = new wxArrayString();
  for (size_t loopc=index;loopc<Num();loopc++) result->Add(UnEscStr(loopc));
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// changing the value
size_t jfLineParse::SetString(const wxString& instr) {
  if (qres!=NULL) delete qres;
  qres = ListToStrArr(instr,wxT(';'));
  return Num();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructor
jfLineParse::~jfLineParse() {
  if (qres!=NULL) delete qres;
}
//*****************************************************************************
/* sort of the opposite of jfLineParse, mean for inserting values in the string.
overrides some of the operators already there... we use ';' as separators and escape
string input... */
//-----------------------------------------------------------------------------
jfOutString& jfOutString::operator<< (const wxString& toadd) {
  if (!IsEmpty()) {
    Append(wxT(";"));
  }
  Append(EscapeNL(toadd,IO_DELIMS));
  return *this;
}
//-----------------------------------------------------------------------------
jfOutString& jfOutString::operator<< (const wxChar* toadd) {
  if (!IsEmpty()) {
    Append(wxT(";"));
  }
  Append(EscapeNL(toadd,IO_DELIMS));
  return *this;
}
//-----------------------------------------------------------------------------
jfOutString& jfOutString::operator<< (const bool& toadd) {
  if (!IsEmpty()) {
    Append(wxT(";"));
  }
  if (toadd) Append(wxT("true"));
  else Append(wxT("false"));
  return *this;
}
//-----------------------------------------------------------------------------
jfOutString& jfOutString::operator<< (const size_t toadd) {
  if (!IsEmpty()) {
    Append(wxT(";"));
  }
  Append(SInt2Str(toadd));
  return *this;
}
//-----------------------------------------------------------------------------
jfOutString& jfOutString::operator<< (const int toadd) {
  if (!IsEmpty()) {
    Append(wxT(";"));
  }
  Append(IInt2Str(toadd));
  return *this;
}
//---------------------------------------------------------------------------
jfOutString& jfOutString::operator<< (const wxDateTime& toadd) {
  if (!IsEmpty()) {
    Append(wxT(";"));
  }
  Append(EscapeNL(toadd.Format(wxT("%m-%d-%y")),IO_DELIMS));
  return *this;
}
//******************************************************************************
// converts a wxArrayString to a delimted list, the items are all escaped first
wxString StrArrToEscList(wxArrayString *source, const wxString& delims) {
  const wxString ichar = wxT(';');
  wxString result;
  wxString buffer;
  // empty result cases
  if (source==NULL) return result;
  if ((source->Count())==0) return result;
  // constructing a non-empty result
  for (size_t i=0;i<(source->Count());i++) {
    buffer = source->Item(i);
    buffer = EscapeString(buffer,delims);
    buffer.Replace(ichar,wxT("\\")+ichar);
    result+= buffer + ichar;
  }
  // done
  return result;
}
//----------------------------------------------------------------------------
// the reverse of the above, converts an escaped, delimited list into a wxArrayString
wxArrayString *EscListToStrArr(const wxString& source, const wxString& delims) {
  const wxString ichar = wxT(';');
  wxString list,buffer;
  wxStringTokenizer *toker;
  wxArrayString *result;
  // checking to see if the source is empty
  list = source;
  Dtrim(list);
  if (list.IsEmpty()) return NULL;
  // tokenizing
  result = new wxArrayString();
  toker = new wxStringTokenizer(list,ichar);
  while(toker->HasMoreTokens()) {
    buffer = toker->GetNextToken();
    // handling escaped items
    while ((buffer[buffer.length()-1]==wxT('\\')) && (toker->HasMoreTokens())) {
      buffer[buffer.length()-1] = ichar.at(0);
      buffer += (toker->GetNextToken());
    }
    UnEscapeNL(buffer,delims);
    result->Add(buffer);
  }
  if ((result->Count())==0) {
    delete result;
    result = NULL;
  }
  delete toker;
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// produces a wxArrayString from the contents of a wxTextCtrl
wxArrayString* TextCtrl2List(const wxTextCtrl* inval, bool trimws, bool noblank) {
  // local variables
  wxString buffer;
  size_t loopc,lcount;
  wxArrayString* result;
  // we start...
  assert(inval!=NULL);
  lcount = inval->GetNumberOfLines();
  assert(lcount>0);
  result = new wxArrayString();
  // the line extracting loop
  for (loopc=0;loopc<lcount;loopc++) {
    buffer = inval->GetLineText(loopc);
    if (trimws) Dtrim(buffer);
    if (buffer.IsEmpty() && noblank) continue;
    result->Add(buffer);
  }
  // final checking
  if (result->Count()==0) {
    delete result;
    return NULL;
  }
  else return result;
}
//****************************************************************************
// constructor
jfFileReader::jfFileReader(wxString infname) {
  thefile = NULL;
  fname = infname;
  lineindex = -1;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file stuff
//------------------------------
bool jfFileReader::OpenFile() {
  CloseFile();
  thefile = GetOpenFile(fname,false);
  if (thefile==NULL) {
    err = wxT("Could not open file!");
    return false;
  }
  lineindex = -1;
  thefile->Open();
  return true;
}
//------------------------------
bool jfFileReader::CloseFile() {
  if (thefile==NULL) return false;
  thefile->Close();
  delete thefile;
  thefile = NULL;
  return true;
}
//-------------------------------------
size_t jfFileReader::LineIndex() const {
  return lineindex;
}
//-------------------------
bool jfFileReader::BeforeEnd(size_t bias) const {
  if (thefile==NULL) return false;
  size_t lcount = thefile->GetLineCount();
  if (lcount==0) return false;
  return (lineindex<((long)lcount-(long)bias));
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file reading
//------------------------------
bool jfFileReader::ReadLine(wxString& outline, const wxString& finname) {
  if (!finname.IsEmpty()) func_name = finname;
  return StartRead(outline);
}
//------------------------------
bool jfFileReader::ReadParseSLine(size_t fcount, const wxString& finname) {
  if (!finname.IsEmpty()) func_name = finname;
  wxString oline;
  bool bres = StartRead(oline);
  if (!bres) return false;
  if (oline.Left(3)!=wxT("###")) {
    return BuildError(wxT("Line does not start with ###! ")+oline);
  }
  else {
    oline = oline.Mid(3);
    return SetParse(oline,fcount);
  }
}
//------------------------------
bool jfFileReader::ReadParseLine(size_t fcount, const wxString& finname) {
  if (!finname.IsEmpty()) func_name = finname;
  wxString oline;
  if (!StartRead(oline)) return false;
  return SetParse(oline,fcount);
}
//--------------------------------------------------
bool jfFileReader::ReadParseLine(size_t fmin, size_t fmax, const wxString& finname) {
  assert(fmin<=fmax);
  if (!finname.IsEmpty()) func_name = finname;
  wxString oline;
  if (!StartRead(oline)) return false;
  lp.SetString(oline);
  if (lp.Num()<fmin) return BuildError(wxT("The line has less than ") + SInt2Str(fmin) + wxT(" fields!"));
  if (lp.Num()<fmin) return BuildError(wxT("The line has more than ") + SInt2Str(fmax) + wxT(" fields!"));
  return true;
}
//------------------------------
bool jfFileReader::ReadLast(const wxString& finname) {
  if (!finname.IsEmpty()) func_name = finname;
  wxString oline;
  if (!StartRead(oline)) return false;
  if (oline!=wxT("###")) return BuildError(wxT("Line is not ###! is ") + oline);
  else return true;
}
//------------------------------
bool jfFileReader::GetType(wxString& outtype, const wxString& finname) {
  if (!finname.IsEmpty()) func_name = finname;
  wxString oline;
  if (!StartRead(oline)) return false;
  if (oline.Left(3)!=wxT("###")) {
    lineindex--;
    if (lineindex!=-1) thefile->GoToLine(lineindex);
    return BuildError(wxT("Line does not start with ###!"));
  }
  else {
    oline = oline.Mid(3);
    lp.SetString(oline);
    lineindex--;
    if (lineindex!=-1) thefile->GoToLine(lineindex);
    if (lp.Num()==0) return BuildError(wxT("No Fields!"));
    outtype = lp.UnEscStr(0);
    if (outtype.IsEmpty()) return BuildError(wxT("Type is empty!"));
    return true;
  }
}
//------------------------------
bool jfFileReader::ReadUnEsc(wxString& outline, const wxString& finname) {
  if (!finname.IsEmpty()) func_name = finname;
  if (!StartRead(outline)) return false;
  UnEscapeNL(outline,IO_DELIMS);
  return true;
}
//------------------------------
bool jfFileReader::ReadULong(unsigned long& outnum, const wxString& finname) {
  if (!finname.IsEmpty()) func_name = finname;
  wxString outline;
  if (!StartRead(outline)) return false;
  if (!Str2ULongC(outline,outnum)) {
    return BuildError(wxT("Line is not a valid unsigned integer!"));
  }
  else return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfFileReader::SkipLine(const wxString& finname) {
  if (!finname.IsEmpty()) func_name = finname;
  // cases
  if (lineindex==-1) {
    if (thefile->GetLineCount()==0) {
      return BuildError(wxT("No Lines to skip!"));
    }
    else lineindex=0;
  }
  if (thefile->Eof()) {
    return BuildError(wxT("Cannot skip to a line after the last!"));
  }
  else {
    lineindex++;
    thefile->GetNextLine();
  }
  return true;
}
//----------------------------------
bool jfFileReader::Backup(const wxString& finname) {
  if (!finname.IsEmpty()) func_name = finname;
  // special cases
  if (lineindex==-1) return BuildError(wxT("Cannot backup anymore!"));
  else {
    lineindex--;
    if (lineindex!=-1) thefile->GoToLine(lineindex);
    return true;
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// other methods
//------------------------------
bool jfFileReader::BuildError(wxString inerr) {
  err = inerr + wxT(" IN ") + func_name + wxT(", LINE: ") + IInt2Str(lineindex);
  err += wxT(", FILE: ") + fname;
  return false;
}
//------------------------------
jfFileReader::~jfFileReader() {
  if (thefile!=NULL) {
    thefile->Close();
    delete thefile;
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal methods
//------------------------------
bool jfFileReader::StartRead(wxString& outline) {
  assert(thefile!=NULL);
  // special case 1
  if (thefile->GetLineCount()==0) {
    return BuildError(wxT("The file is empty!"));
  }
  // special case 2
  if (lineindex==-1) {
    outline = thefile->GetFirstLine();
    lineindex = 0;
    return true;
  }
  // the end
  if (thefile->Eof()) {
    return BuildError(wxT("Unexpected end of file!"));
  }
  // general case
  lineindex++;
  outline = thefile->GetNextLine();
  return true;
}
//------------------------------
bool jfFileReader::SetParse(const wxString& inval, size_t check) {
  assert(check!=0);
  lp.SetString(inval);
  if (lp.NNotX(check)) {
    return BuildError(wxT("The line does not have ") + SInt2Str(check) + wxT(" fields!"));
  }
  else return true;
}
//*****************************************************************************
