/*****************************************************************************
Name    : utils2.h
Basic   : Declares some more utility functions and things used in the other files
Author  : John Q Metro
Started : May 13, 2009
Updated : July 23, 2015 (added UInt64Val to jfLineParse)
Notes   :

******************************************************************************/
#ifndef UTILS2_H_INCLUDED
#define UTILS2_H_INCLUDED
#endif // UTILS2_H_INCLUDED
//---------------------------------------------------------------------------
#ifndef UTILS1_H_INCLUDED
  #include "utils1.h"
#endif // UTILS1_H_INCLUDED

#include <wx/textfile.h>
//*****************************************************************************
// a class used to help in parsing input
class jfLineParse : public wxObject {
  public:
    // constructors
    jfLineParse();
    jfLineParse(const wxString& instr);
    // basic info
    size_t Num() const;
    bool IsEmpt() const;
    bool NNotX(const size_t& X) const;
    // getting values at index
    wxString UnEscStr(size_t index) const;
    wxString UnEscStrE(size_t index) const;
    wxString Get(size_t index) const;
    wxString GetT(size_t index) const;
    bool BoolVal(size_t index, bool& outval) const;
    bool SIntVal(size_t index, size_t& outval) const;
    bool SBoundVal(size_t index, const size_t& ubound, size_t& outval, wxString& oerr) const;
    bool UIntVal(size_t index, wxUint32& outval) const;
    bool IIntVal(size_t index, int& outval) const;
    bool DateVal(size_t index, wxDateTime& outval) const;
    bool CharVal(size_t index, wxChar& outval, const wxString& oallow=wxEmptyString) const;
    bool UInt64Val(size_t index, uint64_t& outval) const;
    // finding a value after a tag
    bool ReturnByTag(wxString tag, wxString& value) const;
    bool ReturnByTagS(wxString tag, size_t& value) const;
    bool ReturnByTagB(wxString tag, bool& value) const;
    // getting multiple values
    wxArrayString* GetMultiple(size_t index, size_t item_count) const;
    wxArrayString* UnEscMultiple(size_t index, size_t item_count) const;
    wxArrayString* GetRest(size_t index) const;
    wxArrayString* UnEscRest(size_t index) const;
    // changing the value
    size_t SetString(const wxString& instr);
    // destructor
    ~jfLineParse();
  private:
    wxArrayString *qres;
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* sort of the opposite of jfLineParse, mean for inserting values in the string.
overrides some of the operators already there... we use ';' as separators and escape
string input... */
class jfOutString : public wxString {
  public:
    jfOutString& operator<< (const wxString& toadd);
    jfOutString& operator<< (const wxChar* toadd);
    jfOutString& operator<< (const bool& toadd);
    jfOutString& operator<< (const size_t toadd);
    jfOutString& operator<< (const int toadd);
    jfOutString& operator<< (const wxDateTime& toadd);
  protected:
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// converts a wxArrayString to a delimted list, the items are all escaped first
wxString StrArrToEscList(wxArrayString *source, const wxString& delims);
// the reverse of the above, converts an escaped, delimited list into a wxArrayString
wxArrayString *EscListToStrArr(const wxString& source, const wxString& delims);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// produces a wxArrayString from the contents of a wxTextCtrl
wxArrayString* TextCtrl2List(const wxTextCtrl* inval, bool trimws=true, bool noblank=true);

//****************************************************************************
class jfFileReader : public wxObject {
  public:
    jfLineParse lp;
    wxString err;
    // constructor
    jfFileReader(wxString infname);
    // file stuff
    bool OpenFile();
    bool CloseFile();
    size_t LineIndex() const;
    bool BeforeEnd(size_t bias=0) const;
    // file reading
    bool ReadLine(wxString& outline, const wxString& finname = wxEmptyString);
    bool ReadParseSLine(size_t fcount, const wxString& finname = wxEmptyString);
    bool ReadParseLine(size_t fcount, const wxString& finname = wxEmptyString);
    bool ReadParseLine(size_t fmin, size_t fmax, const wxString& finname = wxEmptyString);
    bool ReadLast(const wxString& finname = wxEmptyString);
    bool GetType(wxString& outtype, const wxString& finname = wxEmptyString);
    bool ReadUnEsc(wxString& outline, const wxString& finname = wxEmptyString);
    bool ReadULong(unsigned long& outnum, const wxString& finname = wxEmptyString);
    // not really reading
    bool SkipLine(const wxString& finname = wxEmptyString);
    bool Backup(const wxString& finname = wxEmptyString);
    // other methods
    bool BuildError(wxString inerr);
    ~jfFileReader();
  protected:
    // internal methods
    bool StartRead(wxString& outline);
    bool SetParse(const wxString& inval, size_t check);
    // internal data
    wxTextFile* thefile;
    wxString fname;
    wxString func_name;
    int lineindex;
};
//****************************************************************************
