/*****************************************************************************
Name    : utils3.h
Basic   : Declares some more utility functions and things used in the other files
Author  : John Q Metro
Started : January 7, 2011
Updated : July 6, 2015
Notes   :

******************************************************************************/
#ifndef UTILS3_H_INCLUDED
  #include "utils3.h"
#endif // UTILS3_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "logging.h"
#endif // LOGGING_H_INCLUDED
//-----------------------------------------
#include <wx/dir.h>
#include <wx/filename.h>
//****************************************************************************
const wxString query_encode_keep = wxT("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789.-_~");
//============================================================================
bool ToFile(wxString indata, wxString filename) {
  // variables
  wxTextFile* result;
  // creating or opening the file
  result = new wxTextFile(filename);
  if (result->Exists()) {
    result->Open();
    result->Clear();
  }
  else result->Create();
  // adding
  result->AddLine(indata);
  // closing and writing
  result->Write();
  result->Close();
  delete result;
  return true;
}
//--------------------------------------------------------------------
// reads data from a file
bool FromFile(wxString& outdata, wxString filename) {
  // variables
  wxTextFile* infile;
  size_t lcount, lindex;
  wxString cresult;
  // creating or opening the file
  infile = new wxTextFile(filename);
  if (infile->Exists()) {
    // loading the file data
    infile->Open();
    lcount = infile->GetLineCount();
    for (lindex = 0; lindex<lcount; lindex++) {
      cresult += infile->GetLine(lindex);
      if (lindex!=(lcount-1)) cresult += wxT("\n");
    }
    // done
    infile->Close();
    delete infile;
    outdata = cresult;
    return true;
  }
  else {
    delete infile;
    return false;
  }
}
//----------------------------------------------------------------------------
bool TrimTextFile(wxString filename, size_t lcount) {
  // constants
  const wxString fname=wxT("TrimTextFile");
  // variables
  wxTextFile* result;
  size_t clinec;
  int line_loop, line_diff;
  // Opening the file
  /**/jfXLogMessage(fname,1,filename);
  result = new wxTextFile(filename);
  if (result->Exists()) result->Open(wxConvISO8859_1);
  else return false;
  /**/jfXLogMessageS(fname,2,lcount);
  // getting the line count and calculating the diff
  clinec = result->GetLineCount();
  /**/jfXLogMessageS(fname,3,clinec);
  line_diff = (int)clinec - (int)lcount;
  /**/jfXLogMessageI(fname,4,line_diff);
  if (line_diff<=0) {
    result->Close();
    return false;
  }
  /**/jfXLogMessage(fname,5);
  // if there is a positive diff, we loop remove the lines from the start
  for (line_loop=0;line_loop<line_diff;line_loop++) {
    result->RemoveLine(line_loop);
  }
  /**/jfXLogMessage(fname,6);
  // write, close, and exit
  result->Write(wxTextFileType_None,wxConvISO8859_1);
  result->Close();
  /**/jfXLogMessage(fname,7);
  delete result;
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
const wxString UFNAME_DELIMS = wxT(" :;.,\'\\/\"?!@#$%^&");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// produces a version of the string with some characters replaced by _
wxString UBReplace(wxString filename, wxString delimchars) {
  // variables
	size_t dclen = delimchars.length();
	size_t flen = filename.length();
	size_t dloop;
	// special cases
	if ((dclen==0) || (flen==0)) return filename;
	// replacements
	for (dloop=0;dloop<dclen;dloop++) filename.Replace(wxString(delimchars[dloop]),wxT("_"));
	filename.Replace(wxT("__"),wxT("_"));
	// done
	return filename;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o creator
wxTextFile* GetOpenFile(wxString filename, bool write) {
  // variables
  wxTextFile* result;
  size_t lcount;
  bool qval;
  // starting
  result = new wxTextFile(filename);
  // two main branches
  if (write) {
    if (result->Exists()) {
      result->Open();
      result->Clear();
    }
    else {
      qval = result->Create();
      result->Open();
      assert(qval);
    }
  }
  else {
    if (!result->Exists()) {
      delete result;
      return NULL;
    }
    result->Open();
    lcount = result->GetLineCount();
    if (lcount==0) {
      result->Close();
      delete result;
      return NULL;
    }
  }
  // done
  return result;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// more file i/o
//-------------------------------------------------
bool AppendAS2File(wxArrayString* indata, wxTextFile* target) {
  // the usual checks
  assert(indata!=NULL);
  assert(target!=NULL);
  // assert(target->IsOpened());
  // variables
  size_t lcount, lindex;
  // appending
  lcount = indata->Count();
  for(lindex = 0; lindex<lcount; lindex++) {
      target->AddLine((*indata)[lindex]);
  }
  return true;
}
//-------------------------------------------------
bool AppendFile2AS(wxArrayString* target, wxTextFile* source) {
  // the usual checks
  assert(target!=NULL);
  assert(source!=NULL);
  assert(source->IsOpened());
  // variables
  size_t lindex,lcount,lstart;
  // we start
  lcount = source->GetLineCount();
  lstart = source->GetCurrentLine();
  if (lstart>=lcount) return false;
  for (lindex=lstart;lindex<lcount;lindex++) {
    target->Add(source->GetLine(lindex));
  }
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// wxArrayString To File and vice versa
//------------------------------------------------
bool ToFileAS(wxArrayString* indata, wxString filename) {
  // variables
  wxTextFile* result;
  bool rval;
  // special case
  if (indata==NULL) return false;
  // creating or opening the file
  result = GetOpenFile(filename,true);
  // doing
  rval = AppendAS2File(indata,result);
  assert(rval);
  // closing and writing
  result->Write();
  result->Close();
  delete result;
  return true;
}
//---------------------------------------------------
wxArrayString* FromFileAS(wxString filename) {
  // variables
  wxTextFile* infile;
  wxArrayString* result;
  bool rval;
  // we start
  infile = GetOpenFile(filename,false);
  if (infile==NULL) return NULL;
  // the file exists
  result = new wxArrayString();
  // doing
  rval = AppendFile2AS(result,infile);
  assert(rval);
  // finishing
  infile->Close();
  delete infile;
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Query encoding a string for UTL query usage
wxString QueryEncode(const wxString invalue, bool encodetilde) {
  // special case
  if (invalue.IsEmpty()) return wxEmptyString;
  // variables
  wxChar atchar;
  size_t loopc, strlen;
  wxString corestring, result;
  unsigned short int qval;
  // prep
  strlen = invalue.length();
  corestring = wxString((const char*)invalue.mb_str(wxConvUTF8),wxConvISO8859_1);
  Dtrim(corestring);
  // the encoding loop
  for (loopc=0;loopc<strlen;loopc++) {
    atchar = corestring[loopc];
    if (atchar==wxT('~')) {
      if (encodetilde) result += wxT("%7E");
      else result += wxT('~');
    }
    else if (query_encode_keep.Find(atchar)!=wxNOT_FOUND) result += atchar;
    else if (atchar==wxT(' ')) result += wxT('+');
    else {
      qval = atchar;
      result += wxString::Format(wxT("%%%hX"),qval);
    }
  }
  // done
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++
// deletes a directory and it's contents
bool DeleteDirectoryTree(wxString dir_to_delete) {
  // variables
  wxString cfile,wfile;
  wxDir* fdir = new wxDir(dir_to_delete);
  bool opx, dir_empty;
  // starting
  opx = fdir->GetFirst(&cfile);
  wfile = dir_to_delete + wxFileName::GetPathSeparator()+ cfile;
  // the deletion loop
  while(opx) {
    if(wxFileName::FileExists(wfile)) wxRemoveFile(wfile);
    if( wxFileName::DirExists(wfile)) DeleteDirectoryTree(wfile);
    opx = fdir->GetNext(&cfile);
    wfile = dir_to_delete + wxFileName::GetPathSeparator()+ cfile;
  }
  if(!(fdir->HasFiles() || fdir->HasSubDirs())) dir_empty = true;
  delete fdir;
  if(dir_empty){
    bool error = wxRmdir(dir_to_delete);
    return true;
  }
  else return false;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++
/* Annoyingly, wxWidgets provides no simple method to get a complete list of all
files and subdirectories in a directory. (wxDir::GetAllFiles never returns directories).
This function does that, the paths are complete and directories are terminated with the
path separator. */

// traverse class
class jfCustomDirTraverser : public wxDirTraverser {
  public:
    jfCustomDirTraverser(wxArrayString& files) : m_files(files) { }

    virtual wxDirTraverseResult OnFile(const wxString& filename) {
      m_files.Add(filename);
      return wxDIR_CONTINUE;
    }

    virtual wxDirTraverseResult OnDir(const wxString& dirname) {
      m_files.Add(dirname+wxFileName::GetPathSeparator());
      return wxDIR_CONTINUE;
    }

  private:
    wxArrayString& m_files;
};

// the main function
bool GetDirectoryTreeList(const wxString indir, wxArrayString& outdata) {
  // special case
  if (!wxDir::Exists(indir)) return false;
  // variables for more general case
  jfCustomDirTraverser *xtrav;
  wxDir* thedir;
  // doing it
  xtrav = new jfCustomDirTraverser(outdata);
  thedir = new wxDir(indir);
  thedir->Traverse(*xtrav);
  // cleaning up
  delete thedir;
  delete xtrav;
  outdata.Sort();
  // done
  return true;
}

//****************************************************************************
