/******************************************************************************
* Name:        auth_coll.cpp
* Author :     John Q Metro
* Purpose :    Collections involving author data
* Created:     January 19, 2011
* Updated:     January 6, 2014
******************************************************************************/
#ifndef AUTH_COLL_H_INCLUDED
  #include "auth_coll.h"
#endif // AUTH_COLL_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef UTILS3_H_INCLUDED
  #include "../../core/utils/utils3.h"
#endif // UTILS3_H_INCLUDED
/*****************************************************************************/
// construction
//--------------------------------------------------
// defult constructor, SetFromString must be called before object is usable
jfAuthorStories::jfAuthorStories() {
  validdata = false;
  author_data = NULL;
  authnchanged = false;
}
//--------------------------------------------------
// constructor that calls set from string. be sure to check if IsValid is true before using
jfAuthorStories::jfAuthorStories(const wxString& inval,const jfAuthorInfo* in_authdata) {
  bool qval;
  author_data = NULL;
  qval = SetFromString(inval,in_authdata);
  if (!qval) {
    /**/jfXLogMessage(wxT("jfAuthorStories::jfAuthorStories"),1,err_msg);
  }
}
//--------------------------------------------------
// sets the internal contents from the author page downloaded from fanfiction,net
bool jfAuthorStories::SetFromString(const wxString& inval,const jfAuthorInfo* in_authdata) {
  // constants
  const wxString fname = wxT("jfAuthorStories::SetFromString");
  const wxString canlink = wxT("<link rel=\"canonical\" href=\"//www.fanfiction.net/u/");
  // variables
  jfStringParser* sval;
  wxString buffer;
  unsigned long uval;
  size_t fcount = 0;
  jfFFNItemAuthor* tempval;
  // some checks
  /**/jfXLogMessage(fname,1);
  validdata = false;
  author_data = in_authdata;
  sval = new jfStringParser(inval);
  Clear();
  /**/jfXLogMessage(fname,2);
  // what we do here first is check the id against the input
  if (!sval->GetDelimitedULong(canlink,wxT("/"),uval,err_msg)) {
    /**/jfXLogMessage(fname,3);
    delete sval;
    return false;
  }
  if (uval!=(author_data->GetAuthorID())) {
    err_msg = wxT("Author id in page does not match that passed to this method!");
    delete sval;
    /**/jfXLogMessage(fname,4);
    return false;
  }
  // checking the author name vs the input
  if (!sval->GetDelimited(wxT("<title>"),wxT(" | "),buffer)) {
    err_msg = wxT("Cannot find author name!");
    delete sval;
    /**/jfXLogMessage(fname,5);
    return false;
  }
  if (buffer!=(author_data->GetAuthorName())) {
    // it appears that the author name can be changed... so this is not an error
    authnchanged = true;
    new_author_name = buffer;
  }
  else authnchanged = false;
  // skipping past the bloated mess to get the author's stories
  if (sval->MovePast(wxT("<div id='st_inside'>"))) {
    /**/jfXLogMessage(fname,6);
    while(sval->GetDelimited(wxT("<div class='z-list mystories"),wxT("</div></div></div>"),buffer)) {
      tempval = new jfFFNItemAuthor(buffer,*author_data);
      // if processing went okay
      if (tempval->IsValid()) {
        // author check
        if (authnchanged) tempval->ChangeAuthorName(new_author_name);
        // adding
        AddFic(tempval);
        fcount++;
        /**/jfXLogMessage(fname,8);
      }
      // if it did not
      else {
        err_msg = wxT("Error in parsing fic! ");
        err_msg += tempval->GetParseError();
        delete tempval;
        delete sval;
        /**/jfXLogMessage(fname,9,err_msg);
        return false;
      }
    }
  }
  delete sval;
  validdata = true;
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting the author info
//--------------------------------------------------
size_t jfAuthorStories::GetAuthID() const {
  if (author_data==NULL) return 0;
  else return author_data->GetAuthorID();
}
//--------------------------------------------------
wxString jfAuthorStories::GetAuthName() const {
  if (author_data==NULL) return wxEmptyString;
  else return author_data->GetAuthorName();
}
//--------------------------------------------------
bool jfAuthorStories::AuthnameChanged() const {
  return authnchanged;
}
//--------------------------------------------------
wxString jfAuthorStories::GetNewAuthname() const {
  return new_author_name;
}
//----------------------------------------------------
const jfAuthorInfo* jfAuthorStories::GetAuthorInfo() const {
  return author_data;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// public checking methods
//--------------------------------------------------
// true if a fic with the given ID is in this object
bool jfAuthorStories::InclFic(size_t ficidin) const {
  // the iterator
  std::map<size_t,jfFFNItemAuthor*>::const_iterator xval;
  // looking
  xval = maindata.find(ficidin);
  return (xval!=maindata.end());
}
//--------------------------------------------------
// returns true if paring the source data went okay (and was done)
bool jfAuthorStories::IsValid() const {
  return validdata;
}
//--------------------------------------------------
// returns true if a contained fic is a newer version if the 'tocheck' one
bool jfAuthorStories::IsUpdated(const jfFFNItem* tocheck) const {
  // variables
  size_t infid;
  std::map<size_t,jfFFNItemAuthor*>::const_iterator xval;
  const jfFFNItemAuthor* yval;
  // checks. if the fic is bad or not in here, assertion fail!
  assert(tocheck!=NULL);
  infid = tocheck->GetID();
  assert(InclFic(infid));
  // startups
  xval = maindata.find(infid);
  yval = xval->second;
  // passed the checks. if the new fic has not been updated, return false
  if (!(yval->IsUpdated())) return false;
  // the new fic has been dated, we compare the updates
  if ((yval->GetUpdates())>(tocheck->GetUpdates())) return true;
  return false;
}
//--------------------------------------------------
// returns a fic with the specified ID (NULL if not found)
const jfFFNItemAuthor* jfAuthorStories::GetItemPtr(size_t inficidx) const {
  if (!InclFic(inficidx)) return NULL;
  else return maindata.find(inficidx)->second;
}
//-------------------------------------------------
// returns the paring error message
wxString jfAuthorStories::GetErr() const {
  return err_msg;
}
//-------------------------------------------------
// gets the number of fics
size_t jfAuthorStories::GetICount() const {
  return maindata.size();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// clearing and emptying data
//-------------------------------------------------
// deletes all the fics
void jfAuthorStories::Clear() {
  size_t mloop, mmax;
  mmax = GetICount();
  for(mloop=0;mloop<mmax;mloop++) {
    delete maindata[mloop];
    maindata[mloop] = NULL;
  }
  maindata.clear();
}
//-------------------------------------------------
// destructor
jfAuthorStories::~jfAuthorStories() {
  Clear();
  if (author_data!=NULL) {
    delete author_data;
    author_data = NULL;
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special private methods
// adding a fic, we do nothing if already there
void jfAuthorStories::AddFic(jfFFNItemAuthor* inval) {
  assert(inval!=NULL);
  size_t afid = inval->GetID();
  if (!InclFic(afid)) {
    maindata[afid] = inval;
  }
}
//============================================================================
// returns true if data for the given author is in the collection
bool jfAuthorCollection::HasAuthor(const jfAuthorInfo* in_authdata) const {
  const wxString fname = wxT("jfAuthorCollection::HasAuthor");
  // variables
  std::map<size_t,jfAuthorStories*>::const_iterator idx;
  // a basic check
  /**/jfXLogMessage(fname,1);
  if (in_authdata==NULL) return false;
  /**/jfXLogMessage(fname,2);
  // we look for the index
  idx = maindata.find(in_authdata->GetAuthorID());
  /**/jfXLogMessage(fname,3);
  // the result
  return (idx!=maindata.end());
}
//++++++++++++++++++++++++++++++++++++++++++++++
// getting info
//--------------------------------------------
// returns the number of authors
size_t jfAuthorCollection::GetAuthCount() const {
  return maindata.size();
}
//-------------------------------------------
// returns a const pointer to author data
const jfAuthorStories* jfAuthorCollection::GetAuthorStories(size_t inauthid) const {
  // variables
  std::map<size_t,jfAuthorStories*>::const_iterator idx;
  // we look for the index
  idx = maindata.find(inauthid);
  // the result
  if (idx!=maindata.end()) return idx->second;
  else return NULL;
}
//-------------------------------------------------
const jfAuthorInfo* jfAuthorCollection::GetAuthorInfo(size_t inauthid) const {
    // variables
  std::map<size_t,jfAuthorStories*>::const_iterator idx;
  const jfAuthorStories* qval;
  // we look for the index
  idx = maindata.find(inauthid);
  // the result
  if (idx!=maindata.end()) {
    qval = idx->second;
    return qval->GetAuthorInfo();
  }
  else return NULL;
}
//--------------------------------------------
// gets last message that was generated while parsing author pages
wxString jfAuthorCollection::GetParseErr() const {
  return err_msg;
}
//--------------------------------------------
bool jfAuthorCollection::MissingAuthor(size_t inauthid) const {
  std::set<size_t>::const_iterator fresult;
  fresult = missing.find(inauthid);
  return (fresult!=missing.end());
}
//++++++++++++++++++++++++++++++++++++++++++++++
// adding an author (do not delete in_authdata after, it tis not copied)
bool jfAuthorCollection::AddAuthor(const wxString& inval,const jfAuthorInfo* in_authdata) {
  // variables
  jfAuthorStories* newdata;
  size_t auth_id;
  // checks and assertions
  assert(in_authdata!=NULL);
  // we start
  newdata = new jfAuthorStories(inval,in_authdata);
  // if the data parsing went badly...
  if (!newdata->IsValid()) {
    err_msg = newdata->GetErr();
    delete newdata;
    return false;
  }
  // if it went well...
  else {
    auth_id = in_authdata->GetAuthorID();
    if (HasAuthor(in_authdata)) {
      delete maindata[auth_id];
      maindata[auth_id] = NULL;
    }
    maindata[auth_id] = newdata;
  }
  // done
  return true;
}
//--------------------------------------------
bool jfAuthorCollection::AddMissing(size_t inauthid) {
  if (MissingAuthor(inauthid)) return false;
  missing.insert(inauthid);
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++
// deleting
//--------------------------------------------
void jfAuthorCollection::Clear() {
  size_t icount,iloop;
  icount = GetAuthCount();
  for (iloop=0;iloop<icount;iloop++) {
    delete maindata[iloop];
    maindata[iloop] = NULL;
  }
  maindata.clear();
}
//--------------------------------------------
jfAuthorCollection::~jfAuthorCollection() {
  Clear();
}
/*****************************************************************************/
