/******************************************************************************
* Name:        auth_coll.h
* Author :     John Q Metro
* Purpose :    Collections involving author data
* Created:     January 19, 2011
* Updated:     January 6, 2014
******************************************************************************/
#ifndef AUTH_COLL_H_INCLUDED
#define AUTH_COLL_H_INCLUDED
#endif // AUTH_COLL_H_INCLUDED
//----------------------------------------
#include <map>
/////////////////////////////////////////////////////////////////////////////
#ifndef UPD_TYPES_H_INCLUDED
  #include "upd_types.h"
#endif // UPD_TYPES_H_INCLUDED
#ifndef JFFNFICOBJ
  #include "../../fanficnet/data/ffn_ficobj.h"
#endif // BASEOBJ_H_INCLUDED
//*****************************************************************************
// holds the fic contents of a Fanfiction.Net author page
class jfAuthorStories : public wxObject {
  public:
    // construction
    jfAuthorStories();
    jfAuthorStories(const wxString& inval,const jfAuthorInfo* in_authdata);
    bool SetFromString(const wxString& inval,const jfAuthorInfo* in_authdata);
    // getting the author info
    size_t GetAuthID() const;
    wxString GetAuthName() const;
    bool AuthnameChanged() const;
    wxString GetNewAuthname() const;
    const jfAuthorInfo* GetAuthorInfo() const;
    // public checking methods
    bool InclFic(size_t ficidin) const;
    bool IsValid() const;
    bool IsUpdated(const jfFFNItem* tocheck) const;
    const jfFFNItemAuthor* GetItemPtr(size_t inficidx) const;
    wxString GetErr() const;
    size_t GetICount() const;
    // clearing and emptying data
    void Clear();
    ~jfAuthorStories();
  protected:
    // special private methods
    void AddFic(jfFFNItemAuthor* inval);
    // internal data
    bool validdata;
    const jfAuthorInfo* author_data;
    std::map<size_t,jfFFNItemAuthor*> maindata;
    wxString err_msg;
    wxString new_author_name;
    bool authnchanged;
};
//============================================================================
// holds multiple jfAuthorStories
class jfAuthorCollection : public wxObject {
  public:
    // finding an author
    bool HasAuthor(const jfAuthorInfo* in_authdata) const;
    // getting info
    size_t GetAuthCount() const;
    const jfAuthorStories* GetAuthorStories(size_t inauthid) const;
    const jfAuthorInfo* GetAuthorInfo(size_t inauthid) const;
    wxString GetParseErr() const;
    bool MissingAuthor(size_t inauthid) const;
    // adding an author (do not delete in_authdata after, it is not copied)
    bool AddAuthor(const wxString& inval,const jfAuthorInfo* in_authdata);
    bool AddMissing(size_t inauthid);
    // deleting
    void Clear();
    ~jfAuthorCollection();
  protected:
    // internal data
    std::map<size_t,jfAuthorStories*> maindata;
    std::set<size_t> missing;
    wxString err_msg;
};
//*****************************************************************************
