/////////////////////////////////////////////////////////////////////////////
// Name:        newfic_coll.cpp
// Author :     John Q Metro
// Purpose :    fic updating required some extra methods and info in the data collection
// Created:     January 29, 2011
// Updated:     August 22, 2011
/////////////////////////////////////////////////////////////////////////////
#ifndef NEWFIC_COLL_H_INCLUDED
  #include "newfic_coll.h"
#endif // NEWFIC_COLL_H_INCLUDED
//-------------------------------------

/*****************************************************************************/
// constructor
jfFFN_NewItemCollection::jfFFN_NewItemCollection(jfFFNSearch* sdata_in):jfFFNItemCollectionCore(sdata_in) {
  datelim = NULL;
  datelim_reached = false;
  extracted = false;
  newfic = true;
}
//----------------------------------------------------------------------------
// a constructor
jfFFN_NewItemCollection::jfFFN_NewItemCollection(jfFFNSearch* sdata_in, const jfFFNSCategory* cat_linkin,
      const jfFicLimit *indatelim):jfFFNItemCollectionCore(sdata_in,cat_linkin) {
  // starting...
  const wxString fname = wxT("jfFFN_NewItemCollection::jfFFN_NewItemCollection");
  // the date limit stuff
  datelim = indatelim;
  datelim_reached = false;
  extracted = false;
  newfic = true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// date limits
void jfFFN_NewItemCollection::SetDatelimit(const jfFicLimit* flim) {
  datelim = flim;
  datelim_reached = false;
}
//----------------------------------------------------------------
bool jfFFN_NewItemCollection::DatelimitReached() const {
  return datelim_reached;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the destructor
jfFFN_NewItemCollection::~jfFFN_NewItemCollection() {
  ClearContents(true);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methods
bool jfFFN_NewItemCollection::ExitCheck(const jfFFNItem* intest) {
  const wxString fname = wxT("~~~+++ jfFFN_NewItemCollection::ExitCheck");
  if (datelim!=NULL) {
    datelim_reached = !(datelim->IsLater(intest));
    if (datelim_reached) {
      /**/jfXLogMessage(fname,1,cat_link->GetName());
      /**/jfXLogMessageS(fname,2,datelim->IdLimit());
      /**/jfXLogMessageU(fname,3,intest->GetID());
      /**/jfXLogMessage(fname,3,intest->GetName());
    }
  }
  return datelim_reached;
}
//===========================================================================
// core constructor
jfFFN_NewResCollection::jfFFN_NewResCollection(jfFFNSearch* zsearch):jfFFNResColl(zsearch) {
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfFFN_NewResCollection::MakeItemCollection(const wxString& src_in, const jfFFNSCategory* cat_linkin, const jfFicLimit* flim, wxString& outerr) {
   // constants
  const wxString fname = wxT("jfFFNResColl::MakeItemCollection");
  // variables
  jfFFN_NewItemCollection* temp;
  size_t rval;
  // asserts and check
  /**/jfXAssertLog((cat_linkin!=NULL),fname,wxT("The category linkin is NULL!"));
  /**/jfXAssertLog(flim!=NULL,fname,wxT("The input limit is NULL!"));
  /**/jfXLogMessage(fname,1);
  /**/jfXLogMessage(fname,2);
  // starting
  /**/jfXLogMessage(fname,3,typed_search->GetName());
  temp = new jfFFN_NewItemCollection(typed_search,cat_linkin,flim);
  /**/jfXLogMessage(fname,4);
  // creation
  if (!temp->AddFirstPage(src_in,rval)) {
    /**/jfXLogMessage(fname,5);
    outerr = temp->parse_error;
    delete temp;
    return false;
  }
  /**/jfXLogMessage(fname,6);
  // after, processing
  temp->SetName(cat_linkin->GetName());
  /**/jfXLogMessage(fname,7);
  AddItem(temp);
  cindex = itemcount-1;
  /**/jfXLogMessage(fname,8);
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// returns the limit value
size_t jfFFN_NewResCollection::GetLimit(size_t zindex) const {
  // variables and checks
  jfFFN_NewItemCollection* temp;
  assert(CheckIndex(zindex));
  // going ahead
  temp = dynamic_cast<jfFFN_NewItemCollection*>((*collections)[zindex]);
  return temp->GetHighID();
}
//--------------------------------------------------------------
bool jfFFN_NewResCollection::LimitReached(size_t in_index) const {
  // variables and checks
  jfFFN_NewItemCollection* temp;
  assert(CheckIndex(in_index));
  // going ahead
  temp = dynamic_cast<jfFFN_NewItemCollection*>((*collections)[in_index]);
  return temp->DatelimitReached();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// some more methods
//-------------------------------
// typed version of GetCurrPointer
jfFFN_NewItemCollection* jfFFN_NewResCollection::GetCurrPointerT() {
  jfFFN_NewItemCollection* result;
  result = dynamic_cast<jfFFN_NewItemCollection*>(GetCurrPointer());
  //done
  return result;
}
//------------------------------------
// returns true if the input category is the same one as the collection at the current index
bool jfFFN_NewResCollection::CompareCats(const jfFFNSCategory* tocheck) const {
  // the usual checks to see if things are valid
  assert(tocheck!=NULL);
  assert(itemcount>=0);
  assert(cindex<itemcount);
  assert(collections!=NULL);
  // variables
  jfFFN_NewItemCollection* test_coll;
  // we start
  test_coll = dynamic_cast<jfFFN_NewItemCollection*>((*collections)[cindex]);
  // producing the result
  return ((test_coll->GetCat())==tocheck);
}
//====================================================================================
