/******************************************************************************
* Name    :  res_bundle.cpp
* Author  :  John Q Metro
* Purpose :  Declares a class that holds all the data related to the update
* Created :  January 20, 2011
* Updated :  Febuary 11, 2012
******************************************************************************/
#ifndef RES_BUNDLE_H_INCLUDED
  #include "res_bundle.h"
#endif // RES_BUNDLE_H_INCLUDED
/****************************************************************************/
// constructors
jfFFN_UpdateBundle::jfFFN_UpdateBundle() {
  search_data = NULL;
  result_data = NULL;
  new_data = NULL;
  auth_data = NULL;
  chlink = NULL;
  auth_check = false;
  new_fics = false;
  sindex = 0;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special meta-information
wxString jfFFN_UpdateBundle::GetTypeID() const {
  return wxT("jfFFN_UpdateBundle");
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// information about the result
//----------------------------------------------------------
size_t jfFFN_UpdateBundle::ResultCatCount() const {
  assert(result_data!=NULL);
  return result_data->NumSCats();
}
//----------------------------------------------------------
size_t jfFFN_UpdateBundle::GetIDLimit(size_t limindex) {
  assert(result_data!=NULL);
  assert(limindex<(result_data->NumSCats()));
  return result_data->GetLimit(limindex);
}
//------------------------------------------------------------------
bool jfFFN_UpdateBundle::SameCategory(size_t index, const jfFFNSCategory* testee) {
  // constants
  const wxString fname = wxT("jfFFN_UpdateBundle::SameCategory");
  // variables
  const jfFFNSCategory* test1;
  // checks
  /**/jfXLogMessageS(fname,1,index);
  assert(result_data!=NULL);
  assert(testee!=NULL);
  /**/jfXLogMessageS(fname,2,(result_data->NumSCats()));
  /**/jfXLogMessage(fname,3,testee->GetName());
  if (index==(result_data->NumSCats())) return false;
  // we produce the result
  test1 = result_data->GetCat(index);
  /**/jfXLogMessage(fname,4,test1->GetName());
  return test1==testee;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special information
const jfFFNSCategory* jfFFN_UpdateBundle::GetOldCategory() const {
  size_t oindex = result_data->SCatIndex();
  if (oindex>=(result_data->NumSCats())) return NULL;
  else return result_data->GetCat(oindex);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// update status flags
//-----------------------------------
bool jfFFN_UpdateBundle::AuthDone() {
  wxMutexLocker mlock(upd_task_lock);
  auth_check = true;
  return auth_check && new_fics;
}
//-----------------------------------
bool jfFFN_UpdateBundle::NewDone() {
  wxMutexLocker mlock(upd_task_lock);
  new_fics = true;
  return auth_check && new_fics;
}
//--------------------------------------
bool jfFFN_UpdateBundle::CheckSourceNNull() const {
  if (search_data==NULL) return false;
  return (result_data!=NULL);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructors
jfFFN_UpdateBundle::~jfFFN_UpdateBundle() {
  if (search_data!=NULL) delete search_data;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o output
//-----------------------------------
bool jfFFN_UpdateBundle::AddMiddleToFile(wxTextFile* outfile) const {
  // constants and variables
  const wxString fname = wxT("jfFFN_UpdateBundle::AddMiddleToFile");
  wxString buffer;
  bool rval;
  // checking and special conditions
  /**/jfXLogMessage(fname,1);
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  /**/jfXLogMessage(fname,2);
  // we write the date first
  buffer = lastdate.Format(wxT("%m-%d-%Y"));
  outfile->AddLine(buffer);
  /**/jfXLogMessage(fname,3);
  // next, we write the search file
  rval = search_data->AddToFile(outfile);
  /**/jfXLogMessage(fname,4);
  return rval;
}
//-----------------------------------
bool jfFFN_UpdateBundle::AddRestToFile(wxTextFile* outfile) const {
  // constants and variables
  const wxString fname = wxT("jfFFN_UpdateBundle::AddRestToFile");
  bool rval;
  // checking and special conditions
  /**/jfXLogMessage(fname,1);
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  /**/jfXLogMessage(fname,2);
  // here, we write the results collection
  rval = result_data->AddToFile(outfile);
  /**/jfXLogMessage(fname,3);
  return rval;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o input
//-----------------------------------
bool jfFFN_UpdateBundle::ReadMiddleFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfFFN_UpdateBundle::ReadMiddleFromFile");
  // input data
  wxString cline, buffer;
  // starting checks (and reading the line)
  assert(infile!=NULL);
  if (!infile->ReadLine(cline,funcname)) return false;
  if (NULL==lastdate.ParseFormat(cline,wxT("%m-%d-%Y"))) {
    /**/jfXLogMessage(funcname,2,cline);
    return infile->BuildError(wxT("Unable to convert the search date!"));
  }
  /**/jfXLogMessageS(funcname,3,sindex);
  // next we read the search data
  if (search_data!=NULL) delete search_data;
  search_data = new jfFFNSearch(chlink,sindex);
  /**/jfXLogMessage(funcname,4);
  return search_data->GetFromFile(infile);
}
//-----------------------------------
bool jfFFN_UpdateBundle::ReadRestFromFile(jfFileReader* infile) {
  const wxString fname = wxT("jfFFN_UpdateBundle::ReadRestFromFile");
  assert(chlink!=NULL);
  // reading the result data
  /**/jfXLogMessage(fname,1);
  assert(infile!=NULL);
  if (result_data!=NULL) delete result_data;
  /**/jfXLogMessage(fname,2);
  result_data = new jfFFNResColl(search_data);
  // we assign the categories before reading
  result_data->SetCatLinks(chlink);
  /**/jfXLogMessage(fname,3);
  if (!result_data->GetFromFile(infile)) {
    /**/jfXLogMessage(fname,4,infile->err);
    delete result_data;
    result_data = NULL;
    return false;
  }
  /**/jfXLogMessage(fname,5);
  return true;
}
/****************************************************************************/
