/////////////////////////////////////////////////////////////////////////////
// Name:        upd_types.cpp
// Author :     John Q Metro
// Purpose :    Misc types and untils helpful in fanfiction.net updating
// Created:     January 18, 2011
// Updated:     January 15, 2014
/////////////////////////////////////////////////////////////////////////////
#ifndef UPD_TYPES_H_INCLUDED
  #include "upd_types.h"
#endif // UPD_TYPES_H_INCLUDED
//---------------------------------------------------
#ifndef UTILS1_H_INCLUDED
  #include "../../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
//****************************************************************************
jfAuthorInfo::jfAuthorInfo(size_t in_authorid, wxString in_authname) {
  author_id = in_authorid;
  author_name = in_authname;
}
//-----------------------------------------------------------------
jfAuthorInfo::jfAuthorInfo(const jfAuthorInfo& inval) {
  author_id = inval.author_id;
  author_name = inval.author_name;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting info
//------------------------------------------------
size_t jfAuthorInfo::GetAuthorID() const { return author_id;}
//------------------------------------------------
wxString jfAuthorInfo::GetAuthorName() const { return author_name; }
//------------------------------------------------
wxString jfAuthorInfo::GetUrl() const {
  // constants and values
  const wxString sstart = wxT("https://www.fanfiction.net/u/");
  wxString result = sstart;
  wxString nbuffer = author_name;
  // we build from there
  result += SInt2Str(author_id);
  result += wxT("/");
    nbuffer.Replace(wxT("\'"),wxT(""));
  nbuffer.Replace(wxT(" "),wxT("_"));
  nbuffer.Replace(wxT("-"),wxT("_"));
  nbuffer.Replace(wxT("__"),wxT("_"));
  nbuffer.Replace(wxT("__"),wxT("_"));
  result += nbuffer;
  return result;
}
//****************************************************************************
// constructors
//-------------------------------------------
jfFFNItemAuthor::jfFFNItemAuthor():jfFFNItemCore() {}
//-------------------------------------------
jfFFNItemAuthor::jfFFNItemAuthor(const wxString& indata, const jfAuthorInfo& inauth):jfFFNItemCore() {
  SetFromString(indata,inauth);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting data
bool jfFFNItemAuthor::SetFromString(const wxString& indata, const jfAuthorInfo& inauth) {
  // local variables
  wxString buffer;
  // we start
  SetParserString(indata);
  // processing fic id, title, and making the link
  if (!GetLinkTitle()) return false;
  // the author name and author id are inserted manually
  author_id = inauth.GetAuthorID();
  author_name = inauth.GetAuthorName();
  // get we the descrtption
  if (!ExtractDescription()) return false;
  // getting the category name
  if (!xparser->MovePastAlt(wxT("class='gray z-padtop2'>"),wxT("class='z-padtop2 xgray'>"))) {
    perr_string = wxT("Category Name not found! A");
    return false;
  }
  if (!xparser->GetUpto(wxT("- Rated:"),buffer)) {
    perr_string = wxT("Category Name not found! B");
    return false;
  }
  Dtrim(buffer);
  catname = buffer;
  // the rest is just like jfFFNItem
  // next, the rating, language, genres, chapter count, and word count
   if (!Rating2Words()) return false;
  // we skip past the reviews
  if (!xparser->MovePast(wxT("Reviews:"))) {
    // this is okay, sometimes there are no reviews
  }
  if (!Dates_and_Completion()) return false;
  // done with all the parsing, just some finishing up...
  validdata = true;
  return true;
}
//---------------------------------------------------------
void jfFFNItemAuthor::ChangeAuthorName(const wxString& inval) {
  author_name = inval;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting data
wxString jfFFNItemAuthor::GetCatname() const {
  return catname;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methods
//-------------------------------------------
wxString jfFFNItemAuthor::GetTypeID() const {
  return wxT("FFNItemAuthor");
}
//-------------------------------------------
// since we do not actually display the author derived items, this is here for compatabilty
bool jfFFNItemAuthor::LoadValues(jfSkeletonParser* inparser) const {
  assert(false);
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// parsing helper methods
bool jfFFNItemAuthor::GetLinkTitle() {
  // variables
  unsigned long qval;
  wxString buffer;
  // starting
  func_string = wxT("jfFFNItemCore::GetLinkTitle");
  rank = 0;
  // next up is the story id
  if (!xparser->MovePastAlt(wxT("<a class=stitle href=\"/s/"), wxT("<a  class=stitle href=\"/s/"))) {
    perr_string = wxT("Start of Fic ID not found!");
    return false;
  }
  if (!xparser->GetMovePastULong(wxT("/"),qval,perr_string)) {
    perr_string += wxT(" When getting fic id.");
    return false;
  }
  num_id = qval;
  // we get the rest of the url
  if (!xparser->GetMovePast(wxT("\">"),buffer)) {
    perr_string = wxT("End of fic link not found!");
    return false;
  }
  // we then build the url...
  primarylink = wxT("https://www.fanfiction.net/s/") + SInt2Str(num_id);
  primarylink += (wxString(wxT("/")) + buffer);
  // next up is the title
  if (!xparser->MovePast(wxT("<img class='lazy cimage'"))) {
    perr_string = wxT("Cannot find story image!");
    return false;
  }
  if (!xparser->GetDelimited(wxT(">"),wxT("</a>"),buffer)) {
    perr_string = wxT("Title field not found!");
    return false;
  }
  name = buffer;
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// io methods. compatability only
//-------------------------------------------
bool jfFFNItemAuthor::AddExtraStuff(wxTextFile* outfile) const {
  assert(false);
  return false;
}
//-------------------------------------------
bool jfFFNItemAuthor::ReadExtraStuff(jfFileReader* infile) {
  assert(false);
  return false;
}
//=========================================================================
// bundles fic id and published date for a limit: do not get fics with lower id or earlier date
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// constructors and data methods
//---------------------------------------------------
jfFicLimit::jfFicLimit(wxDateTime dlimit, size_t idlimit) {
  fpublished = dlimit;
  id_limit = idlimit;
  nodate = false;
}
//---------------------------------------------------
void jfFicLimit::ChangeLimit(size_t inval, bool innodate) {
  id_limit = inval;
  nodate = innodate;
}
//---------------------------------------------------
size_t jfFicLimit::IdLimit() const {
  return id_limit;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// testing...
bool jfFicLimit::IsLater(const jfFFNItemCore* test) const {
  assert(test!=NULL);
  if ((test->GetID())<id_limit) return false;
  if (nodate) return true;
  if ((test->GetPublished())<fpublished) return false;
  return true;
}

//****************************************************************************
