/******************************************************************************
* Name:        upd_dialogs.cpp
* Author :     John Q Metro
* Purpose :    Update interface dialogs
* Created:     March 3, 2011
* Updated:     January 1, 2014
******************************************************************************/
#ifndef UPD_DIALOGS_H_INCLUDED
  #include "upd_dialogs.h"
#endif // UPD_DIALOGS_H_INCLUDED
//---------------------
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
/*****************************************************************************/
const wxString title_load = wxT("Loading old results to the display...");
const wxString title_merge = wxT("Merging old and new results...");
//===============================================================================
// the constructor
jfLoadOldDialog::jfLoadOldDialog(wxWindow* parent, bool load, wxWindowID id,const wxPoint& pos,
    const wxSize& size, const wxString& name):wxDialog(parent,id,(load)?(title_load):(title_merge),
    pos,size,wxCAPTION|wxRESIZE_BORDER|wxMINIMIZE_BOX,name) {
  // we start
  assert(parent!=NULL);
  xload = load;
  wxString lname = (xload)?(wxT("Loading")):(wxT("Merging"));
  // we create the gauge
  cat_gauge = new jfLabeledGauge(this,24,wxTAB_TRAVERSAL, wxT("Old Category Gauge"));
  cat_gauge->ChangeTypestring(wxT("Category"),lname);
  // we create the sizer
  mainsizer = new wxBoxSizer(wxHORIZONTAL);
  // finishing
  mainsizer->Add(cat_gauge,1,wxALL|wxEXPAND,3);
  SetClientSize(400,70);
  SetSizer(mainsizer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// events
//--------------------------------------------------------------
// thread start event: all we really do here is record the category count
void jfLoadOldDialog::StartEvent(wxCommandEvent& event) {
  cvalue = event.GetInt();
  assert(cvalue>0);
  cat_gauge->SetCount(cvalue);
}
//--------------------------------------------------------------
// category started
void jfLoadOldDialog::SCatEvent(wxCommandEvent& event) {
  // variables
  int partidx = event.GetInt();
  wxString cname = event.GetString();
  // starting
  assert(partidx>0);
  assert((cat_gauge->GetCount())>=1);
  // we do
  cat_gauge->SetNew(partidx,cvalue,cname);
}
//--------------------------------------------------------------
// catgeory ended
void jfLoadOldDialog::ECatEvent(wxCommandEvent& event) {
  // variables
  int partidx = event.GetInt();
  // we do
  cat_gauge->SetDone(wxT("Done with"));
}
//--------------------------------------------------------------
// completely done
void jfLoadOldDialog::EndEvent(wxCommandEvent& event) {
  const wxString fname = wxT("jfLoadOldDialog::EndEvent");
  /**/jfXLogMessage(fname,1);
  wxMilliSleep(400);
  /**/jfXLogMessage(fname,2);
  SendDoneSignal();
  /**/jfXLogMessage(fname,3);
  Destroy();
}
//--------------------------------------------------------------
// handling multiple empty categories
void jfLoadOldDialog::MultECats(wxCommandEvent& event) {
  // getting data
  size_t numskip = event.GetInt();
  wxString lcatname = event.GetString();
  // we do
  cat_gauge->SkipToDone(numskip,wxT("done with"),lcatname);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// used to send a message to the parent window that we are done
void jfLoadOldDialog::SendDoneSignal() {
  wxCommandEvent eventx(jf_UPDATE_OLDUPLOADED_EVENT,GetParent()->GetId());
  wxPostEvent(GetParent(),eventx);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfLoadOldDialog,wxDialog)
  EVT_COMMAND(-1,jf_DOWNL_START_EVENT      , jfLoadOldDialog::StartEvent )
  EVT_COMMAND(-1,jf_DOWNL_CATSTART_EVENT   , jfLoadOldDialog::SCatEvent  )
  EVT_COMMAND(-1,jf_DOWNL_CATEND_EVENT     , jfLoadOldDialog::ECatEvent  )
  EVT_COMMAND(-1,jf_UPDATE_AUXTHREAD_DONE  , jfLoadOldDialog::EndEvent   )
  EVT_COMMAND(-1,jf_OLDLOAD_BULKECATS_EVENT, jfLoadOldDialog::MultECats  )
END_EVENT_TABLE()
/*****************************************************************************/
