/******************************************************************************
* Name:        upd_disp.cpp
* Author :     John Q Metro
* Purpose :    Large panel holding the main status and the result viewlists
* Created:     March 27, 2011
* Updated:     January 18, 2014
******************************************************************************/
#ifndef UPD_DISP_H_INCLUDED
  #include "upd_disp.h"
#endif // UPD_DISP_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef FFNDOWN_PARTS_H_INCLUDED
  #include "../../fanficnet/ficdown/ffndown_parts.h"
#endif // FFNDOWN_PARTS_H_INCLUDED
#ifndef FFNDOWN_DIALOG_H_INCLUDED
  #include "../../ficdown/interface/ffndown_dialog.h"
#endif // FFNDOWN_DIALOG_H_INCLUDED
//------------------------------------

jfUpdateDisplay::jfUpdateDisplay(wxWindow* parent, jfFFN_UpdateBundle* inval,
      wxWindowID id, const wxString& name):wxPanel(parent,id, wxDefaultPosition,
      wxDefaultSize,wxTAB_TRAVERSAL,name){
  // inval testing first
  assert(inval!=NULL);
  maindata = inval;
  // creating the GUI elemets
  main_results = new jfHtmlResultCollection_FFN(this,-1);
  bool mres = main_results->SetupViewers(maindata->search_data->categories);
  assert(mres);
  maindata->search_data->cat_results = main_results;
  status_display = new jfUpDMainpanel(this,maindata);
  // the sizer and arranging things
  main_sizer = new wxBoxSizer(wxVERTICAL);
  main_sizer->Add(main_results,1,wxALL|wxEXPAND,3);
  main_sizer->Add(status_display,0,wxALL|wxEXPAND,3);
  SetSizer(main_sizer);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// starting phase 1
bool jfUpdateDisplay::LoadOld() {
  return status_display->LaunchPhase1();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfUpdateDisplay::SetPopupMenu(jfPopupMenu* inmenu) {
  main_results->SetPopupMenu(inmenu);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfUpdateDisplay::ChangeBHeight(size_t nheight) {
  bool xs = status_display->ChangeBHeight(nheight);
  Layout();
  return xs;
}
//--------------------------------------------------------------------
bool jfUpdateDisplay::ChangeMode(bool tosmall) {
  bool xs = status_display->ChangeMode(tosmall);
  Layout();
  return xs;
}
//---------------------------------------------------------
size_t jfUpdateDisplay::GetSelectedIndex() const {
  return main_results->SelectedIndex();
}
//---------------------------------------------------------
bool jfUpdateDisplay::StopSearch() {
  return status_display->OrderStop();
}
//*****************************************************************************
jfUpdateTopLevel::jfUpdateTopLevel(wxWindow* parent, jfFFN_UpdateBundle* inval,
      wxWindowID id, const wxString& name):DisplayPanelCore(parent,id,name) {
  // variables
  wxString corename;
  // inval testing
  assert(inval!=NULL);
  indata = inval;
  // creating the notebook
  mainbook = new wxNotebook(this,-1);
  // other stuff
  panel1 = new jfUpdateDisplay(mainbook,indata);
  panel2 = new jfFFNUpd_Panel2(mainbook,indata->search_data,-1);
  if (indata->search_data->local_map->IsEmpty()) panel3=NULL;
  else {
    panel3 = new jfFilterMapEditor(mainbook,-1,indata->search_data->local_map,false,filterglobals::fclist[5]);
    panel3->DisableContents();
  }
  if (indata->search_data->categories->IsEmpty()) panel4 = NULL;
  else {
    panel4 = new jfMainCategoriesEditor(mainbook,indata->search_data->categories,-1);
    panel4->SetUnEditable();
  }
  // arranging
  msizer = new wxBoxSizer(wxVERTICAL);
  msizer->Add(mainbook,1,wxALL|wxEXPAND,1);
  corename = indata->search_data->GetName();
  mainbook->AddPage(panel1,corename + wxT(" Results"));
  mainbook->AddPage(panel2,corename + wxT(" Search Data"));
  if (panel3!=NULL) mainbook->AddPage(panel3,corename + wxT(" Custom Filters"));
  if (panel4!=NULL) mainbook->AddPage(panel4,corename + wxT(" Categories"));
  // popup menu
  rc_menu = new jfFFNPopupMenu(this);
  panel1->SetPopupMenu(rc_menu);
  // done
  SetSizer(msizer);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handles the event that everything is done
void jfUpdateTopLevel::HandleAllDone(wxCommandEvent& event) {
  const wxString fname = wxT("jfUpdateTopLevel::HandleAllDone");
  bool qval = (bool)(event.GetInt());
  wxWindow* mparent = GetGrandParent();
  event.SetInt((int)(GetId()));
  /**/jfXLogMessage(fname,1);
  if (qval) wxPostEvent(mparent,event);
  else {
    wxCommandEvent outevent(jf_PP_SRCHFAIL_EVENT,GetId());
    outevent.SetInt((bool)qval);
    wxPostEvent(mparent,outevent);
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfUpdateTopLevel::OnRMenuDownloadFic(wxCommandEvent& event) {
  // local variables
  wxPoint* index_coord;
  jfFicExtract* gficdata;
  jfDown_Dialog* mdialog;
  // we start by getting coordinates for the item
  index_coord = reinterpret_cast<wxPoint*>(event.GetClientData());
  // getting the fanfic data
  gficdata = indata->result_data->GetFicExtract(index_coord->x,index_coord->y);
  delete index_coord;
  // prepare a dialog that is used to download the fic
  mdialog = new jfDown_Dialog(this,-1,wxT(""),gficdata);
  mdialog->Show();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// shared virtual methods
//---------------------------------------------
bool jfUpdateTopLevel::WriteAll2Html() {
  // constants
  const wxString fname = wxT("jfUpdateTopLevel::WriteAll2Html");
  // the stuff
  wxBeginBusyCursor();
  if ((indata->search_data->categories)->UsesDefault()) {
    /**/jfXLogMessage(fname,1);
    indata->result_data->WriteToHTML(32);
  }
  size_t ccount = (indata->search_data->categories)->GetCount();
  /**/jfXLogMessageS(fname,2,ccount);
  if (ccount!=0) {
    for (size_t cloop=0;cloop<ccount;cloop++) {
      /**/jfXLogMessageS(fname,3,cloop);
      indata->result_data->WriteToHTML(cloop);
      /**/jfXLogMessageS(fname,4,cloop);
    }
  }
  /**/jfXLogMessage(fname,5);
  wxEndBusyCursor();
  return true;
}
//----------------------------------------------
bool jfUpdateTopLevel::WriteCurrent2Html() {
  wxBeginBusyCursor();
  size_t curr_res = panel1->GetSelectedIndex();
  indata->result_data->WriteToHTML(curr_res);
  wxEndBusyCursor();
  return true;
}
//--------------------------------------------------
void jfUpdateTopLevel::WindowResized(size_t x, size_t y) {
  const wxString fname = wxT("jfUpdateTopLevel::WindowResized");
  if ((panel4!=NULL) && (panel1!=NULL)) {
    if (y<1000) {
      panel4->ChangeToHorizontal();
      panel1->ChangeBHeight(10);
    }
    else {
      panel4->ChangeToVertical();
      panel1->ChangeBHeight(20);
    }
  }
  if (panel1!=NULL) {
    if (y<950) {
      panel1->ChangeMode(true);
    }
    else {
      panel1->ChangeMode(false);
    }
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++
// io methods
//-------------------------------
wxString jfUpdateTopLevel::GetSearchName() const {
  if (indata==NULL) return wxEmptyString;
  if ((indata->search_data)==NULL) return wxEmptyString;
  else return indata->search_data->GetName();
}
//-------------------------------------------------------------
bool jfUpdateTopLevel::OrderStop() {
  return panel1->StopSearch();
}
//----------------------------------------------------------------------
bool jfUpdateTopLevel::WriteFFNBundle(wxString inpath) const {
  // variables and constants
  const wxString fname = wxT("jfUpdateTopLevel::WriteFFNBundle");
  wxTextFile* outfile;
  // building the result bundle
  /**/jfXLogMessage(fname,2,inpath);
  // opening or creating the output text file
  outfile = new wxTextFile(inpath);
  if (outfile->Exists()) {
    /**/jfXLogMessage(fname,3);
    outfile->Open();
    outfile->Clear();
  }
  else {
    outfile->Create();
    outfile->Open();
  }
  /**/jfXLogMessage(fname,4);
  // we produce the result
  indata->AddToFile(outfile);
  /**/jfXLogMessage(fname,5);
  outfile->Write();
  /**/jfXLogMessage(fname,6);
  // done
  outfile->Close();
  /**/jfXLogMessage(fname,7);
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfUpdateTopLevel::LoadOld() {
  return panel1->LoadOld();
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfUpdateTopLevel,wxPanel)
  EVT_COMMAND  (-1,jf_UPDATE_ALLDONE_EVENT, jfUpdateTopLevel::HandleAllDone )
  EVT_COMMAND(-1,jf_RMENU_DOWNLFIC_EVENT,jfUpdateTopLevel::OnRMenuDownloadFic)
END_EVENT_TABLE()
//*****************************************************************************
