/******************************************************************************
* Name:        upd_mainpanel.cpp
* Author :     John Q Metro
* Purpose :    Update interface main panel
* Created:     March 5, 2011
* Updated:     January 17, 2013
******************************************************************************/
#ifndef UPD_MAINPANEL_H_INCLUDED
  #include "upd_mainpanel.h"
#endif // UPD_MAINPANEL_H_INCLUDED
#ifndef AUTH_THREAD_H_INCLUDED
  #include "../threads/auth_thread.h"
#endif // AUTH_THREAD_H_INCLUDED
#ifndef POST_COMBINE_H_INCLUDED
  #include "../threads/post_combine.h"
#endif // POST_COMBINE_H_INCLUDED
#ifndef UPD_DIALOGS_H_INCLUDED
  #include "upd_dialogs.h"
#endif // UPD_DIALOGS_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//-----------------------------------------------------------------
/*****************************************************************************/
// constructor
jfDualProgressBars::jfDualProgressBars(wxWindow* parent, wxString nlabel, size_t bheight,
      wxWindowID id, const wxString& name):wxPanel(parent,id, wxDefaultPosition,
          wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // starting
  msizer = new wxStaticBoxSizer(wxVERTICAL,this,nlabel);
  size_t qheight = (bheight>=10)?(bheight):(10);
  cat_pbar = new jfLabeledGauge(this,qheight);
  part_pbar = new jfLabeledGauge(this,qheight);
  // arraging
  msizer->Add(cat_pbar,0,wxEXPAND|wxALL,3);
  msizer->Add(part_pbar,0,wxEXPAND|wxALL,3);
  // initializing values
  catcount = 0;
  catindex = 0;
  sbox_mode = true;
  SetLabel(nlabel);
  stored_label = nlabel;
  // setting the sizer
  SetSizer(msizer);
}
//----------------------------------------------------
void jfDualProgressBars::Reset() {
  cat_pbar->SetToZero();
  part_pbar->SetToZero();
  catcount = 0;
  catindex = 0;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// changing/setting category info
//---------------------------------------------
void jfDualProgressBars::SetCatLabels(wxString cat_label, wxString cat_action) {
  cat_pbar->ChangeTypestring(cat_label,cat_action);
}
//---------------------------------------------
bool jfDualProgressBars::SetCatCount(size_t newcount) {
  // category count
  if (newcount==0) return false;
  catcount = newcount;
  cat_pbar->SetCount(newcount);
  return true;
}
//---------------------------------------------
bool jfDualProgressBars::NewCat(size_t in_index, size_t ofcount, wxString name) {
    // checks and asserts
  catindex++;
  if (catindex>catcount) return false;
  cat_pbar->SetNew(in_index,ofcount,name);
  part_pbar->SetToZero();
  // done
  return true;
}
//---------------------------------------------
void jfDualProgressBars::CatDone(wxString dlabel) {
  cat_pbar->SetDone(dlabel);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// changing page info
//---------------------------------------------
void jfDualProgressBars::SetPageLabels(wxString page_label, wxString page_action) {
  part_pbar->ChangeTypestring(page_label,page_action);
}
//---------------------------------------------
bool jfDualProgressBars::SetPageCount(size_t newcount) {
  part_pbar->SetCount(newcount);
  return true;
}
//---------------------------------------------
bool jfDualProgressBars::SetNew(int index_in, int tcx_in, wxString name) {
  part_pbar->SetNew(index_in,tcx_in,name);
  return true;
}
//---------------------------------------------
void jfDualProgressBars::ChangeAction(wxString action) {
  part_pbar->ChangeAction(action);
}
//---------------------------------------------
bool jfDualProgressBars::PageDone(wxString action) {
  part_pbar->SetDone(action);
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// size changing
//---------------------------------------------
bool jfDualProgressBars::ChangeSizes(size_t bheight, size_t psize) {
  bool xval;
  xval = cat_pbar->ChangeSizes(bheight,psize);
  if (!xval) return false;
  xval = part_pbar->ChangeSizes(bheight,psize);
  return xval;
}
//---------------------------------------------
bool jfDualProgressBars::RevertSizes() {
  bool xval;
  xval = cat_pbar->RevertSizes();
  if (!xval) return false;
  xval = part_pbar->RevertSizes();
  return xval;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting info
//---------------------------------------------
size_t jfDualProgressBars::GetCatCount() const {
  return catcount;
}
//---------------------------------------------
size_t jfDualProgressBars::GetCatIndex() const {
  return catindex;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special GUI manipulation
// changing from bars in a labeled sizer to a regular sizer and back
bool jfDualProgressBars::ChangeMode(bool tosbox) {
  // to labeled sizer
  if (tosbox) {
    if (sbox_mode) return false;
    // we do not need to create the static box sizer
    // remove the progress bars from the un-labelled sizer
    nl_sizer->Clear();
    // insert them in the new ordinary sizer
    msizer->Add(cat_pbar,0,wxEXPAND|wxALL,3);
    msizer->Add(part_pbar,0,wxEXPAND|wxALL,3);
    msizer->GetStaticBox()->Show(true);
    // changing the sizer
    SetSizer(msizer,false);
    // dealing with the old sizer
    delete nl_sizer;
    nl_sizer = NULL;
  }
  // to regualar sizer
  else {
    if (!sbox_mode) return false;
    // we create the sizer
    nl_sizer = new wxBoxSizer(wxVERTICAL);
    // remove the progress bars from the labelled sizer
    msizer->Clear();
    // insert them in the new ordinary sizer
    nl_sizer->Add(cat_pbar,0,wxEXPAND|wxALL,3);
    nl_sizer->Add(part_pbar,0,wxEXPAND|wxALL,3);
    // changing the sizer
    SetSizer(nl_sizer,false);
    /* dealing with the old sizer. we keep it because in OS X, it seems that
    these sizers must be created before anything they contain, for the
    containing elements to be clickable */
    msizer->GetStaticBox()->Hide();
  }
  return true;
}
//-------------------------------------------------
wxString jfDualProgressBars::GetStoredLabel() const {
  return stored_label;
}
//=============================================================================
jfUpDMainpanel::jfUpDMainpanel(wxWindow* parent, jfFFN_UpdateBundle* inmaindata,
      wxWindowID id,const wxString& name):wxPanel(parent,id,wxDefaultPosition,
      wxDefaultSize,wxTAB_TRAVERSAL,name) {

  // initializing some info
  assert(inmaindata!=NULL);
  maindata = inmaindata;
  Reset(true);
  ssize_config = false;
  side_sizer = NULL;
  smain_sizer = NULL;
  ssize_nbook = NULL;
  // gui configuration
  GuiSetup(20);
  // final stuff
  SetStatus(jfp_NOTSTARTED);
  SetSizer(main_sizer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handling
//-----------------------------------------------------------------
/* when the phase 1 thread (uploading old results) is done, the thread sends the
signal to the Dialog, which then sends a signal up to this panel. This method
handles that signal */
void jfUpDMainpanel::EventPhase1Done(wxCommandEvent& mevent) {
  const wxString fname = wxT("jfUpDMainpanel::EventPhase1Done");

  /**/jfXLogMessageS(fname,1,phase);
  if (phase==1) {
    stat_label->SetLabel(wxT("READY TO START"));
    phase1_thread = NULL; // threads self-destruct, right?
    startstop_btn->Enable(true);
    /**/jfXLogMessageS(fname,2,phase);
  }
  // we also use the same dialog to handle the display for phase 3
  else if (phase==3) {
    stat_label->SetLabel(wxT("ALL DONE"));
    phase3_thread = NULL;
    /**/jfXLogMessageS(fname,3,phase);
    SendDoneSignal(true);
  }
  else assert(false);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handlers for events generated from this panel
//----------------------------------------------------
void jfUpDMainpanel::StartStopBtnHandler(wxCommandEvent& event) {
  // variables
  const wxString fname = wxT("jfUpDMainpanel::StartStopBtnHandler");
  bool rval;
  wxString oval;
  // start
  if (!updating) {
    // setting the start labels and buttons
    if (total_status!=jfp_NOTSTARTED) Reset(false);
    startstop_btn->Enable(false);
    SetStatus(jfp_STARTING);
    // preparing to launch
    rval = maindata->search_data->VerifyNames(oval);
    // launching the update
    if (rval) {
      maindata->search_data->CopyLoad();
      new_status = old_status = jfp_STARTING;
      DoPhase2();
    }
    // cancelling the update
    else {
      wxMessageBox(oval,wxT("Name Verification Failed!"),wxOK|wxICON_ERROR);
      startstop_btn->Enable(true);
      SetStatus(jfp_NOTSTARTED);
    }
  }
  // stop
  else OrderStop();
}
//----------------------------------------------------
void jfUpDMainpanel::PauseResumeBtnHandler(wxCommandEvent& event) {
  const wxString fname = wxT("jfUpDMainpanel::PauseResumeBtnHandler");
  // only pause when working, only resume when paused
  assert((total_status==jfp_WORKING) || (total_status==jfp_PAUSED));
  // pause
  if (total_status==jfp_WORKING) {
    pauseresume_btn->Enable(false);
    SetStatus(jfp_PAUSING);
    if (IsStatusActive(new_status,true)) {
      new_status = jfp_PAUSING;
      new_pause->Pause();
    }
    if (IsStatusActive(old_status,true)) {
      old_status = jfp_PAUSING;
      auth_pause->Pause();
    }
    rpause = false;
  }
  // resume
  else {
    pauseresume_btn->SetLabel(wxT("Pause"));
    pauseresume_btn->Enable(false);
    if (!IsStatusActive(new_status,true)) new_pause->Resume();
    if (!IsStatusActive(old_status,true)) auth_pause->Resume();
  }
}
//---------------------------------------------------
void jfUpDMainpanel::SpecialStopHandler(wxCommandEvent& event) {
  new_pause->SpecialStop();
  auth_pause->SpecialStop();
  sstop = true;
  specialstop_btn->Enable(false);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handlers for events that come from the getter
//----------------------------------------------------
void jfUpDMainpanel::HandleRetry(wxCommandEvent& event) {
  // we will ignore retry events
}
//----------------------------------------------------
void jfUpDMainpanel::HandleRequestPause(wxCommandEvent& event) {
  if (total_status==jfp_WORKING) {
    pauseresume_btn->Enable(false);
    SetStatus(jfp_PAUSING);
    if (IsStatusActive(new_status,true)) {
      new_status = jfp_PAUSING;
      new_pause->Pause();
    }
    if (IsStatusActive(old_status,true)) {
      old_status = jfp_PAUSING;
      auth_pause->Pause();
    }
    rpause = true;
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handlers for events that come from the pauser
//----------------------------------------------------
void jfUpDMainpanel::HandlePause(wxCommandEvent& event) {
  const wxString fname = wxT("jfUpDMainpanel::HandlePause");
  // starting...
  assert(total_status==jfp_PAUSING);
  // evaluating
  bool toption = (bool)(event.GetInt());
  if (toption) old_status = jfp_PAUSED;
  else new_status = jfp_PAUSED;
  // pausing logic
  if (Both(jfp_PAUSED) || (!IsStatusActive(new_status)) || (!IsStatusActive(old_status))) {
    pauseresume_btn->SetLabel(wxT("Resume"));
    SetStatus(jfp_PAUSED);
    if (rpause) stat_label->SetLabel(wxT("AUTO PAUSED"));
    pauseresume_btn->Enable();
    rpause = false;
  }
}
//----------------------------------------------------
void jfUpDMainpanel::HandleResume(wxCommandEvent& event) {
  const wxString fname = wxT("jfUpDMainpanel::HandleResume");
  // starting...
  bool toption = (bool)(event.GetInt());
  if (toption) old_status = jfp_WORKING;
  else new_status = jfp_WORKING;
  // pausing logic
  if (Both(jfp_WORKING) || (!IsStatusActive(new_status)) || (!IsStatusActive(old_status))) {
    pauseresume_btn->Enable(true);
    SetStatus(jfp_WORKING);
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handlers for thread events
//----------------------------------------------------
void jfUpDMainpanel::HandleStart(jfThreadEvent& tevent) {
  const wxString fname = wxT("jfUpDMainpanel::HandleStart");
  if (tevent.GetOption()) old_status = jfp_WORKING;
  else new_status = jfp_WORKING;
  if (Both(jfp_WORKING)) {
    updating = true;
    startstop_btn->SetLabel(wxT("Stop"));
    pauseresume_btn->Enable(true);
    startstop_btn->Enable(true);
    specialstop_btn->Enable(true);
    SetStatus(jfp_WORKING);
  }
}
//----------------------------------------------------
void jfUpDMainpanel::HandleEnd(jfThreadEvent& tevent) {
  const wxString fname = wxT("jfUpDMainpanel::HandleEnd");
  /**/jfXLogMessage(fname,1);
  if (tevent.GetOption()) old_status = jfp_DONE;
  else new_status = jfp_DONE;
  /**/jfXLogMessage(fname,2);
  EndActions();
  /**/jfXLogMessage(fname,3);
}
//----------------------------------------------------
void jfUpDMainpanel::HandleTotalCount(jfThreadEvent& tevent) {
  const wxString fname = wxT("jfUpDMainpanel::HandleTotalCount");
  if (tevent.GetOption()) oldfic_stats->SetPageCount(tevent.GetInt1());
  else newfic_stats->SetPageCount(tevent.GetInt1());
}
//----------------------------------------------------
void jfUpDMainpanel::HandleLabels(jfThreadEvent& tevent) {
  const wxString fname = wxT("jfUpDMainpanel::HandleLabels");
  if (tevent.GetOption()) oldfic_stats->SetPageLabels(tevent.GetString1(),tevent.GetString2());
  else newfic_stats->SetPageLabels(tevent.GetString1(),tevent.GetString2());
}
//----------------------------------------------------
void jfUpDMainpanel::HandleNewItem(jfThreadEvent& tevent) {
  const wxString fname = wxT("jfUpDMainpanel::HandleNewItem");
  // variables
  int p1 = tevent.GetInt1();
  int p2 = tevent.GetInt2();
  wxString pname = tevent.GetString1();
  /**/jfXLogMessage(fname,1);
  // starting
  if (tevent.GetOption()) oldfic_stats->SetNew(p1,p2,pname);
  else newfic_stats->SetNew(p1,p2,pname);
  /**/jfXLogMessage(fname,2);
}
//----------------------------------------------------
void jfUpDMainpanel::HandleAction(jfThreadEvent& tevent) {
  const wxString fname = wxT("jfUpDMainpanel::HandleAction");
  if (tevent.GetOption()) oldfic_stats->ChangeAction(tevent.GetString1());
  else newfic_stats->ChangeAction(tevent.GetString1());
}
//----------------------------------------------------
void jfUpDMainpanel::HandleDone(jfThreadEvent& tevent) {
  const wxString fname = wxT("jfUpDMainpanel::HandleDone");
  // variables
  jfDown::ResultAction res_action;
  wxString fres;
  wxString errstr;
  bool okay;
  // starting
  /**/jfXLogMessage(fname,1);
  res_action = tevent.GetResultAction();
  okay = ((res_action==jfDown::PROCEED)||(res_action==jfDown::SKIP));
  // things are okay
  if (okay) {
    /**/jfXLogMessage(fname,2);
    fres = (res_action==jfDown::SKIP)?(wxT("Skipped")):(wxT("Done with"));
    if (tevent.GetOption()) oldfic_stats->PageDone(fres);
    else newfic_stats->PageDone(fres);
  }
  // things are not okay
  else {
    /**/jfXLogMessage(fname,3);
    // ignore entirely!
  }
}
//----------------------------------------------------
void jfUpDMainpanel::HandleHalt(jfThreadEvent& tevent) {
  const wxString fname = wxT("jfUpDMainpanel::HandleHalt");
  if (tevent.GetOption()) old_status = jfp_HALTED;
  else new_status = jfp_HALTED;
  /**/jfXLogMessage(fname,1);
  EndActions();
  /**/jfXLogMessage(fname,2);
}
//----------------------------------------------------
void jfUpDMainpanel::HandleReset(jfThreadEvent& tevent) {
  // ignore
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// category level thread events
//----------------------------------------------------
void jfUpDMainpanel::HandleNewCategory(jfThreadEvent& tevent) {
  const wxString fname = wxT("jfUpDMainpanel::HandleNewCategory");
  if (tevent.GetOption()) {
    oldfic_stats->NewCat(tevent.GetInt1(),tevent.GetInt2(),tevent.GetString1());
  }
  else {
    newfic_stats->NewCat(tevent.GetInt1(),tevent.GetInt2(),tevent.GetString1());
  }
}
//----------------------------------------------------
void jfUpDMainpanel::HandleCategoryEnd(jfThreadEvent& tevent) {
  // variables
  const wxString fname = wxT("jfUpDMainpanel::HandleCategoryEnd");
  jfDown::ResultAction res_action;
  wxString fres;
  wxString errstr;
  bool okay;
  // starting
  /**/jfXLogMessage(fname,1);
  res_action = tevent.GetResultAction();
  okay = ((res_action==jfDown::PROCEED)||(res_action==jfDown::SKIP));
  // things are okay
  if (okay) {
    /**/jfXLogMessage(fname,2);
    fres = (res_action==jfDown::SKIP)?(wxT("Skipped")):(wxT("Done with"));
    if (tevent.GetOption()) oldfic_stats->CatDone(fres);
    else newfic_stats->CatDone(fres);
  }
  // things are not okay
  else {
    /**/jfXLogMessage(fname,3);
    // assigning the status
    if (tevent.GetOption()) {
      if (res_action==jfDown::HALT) old_status = jfp_HALTED;
      else old_status = jfp_FAILED;
    }
    else {
      if (res_action==jfDown::HALT) new_status = jfp_HALTED;
      else new_status = jfp_FAILED;
    }
    /**/jfXLogMessage(fname,4);
    // halting/stopping, or ordering that
    if (!EndActions()) OrderStop();
    /**/jfXLogMessage(fname,4);
  }
}
//----------------------------------------------------
void jfUpDMainpanel::HandleCategoryCount(jfThreadEvent& tevent) {
  // variables and checks
  const wxString fname = wxT("jfUpDMainpanel::HandleCategoryCount");
  int ncatcount;
  assert(phase==2);
  // we get the info
  ncatcount = tevent.GetInt1();
  // we do the action
  if (tevent.GetOption()) oldfic_stats->SetCatCount(ncatcount);
  else newfic_stats->SetCatCount(ncatcount);
}
//----------------------------------------------------
void jfUpDMainpanel::HandleCategoryLabels(jfThreadEvent& tevent) {
  const wxString fname = wxT("jfUpDMainpanel::HandleCategoryLabels");
  if (tevent.GetOption()) oldfic_stats->SetCatLabels(tevent.GetString1(),tevent.GetString2());
  else newfic_stats->SetCatLabels(tevent.GetString1(),tevent.GetString2());
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// extra methods
bool jfUpDMainpanel::ChangeBHeight(size_t nheight) {
  if (nheight<10) return false;
  newfic_stats->ChangeSizes(nheight,0);
  oldfic_stats->ChangeSizes(nheight,0);
  return true;
}
//-----------------------------------------------------------------------
bool jfUpDMainpanel::ChangeMode(bool tosmall) {
  if (tosmall) {
    if (ssize_config) return false;
    ToSmallSize();
  }
  else {
    if (!ssize_config) return false;
    FromSmallSize();
  }
  return true;
}
//---------------------------------------------------------------------
bool jfUpDMainpanel::OrderStop() {
  const wxString fname = wxT("jfUpDMainpanel::OrderStop");
  // already stopped or stopping
  /**/jfXLogMessage(fname,1);
  if (!updating) return false;
  if (total_status==jfp_STOPPING) return false;
  // disabling some buttons
  if (total_status!=jfp_PAUSING) pauseresume_btn->Enable(false);
  if (total_status==jfp_PAUSED) pauseresume_btn->SetLabel(wxT("Pause"));
  /**/jfXLogMessage(fname,2);
  SetStatus(jfp_STOPPING);
  startstop_btn->Enable(false);
  specialstop_btn->Enable(false);
  sstop = false;
  // doing the stop
  /**/jfXLogMessage(fname,2);
  if (IsStatusActive(old_status,true)) {
    auth_pause->Stop();
    old_status = jfp_STOPPING;
  }
  if (IsStatusActive(new_status,true)) {
    new_pause->Stop();
    new_status = jfp_STOPPING;
  }
  return true;
}
//------------------------------------------------------------------
bool jfUpDMainpanel::Reset(bool initialize) {
  if (updating) return false;
  if (initialize) phase = 0;
  else phase = 1;
  rpause = false;
  sstop = false;
  new_status = old_status = jfp_NOTSTARTED;
  if (initialize) {
    tparams1 = NULL;
    tparams2 = NULL;
  }
  else {
    SetStatus(jfp_NOTSTARTED);
    tparams1->pauselink = NULL;
    delete new_pause;
    tparams2->pauselink = NULL;
    delete auth_pause;
  }
  new_pause = NULL;
  auth_pause = NULL;
  phase1_thread = NULL;
  phase2_newfics = NULL;
  phase2_updold = NULL;
  phase3_thread = NULL;
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods
// creating and arranging the GUI elements
void jfUpDMainpanel::GuiSetup(size_t bsize) {
  // creating the gui elements
  assert(bsize>=0);
  newfic_stats = new jfDualProgressBars(this,wxT("Looking for New Fics"),bsize);
  oldfic_stats = new jfDualProgressBars(this,wxT("Updating Old Fics"),bsize);
  stat_label = new wxStaticText(this,-1,wxT("NOT STARTED"));
  startstop_btn = new wxButton(this,jfID_STARTSTOPBTN,wxT("Start"));
  startstop_btn->Enable(false);
  pauseresume_btn = new wxButton(this,jfID_PAUSERESUME,wxT("Pause"));
  pauseresume_btn->Enable(false);
  specialstop_btn = new wxButton(this,jfID_SSTOPBTN,wxT("Stop After"));
  specialstop_btn->Enable(false);
  // arranging them
  bottom_sizer = new wxBoxSizer(wxHORIZONTAL);
  main_sizer = new wxBoxSizer(wxVERTICAL);
  bar_sizer = NULL;
  // adding elements to the sizer
  bottom_sizer->Add(stat_label,1,wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL,3);
  bottom_sizer->AddSpacer(6);
  bottom_sizer->Add(startstop_btn,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
  bottom_sizer->Add(pauseresume_btn,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
  bottom_sizer->Add(specialstop_btn,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
  main_sizer->Add(newfic_stats,0,wxALL|wxEXPAND,3);
  main_sizer->Add(oldfic_stats,0,wxALL|wxEXPAND,3);
  main_sizer->Add(bottom_sizer,0,wxALL|wxEXPAND,3);
  // finishing touches
  tparams1 = new jfDownloadParams();
  tparams1->msg_target = this;
  tparams2 = new jfDownloadParams();
  tparams2->msg_target = this;
  ssize_config = false;
}
//-----------------------------------------------------------------------------
// we call this method once done to send the signal up to parents
void jfUpDMainpanel::SendDoneSignal(bool success) {
  wxCommandEvent mevent = wxCommandEvent(jf_UPDATE_ALLDONE_EVENT,-1);
  wxWindow* mwindow = GetGrandParent();
  mevent.SetInt((int)success);
  wxPostEvent(mwindow,mevent);
}
//-------------------------------------------------------------------------------
// conversion from regular to small size format
void jfUpDMainpanel::ToSmallSize() {
  assert(!ssize_config);
  // creating the notebook
  ssize_nbook = new wxNotebook(this,-1);
  // clearing out the old sizers
  bottom_sizer->Clear();
  main_sizer->Clear();
  // creating the new small size format sizers
  side_sizer = new wxBoxSizer(wxVERTICAL);
  smain_sizer = new wxBoxSizer(wxHORIZONTAL);
  // getting rid of one sizer
  bottom_sizer = NULL;
  // putting stuff in the side_sizer
  side_sizer->Add(stat_label,0,wxEXPAND|wxALL|wxALIGN_LEFT,3);
  side_sizer->Add(startstop_btn,0,wxALL|wxALIGN_CENTER,3);
  side_sizer->Add(pauseresume_btn,0,wxALL|wxALIGN_CENTER,3);
  side_sizer->Add(specialstop_btn,0,wxALL|wxALIGN_CENTER,3);
  // putting stuff in the new top_sizer
  smain_sizer->Add(side_sizer,0,wxALL|wxEXPAND,1);
  smain_sizer->Add(ssize_nbook,1,wxALL|wxEXPAND,3);
  // notebook panel modification
  newfic_stats->Reparent(ssize_nbook);
  oldfic_stats->Reparent(ssize_nbook);
  // chaning the panels to remove the un-necessary static box wrapper
  newfic_stats->ChangeMode(false);
  oldfic_stats->ChangeMode(false);
  // inserting the panels
  ssize_nbook->AddPage(newfic_stats,newfic_stats->GetStoredLabel());
  ssize_nbook->AddPage(oldfic_stats,oldfic_stats->GetStoredLabel());
  // changing top level sizer
  SetSizer(smain_sizer,false);
  // deleting the old top level sizaer
  delete main_sizer;
  main_sizer = NULL;
  // refresh
  Layout();
  // clearing and refreshing
  ssize_config = true;
  Layout();
  Refresh();
}
//-----------------------------------------------------------------
// conversion from small size to regular format
void jfUpDMainpanel::FromSmallSize() {
  assert(ssize_config);
  // we remove the panels from the notebook
  ssize_nbook->RemovePage(0);
  ssize_nbook->RemovePage(0);
  // reparenting the progress bar panels
  newfic_stats->Reparent(this);
  oldfic_stats->Reparent(this);
  // clearing the small size sizers
  side_sizer->Clear();
  smain_sizer->Clear();
  // deleting the side sizer
  side_sizer = NULL;
  // we no longer need the notebook
  ssize_nbook->Destroy();
  ssize_nbook = NULL;
  // creating large format sizers
  bottom_sizer = new wxBoxSizer(wxHORIZONTAL);
  main_sizer = new wxBoxSizer(wxVERTICAL);
  // adding elements to the sizer
  bottom_sizer->Add(stat_label,1,wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL,3);
  bottom_sizer->AddSpacer(6);
  bottom_sizer->Add(startstop_btn,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
  bottom_sizer->Add(pauseresume_btn,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
  bottom_sizer->Add(specialstop_btn,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
  main_sizer->Add(newfic_stats,0,wxALL|wxEXPAND,3);
  main_sizer->Add(oldfic_stats,0,wxALL|wxEXPAND,3);
  main_sizer->Add(bottom_sizer,0,wxALL|wxEXPAND,3);
  // changing top level sizer
  SetSizer(main_sizer,false);
  // getting rid of old top level sizer
  delete smain_sizer;
  smain_sizer = NULL;
  // refresh
  Layout();
  // chaning the panels to remove the un-necessary static box wrapper
  newfic_stats->ChangeMode(true);
  oldfic_stats->ChangeMode(true);
  // clearing and refreshing
  ssize_config = false;
  Layout();
  Refresh();
}
//--------------------------------------------------------------------------
void jfUpDMainpanel::SetStatus(jf_PPS instatus) {
  if (instatus==jfp_NOTSTARTED) stat_label->SetLabel(wxT("NOT STARTED"));
  else if (instatus==jfp_STARTING) stat_label->SetLabel(wxT("STARTING"));
  else if (instatus==jfp_WORKING) stat_label->SetLabel(wxT("UPDATING"));
  else if (instatus==jfp_PAUSING) stat_label->SetLabel(wxT("PAUSING"));
  else if (instatus==jfp_PAUSED) stat_label->SetLabel(wxT("PAUSED"));
  else if (instatus==jfp_STOPPING) stat_label->SetLabel(wxT("STOPPING"));
  else if (instatus==jfp_HALTED) stat_label->SetLabel(wxT("HALTED"));
  else if (instatus==jfp_FAILED) stat_label->SetLabel(wxT("FAILED"));
  else if (instatus==jfp_DONE) stat_label->SetLabel(wxT("DONE"));
  else assert(false);
  total_status = instatus;
}
//----------------------------------------------------
// just to shorten some lines
bool jfUpDMainpanel::Both(jf_PPS statest) const {
  return ((new_status==statest) && (old_status==statest));
}
//----------------------------------------------------
/* whether we end by error, halting, or finishing, lots of things are the same */
bool jfUpDMainpanel::EndActions() {
  // we are not actually done
  if (IsStatusActive(new_status) || IsStatusActive(old_status)) return false;
  // common "we have stopped" things
  pauseresume_btn->Enable(false);
  startstop_btn->SetLabel(wxT("Start"));
  specialstop_btn->Enable(false);
  updating = false;
  // if anything has failed, the search is an overall faliure
  if ((new_status==jfp_FAILED) || (old_status==jfp_FAILED)) {
    SetStatus(jfp_FAILED);
    SendDoneSignal(false);
  }
  // then there is the "all done" option
  else if (Both(jfp_DONE)) {
    SetStatus(jfp_DONE);
    DoPhase3();
  }
  // here, we have called a halt
  else {
    SetStatus(jfp_HALTED);
    // if the halt is a special stop, with categories done, we produce the results
    if (sstop) DoPhase3();
    else SendDoneSignal(false);
  }
  // done
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// phase 1 methods: the creates the downloader panel and thread
bool jfUpDMainpanel::LaunchPhase1() {
  // variables
  jfLoadOldDialog* xdialog;
  // we start with the dialog
  phase = 1;
  xdialog = new jfLoadOldDialog(this,true,-1);
  // label setting
  stat_label->SetLabel(wxT("LOADING OLD RESULTS"));
  // next we build the thread
  phase1_thread = new jfLOR_Thread();
  phase1_thread->SetData(maindata,xdialog);
  // we show the dialog
  xdialog->Show(true);
  // launching the thread
  phase1_thread->Create();
  phase1_thread->Run();
  // done
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// phase 2 methods. We use this method to create and launch the threads that update old and get new
bool jfUpDMainpanel::DoPhase2() {
  // start
  assert(phase==1);
  wxBeginBusyCursor();
  stat_label->SetLabel(wxT("PREPARING FOR UPDATES"));
  phase = 2;
  new_pause = new jfNewPauser(this);
  auth_pause = new jfNewPauser(this);
  tparams1->pauselink = auth_pause;
  tparams2->pauselink = new_pause;
  // we create the threads
  // the update old fics thread
  phase2_updold = new jfAuthThread();
  phase2_updold->SetBundle(maindata);
  phase2_updold->SetParams(tparams1);
  phase2_updold->Create();
  // the fetch new fics thread
  phase2_newfics = new jfFFNUpdateNewThread();
  phase2_newfics->SetBundle(maindata);
  phase2_newfics->SetParams(tparams2);
  phase2_newfics->Create();
  // done
  phase2_updold->Run();
  phase2_newfics->Run();
  wxEndBusyCursor();
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// phase 3 methods
bool jfUpDMainpanel::DoPhase3() {
  // constants
  const wxString fname = wxT("jfUpDMainpanel::DoPhase3");
  // variables
  jfLoadOldDialog* xdialog;
  // we start with the dialog
  /**/jfXLogMessage(fname,1);
  phase = 3;
  xdialog = new jfLoadOldDialog(this,false,-1);
  // label setting
  stat_label->SetLabel(wxT("MERGING OLD AND NEW RESULTS"));
  // next we build the thread
  phase3_thread = new jfUCombine_Thread();
  phase3_thread->SetData(maindata,xdialog);
  /**/jfXLogMessage(fname,2);
  // we show the dialog
  xdialog->Show(true);
  // launching the thread
  phase3_thread->Create();
  phase3_thread->Run();
  /**/jfXLogMessage(fname,3);
  // done
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfUpDMainpanel,wxPanel)
  EVT_COMMAND (-1,jf_UPDATE_OLDUPLOADED_EVENT ,jfUpDMainpanel::EventPhase1Done    )
  EVT_BUTTON (jfID_STARTSTOPBTN, jfUpDMainpanel::StartStopBtnHandler  )
  EVT_BUTTON (jfID_PAUSERESUME , jfUpDMainpanel::PauseResumeBtnHandler)
  EVT_BUTTON (jfID_SSTOPBTN    , jfUpDMainpanel::SpecialStopHandler   )
  EVT_COMMAND (-1,jf_DOWNL_RETRY_EVENT ,jfUpDMainpanel::HandleRetry       )
  EVT_COMMAND (-1,jf_DOWNL_RPAUSE_EVENT,jfUpDMainpanel::HandleRequestPause)
  EVT_COMMAND (-1,jf_DOWNL_PAUSE_EVENT ,jfUpDMainpanel::HandlePause )
  EVT_COMMAND (-1,jf_DOWNL_RESUME_EVENT,jfUpDMainpanel::HandleResume)
  EVT_JF_THREAD (-1,jf_DOWNL_START_EVENT ,jfUpDMainpanel::HandleStart     )
  EVT_JF_THREAD (-1,jf_DOWNL_END_EVENT   ,jfUpDMainpanel::HandleEnd       )
  EVT_JF_THREAD (-1,jf_DOWNL_PCOUNT_EVENT,jfUpDMainpanel::HandleTotalCount)
  EVT_JF_THREAD (-1,jf_DOWNL_LABELS_EVENT,jfUpDMainpanel::HandleLabels    )
  EVT_JF_THREAD (-1,jf_DOWNL_PSTART_EVENT,jfUpDMainpanel::HandleNewItem   )
  EVT_JF_THREAD (-1,jf_DOWNL_PPROC_EVENT ,jfUpDMainpanel::HandleAction    )
  EVT_JF_THREAD (-1,jf_DOWNL_PEND_EVENT  ,jfUpDMainpanel::HandleDone )
  EVT_JF_THREAD (-1,jf_DOWNL_PHALT_EVENT ,jfUpDMainpanel::HandleHalt      )
  EVT_JF_THREAD (-1,jf_DOWNL_RESET_EVENT ,jfUpDMainpanel::HandleReset     )
  EVT_JF_THREAD (-1,jf_DOWNL_CATSTART_EVENT ,jfUpDMainpanel::HandleNewCategory    )
  EVT_JF_THREAD (-1,jf_DOWNL_CATEND_EVENT   ,jfUpDMainpanel::HandleCategoryEnd    )
  EVT_JF_THREAD (-1,jf_DOWNL_CATCOUNT_EVENT ,jfUpDMainpanel::HandleCategoryCount  )
  EVT_JF_THREAD (-1,jf_DOWNL_CATLABELS_EVENT,jfUpDMainpanel::HandleCategoryLabels )
END_EVENT_TABLE()


/*****************************************************************************/
