/******************************************************************************
* Name:        upd_mainpanel.h
* Author :     John Q Metro
* Purpose :    Update interface main panel
* Created:     March 3, 2011
* Updated:     January 17, 2013
******************************************************************************/
#ifndef UPD_MAINPANEL_H_INCLUDED
#define UPD_MAINPANEL_H_INCLUDED
#endif // UPD_MAINPANEL_H_INCLUDED
//-----------------------------------------------------------------
#ifndef RES_BUNDLE_H_INCLUDED
  #include "../data/res_bundle.h"
#endif // RES_BUNDLE_H_INCLUDED
#ifndef PPANEL_PARTS_H_INCLUDED
  #include "../../interface/threads/ppanel_parts.h"
#endif // PPANEL_PARTS_H_INCLUDED
#ifndef LOADOLD_TH_H_INCLUDED
  #include "../threads/loadold_th.h"
#endif // LOADOLD_TH_H_INCLUDED
#ifndef NEWF_THREAD_H_INCLUDED
  #include "../threads/newf_thread.h"
#endif // NEWF_THREAD_H_INCLUDED
#ifndef DOWN_DATA_H_INCLUDED
  #include "../../connect/down_data.h"
#endif // DOWN_DATA_H_INCLUDED
#ifndef PROGRESSPANEL_H_INCLUDED
  #include "../../interface/threads/progresspanel.h"
#endif // PROGRESSPANEL_H_INCLUDED

#include <wx/notebook.h>
/******************************************************************************/
// special panel with 2 progress bars
class jfDualProgressBars : public wxPanel {
  public:
    // constructors
    jfDualProgressBars(wxWindow* parent, wxString nlabel, size_t bheight, wxWindowID id = wxID_ANY, const wxString& motif_name = wxT("jfDualProgressBars"));
    void Reset();
    // changing/setting category info
    void SetCatLabels(wxString cat_label, wxString cat_action);
    bool SetCatCount(size_t newcount);
    bool NewCat(size_t in_index, size_t ofcount, wxString name);
    void CatDone(wxString dlabel);
    // changing page info
    void SetPageLabels(wxString page_label, wxString page_action);
    bool SetPageCount(size_t newcount);
    bool SetNew(int index_in, int tcx_in, wxString name);
    void ChangeAction(wxString action);
    bool PageDone(wxString action);
    // size changing
    bool ChangeSizes(size_t bheight, size_t psize);
    bool RevertSizes();
    // getting info
    size_t GetCatCount() const;
    size_t GetCatIndex() const;
    // special GUI manipulation
    bool ChangeMode(bool tosbox = false);
    wxString GetStoredLabel() const;
  protected:
    // the gui elements
    jfLabeledGauge* cat_pbar;
    jfLabeledGauge* part_pbar;
    wxStaticBoxSizer* msizer;
    wxBoxSizer* nl_sizer;
    bool sbox_mode;
    wxString stored_label;
    // internal data
    size_t catcount, catindex;
};
//==============================================================================
class jfAuthThread;
class jfUCombine_Thread;

// the main update panel
class jfUpDMainpanel : public wxPanel {
  public:
    // constructor
    jfUpDMainpanel(wxWindow* parent, jfFFN_UpdateBundle* inmaindata, wxWindowID id = wxID_ANY,const wxString& name = wxT("jfUpDMainpanel"));
    // phase 1 methods
    bool LaunchPhase1();
    // misc events
    void EventPhase1Done(wxCommandEvent& mevent);
    // handlers for events generated from this panel
    void StartStopBtnHandler(wxCommandEvent& event);
    void PauseResumeBtnHandler(wxCommandEvent& event);
    void SpecialStopHandler(wxCommandEvent& event);
    // handlers for events that come from the getter
    void HandleRetry(wxCommandEvent& event);
    void HandleRequestPause(wxCommandEvent& event);
    // handlers for events that come from the pauser
    void HandlePause(wxCommandEvent& event);
    void HandleResume(wxCommandEvent& event);
    // handlers for thread events
    void HandleStart(jfThreadEvent& tevent);
    void HandleEnd(jfThreadEvent& tevent);
    void HandleTotalCount(jfThreadEvent& tevent);
    void HandleLabels(jfThreadEvent& tevent);
    void HandleNewItem(jfThreadEvent& tevent);
    void HandleAction(jfThreadEvent& tevent);
    void HandleDone(jfThreadEvent& tevent);
    void HandleHalt(jfThreadEvent& tevent);
    void HandleReset(jfThreadEvent& tevent);
    // category level thread events
    void HandleNewCategory(jfThreadEvent& tevent);
    void HandleCategoryEnd(jfThreadEvent& tevent);
    void HandleCategoryCount(jfThreadEvent& tevent);
    void HandleCategoryLabels(jfThreadEvent& tevent);
    // extra methods
    bool ChangeBHeight(size_t nheight);
    bool ChangeMode(bool tosmall);
    bool OrderStop();
    bool Reset(bool initialize);
    bool EndActions();
  protected:
    // helper methods
    void GuiSetup(size_t bsize);
    void SendDoneSignal(bool success);
    void ToSmallSize();
    void FromSmallSize();
    void SetStatus(jf_PPS instatus);
    bool Both(jf_PPS statest) const;
    // phase 2 methods
    bool DoPhase2();
    // phase 3 methods
    bool DoPhase3();
    // the primary data holder
    jfFFN_UpdateBundle* maindata;
    // statuses
    jf_PPS new_status, old_status, total_status;
    size_t phase;
    bool rpause, updating, sstop;
    // phase 1 data: uploding old results
    jfLOR_Thread* phase1_thread;
    // phase 2 is doing the updating
    jfFFNUpdateNewThread* phase2_newfics;
    jfAuthThread*         phase2_updold;
    jfDownloadParams* tparams1;
    jfDownloadParams* tparams2;
    jfNewPauser *new_pause, *auth_pause;
    // phase 3 is merging the results
    jfUCombine_Thread*    phase3_thread;
    // gui elements
    jfDualProgressBars* newfic_stats;
    jfDualProgressBars* oldfic_stats;
    wxStaticText* stat_label;
    wxButton* startstop_btn;
    wxButton* pauseresume_btn;
    wxButton* specialstop_btn;
    // sizers
    wxBoxSizer* bottom_sizer;
    wxBoxSizer* main_sizer;
    wxBoxSizer* bar_sizer;
    // small size configuration stuff
    bool ssize_config;
    wxBoxSizer* side_sizer;
    wxBoxSizer* smain_sizer;
    wxNotebook* ssize_nbook;
    // lots of events to handle
    DECLARE_EVENT_TABLE()

};
/******************************************************************************/
