//**************************************************************************
// Name:        upd_panel2.cpp
// Author :     John Q Metro
// Purpose :    Fanfiction.Net update search, panel 2, for displaying static search info
// Created:     September 11, 2011
// Updated:     November 4, 2012
//**************************************************************************
#ifndef UPD_PANEL2_H_INCLUDED
  #include "upd_panel2.h"
#endif // UPD_PANEL2_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//-------------------------------------
//**************************************************************************
jfFFNUpd_Panel2::jfFFNUpd_Panel2(wxWindow* parent, jfFFNSearch* insearch_data, wxWindowID id,
      const wxString& name):wxPanel(parent,id,wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, name) {
  // the usual basic checks
  assert(parent!=NULL);
  assert(insearch_data!=NULL);
  // starting...
  search_data = insearch_data;
  // creating gui parts
  nam_desc_e = new jfNameDescEditor(this,-1,false,false);
  nam_desc_e->SetFromObj(search_data);
	dirpicker = new jfDirPicker(this,-1,false,(search_data->categories)->base_outputdir);
  category_display = new jfCPickerDispl(this,-1,search_data->GetSelector());
  soptions = new jfFFN_ESearchOptions(this,search_data,true,false);
  filter_display = new jfFFN_DFE(this,-1,search_data);
  // finishing gui parts
  ArrangePanels(1,3);
  category_display->Disable();
  filter_display->Disable();
  soptions->Disable();
  // done
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handler
void jfFFNUpd_Panel2::HandleNewDirectory(wxFileDirPickerEvent& inevent) {
  search_data->categories->base_outputdir = dirpicker->GetPath();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++
// method for making a sizer and arranging the panels
void jfFFNUpd_Panel2::ArrangePanels(size_t prop1,size_t prop2) {
  // checks
  assert(filter_display!=NULL);
  assert(category_display!=NULL);
  // we do things
  mainsizer = new wxBoxSizer(wxHORIZONTAL);
  mainsizer->Add(category_display,prop1,wxEXPAND|wxALL,3);
  mainsizer->Add(filter_display,prop2,wxEXPAND|wxTOP|wxBOTTOM|wxRIGHT,3);
  // top level sizer
  topsizer = new wxBoxSizer(wxVERTICAL);
  topsizer->Add(nam_desc_e,0,wxALL|wxALIGN_LEFT|wxEXPAND,3);
  topsizer->Add(dirpicker,0,wxLEFT|wxRIGHT|wxBOTTOM|wxALIGN_LEFT|wxEXPAND,3);
  topsizer->Add(mainsizer,0,wxALL|wxEXPAND,3);
  topsizer->Add(soptions,0,wxALL|wxEXPAND,2);
  // done
  SetSizer(topsizer);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfFFNUpd_Panel2,wxPanel)
  EVT_DIRPICKER_CHANGED(-1,jfFFNUpd_Panel2::HandleNewDirectory)
END_EVENT_TABLE()
//**************************************************************************
