//****************************************************************************
// Name    :  auth_thread.cpp
// Author  :  John Q Metro
// Purpose :  Thread class that handles updating old fics by checking the author pages.
// Created :  January 22, 2011
// Updated :  January 7, 2014
//****************************************************************************
#ifndef AUTH_THREAD_H_INCLUDED
  #include "auth_thread.h"
#endif // AUTH_THREAD_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef TYPED_DISPLAY_H_INCLUDED
  #include "../../interface/display/typed_display.h"
#endif // TYPED_DISPLAY_H_INCLUDED
#ifndef FFN_THREAD
  #include "../../fanficnet/functions/ffn_thread.h"
#endif
/*****************************************************************************/
bool FFN_AuthorMissing(const wxString* in_pagedata) {
  assert(in_pagedata!=NULL);
  const wxString flagstr = wxT("<hr size=1 noshade>User is no longer an active member.");
  return ((in_pagedata->Find(flagstr))!=wxNOT_FOUND);
}
//==========================================================================
// constructor
jfAuthThread::jfAuthThread():jfMultiThreadMixin<jfThreadWithDownload>() {
  sandres = NULL;
  curr_auth = NULL;
  curr_collect = NULL;
  soption = true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// data to set before we start
bool jfAuthThread::SetBundle(jfFFN_UpdateBundle* inval) {
  if (inval==NULL) return false;
  sandres = inval;
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// doing things
wxThread::ExitCode jfAuthThread::Entry() {
  // constants
  const wxString fname = wxT("jfAuthThread::Entry");
  // variables
  bool rval;
  // checks
  threadname = wxT("AUTH THREAD");
  dconst->pauselink->SetOption(soption);
  dconst->CheckMissing = FFN_AuthorMissing;
  /**/TLog(fname,1);
  SendStart();
  // setting some initial values
  max_catcount = sandres->result_data->NumSCats();
  /**/TLogS(fname,2,max_catcount);
  // setting up some more things
  SendCatCount();
  sandres->auth_data = new jfAuthorCollection();
  sandres->result_data->GoToFirst();
  SendCatLabels(wxT("Old Results Category"),wxT("Updating"));
  SendLabels(wxT("Story"),wxT("Checking"));
  /**/TLog(fname,3);
  // the main loop
  for (catindex=1;catindex<=max_catcount;catindex++) {
    /**/TLogS(fname,4,catindex);
    rval = HandleCatgeory();
    /**/TLog(fname,5);
    if (!rval) break;
    rval = PostCatHaltCheck();
    if (!rval) break;
    /**/TLog(fname,6);
    sandres->result_data->NextCategory();
  }
  /**/TLog(fname,8);
  // done
  if (rval) SendEnd();
  return 0;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// messaging
void jfAuthThread::SendDone(jfDown::ResultAction done_status) {
  // creating
  jfThreadEvent eventx(jf_DOWNL_PEND_EVENT,msg_target->GetId(),soption);
  // setting data
  eventx.SetResAction(done_status);
  if (done_status!=jfDown::PROCEED) {
    eventx.SetError(fres);
    if ((fres==jfDown::PARSE_ERROR) || (fres==jfDown::REDIRECTED)) {
      eventx.SetString2(err_string);
    }
  }
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checks all the fics in a collection
bool jfAuthThread::HandleCatgeory() {
  // constants
  const wxString fname = wxT("jfAuthThread::HandleCatgeory");
  // variables
  bool rval = true;
  const jfFFNSCategory* catval;
  // checks
  /**/TLog(fname,1);
  // getting the current collection
  curr_collect = dynamic_cast<jfFFNItemCollection*>(sandres->result_data->GetCurrPointer());
  assert(curr_collect!=NULL);
  catname = curr_collect->GetCatName();
  SendNewCat(catindex,max_catcount);
  // preparing more for the category
  /**/TLog(fname,2,catname);
  catval = curr_collect->GetCat();
  ficcount = curr_collect->GetCount();
  SendTotalCount(ficcount);
  curr_collect->ResetIndex();
  SendUpdateCat(catval);
  /**/TLogS(fname,3,ficcount);
  // the main checking loop
  for (ficindex=0;ficindex<ficcount;ficindex++) {
    /**/TLogS(fname,4,ficindex);
    curr_collect->NextIndex();
    rval = HandleStory();
    if (!rval) break;
  }
  /**/TLog(fname,5);
  // done
  if (rval) SendCatEnd(jfDown::PROCEED);
  return rval;
}
//----------------------------------------------------------------------------
// here, we check a fic
bool jfAuthThread::HandleStory() {
  // constants
  const wxString fname = wxT("jfAuthThread::HandleStory");
  // variables
  jfFFNItem* citem;
  wxString zbuffer,ficname;
  bool cval,aval,xval,allmissing;
  const jfAuthorStories* prxya;
  jfResultUnit* neval;
  jfResultUnit* mneval;
  // starting
  /**/TLog(fname,1);
  assert(curr_collect!=NULL);
  citem = curr_collect->GetCurrent();
  ficname = (citem->GetName()) + wxT(" by ") + (citem->GetAuthor());
  SendNewItem(ficindex+1,ficcount,ficname);
  /**/TLog(fname,2);
  // checking and possibly downloading the author page
  xval = CheckAndDownload(citem,allmissing);
  if (!xval) {
    /**/TLog(fname,3);
    SendDone(jfDown::FAIL);
    return false;
  }
  else {
    /**/TLog(fname,4);
    // determining if the fic is present or not
    if (!allmissing) {
      /**/TLog(fname,5);
      prxya = sandres->auth_data->GetAuthorStories(curr_auth->GetAuthorID());
      assert(prxya!=NULL);
      cval = prxya->InclFic(citem->GetID());
    }
    else cval = false;
    /**/TLogB(fname,6,cval);
    // the fic is missing
    if (!cval) {
      /**/TLog(fname,7);
      citem->SetMissing();
      mneval = citem->GetResultUnit();
      SendUpdateFic(mneval,2);
    }
    // the fic is not missing
    else {
      /**/TLog(fname,8);
      /* oddly, the wordcount from from a few months ago differs from current
      wordcounts even on fics that have not been updated (no other change). Also
      the rating in the older files is always wrong*/
      citem->UpdateWordcount(prxya->GetItemPtr(citem->GetID()));
      /**/TLog(fname,7,citem->GetAuthor());
      aval = prxya->IsUpdated(citem);
      /**/TLogB(fname,8,aval);
      /**/TLogB(fname,9,prxya->AuthnameChanged());
      if ((aval) || (prxya->AuthnameChanged())) {
        zbuffer = prxya->GetAuthName();
        /**/TLogS(fname,10,zbuffer.length());
        /**/TLog(fname,11,zbuffer);
        if (aval) citem->UpdateFromObj(prxya->GetItemPtr(citem->GetID()));
        else citem->ChangeAuthorname(prxya->GetNewAuthname());
        /**/TLog(fname,12);
        neval = citem->GetResultUnit();
        SendUpdateFic(neval,(aval)?(0):(1));
      }
      else {
        neval = citem->GetResultUnit();
        SendUpdateFic(neval,3);
      }

    }
    SendDone(jfDown::PROCEED);
  }
  return true;
}
//---------------------------------------------------------------------------
bool jfAuthThread::CheckAndDownload(jfFFNItem* initem, bool& missing) {
  // constants
  const wxString fname = wxT("jfAuthThread::CheckAndDownload");
  // variables
  wxString authurl;
  bool aval,xval;
  const jfAuthorInfo* x_auth;
  // the usual checks
  /**/TLog(fname,1);
  assert(initem!=NULL);
  missing = false;
  // getting the author
  curr_auth = initem->GetAuthorInfo();
  assert(curr_auth!=NULL);
  // check to see if we already know the author is missing
  if (sandres->auth_data->MissingAuthor(curr_auth->GetAuthorID())) {
    missing = true;
    return true;
  }
  /**/TLog(fname,2);
  // have we downloaded the author already?
  aval = sandres->auth_data->HasAuthor(curr_auth);
  // if not
  if (!aval) {
    authurl = curr_auth->GetUrl();
    // fetching the author page
    xval = GetUrl(authurl);
    if (xval) {
      // if the author page has been fetched, we try to process it
      xval = (sandres->auth_data->AddAuthor(*page_data,curr_auth));
      delete page_data;
      page_data = NULL;
      // not processed, we throw a parse error
      if (!xval) {
        fres = jfDown::PARSE_ERROR;
        err_string = sandres->auth_data->GetParseErr();
        return false;
      }
    }
    else {
      sandres->auth_data->AddMissing(curr_auth->GetAuthorID());
      missing = true;
    }
  }
  else {
    x_auth = curr_auth;
    curr_auth = sandres->auth_data->GetAuthorInfo(curr_auth->GetAuthorID());
    delete x_auth;
  }
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfAuthThread::SendUpdateCat(const jfFFNSCategory* nczat) {
  // variables, checks, and events
  assert(nczat!=NULL);
  jfHtmlResultCollection_FFN* mtarget;
  mtarget = dynamic_cast<jfHtmlResultCollection_FFN*>(sandres->search_data->cat_results);
  wxCommandEvent eventx(jf_UPDATE_NEXTCAT_SIG, mtarget->GetId());
  // we do
  eventx.SetClientData(const_cast<jfFFNSCategory*>(nczat));
  wxPostEvent(mtarget,eventx);
}
//--------------------------------------------------------------------
void jfAuthThread::SendUpdateFic(jfResultUnit* neval, size_t tval) {
  // variables, checks, and events
  assert(neval!=NULL);
  assert(tval<4);
  jfHtmlResultCollection_FFN* mtarget;
  // starting to build the event
  mtarget = dynamic_cast<jfHtmlResultCollection_FFN*>(sandres->search_data->cat_results);
  wxCommandEvent eventx(jf_UPDATE_UPDATE_FIC, mtarget->GetId());
  // we do
  eventx.SetClientData(neval);
  eventx.SetInt(tval);
  wxPostEvent(mtarget,eventx);
}
/*****************************************************************************/
