/******************************************************************************
* Name:        loadold_th.cpp
* Author :     John Q Metro
* Purpose :    Thread to load the old results to the listbox before we start updating
* Created:     February 3, 2011
* Updated:     November 30, 2013
******************************************************************************/
#ifndef LOADOLD_TH_H_INCLUDED
  #include "loadold_th.h"
#endif // LOADOLD_TH_H_INCLUDED
//------------------------------------
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED

/***************************************************************************/
// constructor
jfPhase13_Thread::jfPhase13_Thread() {
  dataset = false;
  old_data = NULL;
  msg_target = NULL;
  plimcount = 0;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// data setting
//---------------------------------------
bool jfPhase13_Thread::DataIsSet() const {
  return dataset;
}
//---------------------------------------
bool jfPhase13_Thread::SetData(jfFFN_UpdateBundle* odata_in, wxWindow* in_msgtarget) {
  // we check for the validity of the input
  if (in_msgtarget==NULL) return false;
  if (odata_in==NULL) return false;
  if (!odata_in->CheckSourceNNull()) return false;
  // things going okay
  msg_target = in_msgtarget;
  old_data = odata_in;
  dataset = true;
  // done
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// extra methods
/* increments plimcount, and then if it equals inlimit, pauses the thread
then resets the count. Implemented because I noticed the main GUI display
was not keeping up with the load old stuff thread */
void jfPhase13_Thread::LimSleep(size_t inlimit, size_t fcount) {
  // basic stuff
  assert(inlimit>0);
  // starting
  size_t sleeplen = fcount * 120;
  plimcount++;
  if (inlimit==plimcount) {
    sleeplen += 100;
    plimcount = 0;
  }
  Sleep(sleeplen);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// status sender events
//---------------------------------------
void jfPhase13_Thread::SendStart(size_t catcount) {
  // creating
  wxCommandEvent eventx(jf_DOWNL_START_EVENT, msg_target->GetId());
  eventx.SetInt(catcount);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//---------------------------------------
void jfPhase13_Thread::StartCat() {
    // creating
  wxCommandEvent eventx(jf_DOWNL_CATSTART_EVENT, msg_target->GetId());
  // setting data
  eventx.SetInt(catindex);
  eventx.SetString(catname);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//---------------------------------------
void jfPhase13_Thread::EndCat() {
  // creating
  wxCommandEvent eventx(jf_DOWNL_CATEND_EVENT, msg_target->GetId());
  // setting data
  eventx.SetInt(catindex);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//**************************************************************************
// constructor
jfLOR_Thread::jfLOR_Thread():jfPhase13_Thread() {
  emptcount = 0;
  ecatdata = NULL;
}
//----------------------------------------------------------------------------
// the main thread
wxThread::ExitCode jfLOR_Thread::Entry() {
  // constants and checks
  const wxString fname = wxT("wxThread::ExitCode jfLOR_Thread::Entry");
  assert(dataset);
  // variables
  size_t cmaxc,numitems;
  jfFFNItemCollection* temp;
  // we start
  /**/jfXLogMessage(fname,1);
  cmaxc = old_data->result_data->NumSCats();
  ecatdata = new jfBulk_EmptyCatInfo();
  SendStart(cmaxc);
  old_data->result_data->GoToFirst();
  /**/jfXLogMessageS(fname,2,cmaxc);
  // the main loop
  for(catindex=1;catindex<=cmaxc;catindex++) {
    // getting current category data
    temp = dynamic_cast<jfFFNItemCollection*>(old_data->result_data->GetCurrPointer());
    ocatname = catname;
    catname = temp->GetName();
    numitems = temp->GetCount();
    // empty category, not full
    if ((numitems==0) && (!ecatdata->IsFull())) {
      temp->LoadIntoBCat(ecatdata);
      emptcount++;
    }
    // either non-empty category, or the bulk empty is full
    else {
      if (emptcount>0) {
        // we send the bulk empty category stuff
        old_data->search_data->SendBulkECat(ecatdata);
        SendBulkECat();
        // resetting the bulk data
        emptcount = 0;
        ecatdata = new jfBulk_EmptyCatInfo();
      }
      // having dealt with the old empties, we start with the new
      if (numitems==0) {
        temp->LoadIntoBCat(ecatdata);
        emptcount++;
      }
      // a non-empty category, hurrah!
      else {
        Sleep(100);
        StartCat();
        temp->SendCore();
        EndCat();
      }
    }
    // next category
    old_data->result_data->NextCategory();
    LimSleep(1,numitems);
  }
  // handling any leftover empty categories
  if ((ecatdata->NumCats())!=0) {
    old_data->search_data->SendBulkECat(ecatdata);
    SendBulkECat();
  }
  else delete ecatdata;
  ecatdata = NULL;
  emptcount = 0;
  // done
  /**/jfXLogMessage(fname,3);
  SendEnd();
  /**/jfXLogMessage(fname,4);
  return 0;
}
//---------------------------------------
void jfLOR_Thread::SendEnd() {
  // creating
  wxCommandEvent eventx(jf_UPDATE_AUXTHREAD_DONE, msg_target->GetId());
  // send the insert flags
  old_data->search_data->SendInsertSignal();
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//----------------------------------------
void jfLOR_Thread::SendBulkECat() {
  wxCommandEvent eventx(jf_OLDLOAD_BULKECATS_EVENT, msg_target->GetId());
  eventx.SetString(ocatname);
  eventx.SetInt(emptcount);
  wxPostEvent(msg_target,eventx);
}
/***************************************************************************/
