//****************************************************************************
// Name    :  newf_thread.cpp
// Author  :  John Q Metro
// Purpose :  Thread class that handles getting new fics, has a threshold limit
// Created :  January 26, 2011
// Updated :  January 7, 2014
//****************************************************************************
#ifndef NEWF_THREAD_H_INCLUDED
  #include "newf_thread.h"
#endif // NEWF_THREAD_H_INCLUDED
#ifndef FFN_THREAD
  #include "../../fanficnet/functions/ffn_thread.h"
#endif

//--------------------------------------------

/****************************************************************************/
jfFFNUpdateNewThread::jfFFNUpdateNewThread() {
  thelimiter = NULL;
  sandres = NULL;
  or_count = 0;
  or_index = 0;
  coldcat = false;
  soption = false;
}
//++++++++++++++++++++++++++++++++++++++++++++++
// data setting
bool jfFFNUpdateNewThread::SetBundle(jfFFN_UpdateBundle* inval) {
  if (inval==NULL) return false;
  sandres = inval;
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++
// a reminder
wxThread::ExitCode jfFFNUpdateNewThread::Entry() {
  // funcname
  const wxString fname = wxT("jfFFNUpdateNewThread::Entry");
  bool rval;
  // starting...
  threadname = wxT("NEW FFN");
  // quite a lot of setup is needed for this
  /**/TLog(fname,1);
  dconst->CheckMissing = FFNPageCheckMissing;
  dconst->CheckTruncated = FFNPageCheck;
  dconst->pauselink->SetOption(soption);
  SendStart();
  /**/TLog(fname,2);
  sandres->search_data->ApplySelection();
  sandres->search_data->ResetIndex();
  max_catcount = sandres->search_data->GetNumCats();
  /**/TLogS(fname,3,max_catcount);
  SendCatCount();
  sandres->new_data = new jfFFN_NewResCollection(sandres->search_data);
  thelimiter = new jfFicLimit(sandres->lastdate,0);
  SendCatLabels(wxT("Category"),wxT("Searching"));
  SendLabels(wxT("Page"),wxT("Fetching"));
  /**/TLog(fname,4);
  // the main category level loop
  while(sandres->search_data->NextIndex()) {
    /**/TLog(fname,5);
    rval = GetCategory();
    /**/TLog(fname,6);
    if (!rval) break;
    rval = PostCatHaltCheck();
    if (!rval) break;
  }
  // prepare stop
  if (rval) SendEnd();
  /**/TLog(fname,7);
  return 0;
}
//++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual plug-in methods to handle some vital bits
//-------------------------------------------
void jfFFNUpdateNewThread::PreFetch1() {
  SendNewItem(pageindex,epagecount);
}
//-------------------------------------------
void jfFFNUpdateNewThread::PreFetch2() {}
//-------------------------------------------
void jfFFNUpdateNewThread::SetUrl() {
  fetchurl = sandres->search_data->GetIndexUrl(pageindex);
}
//-------------------------------------------
bool jfFFNUpdateNewThread::ProcessPage() {
  // constants and variables
  const wxString fname = wxT("jfFFNUpdateNewThread::ProcessPage");
  bool rval;
  const jfFFNSCategory* icat;
  /**/TLog(fname,1);
  // here, we will process the first page *and* make the result collection
  if (pageindex==1) {
    // processing
    icat = sandres->search_data->GetCurrCat();
    rval = (sandres->new_data->MakeItemCollection(*page_data,icat,thelimiter,err_string));
  }
  // otherwise, just add the page
  else {
    rval = sandres->new_data->AddPage(*page_data,err_string);
  }
  // done
  return rval;
}
//-------------------------------------------
size_t jfFFNUpdateNewThread::NewPagecount() const {
  return sandres->new_data->PageTotal();
}
//-------------------------------------------
void jfFFNUpdateNewThread::PostPageAction() {
  // limit checking
  limitr = sandres->new_data->LimitReached(sandres->new_data->SCatIndex());
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// custom internal method
bool jfFFNUpdateNewThread::GetCategory() {
    // constants
  const wxString fname = wxT("jfFFNUpdateNewThread::GetCategory");
  // variables
  bool flastcat;
  const jfFFNSCategory* ccat;
  size_t newlimit;
  bool xres;
  // asserts and checks
  // here we handle stuff at the start of category fetching
  catindex++;
  /**/TLogS(fname,1,catindex);
  flastcat = sandres->search_data->LastCat();
  ccat = sandres->search_data->GetCurrCat();
  catname = ccat->GetName();
  SendNewCat(catindex,max_catcount);
  limitr = false;
  /**/TLog(fname,2,catname);
  // getting the limit index for the limiter
  // we assume that the old result categories are a an in-order subset of the new
  if (sandres->SameCategory(or_index,ccat)) {
    /**/TLog(fname,3);
    // CHANGING THE LIMIT
    newlimit = sandres->GetIDLimit(or_index);
    thelimiter->ChangeLimit(newlimit,false);
    /**/TLogS(fname,4,or_index);
    // getting the old organizer and incrementing the index...
    coldcat = true;
    or_index++;
  }
  else {
    /**/TLog(fname,5);
    coldcat = false;
    thelimiter->ChangeLimit(0,true);
  }
  /**/TLog(fname,6);
  // getting the pages
  xres = LoopGet();
  // handling the result
  PostCatSend(true);
  /**/TLogB(fname,7,xres);
  // done
  if (xres) return true;
  else if (fres==jfDown::ABORTED) return false;
  else return (pageindex==1);
}
/****************************************************************************/
