/////////////////////////////////////////////////////////////////////////////
// Name:        ffndown_data.h
// Author :     John Q Metro
// Purpose :    Downloading fics, we need structures to hold the data
// Created:     June 17, 2011
// Updated:     August 18, 2015
/////////////////////////////////////////////////////////////////////////////
#ifndef FFNDOWN_DATA_H_INCLUDED
#define FFNDOWN_DATA_H_INCLUDED
#endif // FFNDOWN_DATA_H_INCLUDED
//------------------------------------------
#ifndef FFNDOWN_PARTS_H_INCLUDED
  #include "ffndown_parts.h"
#endif // FFNDOWN_PARTS_H_INCLUDED
#ifndef SKELSTORE_H_INCLUDED
  #include "../../core/objs/skelstore.h"
#endif // SKELSTORE_H_INCLUDED
#ifndef FICOBJG_H_INCLUDED
  #include "../../fanfic/data/ficobj.h"
#endif // FICOBJG_H_INCLUDED
#ifndef HTMLPARSE2_H_INCLUDED
  #include "../../core/utils/htmlparse2.h"
#endif // HTMLPARSE2_H_INCLUDED

//****************************************************************************
// groups together skeletons and thier outputs
class jfFD_OutStuff : public wxObject {
  public:
    // public data
    jfSkeletonParser* parse;
    jfSkeletonFicPart* base;
    // constructor
    jfFD_OutStuff();
    // public methods
    bool CheckOk() const;
    wxString MakeResult(wxString inskel);
    // file i/o methods
    bool SetNameInfo(wxString inname, wxString indir, size_t num_files);
    bool GetFilename(wxString& outresult, size_t gindex) const;
    bool StartNewFile();
    bool AddToFile(wxString inval);
    bool CloseFile();
    // destructor
    ~jfFD_OutStuff();
  protected:
    // internal methods
    void ClearFiles();
    wxString FormatNumber(size_t inval) const;
    // internal data
    std::vector<wxTextFile*> outfiles;
    wxArrayString* filenames;
    bool fileopen;
    bool infoset;
    wxString outname;
    wxString outdir;
    size_t gcount;
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handles splitting the results into multiple files
class jfFanficSplitter : public wxObject {
  public:
    // constructor
    jfFanficSplitter(size_t sguide_in);
    // adding parts
    bool AddParts(wxString inpart);
    // getting info
    size_t GetPartcount() const;
    size_t GetFilecount() const;
    size_t GetFileIndex(size_t file_idx, bool start) const;
    wxString GetPart(size_t index) const;
    // analysis methods
    size_t CalculateSplit();
    // destructor
    ~jfFanficSplitter();
  protected:
    wxArrayString* prepared_parts;
    std::vector<float> prep_sizes;
    size_t size_guide;
    size_t ofile_count;
    std::vector<size_t> ofile_index;
};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// holds a downloaded fanfic
class jfFanfic : public wxObject {
  public:
    // constructors
    jfFanfic(jf_FICTYPE intype);
    jfFanfic(jfFicExtract* inval);
    // data setting methods
    bool SetFic(jfFicExtract* inval, bool changefilebase=true);
    bool AddPage(wxString inpage);
    jfFicExtract* DoPageOne(wxString inpage);
    jfFicExtract* DoIndexPage(wxString inpage);
    bool SetFileBase(wxString inval, wxString indir, size_t in_sguide);
    // info methods
    size_t PartCount() const;
    bool MultiParts() const;
    bool IsDataset() const;
    bool ValidData() const;
    wxString GetLastError() const;
    bool LoadCoreValues(jfSkeletonParser* inparser) const;
    // output methods
    bool BuildResult();
    bool StartResult();
    bool WriteFile(size_t findex_in);
    void FinishAfter();
    size_t GetFilecount() const;
    // destructors
    ~jfFanfic();
  protected:
    // internal methods
    void ClearStuff();
    wxString PLabelMaker(size_t indexv) const;
    jfFicPart* GetTypedPart(wxString inpage=wxEmptyString);
    // main data
    jf_FICTYPE fictype;
    std::vector<jfFicPart*> maindata;
    jfFicExtract* sdata;
    bool dataset;
    bool noerr;
    wxString filebase;
    wxString dirbase;
    size_t sguide;
    // parsing stuff
    jfSkeletonFicPart* theskel;
    jfFanficSplitter* split_manager;
    jfFD_OutStuff* output_manager;
    bool output_started;

};
//****************************************************************************
