/////////////////////////////////////////////////////////////////////////////
// Name:        ffndown_parts.cpp
// Author :     John Q Metro
// Purpose :    Downloading fics, we need structures to hold the data
// Created:     July 12, 2011
// Updated:     July 8, 2016
/////////////////////////////////////////////////////////////////////////////
#ifndef FFNDOWN_PARTS_H_INCLUDED
  #include "ffndown_parts.h"
#endif // FFNDOWN_PARTS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//---------------------------
//***************************************************************************
jfFicDownParams::jfFicDownParams(wxString in_base, size_t in_sizeguide, jfSkeletonFicPart* in_theskel) {
  // setting the paramter values
  assert(in_theskel!=NULL);
  theskel = in_theskel;
  sizeguide = in_sizeguide;
  base = in_base;
  // null initializing what we can
  check_complete = false;
  extract_data = NULL;
}
//===========================================================================
// constructors
//------------------
jfFicPart::jfFicPart():wxObject() {
  partcount = 0;
  partindex = 0;
  validdata = false;
  sparser = NULL;
  errmsg = wxT("Not started yet");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// other methods
//------------------
bool jfFicPart::IsValid() const {
  return validdata;
}
//------------------
size_t jfFicPart::GetPartcount() const {
  return partcount;
}
//------------------
wxString jfFicPart::GetErr() const {
  return errmsg;
}
//------------------
wxString jfFicPart::GetContents() const {
  return partcontents;
}
//--------------------
void jfFicPart::LoadCoreValues(jfSkeletonParser* inparser) const {
  assert(inparser!=NULL);
  inparser->AddText(wxT("FICPART_NAME"),partname);
  inparser->AddText(wxT("FICPART_TEXT"),partcontents);
  inparser->AddUInt(wxT("FICPART_INDEX"),partindex);
  inparser->AddText(wxT("FICPART_LABEL"),wxString(wxT("Part")+SInt2Str(partindex)));
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfFicPart::SetFromString(wxString inpage) {
  // we start
  sparser = new jfStringParser(inpage);
  if (!SetFromStringCore()) return false;
  // done
  delete sparser;
  sparser = NULL;
  validdata = true;
  return true;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfFicPart::SetParseError(wxString inval) {
  errmsg = inval;
  validdata = false;
  if (sparser!=NULL) delete sparser;
  sparser = NULL;
  return false;
}

//=========================================================================
// constructors
//-----------------------------------------------
jfFicPart_FFN::jfFicPart_FFN():jfFicPart() {
  first = false;
}
//-----------------------------------------------
jfFicPart_FFN::jfFicPart_FFN(wxString inpage):jfFicPart() {
  const  wxString fname = wxT("jfFicPart_FFN::jfFicPart_FFN");
  first = false;
  SetFromString(inpage);
  if (!validdata) jfXLogMessage(fname,1,GetErr());
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfFicExtract_FFN* jfFicPart_FFN::FirstFromString(wxString inpage) {
  const wxString fname = wxT("jfFicPart::FirstFromString");
  // checks
  assert(!validdata);
  // variables
  jfFicExtract_FFN* result = NULL;
  wxString buffer, errout;
  unsigned long qval;
  // we start
  /**/jfXLogMessage(fname,1);
  first = true;
  if (!FirstProcessStart(inpage,buffer)) return NULL;
  // going ahead
  result = new jfFicExtract_FFN();
  result->fic_title = buffer;
  /**/jfXLogMessage(fname,2,buffer);
  // looking for the author id
  if (!sparser->GetDelimitedULong(wxT("<a class='xcontrast_txt' href='/u/"),wxT("/"),qval,errout)) {
    delete result;
    /**/jfXLogMessage(fname,3);
    SetParseError(wxT("Problems getting author id! ") + errout);
    return NULL;
  }
  result->auth_id = qval;
  // getting the author name
  if (!sparser->GetDelimited(wxT("'>"),wxT("</a>"),buffer)) {
    delete result;
    /**/jfXLogMessage(fname,4);
    SetParseError(wxT("Problems getting author name!"));
    return NULL;
  }
  Dtrim(buffer);
  result->author_name = buffer;
  /**/jfXLogMessage(fname,5,buffer);
  if (!DateAndCompletion(result)) {
    delete result;
    /**/jfXLogMessage(fname,6);
    return NULL;
  }
  // we go ahead with the normal things one would expect from set from string
  if (!SetFromStringCore()) {
    delete result;
    /**/jfXLogMessage(fname,7,errmsg);
    return NULL;
  }
  result->fic_id = sid;
  /**/jfXLogMessage(fname,8);
  result->pcount = partcount;
  // checks and clearing up
  assert(partindex==1);
  delete sparser;
  sparser = NULL;
  validdata = true;
  /**/jfXLogMessage(fname,9);
  return result;
}
//------------------------------------------------------------------------
bool jfFicPart_FFN::DateAndCompletion(jfFicExtract_FFN* in_result) {
  // constants
  const wxString fname = wxT("jfFicPart_FFN::DateAndCompletion");
  // asserts and checks
  assert(in_result!=NULL);
  assert(sparser!=NULL);
  // variables
  unsigned long xoval;
  wxString buffer1,xoerr;
  // looking for the updated date (or published date)
  if (!sparser->GetDelimitedULong(wxT("Updated: <span data-xutime='"),wxT("'>"),xoval,xoerr)) {
    if (!sparser->GetDelimitedULong(wxT("Published: <span data-xutime='"),wxT("'>"),xoval,xoerr)) {
      /**/jfXLogMessage(fname,1);
      SetParseError(wxT("Problems getting date! ") + xoerr);
      return false;
    }
  }
  // the time in xoval is a unix time stamp
  in_result->updated_date.Set((time_t)(xoval));
  in_result->updated_date.ResetTime();
  // getting the place where we look for completion status
  if (!sparser->GetDelimited(wxT("</span>"),wxT(" - id:"),buffer1)) {
    /**/jfXLogMessage(fname,2);
    SetParseError(wxT("Cannot find the section after the time!"));
    return false;
  }
  // looking for completion
  in_result->complete = (wxNOT_FOUND!=buffer1.Find(wxT("Status: Complete")));
  // done
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfFicPart_FFN::SetFromStringCore() {
  // a quick check...
  assert(sparser!=NULL);
  // constants
  const wxString fname = wxT("jfFicPart_FFN::SetFromStringCore");
  const wxString ficstart_tag = wxT("<div role='main' aria-label='story content' class='storytextp' id='storytextp'");
  const wxString patscript_tag = wxT("<FORM name=myselect");
  const wxString script_start = wxT("<script>");
  // variables
  wxString buffer,sstart,errout;
  unsigned long oval;
  // part processing
  /**/jfXLogMessage(fname,1);
  if (!PartsProcessing()) return false;
  // doing things
  if (!sparser->MovePast(ficstart_tag)) {
    return SetParseError(wxT("Missing Start Tag"));
  }
  /**/jfXLogMessage(fname,8);
  // we here make the possibly dubious assumption that there are no <div> tags in the actual story
  if (!sparser->GetDelimited(wxT(" id='storytext'>"),wxT("</div>"),partcontents)) {
    return SetParseError(wxT("Cannot find contents"));
  }
  // just handling a small problem here...
  /*
  partcontents.Replace(wxT("</I><I>"),wxT(" "),true);
  partcontents.Replace(wxT("</em><em>"),wxT(" "),true);
  // just in case
  partcontents.Replace(wxT("</i><i>"),wxT(" "),true);
  partcontents.Replace(wxT("</EM><EM>"),wxT(" "),true);
  partcontents.Replace(wxT("</b><b>"),wxT(" "),true);
  partcontents.Replace(wxT("</B><B>"),wxT(" "),true);
  */
  /**/jfXLogMessage(fname,9);
  // just to check...
  if (!sparser->MovePast(patscript_tag)) {
    return SetParseError(wxT("Cannot find separator after style and social script parts"));
  }
  /**/jfXLogMessage(fname,10);
  // next, we look for the current part index...
  if (!sparser->MovePast(script_start)) {
    return SetParseError(wxT("Cannot find fic info script start"));
  }
  // looking for the fic id
  if (!sparser->GetDelimitedULong(wxT("var storyid = "),wxT(";"),oval,errout)) {
    /**/jfXLogMessage(fname,11);
    SetParseError(wxT("Problems getting fic id! ") + errout);
    return NULL;
  }
  sid = oval;
  /**/jfXLogMessage(fname,12);
  if (!sparser->GetDelimited(wxT("var chapter ="),wxT(";"),buffer)) {
    return SetParseError(wxT("Cannot find chapter index"));
  }
  /**/jfXLogMessage(fname,13);
  // buffer should contain the part index, we convert it to an integer
  if (!Str2ULong(buffer,oval)) {
    return SetParseError(wxT("Chapter index is not a number"));
  }
  partindex = oval;
  /**/jfXLogMessageS(fname,14,oval);
  return true;
}
//-------------------------------------------------------------------------
bool jfFicPart_FFN::FirstProcessStart(wxString inval, wxString& out_title) {
  // constants
  const wxString fname = wxT("jfFicPart_FFN::FirstProcessStart");
  const wxString titlehead_tag = wxT(" id=pre_story_links>");
  const wxString icon_tag1 = wxT("<img src='//fast-fictionpressllc.netdna-ssl.com/static/fcons/arrow-switch.png'");
  const wxString icon_tag2 = wxT("<img src='/static/fcons/arrow-switch.png' ");
   // const wxString chev_tag = wxT("<span class='xcontrast_txt icon-chevron-right xicon-section-arrow'>");
  // variables
  wxString buffer;
  bool crossover;
  // we start
  /**/jfXLogMessage(fname,1);
  sparser = new jfStringParser(inval);
  /**/jfXLogMessage(fname,2);
  if (!sparser->MovePast(titlehead_tag)) {
    /**/jfXLogMessage(fname,3);
    return SetParseError(wxT("Missing Special DIV tag!"));
  }
  crossover = sparser->MovePast(icon_tag2);
  /*
  if (!crossover) {
    if (!sparser->MovePast(chev_tag)) {
      return SetParseError(wxT("Missing Icon or Chevron!"));
    }
  }
  */
  /**/jfXLogMessageB(fname,5,crossover);
  // extracting the title
  if (!sparser->GetDelimited(wxT("<b class='xcontrast_txt'>"),wxT("</b>"),buffer)) {
    /**/jfXLogMessage(fname,7);
    return SetParseError(wxT("Cannot find the title!"));
  }
  Dtrim(buffer);
  /**/jfXLogMessage(fname,8,buffer);
  if (buffer==wxEmptyString) {
    return SetParseError(wxT("Title is Empty!"));
  }
  // returning
  out_title = buffer;
  /**/jfXLogMessage(fname,9);
  return true;
}
//---------------------------------------------------------------------
bool jfFicPart_FFN::PartsProcessing() {
  // constants and variables
  const wxString chapt_start = wxT("<SELECT id=chap_select title=\"Chapter Navigation\"");
  size_t part_counter = 0;
  jfStringParser* opt_parser;
  wxString extopt,bufferx;
  size_t selcount = 0;
  // a quick check...
  assert(sparser!=NULL);
  // checking first...
  if (!sparser->MovePast(chapt_start)) {
    // only one part
    partcount = 1;
    partname = wxEmptyString;
    return true;
  }
  // here, we have multiple parts
  if (!sparser->GetDelimited(wxT(">"),wxT("/select>"),extopt)) {
    return SetParseError(wxT("Could not extract Chapter Navigation!"));
  }
  opt_parser = new jfStringParser(extopt);
  // looping for each chapter
  while (opt_parser->MovePastTwo(wxT("option"),wxT("value="))) {
    part_counter++;
    // looking for the selected item
    if (!opt_parser->GetMovePast(wxT(">"),bufferx)) {
      delete opt_parser;
      return SetParseError(wxT("End of Option (chapter selector) not found!"));
    }
    if (bufferx.Find(wxT("selected"))!=wxNOT_FOUND) {
      if (!opt_parser->GetDelimited(wxT("."),wxT("<"),partname)) {
        delete opt_parser;
        return SetParseError(wxT("Failed at getting chapter label"));
      }
      selcount++;
    }
  }
  // post loop checking
  delete opt_parser;
  if (part_counter==0) return SetParseError(wxT("Chapter selector ugly."));
  if (selcount!=1) return SetParseError(wxT("Selected Chapter off."));
  // no errors
  partcount = part_counter;
  return true;
}
//***************************************************************************
