/////////////////////////////////////////////////////////////////////////////
// Name:        ffndown_data.cpp
// Author :     John Q Metro
// Purpose :    Downloading fics, we need structures to hold the data
// Created:     June 19, 2011
// Updated:     July 7, 2016 (Mediaminer.org changes)
/////////////////////////////////////////////////////////////////////////////
#ifndef FFNDOWN_DATA_H_INCLUDED
  #include "ffndown_data.h"
#endif // FFNDOWN_DATA_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef SKELCOLL_H_INCLUDED
  #include "../../core/structs/skelcoll.h"
#endif // SKELCOLL_H_INCLUDED
#ifndef UTILS3_H_INCLUDED
#include "../../core/utils/utils3.h"
#endif // UTILS3_H_INCLUDED
#ifndef FICDOWN_PARTS2_H_INCLUDED
  #include "ficdown_parts2.h"
#endif // FICDOWN_PARTS2_H_INCLUDED
#ifndef FICDOWN_PARTS3_H_INCLUDED
  #include "ficdown_parts3.h"
#endif // FICDOWN_PARTS3_H_INCLUDED

//------------------------------------------

//****************************************************************************
/*
// groups together skeletons and thier outputs
class jfFD_OutStuff : public wxObject {
  public:
    // public data
    jfSkeletonParser* parse;
    jfSkeletonFicPart* base;
    */
// constructor
jfFD_OutStuff::jfFD_OutStuff() {
  parse = NULL;
  base = NULL;
  fileopen = false;
  infoset = false;
  filenames = NULL;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// public methods
//--------------------------
bool jfFD_OutStuff::CheckOk() const {
  if (parse==NULL) return false;
  if (base==NULL) return false;
  return true;
}
//--------------------------
wxString jfFD_OutStuff::MakeResult(wxString inskel) {
  assert(CheckOk());
  // doing the parsing
  parse->SetSkeleton(inskel);
  parse->ProduceIntermediate();
  parse->MakeResult();
  return parse->GetResult();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o methods
//--------------------------
bool jfFD_OutStuff::SetNameInfo(wxString inname, wxString indir, size_t num_files) {
  // check the input
  if (num_files<1) return false;
  if (inname.IsEmpty()) return false;
  // local variables
  wxString trailer,ofname;
  size_t findex;
  // if file output is/has ocurred, we just erase and start again
  ClearFiles();
  // setting
  outname = inname;
  outdir = indir;
  gcount = num_files;
  filenames = new wxArrayString();
  for (findex=1;findex<=gcount;findex++) {
    if (gcount>1) trailer = FormatNumber(findex);
    trailer += wxT(".html");
    if (gcount>1) trailer = wxT("_") + trailer;
    wxString ofname = outname + trailer;
    filenames->Add(ofname);
  }
  infoset = true;
  return true;
}
//----------------------------
bool jfFD_OutStuff::GetFilename(wxString& outresult, size_t gindex) const {
  // constants
  const wxString fname = wxT("jfFD_OutStuff::GetFilename");
  // stuff for which we return false
  /**/jfXLogMessage(fname,1);
  if (!infoset) return false;
  /**/jfXLogMessageS(fname,2,gindex);
  if (gindex==0) return false;
  /**/jfXLogMessageS(fname,3,gcount);
  if (gindex>gcount) return false;
  /**/jfXLogMessage(fname,4);
  // returning true
  outresult = (*filenames)[gindex-1];
  /**/jfXLogMessage(fname,5);
  return true;
}
//--------------------------
bool jfFD_OutStuff::StartNewFile() {
  // first, check if the pervious file is still open
  if (fileopen) return false;
  // information must be set
  if (!infoset) return false;
  // we should not exceed the preestablished filecount
  if (outfiles.size()==gcount) return false;
  // for the new file, we start with getting the name
  wxString ofname;
  if (!outdir.IsEmpty()) ofname = outdir + wxT("/");
  ofname += (*filenames)[outfiles.size()];
  // next up, we create the outfile file
  wxTextFile* newfile = new wxTextFile(ofname);
  // opening
  if (newfile->Exists()) {
    newfile->Open();
    newfile->Clear();
  }
  else {
    newfile->Create();
    newfile->Open();
  }
  // finishing off things
  outfiles.push_back(newfile);
  fileopen = true;
  return true;
}
//--------------------------
bool jfFD_OutStuff::AddToFile(wxString inval) {
  // problems
  if (!fileopen) return false;
  if (outfiles.size()==0) return false;
  // no problem
  outfiles[outfiles.size()-1]->AddLine(inval);
  return true;
}
//--------------------------
bool jfFD_OutStuff::CloseFile() {
  // problems
  if (!fileopen) return false;
  if (outfiles.size()==0) return false;
  // no problem
  outfiles[outfiles.size()-1]->Write();
  outfiles[outfiles.size()-1]->Close();
  fileopen = false;
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructor
jfFD_OutStuff::~jfFD_OutStuff() {
  ClearFiles();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfFD_OutStuff::ClearFiles() {
  size_t findex, fcount;
  fcount = outfiles.size();
  if (fcount>0) {
    if (fileopen) CloseFile();
    for (findex=0;findex<fcount;findex++) {
      delete outfiles[findex];
      outfiles[findex] = NULL;
    }
    outfiles.clear();
  }
  if (filenames!=NULL) {
    delete filenames;
    filenames = NULL;
  }
}
//-----------------------------------------------------------------------
// takes the input number, 0 pads it to a certain width
wxString jfFD_OutStuff::FormatNumber(size_t inval) const {
  // faliures
  assert(inval>0);
  assert(gcount>1);
  // getting the number of digits
  size_t num_digits = size_t(ceil(log10((double)gcount)));
  // building the format string
  wxString format = wxT("%0") + SInt2Str(num_digits) + wxT("u");
  // producting the results
  return wxString::Format(format,inval);
}

//============================================================================
// constructor
jfFanficSplitter::jfFanficSplitter(size_t sguide_in) {
  assert(sguide_in>0);
  prepared_parts = NULL;
  size_guide = sguide_in;
  ofile_count = 0;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++
// adding parts
bool jfFanficSplitter::AddParts(wxString inpart) {
  size_t lval,lvallen;
  float rize;
  if (prepared_parts==NULL) prepared_parts = new wxArrayString();
  prepared_parts->Add(inpart);
  lval = (prepared_parts->GetCount()) - 1;
  lvallen = (*prepared_parts)[lval].length();
  rize = float(lvallen)/1024;
  prep_sizes.push_back(rize);
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting info
//-----------------------------
size_t jfFanficSplitter::GetPartcount() const {
  if (prepared_parts == NULL) return 0;
  else return (prepared_parts->GetCount());
}
//-----------------------------
size_t jfFanficSplitter::GetFilecount() const {
  return ofile_count;
}
//---------------------------------------------------
// returns the part index of each output file (1 indexed)
size_t jfFanficSplitter::GetFileIndex(size_t file_idx, bool start) const {
  // bad input
  if (file_idx==0) return 0;
  if (file_idx>ofile_count) return 0;
  // we do the result
  // startindex is easy
  if (start) return ofile_index[file_idx-1];
  // endindex is a bit more complicated
  else {
    if (file_idx==ofile_count) return GetPartcount();
    else return ofile_index[file_idx]-1;
  }
}
//-----------------------------------------------------------
// gets back the part text (1 indexed)
wxString jfFanficSplitter::GetPart(size_t index) const {
  // bad results
  if (prepared_parts==NULL) return wxEmptyString;
  if (index==0) return wxEmptyString;
  if (index>(prepared_parts->Count())) return wxEmptyString;
  // goo result
  return (*prepared_parts)[index-1];
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++
// analysis methods
size_t jfFanficSplitter::CalculateSplit() {
  // variables
  size_t pcount;
  size_t pindex,gindex;
  float sumsize,mixsize,runtotal;
  float avg_psize, pmult;
  // we start with special cases
  pcount = GetPartcount();
  if (pcount==0) return 0;
  if (pcount == 1) {
    ofile_count = 1;
    ofile_index.push_back(1);
    return 1;
  }
  // more general case, first we estimate the size of an even split
  sumsize = 0;
  for (pindex=0;pindex<pcount;pindex++) sumsize+=prep_sizes[pindex];
  /* Since we only measure part size, fics with many small parts can result in
  too large files. We attempt to compensate for that here. */
  avg_psize = sumsize / ((float)pcount);
  pmult = 7.0 / avg_psize;
  if (pmult<1) pmult = 1;
  // the final file count
  ofile_count = ceil((sumsize/size_guide)*pmult);
  /* we next loop thru the parts (if more than 1 output file), summing
  sizes to get the part that starts each file */
  if (ofile_count==1) ofile_index.push_back(1);
  else {
    mixsize = sumsize/(float(ofile_count));
    pindex = 0;
    for (gindex=0;gindex<ofile_count;gindex++) {
      /* special case : pindex is at the maximum already, so we should reduce
      ofile_count. This is usually caused by very big parts (>size_guide) */
      if (pindex>=pcount) {
        ofile_count = gindex;
        break;
      }
      // moving with the rest
      runtotal = 0;
      ofile_index.push_back(pindex+1);
      for (;pindex<pcount;pindex++) {
        runtotal += prep_sizes[pindex];
        if (runtotal>mixsize) {
          pindex++;
          break;
        }
      }
      /* Before the next loop, recalculating mixsize because runtotal is always
       more than mixsize, which without recalculation can result in later
       groups/files being undersized or even empty. */
      sumsize -= runtotal;
      mixsize = sumsize/(float(ofile_count-gindex-1));
    }
  }
  // done
  return ofile_count;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructor
jfFanficSplitter::~jfFanficSplitter() {
  if (prepared_parts != NULL) {
    delete prepared_parts;
    prepared_parts = NULL;
  }
}
//============================================================================
// constructors
//------------------
jfFanfic::jfFanfic(jf_FICTYPE intype):wxObject() {
  sdata = NULL;
  dataset = false;
  noerr = true;
  sguide = 500;
  theskel = mainskelc->AtIndexF(2);
  split_manager = NULL;
  output_manager = NULL;
  output_started = false;
  fictype = intype;
}
//------------------
jfFanfic::jfFanfic(jfFicExtract* inval) {
  sdata = NULL;
  dataset = false;
  noerr = true;
  sguide = 500;
  theskel = mainskelc->AtIndexF(2);
  split_manager = NULL;
  output_manager = NULL;
  output_started = false;
  if (inval!=NULL) SetFic(inval);
  fictype = inval->GetFicType();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// data setting methods
//---------------------------------------------
bool jfFanfic::SetFic(jfFicExtract* inval, bool changefilebase) {
  const wxString fname = wxT("jfFanfic::SetFic");
  if (inval==NULL) return false;
  ClearStuff();
  sdata = inval;
  dataset = true;
  if (changefilebase) {
    filebase = UBReplace(inval->fic_title,UFNAME_DELIMS);
    sdata->fname_core = filebase;
  }
  return true;
}
//--------------------------------------------
bool jfFanfic::AddPage(wxString inpage) {
  const wxString fname = wxT("jfFanfic::AddPage");
  jfFicPart* newpart;
  if (!dataset) return false;
  newpart = GetTypedPart(inpage);
  maindata.push_back(newpart);
  noerr = newpart->IsValid();
  return noerr;
}
//-------------------------------------------
jfFicExtract* jfFanfic::DoPageOne(wxString inpage) {
  // variables
  jfFicPart_FFN* xpart;
  jfFicPart_AO3* zpart;
  jfFicPart* newpart;
  jfFicExtract* fic_data;
  bool sfres;
  // we parse the part
  if (fictype==jfft_FFN) {
    xpart = new jfFicPart_FFN();
    fic_data = xpart->FirstFromString(inpage);
    newpart = xpart;
  }
  else if (fictype==jfft_AO3) {
    zpart = new jfFicPart_AO3();
    fic_data = zpart->FirstFromString(inpage);
    newpart = zpart;
  }
  else assert(false);
  // success ?
  if (fic_data!=NULL) {
    sfres = SetFic(fic_data, false);
    assert(sfres);
    maindata.push_back(newpart);
    fic_data->fname_core = filebase;
    noerr = true;
  }
  // faliure
  else {
    noerr = false;
    delete newpart;
  }
  return fic_data;
}
//---------------------------------------------
jfFicExtract* jfFanfic::DoIndexPage(wxString inpage) {
  // constants
  const wxString fname = wxT("jfFanfic::DoIndexPage");
  // variables
  jfFicExtract* fic_data;
  bool sfres;
  // starting
  /**/jfXLogMessage(fname,1);
  if (fictype==jfft_FIM) {
    fic_data = ExtractFromIndex_FIM(inpage);
    /**/jfXLogMessageB(fname,2,fic_data==NULL);
  }
  else if (fictype==jfft_HPF) {
    fic_data = ExtractFromIndex_HPF(inpage);
    /**/jfXLogMessageB(fname,3,fic_data==NULL);
  }
  else if (fictype==jfft_MMO) {
    fic_data = ExtractFromIndex_MMO(inpage);
    /**/jfXLogMessageB(fname,4,fic_data==NULL);
  }
  else assert(false);
  // finishing
  if (fic_data!=NULL) {
    /**/jfXLogMessage(fname,5);
    sfres = SetFic(fic_data, false);
    assert(sfres);
    fic_data->fname_core = filebase;
    /**/jfXLogMessage(fname,6);
    noerr = true;
  }
  else noerr = false;
  /**/jfXLogMessage(fname,7);
  // done
  return fic_data;
}
//-------------------------------------------
bool jfFanfic::SetFileBase(wxString inval, wxString indir, size_t in_sguide) {
  if (inval.IsEmpty()) return false;
  filebase = inval;
  dirbase = indir;
  sguide = in_sguide;
  if (sdata!=NULL) sdata->fname_core = filebase;
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// info methods
//------------------
size_t jfFanfic::PartCount() const {
  if ((sdata==NULL) || (!dataset)) return 0;
  if (maindata.size()==0) return sdata->pcount;
  else return maindata[maindata.size()-1]->GetPartcount();
}
//------------------
bool jfFanfic::MultiParts() const {
  return (PartCount()>1);
}
//------------------
bool jfFanfic::IsDataset() const {
  return dataset;
}
//--------------------------------
bool jfFanfic::ValidData() const {
  return noerr;
}
//------------------
wxString jfFanfic::GetLastError() const {
  if (!dataset) return wxEmptyString;
  if (maindata.size()==0) return wxEmptyString;
  return maindata[maindata.size()-1]->GetErr();
}
//-------------------------------------------
bool jfFanfic::LoadCoreValues(jfSkeletonParser* inparser) const {
  // we do not have the data included
  if (inparser==NULL) return false;
  if (!dataset) return false;
  // calling the FicExtract
  sdata->LoadCoreValues(inparser);
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// produces the the html output
bool jfFanfic::BuildResult() {
  // variables
  size_t lindex;
  wxString work_text, bundle_text;
  // going ahead
  if (!StartResult()) return false;
  // the next thing to do it to make the files, we loop thru the files
  for (lindex=1;lindex<=(split_manager->GetFilecount());lindex++) {
    WriteFile(lindex);
  }
  // done
  return true;
}
//------------------------------------------------------------------------
bool jfFanfic::StartResult() {
  // constants
  const wxString fname = wxT("jfFanfic::StartResult");
  // first, we check to see if things are ok to go ahead
  /**/jfXLogMessage(fname,1);
  if (!dataset) return false;
  /**/jfXLogMessage(fname,2);
  if ((sdata->pcount)!=maindata.size()) return false;
  /**/jfXLogMessage(fname,3);
  if (theskel==NULL) return false;
  /**/jfXLogMessage(fname,4);
  // variables
  size_t lindex, gfcount;
  wxString work_text, bundle_text;
  bool lcvres;
  // initializing
  /**/jfXLogMessage(fname,5);
  split_manager = new jfFanficSplitter(sguide);
  output_manager = new jfFD_OutStuff();
  output_manager->parse = new jfSkeletonParser();
  output_manager->base = theskel;
  /**/jfXLogMessage(fname,6);
  // we now load some initial values
  lcvres = LoadCoreValues(output_manager->parse);
  assert(lcvres);
  /**/jfXLogMessage(fname,7);
  // next, we loop to create the formatted output for the parts
  for (lindex=0;lindex<(sdata->pcount);lindex++) {
    maindata[lindex]->LoadCoreValues(output_manager->parse);
    work_text = output_manager->MakeResult(output_manager->base->item_skel);
    split_manager->AddParts(work_text);
  }
  /**/jfXLogMessage(fname,8);
  // the parts are done, we now get and set the number of filesw
  split_manager->CalculateSplit();
  gfcount = split_manager->GetFilecount();
  output_manager->SetNameInfo(filebase,dirbase,gfcount);
  /**/jfXLogMessage(fname,9);
  // next, the navbar for the files (if necessary)
  bundle_text = wxEmptyString;
  if (gfcount>1) {
    for (lindex=1;lindex<=gfcount;lindex++) {
      // we need to initialize 2 values
      lcvres = output_manager->GetFilename(work_text,lindex);
      assert(lcvres);
      output_manager->parse->AddText(wxT("OFILE_FILENAME"),work_text);
      output_manager->parse->AddText(wxT("OFILE_LABEL"),PLabelMaker(lindex));
      // next, we needs to make the item and separator
      work_text = output_manager->MakeResult(output_manager->base->grouplink);
      bundle_text += work_text;
      if (lindex!=gfcount) {
        work_text = output_manager->MakeResult(output_manager->base->grouplink_separator);
        bundle_text += work_text;
      }
    }
  }
  /**/jfXLogMessage(fname,10);
  // adding the navbar to the parser
  output_manager->parse->AddText(wxT("OFILE_NAVBAR"),bundle_text);
  // done
  output_started = true;
  return true;
}
//------------------------------------------------------------------------
// wriring an output file... StartResult() mult be run first
bool jfFanfic::WriteFile(size_t findex_in) {
  // constants
  const wxString fname = wxT("jfFanfic::WriteFile");
  // asserts
  assert(output_started);
  assert(findex_in!=0);
  assert(findex_in<=(split_manager->GetFilecount()));
  // variables
  wxString work_text, bundle_text;
  size_t pindex;
  size_t part_start, part_end, part_count;
  bool sfres;
  // we need to initialize 2 values
  assert(output_manager->GetFilename(work_text,findex_in));
  output_manager->parse->AddText(wxT("OFILE_FILENAME"),work_text);
  output_manager->parse->AddText(wxT("OFILE_LABEL"),PLabelMaker(findex_in));
  // next up, we need to build the navbar for the parts...
  part_start = split_manager->GetFileIndex(findex_in,true);
  part_end = split_manager->GetFileIndex(findex_in,false);
  part_count = part_end - part_start + 1;
  // if the part_count is 1, we skip it...
  bundle_text = wxEmptyString;
  if (part_count!=1) {
    for (pindex=part_start;pindex<=part_end;pindex++) {
      maindata[pindex-1]->LoadCoreValues(output_manager->parse);
      work_text = output_manager->MakeResult(output_manager->base->partlink);
      bundle_text += work_text;
      if (pindex!=part_end) {
        work_text = output_manager->MakeResult(output_manager->base->partlink_separator);
        bundle_text += work_text;
      }
    }
  }
  // here, we add the result to the parser
  output_manager->parse->AddText(wxT("PARTS_NAVBAR"),bundle_text);
  // we also start building things
  // the header text
  work_text = output_manager->MakeResult(output_manager->base->header_skel);
  sfres = output_manager->StartNewFile();
  assert(sfres);
  output_manager->AddToFile(work_text);
  // we now loop thru the parts...
  for (pindex=part_start;pindex<=part_end;pindex++) {
    // this is probably not really necessary
    maindata[pindex-1]->LoadCoreValues(output_manager->parse);
    // adding the item text to the results
    output_manager->AddToFile(split_manager->GetPart(pindex));
    // the item separattor
    if (pindex!=part_end) {
      work_text = output_manager->MakeResult(output_manager->base->item_separator);
      output_manager->AddToFile(work_text);
    }
  }
  // done with the loop
  work_text = output_manager->MakeResult(output_manager->base->footer_skel);
  output_manager->AddToFile(work_text);
  // finishing
  output_manager->CloseFile();
  return true;
}
//--------------------------------------------
// clearing up some stuff after writing the files
void jfFanfic::FinishAfter() {
  assert(output_started);
  // clearing
  delete split_manager;
  split_manager = NULL;
  delete output_manager->parse;
  delete output_manager;
  output_manager = NULL;
  output_started = false;
}
//----------------------------------------------
size_t jfFanfic::GetFilecount() const {
  if (!output_started) return 0;
  else return split_manager->GetFilecount();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructors
jfFanfic::~jfFanfic() {
  ClearStuff();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal methods
void jfFanfic::ClearStuff() {
  for (int qx=0;qx<maindata.size();qx++) {
    delete maindata[qx];
    maindata[qx] = NULL;
  }
  maindata.clear();
  if (sdata!=NULL) {
    delete sdata;
    sdata = NULL;
  }
  if (split_manager!=NULL) {
    delete split_manager;
    split_manager = NULL;
  }

  dataset = false;
}
//-----------------------------------------------------------------------
wxString jfFanfic::PLabelMaker(size_t indexv) const {
  // checks and asserts
  assert(indexv>=0);
  assert(split_manager!=NULL);
  // variables
  wxString resval = wxT("Part");
  bool single;
  size_t start,end;
  // we get the data values
  start = split_manager->GetFileIndex(indexv,true);
  end = split_manager->GetFileIndex(indexv,false);
  // checking them
  single = ((end-start)==0);
  // making the result
  if (single) resval+= wxT(" ") + SInt2Str(start);
  else {
    resval += wxT("s ") + SInt2Str(start);
    resval += wxT(" to ") + SInt2Str(end);
  }
  // sone
  return resval;
}
//-----------------------------------------------------------------------
jfFicPart* jfFanfic::GetTypedPart(wxString inpage) {
  if (fictype==jfft_FFN) {
    if (inpage==wxEmptyString) return new jfFicPart_FFN();
    else return new jfFicPart_FFN(inpage);
  }
  else if (fictype==jfft_FIM) {
    if (inpage==wxEmptyString) return new jfFicPart_FIM();
    else return new jfFicPart_FIM(inpage);
  }
  else if (fictype==jfft_HPF) {
    if (inpage==wxEmptyString) return new jfFicPart_HPF();
    else return new jfFicPart_HPF(inpage);
  }
  else if (fictype==jfft_MMO) {
    if (inpage==wxEmptyString) return new jfFicPart_MMO();
    else return new jfFicPart_MMO(inpage);
  }
  else if (fictype==jfft_AO3) {
    if (inpage==wxEmptyString) return new jfFicPart_AO3();
    else return new jfFicPart_AO3(inpage);
  }
  else assert(false);
}

//****************************************************************************
