//****************************************************************************
// Name:        ficdown_parts2.cpp
// Author :     John Q Metro
// Purpose :    Downloading fics, we need structures to hold the data
// Created:     March 23, 2012
// Updated:     June 15, 2017 (Fimfiction.Net 4)
//****************************************************************************
#ifndef FICDOWN_PARTS2_H_INCLUDED
  #include "ficdown_parts2.h"
#endif // FICDOWN_PARTS2_H_INCLUDED
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED

#include <wx/datetime.h>

#ifndef FIM_FICOBJ_H_INCLUDED
  #include "../../fanfic/data/fim/fim_ficobj.h"
#endif // FIM_FICOBJ_H_INCLUDED

//****************************************************************************
/* for fimfiction.net, we cannot get the necessary info from the chapter
pages, we need to use the index page. */
jfFicExtract_FIM* ExtractFromIndex_FIM(const wxString& inval) {
  // constants
  const wxString fname = wxT("ExtractFromIndex_FIM");
  // variables
  jfFicExtract_FIM* result;
  jfFIM_Fanfic* fres;
  // starting
  fres = new jfFIM_Fanfic();
  result = fres->ExtractFromString(inval);
  delete fres;
  return result;
}
//=============================================================================
jfFicExtract_HPF* ExtractFromIndex_HPF(const wxString& inval) {
  // constants
  const wxString fname = wxT("ExtractFromIndex_HPF");
  const wxString hpciex = wxT("location = \'http://www.harrypotterfanfiction.com/viewstory2.php?chapterid=");
  // variables
  jfFicExtract_HPF* result;
  jfStringParser* sparser;
  wxString buffer,errval;
  bool success;
  unsigned long qval;
  size_t ccount;
  // starting
  result = new jfFicExtract_HPF();
  sparser = new jfStringParser(inval);
  success = true;
  /**/jfXLogMessage(fname,1);
  // skippting past the header
  if (!sparser->MovePastAlt(wxT("&gt;&gt;</a> </form></span><br clear=all>"),
                            wxT(">Report Abuse</a>&nbsp;]"))) success = false;
  else if (!sparser->GetDelimitedULong(wxT("<a href=\"?psid="),wxT("\">"),qval,errval)) {
    if (!sparser->GetDelimitedULong(wxT(" location = '?psid="),wxT("'\">"),qval,errval)) success = false;
  }
  // skipping faliue
  if (!success) {
    delete sparser;
    delete result;
    return NULL;
  }
  /**/jfXLogMessageS(fname,2,qval);
  // title
  if (!sparser->GetMovePast(wxT("</a>"),buffer)) success = false;
  else {
    result->fic_id = qval;
    result->fic_title = buffer;
    /**/jfXLogMessage(fname,3,buffer);
    // author id
    if (!sparser->GetDelimitedULong(wxT("<a href=\"viewuser.php?showuid="),wxT("\">"),qval,errval)) success = false;
    // author name
    else if (!sparser->GetMovePast(wxT("</a>"),buffer)) success = false;
    else {
      result->auth_id = qval;
      result->author_name = buffer;
      /**/jfXLogMessage(fname,4,buffer);
      // part count
      if (!sparser->GetDelimitedULong(wxT("<b>Chapters:</b>"),wxT("<br>"),qval,errval)) success = false;
      // status
      else if (!sparser->GetDelimited(wxT("<b>Status:</b>"),wxT("<br>"),buffer)) success = false;
      else {
        result->pcount = qval;
        Dtrim(buffer);
        /**/jfXLogMessage(fname,5,buffer);
        result->complete = (buffer==wxT("Completed"));
      }
    }
  }
  /**/jfXLogMessageB(fname,6,success);

  // skipping faliue
  if (!success) {
    delete sparser;
    delete result;
    return NULL;
  }
  // next, all we need is the updated date, much easier than fimfiction
  if (!sparser->GetDelimited(wxT("<b>Last Updated:</b> "),wxT("<br />"),buffer)) success = false;
  else if (NULL==result->updated_date.ParseFormat(buffer,wxT("%Y.%m.%d"))) success = false;
  else {
    // chapter ids
    ccount = 0;
    while (sparser->GetDelimitedULong(wxT("<a href=\"?chapterid="),wxT("\">"),qval,errval)) {
      result->chapterids.push_back(qval);
      ccount++;
    }
    // alternate for mature "adult themes" stories
    if (ccount==0) {
      while (sparser->GetDelimitedULong(hpciex,wxT("&i="),qval,errval)) {
        result->chapterids.push_back(qval);
        ccount++;
      }
    }
    /**/jfXLogMessageS(fname,7,ccount);
  }
  if ((result->pcount)!=ccount) success = false;
  // done
  delete sparser;
  /**/jfXLogMessage(fname,8);
  if (!success) {
    delete result;
    return NULL;
  }
  else return result;
}
//===========================================================================
jfFicExtract_MMO* ExtractFromIndex_MMO(const wxString& inval) {
  // constants
  const wxString fname = wxT("ExtractFromIndex_MMO");
  const wxString startlink = wxT("<link rel=\"canonical\" href=\"http://www.mediaminer.org/fanfic/s/");
  // variables
  jfFicExtract_MMO* result;
  jfStringParser* sparser;
  wxString buffer,errval;
  bool success;
  unsigned long qval;
  size_t ccount;

  // starting
  /**/jfXLogMessage(fname,1);
  sparser = new jfStringParser(inval);
  success = true;
  // first action, trying to get the url fragment and fic id
  if (!sparser->MovePast(startlink)) success = false;
  else if (!sparser->MovePastTwice(wxT("/"))) success = false;
  else {
    buffer = sparser->GetSkipped();
    if (!sparser->GetMovePastULong(wxT("\" />"),qval,errval)) success = false;
  }
  if (!success) {
    delete sparser;
    return NULL;
  }
  /**/jfXLogMessage(fname,2);
  result = new jfFicExtract_MMO();
  result->fic_id = qval;
  result->url_cname_part = buffer;

  // moving past the header
  if (!sparser->MovePastTwo(wxT("</header>"),wxT("<h1 id=\"post-title\">"))) {
    delete sparser;    delete result;
    /**/jfXLogMessage(fname,3);
    return NULL;
  }

  // getting the title and author id
  const wxString auidi = wxT("<b>By: </b><a href=\"/user_info.php/");
  if (!sparser->GetDelimited(wxT("&#10095;"),wxT("</h1>"),buffer)) success = false;
  else if (!sparser->GetDelimitedULong(auidi,wxT("\">"),qval,errval)) success = false;
  else {
    Dtrim(buffer);
    result->fic_title = buffer;
    result->auth_id = qval;
  }
  if (!success) {
    delete sparser;    delete result;
    /**/jfXLogMessage(fname,4);
    return NULL;
  }
  /**/jfXLogMessage(fname,5);
  // author name
  if (!sparser->GetMovePast(wxT("</a>"),buffer)) {
    delete sparser;   delete result;
    return NULL;
  }
  else {
    Dtrim(buffer);  result->author_name = buffer;
  }

  // next up, the uploaded date!
  if (!sparser->GetDelimited(wxT("<b>Uploaded On: </b>"),wxT("|"),buffer)) success = false;
  else {
    Dtrim(buffer);
    if (NULL == result->updated_date.ParseFormat(buffer,wxT("%m.%d.%Y"))) success = false;
  }
  if (!success) {
    delete sparser;   delete result;
    return NULL;
  }
  /**/jfXLogMessage(fname,6);
  // nothing to indicate if a fic is completed
  result->complete = false;

  // chaper/part processing
  ccount = 0;
  while (sparser->MovePast(wxT("<a href=\"/fanfic/c/"))) {
    ccount++;
    /**/jfXLogMessageS(fname,7,ccount);
    if (!sparser->MovePastTwice(wxT("/"))) {
      success = false;
      break;
    }
    /**/jfXLogMessage(fname,8);
    // getting the chapter id
    if (!sparser->GetDelimitedULong(wxT("/"),wxT("\">"),qval,errval)) {
      success = false;
      /**/jfXLogMessage(fname,9,errval);
      break;
    }
    (result->chapterids).push_back(qval);
    /**/jfXLogMessageS(fname,10,ccount);
  }
  // afterwards...
  delete sparser;
  if (!success) {
    delete result;  return NULL;
  }
  else {
    /**/jfXLogMessage(fname,11);
    result->pcount = ccount;
    return result;
  }
}
//=============================================================================
// constructors
//-----------------------------------------------
jfFicPart_FIM::jfFicPart_FIM():jfFicPart() {}
//-----------------------------------------------
jfFicPart_FIM::jfFicPart_FIM(wxString inpage):jfFicPart() {
  const  wxString fname = wxT("jfFicPart_FIM::jfFicPart_FIM");
  errmsg = wxT("Not started yet");
  SetFromString(inpage);
  if (!validdata) jfXLogMessage(fname,1,GetErr());
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfFicPart_FIM::SetFromStringCore() {
  // consstants
  const wxString fname = wxT("jfFicPart_FIM::SetFromStringCore");
  // checks
  assert(sparser!=NULL);
  // varibles
  wxString buffer;
  // chapter title
  if (!sparser->MovePast(wxT("<a id=\"chapter_title\""))) return false;
  if (!sparser->GetDelimited(wxT(">"),wxT("</a>"),buffer)) return false;
  Dtrim(buffer);
  partname = buffer;
  /**/jfXLogMessage(fname,2,partname);
  if (sparser->MovePast(wxT("<div class=\"button-group "))) {
    // we get chapter info (used to determine partcount and partindex)
    if (!sparser->GetDelimited(wxT("<ul"),wxT("</ul>"),buffer)) return false;
    if (!HandleChapters(buffer)) {
      errmsg = wxT("Chapter Processing failed");
      return false;
    }
  }
  else {
    partcount = 1;
    partindex = 1;
  }
  /**/jfXLogMessageS(fname,3,partcount);
  // next, the contents
  if (!sparser->MovePast(wxT("<div id=\"chapter-body\""))) return false;
  /**/jfXLogMessage(fname,4);
  const wxString startm = wxT("<div class=\"bbcode\">");
  const wxString endm1 = wxT("<div class=\"chapter_footer\">");
  const wxString endm2 = wxT("<ins ");
  if (!sparser->GetDelimitedEndPair(startm,endm1,endm2,buffer)) return false;
  /**/jfXLogMessage(fname,5);
  // processing after to set the part contents
  PartProcessing(buffer);
  /**/jfXLogMessage(fname,6);
  // sometimes there is an author's note afterwards
  if (sparser->MovePast(wxT("<div class=\"authors-note"))) {
    if (sparser->GetDelimited(wxT(">"),wxT("</div>"),buffer)) {
      Dtrim(buffer);
      partcontents += wxT("\n<hr>") + buffer;
    }
  }
  // done
  return true;
}
//---------------------------------------------------------------------------
bool jfFicPart_FIM::HandleChapters(wxString inbuffer) {
  // constants
  const wxString fname = wxT("jfFicPart_FIM::HandleChapters");
  // variables
  jfStringParser* qparser;
  unsigned long qval;
  wxString buffer, oxerr;
  bool fail = false;
  bool cselected;
  // we start
  qparser = new jfStringParser(inbuffer);
  partindex = 0;
  // the main parsing loop
  while (qparser->MovePast(wxT("<li"))) {
    // getting everything up to the >
    if (!qparser->GetMovePast(wxT(">"),buffer)) fail = true;
    else {
      Dtrim(buffer);
      cselected = buffer == wxT("class=\"selected\"");
      // extracting the chapter index
      if (!qparser->MovePast(wxT("<a href=\"/story/"))) fail = true;
      else if (!qparser->GetDelimitedULong(wxT("/"),wxT("/"),qval,oxerr)) fail = true;
      else if (cselected) partindex = qval;
    }
  }
  // done with the loop
  delete qparser;
  if (fail) return false;
  if (qval==0) return false;
  partcount = qval;
  return true;
}
//-----------------------------------------------------------------------
void jfFicPart_FIM::PartProcessing(wxString inbuffer) {
  // constants
  const wxString wdiv1 = wxT("<br /><br /><div style='float:left; height:1.0em; width:3.0em;'></div>");
  const wxString wdiv2 = wxT("<br /><div style='float:left; height:1.0em; width:3.0em;'></div>");
  const wxString wdiv3 = wxT("<div style='float:left; height:1.0em; width:3.0em;'>");
  // const wxString ctag = wxT("<br /><center>o.0.o</center></p>");
  // const wxString ctag2 = wxT("<p><center>o.0.o</center></p>");
  const wxString bbcent = wxT("<div class=\"bbcode-center\" style=\"text-align:center\">");
  const wxString newcent = wxT("<div style=\"text-align:center\">");
  // variables
  size_t lpos;
  // we backtrack to just after the last </div>
  lpos = inbuffer.rfind(wxT("</div>"));
  if (lpos != wxString::npos) inbuffer.Truncate(lpos+6);
  // trimming and terminating
  Dtrim(inbuffer);
  // bbcode class names are not wanted
  inbuffer.Replace(bbcent,newcent);
  // we have to remove floating divs (they break certain browsers)
  /*
  inbuffer.Replace(wdiv1,wxT("</p>\n<p>"));
  inbuffer.Replace(wdiv2,wxT("</p>\n<p>"));
  inbuffer.Replace(wdiv3,wxT("</p>\n<p>"));
  */
  inbuffer.Replace(wxT("</p><p class=\"double\">"),wxT("</p>\n<p>"));
  inbuffer.Replace(wxT("</p><p class=\"indented double\">"),wxT("</p>\n<p class=\"indented double\">"));
  // also, why use breaks when we can use paragraphs
  inbuffer.Replace(wxT("<br /><br />"),wxT("</p>\n<p>"));
  // removing more empty things
  inbuffer.Replace(wxT("<p></div></p>\n"),wxT("\n"));
  inbuffer.Replace(wxT("<p></p>\n"),wxT("\n"));
  inbuffer.Replace(wxT("<p> </p>"),wxT("\n"));
  inbuffer.Replace(wxT("<p></div>"),wxT("<p>"));
  // done
  if (inbuffer.Find(wxT("<p"))!=0) inbuffer = wxT("<p>") + inbuffer;
  partcontents = inbuffer + wxT("</p>");
}
//=============================================================================
// constructors
//-----------------------------------------------
jfFicPart_HPF::jfFicPart_HPF():jfFicPart() {}
//-----------------------------------------------
jfFicPart_HPF::jfFicPart_HPF(wxString inpage):jfFicPart() {
  const  wxString fname = wxT("jfFicPart_HPF::jfFicPart_HPF");
  errmsg = wxT("Not started yet");
  SetFromString(inpage);
  if (!validdata) jfXLogMessage(fname,1,GetErr());
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfFicPart_HPF::SetFromStringCore() {
  // consstants
  const wxString fname = wxT("jfFicPart_HPF::SetFromStringCore");
  const wxString hval = wxT("Report Abuse</a>&nbsp;]");
  const wxString cboxs = wxT("<form style=\"margin:0\" enctype=\"multipart/form-data\" method=\"post\" action=\"\">");

  // checks
  assert(sparser!=NULL);
  // varibales
  wxString buffer;
  // starting...
  if (!sparser->MovePast(hval)) return false;
  // we get chapter info (used to determine partcount and partindex)
  if (sparser->GetDelimited(cboxs,wxT("</form>"),buffer)) {
    if (!HandleChapters(buffer)) {
      errmsg = wxT("Chapter Processing failed");
      return false;
    }
  }
  else {
    // we assume 1 part
    partindex = 1;
    partcount = 1;
  }
  // moving to the part name
  if (!sparser->MovePast(wxT("<center>"))) return false;
  if (!sparser->MovePast(wxT("<br>"))) return false;
  if (!sparser->MovePast(wxT(" : "))) return false;
  // getting the part name
  if (!sparser->GetMovePast(wxT("</center>"),buffer)) return false;
  Dtrim(buffer);
  partname = buffer;
  // next, the contents
  if (!sparser->GetDelimited(wxT("<div id='fluidtext'>"),wxT("</div><br><hr>"),buffer)) return false;
  // processing contents
  PartProcessing(buffer);
  // done
  return true;
}
//---------------------------------------------------------------------------
bool jfFicPart_HPF::HandleChapters(wxString inbuffer) {
  // variables
  jfStringParser* qparser;
  wxString buffer;
  size_t ccount;
  bool fail = false;
  // we start
  qparser = new jfStringParser(inbuffer);
  ccount = 0;
  // the main parsing loop
  while (qparser->MovePast(wxT("<option value=\"?chapterid="))) {
    ccount++;
    // looking for the part index
    if (!qparser->MovePast(wxT("\""))) fail = true;
    else if (!qparser->GetMovePast(wxT(">"),buffer)) fail = true;
    else {
      Dtrim(buffer);
      if (buffer==wxT("selected")) partindex = ccount;
      if (!qparser->MovePast(wxT("."))) fail = true;
    }
    if (fail) break;
  }
  // done with the loop
  delete qparser;
  if (fail) return false;
  if (ccount==0) return false;
  partcount = ccount;
  return true;
}
//--------------------------------------------------------------------------
void jfFicPart_HPF::PartProcessing(wxString inbuffer) {
  Dtrim(inbuffer);
  // misc non-breaking spaces
  inbuffer.Replace(wxT("&nbsp;<br />"),wxT("<br />"));
  inbuffer.Replace(wxT("<div>&nbsp;</div><br />"),wxT("<br />"));
  inbuffer.Replace(wxT("<div><br />\n&nbsp;</div><br />"),wxT("<br />\n<br />"));
  // getting rid of too may linebreaks... we do need the loop
  while (0!=inbuffer.Replace(wxT("<br />\n<br />\n<br />"),wxT("<br />\n<br />")));
  // windows 1252 within unicode....
  inbuffer.Replace(wxT("\x93"),wxT("&ldquo;"));
  inbuffer.Replace(wxT("\x94"),wxT("&rdquo;"));
  inbuffer.Replace(wxT("\x92"),wxT("\'"));
  inbuffer.Replace(wxT("\x85"),wxT("..."));
  // done
  partcontents = inbuffer;


}

//****************************************************************************
