//****************************************************************************
// Name:        ficdown_parts3.cpp
// Author :     John Q Metro
// Purpose :    Holders for fanfic parts
// Created:     April 6, 2012
// Updated:     July 18, 2017 (AO3)
//****************************************************************************
#ifndef FICDOWN_PARTS3_H_INCLUDED
  #include "ficdown_parts3.h"
#endif // FICDOWN_PARTS3_H_INCLUDED
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef UTILS3_H_INCLUDED
  #include "../../core/utils/utils3.h"
#endif // UTILS3_H_INCLUDED
//****************************************************************************
// constructors
//---------------------------------------------
jfFicPart_MMO::jfFicPart_MMO():jfFicPart() {
  chapter_id = 0;
}
//---------------------------------------------
jfFicPart_MMO::jfFicPart_MMO(wxString inpage):jfFicPart() {
  const  wxString fname = wxT("jfFicPart_MMO::jfFicPart_MMO");
  errmsg = wxT("Not started yet");
  chapter_id = 0;
  SetFromString(inpage);
  if (!validdata) jfXLogMessage(fname,1,GetErr());
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++
// MediaMiner parsing
bool jfFicPart_MMO::SetFromStringCore() {
  assert(sparser!=NULL);
  // constants
  const wxString fname = wxT("jfFicPart_MMO::SetFromStringCore");
  const wxString headlink = wxT("<link rel=\"canonical\" href=\"http://www.mediaminer.org/fanfic/c/");

  // variables
  wxString buffer,outerr;
  unsigned long oval;

  // starting, and getting the chapter id
  /**/jfXLogMessage(fname,1);
  validdata = false;
  if (!sparser->MovePast(headlink)) {
    return SetParseError(wxT("Missing LINK in head!"));
  }
  if (!sparser->MovePastTwice(wxT("/"))) {
    return SetParseError(wxT("Problems in parsing LINK URL!"));
  }
  if (!sparser->GetDelimitedULong(wxT("/"),wxT("\" />"),oval,outerr)) {
    return SetParseError(wxT("Cannot extract chapter id! : ") + outerr);
  }
  chapter_id = oval;
  /**/jfXLogMessageS(fname,2,chapter_id);

  // next up, moving past the header
  if (!sparser->MovePast(wxT("</header>"))) {
    return SetParseError(wxT("Missing end of header!"));
  }
  if (!sparser->MovePastTwo(wxT("<h1 id=\"post-title\">"),wxT("&#10095;"))) {
    return SetParseError(wxT("Cannot find start of complete Title!"));
  }

  // extracting the part name
  if (!sparser->GetDelimited(wxT("&#10095;"),wxT("</b> <font size=\"-1\">"),buffer)) {
    return SetParseError(wxT("Cannot extract chapter name!"));
  }
  Dtrim(buffer);
  partname = buffer;
  /**/jfXLogMessage(fname,3,partname);

  // extracting the parts contents
  if (!sparser->GetDelimited(wxT("<div id=\"fanfic-text\">"),wxT("<div class=\"adWrap\">"),buffer)) {
    return SetParseError(wxT("Could not extract chapter contents!"));
  }
  // chaper count and index
  if (!HandleChapters()) return false;
  // done
  partcontents = PartProcessing(buffer);
  return true;
}
//-------------------------------------------------------
bool jfFicPart_MMO::HandleChapters() {
  // variables
  size_t pxcount = 0;
  unsigned long oval;
  wxString errout;

  // if there is no chapter form, there is only one part
  if (!sparser->MovePast(wxT("<form method=\"get\" action=\"/fanfic/view_ch.php\""))) {
    partcount = partindex = 1;
  }
  else {
    while (sparser->MovePast(wxT("<option value=\""))) {
      pxcount++;
      if (!sparser->GetMovePastULong(wxT("\""),oval,errout)) {
        return SetParseError(wxT("Could not get chapter id in chapter list! : ") + errout);
      }
      if (oval == chapter_id) partindex = pxcount;

    }
    if (pxcount==0) return SetParseError(wxT("No chapters found while parsing chapter?!"));
    if (partindex == 0) return SetParseError(wxT("Current chapter id is not in the list!"));
    partcount = pxcount;
  }
  // done
  return true;
}
//--------------------------------------------------------------------------
wxString jfFicPart_MMO::PartProcessing(wxString inbuffer) {
  Dtrim(inbuffer);
  inbuffer.RemoveLast(6);
  Dtrim(inbuffer);
  return inbuffer;
}
//****************************************************************************
// constructors
//---------------------------------------
jfFicPart_AO3::jfFicPart_AO3():jfFicPart() {
  ffresult = NULL;
}
//---------------------------------------
jfFicPart_AO3::jfFicPart_AO3(wxString inpage) {
  const  wxString fname = wxT("jfFicPart_AO3::jfFicPart_AO3");
  errmsg = wxT("Not started yet");
  SetFromString(inpage);
  if (!validdata) {
    jfXLogMessage(fname,1,GetErr());
    ToFile(inpage,wxT("errstring.html"));
  }
  ffresult = NULL;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// another way
jfFicExtract_AO3* jfFicPart_AO3::FirstFromString(wxString inpage) {
  // constants
  const wxString fname = wxT("jfFicPart_AO3::FirstFromString");
  const wxString toomuch = wxT("<textarea cols=\"50\" rows=\"4\" id=\"embed_code\">&lt;a href=&quot;http://archiveofourown.org/works/");
  const wxString cstart1 = wxT("<h3 class=\"landmark heading\" id=\"work\">Chapter Text</h3>");
  const wxString cstart2 = wxT("<h3 class=\"landmark heading\" id=\"work\">Work Text:</h3>");
  const wxString select1 = wxT("<select id=\"selected_id\" name=\"selected_id\">");
  const wxString select2 = wxT("<select name=\"selected_id\" id=\"selected_id\">");
  // variables
  unsigned long qval,zval;
  bool firstid;
  wxString buffer,errres,notes;
  size_t ccount;
  bool ficidf = false;
  // initializing
  sparser = new jfStringParser(inpage);
  ffresult = new jfFicExtract_AO3();
  /**/jfXLogMessage(fname,1);
  // skipping past the header gunk
  if (!sparser->MovePast(wxT("<ul class=\"work navigation actions\" role=\"menu\">"))) {
    return FParseError(wxT("Header info error!"));
  }
  /**/jfXLogMessage(fname,2);
  /* chapter navigation is next, we use it to get partcount, chapter ids, and the partname */
  if (!sparser->MovePastAlt(select1,select2)) {
    // single part stories
    ffresult->pcount = 1;
    partcount = 1;
    /**/jfXLogMessage(fname,3);
  }
  else {
    // multipart stories
    ccount = 0;
    /**/jfXLogMessage(fname,6);
    while (sparser->MovePast(wxT("<option"))) {
      if (!sparser->GetDelimitedULong(wxT("value=\""),wxT("\""),qval,errres)) {
        sparser->GoBack();
        break;
      }
      ccount++;
      ffresult->chapterids.push_back(qval);
      if (ccount==1) {
        if (!sparser->MovePast(wxT("."))) return FParseError(wxT("Problems with start of part name!"));
        if (!sparser->GetMovePast(wxT("</option>"),buffer)) {
          /**/jfXLogMessage(fname,7);
          return FParseError(wxT("Problems with end of part name!"));
        }
        Dtrim(buffer);
        partname = buffer;
      }
    }
    /**/jfXLogMessageS(fname,8,ccount);
    if (ccount==0) return FParseError(wxT("Unable to find any parts!"));
    partcount = ccount;
    ffresult->pcount = ccount;
    /**/jfXLogMessage(fname,9);
    // looking for the fic id
    if (!sparser->GetDelimitedULong(wxT("<li><a href=\"/works/"),wxT("/"),qval,errres)) {
      return FParseError(wxT("Problems with getting fic id!"),errres);
    }
    ffresult->fic_id = qval;
    ficidf = true;
    /**/jfXLogMessage(fname,10);
  }
  /**/jfXLogMessage(fname,11);
  // part index
  partindex = 1;
  // trying for the fic id again
  if (!ficidf) {
    /**/jfXLogMessage(fname,12);
    const wxString twit1 = wxT("<a href=\"http://twitter.com/share\"");
    const wxString twit2 = wxT("<a href=\"http://www.twitter.com/share\"");
    if (!sparser->MovePastAlt(twit1,twit2)) {
      return FParseError(wxT("Problems with getting twitter fic id!"));
    }
    /**/jfXLogMessage(fname,13);
    if (!sparser->GetDelimitedULong(wxT("data-url=\"http://archiveofourown.org/works/"),wxT("\""),qval,errres)) {
      return FParseError(wxT("Problems with extrcating fic id from Twitter link!"),errres);
    }
    ffresult->fic_id = qval;
  }
  /**/jfXLogMessage(fname,14);
  // skiiping past more stuff to skip
  if (!sparser->MovePast(wxT("<dt class=\"stats\">Stats:</dt>"))) return FParseError(wxT("Cannot find stats line!"));
  // completion
  ffresult->complete = true;
  /**/jfXLogMessage(fname,15);
  // the date is somewhat complicated, as we have 3 successive options
  if (sparser->GetDelimited(wxT("Completed:</dt><dd class=\"status\">"),wxT("</dd>"),buffer)) Dtrim(buffer);
  else if (sparser->GetDelimited(wxT("Updated:</dt><dd class=\"status\">"),wxT("</dd>"),buffer)) {
    Dtrim(buffer);
    ffresult->complete = false;
  }
  else if (sparser->GetDelimited(wxT("Published:</dt><dd class=\"published\">"),wxT("</dd>"),buffer)) Dtrim(buffer);
  else return FParseError(wxT("Cannot find date!"));
  /**/jfXLogMessage(fname,16);
  // parsing the date
  if (NULL==ffresult->updated_date.ParseFormat(buffer,wxT("%Y-%m-%d"))) {
    return FParseError(wxT("Parsing date failed!"));
  }
  /**/jfXLogMessage(fname,17);
  // title
  if (!sparser->GetDelimited(wxT("<h2 class=\"title heading\">"),wxT("</h2>"),buffer)) {
    return FParseError(wxT("Cannot find title!"));
  }
  Dtrim(buffer);
  ffresult->fic_title = buffer;
  /**/jfXLogMessage(fname,18);
  // author name
  if (!sparser->MovePast(wxT("<a rel=\"author\""))) {
    return FParseError(wxT("Cannot find author name!"));
  }
  if (!sparser->GetDelimited(wxT("\">"),wxT("</a>"),buffer)) {
    return FParseError(wxT("Cannot extract author name!"));
  }
  /**/jfXLogMessage(fname,19);
  Dtrim(buffer);
  ffresult->author_name = buffer;
  // no author id
  ffresult->auth_id = 1;
  /**/jfXLogMessage(fname,20);
  // another attempt to get the fic id
  if (999999999==(ffresult->fic_id)) {
    if (!sparser->GetDelimitedULong(wxT("<a href=\"/works/"),wxT("/"),qval,errres)) {
      return FParseError(wxT("Final Attempt to get fic id failed!"));
    }
    ffresult->fic_id = qval;
  }
  /**/jfXLogMessage(fname,21);
  // getting special notes
  if (sparser->GetDelimited(wxT("<h3 class=\"heading\">Notes:</h3>"),wxT("</div>"),notes)) {
    startnotes = CheckNotes(notes);
  }
  /**/jfXLogMessage(fname,22);
  // getting part contents
  if (!sparser->GetDelimited(cstart1,wxT("<!--/main-->"),buffer)) {
    /**/jfXLogMessage(fname,23);
    if (!sparser->GetDelimited(cstart2,wxT("<!-- end cache -->"),buffer)) {
      return FParseError(wxT("Cannot find contents!"));
    }
  }
  /**/jfXLogMessage(fname,24);
  // getting special notes
  if (sparser->GetDelimited(wxT("<h3 class=\"heading\">Notes:</h3>"),wxT("</div>"),notes)) {
    endnotes = CheckNotes(notes);
  }
  /**/jfXLogMessage(fname,25);
  // after
  PartProcessing(buffer);
  delete sparser;
  sparser = NULL;
  validdata = true;
  /**/jfXLogMessage(fname,26);
  return ffresult;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// private methods
//---------------------------------------
bool jfFicPart_AO3::SetFromStringCore() {
  // constants
  const wxString fname = wxT("jfFicPart_AO3::SetFromStringCore");
  const wxString navstart = wxT("<ul class=\"work navigation actions\" role=\"menu\">");
  const wxString cstart1 = wxT("<h3 class=\"landmark heading\" id=\"work\">Chapter Text</h3>");
  const wxString cstart2 = wxT("<h3 class=\"landmark heading\" id=\"work\">Work Text:</h3>");

  // variables
  wxString buffer,notes;
  // getting the chapter info
  /**/jfXLogMessage(fname,1);
  if (!sparser->GetDelimited(navstart,wxT("</ul>"),buffer)) return SetParseError(wxT("Missing Chapter Data!"));
  // processing chapter info
  Dtrim(buffer);
  /**/jfXLogMessage(fname,2);
  if (!HandleChapters(buffer)) return SetParseError(wxT("Could not process chapter info!"));
  // getting special notes
  if (sparser->MovePast(wxT("<div class=\"notes module\" role=\"complementary\">"))) {
    if (sparser->GetDelimited(wxT("<h3 class=\"heading\">Notes:</h3>"),wxT("</div>"),notes)) {
      startnotes = CheckNotes(notes);
    }
  }
  /**/jfXLogMessage(fname,3);
  // getting part contents
  // single part
  if (partcount==1) {
    if (!sparser->GetDelimited(cstart2,wxT("<!-- end cache -->"),buffer)) {
      return FParseError(wxT("Cannot find contents! AA:") + sparser->GetBlock(500));
    }
  }
  else {
    if (!sparser->GetDelimited(cstart1,wxT("<!--/main-->"),buffer)) {
      return FParseError(wxT("Cannot find contents! BB:") + sparser->GetBlock(500));
    }
  }
  /**/jfXLogMessage(fname,4);
  // getting special end notes
  if (sparser->MovePast(wxT("<div id=\"work_endnotes\" class=\"end notes module\">"))) {
    if (sparser->GetDelimited(wxT("<h3 class=\"heading\">Notes:</h3>"),wxT("</div>"),notes)) {
      endnotes = CheckNotes(notes);
    }
  }
  // after
  PartProcessing(buffer);
  delete sparser;
  sparser = NULL;
  validdata = true;
  /**/jfXLogMessage(fname,5);
  return ffresult;
}
//---------------------------------------
bool jfFicPart_AO3::HandleChapters(wxString inbuffer) {
  // constants
  const wxString fname = wxT("jfFicPart_AO3::HandleChapters");
  const wxString select1 = wxT("<select id=\"selected_id\" name=\"selected_id\">");
  const wxString select2 = wxT("<select name=\"selected_id\" id=\"selected_id\">");
  // variables
  jfStringParser* qparser;
  size_t ccount;
  wxString buffer;
  bool current;
  // starting
  /**/jfXLogMessage(fname,1);
  qparser = new jfStringParser(inbuffer);
  /* chapter navigation is next, we use it to get partcount, chapter ids, and the partname */
  if (!qparser->MovePastAlt(select1,select2)) {
    // single part stories
    partcount = 1;
    partindex = 1;
    /**/jfXLogMessage(fname,2);
  }
  else {
    // multipart stories
    ccount = 0;
    /**/jfXLogMessage(fname,3);
    while (qparser->MovePastLimit(wxT("<option"),wxT("</select>"))) {
      ccount++;
      // checking for selected
      if (!qparser->GetMovePast(wxT("value="),buffer)) {
        delete qparser;
        return false;
      }
      Dtrim(buffer);
      current = (buffer==wxT("selected=\"selected\""));
      // processing the current part to get the name
      if (current) {
        partindex = ccount;
        if (!qparser->GetDelimited(wxT("."),wxT("</option>"),buffer)) {
          delete qparser;
          return false;
        }
        Dtrim(buffer);
        partname = buffer;
      }
      else continue;
    }
    // post loop
    /**/jfXLogMessageS(fname,4,ccount);
    partcount = ccount;
  }
  // finishing touches
  delete qparser;
  return (partcount!=0);
}
//---------------------------------------
void jfFicPart_AO3::PartProcessing(wxString inbuffer) {
  Dtrim(inbuffer);
  // getting rid of trailing div
  inbuffer = inbuffer.substr(0,inbuffer.length()-6);
  Dtrim(inbuffer);
  // start notes
  if (!startnotes.IsEmpty()) {
    partcontents = wxT("<div style=\"font-style:italic\">") + startnotes;
    partcontents += wxT("</div>\n<hr>\n");
  }
  // done
  partcontents += inbuffer;
  // end notes
  if (!endnotes.IsEmpty()) {
    partcontents += wxT("\n<hr>\n");
    partcontents += wxT("<div style=\"font-style:italic\">") + endnotes;
    partcontents += wxT("</div>\n");
  }

}
//---------------------------------------------------------------------
wxString jfFicPart_AO3::CheckNotes(wxString inval) const {
  // variables
  jfStringParser* qparser;
  wxString buffer;
  // starting
  Dtrim(inval);
  qparser = new jfStringParser(inval);
  if (!qparser->GetDelimited(wxT("<blockquote class=\"userstuff\">"),wxT("</blockquote>"),buffer)) {
    delete qparser;
    return wxEmptyString;
  }
  else {
    Dtrim(buffer);
    delete qparser;
    return buffer;
  }
}
//---------------------------------------
jfFicExtract_AO3* jfFicPart_AO3::FParseError(wxString insmg, wxString extra) {
  validdata = false;
  ToFile(sparser->GetData(),wxT("mainfile.html"));
  // deleteing data
  if (sparser!=NULL) delete sparser;
  sparser = NULL;
  if (ffresult!=NULL) delete ffresult;
  ffresult = NULL;
  // building the result message
  errmsg = insmg;
  if (!extra.IsEmpty()) errmsg += wxT(" ") + extra;
  // done
  return NULL;
}
//****************************************************************************
