/****************************************************************************
Name:        ficurl_parsing.cpp
Author  :    John Q Metro
Purpose :    When given a fic url to download, it has to be checked
Created :    July 8, 2016
Updated :    July 8, 2016

****************************************************************************/
#ifndef FICURL_PARSING_H_INCLUDED
  #include "ficurl_parsing.h"
#endif // FICURL_PARSING_H_INCLUDED
//----------------------------------
#ifndef UTILS1_H_INCLUDED
  #include "../../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
/***************************************************************************/
const wxString FFN_TAG = wxT("www.fanfiction.net");
const wxString FIM_TAG = wxT("www.fimfiction.net");
const wxString AO3_TAG = wxT("archiveofourown.org");
const wxString MMO_TAG = wxT("www.mediaminer.org");
const wxString HPF_TAG = wxT("www.harrypotterfanfiction.com");

//================================================================================
// constructor
jfFicURLParser::jfFicURLParser() {
  validURL = false;
  type = jfft_FFN;
}
//++++++++++++++++++++++++++++++++++++++
// data in
bool jfFicURLParser::ParseURL(const wxString& input_url) {
  // variables
  bool okay = true;
  // initial setup
  validURL = false;
  inputURL = input_url;
  fname_base.clear();
  wxArrayString* slashparsed = ListToStrArr(input_url,wxT("/"),false);
  if (slashparsed==NULL) return false;
  // shipping possible stuff before the domain...
  if (!ProcessStart(slashparsed,writehttp)) {
    delete slashparsed;
    return false;
  }
  // checking the domain
  wxString domain = slashparsed->Item(0);
  slashparsed->RemoveAt(0);
  // fanfiction.Net
  if      (domain == FFN_TAG) okay = HandleFFN(slashparsed);
  else if (domain == FIM_TAG) okay = HandleFIM(slashparsed);
  else if (domain == AO3_TAG) okay = HandleAO3(slashparsed);
  else if (domain == MMO_TAG) okay = HandleMMO(slashparsed);
  else if (domain == HPF_TAG) okay = HandleHPF(slashparsed);
  else okay = false;

  // finishing
  delete slashparsed;
  validURL = okay;
  return okay;
}
//++++++++++++++++++++++++++++++++++++++
// data out
//---------------------------
bool jfFicURLParser::isValid() const {
  return validURL;
}
//---------------------------
jf_FICTYPE jfFicURLParser::getType() const {
  return type;
}
//---------------------------
wxString jfFicURLParser::firstURL() {
  return outputURL;
}
//---------------------------
wxString jfFicURLParser::FilenameBase() {
  return fname_base;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfFicURLParser::ProcessStart(wxArrayString* urlarr, bool& writehttp) {
  if ((urlarr->GetCount()) < 2) return false;
  writehttp = true;
  // the first item might be the protocol
  wxString item0 = urlarr->Item(0);
  if ((item0 == wxT("http:")) || (item0 == wxT("https:"))) {
    outputURL = item0 + wxT("//");
    writehttp = false;
    urlarr->RemoveAt(0,2);
    if ((urlarr->GetCount()) < 2)  return false;
  }
  // it might be an empty protocol or one slash (we will treat this as okay)
  else if (item0 == wxEmptyString) {
    urlarr->RemoveAt(0);
    if ((urlarr->Item(0)) == wxEmptyString) urlarr->RemoveAt(0);
  }
  // done, there must still be parts left
  return ((urlarr->GetCount()) >= 2);
}
//--------------------------------------------------
bool jfFicURLParser::HandleFFN(wxArrayString* urlarr) {
  // variables
  wxString buffer,aftdomain;
  unsigned long tval;
  // starting...
  type = jfft_FFN;
  aftdomain = urlarr->Item(0);
  if (aftdomain != wxT("s")) return false;
  if (writehttp) outputURL = wxT("https://");
  outputURL += FFN_TAG;
  // checking to see if we have a fic id
  if ((urlarr->GetCount()) < 2) return false;
  // validating the fic id
  buffer = urlarr->Item(1);
  if (!Str2ULong(buffer,tval)) return false;
  // building the result...
  outputURL += wxT("/s/") + buffer;
  outputURL += wxT("/1/");
  // there can be a name at the end
  if ((urlarr->GetCount()) >= 4) {
    fname_base = urlarr->Item(3);
    outputURL += fname_base;
  }
  return true;
}
//--------------------------------------------------------------
bool jfFicURLParser::HandleFIM(wxArrayString* urlarr) {
  // variables
  wxString buffer,aftdomain;
  unsigned long tval;
  size_t ucount;
  // starting...
  type = jfft_FIM;
  aftdomain = urlarr->Item(0);
  if (aftdomain != wxT("story")) return false;
  ucount = urlarr->GetCount();
  if (writehttp) outputURL = wxT("http://");
  outputURL += FIM_TAG;
  // checking to see if we have a fic id
  if (ucount < 2) return false;
  // validating the fic id
  buffer = urlarr->Item(1);
  if (!Str2ULong(buffer,tval)) return false;
  // building the result...
  outputURL += wxT("/story/") + buffer;
  outputURL += wxT("/");
  // there might be a name afterwards
  if (ucount >= 3) {
    buffer = urlarr->Item(2);
    if (!Str2ULong(buffer,tval)) {
      fname_base = buffer;
    }
    else if (ucount >= 4) {
      fname_base = urlarr->Item(3);
    }
    outputURL += fname_base;
  }
  return true;
}
//-------------------------------------------------
bool jfFicURLParser::HandleAO3(wxArrayString* urlarr) {
  // variables
  wxString buffer1,buffer2,aftdomain;
  unsigned long tval;
  int spos;
  // starting...
  type = jfft_AO3;
  aftdomain = urlarr->Item(0);
  if (aftdomain != wxT("works")) return false;
  if (writehttp) outputURL = wxT("http://");
  outputURL += AO3_TAG;
  // checking to see if we have a fic id
  size_t ucount = urlarr->Count();
  if (ucount < 2) return false;
  // validating the fic id
  buffer1 = urlarr->Item(1);
  spos = buffer1.Find(wxT('?'));
  if (spos != wxNOT_FOUND) buffer2 = buffer1.Left(spos);
  else buffer2 = buffer1;
  if (!Str2ULong(buffer2,tval)) return false;
  // building the result...
  outputURL += wxT("/works/") + buffer2;
  outputURL += wxT("?view_adult=true");
  // names are not included
  return true;
}
//----------------------------------------------
bool jfFicURLParser::HandleHPF(wxArrayString* urlarr) {
  // variables
  wxString aftdomain,buffer;
  unsigned long tval;
  int spos;
  // starting...
  type = jfft_HPF;
  aftdomain = urlarr->Item(0);
  if (writehttp) outputURL = wxT("http://");
  outputURL += HPF_TAG + wxT("/");
  // harry potter fanfic urls must be index pages
  // format "viewstory.php?psid=<num>"
  const wxString hpf_start = wxT("viewstory.php?psid=");
  if (!aftdomain.StartsWith(hpf_start,&buffer)) return false;
  // checking the fic id
  if (!Str2ULong(buffer,tval)) return false;
  // building the result...
  outputURL += hpf_start + buffer;
  // names are not included
  return true;
}
//-----------------------------------------------
bool jfFicURLParser::HandleMMO(wxArrayString* urlarr) {
  // variables
  wxString buffer,aftdomain;
  unsigned long tval;
  size_t ucount;
  // starting...
  type = jfft_MMO;
  aftdomain = urlarr->Item(0);
  if (aftdomain != wxT("fanfic")) return false;
  ucount = urlarr->GetCount();
  if (ucount < 5) return false;
  if (writehttp) outputURL = wxT("http://");
  outputURL += MMO_TAG;
  // the fourth item can be used as the fic name
  fname_base = urlarr->Item(3);
  // checking the fic id
  buffer = urlarr->Item(4);
  if (!Str2ULong(buffer,tval)) return false;
  // building the result...
  outputURL += wxT("/fanfic/s/") + urlarr->Item(2);
  outputURL += wxT("/") + fname_base;
  outputURL += wxT("/") + buffer;
  return true;
}

/***************************************************************************/
