/******************************************************************************
// Name:        ffndown_dialog.cpp
// Author :     John Q Metro
// Purpose :    Defines the dialog for downloading Fanfiction.Net fics
// Created:     July 17, 2011
// Updated:     July 8, 2016
******************************************************************************/
#ifndef FFNDOWN_DIALOG_H_INCLUDED
  #include "ffndown_dialog.h"
#endif // FFNDOWN_DIALOG_H_INCLUDED
#ifndef SKELCOLL_H_INCLUDED
  #include "../../core/structs/skelcoll.h"
#endif // SKELCOLL_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef INITEND_H_INCLUDED
  #include "../../initend.h"
#endif // INITEND_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED


#include <wx/dirdlg.h>
#include <wx/dir.h>
#include <wx/filename.h>
/*****************************************************************************/
jfDown_DialogCore::jfDown_DialogCore(wxWindow* parent, wxWindowID id,
    const wxString& title, wxString inname):wxDialog(parent,id,title,wxDefaultPosition,
    wxDefaultSize,wxDEFAULT_DIALOG_STYLE,inname) {
  // starting...
  fparams = NULL;
  dirstring = (*globalsettings::globaldirs)[4];
  // null setting
  xmain_data = NULL;
  main_sizer = NULL;
  tworowstack = NULL;
  topblock = NULL;
  wax = true;
  data_sizer = new wxStaticBoxSizer(wxVERTICAL,this,wxT("Fanfic Information"));
  // start with the GUI
  BuildLabels();
  BuildWidgets();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handlers from the panel
//----------------------------------------------------------
void jfDown_DialogCore::EvFailSignal(wxCommandEvent& mevent) {
  EndCore(true);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// button handler events
//---------------------------------------------------------------
void jfDown_DialogCore::EventLaunchDirPicker(wxCommandEvent& mevent) {
  /**/jfXLogMessage(wxT("jfDown_DialogCore::EventLaunchDirPicker"),0,dirstring);
  wxDirDialog* xdialog = new wxDirDialog(this,wxDirSelectorPromptStr,dirstring);
  if (wxID_OK==(xdialog->ShowModal())) {
    dirstring = xdialog->GetPath();
    LoadSubdirs();
  }
  xdialog->Destroy();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//private helpher methods
// arranges the labels
void jfDown_DialogCore::BuildLabels() {
  // creating the top row labels
  title_label = new wxStaticText(this,-1,wxEmptyString);
  author_tag = new wxStaticText(this,-1,wxT("by "));
  author_label = new wxStaticText(this,-1,wxEmptyString);
  // arranging in a sizer
  toprow = new wxBoxSizer(wxHORIZONTAL);
  toprow->Add(title_label,0,wxALIGN_CENTRE_VERTICAL|wxALL,3);
  toprow->AddSpacer(8);
  toprow->Add(author_tag,0,wxALIGN_CENTER_VERTICAL|wxALL,3);
  toprow->Add(author_label,0,wxALIGN_CENTRE_VERTICAL|wxTOP|wxBOTTOM|wxRIGHT,3);
  // the next row
  wxString buffer = wxT("Unknown partcount");
  parcount_label = new wxStaticText(this,-1,buffer);
  ficid_label = new wxStaticText(this,-1,wxT("Fic ID : "));
  // arranging in a sizer
  rowtwo = new wxBoxSizer(wxHORIZONTAL);
  rowtwo->Add(parcount_label,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
  rowtwo->AddSpacer(8);
  rowtwo->Add(ficid_label,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
  // done
}
//------------------------------------------------------------
// building the text entries and the status displays
void jfDown_DialogCore::BuildWidgets() {
  // the directory picker button
  LDirPicker = new wxButton(this,jfID_FFNDOWN_DIRBTN,wxT("Directory..."));
  // name editor and split guide
  name_edit = new jfLabeledEdit(this,-1,wxT("Filename base"),wxEmptyString,4,wxHORIZONTAL);
  split_guide = new jfFilesizeHintWidget(this);
  // the subdirectory picker
  TSubLabel = new wxStaticText(this,-1,wxT("Target Subdirectory :"));
  SDirPicker = new wxChoice(this,-1);
  sdp_sizer = new wxBoxSizer(wxVERTICAL);
  sdp_sizer->Add(TSubLabel,0,wxEXPAND|wxALL,2);
  sdp_sizer->Add(SDirPicker,0,wxEXPAND|wxBOTTOM|wxLEFT|wxRIGHT,2);
  // arranging...
  rowthree = new wxBoxSizer(wxHORIZONTAL);
  rowthree->Add(LDirPicker,0,wxALL|wxEXPAND,3);
  rowthree->Add(name_edit,1,wxEXPAND|wxALIGN_LEFT|wxALL,3);
  rowthree->Add(split_guide,0,wxEXPAND|wxALIGN_RIGHT|wxALL,3);
  // the display panel
  mpanel = new jfProgressPanelSingle(this,wxT("Fanfic Download Progress Panel"));

}
//---------------------------------------------------------------
void jfDown_DialogCore::LoadFromExtract(bool loadname) {
  // checks
  assert(xmain_data!=NULL);
  // variables
  wxString buffer1, buffer2;
  // starting
  title_label->SetLabel(xmain_data->fic_title);
  author_label->SetLabel(xmain_data->author_name);
  // partcount
  buffer2 = wxT("Part");
  if ((xmain_data->pcount)>1) buffer2 += wxT("s");
  buffer1 = SInt2Str(xmain_data->pcount);
  parcount_label->SetLabel(buffer1 + wxT(" ") + buffer2);
  // fic id
  buffer1 = wxT("Fic ID : ");
  buffer2 = SInt2Str(xmain_data->fic_id);
  ficid_label->SetLabel(buffer1+buffer2);
  // finishing off
  if (loadname) {
    name_edit->SetFiletag(xmain_data->fic_title);
  }
  Layout();
}
//------------------------------------------------------------------------
void jfDown_DialogCore::FinishSizers(bool sdpnlabel) {
  assert(main_sizer!=NULL);
  assert(data_sizer!=NULL);
  if (sdpnlabel) {
    tworowstack = new wxBoxSizer(wxVERTICAL);
    tworowstack->Add(toprow,0,wxALL|wxEXPAND,3);
    tworowstack->Add(rowtwo,0,wxALL|wxEXPAND,3);
    topblock = new wxBoxSizer(wxHORIZONTAL);
    topblock->Add(tworowstack,3,wxEXPAND,0);
    topblock->Add(sdp_sizer,1,wxEXPAND|wxALL,1);
    data_sizer->Add(topblock,0,wxALL|wxEXPAND,1);
  }
  else {
    data_sizer->Add(toprow,0,wxALL|wxEXPAND,3);
    data_sizer->Add(rowtwo,0,wxALL|wxEXPAND,3);
  }
  data_sizer->Add(rowthree,0,wxALL|wxEXPAND,3);
  main_sizer->Add(data_sizer,0,wxALL|wxEXPAND,1);
  main_sizer->Add(mpanel,1,wxALL|wxEXPAND,6);
}
//-----------------------------------------------------------------------------
bool jfDown_DialogCore::LoadSubdirs() {
  // variables
  wxArrayString* results;
  wxDir dircheck;
  wxString* dvalue;
  bool tres;
  // special case
  if (!wxDir::Exists(dirstring)) return false;
  // setting up the data
  results = new wxArrayString();
  dvalue = new wxString;
  tres = dircheck.Open(dirstring);
  assert(tres);
  results->Add(wxEmptyString);
  // starting
  tres = dircheck.GetFirst(dvalue,wxALL_FILES_PATTERN,wxDIR_DIRS);
  while (tres) {
    results->Add(*dvalue);
    tres = dircheck.GetNext(dvalue);
  }
  // loading the wxChoice with our results
  SDirPicker->Clear();
  SDirPicker->Append(*results);
  SDirPicker->SetSelection(0);
  // getting rid of values
  delete dvalue;
  delete results;
  return true;
}
//-------------------------------------------------------------------------
wxString jfDown_DialogCore::GetFullDir() const {
  wxString sdir,result;
  sdir = SDirPicker->GetStringSelection();
  if (sdir.IsEmpty()) result = dirstring;
  else result = dirstring + wxFileName::GetPathSeparator() + sdir;
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// methods to change things during and after downloads
//-------------------------------------------------------------------
// after a search, we delete some no longer needed data
void jfDown_DialogCore::ClearParams() {
  delete tparams;
  tparams = NULL;
}
//---------------------------------------------------------------------
// ending, a few things to do
void jfDown_DialogCore::EndCore(bool fail) {
  // the rest of it
  name_edit->Enable();
  split_guide->Enable();
  ClearParams();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfDown_DialogCore,wxDialog)
  EVT_COMMAND (-1, jf_PP_SRCHDONE_EVENT , jfDown_DialogCore::EvEndSignal )
  EVT_COMMAND (-1, jf_PP_SRCHFAIL_EVENT , jfDown_DialogCore::EvFailSignal)
  EVT_BUTTON  (jfID_STARTSTOPBTN   , jfDown_DialogCore::EvStartButton       )
  EVT_BUTTON  (jfID_FFNDOWN_DIRBTN , jfDown_DialogCore::EventLaunchDirPicker)
END_EVENT_TABLE()
//****************************************************************************
jfDown_Dialog::jfDown_Dialog(wxWindow* parent, wxWindowID id, const wxString& title,
      jfFicExtract* in_main_data):jfDown_DialogCore(parent,id,title,wxT("jfDown_Dialog")) {
  // we start
  assert(in_main_data!=NULL);
  xmain_data = in_main_data;
  main_thread = NULL;
  // more stuff
  LoadFromExtract(true);
  // now we build and set the toplevel sizer
  main_sizer = new wxBoxSizer(wxVERTICAL);
  FinishSizers(true);
  SetSize(705,245);
  // done
  SetSizer(main_sizer);
  LoadSubdirs();
	CentreOnScreen(wxBOTH);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfDown_Dialog::EvEndSignal(wxCommandEvent& mevent) {
  EndCore(false);
}
//--------------------------------------
void jfDown_Dialog::EvStartButton(wxCommandEvent& mevent) {
  // varibales
  wxString tdvalue;
  bool xres;
  size_t xlong;
  // starting
  wxBeginBusyCursor();
  // start making the values to pass to the thread
  jfNewPauser* pval = mpanel->GetPauser();
  pval->SetTarget(mpanel);
  tparams = new jfDownloadParams();
  tparams->pauselink = pval;
  tparams->msg_target = mpanel;
  // start thread creation
  main_thread = new jfDownThread();
  // passing values
  tdvalue = GetFullDir();
  main_thread->SetParams(tparams);
  // more values to pass to the thread
  xlong= split_guide->GetValue();
  assert(xres);
  fparams = new jfFicDownParams(name_edit->GetValue(),xlong,mainskelc->AtIndexF(2));
  fparams->indir = tdvalue;
  fparams->extract_data = xmain_data;
  fparams->downtype = xmain_data->GetFicType();
  main_thread->SetFicData(fparams);
  // changing some labels
  name_edit->Enable(false);
  split_guide->Enable(false);
  // finishing off
  main_thread->Create();
  main_thread->Run();
  // done
  wxEndBusyCursor();
}
//****************************************************************************
// constructor
jfDown_Dialog2::jfDown_Dialog2(wxWindow* parent, wxWindowID id, const wxString& title):
      jfDown_DialogCore(parent,id,title,wxT("jfDown_Dialog2")) {
  // we start
  xmain_data = NULL;
  main_thread = NULL;
  // creating the url entry widget
  urlentry = new jfLabeledEdit(this,-1,wxT("Please enter the Fanfic URL Below"),wxEmptyString,1,wxVERTICAL);
  wxSize csize(22,22);
  cbtn = new wxButton(this,jfID_FFNDOWN_COPBTN,wxT("c"));
  cbtn->SetMinSize(csize);
  tsizer = new wxBoxSizer(wxHORIZONTAL);
  tsizer->Add(urlentry,7,wxEXPAND,0);
  tsizer->AddSpacer(6);
  tsizer->Add(cbtn,0,wxALL|wxALIGN_BOTTOM,0);
  tsizer->AddSpacer(6);
  tsizer->Add(sdp_sizer,2,wxEXPAND,0);
  // now we build and set the toplevel sizer
  main_sizer = new wxBoxSizer(wxVERTICAL);
  data_sizer->Add(tsizer,0,wxALL|wxEXPAND,3);
  FinishSizers(false);
  name_edit->SetFiletag(wxT("GenericFileName"));
  SetSize(760,305);
  // done
  SetSizer(main_sizer);
  LoadSubdirs();
	CentreOnScreen(wxBOTH);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// events
//------------------------------------------------
void jfDown_Dialog2::EvEndSignal(wxCommandEvent& mevent) {
  EndCore(false);
  // fic data not needed
  delete xmain_data;
  xmain_data = NULL;
}
//--------------------------------------
void jfDown_Dialog2::EvStartButton(wxCommandEvent& mevent) {
  // variables
  wxString tdvalue;
  long xvalue;
  bool xres;
  // starting
  if (TestUrlEntry()) {
    wxBeginBusyCursor();
    mpanel->Reset();
    // start making the values to pass to the thread
    jfNewPauser* pval = mpanel->GetPauser();
    pval->SetTarget(this);
    tparams = new jfDownloadParams();
    tparams->pauselink = pval;
    tparams->msg_target = mpanel;
    // start thread creation
    main_thread = new jfDownThread2();
    // passing values
    tdvalue = GetFullDir();
    main_thread->SetParams(tparams);
    // more values to pass to the thread
    xvalue = split_guide->GetValue();
    fparams = new jfFicDownParams(name_edit->GetValue(),xvalue,mainskelc->AtIndexF(2));
    fparams->indir = tdvalue;
    fparams->downtype = ftype;
    fparams->firsturl = firsturl;
    main_thread->SetFicData(fparams);
    // changing some labels
    name_edit->Enable(false);
    split_guide->Enable(false);
    // finishing off
    main_thread->Create();
    main_thread->Run();
    // done
    wxEndBusyCursor();
  }
  else  mpanel->CancelStart();
}
//------------------------------------------------
// for this version, the fic info is extracted from the first page and sent here
void jfDown_Dialog2::EvHandleLoadInfo(wxCommandEvent& mevent) {
  // variables
  jfFicExtract* qval;
  // starting
  qval = reinterpret_cast<jfFicExtract*>(mevent.GetClientData());
  assert(qval!=NULL);
  xmain_data = qval;
  LoadFromExtract(false);
}
//------------------------------------------------------
void jfDown_Dialog2::EvCopyName(wxCommandEvent& mevent) {
  // variables
  wxString urlcore,bval;
  // starting
  urlcore = urlentry->GetValue();
  Dtrim(urlcore);
  if (!urlcore.IsEmpty()) {
    if (uparse.ParseURL(urlcore)) {
      bval = uparse.FilenameBase();
      if (!bval.IsEmpty()) name_edit->SetValue(bval);
    }
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal helper methods
// takes the url in urlentry, and checks if it is valid
bool jfDown_Dialog2::TestUrlEntry() {
  wxString testurl;

  testurl = urlentry->GetValue();
  Dtrim(testurl);
  if (uparse.ParseURL(testurl)) {
    // valid
    ftype = uparse.getType();
    firsturl = uparse.firstURL();
    urlentry->SetValue(firsturl);
    return true;
  }
  else {
    // not valid
    wxMessageBox(wxT("The URL is not a valid URL!"),
        wxT("Error!"),wxICON_ERROR | wxOK,this);
    return false;
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handler table
BEGIN_EVENT_TABLE(jfDown_Dialog2,jfDown_DialogCore)
  EVT_COMMAND (-1,jf_FICDOWN_FICINFO ,jfDown_Dialog2::EvHandleLoadInfo)
  EVT_BUTTON  (jfID_FFNDOWN_COPBTN   ,jfDown_Dialog2::EvCopyName  )
END_EVENT_TABLE()
/*****************************************************************************/
