/******************************************************************************
// Name:        ffndupd_dialog.cpp
// Author :     John Q Metro
// Purpose :    Defines the dialog for updating downloaded Fanfiction.Net fics
// Created:     January 2, 2012
// Updated:     January 19, 2014
******************************************************************************/
#ifndef FFNDUPD_DIALOG_H_INCLUDED
  #include "ffndupd_dialog.h"
#endif // FFNDUPD_DIALOG_H_INCLUDED
#ifndef INITEND_H_INCLUDED
  #include "../../initend.h"
#endif // INITEND_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef SKELCOLL_H_INCLUDED
  #include "../../core/structs/skelcoll.h"
#endif // SKELCOLL_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED

//-----------------------------------------------------
/*****************************************************************************/
jfFic_UpdatePanel::jfFic_UpdatePanel(wxWindow* parent, jf_MPP_Config qlayout, const wxString& name):
        jfProgressPanelMultiBase(parent,qlayout,name) {
  // some initialization
  toptarget = dynamic_cast<wxPanel*>(parent);
  section_index = 0;
  infics = false;
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// other new event handlers
//------------------------------------------------
void jfFic_UpdatePanel::HandleNoFilesEvent(wxCommandEvent& event) {
  event.Skip();
}
//------------------------------------------------
void jfFic_UpdatePanel::HandleUpdateAnswerEvent(wxCommandEvent& event) {
  event.Skip();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// if we want to relaunch a search, we have to reset things
bool jfFic_UpdatePanel::Reset() {
  BaseReset();
  cat_display->SetToZero();
  catindex = 0;
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// resizer methods
//------------------------------------------------
bool jfFic_UpdatePanel::ReduceSize() {
  return ReduceSizeInt();
}
//------------------------------------------------
bool jfFic_UpdatePanel::UnReduceSize() {
  return UnReduceSizeInt();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// extra internal method
void jfFic_UpdatePanel::SendFicCount(size_t fcount) const {
  wxCommandEvent outevent(jf_FICDOWNUP_FICNUM,GetId());
  outevent.SetInt(fcount);
  wxPostEvent(toptarget,outevent);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented internal virtual methods
//------------------------------------------------
void jfFic_UpdatePanel::PostFinished() {
  wxCommandEvent outevent(jf_PP_SRCHDONE_EVENT,GetId());
  wxPostEvent(toptarget,outevent);
}
//------------------------------------------------
void jfFic_UpdatePanel::PostFaliure(bool halted) {
  wxCommandEvent outevent(jf_PP_SRCHFAIL_EVENT,GetId());
  wxPostEvent(toptarget,outevent);
}
//------------------------------------------------
void jfFic_UpdatePanel::PostStopping() {}
//------------------------------------------------
void jfFic_UpdatePanel::PostStarted() {}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfFic_UpdatePanel,jfProgressPanelMultiBase)
  EVT_COMMAND (-1,jf_FICDOWNUP_NOFILES ,jfFic_UpdatePanel::HandleNoFilesEvent     )
  EVT_COMMAND (-1,jf_FICDOWNUP_UPDATE  ,jfFic_UpdatePanel::HandleUpdateAnswerEvent)
END_EVENT_TABLE()
//****************************************************************************
// constructor
jfFic_UpdateDisplay::jfFic_UpdateDisplay(wxWindow* parent, wxWindowID id, const wxString& name):
      wxPanel(parent,wxID_ANY,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // initializing
  tparams = NULL;
  update_thread = NULL;
  ficucount = 0;
  ficupd = 0;
  launched = false;
  sbox = new wxStaticBoxSizer(wxVERTICAL,this,wxT("Result Information"));
  // orientation
  bool orient;
  size_t vsize;
  #ifdef __WXGTK__
    orient = true;
    vsize = 450;
  #else
    orient = false;
    vsize = 420;
  #endif
  // we start with the GUI elemets
  dir_pick = new jfDirPicker(this,-1,orient,(*globalsettings::globaldirs)[4]);
  dir_pick->ChangeLabel(wxT("Pick the Directory to search for fanfics to update : "));
  split_guide = new jfFilesizeHintWidget(this);
  main_check = new wxCheckBox(this,-1,wxT("Check Completed Stories"));
  main_check->SetValue(false);
  main_display = new jfFic_UpdatePanel(this,jfmpp_Stacked);
  fic_count = new wxStaticText(this,-1,wxT("Nothing Done Yet"));
  results_display = new wxTextCtrl(this,-1,wxEmptyString,wxDefaultPosition,wxDefaultSize,wxTE_MULTILINE|wxTE_READONLY|wxTE_DONTWRAP);
  // arranging in sizers
  top_sizer = new wxBoxSizer(wxHORIZONTAL);
  top_sizer->Add(dir_pick,1,wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL,1);
  top_sizer->AddSpacer(6);
  top_sizer->Add(split_guide,0,wxALL|wxALIGN_CENTER_VERTICAL,1);
  main_sizer = new wxBoxSizer(wxVERTICAL);
  sbox->Add(fic_count,0,wxEXPAND|wxALL,3);
  sbox->Add(results_display,1,wxEXPAND|wxALL,3);
  main_sizer->Add(top_sizer,0,wxEXPAND|wxALL,3);
  main_sizer->Add(main_check,0,wxEXPAND|wxALIGN_LEFT|wxALL,3);
  main_sizer->Add(main_display,0,wxEXPAND|wxALL,3);
  main_sizer->Add(sbox,1,wxEXPAND|wxALL,3);
  // sizing
  wxSize qsize(760,vsize);
  SetSize(qsize);
  Centre();
  // completion
  SetSizer(main_sizer);
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handlers
//------------------------------------------------
void jfFic_UpdateDisplay::HandleNoFilesEvent(wxCommandEvent& event) {
  fic_count->SetLabel(wxT("No properly formatted fanfics found."));
}
//------------------------------------------------
void jfFic_UpdateDisplay::HandleUpdateAnswerEvent(wxCommandEvent& event) {
  // variables
  wxString ficname, resmsg;
  int hasbu;
  // checking the event info
  hasbu = event.GetInt();
  ficname = event.GetString();
  // building the message
  ficucount++;
  resmsg = wxT("The Fic ") + ficname;
  if (hasbu==0) resmsg += wxT(" has not been updated.\n");
  else if (hasbu==1) resmsg += wxT(" has been updated!\n");
  else if (hasbu==2) resmsg += wxT(" is missing!\n");
  else if (hasbu==3) resmsg += wxT(" has failed due to a parse error!\n");
  else if (hasbu==4) resmsg += wxT(" has failed!\n");
  else assert(false);
  // adding the message to the control
  if (hasbu!=0) results_display->AppendText(resmsg);
  if (hasbu==1) ficupd++;
}
//------------------------------------------------
void jfFic_UpdateDisplay::HandleFinished(wxCommandEvent& event) {
  wxString newlabel;
  if (ficucount!=0) {
    // check count
    newlabel = wxT("Check Done. ");
    newlabel += SInt2Str(ficucount) + wxT(" Fic");
    if (ficucount>1) newlabel += wxT("s");
    newlabel += wxT(" checked. ");
    // update count
    if (ficupd==0) newlabel += wxT("None");
    else newlabel += SInt2Str(ficupd);
    newlabel += wxT(" updated.");
    // setting the label
    fic_count->SetLabel(newlabel);
  }
  // this is here just in case we finish okay while ordering the panel to be closed
  wxCommandEvent outevent(jf_PP_SRCHFAIL_EVENT,GetId());
  wxPostEvent(GetParent(),outevent);
}
//-----------------------------------------------
void jfFic_UpdateDisplay::HandleHalted(wxCommandEvent& event) {
  event.SetId(GetId());
  event.Skip();
}
//------------------------------------------------
void jfFic_UpdateDisplay::HandleFicCount(wxCommandEvent& event) {
  wxString lab_msg = wxT("A total of ");
  ficucount = event.GetInt();
  assert(ficucount!=0);
  lab_msg += SInt2Str(ficucount) + wxT(" fanfic");
  if (ficucount==1) lab_msg += wxT(" is");
  else lab_msg += wxT("s are");
  lab_msg += wxT(" being checked.");
  fic_count->SetLabel(lab_msg);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// button event handlers
//------------------------------------------------
void jfFic_UpdateDisplay::StartLaunched(wxCommandEvent& event) {
  // variables
  jfFicDownParams* ficparams;
  long xvalue;
  bool xres;
  // starting
  wxBeginBusyCursor();
  if (launched) Reset();
  launched = true;
  // creating the data objects to start with
  tparams = new jfDownloadParams();
  tparams->msg_target = main_display;
  tparams->pauselink = main_display->GetPauser();
  tparams->no_major_retry = true;
  update_thread = new jfUpdateFicThread();
  // loading the thread values
  update_thread->SetParams(tparams);
  // fic parameters
  xvalue = split_guide->GetValue();
  ficparams = new jfFicDownParams(wxT("dummy"),xvalue,mainskelc->AtIndexF(2));
  ficparams->indir = dir_pick->GetPath();
  ficparams->check_complete = main_check->GetValue();
  update_thread->SetFicData(ficparams);
  // launching the thread
  update_thread->Create();
  update_thread->Run();
  // done
  wxEndBusyCursor();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// other methods
void jfFic_UpdateDisplay::Reset() {
  // status reset
  main_display->Reset();
  // resetting internal data
  update_thread = NULL;
  delete tparams;
  tparams = NULL;
  ficucount = 0;
  ficupd = 0;
  // clearing the labels and text results
  fic_count->SetLabel(wxT("Nothing Done Yet"));
  results_display->Clear();
  launched = false;
}
//----------------------------------------------------------
bool jfFic_UpdateDisplay::StopUpdate() {
  return main_display->OrderStop();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfFic_UpdateDisplay,wxPanel)
  EVT_COMMAND (-1,jf_FICDOWNUP_NOFILES ,jfFic_UpdateDisplay::HandleNoFilesEvent     )
  EVT_COMMAND (-1,jf_FICDOWNUP_UPDATE  ,jfFic_UpdateDisplay::HandleUpdateAnswerEvent)
  EVT_COMMAND (-1,jf_PP_SRCHDONE_EVENT ,jfFic_UpdateDisplay::HandleFinished         )
  EVT_COMMAND (-1,jf_PP_SRCHFAIL_EVENT ,jfFic_UpdateDisplay::HandleHalted           )
  EVT_COMMAND (-1,jf_FICDOWNUP_FICNUM  ,jfFic_UpdateDisplay::HandleFicCount         )
  EVT_BUTTON  (jfID_STARTSTOPBTN,jfFic_UpdateDisplay::StartLaunched   )
END_EVENT_TABLE()
/*****************************************************************************/

