/******************************************************************************
// Name:        ffndupd_dialog.h
// Author :     John Q Metro
// Purpose :    Declares the dialog for updating downloaded fics
// Created:     January 1, 2012
// Updated:     December 31, 2013
// Changing update from a dialog to a panel so the updater can be in the notebook
******************************************************************************/
#ifndef FFNDUPD_DIALOG_H_INCLUDED
#define FFNDUPD_DIALOG_H_INCLUDED
#endif // FFNDUPD_DIALOG_H_INCLUDED
#ifndef WIDGETS3_H_INCLUDED
  #include "../../interface/misc/widgets3.h"
#endif // WIDGETS3_H_INCLUDED
#ifndef FFNDOWN_UPDTH_H_INCLUDED
  #include "../threads/ffndown_updth.h"
#endif // FFNDOWN_UPDTH_H_INCLUDED
#ifndef MULTIPANEL_H_INCLUDED
  #include "../../interface/threads/multipanel.h"
#endif // MULTIPANEL_H_INCLUDED
#ifndef DOWN_DATA_H_INCLUDED
  #include "../../connect/down_data.h"
#endif // DOWN_DATA_H_INCLUDED
//--------------------------------------------------

/*****************************************************************************/
/* GUI class for displaying and handling the progress bars and thread buttons,
handles most of the events as well */
class jfFic_UpdatePanel : public jfProgressPanelMultiBase {
  public:
    // constructor
    jfFic_UpdatePanel(wxWindow* parent, jf_MPP_Config qlayout, const wxString& name = wxT("FFNFic_UpdatePanel"));
    // other new event handlers
    void HandleNoFilesEvent(wxCommandEvent& event);
    void HandleUpdateAnswerEvent(wxCommandEvent& event);
    // if we want to relaunch a search, we have to reset things
    virtual bool Reset();
    // resizer methods
    virtual bool ReduceSize();
    virtual bool UnReduceSize();
  protected:
    // extra internal method
    void SendFicCount(size_t fcount) const;
    // implemented internal virtual methods
    virtual void PostFinished();
    virtual void PostFaliure(bool halted);
    virtual void PostStopping();
    virtual void PostStarted();
    // other things
    wxPanel* toptarget;
    size_t section_index;
    wxString cficname;
    bool bar_inserted, infics;
    // event handling
    DECLARE_EVENT_TABLE()
};
//===========================================================================
// the main interface for updating downloaded Fanfiction.Net fics
class jfFic_UpdateDisplay : public wxPanel {
  public:
    // constructor
    jfFic_UpdateDisplay(wxWindow* parent, wxWindowID id, const wxString& name = wxT("Fic_UpdateDisplay"));
    // event handlers
    void HandleNoFilesEvent(wxCommandEvent& event);
    void HandleUpdateAnswerEvent(wxCommandEvent& event);
    void HandleFinished(wxCommandEvent& event);
    void HandleHalted(wxCommandEvent& event);
    void HandleFicCount(wxCommandEvent& event);
    // button event handlers
    void StartLaunched(wxCommandEvent& event);
    // other methods
    void Reset();
    bool StopUpdate();
  protected:
    // GUI widgets
    jfDirPicker* dir_pick;
    jfFilesizeHintWidget* split_guide;
    wxCheckBox* main_check;
    jfFic_UpdatePanel* main_display;
    wxStaticText* fic_count;
    wxTextCtrl* results_display;
    // sizer
    wxStaticBoxSizer* sbox;
    wxBoxSizer* top_sizer;
    wxBoxSizer* main_sizer;
    // thread data
    bool launched;
    size_t ficucount, ficupd;
    jfDownloadParams* tparams;
    jfUpdateFicThread* update_thread;
    // event handing
    DECLARE_EVENT_TABLE()
};
/*****************************************************************************/
