/******************************************************************************
Name    :   dirupdate_thread.cpp
Author  :   John Q Metro
Purpose :   Defines the thread that does directory updates (intended for
              syncing directories of downloaded fanfics).
Created :   July 5, 2015
Updated :   July 6, 2015
******************************************************************************/
#ifndef DIRUPDATE_THREAD_H_INCLUDED
  #include "dirupdate_thread.h"
#endif // DIRUPDATE_THREAD_H_INCLUDED
//----------------------------------
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef DOWN_DATA_H_INCLUDED
  #include "../../connect/down_data.h"
#endif // DOWN_DATA_H_INCLUDED
#ifndef UTILS3_H_INCLUDED
  #include "../../core/utils/utils3.h"
#endif // UTILS3_H_INCLUDED
/*****************************************************************************/
// constructor
jfDirUpdate_Thread::jfDirUpdate_Thread():jfCoreThread() {
  dupdate_obj = NULL;
  started = false;
  threadname = wxT("DIR UPDATE");
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methods
//---------------------------------------
void jfDirUpdate_Thread::SendDone(jfDown::ResultAction done_status) {
  // creating
  jfThreadEvent eventx(jf_DOWNL_PEND_EVENT,msg_target->GetId(),soption);
  // setting data
  eventx.SetResAction(done_status);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//---------------------------------------
wxThread::ExitCode jfDirUpdate_Thread::Entry() {
  started = true;
  bool xres;
  started = true;
  xres = Startup();
  wxSleep(1);
  //if (xres) xres = TextResults();
  if (xres) xres = DoUpdateActions();
  // checking after
  if (xres) SendEnd();
  started = false;
  return 0;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// passing parameters
void jfDirUpdate_Thread::SetDirectories(wxString sdir_in, wxString tdir_in, jfProgressPanelSingle* ptarget) {
  if ((!started) && (ptarget!=NULL)) {
    if ((!sdir_in.IsEmpty()) && (!tdir_in.IsEmpty())) {
      sdirx = sdir_in;
      tdirx = tdir_in;
      msg_target = ptarget;
    }
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal methods
//---------------------------------------
bool jfDirUpdate_Thread::Startup() {
  bool xres;
  // initials
  SendStart();
  SendTotalCount(4);
  // creating
  dupdate_obj = new jfDirectoryUpdater();
  // loading source directory
  SendLabels(wxT("Source directory content info"), wxT("Loading"));
  SendNewItem(-1,-1);
  xres = dupdate_obj->LoadSource(sdirx);
  if (!xres) {
    SendDone(jfDown::FAIL);
    return false;
  }
  SendDone(jfDown::PROCEED);
  // loading target directory
  SendLabels(wxT("Target directory content info"), wxT("Loading"));
  SendNewItem(-1,-1);
  xres = dupdate_obj->LoadTarget(tdirx);
  if (!xres) {
    SendDone(jfDown::FAIL);
    return false;
  }
  SendDone(jfDown::PROCEED);
  wxSleep(1);
  // calling the comparison function
  SendLabels(wxT("the directory contents"), wxT("Comparing"));
  SendNewItem(-1,-1);
  xres = dupdate_obj->CompareSourceAndTarget();
  if (!xres) {
    SendDone(jfDown::FAIL);
    return false;
  }
  SendDone(jfDown::PROCEED);
  // done
  return true;
}
//------------------------------------------------
bool jfDirUpdate_Thread::TextResults() {
  // constants and variables
  const wxString du_filename = wxT("dirupdate.txt");
  wxString redata;
  bool xres;
  // calling the comparison function
  SendLabels(wxT("Comparison results to a file"), wxT("Sending"));
  SendNewItem(-1,-1);
  // the loop
  dupdate_obj->ResetTargetIndex();
  while (dupdate_obj->NextTarget()) {
    redata += dupdate_obj->GetCurrentTargetInfo();
  }
  // writing
  xres = ToFile(redata,du_filename);
  if (!xres) {
    SendDone(jfDown::FAIL);
    return false;
  }
  SendDone(jfDown::PROCEED);
  // done
  return true;
}
//------------------------------------------------------
bool jfDirUpdate_Thread::DoUpdateActions() {
  const wxString fname = wxT("jfDirUpdate_Thread::DoUpdateActions");
  // variables
  size_t tindex, tcount;
  wxString action, filename;
  bool checkval = true;
  // preparing for the loop
  /**/TLog(fname,1);
  tcount = dupdate_obj->TargetCount();
  SendTotalCount(3+tcount);
  dupdate_obj->ResetTargetIndex();
  /**/TLogS(fname,2,tcount);
  // the loop
  for (tindex = 1; tindex<=tcount; tindex++) {
    // setting up for action
    dupdate_obj->NextTarget();
    action = dupdate_obj->GetCurrentTargetAction();
    /**/TLog(fname,3,action);
    filename = dupdate_obj->CurrentTargetPath();
    /**/TLog(fname,4,filename);
    SendLabels(filename, action);
    SendNewItem(tindex,tcount);
    // action time!
    checkval = dupdate_obj->CurrentTargetOkay();
    /**/TLogB(fname,5,checkval);
    if (checkval) checkval = dupdate_obj->DoCurrentTargetAction();
    // post check
    if (checkval) SendDone(jfDown::PROCEED);
    else {
      SendDone(jfDown::FAIL);
      break;
    }
    /**/TLog(fname,6);
  }
  // after loop
  return checkval;
}
/*****************************************************************************/
