/******************************************************************************
Name    :   ffndown_thread.cpp
Author  :   John Q Metro
Purpose :   Decfines the thread that downloads Fanfiction.Net fic parts
Created :   August 4, 2011
Updated :   July 7, 2016 (MediaMiner.Org changes)
******************************************************************************/
#ifndef FFNDOWN_THREAD_H_INCLUDED
  #include "ffndown_thread.h"
#endif // FFNDOWN_THREAD_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED

/*****************************************************************************/
bool FFN_FicTruncated(const wxString* ival) {
  const wxString footer_string = wxT("The author would like to thank you for your continued support. Your review has been posted.");
  assert(ival!=NULL);
  return ((ival->Find(footer_string))==wxNOT_FOUND);
}
//---------------------------------------------------
bool FIM_FicTruncated(const wxString* ival) {
  const wxString footer_string = wxT("<div id=\"story_comments\" class=\"comment_content_box\"");
  assert(ival!=NULL);
  return ((ival->Find(footer_string))==wxNOT_FOUND);
}
//---------------------------------------------------
bool AO3_FicTruncated(const wxString* ival) {
  const wxString footer_string1 = wxT("<!-- BEGIN comment section -->");

  const wxString footer_string2 = wxT("<!-- END work -->");
  assert(ival!=NULL);
  // ao3 sometimes gives strange results, so this is why I am trying alternate ends
  bool xres = ((ival->Find(footer_string1))==wxNOT_FOUND);
  if (xres) return ((ival->Find(footer_string2))==wxNOT_FOUND);
  else return false;
}
//---------------------------------------------------
bool HPF_FicTruncated(const wxString* ival) {
  const wxString footer_string = wxT("<hr style='width:90%'/><p id='newstitle'><span id='latestnews'>Other Similar Stories</span>");
  assert(ival!=NULL);
  return ((ival->Find(footer_string))==wxNOT_FOUND);
}
//---------------------------------------------------
bool MMO_FicTruncated(const wxString* ival) {
  const wxString footer_string = wxT("<footer>");
  assert(ival!=NULL);
  return ((ival->Find(footer_string))==wxNOT_FOUND);
}
//=================================================================
bool FFN_FicMissing(const wxString* ival) {
  const wxString miss_string = wxT("Story Not Found<hr size=1 noshade>Unable to locate story.");
  assert(ival!=NULL);
  return ((ival->Find(miss_string))!=wxNOT_FOUND);
}
//--------------------------------------------------
// password protected stories are treated as missing
bool FIM_FicMissing(const wxString* ival) {
  const wxString miss_string = wxT("<img src='//static.fimfiction.net/images/missing_story.png' />");
  const wxString pass_string = wxT("<form method=\"post\" action=\"\" id=\"password_form\">");
  assert(ival!=NULL);
  if ((ival->Find(pass_string))!=wxNOT_FOUND) return true;
  return ((ival->Find(miss_string))!=wxNOT_FOUND);
}
//--------------------------------------------------
bool HPF_FicMissing(const wxString* ival) {
  const wxString miss_string = wxT("ERROR locating story meta for psid");
  assert(ival!=NULL);
  return ((ival->Find(miss_string))!=wxNOT_FOUND);
}
//*****************************************************************************
jfDownThreadCore::jfDownThreadCore():jfDownloadThread() {
  xqval = NULL;
  ficparams = NULL;
}
//-----------------------------------
void jfDownThreadCore::SetFicData(jfFicDownParams* in_ficparams) {
  // some basic checks
  assert(in_ficparams!=NULL);
  ficparams = in_ficparams;
  // we start the loading
  // xqval = new jfFanfic(finval);
  MakeFanficObject();
  xqval->SetFileBase(ficparams->base,ficparams->indir,ficparams->sizeguide);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfDownThreadCore::SendDone(jfDown::ResultAction done_status) {
  // creating
  jfThreadEvent eventx(jf_DOWNL_PEND_EVENT,msg_target->GetId(),soption);
  // setting data
  eventx.SetResAction(done_status);
  if (done_status!=jfDown::PROCEED) {
    eventx.SetError(fres);
    if ((fres==jfDown::PARSE_ERROR) || (fres==jfDown::REDIRECTED)) {
      eventx.SetString2(err_string);
    }
  }
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods

//------------------------------------------------------
bool jfDownThreadCore::ProcessAndWrite() {
  // constants and vatiables
  const wxString fname = wxT("jfDownThreadCore::ProcessAndWrite");
  size_t fcount;
  bool wfres;
  /**/TLog(fname,1);
  // we start with processing the page data to get a file count and some basic info
  pageindex = 1;
  SendTotalCount(3);
  SendLabels(wxT("the Fanfic Split"),wxT("Calculating"));
  SendNewItem(-1,-1);
  /**/TLog(fname,2);
  if (!xqval->StartResult()) return false;
  fcount = xqval->GetFilecount();
  /**/TLogS(fname,3,fcount);
  epagecount = fcount + 2;
  SendTotalCount(epagecount);
  SendDone(jfDown::PROCEED);
  /**/TLog(fname,4);
  SendLabels(wxT("File"),wxT("Writing"));
  // next up, we loop for the output files
  for (size_t pageindex = 2; pageindex < epagecount; pageindex++) {
    // setting up file data
    SendNewItem(pageindex-1,fcount);
    wfres = xqval->WriteFile(pageindex-1);
    assert(wfres);
    SendDone(jfDown::PROCEED);
  }
  // once done...
  SendLabels(wxT("Cleanup"),wxT("Doing"));
  SendNewItem(-1,-1);
  xqval->FinishAfter();
  SendDone(jfDown::PROCEED);
  // done
  return true;
}
//---------------------------------------------------------------------------
wxThread::ExitCode jfDownThreadCore::HandleCore() {
  // variables and asserts
  bool pawres;
  // starting
  TypeSetup();
  SendStart();
  // downloading
  bool rval = GetPages();
  // next
  if (rval) {
    SendReset();
    pawres = ProcessAndWrite();
    assert(pawres);
    // done
    SendEnd();
  }
  return 0;
}
//---------------------------------------------------------------------------
void jfDownThreadCore::TypeSetup() {
  jf_FICTYPE ft = ficparams->downtype;
  // setting some values based upon the source of the downloaded fiv
  if (ft==jfft_FFN) {
    dconst->CheckMissing = FFN_FicMissing;
    dconst->CheckTruncated = FFN_FicTruncated;
    indexpage = false;
  }
  else if (ft==jfft_FIM) {
    dconst->CheckMissing = FIM_FicMissing;
    dconst->CheckTruncated = FIM_FicTruncated;
    indexpage = true;
    dconst->incookie = wxT("view_mature=true");
  }
  else if (ft==jfft_HPF) {
    dconst->CheckMissing = HPF_FicMissing;
    dconst->CheckTruncated = HPF_FicTruncated;
    indexpage = true;
  }
  else if (ft==jfft_MMO) {
    dconst->CheckTruncated = MMO_FicTruncated;
    indexpage = true;
  }
  else if (ft==jfft_AO3) {
    dconst->CheckTruncated = AO3_FicTruncated;
    indexpage = false;
  }
  else assert(false);
}
//------------------------------------------------------------------------
/* for archiveofourown.org, there is complication because multipart stories have chapter
pages, and trying to get them causes a redirection to a chapterpage with a new url */
bool jfDownThreadCore::AO3_RCheck() {
  // variables
  wxString backpage;
  bool redval;
  // starting
  backpage = fetchurl;
  redval = RedirectCheck();
  // this option, there is a redirection
  if (redval) {
    if (fetchurl==wxT("http://archiveofourown.org/")) {
      fetchurl = backpage;
      return false;
    }
    fetchurl += wxT("?view_adult=true");
  }
  // no rediction, we have an index page
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// virtual plug-in methods to handle some vital bits
//------------------------------------------------------------------------
void jfDownThreadCore::PreFetch1() {
  // sending the labels
  if (pageindex==1) {
    if (!indexpage) SendLabels(wxT("Part"),wxT("Fetching"));
    else SendLabels(wxT("Index Page"),wxT("Fetching"));
  }
  else if (pageindex==2) {
    if (indexpage) SendLabels(wxT("Part"),wxT("Fetching"));
  }
}
//------------------------------------------------------------------------
void jfDownThreadCore::PreFetch2() {
  bool succ = true;
  size_t pindex;
  // after the url, we probably have to check for redirection for some types
  if ((ficparams->downtype)==jfft_AO3) succ = AO3_RCheck();
  assert(succ);
  // ready to send the part
  if ((pageindex==1) && (indexpage)) SendNewItem(-1,-1);
  else if (pageindex==1) {
    if (soption) SendNewItem(-1,-1);
    else SendNewItem(1,-1);
  }
  else {
    pindex = (indexpage)?(pageindex-1):(pageindex);
    SendNewItem(pindex,ficparams->extract_data->pcount);
  }
}
//------------------------------------------------------------------------
void jfDownThreadCore::SetUrl() {
  if (pageindex==1) fetchurl = ficparams->firsturl;
  else fetchurl = ficparams->extract_data->MakeUrl((indexpage)?(pageindex-1):(pageindex));
}
//------------------------------------------------------------------------
bool jfDownThreadCore::ProcessPage() {
  if (pageindex==1) {
    if (indexpage) ficparams->extract_data = xqval->DoIndexPage(*page_data);
    else ficparams->extract_data = xqval->DoPageOne(*page_data);
    return (ficparams->extract_data!=NULL);
  }
  else {
    return (xqval->AddPage(*page_data));
  }
}

//*****************************************************************************
// the constructor
jfDownThread::jfDownThread():jfDownThreadCore() {}
//--------------------------------------
wxThread::ExitCode jfDownThread::Entry() {
  threadname = wxT("FIC DOWN");
  ficparams->firsturl = ficparams->extract_data->FirstUrl();
  return HandleCore();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
size_t jfDownThread::NewPagecount() const {
  size_t rval = ficparams->extract_data->pcount;
  if (indexpage) rval++;
  return rval;
}
//------------------------------------------------
void jfDownThread::PostPageAction() {}
//------------------------------------------------------------------------
bool jfDownThread::GetPages() {
  // constants
  const wxString fname = wxT("jfDownThread::GetPages");
  // item count setup
  epagecount = ficparams->extract_data->pcount;
  if (indexpage) epagecount++;
  SendTotalCount(epagecount);
  // the rest
  return LoopGet();
}
//------------------------------------------------------------------
void jfDownThread::MakeFanficObject() {
  // asserts
  assert(ficparams!=NULL);
  assert((ficparams->extract_data)!=NULL);
  // doing...
  xqval = new jfFanfic(ficparams->extract_data);
}

//****************************************************************************
jfDownThread2::jfDownThread2():jfDownThreadCore() {}
//---------------------------------------------------
wxThread::ExitCode jfDownThread2::Entry() {
  threadname = wxT("FIC DOWN 2");
  return HandleCore();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods
//---------------------------------------------------
size_t jfDownThread2::NewPagecount() const {
  size_t rval = ficparams->extract_data->pcount;
  if (indexpage) rval++;
  return rval;
}
//---------------------------------------------------
void jfDownThread2::PostPageAction() {
  if (pageindex==1) SendInfoSignal();
}
//---------------------------------------------------
bool jfDownThread2::GetPages() {
  // constants
  const wxString fname = wxT("jfDownThread2::GetPages");
  // item count setup
  epagecount = 2;
  SendTotalCount(epagecount);
  // the rest
  return LoopGet();
}
//--------------------------------------------------
void jfDownThread2::MakeFanficObject() {
  // asserts
  assert(ficparams!=NULL);
  // doing...
  xqval = new jfFanfic(ficparams->downtype);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// messages
//---------------------------------------------------
void jfDownThread2::SendInfoSignal() const {
  // creating
  wxCommandEvent eventx(jf_FICDOWN_FICINFO, msg_target->GetId());
  eventx.SetClientData(ficparams->extract_data);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
/*****************************************************************************/
