/******************************************************************************
Name    :   ffndown_updth.cpp
Author  :   John Q Metro
Purpose :   Defines an update thread for updating downloaded Fanfiction.Net fics
Created :   December 31, 2011
Updated :   July 7, 2017 (MediaMiner.Org changes)
******************************************************************************/
#ifndef FFNDOWN_UPDTH_H_INCLUDED
  #include "ffndown_updth.h"
#endif // FFNDOWN_UPDTH_H_INCLUDED
#ifndef UTILS3_H_INCLUDED
  #include "../../core/utils/utils3.h"
#endif // UTILS3_H_INCLUDED
#ifndef UTILS2_H_INCLUDED
  #include "../../core/utils/utils2.h"
#endif // UTILS2_H_INCLUDED
#ifndef FFNDOWN_THREAD_H_INCLUDED
  #include "ffndown_thread.h"
#endif // FFNDOWN_THREAD_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef INITEND_H_INCLUDED
  #include "../../initend.h"
#endif // INITEND_H_INCLUDED

#include <wx/dir.h>
//***************************************************************************
bool jfFicExtractArray::IdIncluded(const size_t& idtocheck,jf_FICTYPE idcheck) const {
  // variables
  size_t floop,fcount,ficid;
  bool identity;
  // we start
  fcount = size();
  for (floop=0;floop<fcount;floop++) {
    ficid = (*this)[floop]->fic_id;
    identity = ((idtocheck==ficid) && (idcheck==GetFtype(floop)));
    if (identity) break;
  }
  return (floop!=fcount);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// returns a pointer to an extract by id
jfFicExtract* jfFicExtractArray::GetByID(const size_t& idtocheck,jf_FICTYPE idcheck) const {
  // variables
  size_t floop,fcount,ficid;
  bool identity = false;
  // we start
  fcount = size();
  for (floop=0;floop<fcount;floop++) {
    ficid = (*this)[floop]->fic_id;
    identity = ((idtocheck==ficid) && (idcheck==GetFtype(floop)));
    if (identity) break;
  }
  if (identity) return (*this)[floop];
  else return NULL;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handles file deletion
size_t jfFicExtractArray::DeleteSourceFiles(size_t inindex) {
  // out of bounds
  if (inindex>=size()) return 0;
  // no file list
  if ((*this)[inindex]->fnamelist==NULL) return 0;
  // otherwise
  size_t sfcount = (*this)[inindex]->fnamelist->Count();
  for (size_t sfindex=0;sfindex<sfcount;sfindex++) {
    wxRemoveFile((*this)[inindex]->fnamelist->Item(sfindex));
  }
  // last stuff
  delete (*this)[inindex]->fnamelist;
  (*this)[inindex]->fnamelist = NULL;
  return sfcount;
}
//-------------------------------------------------------------------
// Deletes the source file, there can be only one
bool jfFicExtractArray::DeleteOnlySource(size_t inindex) {
  // out of bounds
  if (inindex>=size()) return false;
  // no file list
  if ((*this)[inindex]->fnamelist==NULL) return false;
  // otherwise
  size_t sfcount = (*this)[inindex]->fnamelist->Count();
  if (sfcount!=0) return false; // more than one file
  // deleting
  wxRemoveFile((*this)[inindex]->fnamelist->Item(0));
  // clearing up the name list
  delete (*this)[inindex]->fnamelist;
  (*this)[inindex]->fnamelist = NULL;
  return true;
}
//----------------------------------------------------------------------
bool jfFicExtractArray::ClearSourceFiles(size_t inindex) {
  // out of bounds
  if (inindex>=size()) return false;
  if ((*this)[inindex]->fnamelist==NULL) return true;
  else delete (*this)[inindex]->fnamelist;
  (*this)[inindex]->fnamelist = NULL;
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// deletes all the contained fics and empties the array
void jfFicExtractArray::ClearContents() {
  // variables
  size_t floop,fcount;
  // we start
  fcount = size();
  for (floop=0;floop<fcount;floop++) {
    delete (*this)[floop];
    (*this)[floop] = NULL;
  }
  clear();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jf_FICTYPE jfFicExtractArray::GetFtype(size_t index) const {
  assert(index<size());
  return (*this)[index]->GetFicType();
}
//===========================================================================
// constructor
jfUpdateFicThread::jfUpdateFicThread():jfMultiDownloadThread() {
  filelist = NULL;
  ficlist = NULL;
  xqval = NULL;
  soption = true;
  subdirs = NULL;
  missing_signal = false;
  ficparams = NULL;
  filecount = 0;
  ficcount = 0;
}
//------------------------------------------------
// public methods
void jfUpdateFicThread::SetFicData(jfFicDownParams* in_ficparams) {
  // parameter setting
  assert(in_ficparams!=NULL);
  ficparams = in_ficparams;
  // the directory
  assert(wxDir::Exists(ficparams->indir));
  // size guide
  assert((ficparams->sizeguide)>0);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the main method
wxThread::ExitCode jfUpdateFicThread::Entry() {
  const wxString fname = wxT("jfUpdateFicThread::Entry");
  // we start
  /**/TLog(fname,1);
  bool okay = true;
  SendStart();
  // first section, getting the list of files
  max_catcount = 3;
  catindex = 1;
  SendCatCount();
  /**/TLog(fname,2);
  filecount = GetFileList();
  /**/TLogS(fname,3,filecount);
  // next up, checking the html file list to see if any are fics
  if (filecount>0) {
    /**/TLog(fname,5);
    okay = CheckAllFiles();
    delete filelist;
    if (okay) {
      /**/TLogS(fname,6,ficcount);
      if (ficcount==0) SendNoFiles();
      else ProcessingFanfics();
      /**/TLog(fname,9);
      ficlist->ClearContents();
      delete ficlist;
      delete subdirs;
    }
  }
  else {
    delete filelist;
    SendNoFiles();
  }
  if (okay) SendEnd();
  return 0;
}
//----------------------------------------------------------
void jfUpdateFicThread::TypeSetup() {
  jf_FICTYPE ft = ficparams->downtype;
  // setting some values based upon the source of the downloaded fic
  if (ft==jfft_FFN) {
    dconst->CheckMissing = FFN_FicMissing;
    dconst->CheckTruncated = FFN_FicTruncated;
    indexpage = false;
  }
  else if (ft==jfft_FIM) {
    dconst->CheckMissing = FIM_FicMissing;
    dconst->CheckTruncated = FIM_FicTruncated;
    indexpage = true;
    dconst->incookie = wxT("view_mature=true");
  }
  else if (ft==jfft_HPF) {
    dconst->CheckMissing = HPF_FicMissing;
    dconst->CheckTruncated = HPF_FicTruncated;
    indexpage = true;
  }
  else if (ft==jfft_MMO) {
    dconst->CheckTruncated = MMO_FicTruncated;
    indexpage = true;
  }
  else if (ft==jfft_AO3) {
    dconst->CheckTruncated = AO3_FicTruncated;
    indexpage = false;
  }
  else assert(false);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// additional subsections
//---------------------------------------
// wrapping LoadHtmlFileList with messages
size_t jfUpdateFicThread::GetFileList() {
  const wxString fname = wxT("jfUpdateFicThread::GetFileList");
  size_t fcount;
  // doing
  /**/TLog(fname,1);
  SendCatLabels(wxT("Section"),wxT("Handling"));
  catname = wxT("Getting list of HTML files");
  SendNewCat(1,-1);
  /**/TLog(fname,2);
  epagecount = 1;
  SendTotalCount(1);
  SendLabels(wxT("Action"),wxT("Doing"));
  SendNewItem(-1,-1);
  fcount = LoadHtmlFilelist();
  SendDone(jfDown::PROCEED);
  /**/TLogS(fname,3,fcount);
  max_catcount = 2+fcount;
  SendCatCount();
  SendCatEnd(jfDown::PROCEED);
  return fcount;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// virtual plug-in methods to handle some vital bits
//------------------------------------------------
void jfUpdateFicThread::PreFetch1() {
  // sending the labels
  if (pageindex==1) {
    if (!indexpage) SendLabels(wxT("Part"),wxT("Fetching"));
    else SendLabels(wxT("Index Page"),wxT("Fetching"));
  }
  else if (pageindex==2) {
    if (indexpage) SendLabels(wxT("Part"),wxT("Fetching"));
  }
}
//------------------------------------------------
void jfUpdateFicThread::PreFetch2() {
  const wxString fname = wxT("jfUpdateFicThread::PreFetch2");
  bool succ = true;
  size_t pindex;
  // after the url, we probably have to check for redirection for some types
  /**/jfXLogMessage(fname,1);
  if ((ficparams->downtype)==jfft_AO3) succ = AO3_RCheck();
  assert(succ);
  /**/jfXLogMessage(fname,2,fetchurl);
  // ready to send the part
  if ((pageindex==1) && (indexpage)) SendNewItem(-1,-1);
  else if (pageindex==1) {
    /**/jfXLogMessage(fname,3);
    if (soption) SendNewItem(1,-1);
    else SendNewItem(1,-1);
  }
  else {
    /**/jfXLogMessage(fname,4);
    pindex = (indexpage)?(pageindex-1):(pageindex);
    SendNewItem(pindex,ficparams->extract_data->pcount);
  }
  /**/jfXLogMessage(fname,5);
}
//------------------------------------------------
void jfUpdateFicThread::SetUrl() {
  const wxString fname = wxT("jfUpdateFicThread::SetUrl");
  /**/jfXLogMessage(fname,1);
  if (pageindex==1) fetchurl = ficparams->firsturl;
  else fetchurl = ficparams->extract_data->MakeUrl((indexpage)?(pageindex-1):(pageindex));
  /**/jfXLogMessage(fname,2,fetchurl);
}
//------------------------------------------------
bool jfUpdateFicThread::ProcessPage() {
  if (pageindex==1) {
    if (indexpage) ficparams->extract_data = xqval->DoIndexPage(*page_data);
    else ficparams->extract_data = xqval->DoPageOne(*page_data);
    return (ficparams->extract_data!=NULL);
  }
  else {
    return (xqval->AddPage(*page_data));
  }
}
//------------------------------------------------
size_t jfUpdateFicThread::NewPagecount() const {
  size_t rval = ficparams->extract_data->pcount;
  if (indexpage) rval++;
  return rval;
}
//-------------------------------------
void jfUpdateFicThread::PostPageAction() {
  ficupdated = UpdateCheck(findex);
  limitr = (!ficupdated);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special methods
//------------------------------------------------
// getting a list of all html files in target_dir
size_t jfUpdateFicThread::LoadHtmlFilelist() {
  // variables
  size_t rcount;
  int rindex;
  wxFileName cfname;
  wxString extension;
  // getting the list
  filelist = new wxArrayString();
  rcount = wxDir::GetAllFiles(ficparams->indir,filelist,wxT("*.*"));
  // special check
  if (rcount==0) return 0;
  // filtering the list
  for (rindex = rcount-1;rindex>=0;rindex--) {
    cfname.Assign(filelist->Item(rindex));
    extension = cfname.GetExt();
    if ((extension!=wxT("html")) && (extension!=wxT("htm"))) {
      filelist->RemoveAt(rindex);
    }
  }
  // done
  return filelist->Count();
}
//------------------------------------------------
/* Reading an HTML file, checking if it is a fic, extracting info and avoiding
duplicates */
bool jfUpdateFicThread::CheckHtmlFile() {
  const wxString fname = wxT("jfUpdateFicThread::CheckHtmlFile");
  // variables
  wxString filedata, ficcore, filename, qval;
  bool rval;
  jfFicExtract* testdata;
  jfFicExtract* checkdata;
  jfStringParser* sparser;
  jf_FICTYPE fout_type;
  wxFileName cfname;
  // I've decided to get rid of any assert checks here
  /**/TLogS(fname,1,pageindex);
  filename = filelist->Item(pageindex-1);
  SendNewItem(pageindex,epagecount,filename);
  Sleep(100);
  // loading the data
  rval = FromFile(filedata,filelist->Item(pageindex-1));
  assert(rval);
  SendAction(wxT("Checking"));
  // next, we process the file data to get the fic info
  sparser = new jfStringParser(filedata);
  /**/TLog(fname,3);
  if (!sparser->GetDelimited(STARTINFO_HEADER,wxT("-->"),ficcore)) {
    /**/TLog(fname,4);
    fres = jfDown::PARSE_ERROR;
    err_string = wxT("The file is not a Jaffis fanfic file.");
    // not a fic
    delete sparser;
    SendDone(jfDown::SKIP);
    return true;
  }
  delete sparser;
  /**/TLog(fname,5);
  // processing the extract into a jfFicExtract
  testdata = MakeExtract(fout_type,ficcore);
  if (testdata==NULL) {
    fres = jfDown::PARSE_ERROR;
    /**/TLog(fname,6);
    err_string = wxT("The extract could not be parsed, program bug");
    SendDone(jfDown::FAIL);
    return false;
  }
  // checking if the fic exists already
  checkdata = ficlist->GetByID(testdata->fic_id,fout_type);
  // fic does not exist
  if (checkdata==NULL) {
    testdata->fnamelist = new wxArrayString();
    testdata->fnamelist->Add(filelist->Item(pageindex-1));
    ficlist->push_back(testdata);
    cfname.Assign(filename);
    qval = cfname.GetPath();
    cfname.AssignDir(qval);
    subdirs->Add(cfname.GetFullPath());
    ficcount++;
    SendDone(jfDown::PROCEED);
    return true;
  }
  else {
    /**/TLog(fname,10);
    checkdata->fnamelist->Add(filelist->Item(pageindex-1));
    SendDone(jfDown::PROCEED);
    return true;
  }
}
//-----------------------------------------------------------
// Check HTML File done for all the files
bool jfUpdateFicThread::CheckAllFiles() {
  const wxString fname = wxT("jfUpdateFicThread::CheckAllFiles");
  assert(filelist!=NULL);
  // variables
  bool rval;
  wxString qval;
  wxFileName cfname;
  // trivial cases
  /**/TLogS(fname,1,filecount);
  if (filecount==0) return 0;
  // preparing for the loop, first, we set up some values
  catindex = 2;
  catname = wxT("Checking all the HTML files");
  SendNewCat(2,-1);
  epagecount = filecount;
  SendTotalCount(filecount);
  SendLabels(wxT("File"),wxT("Loading"));
  /**/TLog(fname,2);
  // creating the result data arrays
  ficlist = new jfFicExtractArray();
  subdirs = new wxArrayString();
  /**/TLog(fname,3);
  // the scanning loop
  for (pageindex=1;pageindex<=filecount;pageindex++) {
    /**/TLogS(fname,4,pageindex);
    rval = CheckHtmlFile();
    /**/TLogB(fname,5,rval);
    assert(rval);
    rval = PauseHaltCheck();
    if (rval) break;
  }
  /**/TLogB(fname,6,rval);
  // done
  if (rval) return false;
  max_catcount = 2 + ficcount;
  SendCatCount();
  SendCatEnd(jfDown::PROCEED);
  return true;
}
//---------------------------------------------------------
jfFicExtract* jfUpdateFicThread::MakeExtract(jf_FICTYPE& outtype, wxString ficcore) {
  const wxString fname = wxT("jfUpdateFicThread::MakeExtract");
  // variables
  jfLineParse* lpval;
  jfFicExtract* testdata;
  bool rval;
  // starting
  Dtrim(ficcore);
  /**/TLog(fname,1,ficcore);
  lpval = ParseExtractSource(ficcore, outtype);
  if (lpval==NULL) return NULL;
  /**/TLog(fname,2);
  // determining the type
  if (outtype==jfft_FFN) testdata = new jfFicExtract_FFN();
  else if (outtype==jfft_FIM) testdata = new jfFicExtract_FIM();
  else if (outtype==jfft_HPF) testdata = new jfFicExtract_HPF();
  else if (outtype==jfft_MMO) testdata = new jfFicExtract_MMO();
  else if (outtype==jfft_AO3) testdata = new jfFicExtract_AO3();
  else {
    /**/TLog(fname,3);
    delete lpval;
    return NULL;
  }
  testdata->fnamelist = NULL;
  // loading from parsed data
  /**/TLog(fname,4);
  rval = testdata->GetFromParse(lpval);
  delete lpval;
  if (!rval) {
    /**/TLog(fname,5);
    delete testdata;
    return NULL;
  }
  /**/TLog(fname,6);
  return testdata;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// page downloader methods
bool jfUpdateFicThread::HandleFanfic(size_t ficindex) {
  const wxString fname = wxT("jfUpdateFicThread::HandleFanfic");
  // checking the fic index...
  assert(ficlist!=NULL);
  assert(ficindex<ficcount);
  // variables
  wxDateTime olddate;
  bool getres,pawres;
  jf_FICTYPE ftin;
  // sending the startup message
  /**/TLog(fname,1);
  catindex = ficindex+3;
  catname = (*ficlist)[ficindex]->fic_title;
  SendNewCat(ficindex+1,ficcount);
  // setting up working values
  /**/TLog(fname,2);
  ficupdated = false;
  ftin = ficlist->GetFtype(ficindex);
  xqval = new jfFanfic(ftin);
  ficparams->downtype = ftin;
  limitr = false;
  TypeSetup();
  ficparams->firsturl = (*ficlist)[ficindex]->FirstUrl();
  xqval->SetFileBase((*ficlist)[ficindex]->fname_core,(*subdirs)[ficindex],ficparams->sizeguide);
  /**/TLog(fname,3);
  // item count setup
  epagecount = (*ficlist)[ficindex]->pcount;
  if ((ficparams->downtype)==jfft_MMO) {
    if (epagecount!=0) epagecount++;
  }
  else if (indexpage) epagecount++;
  SendTotalCount(epagecount);
  /**/TLog(fname,4,ficparams->firsturl);
  // fetching
  getres = LoopGet();
  /**/TLogB(fname,5,getres);
  // checks the result status
  if (getres) {
    if (ficupdated) SendUpdateR(1);
    else SendUpdateR(0);
  }
  else {
    if (fres==jfDown::MISSING) SendUpdateR(2);
    else if (fres==jfDown::PARSE_ERROR) SendUpdateR(3);
    else SendUpdateR(4);
  }
  // stuff we do only if the fic is updated
  if (ficupdated) {
    SendReset();
    pawres = ProcessAndWrite(ficindex);
    assert(pawres);
  }
  ficlist->ClearSourceFiles(ficindex);
  // and finally...
  if (getres) SendCatEnd(jfDown::PROCEED);
  else if (fres==jfDown::HALT) SendCatEnd(jfDown::HALT);
  else SendCatEnd(jfDown::SKIP);
  return (fres!=jfDown::HALT);
}
//-----------------------------------------------------------
bool jfUpdateFicThread::ProcessAndWrite(size_t ficindex) {
  // constants and vatiables
  const wxString fname = wxT("jfUpdateFicThread::ProcessAndWrite");
  size_t fcount;
  bool wfres;
  /**/TLog(fname,1);
  // we start with processing the page data to get a file count and some basic info
  pageindex = 1;
  SendTotalCount(3);
  SendLabels(wxT("the Fanfic Split"),wxT("Calculating"));
  SendNewItem(-1,-1);
  /**/TLog(fname,2);
  if (!xqval->StartResult()) return false;
  fcount = xqval->GetFilecount();
  /**/TLogS(fname,3,fcount);
  epagecount = fcount + 2;
  SendTotalCount(epagecount);
  SendDone(jfDown::PROCEED);
  /**/TLog(fname,4);
  SendLabels(wxT("File"),wxT("Writing"));
  // next up, we loop for the output files
  for (size_t pageindex = 2; pageindex < epagecount; pageindex++) {
    // setting up file data
    SendNewItem(pageindex-1,fcount);
    wfres = xqval->WriteFile(pageindex-1);
    assert(wfres);
    SendDone(jfDown::PROCEED);
  }
  // once done...
  SendLabels(wxT("Cleanup"),wxT("Doing"));
  SendNewItem(-1,-1);
  if (fcount>1) ficlist->DeleteOnlySource(ficindex);
  xqval->FinishAfter();
  SendDone(jfDown::PROCEED);
  // done
  return true;
}
//----------------------------------------------------------
bool jfUpdateFicThread::ProcessingFanfics() {
  const wxString fname = wxT("jfUpdateFicThread::ProcessingFanfics");
  // the usual basic checks
  assert(ficlist!=NULL);
  assert(ficlist->size()!=0);
  // variables
  bool rval;
  bool incompletex;
  // the loop
  for (findex=0;findex<ficcount;findex++) {
    catindex = findex+3;
    // doing the fic...
    incompletex = !((*ficlist)[findex]->complete);
    if (incompletex || (ficparams->check_complete)) {
      rval = HandleFanfic(findex);
      if (!rval) break;
    }
    // otherwise, skipping
    else {
      catname = (*ficlist)[findex]->fic_title;
      SendNewCat(findex+1,ficcount);
      rval = PauseHaltCheck();
      if (!rval) SendCatEnd(jfDown::SKIP);
      else SendDone(jfDown::HALT);
      if (rval) break;
    }

    /**/TLog(fname,1,wxT("Here1"));
    rval = PauseHaltCheck();
    /**/TLogB(fname,2,rval);
    if (rval) break;
    // done

  }
  // done
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// virtual methods implemented here
//------------------------------------------------
bool jfUpdateFicThread::IdentifyMissing(const wxString* indata) const {
  // constants and asserts
  assert(indata!=NULL);
  const wxString locval3 = wxT("Story Not Found<hr size=1 noshade>Unable to locate story.");
  const wxString fimmiss = wxT("<img src='//static.fimfiction.net/images/missing_story.png' />");
  const wxString hpfmiss = wxT("ERROR locating story meta for psid ");
  // variables
  bool rval;
  jf_FICTYPE ft;
  // checking for the tell-table sting
  ft = ficparams->downtype;
  if (ft==jfft_FFN) rval = ((indata->Find(locval3))==wxNOT_FOUND);
  else if (ft==jfft_FIM) rval = ((indata->Find(fimmiss))==wxNOT_FOUND);
  else if (ft==jfft_HPF) rval = ((indata->Find(hpfmiss))==wxNOT_FOUND);
  // the following two have no missing story pages, they just redirect to the front page!
  else if ((ft==jfft_MMO) || (ft==jfft_AO3)) rval = true;
  else assert(false);
  return !rval;
}
//----------------------------------------------------
bool jfUpdateFicThread::UpdateCheck(size_t ficindex_in) const {
  // constants
  const wxString fname = wxT("jfUpdateFicThread::UpdateCheck");
  // variables
  wxDateTime olddate,newdate;
  bool completor;
  size_t opcount,npcount;
  // getting dates
  olddate = (*ficlist)[ficindex_in]->updated_date;
  newdate = ficparams->extract_data->updated_date;
  // completion status
  completor = !((*ficlist)[ficindex_in]->complete);
  completor = completor && ficparams->extract_data->complete;
  completor = (olddate.IsEarlierThan(newdate) || completor);
  if (completor) return true;
  /* part count, since I've found that chapter counts in fanfiction.net which
  I used to use are unreliable, this should fix any bad fics */
  opcount = (*ficlist)[ficindex_in]->pcount;
  npcount = ficparams->extract_data->pcount;
  completor = (opcount!=npcount);
  // finishing
  return completor;
}
//------------------------------------------------------------------------
/* for archiveofourown.org, there is complication because multipart stories have chapter
pages, and trying to get them causes a redirection to a chapterpage with a new url */
bool jfUpdateFicThread::AO3_RCheck() {
  const wxString fname = wxT("jfUpdateFicThread::AO3_RCheck");
  // variables
  wxString backpage;
  bool redval;
  // starting
  backpage = fetchurl;
  redval = RedirectCheck();
  /**/jfXLogMessageB(fname,1,redval);
  // this option, there is a redirection
  if (redval) {
    if (fetchurl==wxT("http://archiveofourown.org/")) {
      fetchurl = backpage;
      return false;
    }
    fetchurl += wxT("?view_adult=true");
    /**/jfXLogMessage(fname,2,fetchurl);
  }
  // no rediction, we have an index page
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfUpdateFicThread::SendNoFiles() {
  // creating
  wxCommandEvent eventx(jf_FICDOWNUP_NOFILES, msg_target->GetId());
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//------------------------------------------------
/* New values for sending the update :
  0 : Nothing     3 : Parse Error
  1 : Update      4 : Other Error
  2 : Missing
  */
void jfUpdateFicThread::SendUpdateR(size_t update_res) {
  // creating
  wxCommandEvent eventx(jf_FICDOWNUP_UPDATE, msg_target->GetId());
  // setting data
  assert(update_res<5);
  wxString ostring = (*ficlist)[findex]->fic_title;
  ostring += wxT(" ( by ");
  ostring += (*ficlist)[findex]->author_name + wxT(" )");
  eventx.SetInt(update_res);
  eventx.SetString(ostring);
  // sending the message
  wxPostEvent(msg_target,eventx);
}
//***************************************************************************
