/******************************************************************************
Name    :   ffndown_updth.h
Author  :   John Q Metro
Purpose :   Declares an update thread for updating downloaded Fanfiction.Net fics
Created :   December 31, 2011
Updated :   July 7, 2016 (MediaMiner.Org update)
******************************************************************************/
#ifndef FFNDOWN_UPDTH_H_INCLUDED
#define FFNDOWN_UPDTH_H_INCLUDED
#endif // FFNDOWN_UPDTH_H_INCLUDED
//---------------------------------------
#ifndef FFNDOWN_PARTS_H_INCLUDED
  #include "../data/ffndown_parts.h"
#endif // FFNDOWN_PARTS_H_INCLUDED
#ifndef FFNDOWN_DATA_H_INCLUDED
  #include "../data/ffndown_data.h"
#endif // FFNDOWN_DATA_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef MULTI_THREADS_H_INCLUDED
  #include "../../connect/multi_threads.h"
#endif // MULTI_THREADS_H_INCLUDED
#ifndef HTMLPARSE2_H_INCLUDED
  #include "../../core/utils/htmlparse2.h"
#endif // HTMLPARSE2_H_INCLUDED

#include <vector>
//***************************************************************************
// fic extracts
class jfFicExtractArray : public std::vector<jfFicExtract*> {
  public:
    // returns true if the id is within an extract in the array
    bool IdIncluded(const size_t& idtocheck,jf_FICTYPE idcheck) const;
    // returns a pointer to an extract by id
    jfFicExtract* GetByID(const size_t& idtocheck,jf_FICTYPE idcheck) const;
    // handles file deletion
    size_t DeleteSourceFiles(size_t inindex);
    bool DeleteOnlySource(size_t inindex);
    bool ClearSourceFiles(size_t inindex);
    // deletes all the contained fics and empties the array
    void ClearContents();
    // returns the fictype
    jf_FICTYPE GetFtype(size_t index) const;
};
//=============================================================
class jfUpdateFicThread : public jfMultiDownloadThread {
  public:
    // constructor
    jfUpdateFicThread();
    // public methods
    void SetFicData(jfFicDownParams* in_ficparams);
    // the main method
    virtual wxThread::ExitCode Entry();
    void TypeSetup();
  protected:
    // additional subsections
    size_t GetFileList();
    // virtual plug-in methods to handle some vital bits
    virtual void PreFetch1();
    virtual void PreFetch2();
    virtual void SetUrl();
    virtual bool ProcessPage();
    virtual size_t NewPagecount() const;
    virtual void PostPageAction();
    // special methods (pre-fic download)
    size_t LoadHtmlFilelist();
    bool CheckHtmlFile();
    bool CheckAllFiles();
    jfFicExtract* MakeExtract(jf_FICTYPE& outtype, wxString ficcore);
    // page downloader methods
    bool HandleFanfic(size_t ficindex);
    bool ProcessAndWrite(size_t ficindex);
    bool ProcessingFanfics();
    // additional methods
    bool IdentifyMissing(const wxString* indata) const;
    bool UpdateCheck(size_t ficindex_in) const;
    bool AO3_RCheck();
    // messages
    void SendNoFiles();
    void SendUpdateR(size_t update_res);
    // internal data
    jfFicDownParams* ficparams;
    wxArrayString* filelist;
    size_t filecount,ficcount,findex;
    wxArrayString* subdirs;
    jfFicExtractArray* ficlist;
    jfFanfic* xqval;
    bool missing_signal;
    bool indexpage;
    bool ficupdated;

};
//***************************************************************************
