/*****************************************************************************
Name    : googlecoll.cpp
Basic   : Defines the collection class for google items
Author  : John Q Metro
Started : September 30, 2009
Updated : August 22, 2012

******************************************************************************/
#ifndef GOOGLECOLL_H_INCLUDED
  #include "googlecoll.h"
#endif // GOOGLECOLL_H_INCLUDED
#ifndef GOOGLEOBJ_H_INCLUDED
  #include "googleobj.h"
#endif // GOOGLEOBJ_H_INCLUDED
#ifndef UTILS3_H_INCLUDED
  #include "../../core/utils/utils3.h"
#endif // UTILS3_H_INCLUDED
//------------------------------------------------------------------------------

/*****************************************************************************/
jfGoogleCollection::jfGoogleCollection(jfGoogleSearch* sdata_in) {
  assert(sdata_in!=NULL);
  sdata_typed = sdata_in;
  sdata = sdata_in;
  RegisterCategory();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfGoogleCollection::GetTypeID() const {
  return wxT("GoogleCollection");
}
//-------------------------------------------------------------------------
bool jfGoogleCollection::AddFirstPage(const wxString& inpage, size_t& outcount) {
  int rval;
  pageindex = 0;
  outcount = 0;
  rval = ProcessPage(inpage);
  if (rval<0) {
    UndoPage();
    return false;
  }
  outcount = rval;
  return true;
}
//------------------------------------------------------------------------------
bool jfGoogleCollection::AddPage(const wxString& inpage, size_t& outcount) {
  int rval;
  outcount = 0;
  rval = ProcessPage(inpage);
  if (rval<0) {
    UndoPage();
    return false;
  }
  outcount = rval;
  return true;
}
//---------------------------------------------------------------------
bool jfGoogleCollection::LoadValues(jfSkeletonParser* inparser,size_t which) const {
  LoadCoreValues(inparser,which);
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfGoogleCollection::PageAndItemCount(jfStringParser* inparser, size_t& itemcount) {
  // constants
  const wxString fname = wxT("jfGoogleCollection::PageAndItemCount");
  // local variables
  wxString buffer,zbuffer;
  unsigned long scres;
  size_t leftover,pagemax;
  // we start
  SaveOld();
  // skipping past the start
  /**/jfXLogMessage(fname,1);
  if (!inparser->MovePast(wxT("<div id=\"resultStats\">")) ) return false;
  zbuffer = inparser->GetBlock(2000);
  // first page has no page marker
  if (inparser->StartsWithAtIndex(wxT("About"))) {
    // getting the number of items
    /**/jfXLogMessage(fname,2);
    if (!inparser->GetDelimited(wxT("About"),wxT("results</div>"),buffer )) return false;
  }
  // other pages do
  else {
    // multiple pages
    if (inparser->StartsWithAtIndex(wxT("Page"))) {
      // getting the page index
      /**/jfXLogMessage(fname,3);
      if (!inparser->GetDelimitedEndPair(wxT("Page"),wxT("of about"),wxT("of"),buffer )) return false;
      /**/jfXLogMessage(fname,4,buffer);
      Dtrim(buffer);
      /**/jfXLogMessageS(fname,5,pageindex);
      if (!Str2ULongC(buffer,scres)) return false;
      if (scres!=pageindex) /**/jfXLogMessage(fname,5,zbuffer);
      assert(scres==pageindex);
    }
    // getting the number of items
    /**/jfXLogMessage(fname,6);
    if (!inparser->GetMovePast(wxT("results</div>"),buffer )) return false;
  }
  /**/jfXLogMessage(fname,7,buffer);
  Dtrim(buffer);
  if (!Str2ULongC(buffer,scres)) return false;
  // finishing assignments and calculations
  pagemax = scres / 100;
  leftover = scres % 100;
  /**/jfXLogMessage(fname,8);
    // the current page is the maximum page
  if ((scres)<=(pageindex*100)) {
    pagecount[0] = pageindex;
    itemcount = scres - (pageindex-1)*100;
    return true;
  }
  // finishing
  pagecount[0] = (leftover==0)?pagemax:(pagemax+1);
  itemcount = 100;
  return true;
}
//-----------------------------------------------------------------------
size_t jfGoogleCollection::ProcessPage(const wxString& inpage) {
  const wxString rerror = wxT("jfGoogleCollection::ProcessPage");
  // variables
  jfStringParser* xparser;
  size_t newcount,pindex;
  size_t pit_index;
  size_t q_result;
  size_t imgcount;
  size_t gcount;
  jfGoogleItem* temp;
  wxString buffer,zbuffer;
  //' starting...
  xparser = new jfStringParser(inpage);
  pageindex++;
  if (!PageAndItemCount(xparser,newcount)) {
    pindex = xparser->GetIndex();
    delete xparser;
    xparser = NULL;
    /**/jfXLogMessage(rerror,1);
    //throw new jfParseError(rerror,pindex,wxT("Unable to extract the item count!"));
    return 0;
  }
  page_results = new jfPDVector();
  gcount = pr_size = 0;
  pit_index = (pageindex-1)*100-1;
  // next up is parsing the items
  while (xparser->GetDelimitedEndPair(wxT("<h3 class=\"r\">"),wxT("<li class=\"g\""),wxT("</ol>"),buffer)) {
    pit_index++;
    gcount++;
    if (!ContainsAlt(buffer,wxT("<img "),wxT("<a href=\"/images"))) {
      temp = new jfGoogleItem(buffer,pit_index);
      if (temp->IsValid()) {
        page_results->push_back(temp);
        pr_size++;
      }
      else {
        zbuffer = temp->GetParseError();
        /**/jfXLogMessage(rerror,2,zbuffer);
        /**/jfXLogMessage(rerror,3,buffer);
        delete temp;
      }
      grand_total[0]++;
    }
    else imgcount++;
  }
  assert(pr_size!=0);
  // post page check
  /**/jfXLogMessageS(rerror,4,gcount);
  if (gcount<100) {
    // you can't trust google to display the proper count even on the last page
    pagecount[0] = pageindex;
  }
  // done with the items
  xparser->GetToEnd(zbuffer);
  newpage_items = pr_size;
  /**/jfXLogMessageS(rerror,9,newcount);
  FilterByList();
  CategorySort();
  Condense();
  DispatchPageResults();
  q_result = AppendPageResults();
  delete xparser;
  xparser = NULL;
  return q_result;

}
//----------------------------------------------------------------------------
// some more virtual i/o methods
bool jfGoogleCollection::AddDelta(wxTextFile* outfile) const {
  return true;
}
//---------------------------------------------------------------------------
bool jfGoogleCollection::ReadDelta(jfFileReader* infile) {
  return true;
}
/*****************************************************************************/
