/*****************************************************************************
Name    : googleobj.h
Basic   : Declares a class for holding a google item
Author  : John Q Metro
Started : September 9, 2009
Updated : January 3, 2015
Notes   :

******************************************************************************/
#ifndef GOOGLEOBJ_H_INCLUDED
  #include "googleobj.h"
#endif // GOOGLEOBJ_H_INCLUDED
#ifndef HTMLPARSE_H_INCLUDED
  #include "../../core/utils/htmlparse.h"
#endif // HTMLPARSE_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED

#include <wx/url.h>
/*****************************************************************************/
// constructors
//----------------------------------------------------------------------------
// the base constructor
jfGoogleItem::jfGoogleItem():jfBasePD() {
  gtype = jfGIT_DEFAULT;

}
//----------------------------------------------------------------------------
// the usual constructor
jfGoogleItem::jfGoogleItem(const wxString& insrc, size_t setrank):jfBasePD() {
  SetFromString(insrc,setrank);
}
//----------------------------------------------------------------------------
// copy constructor
jfGoogleItem::jfGoogleItem(const jfGoogleItem& src):jfBasePD() {
  src.CopyOver(this);
  gtype = src.gtype;
  rawhtml = src.rawhtml;
  cached = src.cached;
  similar = src.similar;
  xparser = NULL;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented public virtual methods
wxString jfGoogleItem::GetTypeID() const {
  return wxT("GoogleItem");
}
//-----------------------------------------------------------------------------
// stories the data into the skeleton parser for i/o purposes
bool jfGoogleItem::LoadValues(jfSkeletonParser* inparser) const {
  LoadCoreValues(inparser);
  inparser->AddText(wxT("ITEM_RAWHTML"),rawhtml);
  inparser->AddText(wxT("ITEMG_CACHED"),cached);
  inparser->AddText(wxT("ITEMG_SIMILAR"),similar);
  return true;
}
//-----------------------------------------------------------------------------
void jfGoogleItem::ProcessDescription() {

}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// to string functions
//----------------------------------------------------------------------------
wxString jfGoogleItem::ToText() const {
  wxString result,buffer;
  // title and link
  result = name + wxT('\n');
  result += primarylink + wxT('\n');
  // adding the main description
  buffer = htmlparse::HTML2Text(description);
  result += buffer + wxT('\n');
  // done
  return result;
}
//----------------------------------------------------------------------------
wxString jfGoogleItem::ToHTML() const {
  wxString result;
  // building the title line
  result = wxT("<div class=\"itemwrapper\"><p>\n");
  result += wxT("<h3 class=\"itemtitle\"><a href=\"");
  result += primarylink + wxT("\">");
  result += name + wxT("</a></h3><br>\n");
  // next line: link display
  result += wxT("<span class=\"linkdisplay\">");
  result += primarylink + wxT("</span><br>\n");
  // adding the main description
  result += wxT("<span class=\"itemdescription\">");
  result += description + wxT("</span>\n");
  result += wxT("</p></div>\n");
  // done
  return result;
}
//----------------------------------------------------------------------------
wxString jfGoogleItem::ToDisplayHTML() const {
  wxString result;
  // building the title line
  result += wxT("<font size=+2><a href=\"");
  result += primarylink + wxT("\">");
  result += name + wxT("</a></font><br>\n");
  // next line: link display
  result += wxT("<font color=\"green\">");
  result += primarylink + wxT("</font><br>\n");
  // adding the main description
  result += description;
  // done
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// additional methods
bool jfGoogleItem::SetFromString(wxString indata, size_t setrank) {
  // constants
  func_string = wxT("jfGoogleItem::SetFromString");
  // variables
  wxString buffer;
  // we start with setting the validdata to false, and rank
  rank = setrank;
  validdata = false;
  gtype = jfGIT_DEFAULT;
  // we start the parsing
  SetParserString(indata);
  // url extraction
  if (!ExtractUrl(primarylink, perr_string)) return false;
  // after the url, we go for the title
  if (!xparser->GetDelimited(wxT(">"),wxT("</a>"),buffer)) {
    perr_string = wxT("Title not found!");
		/**/jfXLogMessage(func_string,3);
    return false;
  }
	name = htmlparse::HTML2Text(buffer);
	if (gtype!=jfGIT_BOOK) {
    // next up is the main contents
    if (!(xparser->MovePast(wxT("<div class=\"s\">")))) {
      if (!(xparser->MovePast(wxT("&#9658;")))) {
        perr_string = wxT("End of title part not found!");
        /**/jfXLogMessage(func_string,5);
        return false;
      }
      gtype = jfGIT_VIDEO;
    }
	}
  // skipping past the cite
  if (!xparser->GetDelimited(wxT("<cite"),wxT("</cite>"),buffer)) {
    perr_string = wxT("Link display is missing!");
    /**/jfXLogMessage(func_string,6);
    return false;
  }
  // default, cached & stuff
  if (gtype==jfGIT_DEFAULT) {
    if (!xparser->MovePast(wxT("<span class=\"flc\">"))) {
      perr_string = wxT("Extra links part is missing!");
      /**/jfXLogMessage(func_string,7);
      return false;
    }
    // cached and similar
    if (xparser->GetDelimited(wxT("<a href=\"/url?q=http://webcache."),wxT("\">Cached</a>"),buffer)) {
      cached = wxT("http://webcache.") + buffer;
    }
    if (xparser->GetDelimited(wxT("<a href=\"/search?"),wxT("\">Similar</a>"),buffer)) {
      similar = wxT("http://www.google.com/search?") + buffer;
    }
    if (!(xparser->GetDelimited(wxT("<span class=\"st\">"),wxT("</span><br>"),buffer))) {
      perr_string = wxT("Description not found!");
      /**/jfXLogMessage(func_string,8);
      return false;
    }
  }
  // book links are comparativly simple
  else if (gtype==jfGIT_BOOK) {
    if (!(xparser->GetDelimited(wxT("<div class=\"s\">"),wxT("</div>"),buffer))) {
      perr_string = wxT("Description not found!");
      /**/jfXLogMessage(func_string,9);
      return false;
    }
  }
  // video links are parsed differently
  else {
    if (!(xparser->GetDelimited(wxT("<span class=\"st\">"),wxT("</span></td>"),buffer))) {
      perr_string = wxT("Description not found!");
      /**/jfXLogMessage(func_string,10);
      return false;
    }
  }
  description = htmlparse::ToUTFHtml(buffer);
  /**/jfXLogMessage(func_string,11,description);
  // done
  validdata = true;
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the destructor
jfGoogleItem::~jfGoogleItem() {
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented private virtual methods
//----------------------------------------------------------------------------
bool jfGoogleItem::AddRestToFile(wxTextFile* outfile) const {
  // local variables
  wxString result;
  jfOutString xresult;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // preparing line 4, the raw data
  result =  EscapeNL(rawhtml,IO_DELIMS);
  outfile->AddLine(result);
  // line 5 is the cached/similar links
  xresult << cached << similar;
  outfile->AddLine(xresult);
  // done
  return true;
}
//----------------------------------------------------------------------------
bool jfGoogleItem::ReadRestFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfGoogleItem::ReadRestFromFile");
  // starting with line 4
  assert(infile!=NULL);
  if (!infile->ReadUnEsc(rawhtml,funcname)) return false;
  // line 5
  if (!infile->ReadParseLine(2,funcname)) return false;
  cached = infile->lp.UnEscStr(0);
  similar = infile->lp.UnEscStr(1);
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// extracts the url from the source data
bool jfGoogleItem::ExtractUrl(wxString& olink, wxString& oerror) {
  // local variables
  wxURL tester;
  wxString buffer;
  // extracting the url
  if (!(xparser->GetDelimited(wxT("<a href=\"/url?q="),wxT("&amp;"),buffer))) {
    if (!xparser->GetDelimited(wxT("<a href=\"http://books.google.com/books"),wxT("&amp;"),buffer)) {
      oerror = wxT("Cannot find start of URL!");
      return false;
    }
    gtype = jfGIT_BOOK;
    buffer = wxT("http://books.google.com/books") + buffer;
  }
  tester.SetURL(buffer);
  if (tester.GetError()!=wxURL_NOERR) {
    // wxURL cannot handle some valid protocols like https, so we ignore those
    if (tester.GetError()!=wxURL_NOPROTO) {
        oerror = wxT("URL is malformed!");
        return false;
    }
  }
  olink = buffer;
  return true;
}
/*****************************************************************************/
