/*****************************************************************************
Name    :   googlesearch.cpp
Author  :   John Q Metro
Purpose :   Defines the Google class for search info and parameters
Created :   September 27, 2009
Updated :   August 22, 2012
******************************************************************************/
#ifndef GOOGLESEARCH_H_INCLUDED
  #include "googlesearch.h"
#endif // GOOGLESEARCH_H_INCLUDED
#ifndef HTMLPARSE_H_INCLUDED
  #include "../../core/utils/htmlparse.h"
#endif // HTMLPARSE_H_INCLUDED
#ifndef GOOGLECOLL_H_INCLUDED
  #include "googlecoll.h"
#endif // GOOGLECOLL_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef HTML_DISPLAY_H_INCLUDED
  #include "../../interface/display/html_display.h"
#endif // HTML_DISPLAY_H_INCLUDED
//--------------------------------------------------------------------------
/*****************************************************************************/
// constructors
jfGoogleSearch::jfGoogleSearch():jfSearchCore() {
  pagelimit = 20;
}
//-----------------------------------------------------------------------------------
jfGoogleSearch::jfGoogleSearch(const wxString& inincl,const wxString& inexcl,const wxString& inalt) {
  SetSearchStrings(inincl,inexcl,inalt);
  pagelimit = 20;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting data
void jfGoogleSearch::SetSearchStrings(const wxString& inincl,const wxString& inexcl,const wxString& inalt) {
  includes = inincl;
  excludes = inexcl;
  alternates = inalt;
  Dtrim(includes);
  Dtrim(excludes);
  Dtrim(alternates);
}
//------------------------------------------------------------------------------
void jfGoogleSearch::SetLang(wxString in_langtag) {
  lang_tag = in_langtag;
}
//------------------------------------------------------------------------------
void jfGoogleSearch::SetSitesearch(wxString in_sitesearch) {
  sitesearch = in_sitesearch;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented method for sending category data
void jfGoogleSearch::DispatchCategory() {
  // variables
  wxCommandEvent outevent(jf_RESLISTBOX_NEWCAT_EVENT,jfID_SRCCAT_PARSER);
  wxString* qval;
  // building the event to send
  qval = new wxString;
  (*qval) = GetCatHeader();
  outevent.SetString(GetCatName());
  outevent.SetClientData(qval);
  // sending the event
  wxPostEvent(cat_results,outevent);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfGoogleSearch::GetTypeID() const {
  return wxT("GoogleSearch");
}
//------------------------------------------------------------------------------
wxString jfGoogleSearch::GetCatName() const {
  return wxT("Google");
}
//--------------------------------------------------------------------------
wxString jfGoogleSearch::GetCatHeader() const {
  return wxEmptyString;
}
//-----------------------------------------------------------------------------
// returns the url to fetch google pages
wxString jfGoogleSearch::GetIndexUrl(size_t inp_index) const {
  wxString Result = wxT("http://www.google.com/search?hl=en");
  wxString buffer;
  size_t startindex;
  // we start adding the search items
  Result += wxT("&as_q=") + htmlparse::GoogleEscape(includes);
  Result += wxT("&as_epq=");
  Result += wxT("&as_oq=") + htmlparse::GoogleEscape(alternates);
  Result += wxT("&as_eq=") + htmlparse::GoogleEscape(excludes);
  // controllers
  Result += wxT("&num=100&lr=") + lang_tag;
  Result += wxT("&as_filetype=&ft=i&as_sitesearch=") + htmlparse::GoogleEscape(sitesearch);
  Result += wxT("&as_qdr=all");
  Result += wxT("&as_rights=&as_occt=any&cr=&as_nlo=&as_nhi=&safe=images");
  // getting the index
  startindex = (inp_index-1)*100;
  if (startindex!=0) {
    buffer = SInt2Str(startindex);
    Dtrim(buffer);
    Result += wxT("&start=") + buffer;
  }
  // done
  return Result;
}
//------------------------------------------------------------------
wxString jfGoogleSearch::GetHTMLHeader(size_t result_category) const {
  wxString result;
  result = wxT("<h1 class=\"maintittle\">Google Search Results</h1>\n");
  result += wxT("<h2 class=\"res_categ_header\">Result Category : ");
  if (result_category == 32) result += categories->GetName();
  else result += categories->NameAtIndex(result_category);
  result += wxT("</h2>");
  return result;
}
//-----------------------------------------------------------------------
void jfGoogleSearch::MakeLocalSkeleton(jfSkeletonBase* src) {
  jfSkeletonSingle* temp, *newval;
  // asserts and checks
  assert(src!=NULL);
  assert(src->GetSkelType()==jfs_SINGLE);
  // we start
  temp = dynamic_cast<jfSkeletonSingle*>(src);
  newval = new jfSkeletonSingle(temp);
  // clearing
  if (local_skel) ClearLocalSkeleton();
  skel_link = newval;
  local_skel = true;
}
//-------------------------------------------------------------------
void jfGoogleSearch::LoadValues(jfSkeletonParser* inparser, size_t result_categ) {
  LoadCoreValues(inparser,result_categ);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// returning string data
//-------------------------------------------------------------------
wxString jfGoogleSearch::GetIncludes() const {
  return includes;
}
//-------------------------------------------------------------------
wxString jfGoogleSearch::GetAlternates() const {
  return alternates;
}
//-------------------------------------------------------------------
wxString jfGoogleSearch::GetExcludes() const {
  return excludes;
}
//-------------------------------------------------------------------
wxString jfGoogleSearch::GetLangTag() const {
  return lang_tag;
}
//-------------------------------------------------------------------
wxString jfGoogleSearch::GetSitesearch() const {
  return sitesearch;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// file i/o
//-----------------------------------------------------------------------------
bool jfGoogleSearch::AddMiddleToFile(wxTextFile* outfile) const {
  // local variables
  wxString result;
  // checking and special conditions
  if (outfile==NULL) return false;
  if (!(outfile->IsOpened())) return false;
  // preparing line 3, the includes (and possibly quoted string) data
  result =  EscapeNL(includes,IO_DELIMS);
  outfile->AddLine(result);
  // line 4 is for alternates and excludes
  result =  EscapeNL(alternates,IO_DELIMS);
  result += wxT(';');
  result +=  EscapeNL(excludes,IO_DELIMS);
  outfile->AddLine(result);
  // line 5 is for lang_tags and site searches
  result =  EscapeNL(lang_tag,IO_DELIMS);
  result += wxT(';');
  result +=  EscapeNL(sitesearch,IO_DELIMS);
  outfile->AddLine(result);
  // done
  return true;
}
//-----------------------------------------------------------------------------
bool jfGoogleSearch::ReadMiddleFromFile(jfFileReader* infile) {
  const wxString funcname = wxT("jfGoogleSearch::ReadMiddleFromFile");
  // local variables
  wxString cline, buffer;
  // starting with line 3
  assert(infile!=NULL);
  if (!infile->ReadUnEsc(includes,funcname)) return false;
  // line 4
  if (!infile->ReadParseLine(2,funcname)) return false;
  alternates = infile->lp.UnEscStr(0);
  excludes = infile->lp.UnEscStr(1);
  // line 5
  if (!infile->ReadParseLine(2,funcname)) return false;
  lang_tag = infile->lp.UnEscStr(0);
  sitesearch = infile->lp.UnEscStr(1);
  return true;
}
//****************************************************************************
jfGoogleResColl::jfGoogleResColl(jfGoogleSearch* xsearch_in):jfResultCollection(xsearch_in, false) {

}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfGoogleResColl::CreateItemCollection(const wxString& src_in, wxString& outerr) {
  jfGoogleCollection* temp;
  assert(itemcount==0);
  size_t rrval;
  // we start...
  temp = new jfGoogleCollection(dynamic_cast<jfGoogleSearch*>(search));
  if (!temp->AddFirstPage(src_in,rrval)) {
    outerr = temp->parse_error;
    delete temp;
    return false;
  }
  cindex = 0;
  return AddItem(temp);
}
//-----------------------------------------------------------------------
// adds an extra page
bool jfGoogleResColl::AddPage(const wxString& src_in, wxString& outerr) {
  jfGoogleCollection* temp;
  size_t rrval;
  outerr = wxT("CheckIndex0");
  if (!CheckIndex(0)) return false;
  temp = dynamic_cast<jfGoogleCollection*>((*collections)[0]);
  if (!temp->AddPage(src_in,rrval)) {
    outerr = temp->parse_error;
    return false;
  }
  else return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfGoogleResColl::GetTypeID() const {
  return wxT("GoogleResColl");
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfGoogleResColl::WriteToHTML(size_t result_category) {
  // local variables
  jfHtmlParams *output;
  jfGoogleCollection* temp;
  // checks
  assert(result_category<33);
  // starting
  output = OpenHTMLOutfile(result_category);
  // we prepare the skeletop
  output->base = search->GetLocalSkeleton();
  if ((output->base)==NULL) {
    output->base = mainskelc->AtIndexS(0);
  }
  // we load up with values first
  search->LoadValues(output->parse,output->GetResCat());
  // header
  WriteHtmlHeader(output);
  // body
  temp = dynamic_cast<jfGoogleCollection*>((*collections)[0]);
  temp->WriteToHTML(output,true);
  // finishing off
  WriteFooterClose(output);
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfGoogleResColl::WriteHtmlHeader(jfHtmlParams* indata) {
  // local variables
  wxString buffer;
  // the usual checks
  assert(indata!=NULL);
  assert(indata->CheckOk());
  // loading values
  // constatnts
  buffer = indata->MakeResult(indata->base->header_skel);
  indata->outfile->AddLine(buffer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented private i/o methods
//--------------------------------------
bool jfGoogleResColl::AddRestToFile(wxTextFile* outfile) const {
  // does nothing
  return true;
}
//--------------------------------------
jfUrlItemCollection* jfGoogleResColl::MakeEmptyCollection() const {
  jfGoogleCollection* result;
  result = new jfGoogleCollection(dynamic_cast<jfGoogleSearch*>(search));
  return result;
}
//--------------------------------------
bool jfGoogleResColl::ReadRestFromFile(jfFileReader* infile) {
  // does nothing
  return true;
}
/*****************************************************************************/
