/*****************************************************************************
Name    :   googlesearch.h
Author  :   John Q Metro
Purpose :   Declares the Google class for search info and parameters
Created :   September 27, 2009
Updated :   August 22, 2012
******************************************************************************/
#ifndef GOOGLESEARCH_H_INCLUDED
#define GOOGLESEARCH_H_INCLUDED
#endif // GOOGLESEARCH_H_INCLUDED
//--------------------------------------------------------------------------
#ifndef SEARCHCORE_H_INCLUDED
  #include "../../core/objs/searchcore.h"
#endif // SEARCHCORE_H_INCLUDED
#ifndef RESULTCOLLECT_H_INCLUDED
  #include "../../core/structs/resultcollect.h"
#endif // RESULTCOLLECT_H_INCLUDED
#ifndef SKELCOLL_H_INCLUDED
  #include "../../core/structs/skelcoll.h"
#endif // SKELCOLL_H_INCLUDED
/*****************************************************************************/
class jfGoogleSearch : public jfSearchCore {
  public:
    // constructors
    jfGoogleSearch();
    jfGoogleSearch(const wxString& inincl,const wxString& inexcl,const wxString& inalt);
    // setting data
    void SetSearchStrings(const wxString& inincl,const wxString& inexcl,const wxString& inalt);
    void SetLang(wxString in_langtag);
    void SetSitesearch(wxString in_sitesearch);
    // implemented method for sending category data
    virtual void DispatchCategory();
    // implemeted virtual methods
    virtual wxString GetTypeID() const;
    virtual wxString GetCatName() const;
    virtual wxString GetCatHeader() const;
    virtual wxString GetIndexUrl(size_t inp_index) const;
    virtual wxString GetHTMLHeader(size_t result_category) const;
    virtual void MakeLocalSkeleton(jfSkeletonBase* src);
    void LoadValues(jfSkeletonParser* inparser, size_t result_categ);
    // returning string data
    wxString GetIncludes() const;
    wxString GetAlternates() const;
    wxString GetExcludes() const;
    wxString GetLangTag() const;
    wxString GetSitesearch() const;
  private:
    // i/o methods for google specific data
    virtual bool AddMiddleToFile(wxTextFile* outfile) const;
    virtual bool ReadMiddleFromFile(jfFileReader* infile);
    // search data used for building an url
    wxString includes, excludes, alternates;
    wxString lang_tag, sitesearch;
};
//-----------------------------------------------------------------------------
class jfGoogleResColl : public jfResultCollection {
  public:
    // the constructor
    jfGoogleResColl(jfGoogleSearch* xsearch_in);
    // making items
    bool CreateItemCollection(const wxString& src_in, wxString& outerr);
    virtual bool AddPage(const wxString& src_in, wxString& outerr);
    // implementede virtual methods
    virtual wxString GetTypeID() const;
    // i/o
    bool WriteToHTML(size_t result_category);
  protected:
    void WriteHtmlHeader(jfHtmlParams* indata);
    // implemented private i/o methods
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual jfUrlItemCollection* MakeEmptyCollection() const;
    virtual bool ReadRestFromFile(jfFileReader* infile);

};
/*****************************************************************************/
