/******************************************************************************
Name    :   g_mainpanel1.cpp
Author  :   John Q Metro
Purpose :   Defines the main panels for google searching
Created :   October 11, 2009
Updated :   January 7, 2013
******************************************************************************/
#ifndef G_MAINPANEL1_H_INCLUDED
  #include "g_mainpanel1.h"
#endif // G_MAINPANEL1_H_INCLUDED
#ifndef GOOGLESEARCH_H_INCLUDED
  #include "../data/googlesearch.h"
#endif // GOOGLESEARCH_H_INCLUDED
#ifndef GOOGLETHREADS_H_INCLUDED
  #include "../threads/googlethreads.h"
#endif // GOOGLETHREADS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef INITEND_H_INCLUDED
  #include "../../initend.h"
#endif // INITEND_H_INCLUDED
#ifndef JFILTERGLOBALS
  #include "../../core/filters/base/filter_globals.h"
#endif
/*****************************************************************************/
// the constructor
jfGoogleSearchGrp::jfGoogleSearchGrp(wxWindow* parent, wxWindowID id, jfGoogleSearch*
        searchin, const wxString& name):jfSearchGrouping(parent,id,filterglobals::fclist[2],name) {
  // constants
	const wxChar* fname=wxT("jfGoogleSearchGrp::jfGoogleSearchGrp");
	// starting with creation of the search object
	jfXLogMessage(fname,1);
  if (searchin==NULL) {
		jfXLogMessage(fname,2);
    searchdata = MakeTypedSearch();
		jfXLogMessage(fname,3);
    typed_search = dynamic_cast<jfGoogleSearch*>(searchdata);
  }
  else {
		jfXLogMessage(fname,4);
    searchdata = searchin;
    typed_search = searchin;
		jfXLogMessage(fname,5);
  }
	jfXLogMessage(fname,6);
  // making panel 1
  panel1 = MakeFirstPanel(-1);
	jfXLogMessage(fname,7);
  custinit = true;
  // finishing off
  FinishConstruction();
	jfXLogMessage(fname,8);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfSearchPanelBase*  jfGoogleSearchGrp::MakeFirstPanel(wxWindowID id_in){
  return new jfGooglePanel1(panel_holder,typed_search,id_in);
}
//----------------------------------------------------------------------------
jfSearchCore*       jfGoogleSearchGrp::MakeTypedSearch() const {
  jfGoogleSearch* temp;
  temp = new jfGoogleSearch();
  temp->MakeEmpty();
  temp->categories->SetUseDefault(true);
  temp->categories->base_outputdir = (*globalsettings::globaldirs)[0];
  return temp;
}
//----------------------------------------------------------------------------
jfResultCollection* jfGoogleSearchGrp::MakeTypedCollection() const {
  assert(custinit);
  return new jfGoogleResColl(typed_search);
}
//----------------------------------------------------------------------------
jfSingleSearchThreadBase* jfGoogleSearchGrp::MakeTypedThread() const {
  return new jfGoogleThread();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfGoogleSearchGrp::MoreResized(size_t x, size_t y) {}
//===========================================================================
// the constructor
jfGoogleOnePanelSearch::jfGoogleOnePanelSearch(wxWindow* parent, wxWindowID id,
      jfGoogleSearch* searchin, const wxString& name):jfOnePanelSearch(parent,
      id,name) {
  // constants
	const wxString fname = wxT("jfGoogleOnePanelSearch::jfGoogleOnePanelSearch");
	// starting with creation of the search object
  if (searchin==NULL) {
    searchdata = MakeTypedSearch();
    typed_search = dynamic_cast<jfGoogleSearch*>(searchdata);
  }
  else {
    searchdata = searchin;
    typed_search = searchin;
  }
  // the panels
  panel1 = MakeFirstPanel(-1);
  custinit = true;
  // finishing off
  FinishConstruction();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// custom  methods
//----------------------------------
jfSearchPanelBase*  jfGoogleOnePanelSearch::MakeFirstPanel(wxWindowID id_in) {
  return new jfGooglePanel1(this,typed_search,id_in);
}
//----------------------------------
jfSearchCore*       jfGoogleOnePanelSearch::MakeTypedSearch() const {
  jfGoogleSearch* temp;
  temp = new jfGoogleSearch();
  temp->MakeEmpty();
  temp->categories->SetUseDefault(true);
  temp->categories->base_outputdir = (*globalsettings::globaldirs)[0];
  return temp;
}
//----------------------------------
jfResultCollection* jfGoogleOnePanelSearch::MakeTypedCollection() const {
  assert(custinit);
  return new jfGoogleResColl(typed_search);
}
//----------------------------------
jfSingleSearchThreadBase* jfGoogleOnePanelSearch::MakeTypedThread() const {
  return new jfGoogleThread();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfGoogleOnePanelSearch::MoreResized(size_t x, size_t y) {

}
/*****************************************************************************/
jfGoogleMain::jfGoogleMain(wxWindow* parent, wxWindowID id, jfSearchCore* searchin,
        bool notebook):jfMainSearchGroup(parent,id,searchin,rtype_BASIC, notebook,false) {
  // the usual constant
	const wxChar* fname = wxT("jfGoogleMain::jfGoogleMain");
  // preparing notebook and results display
  if (noteb) {
    container = new wxNotebook(this,-1);
    r_panel = MakeResultGroup(container,-1);
  }
  else {
    container = NULL;
    r_panel = MakeResultGroup(this,-1);
  }
	// making the custom panel
	jfXLogMessage(fname,1);
  p_search = MakeSearchGroup(-1,searchin,threepanel);
  // the middle panel
  ConsStepTwo(filterglobals::fclist[2]);
	jfXLogMessage(fname,2);
  TopArranger();
	jfXLogMessage(fname,3);
  rc_menu = new jfGooxPopupMenu(this);
	jfXLogMessage(fname,4);
  r_panel->SetPopupMenu(rc_menu);
	jfXLogMessage(fname,5);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handles the append text right click event
void jfGoogleMain::OnRMenuAppendUrl(wxCommandEvent& event) {
  // constants
  const wxString req_msg = wxT("Edit the url to what you want.");
  const wxString req_cap = wxT("Add to the URL exclude list");
  // local variables
  wxString utext, restext;
  wxPoint* index_coord;
  jfUrlFilter* urlptr;
  wxString fil_string;
  // we start by getting coordinates for the item
  index_coord = reinterpret_cast<wxPoint*>(event.GetClientData());
  // getting the text
  utext = results->GetItemUrl(index_coord->x,index_coord->y);
  delete index_coord;
  // we now pop up a dialog that lets the user edit the text before adding it
  restext = wxGetTextFromUser(req_msg,req_cap,utext);
  // if the result is not empty, we now work to add it to the filter
  if (!restext.IsEmpty()) {
    // we get a pointer to the extression filter
    urlptr = dynamic_cast<jfUrlFilter*>(search_data->def_filtermap->GetItem(wxT("02")));
    // appending
    urlptr->AppendToExcludes(restext);
    p_search->ReloadPanel1();
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual method
//-------------------------------------------
jfSearchGroupingCore* jfGoogleMain::MakeSearchGroup(wxWindowID xid, jfSearchCore* xnewsearch, bool threepanels) {
  // local variables
	const wxChar* fname = wxT("jfGoogleMain::MakeSearchGroup");
  jfSearchGroupingCore* result;
  jfGoogleSearch* temp;
  // starting
	jfXLogMessage(fname,1);
  if (xnewsearch!=NULL) temp = dynamic_cast<jfGoogleSearch*>(xnewsearch);
  else temp = NULL;
	jfXLogMessage(fname,2);
  // creating the panel
  if (threepanels) {
    if (noteb) result = new jfGoogleOnePanelSearch(container,xid,temp);
    else result = new jfGoogleOnePanelSearch(this,xid,temp);
  }
  else {
    if (noteb) result = new jfGoogleSearchGrp(container,xid,temp);
    else result = new jfGoogleSearchGrp(this,xid,temp);
  }
	jfXLogMessage(fname,3);
//  typed_mpanel = result;
  return result;
}
//---------------------------------------------------------------------------
jfResultDisplay* jfGoogleMain::MakeResultGroup(wxWindow* inparent,wxWindowID id) {
  return new jfResultDisplay(inparent,this,rtype_BASIC,id,he_ONECATEGORY);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfGoogleMain,jfMainSearchGroup)
  EVT_COMMAND(-1,jf_RMENU_EXLUDEURL_EVENT,jfGoogleMain::OnRMenuAppendUrl)
END_EVENT_TABLE()
/*****************************************************************************/
