/******************************************************************************
Name    :   g_panel1.cpp
Author  :   John Q Metro
Purpose :   Defines the custom first panel for google searching
Created :   October 11, 2009
Updated :   September 29, 2010
******************************************************************************/
#ifndef G_PANEL1_H_INCLUDED
  #include "g_panel1.h"
#endif // G_PANEL1_H_INCLUDED
#ifndef GOOGLESEARCH_H_INCLUDED
  #include "../data/googlesearch.h"
#endif // GOOGLESEARCH_H_INCLUDED
#ifndef GOOGLEINFO_H_INCLUDED
  #include "../data/googleinfo.h"
#endif // GOOGLEINFO_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//-------------------------------------------

/*****************************************************************************/
// the constructor
jfGoogleDFE::jfGoogleDFE(wxWindow* parent,wxWindowID id, jfSearchCore* inobj):
      jfDefaultFilterEditorBase(parent,id,inobj,1,1) {
  // gui elements
  g_urledit = new jfUrlFilEdit(this,-1);
  top_sizer->Add(g_urledit,1,wxALL|wxEXPAND,3);
  // loading data
  ChangeSearch(inobj);
  // finishing
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methds
bool jfGoogleDFE::SaveFiltersExtended() {
  // local variables
  jfUrlFilter *temp2;
  size_t oindex;
  // getting data
  temp2 = g_urledit->GetData();
  if (temp2==NULL) return false;
  temp2->SetName(goourlfilter);
  embedded_filters->ReplaceSame(temp2,oindex);
  return true;
}
//--------------------------------------------------------------------
bool jfGoogleDFE::ChangeSearchExtended(jfSearchCore* obj_data) {
  jfUrlFilter *ftemp2 = NULL;

  ftemp2 = dynamic_cast<jfUrlFilter*>(embedded_filters->GetItem(goourlfilter));
  if (ftemp2!=NULL) {
    if (!(g_urledit->SetData(ftemp2))) return false;
  }
  return true;
}
//===============================================================================
// the constructor
jfGooglePanel1::jfGooglePanel1(wxWindow* parent, jfGoogleSearch* insearch, wxWindowID id,
      const wxString& name):jfSearchPanelBase(parent,insearch,id,false,true,2,name) {
  // starting...
  MakeArrangeEntry();
  // sizer arranging
  filt_panel = MakeTypedFilEdit();
  defil_sizer->Add(filt_panel,1,wxALL|wxEXPAND,2);
  param_sizer->Add(stack,0,wxALL|wxEXPAND,3);
  topsizer->Add(param_sizer,0,wxEXPAND|wxALL,2);
  topsizer->Add(defil_sizer,1,wxEXPAND|wxALL,2);
  // finishing off
  SetSizer(topsizer);
  ChangeObj(mainobj);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// other external methods
bool jfGooglePanel1::LoadToObj() {
  const wxString fname = wxT("jfGooglePanel1::LoadToObj");
  // local variables
  jfGoogleSearch* gtemp;
  wxString omsg, ltag;
  int lselect;
  // starting
  if (mainobj==NULL) return false;
  filt_panel->SaveFilters();
  // setting the edit values
  gtemp = dynamic_cast<jfGoogleSearch*>(mainobj);
  gtemp->SetSearchStrings(includes_edit->GetValue(),excludes_edit->GetValue(),alternates_edit->GetValue());
  gtemp->SetSitesearch(sitesearch_edit->GetValue());
  /**/jfXLogMessage(fname,1);
  // getting the language picker selection
  lselect = langpicker->GetCurrentSelection();
  assert(lselect>=0);
  ltag = m_google_info->GetLangTag(lselect);
  gtemp->SetLang(ltag);
  /**/jfXLogMessage(fname,2);
  // done
  return true;
}
//----------------------------------------------------------------------------
bool jfGooglePanel1::ChangeObj(jfSearchCore* obj_data) {
  // local variables
  jfGoogleSearch* temp;
  wxString omsg,tltag;
  int lresult;
  // starting...
  temp = dynamic_cast<jfGoogleSearch*>(obj_data);
  mainobj = obj_data;
  // filters
  filt_panel->ChangeSearch(obj_data);
  // loading up the strings
  includes_edit->SetValue(temp->GetIncludes());
  alternates_edit->SetValue(temp->GetAlternates());
  excludes_edit->SetValue(temp->GetExcludes());
  sitesearch_edit->SetValue(temp->GetSitesearch());
  // loading the language tag selector
  tltag = temp->GetLangTag();
  lresult = m_google_info->GetIndexByTag(tltag);
  assert(lresult>=0);
  langpicker->SetSelection(lresult);
  // done
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfDefaultFilterEditorBase* jfGooglePanel1::MakeTypedFilEdit() {
  jfGoogleDFE* result;
  result = new jfGoogleDFE(this,-1,mainobj);
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfGooglePanel1::MakeArrangeEntry() {
  // the only local variable
  const wxArrayString* lchoices;
  // making the elements
  includes_label    = new wxStaticText(this,-1,wxT("Includes   :"));
  includes_edit     = new wxTextCtrl(this,-1);
  alternates_label  = new wxStaticText(this,-1,wxT("Alternates :"));
  alternates_edit   = new wxTextCtrl(this,-1);
  excludes_label    = new wxStaticText(this,-1,wxT("Excludes   :"));
  excludes_edit     = new wxTextCtrl(this,-1);
  // making the sizers
  row1 = new wxBoxSizer(wxHORIZONTAL);
  row2 = new wxBoxSizer(wxHORIZONTAL);
  row3 = new wxBoxSizer(wxHORIZONTAL);
  // placing the editors in the sizer
  row1->Add(includes_label,1,wxALL|wxALIGN_CENTER_VERTICAL,3);
  row1->Add(includes_edit,8,wxALL|wxEXPAND,3);
  row2->Add(alternates_label,1,wxALL|wxALIGN_CENTER_VERTICAL,3);
  row2->Add(alternates_edit,8,wxALL|wxEXPAND,3);
  row3->Add(excludes_label,1,wxALL|wxALIGN_CENTER_VERTICAL,3);
  row3->Add(excludes_edit,8,wxALL|wxEXPAND,3);
  // placing within the stack
  stack = new wxBoxSizer(wxVERTICAL);
  stack->Add(row1,0,wxEXPAND);
  stack->Add(row2,0,wxEXPAND);
  stack->Add(row3,0,wxEXPAND);
  // now, there is the site search and language picker
  sitesearch_label = new wxStaticText(this,-1,wxT("Site or Domain:"));
  sitesearch_edit = new wxTextCtrl(this,-1);
  langpicker_label = new wxStaticText(this,-1,wxT("Language"));
  lchoices = m_google_info->GetLangNames();
  langpicker = new wxChoice(this,-1,wxDefaultPosition, wxDefaultSize,(*lchoices));
  // the row4 sizer
  row4 = new wxBoxSizer(wxHORIZONTAL);
  row4->Add(sitesearch_label,0,wxALIGN_CENTER_VERTICAL|wxALL,3);
  row4->Add(sitesearch_edit,1,wxEXPAND|wxALL,3);
  row4->AddSpacer(10);
  row4->Add(langpicker_label,0,wxALIGN_CENTER_VERTICAL|wxALL,3);
  row4->Add(langpicker,0,wxALL,3);
  // adding it to the stack
  stack->Add(row4,0,wxEXPAND);
}
/*****************************************************************************/
