/***************************************************************
 * Name:      initend.cpp
 * Purpose:   Special startup and shutdown methods
 * Author:    John Q Metro (johnqmetro@hotmail.com)
 * Created:   Febuary 13, 2011
 * Updated:   July 23, 2015
 *
 **************************************************************/
#ifndef INITEND_H_INCLUDED
  #include "initend.h"
#endif // INITEND_H_INCLUDED
//---------------------------------------------------------
#ifndef HTMLPARSE_H_INCLUDED
  #include "core/utils/htmlparse.h"
#endif // HTMLPARSE_H_INCLUDED
#ifndef GLOBAL_SKEL_H_INCLUDED
  #include "interface/misc/global_skel.h"
#endif // GLOBAL_SKEL_H_INCLUDED
#ifndef GOOGLEINFO_H_INCLUDED
  #include "google/data/googleinfo.h"
#endif // GOOGLEINFO_H_INCLUDED
#ifndef FFN_ORGANIZER
  #include "fanficnet/categories/ffn_organizer.h"
#endif
#ifndef JFILTERGLOBALS
  #include "core/filters/base/filter_globals.h"
#endif
#ifndef LOGGING_H_INCLUDED
  #include "core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef ALTLOG_H_INCLUDED
  #include "core/utils/altlog.h"
#endif // ALTLOG_H_INCLUDED
#ifndef FFN_ORGANIZER
  #include "fanficnet/categories/ffn_organizer.h"
#endif
#ifndef UTILS3_H_INCLUDED
  #include "core/utils/utils3.h"
#endif // UTILS3_H_INCLUDED
#ifndef AO3_CATS_H_INCLUDED
  #include "fanfic/data/ao3/ao3_cats.h"
#endif // AO3_CATS_H_INCLUDED
#ifndef FFN_LINKDATA_H_INCLUDED
  #include "fanficnet/data/ffn_linkdata.h"
#endif // FFN_LINKDATA_H_INCLUDED
#ifndef AO3_CONSTS_H_INCLUDED
  #include "fanfic/data/ao3/ao3_consts.h"
#endif // AO3_CONSTS_H_INCLUDED
#ifndef FICMARKING_H_INCLUDED
  #include "core/structs/ficmarking.h"
#endif // FICMARKING_H_INCLUDED
#ifndef FIM_OTHERS_H_INCLUDED
  #include "fanfic/data/fim/fim_others.h"
#endif // FIM_OTHERS_H_INCLUDED
//----------------------------
#include <wx/dir.h>
#include <wx/stdpaths.h>
#include <curl/curl.h>
//****************************************************************
namespace globalsettings {
  // variables
  const wxString gfilename = wxT("globalvals.dat");
  bool dologging;
  bool logchange;
  size_t ficsize_hint;
  wxArrayString* globaldirs;
  size_t fimlimits[5];
  bool catobtained;
  // download amount tracking
  uint64_t totaldown;
  size_t reset_day;
  wxDateTime month_later;
  wxFrame* topwindow;
  wxMutex down_mutex;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool globalsettings::LoadSettings() {
  // variables
  wxTextFile* input_file;
  jfLineParse* parser;
  wxString cline;
  bool fail;
  size_t lcount;
  // starting
  topwindow = NULL;
  input_file = GetOpenFile(gfilename,false);
  if (input_file==NULL) return false;
  lcount = input_file->GetLineCount();
  if (lcount!=(3+GV_NUMVALS)) {
    input_file->Close();
    delete input_file;
    return false;
  }
  cline = input_file->GetFirstLine();
  input_file->GoToLine(1);
  fail = false;
  // parsing the first line
  parser = new jfLineParse(cline);
  if (parser->NNotX(2)) fail = true;
  // getting the fic size hint
  else if (!parser->SIntVal(0,ficsize_hint)) fail = true;
  else if (ficsize_hint==0) fail = true;
  // getting the logging value
  else if (!parser->BoolVal(1,dologging)) fail = true;
  // next up, the numeric fim constants
  if (!fail) {
    cline = input_file->GetLine(1);
    input_file->GoToLine(2);
    parser->SetString(cline);
    if (parser->NNotX(5)) fail = true;
    else if (!parser->SIntVal(0,fimlimits[0])) fail = true;
    else if (!parser->SIntVal(1,fimlimits[1])) fail = true;
    else if (!parser->SIntVal(2,fimlimits[2])) fail = true;
    else if (!parser->SIntVal(3,fimlimits[3])) fail = true;
    else if (!parser->SIntVal(4,fimlimits[4])) fail = true;
  }
  // here, we have the download tracking
  if (!fail) {
    cline = input_file->GetLine(2);
    input_file->GoToLine(3);
    parser->SetString(cline);
    if (parser->NNotX(2)) fail = true;
    else if (!parser->UInt64Val(0,totaldown)) fail = true;
    else if (!parser->SIntVal(1,reset_day)) fail = true;
  }
  // done
  delete parser;
  if (fail) {
    input_file->Close();
    delete input_file;
    return false;
  }
  SetZeroDate(reset_day);
  // after cleanup
  logchange = dologging;
  // loading lines from file
  globaldirs = new wxArrayString();
  fail = AppendFile2AS(globaldirs,input_file);
  input_file->Close();
  delete input_file;
  // handling the result
  if (!fail) {
    delete globaldirs;
    globaldirs = NULL;
  }
  return fail;
}
//------------------------------------------------------------
void globalsettings::SetToDefault(bool filldirs) {
  size_t lindex;
  // starting
  dologging = true;
  logchange = true;
  ficsize_hint = 350;
  fimlimits[0] = 2000;
  fimlimits[1] = 1;
  fimlimits[2] = 2200;
  fimlimits[3] = 197100;
  fimlimits[4] = 210000;
  // download info
  down_mutex.Lock();
  totaldown = 0;
  SetZeroNow();
  down_mutex.Unlock();
  // the directories
  if (filldirs) {
    globaldirs = new wxArrayString();
    for (lindex=0;lindex<GV_NUMVALS;lindex++) {
      globaldirs->Add(wxStandardPaths::Get().GetDocumentsDir());
    }
  }
  else globaldirs = NULL;
}
//------------------------------------------------------------
size_t globalsettings::TestGlobalDirs() {
  // variables
  size_t gcount,gindex;
  size_t rcount = 0;
  wxString testval;
  // checking for special case
  if (globaldirs==NULL) globaldirs = new wxArrayString();
  gcount = globaldirs->Count();
  if (gcount<GV_NUMVALS) {
    for (gindex=gcount;gindex<GV_NUMVALS;gindex++) {
      globaldirs->Add(wxStandardPaths::Get().GetDocumentsDir());
      rcount++;
    }
  }
  // testing
  for (gindex=0;gindex<GV_NUMVALS;gindex++) {
    testval = (*globaldirs)[gindex];
    if ((!wxDir::Exists(testval)) && (gindex!=7)){
      (*globaldirs)[gindex] = wxStandardPaths::Get().GetDocumentsDir();
      rcount++;
    }
  }
  return rcount;
}
//------------------------------------------------------------
bool globalsettings::LoadAllSettings() {
  bool rvalue = LoadSettings();
  if (rvalue) {
    TestGlobalDirs();
  }
  else SetToDefault(true);
  return rvalue;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool globalsettings::SaveSettings() {
  // variables
  wxTextFile* ofile;
  jfOutString ostring;
  wxString oxstring;
  // starting
  ofile = GetOpenFile(gfilename,true);
  if (ofile==NULL) return false;
  // making line 1
  ostring << ficsize_hint << logchange;
  oxstring = ostring;
  ofile->AddLine(oxstring);
  // the fim constants
  ostring.Clear();
  for (size_t dxi = 0; dxi < 5 ; dxi++) ostring << fimlimits[dxi];
  oxstring = ostring;
  ofile->AddLine(oxstring);
  // download quantity recording
  down_mutex.Lock();
  oxstring = UInt64_2Str(totaldown) + wxT(';');
  oxstring += SInt2Str(reset_day);
  ofile->AddLine(oxstring);
  down_mutex.Unlock();
  // the rest
  AppendAS2File(globaldirs,ofile);
  ofile->Write();
  ofile->Close();
  delete ofile;
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString globalsettings::GSaveDir(bool is_google) {
  assert(globaldirs!=NULL);
  assert(globaldirs->Count()>=4);
  return (is_google)?((*globaldirs)[1]):((*globaldirs)[3]);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// extra methods for download quantity tracking
//------------------------------------------------
bool globalsettings::SetZeroDate(size_t in_reset_day) {
  // variables
  size_t cur_day, next_month, next_year, days_in_nmonth, cxday;
  // special and basic cases
  if (in_reset_day==0) return false;
  if (in_reset_day>31) return false;
  reset_day = in_reset_day;
  // the cutoff date should be next reset_day
  cur_day = (wxDateTime::Today()).GetDay();
  // the cutoff date is in the current month
  if (cur_day <(wxDateTime::Month) reset_day) {
    month_later = wxDateTime::Today();
    month_later.SetDay(reset_day);
  }
  // here, the cutoff date is next month (which might be next year)
  else {
    // gathering information
    next_year = (size_t)wxDateTime::GetCurrentYear();
    next_month = (size_t)wxDateTime::GetCurrentMonth() + 1;
    if (((wxDateTime::Month)next_month)>wxDateTime::Dec) {
      next_year++;
      next_month = (size_t)wxDateTime::Jan;
    }
    days_in_nmonth = wxDateTime::GetNumberOfDays((wxDateTime::Month)next_month,next_year);
    // checking for a cutoff day past the end of the next month
    if (reset_day>days_in_nmonth) {
      // just in case, since december has 31 days
      assert(((wxDateTime::Month)next_month)!=wxDateTime::Dec);
      next_month++;
      cxday = 1;
    }
    else cxday = reset_day;
    // finally, setting the cutoff date
    month_later = wxDateTime(cxday,(wxDateTime::Month)next_month,next_year);
  }
  // done
  return true;
}
//------------------------------------------------
bool globalsettings::SetZeroNow(){
  size_t cur_day = (wxDateTime::Today()).GetDay();
  return SetZeroDate(cur_day);
}
//===============================================
// changes the working directory to the executable directory
bool ChangeWDir(wxString subdir) {
  bool rval;
  // getting the executable directory in wxFileName format
  wxString expath = wxStandardPaths::Get().GetExecutablePath();
  wxFileName fnpath(expath);
  // removing the filename
  fnpath.SetFullName(wxEmptyString);
  // appending subdir (if nonempty)
  if (!subdir.IsEmpty()) fnpath.AppendDir(subdir);
  // finally changing the directory
  rval = ::wxSetWorkingDirectory(fnpath.GetFullPath());
  return rval;
}
//===============================================================
bool LoadAtStart() {
  // constants
  bool rval;
  const wxString fname = wxT("LoadAtStart");
  // starting
  #if  defined (__WXGTK__) && defined (JRELEASE)
     rval = ChangeWDir();
  #endif
  // logging
  globalsettings::totaldown = 0;
  globalsettings::LoadAllSettings();
  // log test
  #ifdef __FREEBSD__
    altlog::doaltlog = true;
  #endif
  if (globalsettings::dologging) InitMainLog();
  // loading html parser entity data
  htmlparse::LoadEntities();
  // html output skeletons
  LoadSkeletons();
  // Fanfiction.Net category organizer
  ffn_catorganizer::main_manager = new jfFFN_CatManager();
  // google stuff
  m_google_info = new GoogleInfo();
  // loading data
  filterglobals::LoadStrings();
  globalsettings::catobtained = ffn_catorganizer::main_manager->ReadFromFile();
  ao3_catdata::Initialize();
  ffn_consts::SetupLists();
  ao3con::SetupLists();
  ficm::Initialize();
  // done
  return true;
}
//---------------------
bool WriteAtFinish() {
	const wxString fname = wxT("WriteAtFinish");
  filterglobals::Shutdown();
  WriteSkeletons();
  delete m_google_info;
  if (Second_Descr_Store!=NULL) delete Second_Descr_Store;
  ffn_catorganizer::main_manager->WriteToFile();
  delete ffn_catorganizer::main_manager;
	globalsettings::SaveSettings();
	delete globalsettings::globaldirs;
	globalsettings::globaldirs = NULL;
	ao3_catdata::Closing();
	ficm::StoreToFile();
	curl_global_cleanup();
	// finishing the logging
	if ((altlog::doaltlog) && (altlog::logvar!=NULL)) delete altlog::logvar;
  return true;
}
//****************************************************************
