/******************************************************************************
Name    :   display_base.cpp
Author  :   John Q Metro
Purpose :   Defines core classes for the main seach/result interface
Created :   August 23, 2009
Updated :   July 4, 2012
******************************************************************************/
#ifndef DISPLAY_BASE_H_INCLUDED
  #include "display_base.h"
#endif // DISPLAY_BASE_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
//---------------------------------------------------------------------------
/*****************************************************************************/
jfDefaultFilterEditorBase::jfDefaultFilterEditorBase(wxWindow* parent,wxWindowID id,
        jfSearchCore* inobj, size_t proph, size_t propv):wxPanel(parent,id) {
  // constants
  const wxString wxmain = wxT("Main Filter :");
  // some sanity checks
  assert(parent!=NULL);
  assert(inobj!=NULL);
  mainobj = inobj;
  // creating the expression filter
  mainlabel     = new wxStaticText(this,-1,wxmain);
  main_expedit  = new jfExpressionEditor(this,-1,mainobj->local_map,true,true,false);
  // arraning in a sizer...
  exp_sizer = new wxBoxSizer(wxVERTICAL);
  exp_sizer->Add(mainlabel,0,wxALL,1);
  exp_sizer->Add(main_expedit,2,wxALL|wxEXPAND,2);
  // this sizer goes into two nested sizers
  top_sizer = new wxBoxSizer(wxHORIZONTAL);
  top_sizer->Add(exp_sizer,proph,wxALL|wxEXPAND,2);
  main_sizer = new wxBoxSizer(wxVERTICAL);
  main_sizer->Add(top_sizer,propv,wxALL|wxEXPAND,1);
  // done
  SetSizer(main_sizer);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// data i/o
//------------------------------
bool jfDefaultFilterEditorBase::SaveFilters() {
  const wxString fname = wxT("jfDefaultFilterEditorBase::SaveFilters");
  // local variables
  jfExpressionFilter *temp1;
  wxString omsg;
  size_t oindex;
	bool zres = false;
  // the main expression filter
  temp1 = main_expedit->CheckFilter(omsg);
  if (temp1==NULL) {
    return false;
  }
  temp1->SetName(defexprname);
  embedded_filters->ReplaceSame(temp1,oindex);
  // calling the virtual method for the rest
  zres = SaveFiltersExtended();
	if (zres) /**/jfXLogMessage(fname,1);
	return zres;
}
//-------------------------------
bool jfDefaultFilterEditorBase::ChangeSearch(jfSearchCore* obj_data) {
  // constants
  const wxString fname = wxT("jfDefaultFilterEditorBase::ChangeSearch");
  // local variables
  jfExpressionFilter *ftemp1 = NULL;
  wxString omsg;
  // testing...
  /**/jfXLogMessage(fname,1);
  if (obj_data==NULL) return false;
  // assigning internal links
  mainobj = obj_data;
  embedded_filters = mainobj->def_filtermap;
  /**/jfXLogMessage(fname,2);
  // setting the data
  ftemp1 = dynamic_cast<jfExpressionFilter*>(embedded_filters->GetItem(defexprname));
  /**/jfXLogMessage(fname,3);
  if (ftemp1!=NULL) {
    ftemp1->SetFiltermapLink(obj_data->local_map);
    /**/jfXLogMessage(fname,4);
    if (!(main_expedit->SetData(ftemp1->ToString(),omsg))) return false;
    /**/jfXLogMessage(fname,5);
  }
  /**/jfXLogMessage(fname,6);
  // calling the virtual method for the rest
  return ChangeSearchExtended(obj_data);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal helper methods

//=============================================================================
jfSearchPanelBase::jfSearchPanelBase(wxWindow* parent, jfSearchCore* obj_data, wxWindowID id, bool noteb, bool svert, size_t prop, const wxString& name):
      wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // starting...
  assert(obj_data!=NULL);
  mainobj = obj_data;
  propstore = prop;
  // notebook configuration
  if (noteb) {
    #ifdef __WXGTK__
      nb = new wxNotebook(this,-1,wxDefaultPosition,wxDefaultSize,wxNB_TOP );
    #else
      nb = new wxNotebook(this,-1,wxDefaultPosition,wxDefaultSize,wxNB_LEFT );
    #endif
    isnotebook = true;
    param_sizer = NULL;
    defil_sizer = NULL;
  }
  // sizer configuration
  else {
    isnotebook = false;
    nb = NULL;
    // sizer creation
    param_sizer = new wxStaticBoxSizer(wxVERTICAL,this,wxT("Search Parameters"));
    defil_sizer = new wxStaticBoxSizer(wxVERTICAL,this,wxT("Included Filters"));
  }
  topsizer = new wxBoxSizer((svert)?(wxVERTICAL):(wxHORIZONTAL));
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfSearchPanelBase::ReloadObj() {
  bool cores = ChangeObj(mainobj);
  assert(cores);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* handles the task of arranging the top level panels, should be called at the
end of construction for a child class */
bool jfSearchPanelBase::ArrangePanels() {
  // checks
  if (filt_panel==NULL) return false;
  if (SearchPanel==NULL) return false;
  // notebook arranging
  if (isnotebook) {
    nb->AddPage(SearchPanel,wxT("Search Parameters"));
    nb->AddPage(filt_panel,wxT("Included Filters"));
    nb->ChangeSelection(0);
    topsizer->Add(nb,1,wxEXPAND|wxALL,1);
  }
  // sizer arranging
  else {
    defil_sizer->Add(filt_panel,1,wxALL|wxEXPAND,2);
    param_sizer->Add(SearchPanel,0,wxALL|wxEXPAND,3);
    topsizer->Add(param_sizer,propstore,wxEXPAND|wxALL,2);
    topsizer->Add(defil_sizer,10-propstore,wxEXPAND|wxALL,2);
  }
  // done
  SetSizer(topsizer);
  return true;
}
//=============================================================================
jfSearchGroupingCore::jfSearchGroupingCore(wxWindow* parent, wxWindowID id, const wxString& name):
      wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  searchdata = NULL;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfSearchCore* jfSearchGroupingCore::GetSearchObj() {
  return searchdata;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// causes panel1 to be updated from the search
void jfSearchGroupingCore::ReloadPanel1() {
  panel1->ReloadObj();
}
//============================================================================
jfSearchGrouping::jfSearchGrouping(wxWindow* parent, wxWindowID id, const wxArrayString* infclist, const wxString& name):
      jfSearchGroupingCore(parent,id,name) {
  custinit = false;
  assert(infclist!=NULL);
  fclist = infclist;
  #ifdef __WXGTK__
    panel_holder = new wxNotebook(this,-1,wxDefaultPosition,wxDefaultSize,wxNB_LEFT);
  #else
    panel_holder = new wxNotebook(this,-1);
  #endif
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the puropse of this method is to update the search data with GUI elements
bool jfSearchGrouping::UpdateSearchData() {
  if (!(nam_desc_e->ChangeObj(searchdata))) return false;
  if (!(panel1->LoadToObj())) return false;
  panel2->CoreSave();
  (searchdata->categories)->base_outputdir = dirpicker->GetPath();
  return panel3->SaveCategories();
}
//----------------------------------------------------------------
wxString jfSearchGrouping::GetDisplayName() const {
  return nam_desc_e->TryGetName();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special methods
//------------------------------------------------------------------------
/* some modifications for wxGTK as it appears trying to change everything
triggers two events, and the first one is blank */
void jfSearchGrouping::NameChangeEvent(wxCommandEvent& event) {
  if (!ninit) {
    if (!nam_desc_e->IsNameEmpty()) {
      wxString ndname = nam_desc_e->TryGetName();
      panel3->ChangeSearchName(ndname);
    }
  }
}
//--------------------------------------------------------------------------
void jfSearchGrouping::HandleNewDirectory(wxFileDirPickerEvent& inevent) {
  searchdata->categories->base_outputdir = dirpicker->GetPath();
}
//--------------------------------------------------------------------------
void jfSearchGrouping::WindowResized(size_t x, size_t y) {
  if (y<700) panel3->ChangeToHorizontal();
  else panel3->ChangeToVertical();
  MoreResized(x,y);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfSearchGrouping::FinishConstruction() {
	const wxChar* fname = wxT("jfSearchGrouping::FinishConstruction");
  assert(custinit);
	jfXLogMessage(fname,1);
	ninit = true;
  // panel 2: local filters, either new or from searchin
  if (searchdata->local_map == NULL) searchdata->local_map = new jfFilterMap();
	jfXLogMessage(fname,2);
  panel2 = new jfFilterMapEditor(panel_holder,-1,searchdata->local_map,false,fclist);
	jfXLogMessage(fname,3);
  // panel 3: categories
  panel3 = new jfMainCategoriesEditor(panel_holder,searchdata->categories);
	jfXLogMessage(fname,4);
  // the name editor
  nam_desc_e = new jfNameDescEditor(this,-1,false,false,jfID_SEARCHNAME_EDIT);
	jfXLogMessage(fname,6);
	// the directory picker
	dirpicker = new jfDirPicker(this,-1,false,(searchdata->categories)->base_outputdir);
	jfXLogMessage(fname,7);
  // the notebook
  panel_holder->AddPage(panel1,wxT("Main"),true);
  panel_holder->AddPage(panel2,wxT("Filters"));
  panel_holder->AddPage(panel3,wxT("Categories"));
	jfXLogMessage(fname,8);
  // top level arranging
  top_sizer = new wxBoxSizer(wxVERTICAL);
  top_sizer->Add(nam_desc_e,0,wxALL|wxEXPAND,3);
  top_sizer->Add(dirpicker,0,wxALL|wxEXPAND,3);
  top_sizer->Add(panel_holder,1,wxEXPAND|wxALL,3);
	jfXLogMessage(fname,9);
  // finishing
  SetSizer(top_sizer);
  jfXLogMessage(fname,10);
  ninit = false;
  nam_desc_e->SetFromObj(searchdata);
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfSearchGrouping,wxPanel)
	EVT_TEXT(jfID_SEARCHNAME_EDIT,jfSearchGrouping::NameChangeEvent)
	EVT_DIRPICKER_CHANGED(-1,jfSearchGrouping::HandleNewDirectory)
END_EVENT_TABLE()

/*****************************************************************************/
