/******************************************************************************
Name    :   display_base.h
Author  :   John Q Metro
Purpose :   Declares core classes for the main seach/result interface
Created :   August 23, 2009
Updated :   December 21, 2013
******************************************************************************/
#ifndef DISPLAY_BASE_H_INCLUDED
#define DISPLAY_BASE_H_INCLUDED
#endif // DISPLAY_BASE_H_INCLUDED
//---------------------------------------------------------------------------
#ifndef SEARCHCORE_H_INCLUDED
  #include "../../core/objs/searchcore.h"
#endif // SEARCHCORE_H_INCLUDED
#ifndef EXPR_EDITOR_H_INCLUDED
  #include "../filters/base/expr_editor.h"
#endif // EXPR_EDITOR_H_INCLUDED
#ifndef RESULTCOLLECT_H_INCLUDED
  #include "../../core/structs/resultcollect.h"
#endif // RESULTCOLLECT_H_INCLUDED
#ifndef FILTER_EDITOR_H_INCLUDED
  #include "../filters/base/filter_editor.h"
#endif // FILTER_EDITOR_H_INCLUDED
#ifndef CATEGORY_LISTEDIT_H_INCLUDED
  #include "../filters/base/category_listedit.h"
#endif // CATEGORY_LISTEDIT_H_INCLUDED
#ifndef WIDGETS3_H_INCLUDED
  #include "../misc/widgets3.h"
#endif // WIDGETS3_H_INCLUDED
#ifndef MORE_THREADS_H_INCLUDED
  #include "../../connect/more_threads.h"
#endif // MORE_THREADS_H_INCLUDED
//---------------------------------------
#include <wx/notebook.h>
#include <wx/filepicker.h>
/*****************************************************************************
For the main search/result interface, there are two main sections: one for the
search and 1 for the results:
Search: Has a name and description editor at the top, and 3 panels (contained
        in a tabbed notebook control). The first is custom and is for search
        parameters and embedded filters. The second is the local filter editor.
        The third is for catgorization.
Results: The status display at the bottom, jfResult panel for displaying the
         results at the top.


*****************************************************************************/
const wxString defexprname = wxT("Default Expression Filter");
/* because default filters can be complicated, I've decided to put them on a
separate panel */
class jfDefaultFilterEditorBase : public wxPanel {
  public:
    // the constructor
    jfDefaultFilterEditorBase(wxWindow* parent,wxWindowID id, jfSearchCore* inobj, size_t proph, size_t propv);
    // data i/o
    bool SaveFilters();
    bool ChangeSearch(jfSearchCore* obj_data);
  protected:
    // internal abstract methods
    virtual bool SaveFiltersExtended() = 0;
    virtual bool ChangeSearchExtended(jfSearchCore* obj_data) = 0;
    // internal links
    jfFilterMap* embedded_filters;
    jfSearchCore* mainobj;
    // gui elements
    wxStaticText*       mainlabel;
    jfExpressionEditor* main_expedit;
    wxBoxSizer*         exp_sizer;
    wxBoxSizer*         top_sizer;
    wxBoxSizer*         main_sizer;
};
//============================================================================
// base class for the custom first panel
class jfSearchPanelBase : public wxPanel {
  public:
    jfSearchPanelBase(wxWindow* parent, jfSearchCore* obj_data, wxWindowID id, bool noteb,
              bool svert = true, size_t prop = 0, const wxString& name = wxT("jfSearchPanelBase"));
    virtual bool LoadToObj() =0;
    virtual bool ChangeObj(jfSearchCore* obj_data) =0;
    void ReloadObj();
  protected:
    // internal methods
    bool ArrangePanels();
    virtual jfDefaultFilterEditorBase* MakeTypedFilEdit() = 0;
    // internal links
    jfSearchCore* mainobj;
    // gui elements
    jfDefaultFilterEditorBase* filt_panel;
    wxPanel* SearchPanel;
    wxNotebook* nb;
    bool isnotebook;
    size_t propstore;
    // sizers
    wxStaticBoxSizer* param_sizer;
    wxStaticBoxSizer* defil_sizer;
    wxBoxSizer* topsizer;
};
//==========================================================================
class jfSearchGroupingCore : public wxPanel {
  public:
    // the constructor
    jfSearchGroupingCore(wxWindow* parent, wxWindowID id, const wxString& name = wxT("jfSearchGroupingCore"));
    // custom  methods
    virtual jfSearchPanelBase*  MakeFirstPanel(wxWindowID id_in) = 0;
    virtual jfSearchCore*       MakeTypedSearch() const = 0;
    virtual jfResultCollection* MakeTypedCollection() const = 0;
    virtual jfSingleSearchThreadBase* MakeTypedThread() const = 0;
    // returning data
    jfSearchCore* GetSearchObj();
    virtual bool UpdateSearchData() = 0;
		// special methods
    virtual void WindowResized(size_t x, size_t y) = 0;
    // special methods
    void ReloadPanel1();
  protected:
    virtual void FinishConstruction() = 0;
    // the main panels
    jfSearchPanelBase*        panel1;
    // the sizer
    wxBoxSizer* top_sizer;
    // internal data
    jfSearchCore* searchdata;
};
//===========================================================================
class jfSearchGrouping : public jfSearchGroupingCore {
  public:
    // the constructor
    jfSearchGrouping(wxWindow* parent, wxWindowID id, const wxArrayString* infclist,
                     const wxString& name = wxT("jfSearchGrouping"));
    // custom  methods
    virtual jfSearchPanelBase*  MakeFirstPanel(wxWindowID id_in) = 0;
    virtual jfSearchCore*       MakeTypedSearch() const = 0;
    virtual jfResultCollection* MakeTypedCollection() const = 0;
    virtual jfSingleSearchThreadBase* MakeTypedThread() const = 0;
    // returning data
    virtual bool UpdateSearchData();
    wxString GetDisplayName() const;
		// special methods
		void NameChangeEvent(wxCommandEvent& event);
		void HandleNewDirectory(wxFileDirPickerEvent& inevent);
    virtual void WindowResized(size_t x, size_t y);
  protected:
    virtual void FinishConstruction();
    virtual void MoreResized(size_t x, size_t y) = 0;
    // the main panels
    jfSearchPanelBase*        panel1;
    jfFilterMapEditor*        panel2;
    jfMainCategoriesEditor*   panel3;
    // other gui elements
    wxNotebook* panel_holder;
    jfNameDescEditor* nam_desc_e;
    jfDirPicker* dirpicker;
    // the sizer
    wxBoxSizer* top_sizer;
    // internal data
    bool custinit;
    bool ninit;
    // external data
    const wxArrayString* fclist;
	// the event handline declaration
	DECLARE_EVENT_TABLE()
};
/*****************************************************************************/
