/******************************************************************************
Name    :   display_base1.cpp
Author  :   John Q Metro
Purpose :   Main result/Search Interface, 3 tab version
Created :   September 22, 2012
Updated :   September 26, 2012
******************************************************************************/
#ifndef DISPLAY_BASE1_H_INCLUDED
  #include "display_base1.h"
#endif // DISPLAY_BASE1_H_INCLUDED
//----------------------------------------
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED

//*****************************************************************************
jfPanelTwo::jfPanelTwo(wxWindow* parent, wxWindowID id, jfSearchCore* onsearch,
      const wxArrayString* infclist, const wxString& name): wxPanel(parent,id,
      wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  assert(infclist!=NULL);
  assert(onsearch!=NULL);
  fclist = infclist;
  searchdata_link = onsearch;
  #ifdef __WXGTK__
    panel_holder = new wxNotebook(this,-1,wxDefaultPosition,wxDefaultSize,wxNB_LEFT);
  #else
    panel_holder = new wxNotebook(this,-1);
  #endif
  FinishConstruction();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfSearchCore* jfPanelTwo::GetSearchObj() {
  return searchdata_link;
}
//------------------------------------------------------------------------
// the puropse of this method is to update the search data with GUI elements
bool jfPanelTwo::UpdateSearchData() {
  const wxString fname = wxT("jfPanelTwo::UpdateSearchData");
  /**/jfXLogMessage(fname,1);
  if (!(nam_desc_e->ChangeObj(searchdata_link))) return false;
  /**/jfXLogMessage(fname,2);
  panel2->CoreSave();
  (searchdata_link->categories)->base_outputdir = dirpicker->GetPath();
  /**/jfXLogMessage(fname,3);
  return panel3->SaveCategories();
}
//----------------------------------------------------------------
wxString jfPanelTwo::GetDisplayName() const {
  return nam_desc_e->TryGetName();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special methods
//------------------------------------------------------------------------
/* some modifications for wxGTK as it appears trying to change everything
triggers two events, and the first one is blank */
void jfPanelTwo::NameChangeEvent(wxCommandEvent& event) {
  if (!ninit) {
    if (!nam_desc_e->IsNameEmpty()) {
      wxString ndname = nam_desc_e->TryGetName();
      panel3->ChangeSearchName(ndname);
    }
  }
}
//--------------------------------------------------------------------------
void jfPanelTwo::HandleNewDirectory(wxFileDirPickerEvent& inevent) {
  searchdata_link->categories->base_outputdir = dirpicker->GetPath();
}
//--------------------------------------------------------------------------
void jfPanelTwo::WindowResized(size_t x, size_t y) {
  if (y<850) panel3->ChangeToHorizontal();
  else panel3->ChangeToVertical();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfPanelTwo::FinishConstruction() {
	const wxChar* fname = wxT("jfPanelTwo::FinishConstruction");
	jfXLogMessage(fname,1);
	ninit = true;
  // panel 2: local filters, either new or from searchin
  if (searchdata_link->local_map == NULL) searchdata_link->local_map = new jfFilterMap();
	jfXLogMessage(fname,2);
  panel2 = new jfFilterMapEditor(panel_holder,-1,searchdata_link->local_map,false,fclist);
	jfXLogMessage(fname,3);
  // panel 3: categories
  panel3 = new jfMainCategoriesEditor(panel_holder,searchdata_link->categories);
	jfXLogMessage(fname,4);
  // the name editor
  nam_desc_e = new jfNameDescEditor(this,-1,false,false,jfID_SEARCHNAME_EDIT);
	jfXLogMessage(fname,6);
	// the directory picker
	dirpicker = new jfDirPicker(this,-1,false,(searchdata_link->categories)->base_outputdir);
	jfXLogMessage(fname,7);
  // the notebook
  panel_holder->AddPage(panel2,wxT("Filters"));
  panel_holder->AddPage(panel3,wxT("Categories"));
	jfXLogMessage(fname,8);
  // top level arranging
  top_sizer = new wxBoxSizer(wxVERTICAL);
  top_sizer->Add(nam_desc_e,0,wxALL|wxEXPAND,3);
  top_sizer->Add(dirpicker,0,wxALL|wxEXPAND,3);
  top_sizer->Add(panel_holder,1,wxEXPAND|wxALL,3);
	jfXLogMessage(fname,9);
  // finishing
  SetSizer(top_sizer);
  jfXLogMessage(fname,10);
  ninit = false;
  nam_desc_e->SetFromObj(searchdata_link);
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfPanelTwo,wxPanel)
	EVT_TEXT(jfID_SEARCHNAME_EDIT,jfPanelTwo::NameChangeEvent)
	EVT_DIRPICKER_CHANGED(-1,jfPanelTwo::HandleNewDirectory)
END_EVENT_TABLE()
//============================================================================
jfOnePanelSearch::jfOnePanelSearch(wxWindow* parent, wxWindowID id, const wxString& name):
      jfSearchGroupingCore(parent,id,name) {
  custinit = false;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the puropse of this method is to update the search data with GUI elements
bool jfOnePanelSearch::UpdateSearchData() {
  return panel1->LoadToObj();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special methods
//------------------------------------------------------------------------
void jfOnePanelSearch::WindowResized(size_t x, size_t y) {
  MoreResized(x,y);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfOnePanelSearch::FinishConstruction() {
	const wxChar* fname = wxT("jfOnePanelSearch::FinishConstruction");
  assert(custinit);
	jfXLogMessage(fname,1);
  // the notebook
  // top level arranging
  top_sizer = new wxBoxSizer(wxVERTICAL);
  top_sizer->Add(panel1,1,wxALL|wxEXPAND,3);
	jfXLogMessage(fname,2);
  // finishing
  SetSizer(top_sizer);
  Layout();
}
//*****************************************************************************
