/******************************************************************************
Name    :   display_base2.cpp
Author  :   John Q Metro
Purpose :   Split from display_base.cpp on Sept 13, 2012
Created :   August 23, 2009
Updated :   July 1, 2015
******************************************************************************/
#ifndef DISPLAY_BASE2_H_INCLUDED
  #include "display_base2.h"
#endif // DISPLAY_BASE2_H_INCLUDED
//------------------------------------
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
/*****************************************************************************/
// we need a constructor
DisplayPanelCore::DisplayPanelCore(wxWindow* parent, wxWindowID id, wxString inname):
    wxPanel(parent,id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, inname) {
}
/*****************************************************************************/
// the defult constructor
jfMainSearchGroup::jfMainSearchGroup(wxWindow* parent, wxWindowID id, jfSearchCore* searchin, rtype_enum rinval, bool notebook, bool multiple):DisplayPanelCore(parent,id,wxT("MainSearchGroup")) {
  // starting preps
  p_search = NULL;
  noteb = notebook;
  search_data = NULL;
  thread_holder = NULL;
  results = NULL;
  launched = false;
  beensaved = false;
  rc_menu = NULL;
  #ifdef __WXGTK__
    threepanel = true;
  #else
    threepanel = false;
  #endif
  panel_two = NULL;
  // thread constants
  thread_constants = NULL;
  // the search data has to be completed before construction is done
}
//---------------------------------------------------------------------------
void jfMainSearchGroup::StartButton(wxCommandEvent& event) {
	const wxString fname=wxT("jfMainSearchGroup::StartButton");
	// we start here...
  ClearOldData();
  launched = true;
  wxBeginBusyCursor();
  starttime = wxDateTime::Today();
  if (!BuildSearch()) {
		r_panel->SearchAbort();
		// if BuildSearch returns false we abort the search
		launched = false;
	}
	else {
		SendNotDoneSignal();
		MakeThreadCollection();
    wxThreadError terror = thread_holder->Create();
		assert(wxTHREAD_NO_ERROR==terror);
		// running the thread
		thread_holder->Run();
	}
	wxEndBusyCursor();
}
//-------------------------------------------------------------------------
// what we do for this is to signal the topmost window
void jfMainSearchGroup::PPSearchDoneSignal(wxCommandEvent& event) {
  // we build the vent to sent
  wxCommandEvent outevent(jf_MSG_SRCHDONE_EVENT,GetId());
  outevent.SetInt(GetId());
  // we get where to send it to
  wxWindow* gparent = GetGrandParent();
  assert (gparent!=NULL);
  // we sent it
  wxPostEvent(gparent,outevent);
}
//------------------------------------------------------------------------
// signalling the top window that the search has failed
void jfMainSearchGroup::PPSearchFailSignal(wxCommandEvent& event) {
  bool halted = (bool)(event.GetInt());
  event.SetInt(GetId());
  // we get where to send it to
  wxWindow* gparent = GetGrandParent();
  assert (gparent!=NULL);
  // we sent it
  wxPostEvent(gparent,event);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// right click menu event handlers
//-------------------------------------------------------------------------
// handles the append text right click event
void jfMainSearchGroup::OnRMenuAppendText(wxCommandEvent& event) {
  // constants
  const wxString req_msg = wxT("Edit the text to what you want.");
  const wxString req_cap = wxT("Append to Expression Filter");
  const wxString inc_msg = wxT("The resulting expression is invalid! Try again?");
  const wxString inc_cap = wxT("Result is invalid!");
  // local variables
  wxString etext, restext;
  wxPoint* index_coord;
  jfExpressionFilter* filptr;
  wxString fil_string;
  int inc_res;
  // starting...
  // we start by getting coordinates for the item
  index_coord = reinterpret_cast<wxPoint*>(event.GetClientData());
  // getting the text
  etext = results->GetItemDescription(index_coord->x,index_coord->y);
  delete index_coord;
  // we now pop up a dialog that lets the user edit the text before adding it
  etext = wxT("& ( ") + etext + wxT(" )");
  restext = etext;
  while (true) {
    restext = wxGetTextFromUser(req_msg,req_cap,restext);
    // if the result is not empty, we now work to add it to the filter
    if (!restext.IsEmpty()) {
      // we get a pointer to the extression filter
      filptr = dynamic_cast<jfExpressionFilter*>(search_data->def_filtermap->GetItem(wxT("01")));
      // appending the text
      fil_string = filptr->ToString();
      fil_string += restext;
      // appending worked!
      if (filptr->FromString(fil_string)) {
        // we re-load the gui elements
        p_search->ReloadPanel1();
        break;
      }
      // the result is not a valid expression
      else {
        // ask the user if he/she wants to try again
        inc_res = wxMessageBox(inc_msg,inc_cap,wxYES_NO);
        if (inc_res == wxNO) break;

      }
    }
    else break;
  }
}
//-----------------------------------------------
void jfMainSearchGroup::OnMarkDiscard(wxCommandEvent& event) {
  wxPoint* index_coord;
  bool discval = (bool)(event.GetInt());
  index_coord = reinterpret_cast<wxPoint*>(event.GetClientData());
  results->SetDiscardValue(index_coord->x,index_coord->y,discval);
  delete index_coord;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// returns whether downloading and parsing in in progress
bool jfMainSearchGroup::InProgress() const {
  return r_panel->InProgress();
}
//---------------------------------------------------------------------------
// orders a halt to any downloads/parsing in progress
bool jfMainSearchGroup::CStop() {
  return r_panel->CStop();
}
//---------------------------------------------------------------------------
wxString jfMainSearchGroup::GetSearchName() const {
  if (search_data==NULL) return wxEmptyString;
  else return search_data->GetName();
}
//-------------------------------------------------------------------------
wxString jfMainSearchGroup::GetDisplayName() const {
  jfSearchGrouping* typ;
  if (threepanel) {
    return panel_two->GetDisplayName();
  }
  else {
    typ = dynamic_cast<jfSearchGrouping*>(p_search);
    return typ->GetDisplayName();
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// doing output
//-------------------------------------------------------------------------
bool jfMainSearchGroup::WriteAll2Html() {
  wxBeginBusyCursor();
  if ((search_data->categories)->UsesDefault()) {
    results->WriteToHTML(32);
  }
  size_t ccount = (search_data->categories)->GetCount();
  if (ccount!=0) {
    for (size_t cloop=0;cloop<ccount;cloop++) {
        results->WriteToHTML(cloop);
    }
  }
  wxEndBusyCursor();
  return true;
}
//-------------------------------------------------------------------------
bool jfMainSearchGroup::WriteCurrent2Html() {
  wxBeginBusyCursor();
  size_t curr_res = r_panel->SelectedIndex();
  results->WriteToHTML(curr_res);
  wxEndBusyCursor();
  return true;
}
//---------------------------------------------------------------------------
// saves the current contents to a data file
bool jfMainSearchGroup::SaveToFile(wxString fname) {
  const wxString ifname = wxT("jfMainSearchGroup::SaveToFile");
	// local variables
  wxTextFile* outfile;
  bool usd_res;
  // starting
  /**/jfXLogMessage(ifname, 1);
  assert(search_data!=NULL);
  /**/jfXLogMessage(ifname, 2);
  wxBeginBusyCursor();
  /**/jfXLogMessage(ifname, 3);
  usd_res = p_search->UpdateSearchData();
  assert(usd_res);
  if (threepanel) {
    usd_res = panel_two->UpdateSearchData();
    assert(usd_res);
  }
  /**/jfXLogMessage(ifname, 4);
  // file opening
  outfile = new wxTextFile(fname);
  /**/jfXLogMessage(fname, 5);
  if (!outfile->Exists()) {
		outfile->Create();
		outfile->Open();
	}
  else {
    outfile->Open();
    outfile->Clear();
  }
  assert(outfile->IsOpened());
  // we have the file written
  search_data->AddToFile(outfile);
  // saving file data
  outfile->Write();
  outfile->Close();
  savfpath = fname;
  beensaved = true;
  // done
	jfXLogMessage(ifname,8);
  wxEndBusyCursor();
  return true;
}
//--------------------------------------------------------------------------
bool jfMainSearchGroup::SaveToFile() {
  if (!beensaved) return false;
  return SaveToFile(savfpath);
}
//--------------------------------------------------------------------------
bool jfMainSearchGroup::HasBeenSaved() const {
  return beensaved;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfSearchCore* jfMainSearchGroup::GetSearchPointer() {
  return search_data;
}
//-----------------------------------------------------------------------
void jfMainSearchGroup::WindowResized(size_t x, size_t y) {
  p_search->WindowResized(x,y);
  r_panel->WindowResized(x,y);
  if (threepanel) panel_two->WindowResized(x,y);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* finishes construction, should be called by a constructor of a child class
after the calling of (their implemented version of) MakeSearchGroup */
void jfMainSearchGroup::TopArranger() {
  const wxString fname = wxT("jfMainSearchGroup::TopArranger");
  /**/jfXLogMessage(fname,1);
  /**/jfXLogMessage(fname,2,search_data->GetName());
  top_sizer = new wxBoxSizer(wxHORIZONTAL);
  if (noteb) {
    /**/jfXLogMessage(fname,3);
    if (threepanel) {
      container->AddPage(p_search,wxT("Parameters"));
      container->AddPage(panel_two,wxT("Filters and Categories"));
    }
    else {
      container->AddPage(p_search,wxT("Params and Filters"));
    }
    container->AddPage(r_panel,wxT("Results"));
    top_sizer->Add(container,1,wxALL|wxEXPAND,3);
  }
  else {
    top_sizer->Add(p_search,1,wxALL|wxEXPAND,3);
    if (threepanel) top_sizer->Add(panel_two,1,wxALL|wxEXPAND,3);
    top_sizer->Add(r_panel,1,wxALL|wxEXPAND,3);
  }
  /**/jfXLogMessage(fname,4);
  SetSizer(top_sizer);
  Layout();
}
//---------------------------------------------------------------------------
// creates (does not launch) the search thread and result collection
void jfMainSearchGroup::MakeThreadCollection() {
  // variables
  jfProgressPanelBase* temp1;
  jfNewPauser* temp2;
  // making the thread
  temp1 = r_panel->GetProgressPanel();
  thread_holder = p_search->MakeTypedThread();
  // loading the thread with data
  temp2 = r_panel->GetPauser();
  thread_constants = new jfDownloadParams();
  thread_constants->msg_target = temp1;
  thread_constants->pauselink = temp2;
  thread_holder->SetParams(thread_constants);
  // making the collection and loading it into the thread
  results = p_search->MakeTypedCollection();
  thread_holder->SetResultsStore(results);
}
//-----------------------------------------------------------------------------
bool jfMainSearchGroup::BuildSearch() {
  // variables and asserts
  assert(launched);
  bool rval;
  wxString oval;
  if (!(p_search->UpdateSearchData())) return false;
  if (threepanel) {
    if (!(panel_two->UpdateSearchData())) return false;
  }
  rval = search_data->VerifyNames(oval);
  if (rval) {
    search_data->CopyLoad();
    if (!r_panel->LoadGetLinks(search_data)) return false;
    return true;
  }
  else {
    wxMessageBox(oval,wxT("Name Verification Failed!"),wxOK|wxICON_ERROR);
    return false;
  }
}
//-----------------------------------------------------------------------------
// when we want to re-do a search, we call this first to clear out the old data
void jfMainSearchGroup::ClearOldData() {
	const wxChar* fname = wxT("jfMainSearchGroup::ClearOldData");
  // resetting the result display
  /**/jfXLogMessage(fname,1);
  if (launched) r_panel->ClearOld();
  /**/jfXLogMessage(fname,2);
  // removing the html result pointer vector, now useless
  if (NULL!=(search_data->cat_results)) {
    search_data->cat_results = NULL;
  }
  /**/jfXLogMessage(fname,3);
  // iirc, the thread object self-deletes
  if (thread_holder!=NULL) thread_holder = NULL;
  // deleting the old results
  /**/jfXLogMessage(fname,4);
  if (results!=NULL) {
    delete results;
    results = NULL;
  }
  /**/jfXLogMessage(fname,5);
  // setting a flag
  launched = false;
}
//----------------------------------------------------------------------------
void jfMainSearchGroup::SendNotDoneSignal() {
  // we build the vent to sent
  wxCommandEvent outevent(jf_MSG_SRCHNDONE_EVENT,GetId());
  outevent.SetInt(GetId());
  // we get where to send it to
  wxWindow* gparent = GetGrandParent();
  assert (gparent!=NULL);
  // we sent it
  wxPostEvent(gparent,outevent);
}
//-------------------------------------------------------------------------
wxString jfMainSearchGroup::GSearchName() const {
  if (p_search==NULL) return wxEmptyString;
  else return p_search->GetSearchObj()->GetName();
}
//-------------------------------------------------------------------------------
bool jfMainSearchGroup::ConsStepTwo(const wxArrayString* infclist) {
  // asserts and checks
  assert(p_search!=NULL);
  assert(infclist);
  // assifgning search data
  search_data = p_search->GetSearchObj();
  // the middle panel
  if (threepanel) {
    if (noteb) panel_two = new jfPanelTwo(container,-1,search_data,infclist);
    else panel_two = new jfPanelTwo(this,-1,search_data,infclist);
  }
  return true;
}
//-------------------------------------------------------------------------------
void jfMainSearchGroup::SetFicMark(size_t xindex, jf_FicMark mkval, size_t c_index, size_t i_index) {
  size_t res_id;
  // getting the fanfic data
  res_id = results->GetItemID(c_index, i_index);
  ficm::data->fd[xindex]->SetMark(res_id,mkval);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfMainSearchGroup,wxPanel)
  EVT_BUTTON  (jfID_STARTSTOPBTN , jfMainSearchGroup::StartButton )
  EVT_COMMAND (-1,jf_PP_SRCHDONE_EVENT,jfMainSearchGroup::PPSearchDoneSignal )
  EVT_COMMAND (-1,jf_PP_SRCHFAIL_EVENT,jfMainSearchGroup::PPSearchFailSignal )
  EVT_COMMAND (-1,jf_RMENU_APPTXTFIL_EVENT,jfMainSearchGroup::OnRMenuAppendText)
  EVT_COMMAND (-1,jf_FICDISCARD_EVENT,jfMainSearchGroup::OnMarkDiscard)
END_EVENT_TABLE()

//==========================================================================
/*****************************************************************************/
