/*****************************************************************************
Name:        html_display.cpp
Author :     John Q Metro
Purpose :    A replacement for HTML listbox for showing results
Created:     April 26, 2011
Updated:     Sepetmber 21, 2014
******************************************************************************/
#ifndef HTML_DISPLAY_H_INCLUDED
  #include "html_display.h"
#endif // HTML_DISPLAY_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
//----------------------------------------------------------------------
//****************************************************************************
// the constructor
jfHtmlResultViewer::jfHtmlResultViewer(size_t inresindex, wxWindow* parent, wxWindowID id):wxHtmlListBox(parent,id) {
  // status info
  assert(inresindex<33);
  res_index = inresindex;
  gparent = dynamic_cast<jfHtmlResultCollection*>(GetGrandParent());
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the html list box method that must be overridden
wxString jfHtmlResultViewer::OnGetItem(size_t n) const {
  return gparent->GetHTML(res_index,n);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* right click menu: at this level, all we do is throw another event which
contains the collection and item index */
void jfHtmlResultViewer::OnRightUp(wxMouseEvent& mevent) {
  // local variables
  const wxString fname = wxT("jfHtmlResultViewer::OnRightUp");
  wxPoint c_point;
  int list_index;
	wxString srcurl;
  // getting the position
  c_point = mevent.GetPosition();
  list_index = HitTest(c_point);
  // we start the processing...
  if (list_index!=wxNOT_FOUND) {
    // creating the event
    wxCommandEvent outevent(jf_HTML_RIGHTCLICK_EVENT,jfID_DISPLAY_RESORG);
    outevent.SetInt(res_index);
    outevent.SetExtraLong(list_index);
    srcurl = gparent->GetURL(res_index,list_index);
    /**/jfXLogMessageI(fname,1,list_index);
    /**/jfXLogMessageS(fname,2,res_index);
    /**/jfXLogMessage(fname,3,srcurl);
    outevent.SetString(srcurl);
    wxPostEvent(GetGrandParent(),outevent);
  }
}
//----------------------------------------------------------------------------
/* for some reason, mouse wheel scrolling is not automatic... */
void jfHtmlResultViewer::OnMouseWheel(wxMouseEvent& mevent) {
  int rot = mevent.GetWheelRotation() / mevent.GetWheelDelta();
  #if wxCHECK_VERSION(2 ,9 ,0 )
    ScrollRows(-rot);
  #else
    ScrollLines(-rot);
  #endif
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfHtmlResultViewer,wxHtmlListBox)
  EVT_RIGHT_UP(jfHtmlResultViewer::OnRightUp)
  EVT_MOUSEWHEEL(jfHtmlResultViewer::OnMouseWheel)
END_EVENT_TABLE()
//****************************************************************************
// constructor
jfHtmlResultCollection::jfHtmlResultCollection(header_enum inhtype, wxWindow* parent,
        wxWindowID id,wxString inname):wxPanel(parent,id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL,inname) {
  // flags
  setup = false;
  use_default = false;
  num_rcats = 0;
  num_items = 0;
  hype = inhtype;
  popup_menu = NULL;
  // data
  mgroup = NULL;
  for (size_t q = 0;q<33;q++) panels[q] = NULL;
  // gui elemets
  mainbook = new wxNotebook(this,-1);
  msizer = new wxBoxSizer(wxVERTICAL);
  dispbar = new wxBoxSizer(wxHORIZONTAL);
  result_label = new wxStaticText(this,-1,wxT("No Results"));
  dispbar->Add(result_label,1,wxALL|wxALIGN_CENTER_VERTICAL|wxLEFT,1);
  msizer->Add(mainbook,1,wxEXPAND|wxALL,3);
  msizer->Add(dispbar,0,wxEXPAND|wxALL,1);
  SetSizer(msizer);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// methods accessed by viewer
//--------------------------------
// returns the HTML data
wxString jfHtmlResultCollection::GetHTML(size_t rcat_index, size_t item_index) const {
  // asserts
  assert(setup);
  if (rcat_index==32) assert(use_default);
  else assert(rcat_index<num_rcats);
  // we return
  return mgroup->GetHTML(rcat_index,item_index);
}
//--------------------------------------------------------------
wxString jfHtmlResultCollection::GetURL(size_t rcat_index, size_t item_index) const {
  // asserts
  assert(setup);
  if (rcat_index==32) assert(use_default);
  else assert(rcat_index<num_rcats);
  // we return
  return mgroup->GetUrl(rcat_index,item_index);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfHtmlResultCollection::SetPopupMenu(jfPopupMenu* inmenu) {
  assert(inmenu!=NULL);
  popup_menu = inmenu;
}
//-----------------------------------------------------------------
bool jfHtmlResultCollection::SetupViewers(jfCategories* input) {
  // a special case
  if (setup) return false;
  // checks and asserts
  assert(input!=NULL);
  // we start
  use_default = input->UsesDefault();
  num_rcats = input->GetCount();
  // the data
  mgroup = MakeIndexers();
  // we set up the HTML viewers
  if (use_default) {
    panels[32] = new jfHtmlResultViewer(32,mainbook);
    mainbook->AddPage(panels[32],wxString(wxT("Default")));
  }
  for(size_t qrun=0;qrun<num_rcats;qrun++) {
    panels[qrun] = new jfHtmlResultViewer(qrun,mainbook);
    mainbook->AddPage(panels[qrun],input->NameAtIndex(qrun));
  }
  // done
  setup = true;
  return true;
}
//---------------------------------------------------------------
bool jfHtmlResultCollection::ClearViewers() {
  const wxString fname = wxT("jfHtmlResultCollection::ClearViewers");
  // variables
  size_t offset;
  // checks and asserts
  /**/jfXLogMessageS(fname,1,num_rcats);
  if (!setup) return false;
  /**/jfXLogMessageB(fname,2, use_default);
  offset = (use_default)?1:0;
  /**/jfXLogMessageS(fname,3,offset);
  // removing the HTML Viewers list
  for(int qrun=(num_rcats-1);qrun>=0;qrun--) {
    mainbook->RemovePage(qrun+offset);
    panels[qrun]->Destroy();
    panels[qrun] = NULL;
  }
  /**/jfXLogMessage(fname,4);
  if (use_default) {
    mainbook->RemovePage(0);
    panels[32]->Destroy();
    panels[32] = NULL;
  }
  /**/jfXLogMessage(fname,5);
  // deleting the data
  delete mgroup;
  mgroup = NULL;
    /**/jfXLogMessage(fname,6);
  // setting thje default values
  setup = false;
    /**/jfXLogMessageB(fname,7,setup);
  use_default = false;
  num_rcats = 0;
  num_items = 0;
  result_label->SetLabel(wxT("No Results"));
    /**/jfXLogMessage(fname,8);
  // done
  return true;

}

//--------------------------------------------------------------------------
size_t jfHtmlResultCollection::SelectedIndex() const {
  size_t sindex;
  sindex = mainbook->GetSelection();
  if (!use_default) return sindex;
  else {
    if (sindex==0) return 32;
    else return (sindex-1);
  }
}
//-------------------------------------------------------------------
bool jfHtmlResultCollection::CheckRCounts(bool uses_default,size_t ch_rcats) const {
  if (use_default!=uses_default) return false;
  return (ch_rcats==num_rcats);
}
//-------------------------------------------------------------------
bool jfHtmlResultCollection::ChangeFicMark(size_t rcat_index, size_t item_index, jf_FicMark fmval) {
  // asserts
  assert(setup);
  if (rcat_index==32) assert(use_default);
  else assert(rcat_index<num_rcats);
  // we return
  mgroup->SetFicMarkHTML(rcat_index,item_index,fmval);
  panels[rcat_index]->RefreshAll();
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfHtmlResultCollection::IncreaseCount(wxCommandEvent& event) {
  size_t inncount = event.GetInt();
  assert(inncount>0);
  assert(setup);
  mgroup->ItemsProcessed(inncount);
}
//---------------------------------------------------------------------
// special events
void jfHtmlResultCollection::OnRClickMsg(wxCommandEvent& event) {
  // local variables
	wxString theurl;
	size_t rcindex,ninindex;
	size_t collindex;
	wxInt32 itemindex;
  // getting the point data
  assert(popup_menu!=NULL);
  rcindex = event.GetInt();
  ninindex = event.GetExtraLong();
  theurl = mgroup->GetUrl(rcindex,ninindex);
  mgroup->ReverseLookup(rcindex,ninindex,collindex,itemindex);
  // setting the menu with this data
  popup_menu->SetIndexes(event.GetString() ,itemindex,collindex);
  popup_menu->SetMoreIndexes(this,rcindex,ninindex);
  // we now show the menu
  popup_menu->UpdateUI();
  PopupMenu(popup_menu);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructor
jfHtmlResultCollection::~jfHtmlResultCollection() {
  if (mgroup!=NULL) delete mgroup;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handles incrementing itemcounts for the listboxes
void jfHtmlResultCollection::ChangeItemCount(size_t index) {
  // const
  const wxString fname = wxT("jfHtmlResultCollection::ChangeItemCount");
  // the usual checks
  assert(setup);
  assert(index<33);
  if (index==32) assert(use_default);
  else assert(index<num_rcats);
  // variables
  size_t lindex;
  size_t newcount;
  // fetching the oldcount
  lindex = panels[index]->GetVisibleBegin();
  newcount = panels[index]->GetItemCount();
  /**/jfXLogMessageS(fname,1,newcount);
  // producing the newcount
  newcount += mgroup->GetNewcount(index);
  /**/jfXLogMessageS(fname,2,newcount);
  // changing
  panels[index]->SetItemCount(newcount);
  panels[index]->RefreshAll();
  panels[index]->ScrollToLine(lindex);
}
//-------------------------------------------------------------------
void jfHtmlResultCollection::ChangeAllCounts() {
  if (use_default) ChangeItemCount(32);
  for (size_t qval = 0;qval<num_rcats;qval++) {
    ChangeItemCount(qval);
  }
}
//-------------------------------------------------------------------
void jfHtmlResultCollection::UpdateResultLabel() {
  wxString labfile;
  if (num_items==0) labfile = wxT("No");
  else labfile = SInt2Str(num_items);
  labfile += wxT(" Results");
  result_label->SetLabel(labfile);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfHtmlResultCollection,wxPanel)
  EVT_COMMAND (-1,jf_HTML_RIGHTCLICK_EVENT,jfHtmlResultCollection::OnRClickMsg )
  EVT_COMMAND (jfID_SRCCAT_PARSER,jf_RESORG_PAGEDONE_EVENT,jfHtmlResultCollection::AddResults )
  EVT_COMMAND (jfID_SRCCAT_PARSER,jf_RESLISTBOX_NEWCAT_EVENT,jfHtmlResultCollection::AddCategory )
  EVT_COMMAND (jfID_SRCCAT_PARSER,jf_PARSER_ITEMSDONE_EVENT,jfHtmlResultCollection::IncreaseCount )
END_EVENT_TABLE()
//****************************************************************************
