/*****************************************************************************
Name:        html_display.h
Author :     John Q Metro
Purpose :    A replacement for HTML listbox for showing results
Created:     April 26, 2011
Updated:     September 21, 2014
******************************************************************************/
#ifndef HTML_DISPLAY_H_INCLUDED
#define HTML_DISPLAY_H_INCLUDED
#endif // HTML_DISPLAY_H_INCLUDED
//----------------------------------------------------------------------
#ifndef DISPLAY_INDEX_H_INCLUDED
  #include "../results/display_index.h"
#endif // DISPLAY_INDEX_H_INCLUDED
#ifndef MENU_H_INCLUDED
  #include "../misc/menu.h"
#endif // MENU_H_INCLUDED
#include <wx/htmllbox.h>
#include <wx/notebook.h>
//****************************************************************************
class jfHtmlResultCollection;
//---------------------------------------------------------------------------
class jfHtmlResultViewer : public wxHtmlListBox {
  public:
    // the constructor
    jfHtmlResultViewer(size_t inresindex, wxWindow* parent, wxWindowID id = wxID_ANY);
    // the html list box method that must be overridden
    virtual wxString OnGetItem(size_t n) const;
    // other events
    void OnRightUp(wxMouseEvent& mevent);
    void OnMouseWheel(wxMouseEvent& mevent);
  protected:
    // status info
    size_t res_index;
    jfHtmlResultCollection* gparent;
    // the event table
    DECLARE_EVENT_TABLE()
};
//***************************************************************************
class jfHtmlResultCollection : public wxPanel{
  public:
    // constructor
    jfHtmlResultCollection(header_enum inhtype, wxWindow* parent, wxWindowID id = wxID_ANY,wxString inname = wxT("jfHtmlResultCollection"));
    // methods accessed by viewer
    wxString GetHTML(size_t rcat_index, size_t item_index) const;
    wxString GetURL(size_t rcat_index, size_t item_index) const;
    // additional methods
    void SetPopupMenu(jfPopupMenu* inmenu);
    bool SetupViewers(jfCategories* input);
    bool ClearViewers();
    size_t SelectedIndex() const;
    bool CheckRCounts(bool uses_default,size_t ch_rcats) const;
    bool ChangeFicMark(size_t rcat_index, size_t item_index, jf_FicMark fmval);
    // special events
    virtual void AddCategory(wxCommandEvent& event) = 0;
    virtual void AddResults(wxCommandEvent& event) = 0;
    void IncreaseCount(wxCommandEvent& event);
    void OnRClickMsg(wxCommandEvent& event);
    // destructor
    ~jfHtmlResultCollection();
  protected:
    // helper methods
    virtual jfDisplayIndexGroup* MakeIndexers() = 0;
    void ChangeItemCount(size_t index);
    void ChangeAllCounts();
    void UpdateResultLabel();
    // GUI elements
    wxNotebook* mainbook;
    jfHtmlResultViewer* panels[33];
    wxBoxSizer* msizer;
    wxBoxSizer* dispbar;
    wxStaticText* result_label;
    // internal flags and counts
    bool setup;
    bool use_default;
    size_t num_rcats;
    size_t num_items;
    header_enum hype;
    // the data
    jfDisplayIndexGroup* mgroup;
    jfPopupMenu* popup_menu;
    // the event table
    DECLARE_EVENT_TABLE()
};
//****************************************************************************
