/*****************************************************************************
Name:        result_panel.cpp
Author :     John Q Metro
Purpose :    Decfines a panel for displaying search results
Created:     July 26, 2009
Updated:     January 8, 2014
******************************************************************************/
#ifndef RESULT_PANEL_H_INCLUDED
  #include "result_panel.h"
#endif // RESULT_LISTBOX_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef EXTRAPANELS_H_INCLUDED
  #include "../threads/extrapanels.h"
#endif // EXTRAPANELS_H_INCLUDED
//*****************************************************************************
//=============================================================================
// constructor
jfResultDisplay::jfResultDisplay(wxWindow* parent,jfMainSearchGroup* toplevel,
      rtype_enum rinval, wxWindowID id,header_enum inval): wxPanel(parent,id,
            wxDefaultPosition, wxDefaultSize,wxTAB_TRAVERSAL,wxT("jfResultDisplay")) {
  // initial checks and assignments
  assert(toplevel!=NULL);
  started = false;
  ssize = false;
  p_menu = NULL;
  // start construction gui elements
  if (rinval==rtype_BASIC) {
    mainviewer = new jfHtmlResultCollection_Base(he_ONECATEGORY,this,-1);
    progpanel = new jfProgressPanelSingle(this,wxT("Single Category Progress Panel"));
  }
  else if (rinval==rtype_MCNU) {
    mainviewer = new jfHtmlResultCollection_Base(he_NONEMPTY,this,-1);
    progpanel = new jfProgressPanelMulti(this,jfmpp_Sideways,wxT("Multi Category Progress Panel"));
  }
  else if (rinval==rtype_FFN) {
    mainviewer = new jfHtmlResultCollection_FFN(this,-1);
    progpanel = new jfProgressPanelFFNStyle(this,jfmpp_Stacked,wxT("FFN Progress Panel"));
  }
  else assert(false);
  // starting the arrangement
  topsizer = new wxBoxSizer(wxVERTICAL);
  topsizer->Add(mainviewer,1,wxALL|wxEXPAND,3);
  topsizer->Add(progpanel,0,wxALL|wxEXPAND,3);
  // finishing
  SetSizer(topsizer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// creating the result displays and returning the links
bool jfResultDisplay::LoadGetLinks(jfSearchCore* results) {
  // local variables
  bool svres;
  // special checks
  if (started) return false;
  if (results==NULL) return false;
  if ((results->categories)==NULL) return false;
  if (!(results->categories->NotEmpty())) return false;
  if (p_menu==NULL) return false;
  // we start...
  started = true;
  svres = mainviewer->SetupViewers(results->categories);
  assert(svres);
  results->cat_results = mainviewer;
  // done
  return true;

}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// functions to return info that has to be passed to the search thread
//---------------------------------------------------------------------------
jfNewPauser* jfResultDisplay::GetPauser() {
  return progpanel->GetPauser();
}
//---------------------------------------------------------------------------
jfProgressPanelBase* jfResultDisplay::GetProgressPanel() {
  return progpanel;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfResultDisplay::SetPopupMenu(jfPopupMenu* inmenu) {
  assert(inmenu!=NULL);
  p_menu = inmenu;
  mainviewer->SetPopupMenu(inmenu);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// we call this when we want to clear old results
void jfResultDisplay::ClearOld() {
  mainviewer->ClearViewers();
  progpanel->Reset();
  started = false;
}
//------------------------------------------------------------------------
// say after pressing a start. we have to halt before actually starting..
void jfResultDisplay::SearchAbort() {
	progpanel->CancelStart();
	started = false;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// returns whether downloading and parsing in in progress
bool jfResultDisplay::InProgress() const {
  return progpanel->InProgress();
}
//---------------------------------------------------------------------------
// orders a halt to any downloads/parsing in progress
bool jfResultDisplay::CStop() {
  return progpanel->OrderStop();
}
//---------------------------------------------------------------------------
size_t jfResultDisplay::SelectedIndex() const {
  return mainviewer->SelectedIndex();
}
//------------------------------------------------------------------------
void jfResultDisplay::WindowResized(size_t x, size_t y) {
  const wxString fname = wxT("jfResultDisplay::WindowResized");
  #ifdef __WXGTK__
    /**/jfXLogMessageB(fname,1,ssize);
    /**/jfXLogMessageS(fname,2,y);
    if (y<640) {
      progpanel->ReduceSize();
      ssize = true;
    }
    else {
      progpanel->UnReduceSize();
      ssize = false;
    }
  #endif
}
//******************************************************************************
