/*****************************************************************************
Name:        typed_display.cpp
Author :     John Q Metro
Purpose :    Typed HTML Listbox results
Created:     April 27, 2011
Updated:     November 30, 2013
******************************************************************************/
#ifndef TYPED_DISPLAY_H_INCLUDED
  #include "typed_display.h"
#endif // TYPED_DISPLAY_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//-------------------------------------------
/*****************************************************************************/
// constructor
jfHtmlResultCollection_Base::jfHtmlResultCollection_Base(header_enum inhtype,
        wxWindow* parent, wxWindowID id):jfHtmlResultCollection(inhtype,parent,id,wxT("jfHtmlResultCollection_Base")) {
  typed_group = NULL;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special events
//-------------------------
void jfHtmlResultCollection_Base::AddCategory(wxCommandEvent& event) {
  assert(setup);
  wxString* cxhtmlval;
  // extracting the cat html
  cxhtmlval = reinterpret_cast<wxString*>(event.GetClientData());
  (dynamic_cast<jfDisplayIndex_Basic*>(mgroup))->AppendCategory(event.GetString(),*cxhtmlval);
  delete cxhtmlval;
}
//-------------------------
void jfHtmlResultCollection_Base::AddResults(wxCommandEvent& event) {
  const wxString fname = wxT("jfHtmlResultCollection_Base::AddResults");
  // variables
  jfResultUnitVector* newresult;
  size_t in_stotal;
  size_t plusval;
  bool ares;
  // checks
  assert(setup);
  // starting
  /**/jfXLogMessage(fname,1);
  newresult = reinterpret_cast<jfResultUnitVector*>(event.GetClientData());
  in_stotal = event.GetInt();
  plusval = newresult->size();
  ares = mgroup->AppendResults(newresult,in_stotal);
  assert(ares);
  ChangeAllCounts();
  num_items += plusval;
  UpdateResultLabel();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods
jfDisplayIndexGroup* jfHtmlResultCollection_Base::MakeIndexers() {
  typed_group = new jfDisplayIndex_Basic(use_default,num_rcats,hype);
  return typed_group;
}
/*****************************************************************************/
// constructor
jfHtmlResultCollection_FFN::jfHtmlResultCollection_FFN(wxWindow* parent, wxWindowID id):
        jfHtmlResultCollection(he_NONEMPTY,parent,id,wxT("jfHtmlResultCollection_FFN")) {
  typed_group = NULL;
  insert_mode = false;
  NullUpdateLabels();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special events
//-------------------------
void jfHtmlResultCollection_FFN::AddCategory(wxCommandEvent& event) {
  // constants
  const wxString fname = wxT("jfHtmlResultCollection_FFN::AddCategory");
  assert(setup);
  wxString cxhtmlval;
  bool ic_res;
  const jfFFNSCategory* catdata;
  // extracting the category data
  /**/jfXLogMessage(fname,1);
  catdata = reinterpret_cast<const jfFFNSCategory*>(event.GetClientData());
  cxhtmlval = event.GetString();
  /**/jfXLogMessage(fname,2,cxhtmlval);
  /**/jfXLogMessage(fname,3,catdata->GetName());
  // we append or insert, depending upon mode
  if (insert_mode) {
    ic_res = typed_group->InsertCategory(catdata,cxhtmlval);
    assert(ic_res);
  }
  else {
    ic_res = typed_group->AppendCategory(catdata,cxhtmlval);
    assert(ic_res);
  }
  /**/jfXLogMessage(fname,4);
}
//-------------------------
void jfHtmlResultCollection_FFN::AddEmptyCategories(wxCommandEvent& event) {
  // constants
  const wxString fname = wxT("jfHtmlResultCollection_FFN::AddCategory");
  // checks
  assert(setup);
  assert(!insert_mode);
  // variables
  bool resval;
  jfBulk_EmptyCatInfo* inbcat;
  // getting
  inbcat = reinterpret_cast<jfBulk_EmptyCatInfo*>(event.GetClientData());
  resval = typed_group->AppendEmptCategories(inbcat);
  delete inbcat;
  assert(resval);
}
//-------------------------
void jfHtmlResultCollection_FFN::AddResults(wxCommandEvent& event) {
  // constants
  const wxString fname = wxT("jfHtmlResultCollection_FFN::AddResults");
  // variables
  jfResultUnitVector* newresult;
  size_t in_stotal;
  bool ic_res;
  size_t plusval;
  // checks
  assert(setup);
  // getting the input values
  /**/jfXLogMessage(fname,1);
  newresult = reinterpret_cast<jfResultUnitVector*>(event.GetClientData());
  in_stotal = event.GetInt();
  plusval = newresult->size();
  /**/jfXLogMessageB(fname,2,insert_mode);
  /**/jfXLogMessageS(fname,3, in_stotal);
  /**/jfXLogMessageS(fname,4,plusval);
  // we insert or append
  if (insert_mode) {
    ic_res = typed_group->InsertResults(newresult,in_stotal);
    assert(ic_res);
  }
  else {
    ic_res = mgroup->AppendResults(newresult,in_stotal);
    assert(ic_res);
  }
  /**/jfXLogMessage(fname,5);
  ChangeAllCounts();
  /**/jfXLogMessage(fname,6);
  if (insert_mode) {
    new_items += plusval;
    UpdateNewLabel();
  }
  else {
    num_items += plusval;
    UpdateResultLabel();
  }
  /**/jfXLogMessage(fname,7);
}
//-----------------------------
void jfHtmlResultCollection_FFN::NextUpdateCategory(wxCommandEvent& event) {
  assert(setup);
  const wxString fname = wxT("jfHtmlResultCollection_FFN::NextUpdateCategory");
  const jfFFNSCategory* catdata;
  /**/jfXLogMessage(fname,1);
  catdata = reinterpret_cast<const jfFFNSCategory*>(event.GetClientData());
  /**/jfXLogMessage(fname,2,catdata->GetName());
  bool nuc_res = typed_group->NextUpdateCat(catdata);
  assert(nuc_res);
  /**/jfXLogMessage(fname,3);
}
//-----------------------------
void jfHtmlResultCollection_FFN::UpdatedFic(wxCommandEvent& event) {
  assert(setup);
  const wxString fname = wxT("jfHtmlResultCollection_FFN::UpdatedFic");
  jfResultUnit* inval;
  int utype;
  // starting...
  /**/jfXLogMessage(fname,1);
  utype = event.GetInt();
  inval = reinterpret_cast<jfResultUnit*>(event.GetClientData());
  bool uf_res = typed_group->UpdateFic(inval);
  assert(uf_res);
  if (use_default) panels[32]->RefreshAll();
  for (size_t qval = 0;qval<num_rcats;qval++) {
    panels[qval]->RefreshAll();
  }
  /**/jfXLogMessage(fname,2);
  if (utype==0) {
    updated_items++;
    ChangeUpdateLabel(true);
  }
  else if (utype==2) {
    missing_items++;
    ChangeUpdateLabel(false);
  }
}
//-----------------------------------------------
void jfHtmlResultCollection_FFN::InsertSignal(wxCommandEvent& event) {
  assert(setup);
  SetInsertMode(true);
  CreateUpdateLabels();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// custom methods
bool jfHtmlResultCollection_FFN::SetInsertMode(bool inmode) {
  if (!setup) return false;
  else {
    insert_mode = inmode;
    return typed_group->SetInsertMode(inmode);
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods
jfDisplayIndexGroup* jfHtmlResultCollection_FFN::MakeIndexers() {
  typed_group = new jfDisplayIndex_FFN(use_default,num_rcats);
  return typed_group;
}
//-----------------------------------------------
void jfHtmlResultCollection_FFN::CreateUpdateLabels() {
  new_label = new wxStaticText(this,-1,wxT("No New Results"));
  updated_label = new wxStaticText(this,-1,wxT("No Updated Items"));
  missing_label = new wxStaticText(this,-1,wxT("No Missing Items"));
  dispbar->Add(new_label,1,wxALL|wxALIGN_CENTER_VERTICAL|wxLEFT,1);
  dispbar->Add(updated_label,1,wxALL|wxALIGN_CENTER_VERTICAL|wxLEFT,1);
  dispbar->Add(missing_label,1,wxALL|wxALIGN_CENTER_VERTICAL|wxLEFT,1);
  Layout();
}
//-----------------------------------------------
void jfHtmlResultCollection_FFN::NullUpdateLabels() {
  new_label = NULL;
  updated_label = NULL;
  missing_label = NULL;
  new_items = 0;
  updated_items = 0;
  missing_items = 0;
}
//-----------------------------------------------------------
void jfHtmlResultCollection_FFN::UpdateNewLabel() {
  wxString labfile;
  if (new_items==0) labfile = wxT("No");
  else labfile = SInt2Str(new_items);
  labfile += wxT(" New Results");
  new_label->SetLabel(labfile);
}
//-----------------------------------------------------------
void jfHtmlResultCollection_FFN::ChangeUpdateLabel(bool updated) {
  // variables
  wxString labfile;
  size_t uitem = (updated)?(updated_items):(missing_items);
  // starting
  if (uitem==0) labfile = wxT("No");
  else labfile = SInt2Str(uitem);
  labfile += (updated)?(wxT(" Updated")):(wxT(" Missing"));
  labfile += wxT(" Items");
  // setting the label
  if (updated) updated_label->SetLabel(labfile);
  else missing_label->SetLabel(labfile);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfHtmlResultCollection_FFN,jfHtmlResultCollection)
  EVT_COMMAND (-1,jf_UPDATE_NEXTCAT_SIG,jfHtmlResultCollection_FFN::NextUpdateCategory )
  EVT_COMMAND (-1,jf_UPDATE_UPDATE_FIC,jfHtmlResultCollection_FFN::UpdatedFic )
  EVT_COMMAND (-1,jf_UPDATE_INSERT_SIG,jfHtmlResultCollection_FFN::InsertSignal )
  EVT_COMMAND (-1,jf_RESLISTBOX_BULKECAT_EVENT,jfHtmlResultCollection_FFN::AddEmptyCategories )
END_EVENT_TABLE()
/*****************************************************************************/
