//***************************************************************************
// Name:        ao3_sexpeditors.cpp
// Author :     John Q Metro
// Purpose :    Simple expression filter editors specific to archiveofourown.org
// Created:     September 8, 2012
// Updated:     March 16, 2014
//***************************************************************************
#ifndef AO3_SEXPEDITORS_H_INCLUDED
  #include "ao3_sexpeditors.h"
#endif // AO3_SEXPEDITORS_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
//-----------------------------------------------
//***************************************************************************
// the default constructor
jfExtraTagFilterEditor::jfExtraTagFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, const jfExtraTagFilter* infilt, wxWindowID id, const wxString&
      name):jfSimpleSFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// creating a filter from a simple expression
jfSimpleExpFilterCore* jfExtraTagFilterEditor::MakeTypedFilter(jfSimpleExpr* inpval) {
  jfExtraTagFilter* result = new jfExtraTagFilter(inpval);
  return result;
}
//===========================================================================
// constructor
jfAO3FandomFilterEditor::jfAO3FandomFilterEditor(wxWindow* parent, const jfFilterMap* infmap, const jfAO3FandomFilter* infilt,
    wxWindowID id, const wxString& name):jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
  wxString omsg;
  assert(infmap!=NULL);
  typed_fpointer = infilt;
  // calling the custom methods
  MakeWidgets();
  ArrangeSizers2();
  // finalizing things
  insert = msizer;
  // calling the sizers
  ArrangeSizers();
  // setting empty values
  if (infilt==NULL) {
    allmatch_box->Enable(false);
    perf_edit->Enable(false);
  }
  // loafing the filter
  else LoadFilter(typed_fpointer);

}
//+++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methods
//-----------------------------------
void jfAO3FandomFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  // variables
  const jfAO3FandomFilter* thefilt;
  wxString omsg;
  bool xvalue;
  // basic preps
  assert(infilter!=NULL);
  thefilt = dynamic_cast<const jfAO3FandomFilter*>(infilter);
  filt_pointer = typed_fpointer = thefilt;
  // loading the name info
  NameLoad();
  // loading the main data
  main_edit->SetData(typed_fpointer->GetMainSrc(),omsg);
  // loading the crossover data
  xvalue = typed_fpointer->GetCrossover();
  crossover_box->SetValue(xvalue);
  allmatch_box->SetValue(typed_fpointer->GetAllMatch());
  perf_edit->SetData(typed_fpointer->GetPerFandomSrc(),omsg);
  // enabling or disabling basic on crossover
  allmatch_box->Enable(xvalue);
  perf_edit->Enable(xvalue);
  // done
}
//-----------------------------------
jfBaseFilter* jfAO3FandomFilterEditor::GetFilter() {
  // variables
  jfSimpleExpr* toutput;
  jfAO3FandomFilter* result;
  wxString omsg;
  bool cvalue;
  // building the result
  result = new jfAO3FandomFilter();
  // adding main expression
  toutput = main_edit->CheckFilter(omsg);
  result->GeneralFromExpr(toutput);
  delete toutput;
  // adding crossover stuff
  cvalue = crossover_box->IsChecked();
  result->SetCrossover(cvalue);
  if (cvalue) {
    result->SetAllMatch(allmatch_box->IsChecked());
    toutput = perf_edit->CheckFilter(omsg);
    result->PerFandomFromExpr(toutput);
    delete toutput;
  }
  // final details
  namedesc_edit->ChangeObj(result);
  // done
  return result;
}
//+++++++++++++++++++++++++++++++++++++++++++
// checking
bool jfAO3FandomFilterEditor::GeneralCheck() const {
  // local variables
  const wxString pfmsg = wxT("Problem with filter contents!");
  jfSimpleExpr* toutput;
  wxString omsg;
  // first we see if the name is correct
  if (NameNUniq()) return false;
  // next, we check the main filter
  toutput = main_edit->CheckFilter(omsg);
  if ((toutput!=NULL) && (toutput->IsValid())) {
    // things are ok
    delete toutput;
    if (crossover_box->IsChecked()) {
      toutput = main_edit->CheckFilter(omsg);
      if ((toutput!=NULL) && (toutput->IsValid())) {
        delete toutput;
        return true;
      }
    }
    else return true;
  }
  // things are not ok
  wxMessageBox(omsg,pfmsg,wxCANCEL|wxICON_ERROR);
  if (toutput!=NULL) delete toutput;
  return false;
}
//+++++++++++++++++++++++++++++++++++++++++++
// event handlers
void jfAO3FandomFilterEditor::HandleCrossoverCheck(wxCommandEvent& event) {
  allmatch_box->Enable(event.IsChecked());
  perf_edit->Enable(event.IsChecked());
  event.Skip();
}
//+++++++++++++++++++++++++++++++++++++++++++
// internal setup methods
//-----------------------------------
void jfAO3FandomFilterEditor::MakeWidgets() {
  // making the checkboxes
  crossover_box = new wxCheckBox(this,jfID_AO3FANFIL_CHECK1,wxT("Must have Multiple Fandoms"));
  allmatch_box = new wxCheckBox(this,jfID_AO3FANFIL_CHECK2,wxT("All Fandoms instead"));
  // making the other stuff
  perf_label = new wxStaticText(this,-1,wxT("At least one fandom must match:"));
  perf_edit = new jfSimpleExprEdit(this,-1,false);
  main_label = new wxStaticText(this,-1,wxT("The list of all the fandoms should match :"));
  main_edit = new jfSimpleExprEdit(this,-1,false);
}
//-----------------------------------
void jfAO3FandomFilterEditor::ArrangeSizers2() {
  csizer = new wxBoxSizer(wxHORIZONTAL);
  msizer = new wxBoxSizer(wxVERTICAL);
  csizer->Add(perf_label,1,wxEXPAND|wxALL,2);
  csizer->Add(allmatch_box,0,wxEXPAND|wxALL,2);
  msizer->Add(crossover_box,0,wxEXPAND|wxALL|wxALIGN_LEFT,2);
  msizer->AddSpacer(3);
  msizer->Add(csizer,0,wxEXPAND|wxALL,1);
  msizer->Add(perf_edit,0,wxEXPAND|wxALL,2);
  msizer->AddSpacer(6);
  msizer->Add(main_label,0,wxEXPAND|wxALL|wxALIGN_LEFT,2);
  msizer->Add(main_edit,1,wxEXPAND|wxALL,2);
}

//+++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfAO3FandomFilterEditor,jfBaseFilterEditor)
  EVT_CHECKBOX(jfID_AO3FANFIL_CHECK1 , jfAO3FandomFilterEditor::HandleCrossoverCheck)
END_EVENT_TABLE()
//===========================================================================
jfAO3KudoFilterEditor::jfAO3KudoFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, const jfBaseFilter* infilt, wxWindowID id, const wxString&
      name):jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
  // we create the insert.. the *actual* expression editor
  insert_panel = new jfMinMaxUPanel(this,-1,NULL);
  if (filt_pointer!=NULL) {
    insert_panel->SetFromObj(dynamic_cast<const jfAO3KudoFilter*>(filt_pointer));
  }
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfAO3KudoFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  const wxString fname = wxT("jfAO3KudoFilterEditor::LoadFilter");
  assert(infilter!=NULL);
  // loading the data
  filt_pointer = infilter;
  insert_panel->SetFromObj(dynamic_cast<const jfAO3KudoFilter*>(filt_pointer));
  NameLoad();
}
//------------------------------------------------------------------------------
jfBaseFilter* jfAO3KudoFilterEditor::GetFilter() {
  jfAO3KudoFilter* toutput;
  // getting the output
  toutput = new jfAO3KudoFilter();
  insert_panel->LoadToObj(toutput);
  namedesc_edit->ChangeObj(toutput);
  // done
  return toutput;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checks if the contents are ok, returns false if they are not
bool jfAO3KudoFilterEditor::GeneralCheck() const {
  return insert_panel->Check();
}

//***************************************************************************
