/******************************************************************************
Name    :   ao3_special2.cpp
Author  :   John Q Metro
Purpose :   Defines some interface panels for some AO3 Filters
Created :   October 6, 2012
Updated :   October 10, 2012
******************************************************************************/
#ifndef AO3_SPECIAL2_H_INCLUDED
  #include "ao3_special2.h"
#endif // AO3_SPECIAL2_H_INCLUDED
#ifndef AO3_SPECIALS2_H_INCLUDED
  #include "../../../core/filters/ao3/ao3_specials2.h"
#endif // AO3_SPECIALS2_H_INCLUDED
/*****************************************************************************/
jfAO3PairingPanel::jfAO3PairingPanel(wxWindow* parent, wxWindowID id, bool orient,
        const wxString& name):wxPanel(parent, id,wxDefaultPosition,wxDefaultSize,
        wxTAB_TRAVERSAL, name) {
  // starting...
  omode = orient;
  // creating GUI elemnts
  altbox = new wxCheckBox(this,-1,wxT("Check if only one pairing needs to match"));
  altbox->SetValue(true);
  pairlistbox = new jfLabeledEdit(this,-1,wxT("Pairings to find"),wxEmptyString,3,(omode)?(wxVERTICAL):(wxHORIZONTAL));
  // arranging
  bsizer = new wxBoxSizer(wxVERTICAL);
  bsizer->Add(altbox,0,wxALL|wxEXPAND,2);
  bsizer->Add(pairlistbox,0,wxALL|wxEXPAND,2);
  // finishing
  SetSizer(bsizer);
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++
// i/o
//-------------------------------------
bool jfAO3PairingPanel::SetFromString(const wxString& inval) {
  pairlistbox->SetValue(inval);
  return true;
}
//-------------------------------------
void jfAO3PairingPanel::SetAlternateCheck(bool cval) {
  altbox->SetValue(cval);
}
//-------------------------------------
wxString jfAO3PairingPanel::GetStringValue() const {
  return pairlistbox->GetValue();
}
//-------------------------------------
bool jfAO3PairingPanel::GetAlternateCheck() const {
  return altbox->IsChecked();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++
// filter based i/o
//-------------------------------------
jfAO3PairFilter* jfAO3PairingPanel::GetFilter() const {
  jfAO3PairFilter* result;
  result = new jfAO3PairFilter();
  if (!result->SetNamesData(pairlistbox->GetValue())) {
    delete result;
    return NULL;
  }
  result->SetAlternates(altbox->IsChecked());
  return result;
}
//-------------------------------------
bool jfAO3PairingPanel::StoreToFilter(jfAO3PairFilter* outval) {
  assert(outval!=NULL);
  if (!outval->SetNamesData(pairlistbox->GetValue())) return false;
  outval->SetAlternates(altbox->IsChecked());
  return true;
}
//-------------------------------------
bool jfAO3PairingPanel::LoadFromFilter(const jfAO3PairFilter* inval) {
  assert(inval!=NULL);
  if (!inval->IsValid()) return false;
  pairlistbox->SetValue(inval->GetNamesData());
  altbox->SetValue(inval->GetAlternate());
  return true;
}
//=============================================================================
jfAO3_PairingFilterEditor::jfAO3_PairingFilterEditor(wxWindow* parent,const jfBaseFilter* infilt,
          const jfFilterMap* infmap, wxWindowID id, const wxString& name):
          jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // we start...
  // we create the insert.. the *actual*  editor
  insert_panel = new jfAO3PairingPanel(this,-1,false);
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
  if (infilt!=NULL) LoadFilter(infilt);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methods
//-------------------------------------
void jfAO3_PairingFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  assert(infilter!=NULL);
  filt_pointer = infilter;
  const jfAO3PairFilter* temp = dynamic_cast<const jfAO3PairFilter*>(infilter);
  insert_panel->LoadFromFilter(temp);
  NameLoad();
}
//-------------------------------------
jfBaseFilter* jfAO3_PairingFilterEditor::GetFilter() {
  jfAO3PairFilter* result = insert_panel->GetFilter();
  namedesc_edit->ChangeObj(result);
  return result;
}
//--------------------------------------------------------------
bool jfAO3_PairingFilterEditor::GeneralCheck() const {
  jfAO3PairFilter* result = insert_panel->GetFilter();
  bool resval = result->IsValid();
  delete result;
  return resval;
}
//=============================================================================
jfAO3_WarnFilterEditor::jfAO3_WarnFilterEditor(wxWindow* parent,const jfBaseFilter* infilt,
          const jfFilterMap* infmap, wxWindowID id, const wxString& name):
          jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // we start...
  // we create the insert.. the *actual*  editor
  insert_panel = new jfCharCheckBoxGroup(this,-1,wxT("Warnings to Exclude"),5,warnlist,warn3_ac,3);
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
  if (infilt!=NULL) LoadFilter(infilt);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methods
//-------------------------------------
void jfAO3_WarnFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  assert(infilter!=NULL);
  filt_pointer = infilter;
  const jfAO3WarnFilter* temp = dynamic_cast<const jfAO3WarnFilter*>(infilter);
  insert_panel->SetFromString(temp->ToString());
  NameLoad();
}
//-------------------------------------
jfBaseFilter* jfAO3_WarnFilterEditor::GetFilter() {
  jfAO3WarnFilter* result = new jfAO3WarnFilter(insert_panel->GetStringValue());
  namedesc_edit->ChangeObj(result);
  return result;
}
//--------------------------------------------------------------
bool jfAO3_WarnFilterEditor::GeneralCheck() const {
  return true;
}
/*****************************************************************************/
