/******************************************************************************
Name    :   bfilt_edit.cpp
Author  :   John Q Metro
Purpose :   Defines some base class for filter diting
Created :   July 28, 2009
Updated :   August 26 2012
******************************************************************************/
#ifndef BFILT_EDIT_H_INCLUDED
  #include "bfilt_edit.h"
#endif // BFILT_EDIT_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
//---------------------------------------------------------------------------
/*****************************************************************************/
// messaages
const wxString pfmsg = wxT("Problem with filter contents!");

//---------------------------------------------------------------------------------
jfBaseFilterEditor::jfBaseFilterEditor(wxWindow* parent, const jfFilterMap* infmap,
      const jfBaseFilter* infilt, wxWindowID id, const wxString& name):wxPanel(
      parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // checks
  assert(infmap!=NULL);
  assert(infilt!=NULL);
  // setting the pointers
  local_fmap = infmap;
  filt_pointer = infilt;
  // we start creating the gui elements
	topsizer = new wxStaticBoxSizer(wxVERTICAL,this,filt_pointer->GetName());
  namedesc_edit = new jfNameDescEditor(this,-1,true,false);
  reload_btn = new wxButton(namedesc_edit,jfID_BASE_RELOAD_BTN,wxT("Reload"));
  namedesc_edit->AddItem(reload_btn);
  namedesc_edit->SetFromObj(filt_pointer);
  /* the rest of the arranging should be done by the arrange sizers method,
  which should be called by the constructor of child classes */
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* changes the filter without loading the contents. Useful, for example,
if you have deleted the original filter and have replaced it by what is
returned by GetFilter()  */
bool jfBaseFilterEditor::ChangeFilter(const jfBaseFilter* infilter) {
  if (infilter==NULL) return false;
  filt_pointer = infilter;
  return true;
}
//--------------------------------------------------------------------------
// calls the virtaul method LoadFilter
void jfBaseFilterEditor::ReloadFilter() {
  LoadFilter(filt_pointer);
}
//-------------------------------------------------------------------------
void jfBaseFilterEditor::ReloadBtnPressed(wxCommandEvent& event) {
  ReloadFilter();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* checks if the name in the name editor is elsewhere in the list, returns
FALSE is this is so */
bool jfBaseFilterEditor::NameCheck() const {
  // starting...
  wxString tname;
  tname = namedesc_edit->TryGetName();
  if (tname==(filt_pointer->GetName())) return true;
  // at this point, the name has been changed, we look for it
  return !(local_fmap->NameExists(tname));
}
//--------------------------------------------------------------------------
void jfBaseFilterEditor::ChangeBoxName(wxString newname) {
  (topsizer->GetStaticBox())->SetLabel(newname);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfBaseFilterEditor::ArrangeSizers() {
  // we run some basic checks
  assert(insert!=NULL);
  // creating the top sizer
  topsizer->Add(namedesc_edit,0,wxEXPAND);
  topsizer->Add(insert,1,wxEXPAND);
  // done..
  SetSizer(topsizer);
}
//----------------------------------------------------------------------------
// returns true if NameCheck() fails after showing an error MessageBox
bool jfBaseFilterEditor::NameNUniq() const {
  if (!NameCheck()) {
    wxMessageBox(wxT("The name has been changed to one that is already in the\
     filter list. Names must be unique!"),pfmsg,wxCANCEL|wxICON_ERROR);
    return true;
  }
  else return false;
}
//----------------------------------------------------------------
void jfBaseFilterEditor::NameLoad() {
  namedesc_edit->SetFromObj(filt_pointer);
  ChangeBoxName(filt_pointer->GetName());
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfBaseFilterEditor,wxPanel)
  EVT_BUTTON(jfID_BASE_RELOAD_BTN,jfBaseFilterEditor::ReloadBtnPressed)
END_EVENT_TABLE()

/*****************************************************************************/
// the default constructor
jfExprFilterEditor::jfExprFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, bool is_inglobal, const jfBaseFilter* infilt, wxWindowID id, const wxString&
      name):jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
  wxString omsg;
  assert(infmap!=NULL);
  is_global = is_inglobal;
  // we create the insert.. the *actual* expression editor
  insert_panel = new jfExpressionEditor(this,-1,const_cast<jfFilterMap*>(infmap),false,true,is_global);
  insert_panel->SetData(filt_pointer->ToString(),omsg);
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfExprFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  assert(infilter!=NULL);
  wxString omsg;
  // loading the data
  filt_pointer = infilter;
  insert_panel->SetData(filt_pointer->ToString(),omsg);
  NameLoad();
}
//------------------------------------------------------------------------------
jfBaseFilter* jfExprFilterEditor::GetFilter() {
  jfExpressionFilter* toutput;
  wxString omsg;
  // getting the output
  toutput = insert_panel->CheckFilter(omsg);
  namedesc_edit->ChangeObj(toutput);
  // done
  return toutput;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checks if the contents are ok, returns false if they are not
bool jfExprFilterEditor::GeneralCheck() const {
  // local variables
  jfExpressionFilter* toutput;
  wxString omsg;
  // first we see if the name is correct
  if (NameNUniq()) return false;
  // next, we check the filter
  toutput = insert_panel->CheckFilter(omsg);
  if (toutput!=NULL) {
    // things are ok
    delete toutput;
    return true;
  }
  else {
    // things are not ok
    wxMessageBox(omsg,pfmsg,wxCANCEL|wxICON_ERROR);
    return false;
  }
}
//****************************************************************************
jfUrlFilterEditor::jfUrlFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, const jfBaseFilter* infilt, wxWindowID id, const wxString&
      name):jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
  // we create the insert.. the *actual* expression editor
  insert_panel = new jfUrlFilEdit(this,-1,true);
  insert_panel->SetData(dynamic_cast<const jfUrlFilter*>(filt_pointer));
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfUrlFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  assert(infilter!=NULL);
  // loading the data
  filt_pointer = infilter;
  insert_panel->SetData(dynamic_cast<const jfUrlFilter*>(filt_pointer));
  NameLoad();
}
//------------------------------------------------------------------------------
jfBaseFilter* jfUrlFilterEditor::GetFilter() {
  jfUrlFilter* toutput;
  // getting the output
  toutput = insert_panel->GetData();
  namedesc_edit->ChangeObj(toutput);
  // done
  return toutput;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checks if the contents are ok, returns false if they are not
bool jfUrlFilterEditor::GeneralCheck() const {
  return true;
}
/*****************************************************************************/
