/******************************************************************************
// Name:        category_editor.cpp
// Author :     John Q Metro
// Purpose :    Defines category editing interface stuff
// Created:     April 3, 2009
// Updated:     January 1, 2011
******************************************************************************/
#ifndef CATEGORY_EDITOR_H_INCLUDED
  #include "category_editor.h"
#endif // CATEGORY_EDITOR_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../../../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED

#include <wx/filename.h>
//*****************************************************************************
const wxString cbmd_base = wxT("The path cannot be made relative to the base directory for results");
//=============================================================================
jfRelativeFilePicker::jfRelativeFilePicker(wxWindow* parent, wxWindowID id,
      jfCategories* catparent,size_t cateindex,bool vertical, const wxString& name):
      wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // constants
  const wxString fname=wxT("jfRelativeFilePicker::jfRelativeFilePicker");
  // assignments
  /**/jfXLogMessage(fname,1);
  assert(catparent!=NULL);
  datalink = catparent;
  if (cateindex!=32) assert(cateindex<(datalink->GetCount()));
  this_index = cateindex;
  /**/jfXLogMessageS(fname,2,this_index);
  // file path picker
  frlabel = new wxStaticText(this,-1,wxT("Result filename and location:"));
  filepath = new wxStaticText(this,-1,datalink->PathAtIndex(this_index));
  /**/jfXLogMessage(fname,3,datalink->PathAtIndex(this_index));
  pickabsolute = new wxCheckBox(this,jfID_CATEGORY_EDITOR_ABSCHK,wxT("Absolute Path"));
  pickabsolute->SetValue(!(datalink->RelativePathAtIndex(this_index)));
  /**/jfXLogMessage(fname,4);
  select_file = new wxButton(this,jfID_CATEGORY_EDITOR_PICKPATH,wxT("Pick Path"));
  /**/jfXLogMessage(fname,5);
	//arraging options
	if (vertical) {
		// creating the top line
		filepath_sizer = new wxBoxSizer(wxHORIZONTAL);
		filepath_sizer->Add(frlabel,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
		filepath_sizer->Add(filepath,1,wxALL|wxALIGN_CENTER_VERTICAL|wxEXPAND,3);
		// the second row sizer
		line2sizer = new wxBoxSizer(wxHORIZONTAL);
		line2sizer->Add(pickabsolute,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
		line2sizer->Add(select_file,0,wxALL,3);
		// making the toplevel sizer
		topsizer = new wxBoxSizer(wxVERTICAL);
		// arranging within the top level
		topsizer->Add(filepath_sizer,0,wxALIGN_TOP|wxALL,1);
		topsizer->Add(line2sizer,0,wxALIGN_TOP|wxALL,3);
	}
	else {
		topsizer = new wxBoxSizer(wxHORIZONTAL);
		topsizer->Add(frlabel,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
		topsizer->Add(filepath,1,wxALL|wxALIGN_CENTER_VERTICAL|wxEXPAND,3);
		topsizer->AddSpacer(6);
		topsizer->Add(pickabsolute,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
		topsizer->Add(select_file,0,wxALL,3);
	}
	/**/jfXLogMessage(fname,6);
  // now, we set the sizer to top and recalc sizes
  SetSizer(topsizer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// a helper method
void jfRelativeFilePicker::ChangeCategoryIndex(size_t newindex) {
  if (newindex!=33) assert(newindex<(datalink->GetCount()));
  this_index = newindex;
}
//-------------------------------------------------------------------------------
wxString jfRelativeFilePicker::GetPath() const {
  return filepath->GetLabel();
}
//-------------------------------------------------------------------------------
bool jfRelativeFilePicker::IsAbsolute() const {
  return pickabsolute->IsChecked();
}
//------------------------------------------------------------------------------
void jfRelativeFilePicker::ReloadNames() {
	filepath->SetLabel(datalink->PathAtIndex(this_index));
	pickabsolute->SetValue(!(datalink->RelativePathAtIndex(this_index)));
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handlers
//---------------------------------------------------------------------------
void jfRelativeFilePicker::ProcessFnameCheck(wxCommandEvent& event) {
  wxFileName fpath;
  bool rvalue;
  // we start
  fpath.Assign(datalink->PathAtIndex(this_index));
  // checked, we convert from relative to absolute
  if (IsAbsolute()) {
    // if try to do this
    rvalue = fpath.MakeAbsolute(datalink->base_outputdir);
    // error checking
    assert(rvalue);
    // things okay, we set the values
    filepath->SetLabel(fpath.GetFullPath());
    datalink->SetFilePath(this_index,fpath.GetFullPath(),false);
  }
  // non checked, we convert from absolute to relative
  else {
    rvalue = fpath.MakeRelativeTo(datalink->base_outputdir);
    // err checking
    if (!rvalue) {
      // we build the result
      wxMessageBox(cbmd_base+wxT(",\n")
        + wxT("since the current path is in a different volume than the base. Sorry."),
        wxT("The operation cannot be done"),wxICON_ERROR | wxOK,this );
      // undoing the check
      pickabsolute->SetValue(true);
    }
    else {
      // we set the value
      filepath->SetLabel(fpath.GetFullPath());
      datalink->SetFilePath(this_index,fpath.GetFullPath(),true);
    }
  }
}
//-------------------------------------------------------------------------------
void jfRelativeFilePicker::ProcessFnameBtn(wxCommandEvent& event) {
  // constants
  const wxString fname = wxT("jfRelativeFilePicker::ProcessFnameBtn");
  const wxString resfilter = wxT("HTML files (*.html;*.htm)|*.html;*.htm");
  // variables
  wxFileDialog* otlocation;
  wxString px_filename, px_fpath, res_fpath;
  wxFileName fpath;
  int dialog_result, edlg_result;
  bool rvalue;
  // trying to get the filename and path to use as the default value
  /**/jfXLogMessage(fname,1,datalink->base_outputdir);
  fpath.Assign(datalink->PathAtIndex(this_index));
  if (!IsAbsolute()) {
    rvalue = fpath.MakeAbsolute(datalink->base_outputdir);
    assert(rvalue);
  }
  px_filename = fpath.GetFullName();
  px_fpath = fpath.GetPath();
  // producing the picker dialog
  otlocation = new wxFileDialog(this,wxT("Choose a result path and filename"),
      px_fpath,px_filename,resfilter,wxFD_SAVE);
      // showing and getting the result
  dialog_result = otlocation->ShowModal();
  if (dialog_result==wxID_OK) {
    res_fpath = otlocation->GetPath();
    // we've got the result, if need be, we make it relative
    if (!(pickabsolute->IsChecked())) {
      // trying to make the path relative
      fpath.Assign(res_fpath);
      rvalue = fpath.MakeRelativeTo(datalink->base_outputdir);
      // if this fails, we ask that the path be made absolute
      if (!rvalue) {
        edlg_result = wxMessageBox(cbmd_base+ wxT(".\n")
            + wxT("Do we just make this path absolute?"),
            wxT("Something must be done"),wxICON_ERROR | wxOK | wxCANCEL,this );
        if (edlg_result==wxOK) {
          filepath->SetLabel(res_fpath);
          datalink->SetFilePath(this_index,res_fpath,false);
          pickabsolute->SetValue(true);
        }
      }
      // if it does not fail, load the values
      else {
        filepath->SetLabel(fpath.GetFullPath());
        datalink->SetFilePath(this_index,fpath.GetFullPath(),true);
      }
    }
    // the path is supposed to be absolute, so we just copy it
    else {
      filepath->SetLabel(res_fpath);
      datalink->SetFilePath(this_index,res_fpath,false);
    }
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfRelativeFilePicker, wxPanel)
  EVT_BUTTON  (jfID_CATEGORY_EDITOR_PICKPATH ,jfRelativeFilePicker::ProcessFnameBtn)
  EVT_CHECKBOX(jfID_CATEGORY_EDITOR_ABSCHK ,  jfRelativeFilePicker::ProcessFnameCheck)
END_EVENT_TABLE()
//=============================================================================
jfCategoryEditor::jfCategoryEditor(wxWindow* parent, wxWindowID id,
      jfCategories* catparent, size_t cateindex, bool multiline, const wxString& name):
      wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // local variables
	const wxChar* fname = wxT("jfCategoryEditor::jfCategoryEditor");
  wxString errout;
  // assignments
  assert(catparent!=NULL);
	/**/jfXLogMessageS(fname,1,cateindex);
  datalink = catparent;
  assert(cateindex<(datalink->GetCount()));
	/**/jfXLogMessageS(fname,2,datalink->GetCount());
  this_index = cateindex;
  currdata = datalink->GetAtIndex(this_index);
	/**/jfXLogMessageS(fname,3,cateindex);
	// making the toplevel sizer
	topsizer = new wxStaticBoxSizer(wxVERTICAL,this,MakeMainLabel());
	/**/jfXLogMessage(fname,4);
  // we start preparing the top part, with name+description editor and delete
  namedesc = new jfNameDescEditor(this,-1,false,false);
  del_btn = new wxButton(namedesc,jfID_CATEGORY_EDITOR_DELETE,wxT("Delete"));
	/**/jfXLogMessage(fname,5);
  namedesc->AddItem(del_btn);
  namedesc->SetFromObj(currdata);
  namedesc->Layout();
	/**/jfXLogMessage(fname,6);
  // expresion editor
  expr_label = new wxStaticText(this,-1,wxT("Sorting Expression"));
  expr_edit = new jfExpressionEditor(this,-1,(datalink->localmap_ptr),true,multiline,false);
	/**/jfXLogMessage(fname,7);
  expr_edit->SetData(currdata->ToString(),errout);
	/**/jfXLogMessageS(fname,8,cateindex);
  // making the filepicker
  filepicker = new jfRelativeFilePicker(this,-1,datalink,this_index,false);
  // arranging within the top level
	/**/jfXLogMessageS(fname,9,cateindex);
  topsizer->Add(namedesc,0,wxALIGN_TOP|wxALL|wxEXPAND,1);
  topsizer->Add(expr_label,0,wxALIGN_TOP|wxALL,3);
  if (multiline) topsizer->Add(expr_edit,1,wxALIGN_TOP|wxLEFT|wxRIGHT|wxBOTTOM|wxEXPAND,3);
  else topsizer->Add(expr_edit,0,wxALIGN_TOP|wxLEFT|wxRIGHT|wxBOTTOM|wxEXPAND,3);
  topsizer->Add(filepicker,0,wxALIGN_TOP|wxALL|wxEXPAND,1);
	/**/jfXLogMessage(fname,10);
  // now, we set the sizer to top and recalc sizes
  SetSizer(topsizer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// changes the index of this editor
void jfCategoryEditor::ChangeCategoryIndex(size_t newindex) {
	const wxChar* fname = wxT("jfCategoryEditor::ChangeCategoryIndex");
  if (newindex!=33) assert(newindex<(datalink->GetCount()));
  this_index = newindex;
  (topsizer->GetStaticBox())->SetLabel(MakeMainLabel());
  filepicker->ChangeCategoryIndex(newindex);
}
//---------------------------------------------------------------------
// tries to save the contents of the widgets to the jfCategroies object
bool jfCategoryEditor::SaveData(wxString& outmessage){
  // local variables
  jfExpressionFilter* expr_result;
  wxString qname;
  // what we do is call the embedded expression editor to convert it's contents
  expr_result = expr_edit->CheckFilter(outmessage);
  // if wrong, we the error message is passed out and we fail
  if (expr_result == NULL) return false;
  // otherwise, we save the name and description...
  qname = namedesc->TryGetName();
  expr_result->SetName(qname);
  expr_result->SetDescription(namedesc->GetDesc());
  // and then copy over the result
  datalink->ReplaceAtIndex(this_index,expr_result);
	currdata = expr_result;
  // done
  return true;
}
//-------------------------------------------------------------------------
// creates the main label for the editor
wxString jfCategoryEditor::MakeMainLabel() const {
  wxString result;
	const wxChar* fname=wxT("jfCategoryEditor::MakeMainLabel");
  result = wxT("Category  ") + SInt2Str(this_index);
  result += wxT(": ");
  result += GetExpName();
  return result;
}
//-------------------------------------------------------------------------
wxString jfCategoryEditor::GetExpName() const {
	return currdata->GetName();
}
//-------------------------------------------------------------------------
wxString jfCategoryEditor::GetExpStr() const {
	return currdata->ToString();
}
//-------------------------------------------------------------------------
void jfCategoryEditor::UnEnable(bool doit) {
  del_btn->Enable(!doit);
  expr_edit->Enable(!doit);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfCategoryEditor::ProcessDeleteBtn(wxCommandEvent& event) {
  /* we actually use the parent to handle most of this, but we need to
    identify the index */
    event.SetInt(this_index);
    event.Skip();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfCategoryEditor, wxPanel)
  EVT_BUTTON  (jfID_CATEGORY_EDITOR_DELETE ,  jfCategoryEditor::ProcessDeleteBtn)
END_EVENT_TABLE()

//*****************************************************************************/

