/******************************************************************************
// Name:        category_editor.h
// Author :     John Q Metro
// Purpose :    Declares category editing interface stuff
// Created:     April 1, 2009
// Updated:     October 29, 2011
******************************************************************************/
#ifndef CATEGORY_EDITOR_H_INCLUDED
#define CATEGORY_EDITOR_H_INCLUDED
#endif // CATEGORY_EDITOR_H_INCLUDED
//-----------------------------------------------------------------------------
#ifndef WIDGETS1_H_INCLUDED
  #include "../../misc/widgets1.h"
#endif // WIDGETS1_H_INCLUDED
#ifndef EXPR_EDITOR_H_INCLUDED
  #include "expr_editor.h"
#endif // EXPR_EDITOR_H_INCLUDED
#ifndef CATEGORIES_H_INCLUDED
  #include "../../../core/structs/categories.h"
#endif // CATEGORIES_H_INCLUDED
//****************************************************************************
/* for filename setting, we have a special panel. We use the standard file
picker dialog (in save mode) to pick where to store the result file. By default
(unless the Absolute Path checkbox is selected) this path is then converted into
a path relative to the base stored in jfCategories. This is to allow changing
the location where all the results will be stores by just changing the base.
*/
class jfRelativeFilePicker : public wxPanel {
  public:
    // the default
    jfRelativeFilePicker(wxWindow* parent, wxWindowID id, jfCategories* catparent,
        size_t cateindex, bool vertical=true, const wxString& name = wxT("jfRelativeFilePicker"));
    // a helper method
    void ChangeCategoryIndex(size_t newindex);
    wxString GetPath() const;
    bool IsAbsolute() const;
		void ReloadNames();
    // event handlers
    void ProcessFnameCheck(wxCommandEvent& event);
    void ProcessFnameBtn(wxCommandEvent& event);
  protected:
    // local widgets
    wxStaticText *frlabel;
    wxStaticText *filepath;
    wxCheckBox *pickabsolute;
    wxButton *select_file;
    // sizers
    wxBoxSizer *filepath_sizer;
    wxBoxSizer *line2sizer;
    wxBoxSizer *topsizer;
    // referenced data
    jfCategories* datalink;
    size_t this_index;
    // the event table declaration
    DECLARE_EVENT_TABLE()

};
//****************************************************************************
/* the category ediotor has, arranged top to bottom:
-an editor for the category name and a delete button
- a description editor
- the expression editor (jfExpressionEditor)
- file name setting.

For the filename/path, it is either absolute or relative to an extenally
proveded path. This is to allow the results to be placed particular directories
which can be relative to a base.

*/
class jfCategoryEditor : public wxPanel {
  public:
    // the default constructor
    jfCategoryEditor(wxWindow* parent, wxWindowID id, jfCategories* catparent,
        size_t cateindex, bool multiline, const wxString& name = wxT("jfCategoryEditor"));
    // helper methods
    void ChangeCategoryIndex(size_t newindex);
    bool SaveData(wxString& outmessage);
    wxString MakeMainLabel() const;
		wxString GetExpName() const;
		wxString GetExpStr() const;
		void UnEnable(bool doit = true);
    // the event handlers
    void ProcessDeleteBtn(wxCommandEvent& event);
  protected:
    // top parts widgets
    jfNameDescEditor* namedesc;
    wxButton* del_btn;
    // the expression editor
    wxStaticText* expr_label;
    jfExpressionEditor* expr_edit;
    // the filepicker
    jfRelativeFilePicker* filepicker;
    // top-level sizer
    wxStaticBoxSizer *topsizer;
    // internal data
    jfCategories* datalink;
    jfExpressionFilter* currdata;
    size_t this_index;

    DECLARE_EVENT_TABLE()

};

//****************************************************************************
