/******************************************************************************
// Name:        category_listedit.cpp
// Author :     John Q Metro
// Purpose :    Split off from category_editor, handles multipl category edits
// Created:     October 23, 2009
// Updated:     January 1, 2011
******************************************************************************/
#ifndef CATEGORY_LISTEDIT_H_INCLUDED
  #include "category_listedit.h"
#endif // CATEGORY_LISTEDIT_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//==============================================================================
// the defult constructor
jfCategoryEditorsList::jfCategoryEditorsList(jfCategories* in_datalink,
    wxWindow* parent, wxWindowID id,const wxString& name):wxScrolledWindow(parent,id,
    wxDefaultPosition,wxDefaultSize,wxVSCROLL,name) {
  // local variables
  size_t ccount,cloop,ncloop;
  // first, we check and assign the categories data object
  assert(in_datalink!=NULL);
  datalink = in_datalink;
  // we create the top-level sizer and set it
  cat_box = new wxBoxSizer(wxVERTICAL);
  SetSizer(cat_box);
  // we now load the categories
  ccount = datalink->GetCount();
  for (cloop=0;cloop<ccount;cloop++) {
    catedits[cloop] = new jfCategoryEditor(this,-1,datalink,cloop,false);
    AddCategory(catedits[cloop]);
  }
  // for the remainder, we set null
  for (ncloop=ccount;ncloop<32;ncloop++) catedits[ncloop] = NULL;
  // done
  SetScrollRate(0,5);
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods
//----------------------------------------------------------------------------
// we delete a category
bool jfCategoryEditorsList::DeleteCategory(size_t cindex) {
  size_t ccount,cloop;
  // starting up...
  ccount = datalink->GetCount();
  assert(cindex<ccount);
  // we delete the physical data object
  datalink->DeleteAtIndex(cindex);
  // we then remove and destroy associated editor
  cat_box->Detach(cindex);
  catedits[cindex]->Destroy();
  // we re-assign the indexes for categories after the deleted one
  for (cloop=(cindex+1);cloop<ccount;cloop++) {
    catedits[cloop-1] = catedits[cloop];
    catedits[cloop-1]->ChangeCategoryIndex(cloop-1);
  }
  catedits[ccount-1] = NULL;
  // redrawing
  cat_box->Layout();
  SetSizer(cat_box);
  FitInside();
  // done
  return true;
}
//----------------------------------------------------------------------------
// we try to save all of the data in the editors to the jfCategories object
bool jfCategoryEditorsList::SaveAll() {
  const wxString verif_error = wxT("Error in ");
  // local variables
  size_t ccount,cloop;
  bool tres;
  wxString err_string, outstring;
  // starting
  ccount = datalink->GetCount();
  // the loop
  for (cloop=0;cloop<ccount;cloop++) {
    tres = catedits[cloop]->SaveData(err_string);
    if (!tres) {
      // we create and show an error message
      outstring = verif_error + catedits[cloop]->MakeMainLabel() + wxT(":\n");
      outstring += err_string;
      wxMessageBox(outstring,wxT("Expression Error!"),wxICON_ERROR |wxOK,this);
      return false;
    }
  }
  return true;
}
//----------------------------------------------------------------------------
// a category has been added: we make a new editor for it and add it
bool jfCategoryEditorsList::CategoryAdded() {
  size_t ccount;
  wxSize isize,qsize;
  // we start
  ccount = datalink->GetCount();
  assert(catedits[ccount-1]==NULL);
  // making the category
  catedits[ccount-1] = new jfCategoryEditor(this,-1,datalink,ccount-1,false);
  // adding it to the sizer
  AddCategory(catedits[ccount-1]);
  EnableScrolling(true,false);
  SetSizer(cat_box);
  FitInside();
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handlers
void jfCategoryEditorsList::ProcessDeleteBtn(wxCommandEvent& event) {
  DeleteCategory(event.GetInt());
  event.Skip();
}
//--------------------------------------------------------------------------
void jfCategoryEditorsList::OnSize(wxSizeEvent& event) {
  // local variables
  size_t ccount,cloop;
  wxSize isize,qsize;
  // starting
  ccount = datalink->GetCount();
  Layout();
  // the resizing loop
  for (cloop=0;cloop<ccount;cloop++) {
    ResizeCategory(catedits[cloop]);
  }
  // done
  Layout();
  event.Skip();
}
//--------------------------------------------------------------------
void jfCategoryEditorsList::OnScroll(wxScrollEvent& event) {
  wxClientDC dc(this);
  DoPrepareDC(dc);
  Update();
  event.Skip();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfCategoryEditorsList::ResizeCategory(jfCategoryEditor* inedit) {
  // local variables
  wxSize isize,qsize;
  int xwidth, xheight;
  // checking
  assert(inedit!=NULL);
  // size modification
  isize = inedit->GetClientSize();
  qsize = GetClientSize();
  xheight = isize.GetHeight();
  xwidth = qsize.GetWidth();
  cat_box->SetItemMinSize(inedit,xwidth-20,xheight);
}
//---------------------------------------------------------------------------
void jfCategoryEditorsList::AddCategory(jfCategoryEditor* inedit) {
  assert(inedit!=NULL);
  // adding sizes
  cat_box->Add(inedit,0,wxALL,2);
  Layout();
  // manipulating the size
  ResizeCategory(inedit);
  Layout();
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfCategoryEditorsList,wxScrolledWindow)
  EVT_BUTTON  (jfID_CATEGORY_EDITOR_DELETE ,jfCategoryEditorsList::ProcessDeleteBtn)
  EVT_SIZE    (jfCategoryEditorsList::OnSize)
  EVT_SCROLL  (jfCategoryEditorsList::OnScroll)
END_EVENT_TABLE()
//============================================================================
// the constructor
jfCategoryEditorsNotebook::jfCategoryEditorsNotebook(jfCategories* in_datalink,
        wxWindow* parent, wxWindowID id,const wxString& name):wxPanel(parent,
        id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // we start
	const wxChar* fname = wxT("jfCategoryEditorsNotebook::jfCategoryEditorsNotebook");
  // local variables
  size_t ccount,cloop,ncloop;
  // setting the default values
  defcat_proxy = NULL;
  incl_default = false;
  // first, we check and assign the categories data object
	jfXLogMessage(fname,1);
  assert(in_datalink!=NULL);
  datalink = in_datalink;
	jfXLogMessage(fname,2);
  // we create the top-level sizer and set it
  mainsizer = new wxBoxSizer(wxHORIZONTAL);
  mainbook = new wxNotebook(this,-1,wxDefaultPosition,wxDefaultSize,wxNB_TOP);
	jfXLogMessage(fname,3);
  mainsizer->Add(mainbook,1,wxALL|wxEXPAND,3);
	jfXLogMessage(fname,4);
  // we now load the categories
  ccount = datalink->GetCount();
	jfXLogMessage(fname,5);
  for (cloop=0;cloop<ccount;cloop++) {
		jfXLogMessageS(fname,6,cloop);
    catedits[cloop] = new jfCategoryEditor(mainbook,-1,datalink,cloop,true);
		jfXLogMessageS(fname,7,cloop);
    mainbook->AddPage(catedits[cloop],datalink->NameAtIndex(cloop));
		jfXLogMessageS(fname,8,cloop);
  }
	jfXLogMessage(fname,9);
  // for the remainder, we set null
  for (ncloop=ccount;ncloop<32;ncloop++) catedits[ncloop] = NULL;
	jfXLogMessage(fname,10);
  // done
  Layout();
	jfXLogMessage(fname,11);
  SetSizer(mainsizer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods
//---------------------------------------------------------------------------
bool jfCategoryEditorsNotebook::DeleteCategory(size_t cindex) {
  size_t ccount,cloop;
	const wxChar* fname = wxT("jfCategoryEditorsNotebook::DeleteCategory");
  // starting up...
  ccount = datalink->GetCount();
  assert(cindex<ccount);
  // we then remove and destroy associated editor
  mainbook->RemovePage(GIndex(cindex));
  // we delete the physical data object
  datalink->DeleteAtIndex(cindex);
  catedits[cindex]->Destroy();
  // we re-assign the indexes for categories after the deleted one
  for (cloop=(cindex+1);cloop<ccount;cloop++) {
    catedits[cloop-1] = catedits[cloop];
    catedits[cloop-1]->ChangeCategoryIndex(cloop-1);
  }
  catedits[ccount-1] = NULL;
  // redrawing
  // done
  return true;
}
//---------------------------------------------------------------------------
bool jfCategoryEditorsNotebook::SaveAll() {
  const wxString verif_error = wxT("Error in ");
  const wxString fname = wxT("jfCategoryEditorsNotebook::SaveAll");
  // local variables
  size_t ccount,cloop;
  bool tres;
  wxString err_string, outstring;
  // starting
  ccount = datalink->GetCount();
  // the loop
  for (cloop=0;cloop<ccount;cloop++) {
    tres = catedits[cloop]->SaveData(err_string);
    /**/jfXLogMessageB(fname,1,tres);
    if (!tres) {
      // we create and show an error message
      outstring = verif_error + catedits[cloop]->MakeMainLabel() + wxT(":\n");
      outstring += err_string;
      wxMessageBox(outstring,wxT("Expression Error!"),wxICON_ERROR |wxOK,this);
      return false;
    }
  }
  return true;
}
//---------------------------------------------------------------------------
bool jfCategoryEditorsNotebook::CategoryAdded() {
	// constants
	const wxString fname = wxT("jfCategoryEditorsNotebook::CategoryAdded");
	// variables
  size_t ccount;
  wxSize isize,qsize;
  // we start
	/**/jfXLogMessage(fname,1);
  ccount = datalink->GetCount();
  assert(catedits[ccount-1]==NULL);
	/**/jfXLogMessageS(fname,2,ccount-1);
  // making the category
  catedits[ccount-1] = new jfCategoryEditor(mainbook,-1,datalink,ccount-1,true);
	/**/jfXLogMessage(fname,3);
  // adding it to the sizer
  mainbook->AddPage(catedits[ccount-1],datalink->NameAtIndex(ccount-1));
	/**/jfXLogMessage(fname,4);
  Refresh();
	/**/jfXLogMessage(fname,5);
  return true;
}
//------------------------------------------------------------------------
void jfCategoryEditorsNotebook::UnEnable(bool doit) {
  size_t cloop,ccount;
  ccount = datalink->GetCount();
  // the loop
  for (cloop=0;cloop<ccount;cloop++) {
    catedits[cloop]->UnEnable(doit);
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// change default
//----------------------------------
bool jfCategoryEditorsNotebook::SetToInclDefault(jfDefaultCategoryEditor* in_defcatproxy) {
  // some tests
  if (incl_default) return false;
  if (in_defcatproxy==NULL) return false;
  // going past the tests
  defcat_proxy = in_defcatproxy;
  incl_default = true;
  // adding to the notebook
  mainbook->InsertPage(0,in_defcatproxy,wxT("Default"));
  // done
  return true;
}
//----------------------------------
bool jfCategoryEditorsNotebook::SetToExclDefault() {
  if (!incl_default) return false;
  // doing
    mainbook->RemovePage(0);
    defcat_proxy = NULL;
    incl_default = false;
    return false;
}
//------------------------------------------------------
wxNotebook* jfCategoryEditorsNotebook::GetNotebookPointer() const {
  return mainbook;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handlers
void jfCategoryEditorsNotebook::ProcessDeleteBtn(wxCommandEvent& event) {
	jfXLogMessage(wxT("jfCategoryEditorsNotebook::ProcessDeleteBtn"),event.GetInt());
  DeleteCategory(event.GetInt());
	jfXLogMessage(wxT("jfCategoryEditorsNotebook::ProcessDeleteBtn"),2);
  event.Skip();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// no checking done here
size_t jfCategoryEditorsNotebook::GIndex(size_t inindex) const {
  return (incl_default)?(inindex+1):(inindex);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfCategoryEditorsNotebook,wxPanel)
  EVT_BUTTON  (jfID_CATEGORY_EDITOR_DELETE ,jfCategoryEditorsNotebook::ProcessDeleteBtn)
END_EVENT_TABLE()
//============================================================================
// the constructor
jfDefaultCategoryEditor::jfDefaultCategoryEditor(wxWindow* parent, jfCategories* in_datalink, wxWindowID id,
        const wxString& name):wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // setting the links
  assert(in_datalink!=NULL);
  datalink = in_datalink;
	// making the sizer for the default category
	defaultszier = new wxStaticBoxSizer(wxVERTICAL,this,wxT("Default Catgeory"));
  // name and description editor
  ndesc_edit = new jfNameDescEditor(this,-1,false,false,-1,datalink->GetName(),datalink->GetDescription());
  // making the filepicker
  filepicker = new jfRelativeFilePicker(this,-1,datalink,32,false);
  // arranging within the top level
  defaultszier->Add(ndesc_edit,0,wxALIGN_TOP|wxALL|wxEXPAND,1);
  defaultszier->Add(filepicker,0,wxALIGN_TOP|wxALL|wxEXPAND,1);
  // by default, the default catgeory is disbaled
  use_default = new wxCheckBox(this,jfID_CATEGORIES_EDITOR_UDCHKBOX,wxT("Use remainder as an extra Category"));
  SetDefaultEnable(false);
  // finishing sizers
  mainsizer = new wxBoxSizer(wxVERTICAL);
  mainsizer->Add(use_default,0,wxALL|wxEXPAND,3);
  mainsizer->Add(defaultszier,1,wxALL|wxEXPAND,3);
  // value setting
  SetDefaultEnable(datalink->UsesDefault());
  use_default->SetValue(datalink->UsesDefault());
  // last
  SetSizer(mainsizer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// methods
//---------------------------------------------------
void jfDefaultCategoryEditor::SetDefaultEnable(bool xenable) {
  ndesc_edit->Enable(xenable);
  filepicker->Enable(xenable);
}
//---------------------------------------------------
void jfDefaultCategoryEditor::SaveDefault() {
  datalink->SetName(ndesc_edit->TryGetName());
  datalink->SetDescription(ndesc_edit->GetDesc());
  datalink->SetUseDefault(use_default->IsChecked());
}
//------------------------------------------------------
void jfDefaultCategoryEditor::EnableCheck(bool value) {
  use_default->Enable(value);
}
//---------------------------------------------------------
void jfDefaultCategoryEditor::ReloadNames() {
	ndesc_edit->SetFromObj(datalink);
	filepicker->ReloadNames();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// events
void jfDefaultCategoryEditor::ProcessCheckDefault(wxCommandEvent& event) {
  if (!(use_default->IsChecked())) {
    SaveDefault();
    SetDefaultEnable(false);
  }
  else SetDefaultEnable(true);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfDefaultCategoryEditor, wxPanel)
  EVT_CHECKBOX(jfID_CATEGORIES_EDITOR_UDCHKBOX, jfDefaultCategoryEditor::ProcessCheckDefault)
END_EVENT_TABLE()
//=============================================================================
// the constructor for the top-level catgeories editing panel
jfMainCategoriesEditor::jfMainCategoriesEditor(wxWindow* parent,
          jfCategories* in_datalink, wxWindowID id, const wxString& name):
          wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // constants
	const wxChar* fname=wxT("jfMainCategoriesEditor::jfMainCategoriesEditor");

	// checking the datalink
	jfXLogMessage(fname,1);
  assert(in_datalink!=NULL);
  datalink = in_datalink;
	jfXLogMessage(fname,2);
	vertical = true;
  // next up, we have the box which contains the proper category editors
		categories_sizer = new wxStaticBoxSizer(wxVERTICAL,this,wxT("Category Editing List"));
		// the default category editor
		dfcat_panel = new jfDefaultCategoryEditor(this,datalink);
		jfXLogMessage(fname,3);
    // making the widgets
    mainlist_panel = new jfCategoryEditorsNotebook(datalink,this);
    addcat_btn = new wxButton(this,jfID_CATEGORIES_EDITOR_ADDCAT,wxT("Add Category"));
		jfXLogMessage(fname,4);
    // the sizer, and arranging
    categories_sizer->Add(addcat_btn,0,wxALL|wxALIGN_TOP,3);
    categories_sizer->Add(mainlist_panel,1,wxALL|wxEXPAND,3);
		jfXLogMessage(fname,5);
  // making the explanantory labels that go at the top
    wxString desclabel = wxT("Categories are used to sort the results of a ");
    desclabel += wxT("search after filtering) into multiple groups, each of");
    desclabel += wxT(" which has a separate results display window and ");
    desclabel += wxT("results file. An expression  for each category is used");
    desclabel += wxT(" to match results to determine if it belongs to that ");
    desclabel += wxT("that catgory. Individual results can belong to more ");
    desclabel += wxT("than 1 category. A maximum of 32 categories are ");
    desclabel += wxT("allowed, but you can treat whatever might be left over");
    desclabel += wxT(" as another category.");
		/**/jfXLogMessage(fname,6);
    mainlabel = new wxStaticText(this,-1,wxT("Category Editing"));
    maindesc = new wxStaticText(this,-1,desclabel);
		/**/jfXLogMessage(fname,7);
  // making the toplevel sizer and arrangine things
    topsizer = new wxBoxSizer(wxVERTICAL);
    topsizer->Add(mainlabel,0,wxALL|wxALIGN_TOP,3);
    topsizer->Add(maindesc,0,wxALL|wxALIGN_TOP,3);
    topsizer->Add(dfcat_panel,0,wxALL|wxALIGN_TOP|wxEXPAND,1);
    topsizer->Add(categories_sizer,1,wxALL|wxEXPAND,3);
		/**/jfXLogMessage(fname,8);
    SetSizer(topsizer);
		/**/jfXLogMessage(fname,10);
    Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfMainCategoriesEditor::SaveCategories() {
  dfcat_panel->SaveDefault();
  return mainlist_panel->SaveAll();
}
//---------------------------------------------------------------------------
void jfMainCategoriesEditor::SetUnEditable(bool unedit) {
  dfcat_panel->EnableCheck(!unedit);
  addcat_btn->Enable(!unedit);
  mainlist_panel->UnEnable(unedit);
}
//--------------------------------------------------------------------------
bool jfMainCategoriesEditor::ChangeSearchName(wxString sname_in) {
  // constants and asserts
  const wxString fname = wxT("jfMainCategoriesEditor::ChangeSearchName");
	assert(datalink!=NULL);
	// changing the data
	/**/jfXLogMessage(fname,1);
	bool rval = datalink->ChangeDefaultFName(sname_in);
	if (!rval) return false;
	/**/jfXLogMessage(fname,2);
	// updating the views
	dfcat_panel->ReloadNames();
	/**/jfXLogMessage(fname,3);
	// done
	return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special methods
//--------------------------------------------------------------------------
/* we have 2 configurations here, vertical, where the default category info is
in a separate panel above the notebook of category editors, and horizontal, where
the editor for the default category is the first tab in the notebook */
bool jfMainCategoriesEditor::ChangeToHorizontal() {
  if (!vertical) return false;
  // creating the new default panel
  jfDefaultCategoryEditor* newedit,*oldedit;
  newedit = new jfDefaultCategoryEditor(mainlist_panel->GetNotebookPointer(),datalink);
  mainlist_panel->SetToInclDefault(newedit);
  // removing the old panel
  oldedit = dfcat_panel;
  topsizer->Detach(dfcat_panel);
  dfcat_panel = newedit;
  // destroying the old panel
  oldedit->Destroy();
  // finishing
  vertical = false;
  Layout();
  return true;
}
//--------------------------------------------------------------------------
bool jfMainCategoriesEditor::ChangeToVertical() {
  if (vertical) return false;
  // creating the new panel
  jfDefaultCategoryEditor* newedit,*oldedit;
  newedit = new jfDefaultCategoryEditor(this,datalink);
  // removing the old panel
  oldedit = dfcat_panel;
  mainlist_panel->SetToExclDefault();
  // inserting the new panel
  dfcat_panel = newedit;
  topsizer->Insert(2,dfcat_panel,0,wxALL|wxALIGN_TOP|wxEXPAND,1);
  // destroying the old panel
  oldedit->Destroy();
  // finishing
  vertical = true;
  Layout();
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handlers
//----------------------------------------------------------------------------
/* most of deletion is handles at the categories editor list, but we do have
this here since the Add Button can be disabled if the number of categories is
at 32 */
void jfMainCategoriesEditor::ProcessDeleteBtn(wxCommandEvent& event) {
  addcat_btn->Enable();
}
//----------------------------------------------------------------------------
void jfMainCategoriesEditor::ProcessAddBtn(wxCommandEvent& event) {
	//
  const wxString pen=wxT("Please enter a non-blank name for the Category");
  const wxString bnana=wxT("Blank names are not allowed!\n");
  const wxString msgcap = wxT("Catalog Name Entry");
  wxString result;
  // getting the name from the user
  result = wxGetTextFromUser(pen,msgcap,wxT("New Category"),this);
  if (!result.IsEmpty()) {
		// now, we add the category to the collections list
		datalink->AddNewCategory(result);
		// and we tell the categories editing list to add it
		mainlist_panel->CategoryAdded();
		// also, we check if we have reached the category max size
		if ((datalink->GetCount())==32) addcat_btn->Enable(false);
	}
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfMainCategoriesEditor, wxPanel)
  EVT_BUTTON  (jfID_CATEGORY_EDITOR_DELETE ,jfMainCategoriesEditor::ProcessDeleteBtn)
  EVT_BUTTON  (jfID_CATEGORIES_EDITOR_ADDCAT ,jfMainCategoriesEditor::ProcessAddBtn)
END_EVENT_TABLE()
//*****************************************************************************
