/******************************************************************************
// Name:        category_listedit.h
// Author :     John Q Metro
// Purpose :    Split off from category_editor, handles multipl category edits
// Created:     October 23, 2009
// Updated:     November 17, 2011
******************************************************************************/

#ifndef CATEGORY_LISTEDIT_H_INCLUDED
#define CATEGORY_LISTEDIT_H_INCLUDED
#endif // CATEGORY_LISTEDIT_H_INCLUDED
#ifndef CATEGORY_EDITOR_H_INCLUDED
  #include "category_editor.h"
#endif // CATEGORY_EDITOR_H_INCLUDED

#include <wx/notebook.h>
//===========================================================================
/* next up we have the scrolled window implementation that is supposed to
hold between 0 and 32 category editors. no extra widgets but multiple methods
for handling things */
class jfCategoryEditorsList : public wxScrolledWindow {
  public:
    // the default constructor
    jfCategoryEditorsList(jfCategories* in_datalink, wxWindow* parent,
          wxWindowID id = -1,const wxString& name = wxT("jfCategoryEditorsList"));
    // helper methods
    bool DeleteCategory(size_t cindex);
    bool SaveAll();
    bool CategoryAdded();
    // event handlers
    void ProcessDeleteBtn(wxCommandEvent& event);
    void OnSize(wxSizeEvent& event);
    void OnScroll(wxScrollEvent& event);
  protected:
    void ResizeCategory(jfCategoryEditor* inedit);
    void AddCategory(jfCategoryEditor* inedit);
    // gui elements
    wxBoxSizer* cat_box;
    jfCategoryEditor* catedits[32];
    // internal data
    jfCategories* datalink;

    DECLARE_EVENT_TABLE()
};
//=========================================================================
class jfDefaultCategoryEditor;
/* I'm having definite problems with getting jfCategoryEditorsList to scroll,
so time the try the notebook verion! */
class jfCategoryEditorsNotebook : public wxPanel {
  public:
    jfCategoryEditorsNotebook(jfCategories* in_datalink, wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& name = wxT("jfCategoryEditorsNotebook"));
    // helper methods
    bool DeleteCategory(size_t cindex);
    bool SaveAll();
    bool CategoryAdded();
    void UnEnable(bool doit=true);
    // change default
    bool SetToInclDefault(jfDefaultCategoryEditor* in_defcatproxy);
    bool SetToExclDefault();
    wxNotebook* GetNotebookPointer() const;
    // event handlers
    void ProcessDeleteBtn(wxCommandEvent& event);
  protected:
    // helper methods
    size_t GIndex(size_t inindex) const;
    // gui elements
    wxNotebook* mainbook;
    jfCategoryEditor* catedits[32];
    wxBoxSizer* mainsizer;
    // default (if available)
    bool incl_default;
    jfDefaultCategoryEditor* defcat_proxy;
    // internal elements
    jfCategories* datalink;

    DECLARE_EVENT_TABLE()

};
//==========================================================================
/* Panel with controls to edit the default category. I've separated this one
out because as it takes up too much space on small screens */
class jfDefaultCategoryEditor : public wxPanel {
  public:
    // the constructor
    jfDefaultCategoryEditor(wxWindow* parent, jfCategories* in_datalink, wxWindowID id = wxID_ANY, const wxString& name = wxT("jfDefaultCategoryEditor"));
    // methods
    void SetDefaultEnable(bool xenable=true);
    void SaveDefault();
    void EnableCheck(bool value=true);
    void ReloadNames();
    // events
    void ProcessCheckDefault(wxCommandEvent& event);

  protected:
    // internal methods
    // gui controls for default stuff
      wxCheckBox* use_default;
      wxStaticBoxSizer* defaultszier;
      wxBoxSizer* mainsizer;
      // name and description editor
      jfNameDescEditor* ndesc_edit;
      // the filepicker
      jfRelativeFilePicker* filepicker;
    // links
      jfCategories* datalink;
    // the event table
    DECLARE_EVENT_TABLE()
};

//===========================================================================
/* This is the main top level categories editor, with the editor list, some
controls to edit the default category, an add button, and some labels
*/
class jfMainCategoriesEditor : public wxPanel {
  public:
    // the constructor
    jfMainCategoriesEditor(wxWindow* parent, jfCategories* in_datalink,
          wxWindowID id = wxID_ANY, const wxString& name = wxT("jfMainCategoriesEditor"));
    // helper methods
    void SetDefaultEnable(bool xenable=true);
    bool SaveCategories();
    void SetUnEditable(bool unedit = true);
		bool ChangeSearchName(wxString sname_in);
		// special methods
		bool ChangeToHorizontal();
		bool ChangeToVertical();
    // event handlers
    void ProcessDeleteBtn(wxCommandEvent& event);
    void ProcessAddBtn(wxCommandEvent& event);
  protected:
    // the linked data
    jfCategories* datalink;
    bool vertical;
    // default category
    jfDefaultCategoryEditor* dfcat_panel;
    // next up is the proper catgeories section
      wxStaticBoxSizer* categories_sizer;
      jfCategoryEditorsNotebook *mainlist_panel;
      wxButton* addcat_btn;
      // labels at the top
      wxStaticText *mainlabel;
      wxStaticText* maindesc;
      wxBoxSizer* topsizer;

    // the event table
    DECLARE_EVENT_TABLE()

};
