/******************************************************************************
Name    :   expr_editor.h
Author  :   John Q Metro
Purpose :   Editors for jfExpressionFiler, and also jfUrlFilter
Created :   March 26, 2009
Updated :   Febuary 19, 2012
******************************************************************************/
#ifndef EXPR_EDITOR_H_INCLUDED
#define EXPR_EDITOR_H_INCLUDED
#endif // EXPR_EDITOR_H_INCLUDED
//-----------------------------------------------------------------------------
// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif
//-----------------------------------------------------------------------------
#ifndef JFILTERGROUPS
  #include "../../../core/filters/base/filter_groups.h"
#endif
#ifndef JFMOREBASE
  #include "../../../core/filters/base/morebase.h"
#endif
#ifndef JFBASEFILTER
  #include "../../../core/base/filterbase.h"
#endif
//*****************************************************************************
/* The expression editor has an edit box and two buttons: one for inserting
a filter reference (using the filter picker dialog), and the other for checking
the validity of an expression and it's filter links.
Two modes: single and multiple lines

Color coding will be used to determine errors:
0: Green, okay
1: Black, unchecked
2: Red, in error
*/
class jfExpressionEditor : public wxPanel {
  public:
    jfExpressionEditor(wxWindow* parent, wxWindowID id,jfFilterMap* inmap, bool notinmap, const bool& multiline, bool in_global);
    // events
    void PressPickFilter(wxCommandEvent& event);
    void PressCheckFilter(wxCommandEvent& event);
    void TextChanged(wxCommandEvent& event);
    // checking method
    jfExpressionFilter* CheckFilter(wxString& outmessage);
    bool CheckExpr(const wxString& inexpr,wxString& outmessage) const;
    // setting data
    bool SetData(const wxString& inexpr,wxString& outmessage);
  protected:
    // internal helper
    void SetStatus(size_t instatus);
    // the internal controls
    wxTextCtrl* editcore;
    wxButton* finsert_btn;
    wxButton* fcheck_btn;
    // sizers (horizontal, buttons on the right)
    wxBoxSizer* topsizer;
    // in multiline mode
    wxBoxSizer* rightsizer;
    // linked reference data
    jfFilterMap* local_fmap;
    size_t error_code;
    bool map_nomap;
    bool isglobal;

    DECLARE_EVENT_TABLE()
};
//****************************************************************************
/* The url filter has two multiline Text control boxes, one to list alternate
includes, the other for excludes, and a checkbox to disbale the first. Another
checkbox controls whether https protocol is counted when applying the filter
*/
class jfUrlFilEdit : public wxPanel {
  public:
    jfUrlFilEdit(wxWindow* parent, wxWindowID id, bool side_arrangement=false);
    // events
    void ExcludesOnlyChecked(wxCommandEvent& event);
    // getting data
    jfUrlFilter* GetData() const;
    bool IsEmpty() const;
    // setting data
    bool SetData(const jfUrlFilter* indata);
  protected:
    // the lists
    wxTextCtrl* alt_incls;
    wxTextCtrl* excludes;
    // the checkboxes
    wxCheckBox* case_insensetive;
    wxCheckBox* excludes_only;
    wxCheckBox* https_counted;
    // text labels
    wxStaticText* alt_labl;
    wxStaticText* excl_labl;
    // sizers
    wxBoxSizer* totalsizer;
    wxBoxSizer* left_sizer;
    wxBoxSizer* right_sizer;
    bool sidearr;

    DECLARE_EVENT_TABLE()
};

//*****************************************************************************
