/******************************************************************************
Name    :   filter_editor.cpp
Author  :   John Q Metro
Purpose :   Defines the interface for a local and global filter editor
Created :   August 15, 2009
Updated :   March 2, 2014
******************************************************************************/
#ifndef FILTER_EDITOR_H_INCLUDED
  #include "filter_editor.h"
#endif // FILTER_EDITOR_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../../../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef FILTER_UTILS_H_INCLUDED
  #include "filter_utils.h"
#endif // FILTER_UTILS_H_INCLUDED
#ifndef JFILTERGLOBALS
  #include "../../../core/filters/base/filter_globals.h"
#endif
#ifndef JFILTERGROUPS
  #include "../../../core/filters/base/filter_groups.h"
#endif
/*****************************************************************************/
wxString new_dialog_name;
//============================================================================
// the constructor for this picker
jfFilterPicker2::jfFilterPicker2(wxWindow* parent,wxWindowID id, const wxString& name):
      jfFilterPickerBase(parent,false,id,name) {
  // not much to do here
  sloaded = false;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// item manipulation
//---------------------------------------------------------------------------
// inserts a name at the specified index
bool jfFilterPicker2::InsertAtIndex(const size_t& nindex, const jfBaseFilter* infilt) {
  long resindex;
  // checks
  if (infilt==NULL) return false;
  if (nindex>GetItemCount()) return false;
  // we start with the insert
  resindex = InsertItem(nindex,infilt->GetName());
  SetItem(resindex,1,infilt->GetTypeID());
  // if the new index is less than selected, we increment the selected index
  if (resindex<=select_index) select_index++;
  // done
  return true;
}
//---------------------------------------------------------------------------
/* we delete the current selected item, and use InsertAtIndex to place the
new selected item onto the list */
bool jfFilterPicker2::ReplaceSelected(const size_t& nindex, const jfBaseFilter* infilt) {
  // checks
  if (infilt==NULL) return false;
  if (nindex>=GetItemCount()) return false;
  if (!IsFilterSelected()) return false;
  // we delete the selected filter
  DeleteItem(select_index);
  // add the new one
  if (!InsertAtIndex(nindex,infilt)) return false;
  // setting the index
  return ChangeSelected(nindex);
}
//-----------------------------------------------------------------------------
bool jfFilterPicker2::RenameItem(size_t oldindex, size_t newindex, wxString newname) {
  // variables
  wxString typeval;
  long sval;
  bool issval;
  int del = 0;
  long resindex, lbackval;
  // special checks
  if (oldindex>=GetItemCount()) return false;
  if (newindex>=GetItemCount()) return false;
  if (newname.IsEmpty()) return false;
  // special case
  if (oldindex==newindex) {
    SetItemText(oldindex,newname);
    return true;
  }
  // otherwise
  typeval = OnGetItemText(oldindex,1);
  // checking the selected value
  sval = select_index;
  issval = (sval==oldindex);
  // calculating the selected index shift
  if (!issval) {
    if (oldindex<newindex) {
      if (sval<=newindex) del = -1;
    }
    else {
      if (sval<newindex) del = 1;
    }
  }
  // removing the old item, inserting the new
  DeleteItem(oldindex);
  resindex = InsertItem(newindex,newname);
  SetItem(resindex,1,typeval);
  // changing the selection if need be
  if (del!=0) {
    lbackval = old_selection;
    SelectNoEvent(sval+del,true);
    old_selection = lbackval;
  }
  // done
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// replace at index
bool jfFilterPicker2::ReplaceAtIndex(size_t oldindex, size_t newindex, const jfBaseFilter* infilt) {
  // special checks
  if (oldindex>=GetItemCount()) return false;
  if (newindex>=GetItemCount()) return false;
  if (infilt==NULL) return false;
  // we delete the oldindex filter
  DeleteItem(oldindex);
  // add the new one
  if (!InsertAtIndex(newindex,infilt)) return false;
  SetSelected();
  // done
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfFilterPicker2::DistribColumns(wxSizeEvent &event) {
  SetColWidth(3,3,0);
  event.Skip();
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event table
BEGIN_EVENT_TABLE(jfFilterPicker2,jfFilterPickerBase)
  EVT_SIZE(jfFilterPicker2::DistribColumns)
END_EVENT_TABLE()
//=============================================================================
// the constructor
jfStartFilterDialog::jfStartFilterDialog(wxWindow* parent, wxWindowID id,
    const wxString& startname, const wxArrayString* inpflist, const wxPoint& pos, const wxSize& size):
    wxDialog(parent,id,wxT("Pick a Name and Filter Type"),pos,size,
    wxDEFAULT_DIALOG_STYLE,wxT("jfStartFilterDialog")) {
  // the name
  assert(!startname.IsEmpty());
  assert(inpflist!=NULL);
  name_entry = new jfNameEditor(this,startname);
  // the label for the type picker
  picker_label = new wxStaticText(this,-1,wxT("Filter Type:"));
  // the type picker
  ftchoices = inpflist;
  type_picker = new wxChoice(this,-1,wxDefaultPosition,wxDefaultSize,(*ftchoices));
  // the buttons
  button_sizer = CreateButtonSizer(wxOK|wxCANCEL);
  assert(button_sizer!=NULL);
  // we set up the sizers
  top_sizer = new wxBoxSizer(wxVERTICAL);
  top_sizer->Add(name_entry,0,wxALL|wxEXPAND,2);
  top_sizer->AddSpacer(3);
  top_sizer->Add(picker_label,0,wxALL,2);
  top_sizer->Add(type_picker,0,wxALL|wxEXPAND,2);
  top_sizer->Add(button_sizer,0,wxALL,2);
  // the last thing
  SetSizer(top_sizer);
  SetSize(300,140);
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handler
//---------------------------------------------------------------------------
void jfStartFilterDialog::OKPressed(wxCommandEvent& event) {
  new_dialog_name = name_entry->TryGet();
  EndModal(type_picker->GetSelection());
}
//--------------------------------------------------------------------------
void jfStartFilterDialog::CancelPressed(wxCommandEvent& event) {
  EndModal(-1);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the vent handling table, for the buttons in button_sizer
BEGIN_EVENT_TABLE(jfStartFilterDialog,wxDialog)
  EVT_BUTTON(wxID_OK,jfStartFilterDialog::OKPressed)
  EVT_BUTTON(wxID_CANCEL,jfStartFilterDialog::CancelPressed)
END_EVENT_TABLE()

//=================================================================================
// the default constructor
jfFilterMapEditor::jfFilterMapEditor(wxWindow* parent, wxWindowID id,jfFilterMap* inlink,bool in_global, const wxArrayString* inpflist):
        wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,wxT("jfFilterMapEditor")) {
  // starting
  fdata = inlink;
  is_inglobal = in_global;
  if (!in_global) {
    assert(inpflist!=NULL);
    pflist = inpflist;
  }
  else pflist = filterglobals::fclist[0];
  // button creation
  new_button = new wxButton(this,jfID_MFE_NEW_BTN,wxT("New..."));
  del_button = new wxButton(this,jfID_MFE_DEL_BTN,wxT("Delete"));
  save_button = new wxButton(this,jfID_MFE_SAV_BTN,wxT("Save"));
  // sticking the buttons in the sizer
  btn_sizer = new wxBoxSizer(wxHORIZONTAL);
  btn_sizer->Add(new_button,1,wxALL,3);
  btn_sizer->Add(del_button,1,wxALL,3);
  btn_sizer->Add(save_button,1,wxALL,3);
  // next up, we create the picker and the side sizer
  picker = new jfFilterPicker2(this,jfID_MFE_LPICKER);
  side_sizer = new wxBoxSizer(wxVERTICAL);
  side_sizer->Add(btn_sizer,0);
  side_sizer->Add(picker,1,wxEXPAND|wxALL,3);
  // we make a blank panel
  blank_panel = new wxPanel(this);
  editing_panel = NULL;
  isblank = true;
  // arranging that
  top_sizer = new wxBoxSizer(wxHORIZONTAL);
  top_sizer->Add(side_sizer,1,wxALL|wxEXPAND,2);
  top_sizer->Add(blank_panel,local_ratio,wxALL|wxEXPAND,2);
  SetSizer(top_sizer);
  // setting the data
  BtnEnable(false);
  disablec = false;
  selindex = -1;
  if (inlink!=NULL) ChangeMap(inlink);

}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//data setting
bool jfFilterMapEditor::ChangeMap(jfFilterMap* inlink) {
  // basic checks
  size_t fcounty;
  assert(inlink!=NULL);
  if (fdata!=inlink) fdata = inlink;
  // we start loading the data
  picker->SetContents(fdata);
  fcounty = fdata->GetCount();
  // if the filter map is empty, we remove the editing panel and use a blank
  if (fdata->IsEmpty()) SetNoFilter();
  // the filter map is not empty, we have to change the filter to the first one
  else {
    ChangeFilter(0);
    BtnEnable(true);
  }
  // done
  Layout();
  return true;
}
//------------------------------------------------------------------------
// changes the filter to one at a particular index
bool jfFilterMapEditor::ChangeFilter(size_t findex) {
  // things that make us return false
  if (fdata->IsEmpty()) return false;
  if ((fdata->GetCount())<=findex) return false;
  // doing the thing, then returning true
  bool cs_res = picker->ChangeSelected(findex);
  assert(cs_res);
  selindex = findex;
  LoadSelected();
  return true;
}
//--------------------------------------------------------------------------
void jfFilterMapEditor::SetNoFilter() {
  // basic checks
  assert(blank_panel!=NULL);
  // we only need to do something if we are non-blank
  if (!isblank) {
    assert(editing_panel!=NULL);
    // replacing
    editing_panel->Hide();
    bool ts_res = top_sizer->Replace(editing_panel,blank_panel);
    assert(ts_res);
    blank_panel->Show();
    // removing the old panel
    editing_panel->Destroy();
    editing_panel = NULL;
    isblank = true;
    picker->Deselect();
    selindex = -1;
    top_sizer->Layout();
    oldtype = wxT("");
    BtnEnable(false);
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//event handlers
//--------------------------------------------------------------------------
// making a blank new filter and adding it to the map: we have to pick the name
void jfFilterMapEditor::NewPressed(wxCommandEvent& event) {
  // constants
  const wxString fname = wxT("jfFilterMapEditor::NewPressed");
  const wxString nfbasen=wxT("NewFilter");
  // local variables
  jfStartFilterDialog*  newdialog;
  jfBaseFilter*          newf1;
  long moderes;
  bool okchosen = false;
  size_t oindex;
  // the main loop
  while(true) {
    // making the dialog and showing it
    newdialog = new jfStartFilterDialog(this,-1,MakeName(nfbasen),pflist);
    moderes = newdialog->ShowModal();
    newdialog->Destroy();
    // we check the result
    if (moderes<0) break;
    // here, the result is OK, we check the name
    else {
      // if the name is already in use, we show an error message and loop
      if (fdata->NameExists(new_dialog_name)) {
        wxMessageBox(wxT("The name \'")+new_dialog_name+
              wxT("\' already exists. Please pick another."),wxT("Name exists problem"));
      }
      else {
        okchosen = true;
        break;
      }
    }
  }
  // we have got our result, we make and insert the new filter if need be
  if (okchosen) {
    assert(moderes<(pflist->Count()));
    // we create the filter
    newf1 = MakeFilter((*pflist)[moderes],new_dialog_name);
    /**/jfXLogMessage(fname,0,newf1->GetName());
    // insert the filter
    wxString oname = newf1->GetName();
    bool atest = fdata->AddFilter(newf1,oindex);
    assert(atest);
    atest = picker->InsertAtIndex(oindex,newf1);
    assert(atest);
    /**/jfXLogMessageS(fname,1,oindex);
    /**/jfXLogMessageS(fname,2,fdata->GetCount());
    // we change the current filter to the inserted one
    atest = ChangeFilter(oindex);
    assert(atest);
    BtnEnable(true);
  }
}
//--------------------------------------------------------------------------
// deleting a filter
void jfFilterMapEditor::DeletePressed(wxCommandEvent& event) {
  // we record the selected index to determine what it will be
  long oselected = picker->SelectedIndex();
  if (oselected>=0) {
    // we delete the actual filter first
    wxString fname = picker->SelectedName();
    fdata->DeleteByName(fname);
    // we remove the item from the picker
    picker->DeleteItem(oselected);
    // now we decide on the new selected index
    int icount = picker->GetItemCount();
    // there is nothing left
    if (icount==0)  SetNoFilter();
    else {
      // if the selected insex is now past the last, select the last
      if (icount==oselected) oselected-=1;
      ChangeFilter(oselected);
    }
  }
}
//--------------------------------------------------------------------------
// saving a filter
void jfFilterMapEditor::SavePressed(wxCommandEvent& event) {
  const wxString fname = wxT("jfFilterMapEditor::SavePressed");
  assert(!isblank);
  // we only go ahead with the save if the check passes
  if (editing_panel->GeneralCheck()) {
    CoreSave();
  }
}
//--------------------------------------------------------------------------
void jfFilterMapEditor::FilterPicked(wxListEvent& event) {
  int newindex = picker->SelectedIndex();
  if (selindex!=newindex) {
    CoreSave(true);
    selindex = picker->SelectedIndex();
    LoadSelected();
  }
}
//--------------------------------------------------------------------------
// special methods
void jfFilterMapEditor::DisableContents(bool disable) {
  disablec = disable;
  // setting the buttons
  new_button->Enable(!disable);
  del_button->Enable(!disable);
  save_button->Enable(!disable);
  // the main thing...
  if (!isblank) {
    editing_panel->Enable(!disablec);
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfFilterMapEditor::LoadSelected() {
  // const
  const wxString fname = wxT("jfFilterMapEditor::LoadSelected");
  // local variables
  jfBaseFilter* tfilter;
  wxString tfiltype;
  jfBaseFilterEditor* eholder;
  bool atest;
  // checks
  assert(picker->IsFilterSelected());
  /**/jfXLogMessage(fname,0,picker->SelectedName());
  // we get the filter in question
  tfilter = fdata->GetItem(picker->SelectedName());
  assert(tfilter!=NULL);
  // we get the type in question
  tfiltype = tfilter->GetTypeID();
  /**/jfXLogMessage(fname,1,tfiltype);
  /**/jfXLogMessage(fname,2,oldtype);
  /**/jfXLogMessage(fname,3,tfilter->GetName());
  // we do not need to change the panel
  if (tfiltype==oldtype) {
    /**/jfXLogMessage(fname,4);
    editing_panel->LoadFilter(tfilter);
    /**/jfXLogMessage(fname,5);
  }
  // we do need to change the panel
  else {
    // making the editor
    eholder = MakeFilterEditor(tfiltype,this,fdata,tfilter,is_inglobal);
    eholder->Enable(!disablec);
    // how we replace it depends on whether it is blank
    if (isblank) {
      atest = top_sizer->Replace(blank_panel,eholder);
      assert(atest);
      blank_panel->Hide();
      isblank = false;
    }
    else {
      atest = top_sizer->Replace(editing_panel,eholder);
      assert(atest);
      editing_panel->Destroy();
    }
    // finializing steps
    editing_panel = eholder;
    oldtype = tfiltype;
  }
  // done
  top_sizer->Layout();
  return true;
}
//------------------------------------------------------------------------
// makes up a general filter name
wxString jfFilterMapEditor::MakeName(const wxString& nfbase) const {
  // variables
  size_t loopc=1;
  wxString buffer;
  // starting
  assert(fdata!=NULL);
  // the loop
  while (true) {
    buffer = nfbase + SInt2Str(loopc);
    if (!(fdata->NameExists(buffer))) break;
    loopc++;
  }
  return buffer;
}
//---------------------------------------------------------------------------
void jfFilterMapEditor::BtnEnable(bool enable) {
  del_button->Enable(enable);
  save_button->Enable(enable);
}
//------------------------------------------------------------------
bool jfFilterMapEditor::CoreSave(bool saveold) {
  //constants
  const wxString fname = wxT("jfFilterMapEditor::CoreSave");
  // variables
  wxString repname;
  // starting
  if (!isblank) {
    // variables
    jfBaseFilter *newfilter;
    size_t ins_place;
    bool atest;
    // we get the replacement filter
    /**/jfXLogMessage(fname,1);
    newfilter = editing_panel->GetFilter();
    assert(newfilter!=NULL);
    // replace the filter
    /**/jfXLogMessage(fname,2,newfilter->GetName());
    if (saveold) repname = picker->GetItemText(selindex);
    else repname = picker->SelectedName();
    atest = fdata->ReplaceByName(repname,newfilter,ins_place);
    assert(atest);
    /**/jfXLogMessage(fname,3);
    // and change change the selection index (only truly needed if the name has changed)
    if (saveold) picker->ReplaceAtIndex(selindex, ins_place, newfilter);
    else picker->ReplaceSelected(ins_place,newfilter);
    selindex = ins_place;
    if (!saveold) {
      editing_panel->ChangeBoxName(newfilter->GetName());
      /**/jfXLogMessage(fname,4);
      editing_panel->ChangeFilter(newfilter);
    }
    // done
  return atest;
  }
  else return false;
}
//-------------------------------------------------------------------------
// saving a filter when we are switching to another
bool jfFilterMapEditor::SaveFilter(long oindex) {
  if (oindex<0) return false;
  // variables
  jfBaseFilter *newfilter;
  size_t ins_place;
  bool atest;
  // we get the replacement filter
  newfilter = editing_panel->GetFilter();
  assert(newfilter!=NULL);
  // replace the filter
  atest = fdata->ReplaceByName(picker->GetItemText(oindex),newfilter,ins_place);
  assert(atest);
	return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfFilterMapEditor,wxPanel)
  EVT_BUTTON(jfID_MFE_NEW_BTN,jfFilterMapEditor::NewPressed)
  EVT_BUTTON(jfID_MFE_DEL_BTN,jfFilterMapEditor::DeletePressed)
  EVT_BUTTON(jfID_MFE_SAV_BTN,jfFilterMapEditor::SavePressed)
  EVT_LIST_ITEM_SELECTED(jfID_MFE_LPICKER,jfFilterMapEditor::FilterPicked)
END_EVENT_TABLE()
//*****************************************************************************
// methods for jfMapEditPanel
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfMapEditPanel::jfMapEditPanel(wxWindow* parent, wxWindowID id,jfFilterMap* inlink,
      const wxString& name):wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,
      wxTAB_TRAVERSAL,name) {
  // initial stuff
  itemlink = inlink;
  mappset = !(inlink==NULL);
	// for the sake of OSX, static box sizers must be made first
	edit_wrapper = new wxStaticBoxSizer(wxVERTICAL,this,wxT("NULL"));
  // making the name editor
  ndesce = new jfNameDescEditor(this,-1,false,false);
  save_name = new wxButton(ndesce,jfID_GFE_SAV_BTN, wxT("Save Names and Descriptions"));
  ndesce->AddItem(save_name);
  // the main editor
  map_editor = new jfFilterMapEditor(this,-1,NULL,true,NULL);
  // Arranging things
  edit_wrapper->Add(ndesce,0,wxEXPAND);
  edit_wrapper->Add(map_editor,1,wxEXPAND|wxALL,3);
  // done
  SetSizer(edit_wrapper);
  if (mappset) ChangeMap(inlink);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// accessible methods
bool jfMapEditPanel::ChangeMap(jfFilterMap* inlink) {
  if (inlink==NULL) return false;
  // we set the name information
  (edit_wrapper->GetStaticBox())->SetLabel(inlink->GetName());
  ndesce->SetFromObj(inlink);
  // passing the map to the contents editor
  map_editor->ChangeMap(inlink);
  // setting soem values
  itemlink = inlink;
  mappset = true;
  return true;
}
//--------------------------------------------------------------------------
wxString jfMapEditPanel::GetName() {
  return ndesce->TryGetName();
}
//--------------------------------------------------------------------------
wxString jfMapEditPanel::GetDescription() const {
  return ndesce->GetDesc();
}
//--------------------------------------------------------------------------
void jfMapEditPanel::ChangeBoxName(wxString newname) {
  (edit_wrapper->GetStaticBox())->SetLabel(newname);
}
//*****************************************************************************
// --- methods for jfGlobalFilterEditor ----------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfGlobalFilterEditor::jfGlobalFilterEditor(wxWindow* parent, wxWindowID id,jfMapOfFiltermaps* inlink):wxPanel(parent,id) {
  assert(inlink!=NULL);
  maindata = inlink;
  wxArrayString* mstr;
  // we start with the things in the side panel
  add_btn = new wxButton(this,jfID_GFE_NEW_BTN,wxT("New Map"));
  dele_btn = new wxButton(this,jfID_GFE_DEL_BTN,wxT("Delete Map"));
  mstr = maindata->GetMapNames();
  if (mstr == NULL) mstr = new wxArrayString();
  map_list = new wxListBox(this,jfID_GFE_MPICKER,wxDefaultPosition,wxDefaultSize,(*mstr),wxLB_SINGLE|wxLB_NEEDED_SB);
  delete mstr;
  // we start arranging the size panel
  btn_sizer = new wxBoxSizer(wxHORIZONTAL);
  btn_sizer->Add(add_btn,1,wxALL|wxEXPAND,3);
  btn_sizer->Add(dele_btn,1,wxEXPAND|wxTOP|wxBOTTOM|wxRIGHT,3);
  side_sizer = new wxBoxSizer(wxVERTICAL);
  side_sizer->Add(btn_sizer,0);
  side_sizer->Add(map_list,1,wxEXPAND|wxALL,3);
  // the right side has the map editing panel or a blank panel
  blank_panel = new wxPanel(this,-1);
  main_mapedit = new jfMapEditPanel(this,-1);
  // we create the topsizer
  top_sizer = new wxBoxSizer(wxHORIZONTAL);
  top_sizer->Add(side_sizer,1,wxEXPAND);
  // the following is for text purposes
  // if (maindata->empty()) MakeDefaultMap(wxT("TheDefaultTestMap"));
  // what we do next depends on whether there is no maps
  if (maindata->empty()) {
    main_mapedit->Hide();
    top_sizer->Add(blank_panel,global_ratio,wxEXPAND);
    dele_btn->Enable(false);
    isblank = true;
  }
  else {
    blank_panel->Hide();
    top_sizer->Add(main_mapedit,global_ratio,wxEXPAND|wxALL,3);
    ChangeMap(0);
    isblank = false;
  }
  SetSizer(top_sizer);
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//event handlers
//--------------------------------------------------------------------------
// this event is for making a new, empty filter map
void jfGlobalFilterEditor::NewPressed(wxCommandEvent& event) {
  // constants
  const wxString headmsg = wxT("Please enter a name");
  const wxString prompt = wxT("Please create a unique, non-blank Group name:");
  // local variables
  wxString newname;
  bool cancel;
  size_t nindex;
  // we start with the generic dialog!
  newname = wxGetTextFromUser(headmsg,prompt,wxT("FilterGroup"));
  Dtrim(newname);
  cancel = true;
  // the checking loop
  while(true) {
    if (newname.IsEmpty()) break;
    else if (maindata->NameExists(newname)) {
      newname = wxGetTextFromUser(headmsg,wxT("The name is already in use! ")+prompt,wxT("FilterGroup"));
    }
    else {
      cancel = false;
      break;
    }
  }
  // here we are!
  if (!cancel) {
    // making the new map
    nindex = MakeDefaultMap(newname);
    // done
    ChangeMap(nindex);
  }

}
//--------------------------------------------------------------------------
// deletes the currently selected filter group
void jfGlobalFilterEditor::DeletePressed(wxCommandEvent& event) {
  // a special case
  if ((map_list->GetSelection())!=wxNOT_FOUND) {
    // we now remove the map from the maindata and list
    maindata->erase(selname);
    map_list->Delete(selindex);
    // we pick a new index
    if (maindata->empty()) SetNoMap();
    else {
      if ((maindata->size())>=selindex) selindex--;
      ChangeMap(selindex);
    }
    /* now that the map is removed and a new focus is picked, we
    dispose of the old map */
    selmap->DeleteAll();
    delete selmap;
  }

}
//--------------------------------------------------------------------------
// this event is for saving the name and description of the map
void jfGlobalFilterEditor::SavePressed(wxCommandEvent& event) {
  // asserts
  assert(!isblank);
  // variables
  size_t newindex;
  wxString newname = main_mapedit->GetName();
  bool atest;
  // starting
  if (newname!=selname) {
    // we rename the map
    atest = maindata->RenameMap(selname,newname,newindex) ;
    assert(atest);
    // chaning the name in the list box
    map_list->Delete(selindex);
    map_list->Insert(newname,newindex);
    map_list->SetSelection(newindex);
    // updating some variables
    selname = newname;
    selindex = newindex;
    main_mapedit->ChangeBoxName(selname);
  }
  selmap->SetDescription(main_mapedit->GetDescription());
}
//--------------------------------------------------------------------------
void jfGlobalFilterEditor::MapPicked(wxCommandEvent& event) {
  LoadSelectedMap();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// changing the editing panel to the blank one
void jfGlobalFilterEditor::SetNoMap() {
  if (!isblank) {
    main_mapedit->Hide();
    top_sizer->Replace(main_mapedit,blank_panel);
    blank_panel->Show();
    isblank = true;
    map_list->SetSelection(wxNOT_FOUND);
    dele_btn->Enable(false);
    Layout();
  }
}
//--------------------------------------------------------------------------
// loads the map at the selected in dex
bool jfGlobalFilterEditor::LoadSelectedMap() {
  // a special case
  int q = map_list->GetSelection();
  if (q==wxNOT_FOUND) return false;
  else {
    // getting the map...
    selindex = map_list->GetSelection();
    selname = map_list->GetString(selindex);
    selmap = (*maindata)[selname];
    // loading the editor
    main_mapedit->ChangeMap(selmap);
    // now, we see if the main editor is blank!
    if (isblank) {
      blank_panel->Hide();
      top_sizer->Replace(blank_panel,main_mapedit);
      main_mapedit->Show();
      dele_btn->Enable(true);
      isblank = false;
      top_sizer->Layout();
    }
    // done
    return true;
  }
}
//--------------------------------------------------------------------------
// changes the map, mostly consists of setting it and using LoadSelectedMap
bool jfGlobalFilterEditor::ChangeMap(const size_t& index) {
  if (index>=(maindata->size())) return false;
  map_list->SetSelection(index);
  return LoadSelectedMap();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// creates a map and adds it to the map group
size_t jfGlobalFilterEditor::MakeDefaultMap(wxString thename) {
  // variables
  size_t nindex;
  bool atest;
  // we make the map
  (*maindata)[thename] = new jfFilterMap();
  (*maindata)[thename]->SetName(thename);
  // for debugging only
  atest = (*maindata)[thename]->AddFilter(new jfUrlFilter(),nindex);
  assert(atest);
  // test 1
  atest = maindata->FindMapIndex(thename,nindex);
  assert(atest);
  // adding it to the list ctrl
  map_list->Insert(thename,nindex);
  return nindex;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfGlobalFilterEditor,wxPanel)
  EVT_BUTTON(jfID_GFE_NEW_BTN,jfGlobalFilterEditor::NewPressed)
  EVT_BUTTON(jfID_GFE_DEL_BTN,jfGlobalFilterEditor::DeletePressed)
  EVT_BUTTON(jfID_GFE_SAV_BTN,jfGlobalFilterEditor::SavePressed)
  EVT_LISTBOX(jfID_GFE_MPICKER,jfGlobalFilterEditor::MapPicked)
END_EVENT_TABLE()
/*****************************************************************************/
