/******************************************************************************
Name    :   filter_editor.h
Author  :   John Q Metro
Purpose :   Declares the interface for a local and global filter editor
Created :   August 13, 2009
Updated :   March 11, 2013 (column resizing for jfFilterPicker2)
******************************************************************************/
#ifndef FILTER_EDITOR_H_INCLUDED
#define FILTER_EDITOR_H_INCLUDED
#endif // FILTER_EDITOR_H_INCLUDED
#ifndef FILTER_WIDGETS_H_INCLUDED
  #include "filter_picker.h"
#endif // FILTER_WIDGETS_H_INCLUDED
#ifndef WIDGETS1_H_INCLUDED
  #include "../../misc/widgets1.h"
#endif // WIDGETS1_H_INCLUDED
#ifndef BFILT_EDIT_H_INCLUDED
  #include "bfilt_edit.h"
#endif // BFILT_EDIT_H_INCLUDED
#ifndef JFILTERGROUPS
  #include "../../core/filters/filter_groups.h"
#endif
//--------------------------------------------------------------------------------
/******************************************************************************/
extern wxString new_dialog_name;
const size_t global_ratio = 5;
const size_t local_ratio = 2;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// a 2 columned filter picker
class jfFilterPicker2 : public jfFilterPickerBase {
  public:
    // the constructor
    jfFilterPicker2(wxWindow* parent,wxWindowID id = wxID_ANY, const wxString& name = wxT("jfFilterPicker2"));
    // item manipulation
    bool InsertAtIndex(const size_t& nindex, const jfBaseFilter* infilt);
    bool ReplaceSelected(const size_t& nindex, const jfBaseFilter* infilt);
    bool RenameItem(size_t oldindex, size_t newindex, wxString newname);
    bool ReplaceAtIndex(size_t oldindex, size_t newindex, const jfBaseFilter* infilt);
    // events
    void DistribColumns(wxSizeEvent &event);
  protected:

  DECLARE_EVENT_TABLE()

};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* A small dialog used in new filter creation, to pick the name and type */
class jfStartFilterDialog : public wxDialog {
  public:
    // the constructor
    jfStartFilterDialog(wxWindow* parent, wxWindowID id, const wxString& startname,
         const wxArrayString* inpflist, const wxPoint& pos = wxDefaultPosition,
        const wxSize& size = wxDefaultSize);
    // event handler
    void OKPressed(wxCommandEvent& event);
    void CancelPressed(wxCommandEvent& event);
  protected:
    // the gui elements
    jfNameEditor* name_entry;
    wxStaticText* picker_label;
    wxChoice* type_picker;
    wxBoxSizer* top_sizer;
    wxSizer* button_sizer;
    // choices
    const wxArrayString* ftchoices;

    DECLARE_EVENT_TABLE()
};
//=============================================================================
/* the local filter editor, which handles one filter map. Includes a picker,
buttons to add and delete filters, and a panel to hold the filter-specific
editor for the currently selected filter */
class jfFilterMapEditor : public wxPanel {
  public:
    // constructor
    jfFilterMapEditor(wxWindow* parent, wxWindowID id,jfFilterMap* inlink,bool in_global, const wxArrayString* inpflist);
    //data setting
    bool ChangeMap(jfFilterMap* inlink);
    bool ChangeFilter(size_t findex);
    void SetNoFilter();
    //event handlers
    void NewPressed(wxCommandEvent& event);
    void DeletePressed(wxCommandEvent& event);
    void SavePressed(wxCommandEvent& event);
    void FilterPicked(wxListEvent& event);
    // special methods
    void DisableContents(bool disable=true);
    bool CoreSave(bool saveold = false);
  protected:
    // helper methods
    bool SaveFilter(long oindex);
    bool LoadSelected();
    wxString MakeName(const wxString& nfbase) const;
    void BtnEnable(bool enable);
    // gui elements
    wxButton* new_button;
    wxButton* del_button;
    wxButton* save_button;
    jfFilterPicker2* picker;
    wxPanel* blank_panel;
    jfBaseFilterEditor* editing_panel;
    bool isblank;
    bool disablec;
    // sizers
    wxBoxSizer* top_sizer;
    wxBoxSizer* side_sizer;
    wxBoxSizer* btn_sizer;
    // other data
    jfFilterMap* fdata;
    wxString oldtype;
    bool is_inglobal;
    int selindex;
    // external link
    const wxArrayString* pflist;

    DECLARE_EVENT_TABLE()
};
//==============================================================================
/* Used in Global filter editor, a panel including a name editor and a local
filter editor */
class jfMapEditPanel : public wxPanel {
  public:
    // the constructor
    jfMapEditPanel(wxWindow* parent, wxWindowID id,jfFilterMap* inlink = NULL,const wxString& name = wxT("jfMapEditPanel"));
    // accessible methods
    bool ChangeMap(jfFilterMap* inlink);
    wxString GetName();
    wxString GetDescription() const;
    void ChangeBoxName(wxString newname);
  protected:
    // gui elements
    jfNameDescEditor* ndesce;
    wxButton* save_name;
    jfFilterMapEditor* map_editor;
    // sizers
    wxStaticBoxSizer* edit_wrapper;
    // internal data
    bool mappset;
    jfFilterMap* itemlink;
};
//==============================================================================
/* the global filter editor, which has a list of maps, a map editor, two buttons
for adding and deleting maps, and an editor for the map name+desc
*/
class jfGlobalFilterEditor : public wxPanel {
  public:
    // the constructor
    jfGlobalFilterEditor(wxWindow* parent, wxWindowID id,jfMapOfFiltermaps* inlink);
    //event handlers
    void NewPressed(wxCommandEvent& event);
    void DeletePressed(wxCommandEvent& event);
    void SavePressed(wxCommandEvent& event);
    void MapPicked(wxCommandEvent& event);
    // methods
    void SetNoMap();
    bool LoadSelectedMap();
    bool ChangeMap(const size_t& index);
  protected:
    // helper methods
    size_t MakeDefaultMap(wxString thename);
    // gui elemts
    wxButton* add_btn;
    wxButton* dele_btn;
    wxListBox* map_list;
    wxPanel* blank_panel;
    jfMapEditPanel* main_mapedit;
    bool isblank;
    // sizers
    wxBoxSizer* btn_sizer;
    wxBoxSizer* side_sizer;
    wxBoxSizer* top_sizer;
    // internal data
    jfMapOfFiltermaps* maindata;
    size_t selindex;
    wxString selname;
    jfFilterMap* selmap;

    DECLARE_EVENT_TABLE()
};
/******************************************************************************/
