/******************************************************************************
Name    :   filter_picker.h
Author  :   John Q Metro
Purpose :   Declares some custom interface elements for a filter picker dialog.
Created :   March 25, 2009
Updated :   March 10, 2013 (column resizing for jfFilterPicker)
******************************************************************************/
#ifndef FILTER_WIDGETS_H_INCLUDED
#define FILTER_WIDGETS_H_INCLUDED
#endif // FILTER_WIDGETS_H_INCLUDED
//----------------------------------------------------------------------------
#ifndef BASECOLLECT_H_INCLUDED
  #include "../../../core/structs/basecollect.h"
#endif // BASECOLLECT_H_INCLUDED
#ifndef JFILTERGROUPS
  #include "../../../core/filters/base/filter_groups.h"
#endif
//----------------------------------------------------------------------------
#include <wx/listctrl.h>
#include <wx/notebook.h>
//*****************************************************************************
/* A 3-columed list box for picking filters. Column 1 is the filter name,
Column 2 is the filter type, and Column 3 (optional)is the filter description */
class jfFilterPickerBase : public wxListCtrl {
  public:
    jfFilterPickerBase(wxWindow* parent,bool three_cols, wxWindowID id = wxID_ANY,
          const wxString& name = wxListCtrlNameStr);
    size_t SetContents(const jfFilterMap* data, wxString inprefix=wxT(""));
    bool ClearLoaded();
    // selected data info
    bool IsFilterSelected() const;
    bool ChangeSelected(size_t nindex);
    long SelectedIndex() const;
    long OldSelected() const;
    wxString SelectedName() const;
    wxString InsertString() const;
    void Deselect();
    // column sizing
    void SetColWidth(size_t prop1, size_t prop2, size_t prop3);
    // events
    void OnItemSelect(wxListEvent& event);
  protected:
    // a helper method for setting contents
    bool InsertFLine(const size_t& iindex);
    bool SelectNoEvent(size_t index, bool issel);
    bool SetSelected();
    // local stuff
    bool loaded;
    wxArrayString *fnames, *ftypes, *fdesc;
    wxString prefix;
    // selection info
    bool sloaded;
    bool three_columns;
    wxString select_name;
    long select_index;
    long old_selection;
    bool bypass;

    DECLARE_EVENT_TABLE()

};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* A 3-columed derivate of jfFilterPickerBase. Includes some methods and things
for creating an embedded link string for expressions */
class jfFilterPicker1 : public jfFilterPickerBase {
  public:
    jfFilterPicker1(wxWindow* parent, wxStaticText* displaylink, wxWindowID id = wxID_ANY, const wxString& name = wxListCtrlNameStr);
    // insert string item
    wxString InsertString() const;
    // events
    void OnItemSelect(wxListEvent& event);
    void DistribColumns(wxSizeEvent &event);
  protected:
    // displaying the selection
    wxStaticText* display;

    DECLARE_EVENT_TABLE()

};
//============================================================================
/* global filter picker, with a listbox to pick the group, and a
jfFilterPicker to pick the individual filter */
class jfGlobalFilterPicker : public wxPanel {
  public:
    jfGlobalFilterPicker(wxWindow* parent, wxStaticText* displaylink, wxWindowID id = wxID_ANY, const wxString& name = wxT("GlobalFilterPicker"));
    // loads data
    bool LoadData(jfMapOfFiltermaps* inlink);
    // sets the data in the picker control to the map at the index
    bool LoadPicker(const size_t& pindex);
    // returns some internal data
    bool IsFilterChosen() const;
    wxString CurrentMapname() const;
    wxString InsertString() const;
    // the event
    void MapSelected(wxCommandEvent& event);
  protected:
    // the controls
    wxBoxSizer*     topsizer;
    jfFilterPicker1* fpicker;
    wxListBox*      lbox;
    // external data structure links
    bool loaded;
    jfMapOfFiltermaps* proxy_link;
    const jfFilterMap* sel_map;
    wxString sel_mapname;

    DECLARE_EVENT_TABLE()
};
//============================================================================
/* A filter picking dialog, with two tabs; one for the local list, and one
for selecting a global filter. This produces a string reference, ready to be
inserted into an expression */
class jfFilterPickerDialog : public wxDialog {
  public:
    jfFilterPickerDialog(wxWindow* parent, wxWindowID id, const wxString& title ,const jfFilterMap* inlocal_map, wxString* inresult_ptr);
    // the actions
    void ProcessPick(wxCommandEvent& event);
  protected:
    // arrangement controls
    wxNotebook* display_book;
    wxBoxSizer* topsizer;
    wxBoxSizer* bottom_sizer;
    // notebook pages
    jfFilterPicker1        *local_page;
    jfGlobalFilterPicker  *global_page;
    // stuff at the bottom
    wxButton* pick_btn;
    wxButton* cancel_btn;
    wxStaticText* labelresult;
    wxStaticText* showresult;
    // external links
    const jfFilterMap* local_map;
    wxString* result_ptr;

    DECLARE_EVENT_TABLE()
};
//=============================================================================
/* New filter picker dialog -- decided that there is no such thing as a local
filter for the global filters, the filter picker there must be global only */
class jfGlobalFilterPickerDialog : public wxDialog {
  public:
    jfGlobalFilterPickerDialog(wxWindow* parent, wxWindowID id, const wxString& title ,wxString* inresult_ptr);
    // the actions
    void ProcessPick(wxCommandEvent& event);
  protected:
    // arrangement controls
    wxBoxSizer* topsizer;
    wxBoxSizer* bottom_sizer;
    // notebook pages
    jfGlobalFilterPicker  *global_page;
    // stuff at the bottom
    wxButton* pick_btn;
    wxButton* cancel_btn;
    wxStaticText* labelresult;
    wxStaticText* showresult;
    // external links
    wxString* result_ptr;

    DECLARE_EVENT_TABLE()
};



//*****************************************************************************
