/******************************************************************************
Name    :   filter_utils.cpp
Author  :   John Q Metro
Purpose :   A filter editor creator, and anything else that comes to mind
Created :   December 27, 2010
Updated :   July 12, 2015
******************************************************************************/
#ifndef FILTER_UTILS_H_INCLUDED
  #include "filter_utils.h"
#endif // FILTER_UTILS_H_INCLUDED
//-----------------------------------------------------------
#ifndef JFMOREBASE
  #include "../../../core/filters/base/morebase.h"
#endif
#ifndef MOREFILTERS1_H_INCLUDED
  #include "../../../core/filters/extended/morefilters1.h"
#endif
#ifndef SEXP_FILTERS_H_INCLUDED
  #include "../../../core/filters/base/sexp_filters.h"
#ifndef FIMFILTERS2_H_INCLUDED
  #include "../../../core/filters/fim/fimfilters2.h"
#endif // FIMFILTERS2_H_INCLUDED
//------------------------------------------------------------
#endif // SEXP_FILTERS_H_INCLUDED
#ifndef BFILT_EDIT_H_INCLUDED
  #include "bfilt_edit.h"
#endif // BFILT_EDIT_H_INCLUDED
#ifndef FILTERS_EDIT1_H_INCLUDED
  #include "../extended/filters_edit1.h"
#endif // FILTERS_EDIT1_H_INCLUDED
#ifndef SFILT_EDIT_H_INCLUDED
  #include "sfilt_edit.h"
#endif // SFILT_EDIT_H_INCLUDED
#ifndef PAIRS_EDIT_H_INCLUDED
  #include "../extended/pairs_edit.h"
#endif // PAIRS_EDIT_H_INCLUDED
#ifndef MFILT_EDIT1_H_INCLUDED
  #include "../extended/mfilt_edit1.h"
#endif // MFILT_EDIT1_H_INCLUDED
#ifndef DRANGE_EDIT_H_INCLUDED
  #include "../extended/drange_edit.h"
#endif // DRANGE_EDIT_H_INCLUDED
#ifndef FIM1_FILEDIT_H_INCLUDED
  #include "../fim/fim1_filedit.h"
#endif // FIM1_FILEDIT_H_INCLUDED
#ifndef FIM2_FILEDIT_H_INCLUDED
  #include "../fim/fim2_filedit.h"
#endif // FIM2_FILEDIT_H_INCLUDED
#ifndef AO3_STRINGFILTERS_H_INCLUDED
  #include "../ao3/ao3_sexpeditors.h"
#endif // AO3_STRINGFILTERS_H_INCLUDED
#ifndef AO3_SPECIALS1_H_INCLUDED
  #include "../ao3/ao3_special1.h"
#endif // AO3_SPECIALS1_H_INCLUDED
#ifndef AO3_SPECIAL2_H_INCLUDED
  #include "../ao3/ao3_special2.h"
#endif // AO3_SPECIAL2_H_INCLUDED
//------------------------------------------------------
#ifndef AO3_SPECIALS1_H_INCLUDED
  #include "../../../core/filters/ao3/ao3_specials1.h"
#endif // AO3_SPECIALS1_H_INCLUDED
#ifndef AO3_SPECIAL2_H_INCLUDED
  #include "../../../core/filters/ao3/ao3_specials2.h"
#endif // AO3_SPECIAL2_H_INCLUDED


//***************************************************************************
// creates an appropriate editor using the provided links
jfBaseFilterEditor* MakeFilterEditor(wxString in_typeid, wxWindow* parent, jfFilterMap* containing_map, jfBaseFilter* input_filter, bool isglobal) {
  // we first check the inputs. anything wrong results in assert faliure
  assert(parent!=NULL);
  assert(containing_map!=NULL);
  assert(input_filter!=NULL);
  // variables
  // filters of the right type
  jfUrlFilter*          a01;
  jfExpressionFilter*   a02;
  jfAuthExprFilter*     a03;
  jfCharExprFilter*     a04;
  jfCategoryExprFilter* a05;
  jfWordCountFilter*    a06;
  jfFFNTagFilter*       a07;
  jfPairFilterSingle*   a08;
  jfPairFilterList*     a09;
  jfPairFilterMultiple* a10;
  jfCompletedFilter*    a11;
  jfLanguageExprFilter* a12;
  jfFFNRatingFilter*    a13;
  jfUpdatedDateFilter*  a14;
  jfPublishedDateFilter*a15;
  jfFimThumbsFilter*    a16;
  jfFIMGenreFilter*     a17;
  jfFIMCharacterFilter* a18;
  jfFimRatingFilter*    a19;
  jfExtraTagFilter*     a20;
  jfAO3FandomFilter*    a21;
  jfAO3RatingFilter*      a22;
  jfAO3OrientationFilter* a23;
  jfAO3PairFilter*        a24;
  jfAO3WarnFilter*        a25;
  jfFFNFavsFilter*        a26;
  jfFIMGroupSCFilter*      a27;
  jfFIMGroupMCFilter*      a28;
  jfFimThumbPercentFilter* a29;
  jfAO3KudoFilter*   a30;
  jfFIM_ShortDesc_ExprFilter* a31;
  jfFIM_DualDesc_ExprFilter* a32;
  // filter link
  jfBaseFilterEditor* result;
  // the main check
  if (in_typeid==wxT("UrlFilter")) {
    a01 = dynamic_cast<jfUrlFilter*>(input_filter);
    result = new jfUrlFilterEditor(parent,containing_map,a01);
  }
  else if (in_typeid==wxT("ExprFilter")) {
    a02 = dynamic_cast<jfExpressionFilter*>(input_filter);
    result = new jfExprFilterEditor(parent,containing_map,isglobal,a02);
  }
  else if (in_typeid==wxT("AuthorFilter")) {
    a03 = dynamic_cast<jfAuthExprFilter*>(input_filter);
    result = new jfAuthorFilterEditor(parent,containing_map,a03);
  }
  else if (in_typeid==wxT("CharacterFilter")) {
    a04 = dynamic_cast<jfCharExprFilter*>(input_filter);
    result = new jfCharFilterEditor(parent,containing_map,a04);
  }
  else if (in_typeid==wxT("CategoryFilter")) {
    a05 = dynamic_cast<jfCategoryExprFilter*>(input_filter);
    result = new jfCategoryFilterEditor(parent,containing_map,a05);
  }
  else if (in_typeid==wxT("WordCountFilter")) {
    a06 = dynamic_cast<jfWordCountFilter*>(input_filter);
    result = new jfWCFilterEditor(parent,containing_map,a06);
  }
  else if (in_typeid==wxT("FFNTagFilter")) {
    a07 = dynamic_cast<jfFFNTagFilter*>(input_filter);
    result = new jfFFNTagFilterEditor(parent,containing_map,a07);
  }
  else if (in_typeid==wxT("PairFilterSingle")) {
    a08 = dynamic_cast<jfPairFilterSingle*>(input_filter);
    result = new jfPairFS_Editor(parent,containing_map,a08);
  }
  else if (in_typeid==wxT("PairFilterList")) {
    a09 = dynamic_cast<jfPairFilterList*>(input_filter);
    result = new jfPairFL_Editor(parent,containing_map,a09);
  }
  else if (in_typeid==wxT("PairFilterMultiple")) {
    a10 = dynamic_cast<jfPairFilterMultiple*>(input_filter);
    result = new jfPairFM_Editor(parent,containing_map,a10);
  }
  else if (in_typeid==wxT("CompletedFilter")) {
    a11 = dynamic_cast<jfCompletedFilter*>(input_filter);
    result = new jfCompletedFilterEditor(parent,a11,containing_map);
  }
  else if (in_typeid==wxT("LanguageFilter")) {
    a12 = dynamic_cast<jfLanguageExprFilter*>(input_filter);
    result = new jfLanguageFilterEditor(parent,containing_map,a12);
  }
  else if (in_typeid==wxT("FFNRatingFilter")) {
    a13 = dynamic_cast<jfFFNRatingFilter*>(input_filter);
    result = new jfFFN_RatingFilterEditor(parent,a13,containing_map);
  }
  else if (in_typeid==wxT("UpdatedDateFilter")) {
    a14 = dynamic_cast<jfUpdatedDateFilter*>(input_filter);
    result = new jfUpdatedDateFilterEditor(parent,a14, containing_map);
  }
  else if (in_typeid==wxT("PublishedDateFilter")) {
    a15 = dynamic_cast<jfPublishedDateFilter*>(input_filter);
    result = new jfPublishedDateFilterEditor(parent,a15,containing_map);
  }
  else if (in_typeid==wxT("FimThumbsFilter")) {
    a16 = dynamic_cast<jfFimThumbsFilter*>(input_filter);
    result = new jfFimThumbsFilterEditor(parent,containing_map,a16);
  }
  else if (in_typeid==wxT("FIMGenreFilter")) {
    a17 = dynamic_cast<jfFIMGenreFilter*>(input_filter);
    result = new jfFimGenreFilterEditor(parent,containing_map,a17);
  }
  else if (in_typeid==wxT("FIMCharacterFilter")) {
    a18 = dynamic_cast<jfFIMCharacterFilter*>(input_filter);
    result = new jfFimCharacterFilterEditor(parent,containing_map,a18);
  }
  else if (in_typeid==wxT("FimRatingFilter")) {
    a19 = dynamic_cast<jfFimRatingFilter*>(input_filter);
    result = new jfFIM_RatingFilterEditor(parent,a19,containing_map);
  }
  else if (in_typeid==wxT("ExtraTagFilter")) {
    a20 = dynamic_cast<jfExtraTagFilter*>(input_filter);
    result = new jfExtraTagFilterEditor(parent,containing_map,a20);
  }
  else if (in_typeid==wxT("AO3FandomFilter")) {
    a21 = dynamic_cast<jfAO3FandomFilter*>(input_filter);
    result = new jfAO3FandomFilterEditor(parent,containing_map,a21);
  }
  else if (in_typeid==wxT("AO3RatingFilter")) {
    a22 = dynamic_cast<jfAO3RatingFilter*>(input_filter);
    result = new jfAO3_RatingFilterEditor(parent,a22,containing_map);
  }
  else if (in_typeid==wxT("AO3OrientationFilter")) {
    a23 = dynamic_cast<jfAO3OrientationFilter*>(input_filter);
    result = new jfAO3_OrientFilterEditor(parent,a23,containing_map);
  }
  else if (in_typeid==wxT("AO3PairFilter")) {
    a24 = dynamic_cast<jfAO3PairFilter*>(input_filter);
    result = new jfAO3_PairingFilterEditor(parent,a24,containing_map);
  }
  else if (in_typeid==wxT("AO3WarnFilter")) {
    a25 = dynamic_cast<jfAO3WarnFilter*>(input_filter);
    result = new jfAO3_WarnFilterEditor(parent,a25,containing_map);
  }
  else if (in_typeid==wxT("FFNFavsFilter")) {
    a26 = dynamic_cast<jfFFNFavsFilter*>(input_filter);
    result = new jfFFNFavsFilterEditor(parent,containing_map,a26);
  }
  else if (in_typeid==wxT("FIMGroupSCFilter")) {
    a27 = dynamic_cast<jfFIMGroupSCFilter*>(input_filter);
    result = new jfFIMGroupSCFilterEditor(parent,containing_map,a27);
  }
  else if (in_typeid==wxT("FIMGroupMCFilter")) {
    a28 = dynamic_cast<jfFIMGroupMCFilter*>(input_filter);
    result = new jfFIMGroupMCFilterEditor(parent,containing_map,a28);
  }
  else if (in_typeid==wxT("FimThumbPercentFilter")) {
    a29 = dynamic_cast<jfFimThumbPercentFilter*>(input_filter);
    result = new jfFimThumbPercentFilterEditor(parent,a29,containing_map);
  }
  else if (in_typeid==wxT("AO3KudoCountFilter")) {
    a30 = dynamic_cast<jfAO3KudoFilter*>(input_filter);
    result = new jfAO3KudoFilterEditor(parent,containing_map,a30);
  }
  else if (in_typeid==wxT("FIMShortDescFilter")) {
    a31 = dynamic_cast<jfFIM_ShortDesc_ExprFilter*>(input_filter);
    result = new jfFIMShortDesc_FilterEditor(parent,containing_map,a31);
  }
  else if (in_typeid==wxT("FIMDualDescFilter")) {
    a32 = dynamic_cast<jfFIM_DualDesc_ExprFilter*>(input_filter);
    result = new jfFIMDualDesc_FilterEditor(parent,containing_map,a32);
  }
  else assert(false);
  // done
  return result;
}
//***************************************************************************
