/******************************************************************************
Name    :   sexpr_edit.cpp
Author  :   John Q Metro
Purpose :   Simple Expression editors
Created :   December 4, 2010
Updated :   August 22, 2012
******************************************************************************/
#ifndef SEXPR_EDIT_H_INCLUDED
  #include "sexpr_edit.h"
#endif // SEXPR_EDIT_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
//****************************************************************************
jfSimpleExprEdit::jfSimpleExprEdit(wxWindow* parent, wxWindowID id, bool mline):wxPanel(parent,id) {
  // we start with checking the inmap
  error_code = 1;
  // sw start with button creation
  fcheck_btn = new wxButton(this,jfID_EXPR_EDITOR_CHECK,wxT("Check"));
  // creating the top sizer, which is always horizontal
  topsizer = new wxBoxSizer(wxHORIZONTAL);
  // we create and arrange
  editcore = new wxTextCtrl(this,jfID_EXPR_EDITOR_EDIT,wxT(""),wxDefaultPosition,
                            wxDefaultSize,(mline)?(wxTE_MULTILINE):(wxTE_LEFT));
  // we start arranging
  topsizer->Add(editcore,1,wxEXPAND|wxALL|wxALIGN_CENTER_VERTICAL,3);
  topsizer->Add(fcheck_btn,0,wxALL ,3);
  // done with creation and sizing
  SetSizer(topsizer);
  // topsizer->RecalcSizes();
}
//--------------------------------------------------------------------------
void jfSimpleExprEdit::PressCheckFilter(wxCommandEvent& event) {
  // local variables
  const wxString rvalid = wxT("Expression Valid");
  const wxString nvalid = wxT("Expression Not Valid");
  wxString dstring, errstring;
  jfSimpleExpr *result_expression;
  wxMessageDialog* message;
  // starting
  result_expression = CheckFilter(errstring);
  if (result_expression==NULL) {
    // we are in error here
    message = new wxMessageDialog(this,errstring,nvalid,wxOK);
    message->ShowModal();
  }
  else {
    // no error
    if (result_expression->IsEmpty()) dstring = wxT("Empty expressions match everything.");
    else dstring = wxT("Everthing okay.");
    message = new wxMessageDialog(this,dstring,rvalid,wxOK);
    message->ShowModal();
  }
  // done
  delete result_expression;
  message->Destroy();
}
//--------------------------------------------------------------------------
void jfSimpleExprEdit::TextChanged(wxCommandEvent& event) {
  SetStatus(1);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfSimpleExpr* jfSimpleExprEdit::CheckFilter(wxString& outmessage) {
  // local variables
  wxString dstring;
  jfSimpleExpr *result_expression;
  // starting...
  dstring = editcore->GetValue();
  // empty expresssions
  if (dstring.IsEmpty()) {
    outmessage = wxT("Empty expressions match everything.");
    result_expression = new jfSimpleExpr(dstring);
    SetStatus(0);
    return result_expression;
  }
  // non-empty expressions
  else {
    result_expression = new jfSimpleExpr();
    if (!result_expression->FromString(dstring)) {
      outmessage = result_expression->parse_error;
      delete result_expression;
      SetStatus(2);
      return NULL;
    }
    outmessage=wxT("No errors detected.");
    SetStatus(0);
    return result_expression;
  }
}
//-----------------------------------------------------------------------------
/* This method treats in input string as an expression and reports on it's
validity */
bool jfSimpleExprEdit::CheckExpr(const wxString& inexpr,wxString& outmessage) const {
  // local variables
  jfSimpleExpr *result_expression;
  // starting...
  // empty expresssions
  if (inexpr.IsEmpty()) {
    outmessage = wxT("Empty expressions match everything.");
    return true;
  }
  // non-empty expressions
  else {
    result_expression = new jfSimpleExpr();
    if (!result_expression->FromString(inexpr)) {
      outmessage = result_expression->parse_error;
      delete result_expression;
      return false;
    }
    // we check for validity
    outmessage=wxT("No errors detected.");
    delete result_expression;
    return true;
  }
}
//-----------------------------------------------------------------------------
// setting data
bool jfSimpleExprEdit::SetData(const wxString& inexpr,wxString& outmessage) {
  // getting the check result
  bool chkres = CheckExpr(inexpr,outmessage);
  editcore->ChangeValue(inexpr);
  if (chkres) SetStatus(0);
  else SetStatus(2);
  return chkres;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal helper
void jfSimpleExprEdit::SetStatus(size_t instatus) {
  assert(instatus<3);
  if (instatus==0) {
    editcore->SetForegroundColour(*wxGREEN);
    editcore->SetModified(false);
  }
  else if (instatus==1) {
    editcore->SetForegroundColour(wxNullColour);
    editcore->SetModified(true);
  }
  else {
    editcore->SetForegroundColour(*wxRED);
    editcore->SetModified(false);
  }
  error_code = instatus;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfSimpleExprEdit,wxPanel)
  EVT_BUTTON(jfID_EXPR_EDITOR_CHECK,jfSimpleExprEdit::PressCheckFilter)
  EVT_TEXT(jfID_EXPR_EDITOR_EDIT,jfSimpleExprEdit::TextChanged)
END_EVENT_TABLE()
//****************************************************************************
//****************************************************************************
