/******************************************************************************
Name    :   sfilt_edit.cpp
Author  :   John Q Metro
Purpose :   Filter editors (wraaping the core). Simple expression & mo0re
Created :   December 5, 2010
Updated :   November 15, 2011
******************************************************************************/
#ifndef SFILT_EDIT_H_INCLUDED
  #include "sfilt_edit.h"
#endif // SFILT_EDIT_H_INCLUDED
//+++++++++++++++++++++++++++++++++++

//****************************************************************************
// messaages
const wxString pfmsg = wxT("Problem with filter contents!");
/*****************************************************************************/
// the default constructor
jfSimpleSFilterEditor::jfSimpleSFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, const jfSimpleExpFilterCore* infilt, wxWindowID id, const wxString&
      name):jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
  wxString omsg;
  assert(infmap!=NULL);
  typed_fpointer = infilt;
  // we create the insert.. the *actual* expression editor
  insert_panel = new jfSimpleExprEdit(this,-1,true);
  insert_panel->SetData(filt_pointer->ToString(),omsg);
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
  if (infilt!=NULL) NameLoad();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfSimpleSFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  assert(infilter!=NULL);
  wxString omsg;
  // loading the data
  filt_pointer = infilter;
  insert_panel->SetData(filt_pointer->ToString(),omsg);
  NameLoad();
}
//------------------------------------------------------------------------------
jfBaseFilter* jfSimpleSFilterEditor::GetFilter() {
  jfSimpleExpr* toutput;
  jfSimpleExpFilterCore* result;
  wxString omsg;
  // getting the output
  toutput = insert_panel->CheckFilter(omsg);
  result = MakeTypedFilter(toutput);
  namedesc_edit->ChangeObj(result);
  // done
  return result;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checks if the contents are ok, returns false if they are not
bool jfSimpleSFilterEditor::GeneralCheck() const {
  // local variables
  jfSimpleExpr* toutput;
  wxString omsg;
  // first we see if the name is correct
  if (NameNUniq()) return false;
  // next, we check the filter
  toutput = insert_panel->CheckFilter(omsg);
  if ((toutput!=NULL) && (toutput->IsValid())) {
    // things are ok
    delete toutput;
    return true;
  }
  else {
    // things are not ok
    wxMessageBox(omsg,pfmsg,wxCANCEL|wxICON_ERROR);
    if (toutput!=NULL) delete toutput;
    return false;
  }
}
/*****************************************************************************/
// the default constructor
jfAuthorFilterEditor::jfAuthorFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, const jfAuthExprFilter* infilt, wxWindowID id, const wxString&
      name):jfSimpleSFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// creating a filter from a simple expression
jfSimpleExpFilterCore* jfAuthorFilterEditor::MakeTypedFilter(jfSimpleExpr* inpval) {
  jfAuthExprFilter* result = new jfAuthExprFilter(inpval);
  return result;
}
//================================================================================
// the default constructor
jfCharFilterEditor::jfCharFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, const jfCharExprFilter* infilt, wxWindowID id, const wxString&
      name):jfSimpleSFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// creating a filter from a simple expression
jfSimpleExpFilterCore* jfCharFilterEditor::MakeTypedFilter(jfSimpleExpr* inpval) {
  jfCharExprFilter* result = new jfCharExprFilter(inpval);
  return result;
}
//================================================================================
// the default constructor
jfCategoryFilterEditor::jfCategoryFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, const jfCategoryExprFilter* infilt, wxWindowID id, const wxString&
      name):jfSimpleSFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// creating a filter from a simple expression
jfSimpleExpFilterCore* jfCategoryFilterEditor::MakeTypedFilter(jfSimpleExpr* inpval) {
  jfCategoryExprFilter* result = new jfCategoryExprFilter(inpval);
  return result;
}
//================================================================================
// the default constructor
jfLanguageFilterEditor::jfLanguageFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, const jfLanguageExprFilter* infilt, wxWindowID id, const wxString&
      name):jfSimpleSFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// creating a filter from a simple expression
jfSimpleExpFilterCore* jfLanguageFilterEditor::MakeTypedFilter(jfSimpleExpr* inpval) {
  jfLanguageExprFilter* result = new jfLanguageExprFilter(inpval);
  return result;
}

//****************************************************************************
