/******************************************************************************
Name    :   filters_edit1.h
Author  :   John Q Metro
Purpose :   Editors for more filters : word count
Created :   December 6, 2010
Updated :   November 7, 2015
******************************************************************************/
#ifndef FILTERS_EDIT1_H_INCLUDED
  #include "filters_edit1.h"
#endif // FILTERS_EDIT1_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//******************************************************************************/
jfMinMaxUPanel::jfMinMaxUPanel(wxWindow* parent, wxWindowID id, jfMinMaxUFilter* inobj,
    const wxString& name):wxPanel(parent,id, wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // we start creating objects
  minl = new wxStaticText(this,-1,wxT("min"));
  maxl = new wxStaticText(this,-1,wxT("max"));
  minbox = new wxSpinCtrl(this,-1);
  maxbox = new wxSpinCtrl(this,-1);
  minbox->SetRange(0,20000000);
  maxbox->SetRange(0,20000000);
  // sizer creation
  msizer = new wxFlexGridSizer(2,2,1,1);
  // adding items to the sizer
  msizer->Add(minl,1,wxALIGN_CENTRE);
  msizer->Add(maxl,1,wxALIGN_CENTRE);
  msizer->Add(minbox,1,wxALIGN_CENTRE|wxEXPAND);
  msizer->Add(maxbox,1,wxALIGN_CENTRE|wxEXPAND);
  // setting the data
  if (inobj!=NULL) SetFromObj(inobj);
  // finishing sizer stuff
  SetSizer(msizer);
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfMinMaxUPanel::Check() const {
  // variables
  size_t qmin, qmax;
  return CheckCore(qmin,qmax);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// set
bool jfMinMaxUPanel::SetFromObj(const jfMinMaxUFilter* inval) {
  const wxString fname = wxT("jfMinMaxUPanel::SetFromObj");
  // the cases where things are wrong
  if (inval==NULL) return false;
  if (!(inval->IsValid())) return false;
  // setting things
  minbox->SetValue(inval->GetMin());
  maxbox->SetValue(inval->GetMax());
  return true;
}
//-------------------------------------------------------------------
bool jfMinMaxUPanel::LoadToObj(jfMinMaxUFilter* outval) {
  // checking
  assert(outval!=NULL);
    // constants
  const wxString emsgtitle = wxT("Problem with values");
  const wxString emsgmsg1 = wxT("The maximum count is smaller than the minimum.");
  const wxString emsgmsg2 = wxT("Invalid numbers");
  // variables
  size_t qmin, qmax;
  wxMessageDialog* message;
  bool lres;
  wxString emsgmsg;
  // test
  lres =  CheckCore(qmin,qmax);
  if (lres) {
    // the result
    outval->SetValues(qmin,qmax);
    return true;
  }
  else {
    emsgmsg = (lres)?(emsgmsg1):(emsgmsg2);
    message = new wxMessageDialog(const_cast<jfMinMaxUPanel*>(this),emsgmsg,emsgtitle,wxOK);
    message->ShowModal();
    message->Destroy();
    return false;
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfMinMaxUPanel::CheckCore(size_t& qmin, size_t& qmax) const {
  // test
  qmin = minbox->GetValue();
  qmax = maxbox->GetValue();
  return ((qmax>=qmin) || (qmax==0));
}
//===========================================================================
jfWCFilterEditor::jfWCFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, const jfBaseFilter* infilt, wxWindowID id, const wxString&
      name):jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
  // we create the insert.. the *actual* expression editor
  insert_panel = new jfMinMaxUPanel(this,-1,NULL);
  if (filt_pointer!=NULL) {
    insert_panel->SetFromObj(dynamic_cast<const jfWordCountFilter*>(filt_pointer));
  }
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfWCFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  const wxString fname = wxT("jfWCFilterEditor::LoadFilter");
  assert(infilter!=NULL);
  // loading the data
  filt_pointer = infilter;
  insert_panel->SetFromObj(dynamic_cast<const jfWordCountFilter*>(filt_pointer));
  NameLoad();
}
//------------------------------------------------------------------------------
jfBaseFilter* jfWCFilterEditor::GetFilter() {
  jfWordCountFilter* toutput;
  // getting the output
  toutput = new jfWordCountFilter();
  insert_panel->LoadToObj(toutput);
  namedesc_edit->ChangeObj(toutput);
  // done
  return toutput;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checks if the contents are ok, returns false if they are not
bool jfWCFilterEditor::GeneralCheck() const {
  return insert_panel->Check();
}
//===============================================================================
// the default constructor
jfTagFilterEditor::jfTagFilterEditor(wxWindow* parent, const jfFilterMap* infmap,
    const jfTagFilterCore* infilt, wxWindowID id, const wxString& name ):
    jfBaseFilterEditor(parent,infmap,infilt,id ,name) {
  // variables and constants
  jfTagListing* tvalue;
  // we create the insert.. the *actual* expression editor
  insert_panel = new jfTagCategorizer(this,-1,true,NULL);
  // checking the inputted filter
  if (infilt!=NULL) {
    assert(infilt->IsValid());
    tvalue = infilt->GetTagsCopy();
    insert_panel->SetFromSource(tvalue);
  }
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methods
//----------------------------------------------
void jfTagFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  // variables and constants
  const jfTagFilterCore* tvalue;
  jfTagListing* tcopy;
  // c hecking and converting the input
  assert(infilter!=NULL);
  filt_pointer = infilter;
  tvalue = dynamic_cast<const jfTagFilterCore*>(infilter);
  tcopy = tvalue->GetTagsCopy();
  insert_panel->SetFromSource(tcopy);
  NameLoad();
}
//---------------------------------------------
jfBaseFilter* jfTagFilterEditor::GetFilter() {
  // variables
  jfTagFilterCore* tvalue;
  const jfTagListing* tcopy;
  bool testval;
  // creating filters and loading tags
  tvalue = GetTagFilter();
  tcopy = insert_panel->GetList();
  testval = tvalue->SetTags(tcopy);
  // name setting
  namedesc_edit->ChangeObj(tvalue);
  return tvalue;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checking
bool jfTagFilterEditor::GeneralCheck() const {
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfTagFilterEditor::LoadBlank() {
  jfTagFilterCore* tvalue = MakeBlankTagFilter();
  LoadFilter(tvalue);
  delete tvalue;
}
//===============================================================================
// the default constructor
jfFFNTagFilterEditor::jfFFNTagFilterEditor(wxWindow* parent, const jfFilterMap* infmap,
    const jfFFNTagFilter* infilt, wxWindowID id, const wxString& name):
    jfTagFilterEditor(parent,infmap,infilt,id,name ) {
  if (infilt==NULL) LoadBlank();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// extra methods
//---------------------------
jfTagFilterCore* jfFFNTagFilterEditor::GetTagFilter() {
  jfFFNTagFilter* result;
  result = new jfFFNTagFilter();
  result->SetToEmpty();
  return result;
}
//---------------------------
jfTagFilterCore* jfFFNTagFilterEditor::MakeBlankTagFilter() {
  jfFFNTagFilter* result;
  bool atest;
  result = new jfFFNTagFilter();
  atest = result->SetTags(insert_panel->GetList());
  assert(atest);
  return result;
}

//******************************************************************************/

