/******************************************************************************
Name    :   mfilt_edit1.cpp
Author  :   John Q Metro
Purpose :   Declares some base class for filter diting
Created :   November 11, 2011
Updated :   August 30, 2014
******************************************************************************/
#ifndef MFILT_EDIT1_H_INCLUDED
  #include "mfilt_edit1.h"
#endif // MFILT_EDIT1_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
//==========================================================================
/*****************************************************************************/
jfCFE_Core1::jfCFE_Core1(wxWindow* parent, wxWindowID id,const jfCompletedFilter* inmap):
      wxPanel(parent,id, wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,wxT("CompletedFilterEditor")) {
  // we start
  mainval1 = new wxCheckBox(this,-1,wxT("Completed Stories (check to include, blank to exclude)"),
            wxDefaultPosition,wxDefaultSize,wxCHK_ALLOW_3RD_STATE_FOR_USER|wxCHK_3STATE);
  mainsizer = new wxBoxSizer(wxHORIZONTAL);
  mainsizer->Add(mainval1,wxALL|wxEXPAND,3);
  SetSizer(mainsizer);
  // setting the value
  if (inmap==NULL) mainval1->Set3StateValue(wxCHK_UNDETERMINED);
  else LoadValue(inmap);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting and getting values
//--------------------------------------------------------------------
bool jfCFE_Core1::StoreValue(jfCompletedFilter* saveto) const {
  if (saveto==NULL) return false;
  else {
    if ((mainval1->Get3StateValue())==wxCHK_UNDETERMINED) saveto->SetEmpty();
    else saveto->SetValue((mainval1->Get3StateValue())==wxCHK_CHECKED);
    return true;
  }
}
//--------------------------------------------------------------------
bool jfCFE_Core1::LoadValue(const jfCompletedFilter* getfrom) {
  if (getfrom==NULL) return false;
  else {
    if (getfrom->IsEmpty()) mainval1->Set3StateValue(wxCHK_UNDETERMINED);
    else {
      mainval1->SetValue((getfrom->GetValue())?(wxCHK_CHECKED):(wxCHK_UNCHECKED));
    }
    return true;
  }
}
//--------------------------------------------------------------------
jfCompletedFilter* jfCFE_Core1::GetValue() const {
  jfCompletedFilter* result = new jfCompletedFilter();
  bool qval = StoreValue(result);
  return result;
}
//=============================================================================
jfComplFiltEdit::jfComplFiltEdit(wxWindow* parent, wxWindowID id,const jfCompletedFilter* inmap):jfCFE_Core1(parent,id,inmap) {}
//*****************************************************************************
jfCompletedFilterEditor::jfCompletedFilterEditor(wxWindow* parent,const jfBaseFilter* infilt,
          const jfFilterMap* infmap, wxWindowID id, const wxString& name):
          jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // we start...
  // we create the insert.. the *actual*  editor
  insert_panel = new jfComplFiltEdit(this,-1,dynamic_cast<const jfCompletedFilter*>(infilt));
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methods
//-------------------------------------
void jfCompletedFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  assert(infilter!=NULL);
  filt_pointer = infilter;
  insert_panel->LoadValue(dynamic_cast<const jfCompletedFilter*>(filt_pointer));
  NameLoad();
}
//-------------------------------------
jfBaseFilter* jfCompletedFilterEditor::GetFilter() {
  jfCompletedFilter* result = insert_panel->GetValue();
  namedesc_edit->ChangeObj(result);
  return result;
}
//--------------------------------------------------------------
bool jfCompletedFilterEditor::GeneralCheck() const {
  return true;
}
//============================================================================
// constructor
jfFFN_RatingEditPanel::jfFFN_RatingEditPanel(wxWindow* parent, wxWindowID id, bool horizontal,
          const jfFFNRatingFilter* invalue):wxPanel(parent,id, wxDefaultPosition, wxDefaultSize,wxTAB_TRAVERSAL,wxT("FFN_RatingEditPanel")) {
  // creating gui elemnts
  kch = new wxCheckBox(this,-1,wxT("K"));
  kpch = new wxCheckBox(this,-1,wxT("K+"));
  tch = new wxCheckBox(this,-1,wxT("T"));
  mch = new wxCheckBox(this,-1,wxT("M"));
  // border styles
  const int lohs = wxTOP|wxBOTTOM|wxRIGHT;
  int thirdborder;
  // sizer creation
  ishoriz = horizontal;
  if (ishoriz) {
    fsizer=NULL;
    bsizer = new wxBoxSizer(wxHORIZONTAL);
    thesizer = bsizer;
    thirdborder = lohs;
  }
  else {
    bsizer = NULL;
    fsizer = new wxFlexGridSizer(2,2,0.0);
    thesizer = fsizer;
    thirdborder = wxALL;
  }
  // adding items in the sizer
  thesizer->Add(kch,0,wxALL,3);
  thesizer->Add(kpch,0,lohs,3);
  thesizer->Add(tch,0,thirdborder,3);
  thesizer->Add(mch,0,lohs,3);
  // done
  SetSizer(thesizer);
  LoadValue(invalue);

}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting and getting values
//--------------------------------------------------------------
bool jfFFN_RatingEditPanel::StoreValue(jfFFNRatingFilter* saveto) const {
  if (saveto==NULL) return false;
  wxString inval;
  if (kch->IsChecked()) inval +=wxT("K");
  if (kpch->IsChecked()) inval +=wxT("+");
  if (tch->IsChecked()) inval +=wxT("T");
  if (mch->IsChecked()) inval +=wxT("M");
  saveto->FromString(inval);
  return true;
}
//--------------------------------------------------------------
bool jfFFN_RatingEditPanel::LoadValue(const jfFFNRatingFilter* getfrom) {
  if (getfrom==NULL) return false;
  kch->SetValue(getfrom->Includes(wxT('K')));
  kpch->SetValue(getfrom->Includes(wxT('+')));
  tch->SetValue(getfrom->Includes(wxT('T')));
  mch->SetValue(getfrom->Includes(wxT('M')));
  return true;
}
//--------------------------------------------------------------
jfFFNRatingFilter* jfFFN_RatingEditPanel::GetValue() const {
  jfFFNRatingFilter* resval = new jfFFNRatingFilter();
  StoreValue(resval);
  return resval;
}
/*****************************************************************************/
jfFFN_RatingFilterEditor::jfFFN_RatingFilterEditor(wxWindow* parent,const jfBaseFilter* infilt,
          const jfFilterMap* infmap, wxWindowID id, const wxString& name):
          jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // we start...
  // we create the insert.. the *actual*  editor
  insert_panel = new jfFFN_RatingEditPanel(this,-1,false,dynamic_cast<const jfFFNRatingFilter*>(infilt));
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methods
//-------------------------------------
void jfFFN_RatingFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  assert(infilter!=NULL);
  filt_pointer = infilter;
  insert_panel->LoadValue(dynamic_cast<const jfFFNRatingFilter*>(filt_pointer));
  NameLoad();
}
//-------------------------------------
jfBaseFilter* jfFFN_RatingFilterEditor::GetFilter() {
  jfFFNRatingFilter* result = insert_panel->GetValue();
  namedesc_edit->ChangeObj(result);
  return result;
}
//--------------------------------------------------------------
bool jfFFN_RatingFilterEditor::GeneralCheck() const {
  return true;
}
//*****************************************************************************
jfDateRangePanel::jfDateRangePanel(wxWindow* parent, wxWindowID id):wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,wxT("DateRangePanel")) {

  // making the labels first
  from_label = new wxStaticText(this,-1,wxT("From :"));
  to_label = new wxStaticText(this,-1,wxT("Back to :"));
  // making the checkboxes
  undefined_new = new wxCheckBox(this,jfID_DRANGEP_NEWDATE,wxT("Undefined"));
  undefined_old = new wxCheckBox(this,jfID_DRANGEP_OLDDATE,wxT("Undefined"));
  // making the other gui elements
  MakeBlock(true);
  MakeBlock(false);
  // TOP LEVEL SIZING !!
  topsizer = new wxFlexGridSizer(2,2,1,3);
  topsizer->Add(fsizer,1,wxEXPAND,0);
  topsizer->Add(tsizer,1,wxEXPAND,0);
  topsizer->Add(undefined_new,1,wxEXPAND,0);
  topsizer->Add(undefined_old,1,wxEXPAND,0);
  // done
  SetSizer(topsizer);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting and getting values
//---------------------------------
bool jfDateRangePanel::CheckOk() const {
  // undefined is always okay
  if ((undefined_new->IsChecked()) || (undefined_old->IsChecked())) return true;
  // otherwise, we check the dates
  wxDateTime newdatex, oldatex;
  newdatex = newdate->GetValue();
  oldatex = oldate->GetValue();
  if (oldatex<=newdatex) return true;
  else return false;
}
//---------------------------------
bool jfDateRangePanel::StoreValue(jfDateRangeFilter* saveto) const {
  // returning false...
  if (saveto==NULL) return false;
  if (!CheckOk()) return false;
  // extracting the values
  wxDateTime newdatex, olddatex;
  if (undefined_new->IsChecked()) newdatex = wxInvalidDateTime;
  else newdatex = newdate->GetValue();
  if (undefined_old->IsChecked()) olddatex = wxInvalidDateTime;
  else olddatex = oldate->GetValue();
  // entering the valuies
  saveto->SetDates(newdatex,olddatex);
  return true;
}
//---------------------------------
bool jfDateRangePanel::LoadValue(const jfDateRangeFilter* getfrom) {
  // faliure options
  if (getfrom==NULL) return false;
  if (!getfrom->IsValid()) return false;
  // getting the from date
  wxDateTime dvalue = getfrom->GetValue(true);
  if (!dvalue.IsValid()) {
    undefined_new->SetValue(true);
    newdate->Enable(false);
  }
  else {
    newdate->Enable(true);
    undefined_new->SetValue(false);
    newdate->SetValue(dvalue);
  }
  // getting the to date
  dvalue = getfrom->GetValue(false);
  if (!dvalue.IsValid()) {
    undefined_old->SetValue(true);
    oldate->Enable(false);
  }
  else {
    oldate->Enable(true);
    undefined_old->SetValue(false);
    oldate->SetValue(dvalue);
  }
  // done
  return true;
}
//---------------------------------
void jfDateRangePanel::MakeUndefined() {
  undefined_new->SetValue(true);
  newdate->Enable(false);
  undefined_old->SetValue(true);
  oldate->Enable(false);
}
//---------------------------------
void jfDateRangePanel::ShowErrorMsg() const {
  // constants and variables
  const wxString errtitle = wxT("Date Error");
  const wxString errmsg = wxT("The \'From\' date must not be older than the \'Back to\' date!");
  wxMessageDialog* message;
  wxWindow* this_parent;
  // doing
  this_parent = const_cast<jfDateRangePanel*>(this);
  message = new wxMessageDialog(this_parent,errmsg,errtitle,wxOK);
  message->ShowModal();
  message->Destroy();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// events
//---------------------------------
void jfDateRangePanel::EvUndefinedFrom(wxCommandEvent& inevent) {
  newdate->Enable(!undefined_new->IsChecked());
}
//---------------------------------
void jfDateRangePanel::EvUndefinedTo(wxCommandEvent& inevent) {
  oldate->Enable(!undefined_old->IsChecked());
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// private methods
bool jfDateRangePanel::MakeBlock(bool from) {
  // variables
  wxBoxSizer* thesizer;
  wxDatePickerCtrl*  thedatepicker;
  // doing it
  thesizer = new wxBoxSizer(wxHORIZONTAL);
  thedatepicker = new wxDatePickerCtrl(this,-1,wxDefaultDateTime,wxDefaultPosition,wxDefaultSize,wxDP_SHOWCENTURY);
  // adding to the sizer
  thesizer->Add((from)?(from_label):(to_label),0,wxLEFT|wxTOP|wxBOTTOM|wxALIGN_CENTER_VERTICAL,3);
  thesizer->Add(thedatepicker,1,wxRIGHT|wxTOP|wxBOTTOM|wxEXPAND,3);
  // setting the values
  if (from) {
    newdate = thedatepicker;
    fsizer = thesizer;
  }
  else {
    oldate = thedatepicker;
    tsizer = thesizer;
  }
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfDateRangePanel,wxPanel)
  EVT_CHECKBOX(jfID_DRANGEP_NEWDATE,jfDateRangePanel::EvUndefinedFrom)
  EVT_CHECKBOX(jfID_DRANGEP_OLDDATE,jfDateRangePanel::EvUndefinedTo)
END_EVENT_TABLE()
//*****************************************************************************
jfFFNFavsFilterEditor::jfFFNFavsFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, const jfBaseFilter* infilt, wxWindowID id, const wxString&
      name):jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
  // we create the insert.. the *actual* expression editor
  insert_panel = new jfMinMaxUPanel(this,-1,NULL);
  if (filt_pointer!=NULL) {
    insert_panel->SetFromObj(dynamic_cast<const jfFFNFavsFilter*>(filt_pointer));
  }
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,0,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfFFNFavsFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  const wxString fname = wxT("jfFFNFavsFilterEditor::LoadFilter");
  assert(infilter!=NULL);
  // loading the data
  filt_pointer = infilter;
  insert_panel->SetFromObj(dynamic_cast<const jfFFNFavsFilter*>(filt_pointer));
  NameLoad();
}
//------------------------------------------------------------------------------
jfBaseFilter* jfFFNFavsFilterEditor::GetFilter() {
  jfFFNFavsFilter* toutput;
  // getting the output
  toutput = new jfFFNFavsFilter();
  insert_panel->LoadToObj(toutput);
  namedesc_edit->ChangeObj(toutput);
  // done
  return toutput;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checks if the contents are ok, returns false if they are not
bool jfFFNFavsFilterEditor::GeneralCheck() const {
  return insert_panel->Check();
}
/*****************************************************************************/
