//***************************************************************************
// Name:        pairf_editor.cpp
// Author :     John Q Metro
// Purpose :    Relationship/Pairing filters editing, various panels.
// Created:     January 1, 2011
// Updated:     April 7, 2013
//***************************************************************************
#ifndef PAIRF_EDITOR_H_INCLUDED
  #include "pairf_editor.h"
#endif // PAIRF_EDITOR_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//***************************************************************************
// the constructor
jfPairFilEditPanelCore::jfPairFilEditPanelCore(wxWindow* parent,wxWindowID id,
      const wxString& name):wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // we start with the special phrases editor
  spec_label = new wxStaticText(this,-1,wxT("Special pairing keywords (separated by semicolon) :"));
  spec_edit = new wxTextCtrl(this,-1);
  // next, the characters used the separate names in a a pairing
  delim_label = new wxStaticText(this,-1,wxT("Pairing separator characters :"));
  delim_edit = new wxTextCtrl(this,-1);
  delim_sizer = new wxBoxSizer(wxHORIZONTAL);
  delim_sizer->Add(delim_label,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL,1);
  delim_sizer->Add(delim_edit,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL,1);
  // the various checkboxes and thier sizer
  usetokens = new wxCheckBox(this,-1,wxT("Use tokens, not substrings"));
  sidespaces = new wxCheckBox(this,-1,wxT("Look for pairs with spaces around the separator as well."));
  nofriend = new wxCheckBox(this,-1,wxT("Make certain the word 'friendship' does not follow"));
  nospace = new wxCheckBox(this,-1,wxT("Also look for name pairs with no separation"));
  cb_sizer = new wxFlexGridSizer(2,2,2,2);
  cb_sizer->Add(usetokens);
  cb_sizer->Add(sidespaces);
  cb_sizer->Add(nofriend);
  cb_sizer->Add(nospace);
  // the top level sizer
  top_sizer = new wxBoxSizer(wxVERTICAL);
  top_sizer->Add(spec_label,0,wxALIGN_LEFT);
  top_sizer->Add(spec_edit,0,wxEXPAND);
  top_sizer->AddSpacer(3);
  top_sizer->Add(delim_sizer,0,wxEXPAND);
  top_sizer->AddSpacer(3);
  top_sizer->Add(cb_sizer,0,wxEXPAND);
  top_sizer->AddSpacer(3);

}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading data
bool jfPairFilEditPanelCore::LoadData(const jfPairFilterCore* inval) {
  if (inval==NULL) return false;
  // separators
  wxString buffer = inval->GetSeparators();
  delim_edit->SetValue(buffer);
  // specials
  buffer = inval->GetSpecials();
  spec_edit->SetValue(buffer);
  // checkboxes
  usetokens->SetValue((inval->GetFlagsRef()).usetokens );
  sidespaces->SetValue((inval->GetFlagsRef()).sidespaces );
  nofriend->SetValue((inval->GetFlagsRef()).nofriend );
  nospace->SetValue((inval->GetFlagsRef()).nospace );
  // done
  return true;

}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting data. more complicated because we have to diplay errors for bad data
bool jfPairFilEditPanelCore::SaveData(jfPairFilterCore* outval) const {
  // constants
  const wxString invalid = wxT("There is a problem with this Pairing Filter.");
  const wxString err1 = wxT("Is is not valid for the separators to be empty.");
  // local variables
  wxMessageDialog* message;
  wxString bufval;
  bool atest;
  // checks
  if (outval==NULL) return false;
  // we start with the separators
  bufval = delim_edit->GetValue();
  Dtrim(bufval);
  if (bufval.IsEmpty()) {
    message = new wxMessageDialog(NULL,err1,invalid,wxOK);
    message->ShowModal();
    delete message;
    return false;
  }
  // setting the separator string
  bufval = delim_edit->GetValue();
  atest = outval->SetSeparators(bufval);
  assert(atest);
  // the specials
  bufval = spec_edit->GetValue();
  outval->AddSpecials(bufval);
  // doing the booleans
  (outval->GetFlagsRef()).usetokens = usetokens->GetValue();
  (outval->GetFlagsRef()).sidespaces = sidespaces->GetValue();
  (outval->GetFlagsRef()).nofriend = nofriend->GetValue();
  (outval->GetFlagsRef()).nospace= nospace->GetValue();
  // done
  return true;
}
//-----------------------------------------------------------------------
bool jfPairFilEditPanelCore::CheckSeps() const {
  // constants
  const wxString invalid = wxT("There is a problem with this Pairing Filter.");
  const wxString err1 = wxT("Is is not valid for the separators to be empty.");
  // local variables
  wxString bufval;
  // getting separators
  bufval = delim_edit->GetValue();
  Dtrim(bufval);
  // are the separators empty or occupied?
  if (bufval.IsEmpty()) {
    wxMessageBox(err1,invalid,wxCANCEL|wxICON_ERROR);
    return false;
  }
  else return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checking function
bool jfPairFilEditPanelCore::CheckValid() const {
  if (!CheckSeps()) return false;
  return CheckNames();
}
//==================================================================================
// constructor
jfPairFilEditPanelSingle::jfPairFilEditPanelSingle(wxWindow* parent,const jfPairFilterSingle* infil,
      wxWindowID id,const wxString& name):jfPairFilEditPanelCore(parent,id,name) {
  // the gui elements we do here
  name1_label = new wxStaticText(this,-1,wxT("Names (semicolon separated) of the first person in the pairing."));
  name1_edit = new wxTextCtrl(this,-1);
  name2_label = new wxStaticText(this,-1,wxT("Names (semicolon separated) of the second person in the pairing."));
  name2_edit = new wxTextCtrl(this,-1);
  // adding to the sizer
  top_sizer->Add(name1_label,0,wxALIGN_LEFT);
  top_sizer->Add(name1_edit,0,wxEXPAND);
  top_sizer->AddSpacer(3);
  top_sizer->Add(name2_label,0,wxALIGN_LEFT);
  top_sizer->Add(name2_edit,0,wxEXPAND);
  // next..
  SetSizer(top_sizer);
  if (infil==NULL) {
    jfPairFilterSingle* tval = new jfPairFilterSingle();
    LoadFilterData(tval);
    delete tval;
  }
  else LoadFilterData(infil);
  // done
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// virtual load and store stuff
bool jfPairFilEditPanelSingle::LoadFilterData(const jfPairFilterCore* inval) {
  size_t ncount;
  bool atest;
  // various checks
  assert(inval!=NULL);
  ncount = inval->GetNamesetCount();
  assert(ncount<3);
  // handling the common bits
  atest = LoadData(inval);
  assert(atest);
  // setting the namesets
  if (ncount>0) name1_edit->SetValue(inval->GetNameset(0));
  if (ncount>1) name2_edit->SetValue(inval->GetNameset(1));
  // done
  return false;
}
//-----------------------------
bool jfPairFilEditPanelSingle::SaveFilterData(jfPairFilterCore* outval) const {
  // local variables
  wxString bufval1,bufval2;
  // checks
  if (outval==NULL) return false;
  if (!CheckNames()) return false;
  // getting the name sets
  bufval1 = name1_edit->GetValue();
  Dtrim(bufval1);
  bufval2 = name2_edit->GetValue();
  Dtrim(bufval2);
  // doing the core stuff
  if (!SaveData(outval)) return false;
  outval->AddNameset(bufval1);
  outval->AddNameset(bufval2);
  // done
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checking function
bool jfPairFilEditPanelSingle::CheckNames() const {
  // constants
  const wxString invalid = wxT("There is a problem with this Pairing Filter.");
  const wxString err1 = wxT("Neither of the names (or names list) should be empty.");
  // local variables
  wxString bufval1,bufval2;
  // getting the name sets
  bufval1 = name1_edit->GetValue();
  Dtrim(bufval1);
  bufval2 = name2_edit->GetValue();
  Dtrim(bufval2);
  // testing
  if (bufval1.IsEmpty() || bufval2.IsEmpty()) {
    wxMessageBox(err1,invalid,wxCANCEL|wxICON_ERROR);
    return false;
  }
  return true;
}
//==================================================================================
// constructor
jfPairFilEditPanelList::jfPairFilEditPanelList(wxWindow* parent,const jfPairFilterList* infil,
      wxWindowID id,const wxString& name):jfPairFilEditPanelCore(parent,id,name) {
  // the gui elements we do here
  name1_label = new wxStaticText(this,-1,wxT("Names (semicolon separated) of the first person in the pairing."));
  name1_edit = new wxTextCtrl(this,-1);
  namelist_label = new wxStaticText(this,-1,wxT("Names (semicolon separated) of the other people (separate lines) in the pairing "));
  namelist_edit = new wxTextCtrl(this,-1,wxEmptyString,wxDefaultPosition,wxDefaultSize,wxTE_MULTILINE|wxTE_DONTWRAP);
  // adding to the sizer
  top_sizer->Add(name1_label,0,wxALIGN_LEFT);
  top_sizer->Add(name1_edit,0,wxEXPAND);
  top_sizer->AddSpacer(3);
  top_sizer->Add(namelist_label,0,wxALIGN_LEFT);
  top_sizer->Add(namelist_edit,1,wxEXPAND);
  // next..
  SetSizer(top_sizer);
  if (infil==NULL) {
    jfPairFilterList* tval = new jfPairFilterList();
    LoadFilterData(tval);
    delete tval;
  }
  else LoadFilterData(infil);
  // done
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// virtual load and store stuff
bool jfPairFilEditPanelList::LoadFilterData(const jfPairFilterCore* inval) {
  size_t ncount,nloop;
  bool atest;
  // various checks
  assert(inval!=NULL);
  ncount = inval->GetNamesetCount();
  // handling the common bits
  atest = LoadData(inval);
  assert(atest);
  // setting the namesets
  if (ncount>0) name1_edit->SetValue(inval->GetNameset(0));
  namelist_edit->Clear();
  if (ncount>=2) {
    for (nloop=1;nloop<ncount;nloop++) {
      namelist_edit->AppendText(inval->GetNameset(nloop));
      if (nloop!=(ncount-1)) {
        namelist_edit->AppendText(wxT("\n"));
      }
    }
  }
  // done
  return false;
}
//-----------------------------
bool jfPairFilEditPanelList::SaveFilterData(jfPairFilterCore* outval) const {
  // local variables
  wxString bufval1,bufval2;
  size_t lloop,linenum;
  // checks
  if (outval==NULL) return false;
  if (!CheckNames()) return false;
  // doing the core stuff
  if (!SaveData(outval)) return false;
  // getting the first name
  bufval1 = name1_edit->GetValue();
  Dtrim(bufval1);
  outval->AddNameset(bufval1);
  // the writing loop
  // the writing loop
  linenum = namelist_edit->GetNumberOfLines();
  for (lloop=0;lloop<linenum;lloop++) {
    bufval1 = namelist_edit->GetLineText(lloop);
    Dtrim(bufval1);
    if (!bufval1.IsEmpty()) outval->AddNameset(bufval1);
  }
  // done
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checking function
bool jfPairFilEditPanelList::CheckNames() const {
  // constants
  const wxString invalid = wxT("There is a problem with this Pairing Filter.");
  const wxString err1 = wxT("Neither of the names (or names list) should be empty.");
  // local variables
  wxString bufval1,bufval2;
  // getting the name sets
  bufval1 = name1_edit->GetValue();
  Dtrim(bufval1);
  bufval2 = namelist_edit->GetValue();
  Dtrim(bufval2);
  // testing
  if (bufval1.IsEmpty() || bufval2.IsEmpty()) {
    wxMessageBox(err1,invalid,wxCANCEL|wxICON_ERROR);
    return false;
  }
  return true;
}
//==================================================================================
// constructor
jfPairFilEditPanelMulti::jfPairFilEditPanelMulti(wxWindow* parent,const jfPairFilterMultiple* infil,
      wxWindowID id,const wxString& name):jfPairFilEditPanelCore(parent,id,name) {
  // the gui elements we do here
  name_label = new wxStaticText(this,-1,wxT("Names (semicolon separated) of the people to be paired (one person per line)"));
  names_edit = new wxTextCtrl(this,-1,wxEmptyString,wxDefaultPosition,wxDefaultSize,wxTE_MULTILINE|wxTE_DONTWRAP);
  // adding to the sizer
  top_sizer->Add(name_label,0,wxALIGN_LEFT);
  top_sizer->Add(names_edit,1,wxEXPAND);
  // next..
  SetSizer(top_sizer);
  if (infil==NULL) {
    jfPairFilterList* tval = new jfPairFilterList();
    LoadFilterData(tval);
    delete tval;
  }
  else LoadFilterData(infil);
  // done
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// virtual load and store stuff
bool jfPairFilEditPanelMulti::LoadFilterData(const jfPairFilterCore* inval) {
  const wxString fname = wxT("jfPairFilEditPanelMulti::LoadFilterData");
  // variables
  wxString nameset;
  size_t ncount,nloop;
  bool atest;
  // various checks
  /**/jfXLogMessage(fname,1);
  assert(inval!=NULL);
  ncount = inval->GetNamesetCount();
  // handling the common bits
  atest = LoadData(inval);
  assert(atest);
  /**/jfXLogMessage(fname,2);
  // setting the namesets
  names_edit->Clear();
  if (ncount==0) return false;
  /**/jfXLogMessageS(fname,3,ncount);
  for (nloop=0;nloop<ncount;nloop++) {
    /**/jfXLogMessageS(fname,4,nloop);
    nameset = inval->GetNameset(nloop);
    /**/jfXLogMessage(fname,5,nameset);
    names_edit->AppendText(nameset);
    if (nloop!=(ncount-1)) {
      names_edit->AppendText(wxT("\n"));
    }
  }
  // done
  return false;
}
//-----------------------------
bool jfPairFilEditPanelMulti::SaveFilterData(jfPairFilterCore* outval) const {
  const wxString fname = wxT("jfPairFilEditPanelMulti::SaveFilterData");
  // local variables
  wxString bufval1;
  size_t linenum, lloop;
  // checks
  /**/jfXLogMessage(fname,1);
  if (outval==NULL) return false;
  /**/jfXLogMessage(fname,2);
  if (!CheckNames()) return false;
  // doing the core stuff
  /**/jfXLogMessage(fname,3);
  if (!SaveData(outval)) return false;
  // the writing loop
  linenum = names_edit->GetNumberOfLines();
  /**/jfXLogMessage(fname,4);
  for (lloop=0;lloop<linenum;lloop++) {
    /**/jfXLogMessageS(fname,5,linenum);
    bufval1 = names_edit->GetLineText(lloop);
    Dtrim(bufval1);
    /**/jfXLogMessage(fname,6,bufval1);
    if (!bufval1.IsEmpty()) outval->AddNameset(bufval1);
    /**/jfXLogMessageS(fname,7,lloop);
  }
  // done
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checking function
bool jfPairFilEditPanelMulti::CheckNames() const {
  // constants
  const wxString fname = wxT("jfPairFilEditPanelMulti::CheckNames");
  const wxString invalid = wxT("There is a problem with this Pairing Filter.");
  const wxString err1 = wxT("There must be at least two people (lines) in the names box.");
  // local variables
  size_t lloop, lsize;
  wxString line;
  size_t lcount;
  // initializing values
  lcount = names_edit->GetNumberOfLines();
  lsize = 0;
  /**/jfXLogMessageS(fname,1,lcount);
  // looping thru the lines in the listbox
  for (lloop=0;lloop<lcount;lloop++) {
    /**/jfXLogMessageS(fname,2,lloop);
    line = names_edit->GetLineText(lloop);
    Dtrim(line);
    if (line.length()>0) lsize++;
    /**/jfXLogMessage(fname,3);
  }
  // testing
  /**/jfXLogMessage(fname,4);
  if (lsize<2) {
    /**/jfXLogMessage(fname,5);
    wxMessageBox(err1,invalid,wxCANCEL|wxICON_ERROR);
    return false;
  }
  /**/jfXLogMessage(fname,6);
  return true;
}

//***************************************************************************
