//***************************************************************************
// Name:        pairf_editor.h
// Author :     John Q Metro
// Purpose :    Relationship/Pairing filters editing, various panels.
// Created:     January 1, 2011
// Updated:     January 2, 2011
//***************************************************************************
#ifndef PAIRF_EDITOR_H_INCLUDED
#define PAIRF_EDITOR_H_INCLUDED
#endif // PAIRF_EDITOR_H_INCLUDED
//----------------------------------------------------------------------------
#ifndef PAIRFILTER_H_INCLUDED
  #include "../../../core/filters/extended/pairfilter.h"
#endif // PAIRFILTER_H_INCLUDED
//***************************************************************************
// pair filter editing, the core panel
class jfPairFilEditPanelCore : public wxPanel {
  public:
    // the constructor
    jfPairFilEditPanelCore(wxWindow* parent,wxWindowID id = wxID_ANY,const wxString& name = wxT("jfPairFilEditPanelCore"));
    // virtual load and store stuff
    virtual bool LoadFilterData(const jfPairFilterCore* inval) = 0;
    virtual bool SaveFilterData(jfPairFilterCore* outval) const = 0;
    // checking function
    bool CheckValid() const;
  protected:
    // loading data
    bool LoadData(const jfPairFilterCore* inval);
    // getting data
    bool SaveData(jfPairFilterCore* outval) const;
    // other
    bool CheckSeps() const;
    virtual bool CheckNames() const = 0;
    // gui elements
    wxStaticText* spec_label;
    wxTextCtrl* spec_edit;
    wxStaticText* delim_label;
    wxTextCtrl* delim_edit;
    wxCheckBox* usetokens;
    wxCheckBox* sidespaces;
    wxCheckBox* nofriend;
    wxCheckBox* nospace;
    // sizers
    wxBoxSizer* delim_sizer;
    wxFlexGridSizer* cb_sizer;
    wxBoxSizer* top_sizer;

};
//==========================================================================
// single pair filter edit panel
class jfPairFilEditPanelSingle : public jfPairFilEditPanelCore {
  public:
    // constructor
    jfPairFilEditPanelSingle(wxWindow* parent,const jfPairFilterSingle* infil,wxWindowID id = wxID_ANY,const wxString& name = wxT("jfPairFilEditPanelSingle"));
    // virtual load and store stuff
    virtual bool LoadFilterData(const jfPairFilterCore* inval);
    virtual bool SaveFilterData(jfPairFilterCore* outval) const;
    // checking function
  protected:
    virtual bool CheckNames() const;
    // gui elements
    wxStaticText* name1_label;
    wxTextCtrl* name1_edit;
    wxStaticText* name2_label;
    wxTextCtrl* name2_edit;
};
//==========================================================================
// list pair filter edit panel
class jfPairFilEditPanelList : public jfPairFilEditPanelCore {
  public:
    // constructor
    jfPairFilEditPanelList(wxWindow* parent,const jfPairFilterList* infil,wxWindowID id = wxID_ANY,const wxString& name = wxT("jfPairFilEditPanelList"));
    // virtual load and store stuff
    virtual bool LoadFilterData(const jfPairFilterCore* inval);
    virtual bool SaveFilterData(jfPairFilterCore* outval) const;
  protected:
    virtual bool CheckNames() const;
    // gui elements
    wxStaticText* name1_label;
    wxTextCtrl* name1_edit;
    wxStaticText* namelist_label;
    wxTextCtrl* namelist_edit;
};
//==========================================================================
// multiple pair filter edit panel
class jfPairFilEditPanelMulti : public jfPairFilEditPanelCore {
  public:
    // constructor
    jfPairFilEditPanelMulti(wxWindow* parent,const jfPairFilterMultiple* infil,wxWindowID id = wxID_ANY,const wxString& name = wxT("jfPairFilEditPanelMulti"));
    // virtual load and store stuff
    virtual bool LoadFilterData(const jfPairFilterCore* inval);
    virtual bool SaveFilterData(jfPairFilterCore* outval) const;
  protected:
    virtual bool CheckNames() const;
    // gui elements
    wxStaticText* name_label;
    wxTextCtrl* names_edit;
};
//***************************************************************************
