/******************************************************************************
Name    :   fim1_filedit.cpp
Author  :   John Q Metro
Purpose :   Editors for fim filters : thumbs rating
Created :   July 5, 2012
Updated :   November 10, 2015
******************************************************************************/
#ifndef FIM1_FILEDIT_H_INCLUDED
  #include "fim1_filedit.h"
#endif // FIM1_FILEDIT_H_INCLUDED
//---------------------------------------------------------------------------
/*****************************************************************************/
jfFIMThumbsPanel::jfFIMThumbsPanel(wxWindow* parent, wxWindowID id, jfFimThumbsFilter* inobj,
    const wxString& name):wxPanel(parent,id, wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // we start creating objects
  minl = new wxStaticText(this,-1,wxT("min"));
  maxl = new wxStaticText(this,-1,wxT("max"));
  minbox = new wxSpinCtrl(this,-1);
  minbox->SetRange(-999999,999999);
  maxbox = new wxSpinCtrl(this,-1);
  maxbox->SetRange(-999999999,999999);
  // sizer creation
  msizer = new wxFlexGridSizer(2,2,1,1);
  // adding items to the sizer
  msizer->Add(minl,1,wxALIGN_CENTRE);
  msizer->Add(maxl,1,wxALIGN_CENTRE);
  msizer->Add(minbox,1,wxALIGN_CENTRE|wxEXPAND);
  msizer->Add(maxbox,1,wxALIGN_CENTRE|wxEXPAND);
  // setting the data
  if (inobj!=NULL) SetFromObj(inobj);
  else {
    minbox->SetValue(-99999);
    maxbox->SetValue(99999);
  }
  // finishing sizer stuff
  SetSizer(msizer);
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// get
jfFimThumbsFilter* jfFIMThumbsPanel::GetNewFilter() const {
  // constants
  const wxString emsgtitle = wxT("Problem with thumbs count");
  const wxString emsgmsg1 = wxT("The maximum count is smaller than the minimum.");
  const wxString emsgmsg2 = wxT("Invalid numbers");
  // variables
  jfFimThumbsFilter* result;
  int qmin, qmax;
  wxMessageDialog* message;
  bool lres;
  long lval;
  wxString emsgmsg;
  // test
  qmin = minbox->GetValue();
  qmax = maxbox->GetValue();
  if (qmax>=qmin) {
    // the result
    result = new jfFimThumbsFilter(qmin,qmax);
    return result;
  }
  else {
    emsgmsg = (lres)?(emsgmsg1):(emsgmsg2);
    message = new wxMessageDialog(const_cast<jfFIMThumbsPanel*>(this),emsgmsg,emsgtitle,wxOK);
    message->ShowModal();
    message->Destroy();
    return NULL;
  }
}
//---------------------------------------------------------------------
bool jfFIMThumbsPanel::Check() const {
  // variables
  size_t qmin, qmax;
  bool lres;
  long lval;
  // test
  qmin = minbox->GetValue();
  qmax = maxbox->GetValue();
  return (qmax>=qmin);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// set
bool jfFIMThumbsPanel::SetFromObj(const jfFimThumbsFilter* inval) {
  // the cases where things are wrong
  if (inval==NULL) return false;
  if (!(inval->IsValid())) return false;
  // setting things
  minbox->SetValue(inval->GetMin());
  maxbox->SetValue(inval->GetMax());
  return true;
}
//===========================================================================
jfFimThumbsFilterEditor::jfFimThumbsFilterEditor(wxWindow* parent, const jfFilterMap*
      infmap, const jfBaseFilter* infilt, wxWindowID id, const wxString&
      name):jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // most things are handled by the parent constructor
  // we create the insert.. the *actual* expression editor
  insert_panel = new jfFIMThumbsPanel(this,-1,NULL);
  if (filt_pointer!=NULL) {
    insert_panel->SetFromObj(dynamic_cast<const jfFimThumbsFilter*>(filt_pointer));
  }
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfFimThumbsFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  const wxString fname = wxT("jfFimThumbsFilterEditor::LoadFilter");
  assert(infilter!=NULL);
  // loading the data
  filt_pointer = infilter;
  insert_panel->SetFromObj(dynamic_cast<const jfFimThumbsFilter*>(filt_pointer));
  NameLoad();
}
//------------------------------------------------------------------------------
jfBaseFilter* jfFimThumbsFilterEditor::GetFilter() {
  jfFimThumbsFilter* toutput;
  // getting the output
  toutput = insert_panel->GetNewFilter();
  namedesc_edit->ChangeObj(toutput);
  // done
  return toutput;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checks if the contents are ok, returns false if they are not
bool jfFimThumbsFilterEditor::GeneralCheck() const {
  return true;
}
//===============================================================================
// the default constructor
jfFimGenreFilterEditor::jfFimGenreFilterEditor(wxWindow* parent, const jfFilterMap* infmap,
    const jfFIMGenreFilter* infilt, wxWindowID id, const wxString& name):
    jfTagFilterEditor(parent,infmap,infilt,id,name ) {
  if (infilt==NULL) LoadBlank();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// extra methods
//---------------------------
jfTagFilterCore* jfFimGenreFilterEditor::GetTagFilter() {
  jfFIMGenreFilter* result;
  result = new jfFIMGenreFilter();
  result->SetToEmpty();
  return result;
}
//---------------------------
jfTagFilterCore* jfFimGenreFilterEditor::MakeBlankTagFilter() {
  jfFIMGenreFilter* result;
  bool atest;
  result = new jfFIMGenreFilter();
  atest = result->SetTags(insert_panel->GetList());
  assert(atest);
  return result;
}
//===============================================================================
// the default constructor
jfFimCharacterFilterEditor::jfFimCharacterFilterEditor(wxWindow* parent, const jfFilterMap* infmap,
    const jfFIMCharacterFilter* infilt, wxWindowID id, const wxString& name):
    jfTagFilterEditor(parent,infmap,infilt,id,name ) {
  if (infilt==NULL) LoadBlank();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// extra methods
//---------------------------
jfTagFilterCore* jfFimCharacterFilterEditor::GetTagFilter() {
  jfFIMCharacterFilter* result;
  result = new jfFIMCharacterFilter();
  result->SetToEmpty();
  return result;
}
//---------------------------
jfTagFilterCore* jfFimCharacterFilterEditor::MakeBlankTagFilter() {
  jfFIMCharacterFilter* result;
  bool atest;
  result = new jfFIMCharacterFilter();
  atest = result->SetTags(insert_panel->GetList());
  assert(atest);
  return result;
}
//===============================================================================================================

// constructor
jfFIM_RatingEditPanel::jfFIM_RatingEditPanel(wxWindow* parent, wxWindowID id, bool horizontal,
          const jfFimRatingFilter* invalue):wxPanel(parent,id, wxDefaultPosition, wxDefaultSize,wxTAB_TRAVERSAL,wxT("FIM_RatingEditPanel")) {
  // creating gui elemnts
  ech = new wxCheckBox(this,-1,wxT("Everyone"));
  tch = new wxCheckBox(this,-1,wxT("Teen"));
  mch = new wxCheckBox(this,-1,wxT("Mature "));
  // border styles
  const int lohs1 = wxTOP|wxBOTTOM|wxRIGHT;
  const int lohs2 = wxLEFT|wxRIGHT|wxBOTTOM;
  int thirdborder;
  // sizer creation
  ishoriz = horizontal;
  if (ishoriz) {
    msizer = new wxBoxSizer(wxHORIZONTAL);
    thirdborder = lohs1;
  }
  else {
    msizer = new wxBoxSizer(wxVERTICAL);
    thirdborder = lohs2;
  }
  // adding items in the sizer
  msizer->Add(ech,0,wxALL,3);
  msizer->Add(tch,0,thirdborder,3);
  msizer->Add(mch,0,thirdborder,3);
  // done
  SetSizer(msizer);
  LoadValue(invalue);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting and getting values
//--------------------------------------------------------------
bool jfFIM_RatingEditPanel::StoreValue(jfFimRatingFilter* saveto) const {
  if (saveto==NULL) return false;
  wxString inval;
  if (ech->IsChecked()) inval +=wxT("E");
  else if (tch->IsChecked()) inval +=wxT("T");
  else if (mch->IsChecked()) inval +=wxT("M");
  saveto->FromString(inval);
  return true;
}
//--------------------------------------------------------------
bool jfFIM_RatingEditPanel::LoadValue(const jfFimRatingFilter* getfrom) {
  if (getfrom==NULL) return false;
  ech->SetValue(getfrom->Includes(wxT('E')));
  tch->SetValue(getfrom->Includes(wxT('T')));
  mch->SetValue(getfrom->Includes(wxT('M')));
  return true;
}
//--------------------------------------------------------------
jfFimRatingFilter* jfFIM_RatingEditPanel::GetValue() const {
  jfFimRatingFilter* resval = new jfFimRatingFilter();
  StoreValue(resval);
  return resval;
}
/*****************************************************************************/
jfFIM_RatingFilterEditor::jfFIM_RatingFilterEditor(wxWindow* parent,const jfBaseFilter* infilt,
          const jfFilterMap* infmap, wxWindowID id, const wxString& name):
          jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // we start...
  // we create the insert.. the *actual*  editor
  insert_panel = new jfFIM_RatingEditPanel(this,-1,false,dynamic_cast<const jfFimRatingFilter*>(infilt));
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  // calling the sizers
  ArrangeSizers();
  if (infilt!=NULL) NameLoad();
  if (infilt!=NULL) LoadFilter(infilt);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methods
//-------------------------------------
void jfFIM_RatingFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  assert(infilter!=NULL);
  filt_pointer = infilter;
  insert_panel->LoadValue(dynamic_cast<const jfFimRatingFilter*>(filt_pointer));
  NameLoad();
}
//-------------------------------------
jfBaseFilter* jfFIM_RatingFilterEditor::GetFilter() {
  jfFimRatingFilter* result = insert_panel->GetValue();
  namedesc_edit->ChangeObj(result);
  return result;
}
//--------------------------------------------------------------
bool jfFIM_RatingFilterEditor::GeneralCheck() const {
  return true;
}
/*****************************************************************************/
jfFimThumbPercentFilterEditor::jfFimThumbPercentFilterEditor(wxWindow* parent,const jfBaseFilter* infilt,
          const jfFilterMap* infmap, wxWindowID id, const wxString& name):
          jfBaseFilterEditor(parent,infmap,infilt,id,name) {
  // we start...
  // we create the insert.. the *actual*  editor
  insert_panel = new jfLabeledUIntEdit(this,wxT("Percentage of Thumbs up"),100,85,3);
  // finalizing things
  insert = new wxBoxSizer(wxVERTICAL);
  insert->Add(insert_panel,1,wxALL|wxEXPAND,2);
  insert->AddStretchSpacer(1);
  // calling the sizers
  ArrangeSizers();
  if (infilt!=NULL) NameLoad();
  if (infilt!=NULL) LoadFilter(infilt);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// implemented virtual methods
//-------------------------------------
void jfFimThumbPercentFilterEditor::LoadFilter(const jfBaseFilter* infilter) {
  assert(infilter!=NULL);
  const jfFimThumbPercentFilter* xvalue;
  filt_pointer = infilter;
  xvalue = dynamic_cast<const jfFimThumbPercentFilter*>(filt_pointer);
  insert_panel->SetValue(xvalue->GetPercent());
  NameLoad();
}
//-------------------------------------
jfBaseFilter* jfFimThumbPercentFilterEditor::GetFilter() {
  // the values
  const wxString tvalalert= wxT("The value is not a valid percentage");
  bool tval;
  long oval;
  jfFimThumbPercentFilter* outfilt;
  wxMessageDialog* message;
  wxString msg1;
  // creating
  outfilt = new jfFimThumbPercentFilter();
  outfilt->SetPercent(oval);
  namedesc_edit->ChangeObj(outfilt);
  return outfilt;
}
//--------------------------------------------------------------
bool jfFimThumbPercentFilterEditor::GeneralCheck() const {
  return true;
}
/*****************************************************************************/
