/******************************************************************************
Name    :   charo_editor.cpp
Author  :   John Q Metro
Purpose :   Editor with a collection of checkboxes that have single-char i/o
Created :   September 8, 2012
Updated :   September 11, 2012
******************************************************************************/
#ifndef CHARO_EDITOR_H_INCLUDED
  #include "charo_editor.h"
#endif // CHARO_EDITOR_H_INCLUDED
//---------------------------------------------
#include <vector>
#include <wx/checkbox.h>
#include <math.h>
/*****************************************************************************/
jfCharCheckBoxGroup::jfCharCheckBoxGroup(wxWindow* parent, wxWindowID id, wxString inname,
      size_t icount, const wxChar* inlabels[], const wxString inchars, size_t incols,
      const wxString& name):wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,
      wxTAB_TRAVERSAL, name) {
  // after the whopping paramater list...
  // we check...
  assert(icount>0);
  assert(incols<=icount);
  assert(inchars.length()==icount);
  // we don't check this
  cbcount = icount;
  mainlabel = inname;
  charvals = inchars;
  // starting the setup...
  SetupSizers(icount,incols);
  // the checkbox making loop
  wxCheckBox* temp;
  for (size_t cloop=0;cloop<icount;cloop++) {
    temp = new wxCheckBox(this,-1,inlabels[cloop]);
    cboxes.push_back(temp);
    qsizer->Add(temp,0,wxALL,1);
  }
  // done
  SetSizer(msizer);
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfCharCheckBoxGroup::SetFromString(const wxString& inval) {
  // variables
  size_t rescount = 0, loopval;
  wxChar tcahr;
  for (loopval=0;loopval<cbcount;loopval++) {
    tcahr = charvals[loopval];
    if (inval.Find(tcahr)!=wxNOT_FOUND) {
      cboxes[loopval]->SetValue(true);
      rescount++;
    }
    else cboxes[loopval]->SetValue(false);
  }
  return (rescount==inval.length());
}
//----------------------------------------------------
wxString jfCharCheckBoxGroup::GetStringValue() const {
  wxString result;
  for (size_t cloop=0;cloop<cbcount;cloop++) {
    if (cboxes[cloop]->IsChecked()) result += charvals[cloop];
  }
  return result;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special
void jfCharCheckBoxGroup::CheckAll(bool doit) {
  for (size_t cloop=0;cloop<cbcount;cloop++) {
    cboxes[cloop]->SetValue(doit);
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfCharCheckBoxGroup::SetupSizers(size_t tcount, size_t colcount) {
  // rowcalc
  int rowin; float rowest;
  // one column
  if (colcount==1) {
    mode = 1;
    msizer = new wxStaticBoxSizer(wxVERTICAL,this,mainlabel);
    qsizer = msizer;
    fsizer = NULL;
  }
  // one row
  else if ((colcount==0) || (colcount==tcount)) {
    mode = 2;
    msizer = new wxStaticBoxSizer(wxHORIZONTAL,this,mainlabel);
    qsizer = msizer;
    fsizer = NULL;
  }
  // grid
  else {
    mode = 3;
    msizer = new wxStaticBoxSizer(wxVERTICAL,this,mainlabel);
    rowest = (float)(tcount) / (float)(colcount);
    rowin = ceil(rowest);
    fsizer = new wxFlexGridSizer(rowin,colcount,3,3);
    msizer->Add(fsizer,1,wxEXPAND|wxALL,1);
    qsizer = fsizer;
  }
}
/*****************************************************************************/
