/***************************************************************
 * Name:      defaultdir_dilaog.cpp
 * Purpose:   Declares dialog for default directory editing
 * Author:    John Q Metro (johnqmetro@hotmail.com)
 * Created:   December 18, 2011
 * Updated:   March 3, 2012
 *
**************************************************************/
#ifndef DEFAULTDIR_DIALOG_H_INCLUDED
  #include "defaultdir_dialog.h"
#endif // DEFAULTDIR_DIALOG_H_INCLUDED
#include <wx/dir.h>
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//****************************************************************************
// constructor
jfDefaultDirDisplay::jfDefaultDirDisplay(wxWindow* parent, const wxString& name):
      wxPanel(parent,-1,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // starting...
  MakeLabels();
  MakeRest();
  // creating the sizer
  main_sizer = new wxFlexGridSizer(globalsettings::GV_NUMVALS,4,3,3);
  main_sizer->SetFlexibleDirection(wxHORIZONTAL);
  main_sizer->AddGrowableCol(2,1);
  for(size_t loopv = 0; loopv<globalsettings::GV_NUMVALS;loopv++) {
    InsertRow(loopv);
  }
  // finishing
  SetSizer(main_sizer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// load and save
//---------------------------------------------
void jfDefaultDirDisplay::LoadValues() {
  size_t lloop;
  for (lloop=0;lloop<globalsettings::GV_NUMVALS;lloop++) {
    displays[lloop]->SetValue((*globalsettings::globaldirs)[lloop]);
  }
}
//---------------------------------------------
void jfDefaultDirDisplay::SaveValues() const {
  size_t lloop;
  for (lloop=0;lloop<globalsettings::GV_NUMVALS;lloop++) {
    (*globalsettings::globaldirs)[lloop] = displays[lloop]->GetValue();
  }
}
//---------------------------------------------
wxString jfDefaultDirDisplay::GetValue(size_t tindex) const {
  if (tindex>=globalsettings::GV_NUMVALS) return wxEmptyString;
  else return displays[tindex]->GetValue();
}
//---------------------------------------------
bool jfDefaultDirDisplay::SetValue(size_t tindex, wxString inval) {
  if (tindex>=globalsettings::GV_NUMVALS) return false;
  displays[tindex]->SetValue(inval);
  return true;
}
//---------------------------------------------
int jfDefaultDirDisplay::TestDirs() const {
  // variables
  size_t lloop;
  wxString tval;
  // the test loop
  for (lloop=0;lloop<globalsettings::GV_NUMVALS;lloop++) {
    tval = displays[lloop]->GetValue();
    if (!wxDir::Exists(tval)) break;
  }
  if (lloop==globalsettings::GV_NUMVALS) return -1;
  else return lloop;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// events
void jfDefaultDirDisplay::DisplayDirpicker(wxCommandEvent& mevent) {
  // variables
  const wxString fname = wxT("jfDefaultDirDisplay::DisplayDirpicker");
  int event_zindex;
  wxDirDialog* showdialog;
  // getting the specified control
  /**/jfXLogMessageI(fname,0,dynamic_cast<wxButton*>(mevent.GetEventObject())->GetId());
  /**/jfXLogMessageI(fname,1,jfID_DEFDIRPICK_BTNBASE);
  event_zindex = (int)(dynamic_cast<wxButton*>(mevent.GetEventObject())->GetId()) - (int)(jfID_DEFDIRPICK_BTNBASE);
  /**/jfXLogMessageI(fname,2,event_zindex);
  assert(event_zindex>=0);
  assert(event_zindex<globalsettings::GV_NUMVALS);
  // building the dialog to show
  showdialog = new wxDirDialog(this,wxT("Choose a directory"),displays[event_zindex]->GetValue());
  // showing
  if (wxID_OK==showdialog->ShowModal()) {
    displays[event_zindex]->SetValue(showdialog->GetPath());
  }
  // done
  showdialog->Destroy();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// gui builder methods
//---------------------------------------------
void jfDefaultDirDisplay::MakeLabels() {
  // variables
  size_t lloop;
  // the label making loop
  for (lloop = 0; lloop<globalsettings::GV_NUMVALS; lloop++) {
    labels[lloop] = new wxStaticText(this,-1,globalsettings::gv_dirlabels[lloop]);
    colons[lloop] = new wxStaticText(this,-1,wxT(" : "));
  }
}
//---------------------------------------------
void jfDefaultDirDisplay::MakeRest() {
  // variables
  size_t lloop;
  // the label making loop
  for (lloop = 0; lloop<globalsettings::GV_NUMVALS; lloop++) {
    displays[lloop]   = new wxTextCtrl(this,-1,(*globalsettings::globaldirs)[lloop]);
    dirpickers[lloop] = new wxButton(this,jfID_DEFDIRPICK_BTNBASE+lloop,wxT("Pick..."));
  }
}
//---------------------------------------------
bool jfDefaultDirDisplay::InsertRow(size_t rowval) {
  // the usual asserts
  assert(rowval<globalsettings::GV_NUMVALS);
  assert(main_sizer!=NULL);
  // after that, we assume things are okay
  main_sizer->Add(labels[rowval],0,wxALIGN_CENTER_VERTICAL);
  main_sizer->Add(colons[rowval],0,wxALIGN_CENTER_VERTICAL);
  main_sizer->Add(displays[rowval],1,wxALIGN_CENTER_VERTICAL|wxEXPAND);
  main_sizer->Add(dirpickers[rowval],0,wxALIGN_CENTER_VERTICAL|wxLEFT,3);
  // done
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfDefaultDirDisplay,wxPanel)
  EVT_BUTTON  (-1, jfDefaultDirDisplay::DisplayDirpicker)
END_EVENT_TABLE()
//============================================================================
// constructor
jfDefaultDirsDialog::jfDefaultDirsDialog(wxWindow* parent, wxWindowID id, const wxString& name):
        wxDialog(parent,id,wxT(""),wxDefaultPosition,wxDefaultSize,wxDEFAULT_DIALOG_STYLE,name) {
  // inititalize sizer
  main_sizer = new wxBoxSizer(wxVERTICAL);
  // the topbar values
  loggingon_cb = new wxCheckBox(this,-1,wxT("Do Logging (changes take effect after restart)"));
  loggingon_cb->SetValue(globalsettings::dologging);
  shint = new jfFilesizeHintWidget(this);
  // adding to the topbar
  topbar = new wxBoxSizer(wxHORIZONTAL);
  topbar->Add(shint,0,wxALL|wxEXPAND,3);
  topbar->Add(loggingon_cb,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
  // visual elements
  contents = new jfDefaultDirDisplay(this);
  saveclose_btn = new wxButton(this,jfID_DEFDIRPICK_SAVBTN,wxT("Save and Close"));
  // arranging
  main_sizer->Add(topbar,0,wxALL|wxEXPAND,1);
  main_sizer->Add(contents,0,wxALL|wxEXPAND,3);
  main_sizer->Add(saveclose_btn,0,wxALIGN_RIGHT|wxALL,3);
  // finsihing
  SetSizer(main_sizer);
  SetInitialSize();
  wxSize qsize = GetSize();
  qsize.SetWidth(1.3*qsize.GetWidth());
  SetSize(qsize);
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// events
void jfDefaultDirsDialog::SaveButton(wxCommandEvent& mevent) {
  // variables
  int indexval = contents->TestDirs();
  bool xval;
  size_t xres;
  // starting
  if (indexval<0) {
    wxBeginBusyCursor();
    globalsettings::logchange = loggingon_cb->GetValue();
    xres = shint->GetValue();
    globalsettings::ficsize_hint = xres;
    contents->SaveValues();
    wxEndBusyCursor();
    Close();
  }
  else ShowErrorMessage(indexval);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfDefaultDirsDialog::ShowErrorMessage(int dindex) {
  // asserts
  assert(dindex>=0);
  assert(dindex<globalsettings::GV_NUMVALS);
  // constants
  const wxString main_title = wxT("Save Aborted");
  const wxString errpart1 = wxT("Error: The directory '");
  const wxString errpart2 = wxT("' does not exist. Please correct the Error.");
  // variables
  wxMessageDialog* msgbox;
  wxString thedir, themsg;
  // we start
  thedir = contents->GetValue(dindex);
  themsg = errpart1 + thedir + errpart2;
  msgbox = new wxMessageDialog(this, themsg, main_title,wxICON_ERROR|wxOK);
  // showing
  msgbox->ShowModal();
  msgbox->Destroy();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfDefaultDirsDialog,wxDialog)
  EVT_BUTTON  (jfID_DEFDIRPICK_SAVBTN, jfDefaultDirsDialog::SaveButton)
END_EVENT_TABLE()
//****************************************************************************
