/***************************************************************
 * Name:      dirupdate_dialog.h
 * Purpose:   Declares dialog for updating directories
 * Author:    John Q Metro (johnqmetro@hotmail.com)
 * Created:   June 30, 2015
 * Updated:   July 3, 2015
 *
**************************************************************/
#ifndef DIRUPDATE_DIALOG_H_INCLUDED
  #include "dirupdate_dialog.h"
#endif // DIRUPDATE_DIALOG_H_INCLUDED
#include <wx/dir.h>
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef INITEND_H_INCLUDED
  #include "../../initend.h"
#endif // INITEND_H_INCLUDED
//****************************************************************************
// constructor
jfUpdateDirectoryPickPanel::jfUpdateDirectoryPickPanel(wxWindow* parent, const wxString& name):
      wxPanel(parent,-1,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // starting...
  MakeWidgets();
  // creating the sizer
  main_sizer = new wxFlexGridSizer(2,3,3,3);
  main_sizer->SetFlexibleDirection(wxHORIZONTAL);
  main_sizer->AddGrowableCol(1,1);
  // adding the widgets to the sizer
  main_sizer->Add(labels[0],0,wxALIGN_CENTER_VERTICAL);
  main_sizer->Add(displays[0],1,wxALIGN_CENTER_VERTICAL|wxEXPAND);
  main_sizer->Add(dirpickers[0],0,wxALIGN_CENTER_VERTICAL|wxLEFT,3);
  main_sizer->Add(labels[1],0,wxALIGN_CENTER_VERTICAL);
  main_sizer->Add(displays[1],1,wxALIGN_CENTER_VERTICAL|wxEXPAND);
  main_sizer->Add(dirpickers[1],0,wxALIGN_CENTER_VERTICAL|wxLEFT,3);
  // finishing
  SetSizer(main_sizer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// load and save
//---------------------------------------------
wxString jfUpdateDirectoryPickPanel::GetValue(bool source) const {
  if (source) return displays[0]->GetValue();
  else return displays[1]->GetValue();
}
//---------------------------------------------
bool jfUpdateDirectoryPickPanel::SetValue(bool source, wxString inval) {
  if (source) displays[0]->SetValue(inval);
  else displays[1]->SetValue(inval);
  return true;
}
//---------------------------------------------
int jfUpdateDirectoryPickPanel::TestDirs() const {
  // variables
  size_t lloop;
  wxString tval;
  // the test loop
  for (lloop=0;lloop<2;lloop++) {
    tval = displays[lloop]->GetValue();
    if (!wxDir::Exists(tval)) break;
  }
  if (lloop==2) return -1;
  else return lloop;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// events
void jfUpdateDirectoryPickPanel::DisplayDirpicker(wxCommandEvent& mevent) {
  // variables
  const wxString fname = wxT("jfUpdateDirectoryPickPanel::DisplayDirpicker");
  int event_zindex;
  wxDirDialog* showdialog;
  // getting the specified control
  event_zindex = (int)(dynamic_cast<wxButton*>(mevent.GetEventObject())->GetId()) - (int)(jfID_DIRUPDATE_BTNBASE);
  assert(event_zindex>=0);
  assert(event_zindex<2);
  // building the dialog to show
  showdialog = new wxDirDialog(this,wxT("Choose a directory"),displays[event_zindex]->GetValue());
  // showing
  if (wxID_OK==showdialog->ShowModal()) {
    displays[event_zindex]->SetValue(showdialog->GetPath());
  }
  // done
  showdialog->Destroy();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// gui builder methods
//---------------------------------------------
void jfUpdateDirectoryPickPanel::MakeWidgets() {
  // label making
  labels[0] = new wxStaticText(this,-1,wxT("Source Directory : "));
  labels[1] = new wxStaticText(this,-1,wxT("Target Directory : "));
  // the label making loop
  displays[0]   = new wxTextCtrl(this,-1,(*globalsettings::globaldirs)[6]);
  dirpickers[0] = new wxButton(this,jfID_DIRUPDATE_BTNBASE,wxT("Pick..."));
  displays[1]   = new wxTextCtrl(this,-1,(*globalsettings::globaldirs)[7]);
  dirpickers[1] = new wxButton(this,jfID_DIRUPDATE_BTNBASE+1,wxT("Pick..."));
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfUpdateDirectoryPickPanel,wxPanel)
  EVT_BUTTON  (-1, jfUpdateDirectoryPickPanel::DisplayDirpicker)
END_EVENT_TABLE()
//=====================================================================================
jfUpdateDirectory_Dialog::jfUpdateDirectory_Dialog(wxWindow* parent, wxWindowID id,
          const wxString& title, wxString inname):wxDialog(parent,id,title,
          wxDefaultPosition,wxDefaultSize,wxDEFAULT_DIALOG_STYLE,inname) {
  // doing stuff
  dpanel = new jfUpdateDirectoryPickPanel(this);
  mpanel = new jfProgressPanelSingle(this,wxT("UpdateDirectoryProgressPanel"));

  FinishSizers();
  SetSize(705,180);

}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handlers from the panel
void jfUpdateDirectory_Dialog::EvEndSignal(wxCommandEvent& mevent) {

}
//---------------------------------------------------------------
void jfUpdateDirectory_Dialog::EvFailSignal(wxCommandEvent& mevent) {

}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// button handler events
void jfUpdateDirectory_Dialog::EvStartButton(wxCommandEvent& mevent) {
  wxString xerr;

  if (GetDirectories(xerr)) {
    ud_thread = new jfDirUpdate_Thread();
    ud_thread->SetDirectories(sdir,tdir,mpanel);
    ud_thread->Create();
    ud_thread->Run();
  }
  else {
    wxMessageBox(xerr, wxT("Directory Problem"), wxCANCEL | wxICON_ERROR, this);

  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//private helpher methods
void jfUpdateDirectory_Dialog::FinishSizers() {
  mainsizer = new wxBoxSizer(wxVERTICAL);
  mainsizer->Add(dpanel,0,wxALIGN_CENTER|wxALL|wxEXPAND,3);
  mainsizer->Add(mpanel,0,wxALIGN_CENTER|wxLEFT|wxRIGHT|wxEXPAND,3);
  SetSizer(mainsizer);
}
//------------------------------------------------------------------------
bool jfUpdateDirectory_Dialog::GetDirectories(wxString& out_err) {
  sdir = dpanel->GetValue(true);
  tdir = dpanel->GetValue(false);
  if (!wxDir::Exists(sdir)) {
    out_err = wxT("Source Directory does not exist!");
    return false;
  }
  if (!wxDir::Exists(tdir)) {
    out_err = wxT("Target Directory does not exist!");
    return false;
  }
  if (sdir==tdir) {
    out_err = wxT("The directories must be different!");
    return false;
  }
  if (!wxFileName::IsDirReadable(sdir)) {
    out_err = wxT("The Source Directory cannot be read!");
    return false;
  }
  if (!wxFileName::IsDirWritable(tdir)) {
    out_err = wxT("The Target Directory cannot be written to!");
    return false;
  }
  out_err = wxT("No Problem");
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfUpdateDirectory_Dialog,wxDialog)
  EVT_COMMAND (-1, jf_PP_SRCHDONE_EVENT , jfUpdateDirectory_Dialog::EvEndSignal )
  EVT_COMMAND (-1, jf_PP_SRCHFAIL_EVENT , jfUpdateDirectory_Dialog::EvFailSignal)
  EVT_BUTTON  (jfID_STARTSTOPBTN  , jfUpdateDirectory_Dialog::EvStartButton   )
END_EVENT_TABLE()
//*********************************************************************************************
