/******************************************************************************
Name    :   global_skel.cpp
Author  :   John Q Metro
Purpose :   A global skeleton editor
Created :   December 2, 2009
Updated :   September 5, 2011
******************************************************************************/
#ifndef GLOBAL_SKEL_H_INCLUDED
  #include "global_skel.h"
#endif // GLOBAL_SKEL_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//------------------------------------------------------------------
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif
/*****************************************************************************/
// constructor
jfGlobalSkeletonEdit::jfGlobalSkeletonEdit(wxWindow* parent, wxWindowID id,
        jfSkelCollection* icoll_link):wxPanel(parent,id,wxDefaultPosition,
        wxDefaultSize,wxTAB_TRAVERSAL,wxT("jfGlobalSkeletonEdit")) {
  // initializing things
  assert(icoll_link!=NULL);
  coll_link = icoll_link;
  single = NULL;
  multiple = NULL;
  ficpart = NULL;
  stype = jfs_SINGLE;
  i_index = 0;
  // we create and initialize gui elements
  // the list box
  selector = new wxListBox(this,jfID_SKELE_GLLBSEL);
  wxArrayString* items = coll_link->GetNameList();
  selector->Set((*items));
  delete items;
  // we start with skeleton 1, which is single
  single = new jfSkelEditSingle(this,-1,coll_link->AtIndexS(0));
  // creating the sizer and arranging
  topsizer = new wxBoxSizer(wxHORIZONTAL);
  topsizer->Add(selector,1,wxALL|wxEXPAND,3);
  topsizer->Add(single,4,wxALL|wxEXPAND,3);
  // finishing off
  selector->SetSelection(0);
  SetSizer(topsizer);
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// evaluation
void jfGlobalSkeletonEdit::ListSelect(wxCommandEvent& event) {
  size_t selidx = selector->GetSelection();
  if (selidx!=i_index) {
    if ((coll_link->GetSkelType(selidx))==jfs_SINGLE) {
      ToSingle(coll_link->AtIndexS(selidx));
    }
    else if ((coll_link->GetSkelType(selidx))==jfs_MULTIPLE) {
      ToMultiple(coll_link->AtIndexM(selidx));
    }
    else if ((coll_link->GetSkelType(selidx))==jfs_FICPART) {
      ToFicPart(coll_link->AtIndexF(selidx));
    }
    else assert(false);
    i_index = selidx;
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfGlobalSkeletonEdit::ToSingle(jfSkeletonSingle* inval) {
  // constants and variables
  const wxString fname = wxT("jfGlobalSkeletonEdit::ToSingle");
  bool atest;
  // starting
  /**/jfXLogMessage(fname,1);
  assert(inval!=NULL);
  /**/jfXLogMessage(fname,2);
  // single to single
  if (stype==jfs_SINGLE) {
    /**/jfXLogMessage(fname,3);
    if (single==NULL) single = new jfSkelEditSingle(this,-1,inval);
    else if ((single->GetSkeleton())==inval) return false;
    else single->ChangeSkeleton(inval);
    /**/jfXLogMessage(fname,4);
  }
  // multiple to single
  else {
    /**/jfXLogMessage(fname,5);
    if (single==NULL) single = new jfSkelEditSingle(this,-1,inval);
    else single->ChangeSkeleton(inval);
    /**/jfXLogMessage(fname,6);
    atest = ChangePanel(single);
    assert(atest);
  }
  stype = jfs_SINGLE;
  /**/jfXLogMessage(fname,7);
  // done
  return true;
}
//-------------------------------------------------------------------
bool jfGlobalSkeletonEdit::ToMultiple(jfSkeletonMultiple* inval) {
  // constants and variables
  const wxString fname = wxT("jfGlobalSkeletonEdit::ToMultiple");
  bool atest;
  // starting
  /**/jfXLogMessage(fname,1);
  assert(inval!=NULL);
  // multiple to multiple
  if (stype==jfs_MULTIPLE) {
    /**/jfXLogMessage(fname,2);
    if (multiple==NULL) multiple = new jfSkelEditMultiple(this,-1,inval);
    else if ((multiple->GetSkeleton())==inval) return false;
    else multiple->ChangeSkeleton(inval);
    /**/jfXLogMessage(fname,3);
  }
  // single to multiple
  else {
    /**/jfXLogMessage(fname,4);
    if (multiple==NULL) multiple = new jfSkelEditMultiple(this,-1,inval);
    else multiple->ChangeSkeleton(inval);
    /**/jfXLogMessage(fname,5);
    // we now change the panels
    atest = ChangePanel(multiple);
    assert(atest);
  }
  stype = jfs_MULTIPLE;
  /**/jfXLogMessage(fname,6);
  // done
  return true;
}
//------------------------------------------------------------------
bool jfGlobalSkeletonEdit::ToFicPart(jfSkeletonFicPart* inval) {
  // constants and variables
  const wxString fname = wxT("jfGlobalSkeletonEdit::ToFicPart");
  bool atest;
  // starting
  /**/jfXLogMessage(fname,1);
  assert(inval!=NULL);
  // ficpart to ficpart
  if (stype==jfs_FICPART) {
    /**/jfXLogMessage(fname,2);
    if (ficpart==NULL) ficpart = new jfSkelEditFicPart(this,-1,inval);
    else if ((ficpart->GetSkeleton())==inval) return false;
    else ficpart->ChangeSkeleton(inval);
    /**/jfXLogMessage(fname,3);
  }
  // single to multiple
  else {
    /**/jfXLogMessage(fname,4);
    if (ficpart==NULL) ficpart = new jfSkelEditFicPart(this,-1,inval);
    else ficpart->ChangeSkeleton(inval);
    /**/jfXLogMessage(fname,5);
    // we now change the panels
    atest = ChangePanel(ficpart);
    assert(atest);
  }
  stype = jfs_FICPART;
  /**/jfXLogMessage(fname,6);
  // done
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// more helper methods
bool jfGlobalSkeletonEdit::ChangePanel(jfSkelEditCore* inskel) {
  bool atest;
  // special cases
  if (inskel==NULL) return false;
  if (stype==(inskel->GetSkelType())) return false;
  // general
  inskel->Show();
  if (stype==jfs_SINGLE) {
    atest = topsizer->Replace(single,inskel);
    assert(atest);
    single->Hide();
  }
  else if (stype==jfs_MULTIPLE) {
    atest = topsizer->Replace(multiple,inskel);
    assert(atest);
    multiple->Hide();
  }
  else if (stype==jfs_FICPART) {
    atest = topsizer->Replace(ficpart,inskel);
    assert(atest);
    ficpart->Hide();
  }
  else return false;
  Layout();
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfGlobalSkeletonEdit,wxPanel)
  EVT_LISTBOX (jfID_SKELE_GLLBSEL, jfGlobalSkeletonEdit::ListSelect   )
END_EVENT_TABLE()
/*****************************************************************************/
