/*****************************************************************************
Name:        menu.cpp
Author :     John Q Metro
Purpose :    Defines custom popup menu classes
Created:     January 5, 2010
Updated:     July 1, 2015
******************************************************************************/
#ifndef MENU_H_INCLUDED
  #include "menu.h"
#endif // MENU_H_INCLUDED
#ifndef DISPLAY_BASE_H_INCLUDED
  #include "../display/display_base.h"
#endif // DISPLAY_BASE_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef HTML_DISPLAY_H_INCLUDED
  #include "../display/html_display.h"
#endif // HTML_DISPLAY_H_INCLUDED
//------------------
#include <wx/menu.h>
//---------------------
/*****************************************************************************/
jfPopupMenu::jfPopupMenu(wxWindow* destination):wxMenu() {
  dtarget = destination;
	miDisplayUrl = new wxMenuItem(this,jfID_RMENU_OPNURL,wxT("Open the url in a browser..."),wxT("Open the url in the default browser."),wxITEM_NORMAL);
	Append(miDisplayUrl);
	Connect(jfID_RMENU_OPNURL,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&jfPopupMenu::OnLoadURL);
	AppendSeparator();
  miAppendText = new wxMenuItem(this,jfID_RMENU_ADDTXT,wxT("Append Text to filter..."),wxT("Edit the text to add to the expression filter"),wxITEM_NORMAL);
  Append(miAppendText);
  Connect(jfID_RMENU_ADDTXT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&jfPopupMenu::OnAppendToExpressionFilter);
  rc_pointer = NULL;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 // setting data
//----------------------------------------------------
void jfPopupMenu::SetIndexes(wxString inurl, size_t in_itemidx, size_t in_collidx) {
  item_index = in_itemidx;
  coll_index = in_collidx;
	indexurl = inurl;
}
//----------------------------------------------------
void jfPopupMenu::SetMoreIndexes(jfHtmlResultCollection* inrc_pointer, size_t inres_ptr,
                                  size_t inres_index) {
  // doing ...
  assert(inrc_pointer!=NULL);
  rc_pointer = inrc_pointer;
  res_ptr = inres_ptr;
  res_index = inres_index;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loads the url...
void jfPopupMenu::OnLoadURL(wxCommandEvent& event) {
  /**/jfXLogMessage(wxT("jfPopupMenu::OnLoadURL"),1,indexurl);
  if (!indexurl.IsEmpty()) ::wxLaunchDefaultBrowser(indexurl);
}
//--------------------------------------------------------
// all this really does is launch the event
void jfPopupMenu::OnAppendToExpressionFilter(wxCommandEvent& event) {
  // local variables
  wxPoint* outpair;
  // creating the out-event
  wxCommandEvent outevent(jf_RMENU_APPTXTFIL_EVENT,-1);
  // setting up the index information
  outpair = new wxPoint(coll_index,item_index);
  outevent.SetClientData(outpair);
  // posting
  wxPostEvent(dtarget,outevent);
}
//===========================================================================
jfGooxPopupMenu::jfGooxPopupMenu(wxWindow* destination):jfPopupMenu(destination) {
  miExcludeUrl = new wxMenuItem(this,jfID_RMENU_EXLURL,wxT("Add the url to the exclude list..."),wxT("Edit the url to add to the url filter"),wxITEM_NORMAL);
  Append(miExcludeUrl);
  Connect(jfID_RMENU_EXLURL,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&jfGooxPopupMenu::OnExcludeUrl);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfGooxPopupMenu::OnExcludeUrl(wxCommandEvent& event) {
    // local variables
  wxPoint* outpair;
  // creating the out-event
  wxCommandEvent outevent(jf_RMENU_EXLUDEURL_EVENT,-1);
  // setting up the index information
  outpair = new wxPoint(coll_index,item_index);
  outevent.SetClientData(outpair);
  // posting
  wxPostEvent(dtarget,outevent);
}
//===================================================================
jfFFNPopupMenu::jfFFNPopupMenu(wxWindow* destination):jfPopupMenu(destination) {
  miDownloadFic = new wxMenuItem(this,jfID_RMENU_DOWNLFIC ,wxT("Download the fic..."),
                                 wxT("Shows a download dialog for the fanfic."),wxITEM_NORMAL);
  Append(miDownloadFic);
  miMarkAsRead= new wxMenuItem(this,jfID_RMENU_FMREAD ,wxT("Mark as Read"),
                        wxT("Marks the fic as being read, changing the background color"),wxITEM_NORMAL);
  Append(miMarkAsRead);
  miMarkInterest = new wxMenuItem(this,jfID_RMENU_FMINTEREST,wxT("Mark as Interesting"),
                        wxT("Marks the fic as being interesting, changing the background color"),wxITEM_NORMAL);
  Append(miMarkInterest);
  miMarkIgnore= new wxMenuItem(this,jfID_RMENU_FMIGNORE ,wxT("Mark as Ignore"),
                        wxT("Marks the fic as to be ignored, changing the background color"),wxITEM_NORMAL);
  Append(miMarkIgnore);
  miMarkAsDownloaded= new wxMenuItem(this,jfID_RMENU_FMDOWNLOADED ,wxT("Mark as Downloaded"),
                        wxT("Marks the fic as downloaded, changing the background color"),wxITEM_NORMAL);
  Append(miMarkAsDownloaded);
  miUnmark = new wxMenuItem(this,jfID_RMENU_FMUNMARK ,wxT("Remove any Fic Mark"),
                        wxT("Removes any fic mark"),wxITEM_NORMAL);
  Append(miUnmark);
  Connect(jfID_RMENU_DOWNLFIC ,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&jfFFNPopupMenu::OnDownloadFic);
  Connect(jfID_RMENU_FMREAD,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&jfFFNPopupMenu::onMarkAsRead);
  Connect(jfID_RMENU_FMINTEREST,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&jfFFNPopupMenu::onMarkInterest);
  Connect(jfID_RMENU_FMIGNORE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&jfFFNPopupMenu::onMarkIgnore);
  Connect(jfID_RMENU_FMDOWNLOADED,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&jfFFNPopupMenu::onMarkAsDownloaded);
  Connect(jfID_RMENU_FMUNMARK,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&jfFFNPopupMenu::onUnmark);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfFFNPopupMenu::OnDownloadFic(wxCommandEvent& event) {
    // local variables
  wxPoint* outpair;
  // creating the out-event
  wxCommandEvent outevent(jf_RMENU_DOWNLFIC_EVENT,-1);
  // setting up the index information
  outpair = new wxPoint(coll_index,item_index);
  outevent.SetClientData(outpair);
  // posting
  wxPostEvent(dtarget,outevent);
}
//-------------------------------------------------------------
void jfFFNPopupMenu::onMarkAsRead(wxCommandEvent& event) {
  CoreMark(jffm_READ);
}
//------------------------------------
void jfFFNPopupMenu::onMarkInterest(wxCommandEvent& event) {
  CoreMark(jffm_INTEREST);
}
//------------------------------------
void jfFFNPopupMenu::onMarkIgnore(wxCommandEvent& event) {
  CoreMark(jffm_IGNORE);
}
//------------------------------------
void jfFFNPopupMenu::onMarkAsDownloaded(wxCommandEvent& event) {
  CoreMark(jffm_DOWNLOADED);
}
//------------------------------------
void jfFFNPopupMenu::onUnmark(wxCommandEvent& event) {
  CoreMark(jffm_NONE);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// core mark handler
void jfFFNPopupMenu::CoreMark(jf_FicMark inmark) {
    // local variables
  jf_FicMark xqval = inmark;
  wxPoint* outpair;
  // creating the out-event
  wxCommandEvent outevent(jf_FICMARK_EVENT,-1);
  // changing the color
  rc_pointer->ChangeFicMark(res_ptr,res_index,xqval);
  // setting up the index information
  outpair = new wxPoint(coll_index,item_index);
  outevent.SetClientData(outpair);
  outevent.SetInt((int)xqval);
  // posting
  wxPostEvent(dtarget,outevent);
}
//===================================================================
jfFIMGPopupMenu::jfFIMGPopupMenu(wxWindow* destination):jfPopupMenu(destination) {
  miMarkIgnore= new wxMenuItem(this,jfID_RMENU_FMIGNORE ,wxT("Mark as Ignore"),
                        wxT("Marks the group as to be ignored, changing the background color"),wxITEM_NORMAL);
  Append(miMarkIgnore);
  miUnmark = new wxMenuItem(this,jfID_RMENU_FMUNMARK ,wxT("Remove Ignore"),
                        wxT("Removes ignore mark"),wxITEM_NORMAL);
  Append(miUnmark);

  Connect(jfID_RMENU_FMIGNORE,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&jfFIMGPopupMenu::onMarkAsDiscard);
  Connect(jfID_RMENU_FMUNMARK,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&jfFIMGPopupMenu::onUnmark);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfFIMGPopupMenu::onMarkAsDiscard(wxCommandEvent& event) {
  CoreMark(true);
}
//------------------------------------
void jfFIMGPopupMenu::onUnmark(wxCommandEvent& event) {
  CoreMark(false);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// core mark handler
void jfFIMGPopupMenu::CoreMark(bool mark_discard) {
    // local variables
  jf_FicMark xqval;
  wxPoint* outpair;
  // creating the out-event
  if (mark_discard) xqval = jffm_IGNORE;
  else xqval = jffm_NONE;

  wxCommandEvent outevent(jf_FICDISCARD_EVENT,-1);
  // changing the color
  rc_pointer->ChangeFicMark(res_ptr,res_index,xqval);
  // setting up the index information
  outpair = new wxPoint(coll_index,item_index);
  outevent.SetClientData(outpair);
  outevent.SetInt((int)mark_discard);
  // posting
  wxPostEvent(dtarget,outevent);
}

/*****************************************************************************/
