/******************************************************************************
Name    :   skeledit.cpp
Author  :   John Q Metro
Purpose :   Some more miscuser interface stuff
Created :   November 25, 2009
Updated :   September 7, 2011
******************************************************************************/
#ifndef SKELEDIT_H_INCLUDED
  #include "skeledit.h"
#endif
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED

//------------------------------------------------------------------

/*****************************************************************************/
// the constructor
jfSkelEditCore::jfSkelEditCore(wxWindow* parent, wxWindowID id, wxString inname):
    wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,inname) {
  // we start with the name editor
  ndesce = new jfNameDescEditor(this,-1,false,false);
  // buttons
  save_btn = new wxButton(ndesce,jfID_SKELE_SAVEBTN ,wxT("Save"));
  rest_btn = new wxButton(ndesce,jfID_SKELE_RESTBTN ,wxT("Restore"));
  ndesce->AddItem(save_btn);
  ndesce->AddItem(rest_btn);
  // the text control for note displaying
  help_desc = new wxTextCtrl(this,-1,wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxTE_WORDWRAP);
  // creating the notebook
  panelstore = new wxNotebook(this,-1);
  // creating the pages
  // page 0, for the main header and footer
  thepanels[0] = new jfTwoTexts(panelstore,wxT("Main Header Skeleton"),wxT("Main Footer Skeleton"),6,true);
  // page 1, for items
  thepanels[1] = new jfTwoTexts(panelstore,wxT("Item Skeleton"),wxT("Item Separator Skeleton"),7,true);
  // page 2, category headers and footers
  // we start arranging
  sizesizer = new wxBoxSizer(wxHORIZONTAL);
  sizesizer->Add(panelstore,3,wxALL|wxEXPAND,3);
  sizesizer->Add(help_desc,2,wxALL|wxEXPAND,3);
  // sticking panels in the notebook
  panelstore->AddPage(thepanels[0],wxT("Header and Footer"));
  panelstore->AddPage(thepanels[1],wxT("Items"));
  // more arranging
  topsizer = new wxBoxSizer(wxVERTICAL);
  topsizer->Add(ndesce,0,wxALL|wxEXPAND,1);
  topsizer->Add(sizesizer,1,wxALL|wxEXPAND,3);
  // finishing
  thepanels[2] = NULL;
  thepanels[3] = NULL;
  thepanels[4] = NULL;
  SetSizer(topsizer);
  item = NULL;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// additional methods
jfSkeletonCore* jfSkelEditCore::GetSkeleton() {
  return item;
}
//---------------------------------------------------------------------------
jfs_Skeltype jfSkelEditCore::GetSkelType() const {
  assert(item!=NULL);
  return item->GetSkelType();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal helper methods
bool jfSkelEditCore::CLoadItems(jfSkeletonCore* source) {
  if (source==NULL) return false;
  help_desc->SetValue(source->GetGuide());
  thepanels[0]->SetBoth(source->header_skel,source->footer_skel);
  thepanels[1]->SetBoth(source->item_skel,source->item_separator);
  ndesce->SetFromObj(source);
  return true;
}
//---------------------------------------------------------------------------
bool jfSkelEditCore::CStoreItems(jfSkeletonCore* destination) const {
  if (destination==NULL) return false;
  destination->SetGuide(help_desc->GetValue());
  thepanels[0]->GetBoth(destination->header_skel,destination->footer_skel);
  thepanels[1]->GetBoth(destination->item_skel,destination->item_separator);
  ndesce->ChangeObj(destination);
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfSkelEditCore,wxPanel)
  EVT_BUTTON  (jfID_SKELE_SAVEBTN, jfSkelEditCore::SaveButton     )
  EVT_BUTTON  (jfID_SKELE_RESTBTN, jfSkelEditCore::RestoreButton  )
END_EVENT_TABLE()
//===============================================================================
// the constructor
jfSkelEditBase::jfSkelEditBase(wxWindow* parent, wxWindowID id, wxString inname):
    jfSkelEditCore(parent,id,inname) {
  // page 2, category headers and footers
  thepanels[2] = new jfTwoTexts(panelstore,wxT("Category Header Skeleton"),wxT("Category Footer Skeleton"),6,true);
  // sticking panels in the notebook
  panelstore->AddPage(thepanels[2],wxT("Category Header and Footer"));
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal helper methods
bool jfSkelEditBase::CLoadExtra(jfSkeletonBase* source) {
  if (source==NULL) return false;
  thepanels[2]->SetBoth(source->cat_header,source->cat_footer);
  return true;
}
//---------------------------------------------------------------------------
bool jfSkelEditBase::CStoreExtra(jfSkeletonBase* destination) const {
  if (destination==NULL) return false;
  thepanels[2]->GetBoth(destination->cat_header,destination->cat_footer);
  return true;
}
//===============================================================================
// the constructor
jfSkelEditSingle::jfSkelEditSingle(wxWindow* parent, wxWindowID id, jfSkeletonSingle* indata,
        wxString inname):jfSkelEditBase(parent,id,inname) {
  // stuff for this constructor
  typ_item = indata;
  item = indata;
  if (indata!=NULL) ChangeSkeleton(indata);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// virtual methods
//-------------------------------------------------------------------
bool jfSkelEditSingle::ChangeSkeleton(jfSkeletonCore* source) {
  if (source==NULL) return false;
  if (source->GetSkelType()!=jfs_SINGLE) return false;
  typ_item = dynamic_cast<jfSkeletonSingle*>(source);
  item = typ_item;
  return LoadSkeleton();
}
//-------------------------------------------------------------------
bool jfSkelEditSingle::LoadSkeleton() {
  if (item==NULL) return false;
  return CLoadItems(item) && CLoadExtra(dynamic_cast<jfSkeletonBase*>(item));
}
//-------------------------------------------------------------------
bool jfSkelEditSingle::SaveSkeleton() {
  if (item==NULL) return false;
  return CStoreItems(item) && CStoreExtra(dynamic_cast<jfSkeletonBase*>(item));
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// button press event handlers
//-------------------------------------------------------------------
void jfSkelEditSingle::SaveButton(wxCommandEvent& event) {
  SaveSkeleton();
}
//-------------------------------------------------------------------
void jfSkelEditSingle::RestoreButton(wxCommandEvent& event) {
  LoadSkeleton();
}
//===============================================================================
// the constructor
jfSkelEditMultiple::jfSkelEditMultiple(wxWindow* parent, wxWindowID id, jfSkeletonMultiple* indata,
        wxString inname):jfSkelEditBase(parent,id,inname) {
  // stuff for this constructor
  typ_item = indata;
  item = indata;
  // extra panels for editing
  // page 3, for category links
  thepanels[3] = new jfTwoTexts(panelstore,wxT("Category Link"),wxT("Category Link Separator"),7,true);
  // page 4, for whats left
  thepanels[4] = new jfTwoTexts(panelstore,wxT("Header Below Category Links"),wxT("Category Separator"),6,true);
  // sticking panels in the notebook
  panelstore->AddPage(thepanels[3],wxT("Category Links"));
  panelstore->AddPage(thepanels[4],wxT("Extra"));
  // changing the item
  if (indata!=NULL) ChangeSkeleton(indata);
  // done
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// virtual methods
//-------------------------------------------------------------------
bool jfSkelEditMultiple::ChangeSkeleton(jfSkeletonCore* source) {
  if (source==NULL) return false;
  if ((source->GetSkelType())!=jfs_MULTIPLE) return false;
  typ_item = dynamic_cast<jfSkeletonMultiple*>(source);
  item = typ_item;
  return LoadSkeleton();
}
//-------------------------------------------------------------------
bool jfSkelEditMultiple::LoadSkeleton() {
  if (item==NULL) return false;
  thepanels[3]->SetBoth(typ_item->catlink,typ_item->catlink_separator);
  thepanels[4]->SetBoth(typ_item->top_footer,typ_item->cat_separator);
  return CLoadItems(item) && CLoadExtra(dynamic_cast<jfSkeletonBase*>(item));
}
//-------------------------------------------------------------------
bool jfSkelEditMultiple::SaveSkeleton() {
  if (item==NULL) return false;
  thepanels[3]->GetBoth(typ_item->catlink,typ_item->catlink_separator);
  thepanels[4]->GetBoth(typ_item->top_footer,typ_item->cat_separator);
  return CStoreItems(item) && CStoreExtra(dynamic_cast<jfSkeletonBase*>(item));
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// button press event handlers
//-------------------------------------------------------------------
void jfSkelEditMultiple::SaveButton(wxCommandEvent& event) {
  SaveSkeleton();
}
//-------------------------------------------------------------------
void jfSkelEditMultiple::RestoreButton(wxCommandEvent& event) {
  LoadSkeleton();
}
//===============================================================================
// the constructor
jfSkelEditFicPart::jfSkelEditFicPart(wxWindow* parent, wxWindowID id, jfSkeletonFicPart* indata,
        wxString inname):jfSkelEditCore(parent,id,inname) {
  // stuff for this constructor
  typ_item = indata;
  item = indata;
  // extra panels for editing
  // page 2, group links
  thepanels[2] = new jfTwoTexts(panelstore,wxT("Group Link"),wxT("Group Link Separator"),7,true);
  // page 3, for part links
  thepanels[3] = new jfTwoTexts(panelstore,wxT("Part Link"),wxT("Part Link Separator"),7,true);
  // sticking panels in the notebook
  panelstore->AddPage(thepanels[2],wxT("Group Links"));
  panelstore->AddPage(thepanels[3],wxT("Part Links"));
  // changing the item
  if (indata!=NULL) ChangeSkeleton(indata);
  // done
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// virtual methods
//-------------------------------------------------------------------
bool jfSkelEditFicPart::ChangeSkeleton(jfSkeletonCore* source) {
  if (source==NULL) return false;
  if ((source->GetSkelType())!=jfs_FICPART) return false;
  typ_item = dynamic_cast<jfSkeletonFicPart*>(source);
  item = typ_item;
  return LoadSkeleton();
}
//-------------------------------------------------------------------
bool jfSkelEditFicPart::LoadSkeleton() {
  if (item==NULL) return false;
  thepanels[2]->SetBoth(typ_item->grouplink,typ_item->grouplink_separator);
  thepanels[3]->SetBoth(typ_item->partlink,typ_item->partlink_separator);
  return CLoadItems(item);
}
//-------------------------------------------------------------------
bool jfSkelEditFicPart::SaveSkeleton() {
  if (item==NULL) return false;
  thepanels[2]->GetBoth(typ_item->grouplink,typ_item->grouplink_separator);
  thepanels[3]->GetBoth(typ_item->partlink,typ_item->partlink_separator);
  return CStoreItems(item);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// button press event handlers
//-------------------------------------------------------------------
void jfSkelEditFicPart::SaveButton(wxCommandEvent& event) {
  SaveSkeleton();
}
//-------------------------------------------------------------------
void jfSkelEditFicPart::RestoreButton(wxCommandEvent& event) {
  LoadSkeleton();
}
/*****************************************************************************/

