/******************************************************************************
Name    :   skeledit.h
Author  :   John Q Metro
Purpose :   Some more miscuser interface stuff
Created :   November 25, 2009
Updated :   September 5, 2011
******************************************************************************/
#ifndef SKELEDIT_H_INCLUDED
#define SKELEDIT_H_INCLUDED
#endif

#ifndef WIDGETS1_H_INCLUDED
  #include "widgets1.h"
#endif // WIDGETS1_H_INCLUDED

#ifndef WIDGETS2_H_INCLUDED
  #include "widgets2.h"
#endif // WIDGETS2_H_INCLUDED

#ifndef SKELSTORE_H_INCLUDED
  #include "../../core/objs/skelstore.h"
#endif // SKELSTORE_H_INCLUDED
//------------------------------------------------------------------
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif
#include <wx/notebook.h>
/*****************************************************************************/
class jfSkelEditCore : public wxPanel {
  public:
    // the constructor
    jfSkelEditCore(wxWindow* parent, wxWindowID id, wxString inname);
    // virtual methods
    virtual bool ChangeSkeleton(jfSkeletonCore* source) = 0;
    virtual bool LoadSkeleton() = 0;
    virtual bool SaveSkeleton() = 0;
    // additional methods
    jfSkeletonCore* GetSkeleton();
    jfs_Skeltype GetSkelType() const;
    // events
    // button press event handlers
    virtual void SaveButton(wxCommandEvent& event) = 0;
    virtual void RestoreButton(wxCommandEvent& event) = 0;
  protected:
    // internal helper methods
    bool CLoadItems(jfSkeletonCore* source);
    bool CStoreItems(jfSkeletonCore* destination) const;
    // gui elements
    jfNameDescEditor* ndesce;
    wxButton* save_btn;
    wxButton* rest_btn;
    wxTextCtrl* help_desc;
    wxNotebook* panelstore;
    jfTwoTexts* thepanels[5] ;
    // sizers
    wxBoxSizer* topsizer;
    wxBoxSizer* sizesizer;
    // internaldata
    jfSkeletonCore* item;

    DECLARE_EVENT_TABLE()
};
//============================================================================
class jfSkelEditBase : public jfSkelEditCore {
  public:
    // the constructor
    jfSkelEditBase(wxWindow* parent, wxWindowID id, wxString inname);
  protected:
    // internal helper methods
    bool CLoadExtra(jfSkeletonBase* source);
    bool CStoreExtra(jfSkeletonBase* destination) const;
};
//=============================================================================
class jfSkelEditSingle : public jfSkelEditBase {
  public:
    // the constructor
    jfSkelEditSingle(wxWindow* parent, wxWindowID id, jfSkeletonSingle* indata, wxString inname=wxT("jfSkelEditSingle"));
    // virtual methods
    virtual bool ChangeSkeleton(jfSkeletonCore* source);
    virtual bool LoadSkeleton();
    virtual bool SaveSkeleton();
    // events
    // button press event handlers
    virtual void SaveButton(wxCommandEvent& event);
    virtual void RestoreButton(wxCommandEvent& event);
  protected:
    // internaldata
    jfSkeletonSingle* typ_item;
};
//=============================================================================
class jfSkelEditMultiple : public jfSkelEditBase {
  public:
    // the constructor
    jfSkelEditMultiple(wxWindow* parent, wxWindowID id, jfSkeletonMultiple* indata, wxString inname=wxT("jfSkelEditMultiple"));
    // virtual methods
    virtual bool ChangeSkeleton(jfSkeletonCore* source);
    virtual bool LoadSkeleton();
    virtual bool SaveSkeleton();
    // events
    // button press event handlers
    virtual void SaveButton(wxCommandEvent& event);
    virtual void RestoreButton(wxCommandEvent& event);
  protected:
    // internaldata
    jfSkeletonMultiple* typ_item;
};
//=============================================================================
class jfSkelEditFicPart : public jfSkelEditCore {
  public:
    // the constructor
    jfSkelEditFicPart(wxWindow* parent, wxWindowID id, jfSkeletonFicPart* indata, wxString inname=wxT("jfSkelEditFicPart"));
    // virtual methods
    virtual bool ChangeSkeleton(jfSkeletonCore* source);
    virtual bool LoadSkeleton();
    virtual bool SaveSkeleton();
    // events
    // button press event handlers
    virtual void SaveButton(wxCommandEvent& event);
    virtual void RestoreButton(wxCommandEvent& event);
  protected:
    // internaldata
    jfSkeletonFicPart* typ_item;
};
/*****************************************************************************/
