/******************************************************************************
Name    :   widgets1.cpp
Author  :   John Q Metro
Purpose :   Some misc user interface stuff
Created :   April 1, 2009
Updated :   January 14, 2012
******************************************************************************/
#ifndef WIDGETS1_H_INCLUDED
  #include "widgets1.h"
#endif // WIDGETS1_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
//****************************************************************************
/* one of the things I've decided to do for base objects is that they must
have nome sort of non blank name. below is a basic editor that is supposed to
enforce that */
jfNameEditor::jfNameEditor(wxWindow* parent, wxString default_name,
        wxWindowID id,const wxString& name):wxPanel(parent,id,wxDefaultPosition,
        wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // first, we check the name
  assert(!default_name.IsEmpty());
  backup_data = default_name;
  // creating the widgets
  label = new wxStaticText(this,-1,wxT("Name: "));
  textentry = new wxTextCtrl(this,jfID_NAME_EDITOR_EDIT,backup_data);
  // creating the sizer and arranging things
  topsizer = new wxBoxSizer(wxHORIZONTAL);
  topsizer->Add(label,0,wxALL|wxALIGN_LEFT|wxALIGN_CENTRE_VERTICAL,3);
  topsizer->Add(textentry,1,wxTOP|wxBOTTOM|wxRIGHT|wxEXPAND,3);
  SetSizer(topsizer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// some extra methods
//---------------------------------------------------------------------------
bool jfNameEditor::IsEmpty() const {
  wxString tvalue = textentry->GetValue();
  Dtrim(tvalue);
  return tvalue.IsEmpty();
}
//----------------------------------------------------------------------------
/* copies over (and returns true) the textentry contents if they are not
blank to the variable backup_data */
bool jfNameEditor::TryCopy() {
  wxString tvalue;
  if (textentry->IsModified()) {
    if (IsEmpty()) return false;
    else {
      backup_data = textentry->GetValue();
      Dtrim(backup_data);
      textentry->SetModified(false);
      return true;
    }
  }
  else return true;
}
//---------------------------------------------------------------------------
// returns the (non empty) contents of the internal variable
wxString jfNameEditor::GetBackup() const {
  return backup_data;
}
//---------------------------------------------------------------------------
/* we get the string, if the edit control is blank, an error message is shown
and we revert back to the old value before returning it */
wxString jfNameEditor::TryGet() {
  if (!TryCopy()) {
    // the copying has failed, we show the error
    ShowEmptyMessage();
    textentry->SetValue(backup_data);
    textentry->SetModified(false);
  }
  return backup_data;
}
//-------------------------------------------------------------------------
bool jfNameEditor::SetNameData(const wxString& inname) {
  if (inname.IsEmpty()) return false;
  else {
    backup_data = inname;
    textentry->SetValue(inname);
    textentry->SetModified(false);
  }
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// lossing the focus triggers the check
void jfNameEditor::LooseFocus(wxFocusEvent& event) {
  TryGet();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// shows the error message that the string is empty
void jfNameEditor::ShowEmptyMessage() {
  // constants and local varibales
  const wxString ermsg=wxT("Blank names are not allowed! Reverting to the previous name.");
  wxMessageDialog* msgbox;
  // we create the dialog
  msgbox = new wxMessageDialog(this,ermsg,wxT("Error!"),wxOK);
  msgbox->ShowModal();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfNameEditor, wxPanel)
  EVT_KILL_FOCUS(/*jfID_NAME_EDITOR_EDIT,*/jfNameEditor::LooseFocus)
END_EVENT_TABLE()
//****************************************************************************
// methods for jfNameDescEditor
//=============================================================================
// the constructor
jfNameDescEditor::jfNameDescEditor(wxWindow* parent,wxWindowID id, bool isvert,bool ismulti, wxWindowID nameid,
			wxString default_name,wxString default_desc, const wxString& motifname):wxPanel(parent,id,wxDefaultPosition,
      wxDefaultSize,wxTAB_TRAVERSAL,motifname) {
  // local vars
  long tc_style = (wxTE_LEFT | wxTE_BESTWRAP);
  long bounds = (wxRIGHT|wxBOTTOM|wxEXPAND);
  // we start with checking as default name
  assert(!default_name.IsEmpty());
  // we set the flags
  vertical = isvert;
  multiline = ismulti;
  extra_count = 0;
  // we start with creating the gui elements
  // name
  name_label = new wxStaticText(this,-1,wxT("Name :"));
  name_edit = new wxTextCtrl(this,nameid,default_name);
  backup_data = default_name;
  // description label
  desc_label = new wxStaticText(this,-1,wxT("Description :"));
  // calculating the size difference
  wxSize decsize = desc_label->GetEffectiveMinSize();
  wxSize namsize = name_label->GetEffectiveMinSize();
  int difference = decsize.GetWidth() - namsize.GetWidth();
  // making the description editor
  if (multiline) tc_style|=wxTE_MULTILINE;
  desc_edit = new wxTextCtrl(this,-1,default_desc,wxDefaultPosition,wxDefaultSize,tc_style);
  // now, we start with sizers and arrangement
  // the top row, with the name editor, other things can be added later
  namerow = new wxBoxSizer(wxHORIZONTAL);
  namerow->Add(name_label,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
  namerow->AddSpacer(difference);
  namerow->Add(name_edit,10,wxALL|wxEXPAND,3);
  // the description row
  if (vertical) descrow = new wxBoxSizer(wxVERTICAL);
  else descrow = new wxBoxSizer(wxHORIZONTAL);
  descrow->Add(desc_label,0,wxALL|wxALIGN_CENTRE_VERTICAL,3);
  if (vertical) bounds|= wxLEFT;
  else bounds|=wxTOP;
  descrow->Add(desc_edit,1,bounds,3);
  // finsihing with the arrangement
  mainsizer = new wxBoxSizer(wxVERTICAL);
  mainsizer->Add(namerow,0,wxEXPAND);
  if (vertical) mainsizer->Add(descrow,1,wxEXPAND);
  else mainsizer->Add(descrow,0,wxEXPAND);
  // done..
  SetSizer(mainsizer);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// adding extra objects or sizers
//------------------------------------------------------------------
// adding a control to the toprow
void jfNameDescEditor::AddItem(wxWindow* window, size_t prop) {
  // boundary style
  long bstyle = (wxTOP|wxBOTTOM|wxRIGHT);
  // a basic check
  assert(window!=NULL);
  // adding
  if (prop!=0) bstyle|=wxEXPAND;
  namerow->Add(window,prop,bstyle,3);
  extra_count++;
  namerow->Layout();
}
//------------------------------------------------------------------
// adding a sizer to the top row
void jfNameDescEditor::AddItem(wxSizer* sizer, size_t prop) {
  // boundary style
  long bstyle = (wxTOP|wxBOTTOM|wxRIGHT);
  // a basic check
  assert(sizer!=NULL);
  // adding
  if (prop!=0) bstyle|=wxEXPAND;
  namerow->Add(sizer,prop,bstyle,3);
  extra_count++;
  namerow->Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting some info
//------------------------------------------------------------------
// gets the name (does not allow blanks)
wxString jfNameDescEditor::TryGetName() const {
  if (!TryCopy()) {
    // the copying has failed, we show the error
    ShowEmptyMessage();
    name_edit->ChangeValue(backup_data);
    name_edit->SetModified(false);
  }
  return name_edit->GetValue();
}
//--------------------------------------------------------------
bool jfNameDescEditor::IsNameEmpty() const {
  return name_edit->IsEmpty();
}
//------------------------------------------------------------------
// gets the description
wxString jfNameDescEditor::GetDesc() const {
  wxString res = desc_edit->GetValue();
  Dtrim(res);
  return res;
}
//------------------------------------------------------------------
// return true is the name edit is not blank, false otherwise
bool jfNameDescEditor::NameStatus() const {
  return TryCopy();
}
//------------------------------------------------------------------
// returns true if the description label is above the editor
bool jfNameDescEditor::IsVertical() const {
  return vertical;
}
//------------------------------------------------------------------
// returns true if the description editor is multiline
bool jfNameDescEditor::IsMultiline() const {
  return multiline;
}
//------------------------------------------------------------------
// changes the name and description in inobj to what is in the controls
bool jfNameDescEditor::ChangeObj(jfBaseObj* inobj) const {
  // things that make us return false
  if (inobj==NULL) return false;
  if (!NameStatus()) return false;
  // setting the data
  inobj->SetName(TryGetName());
  inobj->SetDescription(GetDesc());
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting some info
//------------------------------------------------------------------
// name setting
bool jfNameDescEditor::SetName(wxString newname){
  // we have to check the name first
  Dtrim(newname);
  if (newname.IsEmpty()) return false;
  // setting the name
  name_edit->SetValue(newname);
  return true;
}
//------------------------------------------------------------------
// setting the description
void jfNameDescEditor::SetDescription(const wxString& newdesc) {
  desc_edit->SetValue(newdesc);
}
//------------------------------------------------------------------
// loading name and description from an object
bool jfNameDescEditor::SetFromObj(const jfBaseObj* inobj) {
  if (inobj==NULL) return false;

  if (!SetName(inobj->GetName())) return false;
  desc_edit->SetValue(inobj->GetDescription());
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfNameDescEditor::TryCopy() const {
  wxString tvalue;
  if (name_edit->IsModified()) {
    tvalue = name_edit->GetValue();
    Dtrim(tvalue);
    if (tvalue.IsEmpty()) return false;
    else {
      backup_data = name_edit->GetValue();
      name_edit->SetModified(false);
      return true;
    }
  }
  else return true;
}
//-----------------------------------------------------------------------
wxString jfNameDescEditor::GetBackup() const {
  return backup_data;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// lossing the focus triggers the check
void jfNameDescEditor::LooseFocus(wxFocusEvent& event) {
  TryGetName();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// shows the error message that the string is empty
void jfNameDescEditor::ShowEmptyMessage() const {
  // constants and local varibales
  const wxString ermsg=wxT("Blank names are not allowed! Reverting to the previous name.");
  wxMessageDialog* msgbox;
  // we create the dialog
  msgbox = new wxMessageDialog(const_cast<jfNameDescEditor*>(this),ermsg,wxT("Error!"),wxOK);
  msgbox->ShowModal();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfNameDescEditor, wxPanel)
  EVT_KILL_FOCUS(/*jfID_NAME_EDITOR_EDIT,*/jfNameDescEditor::LooseFocus)
END_EVENT_TABLE()
//**************************************************************************
// a buttonless non-modal text label status display dialog
//+++++++++++++++++++++++++++++++++++++++++++++++++
// the constructor
jfTextStatusDialog::jfTextStatusDialog(wxWindow *parent, wxWindowID id, const wxString& title, const wxString& display_msg,
			const wxSize &size):wxDialog(parent,id,title,wxDefaultPosition,size,wxCAPTION,wxT("TextStatusDialog")) {
	// creating the elements
	info_display = new wxStaticText(this,-1,display_msg);
	top_sizer = new wxBoxSizer(wxHORIZONTAL);
	// finishing
	top_sizer->Add(info_display,1,wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL|wxALIGN_CENTER,10);
	SetSizer(top_sizer);
	CentreOnScreen(wxBOTH);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// other methods
//----------------------------------------
wxString jfTextStatusDialog::GetMessage() const {
	return info_display->GetLabel();
}
//----------------------------------------
void jfTextStatusDialog::ChangeMessage(const wxString new_message) {
	info_display->SetLabel(new_message);
}
//****************************************************************************
