/******************************************************************************
Name    :   widgets1.h
Author  :   John Q Metro
Purpose :   Some misc user interface stuff
Created :   April 1, 2009
Updated :   January 14, 2011
******************************************************************************/
// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif
//-------------------------------------------------------------------------------
#ifndef WIDGETS1_H_INCLUDED
#define WIDGETS1_H_INCLUDED
#endif // WIDGETS1_H_INCLUDED
#ifndef BASEOBJ_H_INCLUDED
    #include "../../core/objs/baseobj.h"
#endif // BASEOBJ_H_INCLUDED
//****************************************************************************
/* one of the things I've decided to do for base objects is that they must
have some sort of non blank name. below is a basic editor that is supposed to
enforce that */
class jfNameEditor : public wxPanel {
  public:
    // the constructor
    jfNameEditor(wxWindow* parent, wxString default_name = wxT("default name"),
            wxWindowID id = wxID_ANY, const wxString& name = wxT("jfNameEditor"));
    // some extra methods
    bool IsEmpty() const;
    bool TryCopy();
    wxString GetBackup() const;
    wxString TryGet();
    bool SetNameData(const wxString& inname);
    // events
    void LooseFocus(wxFocusEvent& event);
  protected:
    // an internal helper method
    void ShowEmptyMessage();
    // user interface elements
    wxBoxSizer* topsizer;
    wxStaticText* label;
    wxTextCtrl* textentry;
    // backup data
    wxString backup_data;

    DECLARE_EVENT_TABLE()

};
/****************************************************************************/
// new version of class jfNameDescEditor
class jfNameDescEditor : public wxPanel {
  public:
    // the constructor
    jfNameDescEditor(wxWindow* parent,wxWindowID id, bool isvert, bool ismulti, wxWindowID nameid=-1,
				wxString default_name = wxT("default name"),wxString default_desc = wxT(""), const wxString& motifname=wxT("jfNameDescEditor"));
    // adding extra objects or sizers
    virtual void AddItem(wxWindow* window, size_t prop = 0);
    virtual void AddItem(wxSizer* sizer, size_t prop = 0);
    // getting some info
    wxString TryGetName() const;
    bool IsNameEmpty() const;
    wxString GetDesc() const;
    bool NameStatus() const;
    bool IsVertical() const;
    bool IsMultiline() const;
    bool ChangeObj(jfBaseObj* inobj) const;
    // setting some info
    bool SetName(wxString newname);
    void SetDescription(const wxString& newdesc);
    bool SetFromObj(const jfBaseObj* inobj);
    // some extra methods
    bool TryCopy() const;
    wxString GetBackup() const;
    // events
    void LooseFocus(wxFocusEvent& event);
  protected:
    // an internal helper method
    void ShowEmptyMessage() const;
    // the main widgets
    wxStaticText* name_label;
    wxStaticText* desc_label;
    wxTextCtrl*   name_edit;
    wxTextCtrl*   desc_edit;
    // sizers
    wxBoxSizer* namerow;
    wxBoxSizer* descrow;
    wxBoxSizer* mainsizer;
    // extra flags
    bool multiline,vertical;
    // internal data
    mutable wxString backup_data;
    size_t extra_count;

    DECLARE_EVENT_TABLE()
};
//==========================================================================
// a buttonless non-modal text label status display dialog
class jfTextStatusDialog : public wxDialog {
	public:
		// the constructor
		jfTextStatusDialog(wxWindow *parent, wxWindowID id, const wxString& title, const wxString& display_msg, const wxSize &size=wxDefaultSize);
		// other methods
		wxString GetMessage() const;
		void ChangeMessage(const wxString new_message);
	protected:
		// GUI stuff
		wxStaticText* info_display;
		wxBoxSizer* top_sizer;
};

//****************************************************************************
