/******************************************************************************
Name    :   widgets2.cpp
Author  :   John Q Metro
Purpose :   Some more miscuser interface stuff
Created :   November 24, 2009
Updated :   January 3, 2014
******************************************************************************/
#ifndef WIDGETS2_H_INCLUDED
  #include "widgets2.h"
#endif // WIDGETS2_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//--------------------------------------------------------------------------
/*****************************************************************************/
// the constructor
jfTwoTexts::jfTwoTexts(wxWindow* parent,wxString label1, wxString label2, size_t prp,
      bool multiline,wxWindowID id,const wxString& name):wxPanel(parent,id ,
      wxDefaultPosition, wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // starting
  assert(prp>0);
  assert(prp<10);
  prop = prp;
  // we create the gui elements
  slabel1 = new wxStaticText(this,-1,label1);
  slabel2 = new wxStaticText(this,-1,label2);
  // we create the editors
  if (multiline) {
    edit2 = new wxTextCtrl(this,-1,wxEmptyString,wxDefaultPosition,wxDefaultSize, wxTE_MULTILINE|wxTE_DONTWRAP);
    edit1 = new wxTextCtrl(this,-1,wxEmptyString,wxDefaultPosition,wxDefaultSize, wxTE_MULTILINE|wxTE_DONTWRAP);
  }
  else {
    edit2 = new wxTextCtrl(this,-1);
    edit1 = new wxTextCtrl(this,-1);
  }
  // we create the sizer and arrange
  topsizer = new wxBoxSizer(wxVERTICAL);
  topsizer->Add(slabel1,0,wxALL,3);
  topsizer->Add(edit1,prp,wxALL|wxEXPAND,3);
  topsizer->AddSpacer(8);
  topsizer->Add(slabel2,0,wxALL,3);
  topsizer->Add(edit2,10-prp,wxALL|wxEXPAND,3);
  // done
  SetSizer(topsizer);
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting strings
//------------------------------------------
void jfTwoTexts::Set1(wxString inval) {
  edit1->SetValue(inval);
}
//------------------------------------------
void jfTwoTexts::Set2(wxString inval) {
  edit2->SetValue(inval);
}
//------------------------------------------
void jfTwoTexts::SetBoth(wxString inval1, wxString inval2) {
  edit1->SetValue(inval1);
  edit2->SetValue(inval2);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting strings
//------------------------------------------
wxString jfTwoTexts::Get1() const {
  return edit1->GetValue();
}
//------------------------------------------
wxString jfTwoTexts::Get2() const {
  return edit2->GetValue();
}
//------------------------------------------
void jfTwoTexts::GetBoth(wxString& out1, wxString& out2) const {
  out1 = edit1->GetValue();
  out2 = edit2->GetValue();
}
//=======================================================================
// the constructor
jfTagCategoryEdit::jfTagCategoryEdit(wxWindow* parent, wxWindowID id,
    jfTagListing* inlist, jfTAG_STATUS wstatus, const wxString& name):
		wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
	// the usual asserts to start with
	assert(parent!=NULL);
	assert(inlist!=NULL);
  assert(wstatus!=jfts_NONE);
  // some assignments
  xlink = inlist;
  tag_type = wstatus;
  ssize = false;
  // first, the label
  wxString label_str;
  const wxString ftmm = wxT("tag must match");
  if (tag_type==jfts_INCLUDE) label_str = wxT("Every ") + ftmm;
  else if (tag_type==jfts_ALTERNATE) label_str = wxT("One ") + ftmm;
  else if (tag_type==jfts_EXCLUDE) label_str = wxT("No ") + ftmm;
  else assert(false);
  main_label = new wxStaticText(this,-1,label_str);
  // next, the buttons
  wxSize btnsize(24,24);
  move_into = new wxButton(this,jfID_MOVEINTO_BTN, wxT(">"));
  move_outof = new wxButton(this,jfID_MOVEOUTO_BTN, wxT("<"));
  move_into->SetMinSize(btnsize);
  move_outof->SetMinSize(btnsize);
  // the list box
  tag_display = new wxListBox(this,-1,wxDefaultPosition,wxDefaultSize,0,NULL,wxLB_EXTENDED | wxLB_NEEDED_SB | wxLB_SORT);
  // arranging the sizers
  // the button sizer
  btn_sizer = new wxBoxSizer(wxVERTICAL);
  btn_sizer->Add(move_into,1,wxALL|wxALIGN_BOTTOM,3);
  btn_sizer->Add(move_outof,1,wxALL|wxALIGN_TOP,3);
  // the sizer holding the buttons and list box
  lbox_sizer = new wxBoxSizer(wxHORIZONTAL);
  lbox_sizer->Add(btn_sizer,0,wxALIGN_LEFT|wxALL,3);
  lbox_sizer->Add(tag_display,1,wxEXPAND|wxTOP|wxLEFT|wxRIGHT,3);
  // the top level sizer
  top_sizer = new wxBoxSizer(wxVERTICAL);
  top_sizer->Add(main_label,0,wxALL|wxALIGN_CENTRE,3);
  top_sizer->Add(lbox_sizer,1,wxEXPAND|wxTOP|wxBOTTOM|wxRIGHT,3);
  // finishing off
  SetSizer(top_sizer);
  Layout();
  // loading data
  LoadTags();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting from source...
bool jfTagCategoryEdit::SetFromSource(jfTagListing* inlist) {
  if (inlist==NULL) return false;
  xlink = inlist;
  LoadTags();
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// enables or disables the > button
void jfTagCategoryEdit::MoveIntoEnable(bool doenable) {
  move_into->Enable(doenable);
}
//------------------------------------------------------------------
/* whem moving from none to a subcategory, the caller removes the tags
and makes the list, this functions places the tags in the list box and
sets the values to the one for this panel */
bool jfTagCategoryEdit::MoveInto(wxArrayString* list_of_tags) {
  // local variables
  size_t loopc, loopmax;
  // some basic cases
  if (list_of_tags==NULL) return false;
  loopmax = list_of_tags->Count();
  if (loopmax==0) return false;
  /* the into loop. we will not bother to check if the tag is nonexistant
  or if the value is not jfts_NONE. This method should not be abused */
  for (loopc=0;loopc<loopmax;loopc++) {
    (*xlink)[list_of_tags->Item(loopc)] = tag_type;
    tag_display->Append(list_of_tags->Item(loopc));
  }
  // done
  delete list_of_tags;
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handlers
//--------------------------------------------------------------------
// this is really handles by the parent, but we must identify the source...
void jfTagCategoryEdit::OnMoveInto(wxCommandEvent& event) {
  event.SetInt((int)tag_type);
  event.Skip();
}
//-----------------------------------------------------------------
/* when moving items out, we handle everything here except adding to
the NONE list box, we use an event to pass the data up */
void jfTagCategoryEdit::OnMoveOutof(wxCommandEvent& event) {
  // local variables
  wxArrayString* passout;
  wxArrayInt* out_sel;
  int scount, sindex;
  wxString citem;
  // starting
  passout = new wxArrayString();
  out_sel = new wxArrayInt();
  tag_display->GetSelections((*out_sel));
  scount = out_sel->Count();
  /* there should *never* be nothing selected, the button must be
  disabled if there is nothing selected */
  if (scount==0) {
    delete passout;
    delete out_sel;
    assert(false);
  }
  // we loop (twice) to do everything except pass the data back up
  // the first loop is to build the array and change the tag status
  for (sindex=0;sindex<scount;sindex++) {
    citem = tag_display->GetString(out_sel->Item(sindex));
    (*xlink)[citem] = jfts_NONE;
    passout->Add(citem);
  }
  // the second loop, we remove the selected items (in reverse)
  for (sindex=scount-1;sindex>=0;sindex--) {
    tag_display->Delete(out_sel->Item(sindex));
  }
  // once done with the loops, a bit of clearing up before we send the event
  move_outof->Enable(false);
  delete out_sel;
  wxCommandEvent outevent = wxCommandEvent(jf_TAGEDIT_OUTOF_EVENT);
  outevent.SetClientData(passout);
  wxPostEvent(GetParent(), outevent);
}
//--------------------------------------------------------------------
// we cannot move an item out until there is at least one selected
void jfTagCategoryEdit::OnSelected(wxCommandEvent& event) {
  move_outof->Enable(true);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// graphics re-arranging
bool jfTagCategoryEdit::ToSmallSize(bool doit) {
  if (doit==ssize) return false;
  // to small size
  if (doit) {
    // removing the buttons
    lbox_sizer->Detach(btn_sizer);
    btn_sizer->Detach(move_outof);
    btn_sizer->Detach(move_into);
    // readding them side by side
    lbox_sizer->Prepend(move_outof,0,wxALIGN_CENTER_VERTICAL|wxALL,2);
    lbox_sizer->Prepend(move_into,0,wxALIGN_CENTER_VERTICAL|wxTOP|wxBOTTOM|wxLEFT,2);
    ssize = true;
  }
  // from small to large
  else {
    // removing the buttons
    lbox_sizer->Detach(move_into);
    lbox_sizer->Detach(move_outof);
    // re-adding them stacked
    btn_sizer->Add(move_into,1,wxALL|wxALIGN_BOTTOM,3);
    btn_sizer->Add(move_outof,1,wxALL|wxALIGN_TOP,3);
    lbox_sizer->Prepend(btn_sizer,0,wxALIGN_LEFT|wxALL,3);
    ssize = false;
  }
  // done
  Layout();
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loads data from xlink to the gui elements
void jfTagCategoryEdit::LoadTags() {
  // local variables
  wxString tag;
  jfTagListing::const_iterator indexer;
  // we start by clearing out whatever is there now...
  tag_display->Clear();
  // next, we have a loop that adds the items
  for (indexer=xlink->begin();indexer!=(xlink->end());indexer++) {
    if ((indexer->second)==tag_type) {
      tag = indexer->first;
      tag_display->Append(tag);
    }
  }
  // nothing should be selected after this, so we disbale the move out button
  move_outof->Enable(false);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfTagCategoryEdit,wxPanel)
  EVT_BUTTON  (jfID_MOVEOUTO_BTN, jfTagCategoryEdit::OnMoveOutof   )
  EVT_BUTTON  (jfID_MOVEINTO_BTN, jfTagCategoryEdit::OnMoveInto    )
  EVT_LISTBOX  (-1, jfTagCategoryEdit::OnSelected  )
END_EVENT_TABLE()
//=============================================================================
jfTagCategorizer::jfTagCategorizer(wxWindow* parent, wxWindowID id, bool triple,
    jfTagListing* inlist, const wxString& name):wxPanel(parent,id,
    wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // assert checks
  if (inlist==NULL) xlist = new jfTagListing();
  else xlist = inlist;
  triple_mode = triple;
  // we start with the gui elements
  main_label = new wxStaticText(this,-1,wxT("Main Tag List"));
  main_tagbox =  new wxListBox(this,-1,wxDefaultPosition,wxDefaultSize,0,NULL,wxLB_EXTENDED | wxLB_NEEDED_SB | wxLB_SORT);
  // the sub editors
  sub_editors[0] = new jfTagCategoryEdit(this,-1, xlist,jfts_INCLUDE);
  if (triple_mode) sub_editors[1] = new jfTagCategoryEdit(this,-1, xlist,jfts_ALTERNATE);
  else sub_editors[1] = NULL;
  sub_editors[2] = new jfTagCategoryEdit(this,-1, xlist,jfts_EXCLUDE);
  // we start with sizer arranging
  // the main tag list sizer
  lbox_sizer = new wxBoxSizer(wxVERTICAL);
  lbox_sizer->Add(main_label,0,wxALL,3);
  lbox_sizer->Add(main_tagbox,1,wxALL|wxEXPAND,3);
  // the sub_editor sizer
  subedit_sizer = new wxBoxSizer(wxVERTICAL);
  subedit_sizer->Add(sub_editors[0],1,wxALL|wxEXPAND,2);
  if (triple_mode) subedit_sizer->Add(sub_editors[1],1,wxALL|wxEXPAND,2);
  subedit_sizer->Add(sub_editors[2],1,wxALL|wxEXPAND,2);
  // the topmost sizer
  top_sizer = new wxBoxSizer(wxHORIZONTAL);
  top_sizer->Add(lbox_sizer,2,wxEXPAND);
  top_sizer->Add(subedit_sizer,3,wxEXPAND);
  SetSizer(top_sizer);
  Layout();
  // finishing
  LoadTags();
  Mieall();

}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting from source...
bool jfTagCategorizer::SetFromSource(jfTagListing* inlist) {
  if (inlist==NULL) return false;
  jfTagListing* olist = xlist;
  xlist = inlist;
  LoadTags();
  for (size_t xl = 0; xl<3; xl++) {
    if ((!triple_mode) && (xl==1)) continue;
    sub_editors[xl]->SetFromSource(xlist);
  }
  Mieall();
  delete olist;
  return true;
}
//---------------------------------------------------------------
const jfTagListing* jfTagCategorizer::GetList() const {
  return xlist;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// events
//--------------------------------------------------------------------
// moving selected items into one of the sub categories
void jfTagCategorizer::OnMoveInto(wxCommandEvent& event) {
  // constant
  const wxString fname = wxT("jfTagCategorizer::OnMoveInto");
  // local variables
  wxArrayString* selected_tags;
  wxArrayInt* selected_indexes;
  jfTAG_STATUS where_to;
  int loops,scount;
  wxString seltag;
  // we start by finding out where the tags go to
  where_to = (jfTAG_STATUS)(event.GetInt());
  // next, we find the selected items
  selected_indexes = new wxArrayInt();
  main_tagbox->GetSelections((*selected_indexes));
  scount = selected_indexes->Count();
  assert(scount>0);
  // we use the selected items to build a list...
  selected_tags = new wxArrayString();
  for (loops=0;loops<scount;loops++) {
    seltag = main_tagbox->GetString((*selected_indexes)[loops]);
    selected_tags->Add(seltag);
  }
  // now we remove the selected items from the main listbox
  for (loops=scount-1;loops>=0;loops--) {
    main_tagbox->Delete((*selected_indexes)[loops]);
  }
  // some cleaning up...
  delete selected_indexes;
  Mieall();
  // we now call the appropriate sub-editor with the string
  if (where_to==jfts_INCLUDE) sub_editors[0]->MoveInto(selected_tags);
  else if (where_to==jfts_ALTERNATE) sub_editors[1]->MoveInto(selected_tags);
  else if (where_to==jfts_EXCLUDE) sub_editors[2]->MoveInto(selected_tags);
  else assert(false);

}
//--------------------------------------------------------------------
// here, everything has been handles except adding the tags to the main listboc
void jfTagCategorizer::OnMoveOutOf(wxCommandEvent& event) {
  // local variables
  wxArrayString* namelist;
  size_t tcount,tindex;
  // we've got the event... get the data
  namelist = reinterpret_cast<wxArrayString*> (event.GetClientData());
  // we loop over the data
  tcount = namelist->Count();
  for (tindex=0;tindex<tcount;tindex++) {
    main_tagbox->Append(namelist->Item(tindex));
  }
  // we are done
  delete namelist;
}
//--------------------------------------------------------------------
void jfTagCategorizer::OnSelected(wxCommandEvent& event) {
  Mieall(true);
}
//--------------------------------------------------------------------
void jfTagCategorizer::HandleResize(wxSizeEvent& qevent) {
  const wxString fname = wxT("jfTagCategorizer::HandleResize");
  int yheight = qevent.GetSize().GetHeight();
  /**/jfXLogMessageI(fname,1,yheight);
  if (yheight<275) ToSmallSize(true);
  else if (yheight>=275) ToSmallSize(false);
  qevent.Skip();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// graphics re-arranging
bool jfTagCategorizer::ToSmallSize(bool doit) {
  if (!triple_mode) return false;
  bool qx;

  for (size_t xl = 0; xl<3; xl++) {
    qx = sub_editors[xl]->ToSmallSize(doit);
  }
  return qx;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loads data from xlink to the gui elements
void jfTagCategorizer::LoadTags() {
  // local variables
  wxString tag;
  jfTagListing::const_iterator indexer;
  // we start by clearing out whatever is there now...
  main_tagbox->Clear();
  // next, we have a loop that adds the items
  for (indexer=xlist->begin();indexer!=(xlist->end());indexer++) {
    if ((indexer->second)==jfts_NONE) {
      tag = indexer->first;
      main_tagbox->Append(tag);
    }
  }
}
//---------------------------------------------------------------------
void jfTagCategorizer::Mieall(bool enx) {
  size_t loops;
  for (loops=0;loops<3;loops++) {
    if ((!triple_mode) && (loops==1)) continue;
    sub_editors[loops]->MoveIntoEnable(enx);
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfTagCategorizer,wxPanel)
  EVT_COMMAND (-1,jf_TAGEDIT_OUTOF_EVENT, jfTagCategorizer::OnMoveOutOf   )
  EVT_BUTTON  (jfID_MOVEINTO_BTN, jfTagCategorizer::OnMoveInto   )
  EVT_LISTBOX (-1, jfTagCategorizer::OnSelected  )
  EVT_SIZE(jfTagCategorizer::HandleResize)
END_EVENT_TABLE()
//*****************************************************************************
