/******************************************************************************
Name    :   widgets3.cpp
Author  :   John Q Metro
Purpose :   Some misc user interface stuff
Created :   December 5, 2010
Updated :   September 3, 2015
******************************************************************************/
#ifndef WIDGETS3_H_INCLUDED
  #include "widgets3.h"
#endif // WIDGETS3_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef UTILS3_H_INCLUDED
#include "../../core/utils/utils3.h"
#endif // UTILS3_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef INITEND_H_INCLUDED
  #include "../../initend.h"
#endif // INITEND_H_INCLUDED

#include <wx/dir.h>
#include <limits>
//****************************************************************************
const int ZINTMAX = std::numeric_limits<int>::max();
//============================================================================
/*
// constructor2
jfIntEdit::jfIntEdit(wxWindow* parent, wxWindowID id, bool issigned, long value, const wxString& name):
      wxTextCtrl(parent,id,wxEmptyString,wxDefaultPosition,wxDefaultSize,0,Vnumber,name) {
  // we start...
  svalue = issigned;
  if (!svalue) assert(value>=0);
  SetMaxLength(9);
  if (svalue) SetValue(IInt2Str(value));
  else SetValue(UInt2Str(value));
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// value setting and getting
//------------------------------------------------
bool jfIntEdit::GetSigned() const {
  return svalue;
}
//------------------------------------------------
bool jfIntEdit::GetLValue(long& ovalue) const {
  // vars
  wxString tvalue = GetValue();
  long* oval1;
  unsigned long* oval2;
  bool resval;
  // starting
  Dtrim(tvalue);
  if (tvalue.IsEmpty()) return 0;
  // converting
  if (svalue) {
    oval1 = new long;
    resval = tvalue.ToLong(oval1);
    if (resval) ovalue = (*oval1);
    delete oval1;
  }
  else {
    oval2 = new unsigned long;
    resval = tvalue.ToULong(oval2);
    if (resval) ovalue = (*oval2);
    delete oval2;
  }
  return resval;
}
//------------------------------------------------
bool jfIntEdit::SetLValue(const long& inval) {
  if (svalue) {
    SetValue(IInt2Str(inval));
    return true;
  }
  else {
    if (inval<0) return false;
    SetValue(UInt2Str(inval));
    return true;
  }
}
*/
//==========================================================================
// grouping a wxTextCtrl and it's associated wxStaticTextLabel
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfLabeledEditCore::jfLabeledEditCore(wxWindow* parent, wxWindowID id,
        wxString label, size_t spacsize, int orient, const wxString& name):
        wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  the_editor = NULL;
  the_label = new wxStaticText(this,-1,wxT("a"));
  mksizer = NULL;
  orientval = orient;
  SetLabel(label);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfLabeledEditCore::SetLabel(wxString inval) {
  labeltext = inval;
  wxString buffer = labeltext + wxT(" : ");
  the_label->SetLabel(buffer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wxString jfLabeledEditCore::GetLabel() const {
  return labeltext;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfLabeledEditCore::SetUpWidgets(size_t ssize) {
  assert(the_editor!=NULL);
  mksizer = new wxBoxSizer(orientval);
  mksizer->Add(the_label,0,wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT,0);
  mksizer->AddSpacer(ssize);
  if (orientval==wxVERTICAL) mksizer->Add(the_editor,0,wxALIGN_LEFT|wxEXPAND,0);
  else mksizer->Add(the_editor,1,wxALIGN_LEFT|wxEXPAND,0);
  SetSizer(mksizer);
}
//==================================================================
// grouping a wxTextCtrl and it's associated wxStaticTextLabel
jfLabeledEdit::jfLabeledEdit(wxWindow* parent, wxWindowID id, wxString label,
      wxString data, size_t spacsize,int orient, const wxString& name):jfLabeledEditCore(
      parent,id,label,spacsize,orient,name) {
  // creating the text control
  text_editor = new wxTextCtrl(this,-1,data);
  the_editor = text_editor;
  SetUpWidgets(spacsize);
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting data
//---------------------------------------------
void jfLabeledEdit::SetValue(wxString inval) {
  text_editor->SetValue(inval);
}
//---------------------------------------------
void jfLabeledEdit::SetBoth(wxString in_label, wxString in_value) {
  SetLabel(in_label);
  SetValue(in_value);
}
//------------------------------------------
// sort of like SetValue, except we get rid of certain chars
void jfLabeledEdit::SetFiletag(wxString inval) {
  // replacements
	inval = UBReplace(inval,UFNAME_DELIMS);
  SetValue(inval);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting data
wxString jfLabeledEdit::GetValue() const {
  return text_editor->GetValue();
}
//===========================================================================
// constructor
jfLabeledUIntEdit::jfLabeledUIntEdit(wxWindow* parent,wxString label, size_t maxv,
    size_t data, size_t spacsize,  wxWindowID id, int orient, const wxString& name):
    jfLabeledEditCore(parent,id,label,spacsize,orient, name) {
  // special asserts
  assert(maxv<=ZINTMAX); // because wxSpinCtrl uses int
  assert(data<=maxv);
  // finishing
  spin_editor = new wxSpinCtrl(this,-1);
  spin_editor->SetRange(0,maxv);
  the_editor = spin_editor;
  SetUpWidgets(spacsize);
  spin_editor->SetValue(data);
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting data
//----------------------------------------
bool jfLabeledUIntEdit::SetMaximum(const size_t& maxval) {
  if (maxval>ZINTMAX) return false;
  int qx = spin_editor->GetValue();
  spin_editor->SetRange(0,maxval);
  if (qx>maxval) spin_editor->SetValue(maxval);
  return true;
}
//----------------------------------------
bool jfLabeledUIntEdit::SetValue(size_t inval) {
  if (inval>(spin_editor->GetMax())) return false;
  spin_editor->SetValue(inval);
  return true;
}
//-------------------------------------
bool jfLabeledUIntEdit::SetValMax(size_t inval) {
  if (inval>ZINTMAX) return false;
  int qxmax = spin_editor->GetMax();
  if (inval>qxmax) spin_editor->SetRange(0,inval);
  spin_editor->SetValue(inval);
  return true;
}
//--------------------------------------
bool jfLabeledUIntEdit::SetBoth(wxString in_label, size_t in_value) {
  SetLabel(in_label);
  return SetValue(in_value);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting data
size_t jfLabeledUIntEdit::GetValue() const {
  return spin_editor->GetValue();
}
//===========================================================================
// constructor
jfFilesizeHintWidget::jfFilesizeHintWidget(wxWindow* parent, wxWindowID id):
      jfLabeledUIntEdit(parent,wxT("File Size Guide (KB)"),200000,
      globalsettings::ficsize_hint,3,id) {
  // the minimum possible value will be 1, not 0
  spin_editor->SetRange(1,spin_editor->GetMax());
  SetValue(globalsettings::ficsize_hint);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting data
bool jfFilesizeHintWidget::SetMaximum(const size_t& maxval) {
  if (maxval>ZINTMAX) return false;
  int qx = spin_editor->GetValue();
  spin_editor->SetRange(1,maxval);
  if (qx>maxval) spin_editor->SetValue(maxval);
  return true;
}
//----------------------------------------------------
bool jfFilesizeHintWidget::SetValMax(size_t inval) {
  if (inval>ZINTMAX) return false;
  int qxmax = spin_editor->GetMax();
  if (inval>qxmax) spin_editor->SetRange(1,inval);
  spin_editor->SetValue(inval);
  return true;
}
//===========================================================================
// a directory picker... wrapping a dialog launching button with a text display
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// constructor
jfDirPicker::jfDirPicker(wxWindow* parent, wxWindowID id, bool vertical, wxString indir, const wxString& name):
      wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // constants
  const wxString fname = wxT("jfDirPicker::jfDirPicker");
  // we check the input path here
  if (!wxDir::Exists(indir)) thedirectory = wxStandardPaths::Get().GetDocumentsDir();
  else thedirectory = indir;
  /**/jfXLogMessage(fname,1,thedirectory);
  /**/jfXLogMessage(fname,2,wxStandardPaths::Get().GetDocumentsDir());
  // creating the GUI elements
  dirlabel = new wxStaticText(this,-1,wxT("Output Directory : "));
  dirdisplay = new wxStaticText(this,-1,thedirectory);
  dirpicker = new wxDirPickerCtrl(this,-1,thedirectory,wxT("Select a folder"),wxDefaultPosition,wxDefaultSize,wxDIRP_DIR_MUST_EXIST);
  // finishing the GUI stuff
  SetPath(thedirectory);
  bool atest = ArrangeControls(vertical);
  assert(atest);
  // done
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// path i/o
//------------------------------------
wxString jfDirPicker::GetPath() const {
  return thedirectory;
}
//------------------------------------
bool jfDirPicker::SetPath(wxString inval) {
  const wxString fname = wxT("jfDirPicker::SetPath");
  /**/jfXLogMessage(fname,1);
  if (!wxDir::Exists(inval)) return false;
  /**/jfXLogMessage(fname,2);
  dirpicker->SetPath(inval);
  dirdisplay->SetLabel(inval);
  /**/jfXLogMessage(fname,3);
  thedirectory = inval;
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// other stuff
void jfDirPicker::ChangeLabel(wxString inlabel) {
  dirlabel->SetLabel(inlabel);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event handler
void jfDirPicker::HandleNewDirectory(wxFileDirPickerEvent& inevent) {
  /* For some reason, in FreeBSD I am getting a spurious event with empty
  GetPath() during form resizes. so, check and ignore if false... */
  bool atest = SetPath(dirpicker->GetPath());
  if (atest) inevent.Skip();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods
bool jfDirPicker::ArrangeControls(bool vertical) {
  mainsizer = new wxBoxSizer(wxHORIZONTAL);
  if (vertical) {
    subsizer = new wxBoxSizer(wxVERTICAL);
    subsizer->Add(dirlabel,0,wxALL|wxALIGN_LEFT|wxEXPAND,3);
    subsizer->Add(dirdisplay,0,wxEXPAND|wxALIGN_CENTER_VERTICAL|wxRIGHT|wxBOTTOM|wxLEFT,3);
    mainsizer->Add(subsizer,1,wxEXPAND|wxALIGN_LEFT|wxALL,1);
    mainsizer->Add(dirpicker,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
  }
  else {
    subsizer = NULL;
    mainsizer->Add(dirlabel,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
    mainsizer->Add(dirdisplay,1,wxEXPAND|wxALIGN_CENTER_VERTICAL|wxTOP|wxBOTTOM|wxLEFT,3);
    mainsizer->AddSpacer(6);
    mainsizer->Add(dirpicker);
  }
  SetSizer(mainsizer);
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfDirPicker,wxPanel)
  EVT_DIRPICKER_CHANGED(-1,jfDirPicker::HandleNewDirectory)
END_EVENT_TABLE()
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    // gui elements
    /*
    wxDirPickerCtrl* dirpicker;
    wxStaticText* dirlabel;
    wxStaticText* dirdisplay;
    wxBoxSizer* mainsizer;
    // data
    wxString thedirectory;
}; */
//****************************************************************************
