/*****************************************************************************
Name:        display_index.cpp
Author :     John Q Metro
Purpose :    At the the result display level, we have to to translate indexes in
             result display lists to source category index and offset.
Created:     April 23, 2011
Updated:     September 21, 2014
******************************************************************************/
#ifndef DISPLAY_INDEX_H_INCLUDED
  #include "display_index.h"
#endif // DISPLAY_INDEX_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//-----------------------------------------
//*****************************************************************************
jfDisplayIndexGroup::jfDisplayIndexGroup(bool douse_default, size_t in_num_rcats, header_enum in_hvalue) {
  // some asserts and checks
  assert(in_num_rcats<33);
  // assigning values
  use_default = douse_default;
  num_rcats = in_num_rcats;
  hvalue = in_hvalue;
  // initializing values
  cindex = -1;
  size_t rloop;
  for (rloop=0;rloop<32;rloop++) {
    if (rloop<num_rcats) pointers[rloop] = new jfCatPVector();
    else pointers[rloop] = NULL;
    newresc[rloop] = 0;
  }
  if (use_default) pointers[32] = new jfCatPVector();
  else pointers[32] = NULL;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// public methods
//-----------------------------------------
bool jfDisplayIndexGroup::AppendResults(jfResultUnitVector* newresult, size_t in_stotal) {
  wxMutexLocker mlocker(mlock);
  return AppendPageResults(newresult,in_stotal);
}
//-----------------------------------------
// looking up the HTML data
wxString jfDisplayIndexGroup::GetHTML(size_t rcat_index, size_t nindex) const {
  // constants
  const wxString fname = wxT("jfDisplayIndexGroup::GetHTML");
  // variables
  wxMutexLocker mlocker(mlock);
  jfIndexPack* cpointer;
  wxInt32 fpoint;
  size_t newindex;
  wxString revalue;
  // check if the result category is valid
  jfXAssertLog((rcat_index<num_rcats)||(rcat_index==32),fname,wxT("Output category index is invalid!"));
  jfXAssertLog((pointers[rcat_index]!=NULL),fname,wxT("No pointers exist for the specified output category!"));
  // check if nindex is okay
  jfXAssertLog(nindex<(pointers[rcat_index]->size()),fname,wxT("The specified item is past the end of the item list!"));
  // we get the category
  cpointer = (*pointers[rcat_index])[nindex];
  // getting the index
  fpoint = (cpointer->startpoints)[rcat_index];
  assert(fpoint>=0);
  assert(nindex>=fpoint);
  newindex = nindex - fpoint;
  // now that we have the newindex, what we get depends on whether we have category headers
  if ((hvalue==he_NONEMPTY) || (hvalue==he_ALWAYS)) {
    if (newindex==0) revalue = cpointer->indexdata->GetCatHTML(rcat_index);
    else revalue = cpointer->indexdata->GetHTML(rcat_index,newindex-1);
  }
  else revalue = cpointer->indexdata->GetHTML(rcat_index,newindex);
  // done
  return revalue;
}
//----------------------------------------------------------------------
void jfDisplayIndexGroup::SetFicMarkHTML(size_t rcat_index, size_t nindex, jf_FicMark fmval) {
  // constants
  const wxString fname = wxT("jfDisplayIndexGroup::SetFicMarkHTML");
  // variables
  wxMutexLocker mlocker(mlock);
  // variables
  jfIndexPack* cpointer;
  wxInt32 fpoint;
  size_t newindex;
  wxString newcolor;
  // we get the category

  cpointer = (*pointers[rcat_index])[nindex];
  // getting the index
  fpoint = (cpointer->startpoints)[rcat_index];
  assert(fpoint>=0);
  assert(nindex>=fpoint);
  newindex = nindex - fpoint;
  // getting the color string
  newcolor = fmcolors[(size_t)fmval];
  // now that we have the newindex, what we get depends on whether we have category headers
  if ((hvalue==he_NONEMPTY) || (hvalue==he_ALWAYS)) {
    /**/jfXLogMessage(fname,7);
    if (newindex!=0) cpointer->indexdata->AlterBgColour(rcat_index,newindex-1,newcolor);
  }
  else {
    /**/jfXLogMessage(fname,8);
    cpointer->indexdata->AlterBgColour(rcat_index,newindex,newcolor);
  }
}
//----------------------------------------------------------------------
// returns the url of the specified item
wxString jfDisplayIndexGroup::GetUrl(size_t rcat_index, size_t nindex) const {
  // constants
  const wxString fname = wxT("jfDisplayIndexGroup::GetUrl");
  // variables
  wxMutexLocker mlocker(mlock);
  jfIndexPack* cpointer;
  size_t fpoint, newindex;
  wxString revalue;
  // check if the result category is valid
  /**/jfXLogMessageS(fname,1,rcat_index);
  assert((rcat_index<num_rcats)||(rcat_index==32));
  assert(pointers[rcat_index]!=NULL);
  /**/jfXLogMessageS(fname,2,nindex);
  // check if nindex is okay
  assert(nindex<(pointers[rcat_index]->size()));
  // we get the category
  cpointer = (*pointers[rcat_index])[nindex];
  // getting the index
  fpoint = (cpointer->startpoints)[rcat_index];
  /**/jfXLogMessageS(fname,3,fpoint);
  assert(fpoint>=0);
  assert(nindex>=fpoint);
  newindex = nindex - fpoint;
  /**/jfXLogMessageS(fname,4,newindex);
  // now that we have the newindex, what we get depends on whether we have category headers
  if ((hvalue==he_NONEMPTY) || (hvalue==he_ALWAYS)) {
    /**/jfXLogMessage(fname,5);
    if (newindex==0) revalue = wxEmptyString;
    else revalue = cpointer->indexdata->GetURL(rcat_index,newindex-1);
  }
  else revalue = cpointer->indexdata->GetURL(rcat_index,newindex);
  // done
  return revalue;

}
//-------------------------------------------------------------
void jfDisplayIndexGroup::ItemsProcessed(size_t in_stotal) {
  // checking the inputs
  assert(in_stotal>0);
  assert(categories.size()>0);
  // we start
  bool atest = categories[cindex]->indexdata->NoResultsPage(in_stotal);
  assert(atest);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
size_t jfDisplayIndexGroup::GetNewcount(size_t rindex) const {
  assert(rindex<33);
  return newresc[rindex];
}
//-------------------------------------------------------------
wxInt32 jfDisplayIndexGroup::GetCatIndex() const {
  return cindex;
}
//-------------------------------------------------------------
size_t jfDisplayIndexGroup::GetCatcount() const {
  return categories.size();
}
//-------------------------------------------------------------
// intended to allow getting the source collection index and item index from the result collection and list index
void jfDisplayIndexGroup::ReverseLookup(size_t inres_index, size_t inlist_index, size_t& out_srccoll, wxInt32& out_srcindex) const {
  // variables
    wxMutexLocker mlocker(mlock);
    size_t fpoint, newindex;
    jfIndexPack* cpointer;
  // check if the result category is valid
  assert((inres_index<num_rcats)||(inres_index==32));
  assert(pointers[inres_index]!=NULL);
  // check if inlist_index is okay
  assert(inlist_index<(pointers[inres_index]->size()));
  // we get the category
  cpointer = (*pointers[inres_index])[inlist_index];
  // getting the index
  fpoint = (cpointer->startpoints)[inres_index];
  assert(fpoint>=0);
  assert(inlist_index>=fpoint);
  newindex = inlist_index - fpoint;
  // now that we have the newindex, what we get depends on whether we have category headers
  if ((hvalue==he_NONEMPTY) || (hvalue==he_ALWAYS)) {
    if (newindex==0) out_srcindex = -1;
    else out_srcindex = cpointer->indexdata->GetProxy(inres_index,newindex-1);
  }
  else out_srcindex = cpointer->indexdata->GetProxy(inres_index,newindex);
  // the index is done, but we must get an index for the source category
  out_srccoll = cpointer->catdex;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// clearing out stuff
void jfDisplayIndexGroup::ClearContents() {
  // initializing values
  cindex = -1;
  size_t rloop;
  for (rloop=0;rloop<33;rloop++) {
    if (pointers[rloop]!=NULL) {
      delete pointers[rloop];
      pointers[rloop] = NULL;
    }
  }
  // clearing the categories
  for (rloop=0;rloop<categories.size();rloop++) {
    delete categories[rloop]->indexdata;
    delete categories[rloop];
    categories[rloop] = NULL;
  }
  categories.clear();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special methods
//-----------------------------------
// adds a set of pages to the result vector
bool jfDisplayIndexGroup::AppendPageResults(jfResultUnitVector* newresult, size_t in_stotal) {
  // constants
  const wxString fname = wxT("$$$ jfDisplayIndexGroup::AppendPageResults");
  // variables
  jfIndexPack* cpointer;
  size_t rloop;
  // checking the inputs
  assert(in_stotal>0);
  assert(newresult!=NULL);
  // the category must always be there
  assert(categories.size()>0);
  // we pass on the inputs to the category organizer
  cpointer = categories[categories.size()-1];
  cpointer->indexdata->AppendResult(newresult,in_stotal);
  // we then add pointers to the indexes in accordance with the results
  for (rloop=0;rloop<num_rcats;rloop++) AddPointers(rloop,cpointer);
  // We add pointers to the indexes for the default category
  if (use_default) AddPointers(32,cpointer);
  // done
  return true;
}
//-----------------------------------------------------------------------------
bool jfDisplayIndexGroup::AppendCatCore(jfCategoryResultIndexes* inval) {
  // variables and asserts
  assert(inval!=NULL);
  jfIndexPack* newval, *cpointer;
  size_t rloop;
  // we start
  newval = new jfIndexPack();
  newval->indexdata = inval;
  // adding
  categories.push_back(newval);
  cpointer = newval;
  cindex++;
  cpointer->catdex = cindex;
  // in certain cases, the category headers are added right away
  if (hvalue==he_ALWAYS) {
    for (rloop=0;rloop<num_rcats;rloop++) {
      newresc[rloop] = 1;
      (cpointer->startpoints)[rloop] = pointers[rloop]->size();
      pointers[rloop]->push_back(cpointer);
    }
    // We add pointers to the indexes for the default category
    if (use_default) {
      newresc[32] = 1;
      (cpointer->startpoints)[32] = pointers[32]->size();
      pointers[32]->push_back(cpointer);
    }
  }
  // done
  return true;
}
//---------------------------------------------------------------------------------
bool jfDisplayIndexGroup::AddPointers(size_t rloop, jfIndexPack* cpointer) {
  // constants
  const wxString fname = wxT("jfDisplayIndexGroup::AddPointers");
  // variables
  size_t rres;
  // assertions and checks
  assert(cpointer!=NULL);
  assert(rloop<33);
  // we start
  rres = cpointer->indexdata->CatCurrCount(rloop);
  newresc[rloop] = 0;
  // here, we have some results...
  if (rres!=0) {
    // if this is the first one, we have to set the index ...
    if ((cpointer->startpoints)[rloop]==-1) {
      (cpointer->startpoints)[rloop] = pointers[rloop]->size();
      // next up, in some cases we add a pointer for the category header link
      if (hvalue==he_NONEMPTY) {
        pointers[rloop]->push_back(cpointer);
        newresc[rloop] = 1;
      }
      else assert (hvalue!=he_ALWAYS);
    }
    // after handling the header, we add rres copies of cpointer to the appropriate index
    pointers[rloop]->insert(pointers[rloop]->end(),rres,cpointer);
    newresc[rloop] += rres;
    cpointer->anything = true;
  }
  // done
  return true;
}
//============================================================================
// constructor
jfDisplayIndex_Basic::jfDisplayIndex_Basic(bool douse_default, size_t in_num_rcats,
        header_enum in_hvalue):jfDisplayIndexGroup(douse_default, in_num_rcats, in_hvalue) {
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// main methods
bool jfDisplayIndex_Basic::AppendCategory(wxString catname, wxString cathtml) {
  jfCategoryResultIndexes* newcat;
  wxMutexLocker mlocker(mlock);
  newcat = new jfCategoryResultIndexes(catname,cathtml);
  return AppendCatCore(newcat);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructor
jfDisplayIndex_Basic::~jfDisplayIndex_Basic() {
  ClearContents();
}
//*****************************************************************************
