/*****************************************************************************
Name:        ffn_index.cpp
Author :     John Q Metro
Purpose :    A Fanfiction.Net customized version of jfDisplayIndexGroup. This
             add update capability, which complicates things a great deal.
Created:     April 24, 2011
Updated:     November 30, 2013
******************************************************************************/
#ifndef FFN_INDEX_H_INCLUDED
  #include "ffn_index.h"
#endif // FFN_INDEX_H_INCLUDED
//--------------------------------------------------
#ifndef DISPLAY_INDEX_H_INCLUDED
  #include "display_index.h"
#endif // DISPLAY_INDEX_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//*****************************************************************************
// constructor
jfDisplayIndex_FFN::jfDisplayIndex_FFN(bool douse_default, size_t in_num_rcats):
    jfDisplayIndexGroup(douse_default,in_num_rcats, he_NONEMPTY) {
  insert_mode = false;
  uindex = -1;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// main methods
//---------------------------
bool jfDisplayIndex_FFN::AppendCategory(const jfFFNSCategory* incat, wxString incathb) {
  // variables and checks
  jfCatFFNResultIndexes* newcat;
  wxMutexLocker mlocker(mlock);
  assert(incat!=NULL);
  // we do
  newcat = new jfCatFFNResultIndexes(incat,incathb);
  return AppendCatCore(newcat);
}
//----------------------------
bool jfDisplayIndex_FFN::AppendEmptCategories(jfBulk_EmptyCatInfo* inbcat) {
  // variables
  wxMutexLocker mlocker(mlock);
  jfCatFFNResultIndexes* newcat;
  const jfFFNSCategory* ccat;
  wxString chead;
  size_t tcount;
  size_t mindex,mcat;
  bool xresult;
  // check
  assert(inbcat!=NULL);
  // preparing for the loop
  mcat = inbcat->NumCats();
  // the loop
  for (mindex=0;mindex<mcat;mindex++) {
    ccat = inbcat->GetLink(mindex);
    chead = inbcat->GetHeader(mindex);
    tcount = inbcat->GetTCount(mindex);
    newcat = new jfCatFFNResultIndexes(ccat,chead);
    xresult = AppendCatCore(newcat);
    if (!xresult) return false;
    categories[cindex]->indexdata->NoResultsPage(tcount);
  }
  // done
  return true;
}
//----------------------------------------------------------------------------
bool jfDisplayIndex_FFN::InsertCategory(const jfFFNSCategory* incat, wxString incathb) {
  // constants
  const wxString fname = wxT("------>> jfDisplayIndex_FFN::InsertCategory");
  // variables and checks
  jfCatFFNResultIndexes* newcat;
  wxMutexLocker mlocker(mlock);
  assert(incat!=NULL);
  assert(insert_mode);
  /**/jfXLogMessageS(fname,1,categories.size());
  /**/jfXLogMessageS(fname,2,cindex);
  // we increment and check
  if ((cindex+1)==categories.size()) {
    /**/jfXLogMessage(fname,3);
    // we append in this case
    newcat = new jfCatFFNResultIndexes(incat,incathb);
    newcat->SetInsertMode(true);
    AppendCatCore(newcat);
  }
  else {
    cindex++;

    if (!CategorySame(cindex,incat)) {
      // inserting a category if it does not match
      /**/jfXLogMessage(fname,4);
      newcat = new jfCatFFNResultIndexes(incat,incathb);
      newcat->SetInsertMode(true);
      /**/jfXLogMessage(fname,5,incat->GetName());
      InsertCatCore(newcat);
      if (uindex>=cindex) uindex++;
    }
    /**/jfXLogMessageS(fname,6,categories.size());
    // if the category does match, it is already there for use.
  }
  // done
  return true;
}
//-------------------------------------------------------------------------
bool jfDisplayIndex_FFN::NextUpdateCat(const jfFFNSCategory* incat) {
  // constants
  const wxString fname = wxT("q### jfDisplayIndex_FFN::NextUpdateCat");
  // variables and checks
  wxMutexLocker mlocker(mlock);
  bool found;
  wxInt32 csize;
  // checks
  /**/jfXAssertLog(incat!=NULL,fname,wxT("The input category should not be NULL!"));
  // we start...
  found = false;
  /**/jfXLogMessageI(fname,1,uindex);
  csize = categories.size();
  /**/jfXLogMessageS(fname,2,csize);
  while (uindex<csize) {
    uindex++;
    /**/jfXLogMessageI(fname,3,uindex);
    if (CategorySame(uindex,incat)) {
      found = true;
      break;
    }
  }
  /**/jfXLogMessage(fname,4);
  // at the end
  return found;
}
//---------------------------------------------------------------------------
bool jfDisplayIndex_FFN::InsertResults(jfResultUnitVector* newresult, size_t in_stotal) {
  // some basic checks
  assert(newresult!=NULL);
  assert(cindex!=-1);
  assert(in_stotal>0);
  assert(insert_mode);
  // variables
  wxMutexLocker mlocker(mlock);
  // doing
  return InsertPageResults(newresult,in_stotal);
}
//----------------------------------------------------------------------------
bool jfDisplayIndex_FFN::UpdateFic(jfResultUnit* inval) {
  const wxString fname = wxT("*** jfDisplayIndex_FFN::UpdateFic");
  bool qval;
  // checks
  /**/jfXAssertLog(uindex>=0,fname,wxT("The update index cannot be negative"));
  /**/jfXAssertLog(inval!=NULL,fname,wxT("The inputval must be non NULL"));
  // going ahead
  wxMutexLocker mlocker(mlock);
  qval = TypCpointer(uindex)->UpdateFic(inval);
  return qval;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// extra methods
//----------------------------
size_t jfDisplayIndex_FFN::SetInsertMode(bool inval) {
  wxMutexLocker mlocker(mlock);
  size_t qloop;
  jfCatFFNResultIndexes* xcpointer;
  if (inval!=insert_mode) {
    // setting the main mode
    insert_mode = inval;
    // setting the index
    if (inval) cindex = -1;
    else cindex = wxInt32(categories.size())-1;
    // setting the mode for each category
    for (qloop=0;qloop<categories.size();qloop++) {
      xcpointer = TypCpointer(qloop);
      xcpointer->SetInsertMode(inval);
    }
  }
  return cindex;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// destructor
jfDisplayIndex_FFN::~jfDisplayIndex_FFN() {
  ClearContents();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods
//--------------------------------------------
// checks if the category at tindex is incat
bool jfDisplayIndex_FFN::CategorySame(size_t tindex, const jfFFNSCategory* incat) {
  // constants
  const wxString fname = wxT("jfDisplayIndex_FFN::CategorySame");
  // we do the usual checks
  /**/jfXAssertLog(incat!=NULL,fname,wxT("The input category is NULL!"));
  /**/jfXAssertLog(tindex<categories.size(),fname,wxT("The input category is not less than the size"));
  // variables
  jfCatFFNResultIndexes* cxpointer;
  // we start by getting the cxpointer
    /**/jfXLogMessage(fname,1);
  cxpointer = TypCpointer(tindex);
  /**/jfXLogMessage(fname,2);
  // done
  return cxpointer->SameCategory(incat);
}
//---------------------------------------------------------------------------
// just to shorten lines
jfCatFFNResultIndexes* jfDisplayIndex_FFN::TypCpointer(wxInt32 xindex) {
  assert(xindex<categories.size());
  assert(xindex>=0);
  return dynamic_cast<jfCatFFNResultIndexes*>(categories[xindex]->indexdata);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special methods
//---------------------------------------------------------
// inserting items in current categories
bool jfDisplayIndex_FFN::InsertPageResults(jfResultUnitVector* newresult, size_t in_stotal) {
  // constnsts
  const wxString fname = wxT("((#( jfDisplayIndex_FFN::InsertPageResults");
  // the usual checks
  assert(newresult!=NULL);
  assert(in_stotal>0);
  assert(categories.size()>0);
  // variables
  jfCatFFNResultIndexes* cxpointer;
  size_t rloop,cres, ipoint;
  wxInt32 bloop;
  // we start with getting the category data and inserting there...
  cxpointer = TypCpointer(cindex);
  cxpointer->InsertResult(newresult,in_stotal);
  // next we handle the results
  for (rloop=0;rloop<33;rloop++) {
    if ((rloop<32) && (rloop>=num_rcats)) continue;
    cres = cxpointer->CatCurrCount(rloop);
    // we only need to do anything if we have results
    if (cres==0) {
      newresc[rloop] = 0;
      continue;
    }
    // checks if there are no previous results
    if (-1==((categories[cindex]->startpoints)[rloop])) {
      // increment cres because we will have to add an index for the category cheader
      cres++;
      ipoint = 0;
      // unless this is the first category, we also need an insertion point
      if (cindex!=0) {
        for (bloop=cindex-1;bloop>=0;bloop--) {
          if (((categories[bloop]->startpoints)[rloop])!=-1) {
            ipoint = (categories[bloop]->startpoints)[rloop];
            ipoint += (categories[bloop]->indexdata->CatResCount(rloop) + 1);
            break;
          }
        }
        // after the loop
      }
      // assigning the start index
      (categories[cindex]->startpoints)[rloop] = ipoint;
    }
    // if there are previous entries, the insertion points is looked up
    else {
      ipoint = (categories[cindex]->startpoints)[rloop];
    }
    // we now go ahead and insert
    pointers[rloop]->insert(pointers[rloop]->begin()+ipoint,cres,categories[cindex]);
    // the next thing to do is increment the offsets of later categories
    for (bloop = cindex +1;bloop<categories.size();bloop++) {
      if (((categories[bloop]->startpoints)[rloop])!=-1) {
        ((categories[bloop]->startpoints)[rloop])+=cres;
      }
    }
    // setting the new result count
    newresc[rloop] = cres;
  }
  // done
  categories[cindex]->anything = true;
  return true;
}
//---------------------------------------------------------
// we insert a new category at cindex
bool jfDisplayIndex_FFN::InsertCatCore(jfCatFFNResultIndexes* inval) {
  // the usual checks
  assert(inval!=NULL);
  assert(cindex<categories.size());
  // variables
  jfIndexPack* newval;
  size_t lindex,lindex_max;
  // we start
  newval = new jfIndexPack();
  newval->indexdata = inval;
  // nothing really complicated with insertion (until items are added)
  categories.insert(categories.begin()+cindex,newval);
  // except that we must update the category index values...
  lindex_max = categories.size();
  for (lindex = cindex + 1; lindex < lindex_max; lindex++) (categories[lindex]->catdex)++;
  return true;
}
//*****************************************************************************
