/*****************************************************************************
Name:        organizer.cpp
Author :     John Q Metro
Purpose :    Intended to replace jfDisplayResultOrganizer
             A class that holds html result data for display, and provides a set
             of indexes for the result category to get the html result
Created:     April 16, 2011
Updated:     September 21, 2014
******************************************************************************/
#ifndef ORGANIZER_H_INCLUDED
  #include "organizer.h"
#endif // ORGANIZER_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef BASEOBJ_H_INCLUDED
    #include "../../core/objs/baseobj.h"
#endif // BASEOBJ_H_INCLUDED
#ifndef JFFFNSCATCORE
  #include "../../fanficnet/categories/ffn_catcore.h"
#endif // JFFFNSCATCORE
//------------------------------------------
//*****************************************************************************
jfCategoryResultIndexes::jfCategoryResultIndexes(wxString inname, wxString incathb) {
  // variables
  size_t loopc;
  // initializing
  maindata = NULL;
  /* for the arrays that hold result category data, there is no
  difference to start with between empty and noexistant */
  for (loopc=0;loopc<33;loopc++) {
    num_items[loopc] = 0;
    curr_num_items[loopc] = 0;
    indexes[loopc] = NULL;
    catheaders[loopc] = NULL;
  }
  searched_total = 0;
  // category name and headers
  catname = inname;
  cathtml = incathb;
  MakeEmptyCatHeaders();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// returns the html for result category res_a, index res_idx
wxString jfCategoryResultIndexes::GetHTML(size_t res_a, size_t res_idx) const {
  // constants
  const wxString fname = wxT("~~~ jfCategoryResultIndexes::GetHTML");
  // variables
  size_t proxy;
  wxString res,qbuf;
  wxMutexLocker newlock(cguard);
  // first, the usual checks
  /**/jfXAssertLog(IndexOk(res_a,res_idx),fname,wxT("Index out of range"));
  // we get the proxy
  proxy = CalcProxy(res_a,res_idx);
  // and return the result
  qbuf = wxT("<font size=-2>[[") + SInt2Str(proxy);
  qbuf += wxT("]]</font> ");
  res = qbuf + ((*maindata)[proxy])->GetHTML();
  return res;
}
//---------------------------------------------------------------------------
// returns the proxy
size_t jfCategoryResultIndexes::GetProxy(size_t res_a, size_t res_idx) const {
  // variables and constants
  wxMutexLocker newlock(cguard);
  const wxString fname = wxT("jfCategoryResultIndexes::GetProxy");
  // stuff
  /**/jfXAssertLog(IndexOk(res_a,res_idx),fname,wxT("Index out of range"));
  return CalcProxy(res_a,res_idx);
}
//---------------------------------------------------------------
wxString jfCategoryResultIndexes::GetURL(size_t res_a, size_t res_idx) const {
	wxString outurl;
	size_t proxy;
	cguard.Lock();
	proxy = (*indexes[res_a])[res_idx];
	outurl = ((*maindata)[proxy])->GetUrl();
	cguard.Unlock();
	return outurl;
}
//---------------------------------------------------------------------------
wxString jfCategoryResultIndexes::GetCatHTML(size_t res_a) const {
  wxMutexLocker mlock(cguard);
  return *catheaders[res_a];
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
size_t jfCategoryResultIndexes::CatResCount(size_t res_a) const {
  wxMutexLocker mlock(cguard);
  return num_items[res_a];
}
//----------------------------------------------------------------
size_t jfCategoryResultIndexes::CatCurrCount(size_t res_a) const {
  wxMutexLocker mlock(cguard);
  return curr_num_items[res_a];
}
//-------------------------------------------------------------
wxString jfCategoryResultIndexes::GetCatName() const {
  return catname;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// methods to append results
bool jfCategoryResultIndexes::AppendResult(jfResultUnitVector* newresult, size_t in_stotal) {
  // some basic checks
  assert(in_stotal!=0);
  assert(newresult!=NULL);
  assert((newresult->size())!=0);
  // variables
  wxMutexLocker mlock(cguard);
  size_t spos = 0;
  // we start
  searched_total += in_stotal;
  // if the maindata is null, we just use the tempfile
  if (maindata==NULL) maindata = newresult;
  // otheriwse, we concatenate onto the mainfile
  else {
    spos = AppendNewRes(newresult);
    delete newresult;
  }
  // the sorting loop is in a subfunction
  IndexingLoopAppend(spos);
  RebuildCatHeaders();
  return true;
}
//--------------------------------------------------------------------------
bool jfCategoryResultIndexes::NoResultsPage(size_t in_stotal) {
  // local variables
  wxMutexLocker mlock(cguard);
  // we get the data
  searched_total += in_stotal;
  RebuildCatHeaders();
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// special method
bool jfCategoryResultIndexes::AlterBgColour(size_t res_a, size_t res_idx, wxString newcol) {
  // constants
  const wxString fname = wxT("jfCategoryResultIndexes::AlterBgColour");
  // variables
  size_t proxy;
  size_t xpos;
  wxString res;
  wxMutexLocker newlock(cguard);
  // first, the usual checks
  /**/jfXAssertLog(IndexOk(res_a,res_idx),fname,wxT("Index out of range"));
  // we get the proxy
  proxy = CalcProxy(res_a,res_idx);
  // and return the result
  res = ((*maindata)[proxy])->GetHTML();
  xpos = res.find(wxT("bgcolor=\"#"));
  assert(xpos!=wxString::npos);
  xpos += 10;
  res.replace(xpos,6,newcol);
  ((*maindata)[proxy])->SetHTML(res);
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// extra methods for inserting data
// here, in non-insert mode, we append the results
size_t jfCategoryResultIndexes::AppendNewRes(jfResultUnitVector* in_tempfile) {
  // variables
  size_t startdex,tsize,loopc;
  // checking values
  startdex = maindata->size();
  tsize = in_tempfile->size();
  // resizing the main data
  maindata->resize(tsize+startdex,NULL);
  //copying over
  for (loopc=0;loopc<tsize;loopc++) {
      (*maindata)[startdex+loopc] = (*in_tempfile)[loopc];
  }
  // done
  return startdex;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the destructor
jfCategoryResultIndexes::~jfCategoryResultIndexes() {
  // we remove the indexes...
  ClearIndexes();
  // the maindata can be removed as well
  if (maindata!=NULL) ClearMaindata(maindata);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// some helper methods
bool jfCategoryResultIndexes::IndexOk(size_t res_a, size_t res_idx) const {
  return (res_idx<num_items[res_a]);
}
//----------------------------------------------------------------------
size_t jfCategoryResultIndexes::CalcProxy(size_t res_a, size_t res_idx) const {
  return (*indexes[res_a])[res_idx];
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// empties out maindata
void jfCategoryResultIndexes::ClearMaindata(jfResultUnitVector* xmaindata) {
  size_t loopc,loopm;
  if (xmaindata!=NULL) {
    loopm = xmaindata->size();
    if (loopm!=0) {
      for (loopc=0;loopc<loopm;loopc++) {
        delete (*xmaindata)[loopc];
        (*xmaindata)[loopc] = NULL;
      }
      xmaindata->clear();
    }
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* the category headers contain a a brief summary of the results, and thus
have to be rebuilt after each update */
void jfCategoryResultIndexes::RebuildCatHeaders() {
  size_t loopc;
  wxString tempres, pstring, qstring;
  // we have some results
  qstring = SInt2Str(searched_total) + wxT(" (");
  // handling the regualar category
  for (loopc=0;loopc<33;loopc++) {
    // making the percentage
    pstring = MakePercString(loopc);
    // building the summary
    tempres = SInt2Str(num_items[loopc]) + wxT(" items out of ");
    tempres += qstring + pstring + wxT(") found");
    // updating catheaders
    (*catheaders[loopc]) = cathtml + tempres;
  }
}
//----------------------------------------------------------------------------
wxString jfCategoryResultIndexes::MakePercString(size_t ni_idex) const {
  // variables
  double frac_fics;
  wxString pstring;
  // some checks
  assert(ni_idex<33);
  // special case first
  if (searched_total==0) pstring = wxT("0.00%");
  else {
    frac_fics = double(num_items[ni_idex])/double(searched_total);
    pstring = ToPercent(frac_fics,2);
  }
  // done
  return pstring;
}
//---------------------------------------------------------------------------
/* category header building when there are no items even searched */
void jfCategoryResultIndexes::MakeEmptyCatHeaders() {
  const wxString tempres = wxT("0 items out of 0 (0.00%) found");
  size_t loopc;
  // handling the regualar category
  for (loopc=0;loopc<33;loopc++) {
    // updating catheaders
    if (catheaders[loopc]==NULL) catheaders[loopc] = new wxString(cathtml+tempres);
    else (*catheaders[loopc]) = cathtml + tempres;
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* handles the job of updating or building the indexes that direct
HtmlListBox accesses to the appropriate item */
void jfCategoryResultIndexes::IndexingLoopAppend(size_t istartdex) {
  size_t maxdex, loopc, loopd;
  jfResultUnit* currx;
  // starting
  // we start prepping the indexes
  ZeroCNumItems();
  maxdex = maindata->size();
  // the sorting loop
  for (loopc=istartdex;loopc<maxdex;loopc++) {
    currx = (*maindata)[loopc];
    // for each result category, we check and see if the item belongs
    for (loopd=0;loopd<32;loopd++) {
      if (currx->Marked(loopd)) {
        curr_num_items[loopd]++;
        num_items[loopd]++;
        if (indexes[loopd] == NULL) indexes[loopd] = new jfIndexVector();
        indexes[loopd]->push_back(loopc);
      }
    }
    // we also check the default leftover catgeory
    if (!(currx->HasMark())) {
      curr_num_items[32]++;
      num_items[32]++;
      if (indexes[32] == NULL) indexes[32] = new jfIndexVector();
      indexes[32]->push_back(loopc);
    }
  }
}
//---------------------------------------------------------------------------
void jfCategoryResultIndexes::ClearIndexes() {
  size_t loopc;
  for (loopc=0;loopc<33;loopc++) {
    if (indexes[loopc]!=NULL) delete indexes[loopc];
    indexes[loopc] = NULL;
  }
}
//------------------------------------------------------------------------------
void jfCategoryResultIndexes::ZeroCNumItems() {
  size_t loopc;
  for (loopc=0;loopc<33;loopc++) curr_num_items[loopc]=0;
}
//============================================================================
// the constructor
jfCatFFNResultIndexes::jfCatFFNResultIndexes(const jfFFNSCategory* incat, wxString
    incathb):jfCategoryResultIndexes(incat->GetName(),incathb) {
  // initializing
  uindex = 0;
  insert_mode = false;
  insert_point = 0;
  catlink = incat;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfCatFFNResultIndexes::InsertResult(jfResultUnitVector* newresult, size_t in_stotal) {
  // constants
  const wxString fname = wxT("jfCatFFNResultIndexes::InsertResult");
  // a basic check
  if (!insert_mode) {
    /**/jfXLogMessageS(fname,1,in_stotal);
    /**/jfXAssertLog(newresult!=NULL,fname,wxT("newresult is empty"));
    /**/jfXLogMessageS(fname,2,newresult->size());
    /**/jfXLogMessage(fname,3,(*newresult)[0]->GetUrl());
    /**/jfXLogMessage(fname,4,(*newresult)[0]->GetName());
    assert(insert_mode);
  }
  // variables
  size_t fcount = 0;
  size_t qlen;
  /**/jfXLogMessage(fname,5);
  // we start
  if (maindata!=NULL) fcount = maindata->size();
  /**/jfXLogMessageS(fname,6,fcount);
  // in some cases, inserting is the same as appending
  if (fcount==insert_point) return AppendResult(newresult,in_stotal);
  // otherwise...
  else {
    /**/jfXLogMessage(fname,7);
    // some basic checks
    assert(in_stotal!=0);
    assert(newresult!=NULL);
    qlen = newresult->size();
    assert(qlen!=0);
    // variables
    /**/jfXLogMessage(fname,8);
    wxMutexLocker mlock(cguard);
    size_t spos = 0;
    // doing the operations
    spos = InsertNewRes(newresult);
    IndexingLoopInsert(spos,qlen);
    /**/jfXLogMessage(fname,9);
    // done
    return true;
  }
}
//-------------------------------------------------------------------
bool jfCatFFNResultIndexes::UpdateFic(jfResultUnit* inval) {
  // constants
  const wxString fname = wxT("zzz jfCatFFNResultIndexes::UpdateFic");
  // local variables
  wxMutexLocker Updateg(cguard);
  size_t ficindex,mmax;
  jfResultUnit* temp;
  bool rval;
  // retrieving the data
  /**/jfXLogMessage(fname,1);
  assert(inval!=NULL);
  ficindex = inval->GetID();
  mmax = maindata->size();
  /**/jfXLogMessageS(fname,2,ficindex);
  // checks
  assert(maindata!=NULL);
  assert(uindex<mmax);
  /**/jfXLogMessageS(fname,3,uindex);
  /* we try to locate the fic. note that the updated fic must
  be in the list, and we assume must be found at or past uindex */
  while (uindex<mmax) {
    rval = (*maindata)[uindex]->SameID(inval->GetID());
    if (rval) break;
    uindex++;
  }
  /**/jfXLogMessageS(fname,5,uindex);
  // after the loop
  if (rval) {
    /**/jfXLogMessage(fname,6,inval->GetHTML());
    // we swap ne and old, then get rid of old
    temp = (*maindata)[uindex];
    (*maindata)[uindex] = inval;
    delete temp;
    /**/jfXLogMessage(fname,7);
    uindex++;
    // no rebuilding indexes
  }
  else assert(false);
  /**/jfXLogMessage(fname,8);
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// other
void jfCatFFNResultIndexes::SetInsertMode(bool mode) {
  wxMutexLocker nval(cguard);
  insert_mode = mode;
}
//---------------------------------------------------------------
bool jfCatFFNResultIndexes::SameCategory(const jfFFNSCategory* incat) const {
  const wxString fname=wxT("jfCatFFNResultIndexes::SameCategory");
  /**/jfXLogMessage(fname,1,catlink->GetName());
  /**/jfXLogMessage(fname,2,incat->GetName());
  return (catlink==incat);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++
jfCatFFNResultIndexes::~jfCatFFNResultIndexes() {
  // we remove the indexes...
  ClearIndexes();
  // the maindata can be removed as well
  if (maindata!=NULL) ClearMaindata(maindata);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
size_t jfCatFFNResultIndexes::InsertNewRes(jfResultUnitVector* in_tempfile) {
  // variables
  size_t startdex,tsize,loopc;
  size_t oip;
  jfResultUnitVector* middle;
  jfResultUnitVector* backup;
  // checking values
  startdex = maindata->size();
  tsize = in_tempfile->size();
  // we create a temporary intermediate vector
  middle = new jfResultUnitVector();
  middle->resize(tsize+startdex,NULL);
  // the first loop adds stuff between 0 and insert point
  for (loopc=0;loopc<insert_point;loopc++) {
    (*middle)[loopc] = (*maindata)[loopc];
  }
  // next, we add the temporary files stuff
  for (loopc=0;loopc<tsize;loopc++) {
    (*middle)[insert_point+loopc] = (*in_tempfile)[loopc];
  }
  // we add the rest of the main data
  for (loopc=insert_point;loopc<startdex;loopc++) {
    (*middle)[loopc+tsize] = (*maindata)[loopc];
  }
  // now that the intermediate vector is built, we swap with maindat
  backup = maindata;
  maindata = middle;
  // getting rid of the old maindata and setting the indexes
  delete backup;
  oip = insert_point;
  insert_point+=tsize;
  uindex+=tsize;
  // done
  return oip;
}

//---------------------------------------------------------------------------
void jfCatFFNResultIndexes::IndexingLoopInsert(size_t start_ins, size_t ins_len) {
  // variables
  size_t maxdex, loopc, loopd;
  jfResultUnit* currx;
  // clearing out some old data
  ZeroCNumItems();
  maxdex = maindata->size();
  ClearIndexes();
  // we now loop over the entire set of results
  for (loopc=0;loopc<maxdex;loopc++) {
    currx = (*maindata)[loopc];
    // for each result category, we check and see if the item belongs
    for (loopd=0;loopd<32;loopd++) {
      if (currx->Marked(loopd)) {
        if (indexes[loopd] == NULL) indexes[loopd] = new jfIndexVector();
        indexes[loopd]->push_back(loopc);
        if ((loopc>=start_ins) && (loopc<(start_ins+ins_len))) {
          curr_num_items[loopd]++;
          num_items[loopd]++;
        }
      }
    }
    // defaults
    if (!(currx->HasMark())) {
      if (indexes[32] == NULL) indexes[32] = new jfIndexVector();
      indexes[32]->push_back(loopc);
      if ((loopc>=start_ins) && (loopc<(start_ins+ins_len))) {
        curr_num_items[32]++;
        num_items[32]++;
      }
    }
    // next loop
  }
  // done
}
//===========================================================================
// initializing the data. note that indexdata must have an object before being used.
jfIndexPack::jfIndexPack() {
  anything = false;
  indexdata = NULL;
  size_t loopc;
  catdex = 0;
  for (loopc=0;loopc<33;loopc++) startpoints[loopc] = -1;
}
//------------------------------------------------------------------------
// returns true if there are results for the specifies result index
bool jfIndexPack::HasResults(size_t inindex) const {
  assert(inindex<33);
  assert(indexdata!=NULL);
  return ((indexdata->CatResCount(inindex))!=0);
}
//****************************************************************************

