/*****************************************************************************
Name:        organizer.h
Author :     John Q Metro
Purpose :    Intended to replace jfDisplayResultOrganizer
             A class that holds html result data for display, and provides a set
             of indexes for the result category to get the html result
Created:     April 16, 2011
Updated:     September 21, 2014
******************************************************************************/
#ifndef ORGANIZER_H_INCLUDED
#define ORGANIZER_H_INCLUDED
#endif // ORGANIZER_H_INCLUDED
//------------------------------------------
#ifndef VECTORS_H_INCLUDED
  #include "../../core/structs/vectors.h"
#endif // VECTORS_H_INCLUDED
/******************************************************************************/
/*
This class organizes results for display in HTML list boxes. Generally, things
go as follows:
-The downloading and parsing thread, after doing it's downloading and parsing
for each page, creates a jfResultUnitVector containing data on the result items
for this page.
-This vector is sent off via an event to an object of jfDisplayResultOrganizer
-The event handler adds this vector to the internal of vector of results, and
in doing so, notes which of the result categories have items, how many, and
adds the indexes of these items onto the end of the appropriate index arrays
for each display result category.
-Events are sent off to each HTML List box to tell them to update the number of
items they are showing.


*/
class jfCategoryResultIndexes : public wxObject {
  public:
    // the constructor
    jfCategoryResultIndexes(wxString inname, wxString incathb);
    // returns the html for result category res_a, index res_idx
    wxString GetHTML(size_t res_a, size_t res_idx) const;
    size_t GetProxy(size_t res_a, size_t res_idx) const;
		wxString GetURL(size_t res_a, size_t res_idx) const;
    // returns the category header html
    wxString GetCatHTML(size_t res_a) const;
    // more category info
    size_t CatResCount(size_t res_a) const;
    size_t CatCurrCount(size_t res_a) const;
    wxString GetCatName() const;
    // methods to append results
    bool AppendResult(jfResultUnitVector* newresult, size_t in_stotal);
    bool NoResultsPage(size_t in_stotal);
    // special method
    bool AlterBgColour(size_t res_a, size_t res_idx, wxString newcol);
    // the destructor
    ~jfCategoryResultIndexes();
  protected:
    // some helper methods
    bool IndexOk(size_t res_a, size_t res_idx) const;
    size_t CalcProxy(size_t res_a, size_t res_idx) const;
    // extra methods for inserting data
    size_t AppendNewRes(jfResultUnitVector* in_tempfile);
    // internal helper methods
    void ClearMaindata(jfResultUnitVector* xmaindata);
    void RebuildCatHeaders();
    wxString MakePercString(size_t ni_idex) const;
    void MakeEmptyCatHeaders();
    // indexing
    void IndexingLoopAppend(size_t istartdex);
    void ClearIndexes();
    void ZeroCNumItems();
    // general category data
    wxString catname,cathtml;
    wxString* catheaders[33];
    // the main data
    jfResultUnitVector* maindata;
    size_t searched_total;
    // number of items so far per result category
    size_t num_items[33];
    size_t curr_num_items[33];
    // the index arrays, pointing to results in maindata for each result catgory
    jfIndexVector* indexes[33];
    // a guard for when we change contents
    mutable wxMutex cguard;
};
//=======================================================================
class jfFFNSCategory;
//-------------------------------
class jfCatFFNResultIndexes : public jfCategoryResultIndexes {
  public:
    // the constructor
    jfCatFFNResultIndexes(const jfFFNSCategory* incat, wxString incathb);
    // returns the html for result category res_a, index res_idx
    bool InsertResult(jfResultUnitVector* newresult, size_t in_stotal);
    bool UpdateFic(jfResultUnit* inval);
    // other methods
    void SetInsertMode(bool mode);
    bool SameCategory(const jfFFNSCategory* incat) const;
    // the destructor
    ~jfCatFFNResultIndexes();
  protected:
    // extra methods for inserting data
    size_t InsertNewRes(jfResultUnitVector* in_tempfile);
    void IndexingLoopInsert(size_t start_ins, size_t ins_len);
    // insert mode flag
    bool insert_mode;
    size_t insert_point;
    // update index
    size_t uindex;
    // category link
    const jfFFNSCategory* catlink;
};
//========================================================================
/* when displaying results, we have to keep track of where in the results list
of each result category the source category starts. We wrap this with the indexes */
class jfIndexPack : public wxObject {
  public:
    // data
    jfCategoryResultIndexes* indexdata;
    wxInt32 startpoints[33];
    bool anything;
    size_t catdex;
    // methods
    jfIndexPack();
    bool HasResults(size_t inindex) const;
  protected:
};

/******************************************************************************/
