/******************************************************************************
Name    :   multipanel.cpp
Author  :   John Q Metro
Purpose :   The new progress panel, mutli-source-categories
Created :   December 20, 2013
Updated :   January 26, 2014
******************************************************************************/
#ifndef MULTIPANEL_H_INCLUDED
  #include "multipanel.h"
#endif // MULTIPANEL_H_INCLUDED
//-------------------------------------------
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
/*****************************************************************************/
jfProgressPanelMultiBase::jfProgressPanelMultiBase(wxWindow* parent, jf_MPP_Config in_layout, wxString motif_name):
    jfProgressPanelBase(parent,motif_name) {
  mpp_layout = in_layout;
  if (mpp_layout==jfmpp_Sideways) topsizer = new wxStaticBoxSizer(wxHORIZONTAL,this,wxT("Status Information"));
  else topsizer = new wxStaticBoxSizer(wxVERTICAL,this,wxT("Status Information"));
  SetupWidgets();
  cat_display = new jfLabeledGauge(this,20);
  SetupSizers();
}
//++++++++++++++++++++++++++++++++++++++
jf_MPP_Config jfProgressPanelMultiBase::GetLayout() const {
  return mpp_layout;
}
//++++++++++++++++++++++++++++++++++++++
// handlers for thread events
void jfProgressPanelMultiBase::HandleDone(jfThreadEvent& tevent) {
    // variables
  jfDown::ResultAction res_action;
  wxString fres;
  wxString errstr;
  // starting
  res_action = tevent.GetResultAction();
  if ((res_action==jfDown::FAIL)||(res_action==jfDown::SKIP)) {
    fres = tevent.GetString1();
    errstr = tevent.GetString2();
  }
  // two main options here, we go on, or we stop
  if (res_action==jfDown::PROCEED) {
    SetStatus(status);
    page_display->SetDone(wxT("Done with"));
  }
  else if (res_action==jfDown::SKIP) {
    SetStatus(status);
    page_display->SetDone(wxT("Skipping"));
  }
  else {
    // specials
    if (res_action==jfDown::FAIL) {
      page_display->ChangeLabel(fres+ wxT(" :") + errstr);
    }
  }
}
//++++++++++++++++++++++++++++++++++++++
// handlers for category thread events
//-------------------------------------
void jfProgressPanelMultiBase::HandleNewCategory(jfThreadEvent& tevent) {
  catindex++;
  cat_display->SetNew(tevent.GetInt1(),tevent.GetInt2(),tevent.GetString1());
  page_display->SetToZero();
  pageindex = 0;
}
//-------------------------------------
/* For multi-category downloads, I've decided that only at the category level
will we decide if the search comes to an end or not */
void jfProgressPanelMultiBase::HandleCategoryEnd(jfThreadEvent& tevent) {
  // variables
  jfDown::ResultAction res_action;
  wxString fres;
  wxString errstr;
  // starting
  res_action = tevent.GetResultAction();
  if ((res_action==jfDown::FAIL)||(res_action==jfDown::SKIP)) {
    fres = tevent.GetString1();
    errstr = tevent.GetString2();
  }
  // two main options here, we go on, or we stop
  if (res_action==jfDown::PROCEED) {
    cat_display->SetDone(wxT("Finished"));
  }
  else if (res_action==jfDown::SKIP) {
    cat_display->SetDone(wxT("Skipping"));
  }
  else {
    // specials
    if (res_action==jfDown::HALT) {
      StopAction(jfp_HALTED);
      PostFaliure(true);
    }
    else {
      StopAction(jfp_FAILED);
      PostFaliure(false);
    }
  }
}
//-------------------------------------
void jfProgressPanelMultiBase::HandleCategoryCount(jfThreadEvent& tevent) {
  cat_display->SetCount(tevent.GetInt1());
}
//-------------------------------------
void jfProgressPanelMultiBase::HandleCategoryLabels(jfThreadEvent& tevent) {
  cat_display->ChangeTypestring(tevent.GetString1(),tevent.GetString2());
}
//++++++++++++++++++++++++++++++++++++++
// internal helper methods
void jfProgressPanelMultiBase::SetupSizers() {
  if (mpp_layout!=jfmpp_Sideways) {
    // setting up the buttons and status display
    button_sizer = new wxBoxSizer(wxHORIZONTAL);
    button_sizer->Add(stat_label,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
    button_sizer->AddSpacer(6);
    button_sizer->Add(startstop_btn,0,wxALL,3);
    button_sizer->Add(pauseresume_btn,0,wxALL,3);
    // the inserted layout
    if (mpp_layout==jfmpp_Inserted) {
      // the top row
      q_sizer = new wxBoxSizer(wxHORIZONTAL);
      q_sizer->Add(cat_display,1,wxEXPAND|wxALIGN_CENTER_VERTICAL|wxALL,3);
      q_sizer->Add(button_sizer,0,wxALL|wxALIGN_CENTER_VERTICAL,3);
      // finishing
      topsizer->Add(q_sizer,0,wxEXPAND|wxTOP|wxLEFT|wxRIGHT,3);
      topsizer->Add(page_display,0,wxEXPAND|wxALL,3);
    }
    // stacked layout
    else {
      topsizer->Add(cat_display,0,wxEXPAND|wxTOP|wxLEFT|wxRIGHT,3);
      topsizer->Add(page_display,0,wxEXPAND|wxALL,3);
      topsizer->Add(button_sizer,0,wxALIGN_RIGHT|wxALL,3);
    }
  }
  else {
    // setting up the buttons and status display
    button_sizer = new wxBoxSizer(wxVERTICAL);
    button_sizer->Add(stat_label,0,wxALL,3);
    button_sizer->AddSpacer(3);
    button_sizer->Add(startstop_btn,0,wxALL|wxALIGN_CENTER,3);
    button_sizer->Add(pauseresume_btn,0,wxALIGN_CENTER|wxBOTTOM|wxRIGHT|wxLEFT,3);
    // progress bars
    q_sizer = new wxBoxSizer(wxVERTICAL);
    q_sizer->Add(cat_display,0,wxEXPAND|wxALL|wxALIGN_CENTRE_VERTICAL,3);
    q_sizer->Add(page_display,0,wxEXPAND|wxBOTTOM|wxLEFT|wxRIGHT|wxALIGN_CENTRE_VERTICAL,3);
    // top level arranging
    topsizer->Add(button_sizer,0,wxALIGN_CENTRE_VERTICAL|wxALL,1);
    topsizer->Add(q_sizer,1,wxEXPAND|wxALL|wxALIGN_CENTRE_VERTICAL,1);
  }
  SetSizer(topsizer);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// resizer methods
//---------------------------------
bool jfProgressPanelMultiBase::ReduceSizeInt() {
  // varibales
  wxSize ssize;
  const size_t bsize = 10;
  const size_t fpsize = 8;
  const size_t bheight = 24;
  // changing the button height
  ChangeButtonHights(bheight);
  page_display->ChangeSizes(bsize,fpsize);
  cat_display->ChangeSizes(bsize,fpsize);
  return true;
}
//---------------------------------
bool jfProgressPanelMultiBase::UnReduceSizeInt() {
  ChangeButtonHights(dbheight);
  page_display->RevertSizes();
  cat_display->RevertSizes();
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfProgressPanelMultiBase,jfProgressPanelBase)
  EVT_JF_THREAD (-1,jf_DOWNL_PEND_EVENT,jfProgressPanelMultiBase::HandleDone    )
  EVT_JF_THREAD (-1,jf_DOWNL_CATSTART_EVENT ,jfProgressPanelMultiBase::HandleNewCategory    )
  EVT_JF_THREAD (-1,jf_DOWNL_CATEND_EVENT   ,jfProgressPanelMultiBase::HandleCategoryEnd    )
  EVT_JF_THREAD (-1,jf_DOWNL_CATCOUNT_EVENT ,jfProgressPanelMultiBase::HandleCategoryCount  )
  EVT_JF_THREAD (-1,jf_DOWNL_CATLABELS_EVENT,jfProgressPanelMultiBase::HandleCategoryLabels )
END_EVENT_TABLE()

/*****************************************************************************/
jfProgressPanelMulti::jfProgressPanelMulti(wxWindow* parent, jf_MPP_Config in_layout,
          wxString motif_name):jfProgressPanelMultiBase(parent,in_layout,motif_name) {
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the reset method
bool jfProgressPanelMulti::Reset() {
  BaseReset();
  cat_display->SetToZero();
  catindex = 0;
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// resizer methods
//-----------------------------------------------------
bool jfProgressPanelMulti::ReduceSize() {
  return ReduceSizeInt();
}
//-----------------------------------------------------
bool jfProgressPanelMulti::UnReduceSize() {
  return UnReduceSizeInt();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal helper methods
//-----------------------------------------------------
void jfProgressPanelMulti::PostFinished() {
  wxCommandEvent outevent(jf_PP_SRCHDONE_EVENT,GetId());
  wxPostEvent(GetParent(),outevent);
}
//-----------------------------------------------------
void jfProgressPanelMulti::PostFaliure(bool halted) {
  wxCommandEvent outevent(jf_PP_SRCHFAIL_EVENT,GetId());
  outevent.SetInt(halted);
  wxPostEvent(GetParent(),outevent);
}
//-----------------------------------------------------
void jfProgressPanelMulti::PostStopping() {}
//-----------------------------------------------------
void jfProgressPanelMulti::PostStarted() {}

/*****************************************************************************/
