/******************************************************************************
Name    :   ppanel_parts.cpp
Author  :   John Q Metro
Purpose :   Progress Panel parts and subparts
Created :   November 2, 2009
Updated :   December 15, 2013
******************************************************************************/
#ifndef PPANEL_PARTS_H_INCLUDED
  #include "ppanel_parts.h"
#endif // PPANEL_PARTS_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
/*****************************************************************************/
/* wxGauge with label, counts, and label manipulation */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the constructor
jfLabeledGauge::jfLabeledGauge(wxWindow* parent, size_t bmheight, long style, const wxString& name):
      wxPanel(parent,wxID_ANY,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // we first check the type
  ltype = wxT("Page");
  // size hints for the gauge
  if (bmheight>=10)  minsize = wxSize(100,bmheight);
  else  minsize = wxSize(100,10);
  // we create the GUI elements
  main_label = new wxStaticText(this,-1,wxEmptyString);
  prog_bar = new wxGauge(this,-1,100,wxDefaultPosition,wxDefaultSize,wxGA_HORIZONTAL | wxGA_SMOOTH);
  prog_bar->SetMinSize(minsize);
  // sizer
  main_sizer = new wxBoxSizer(wxVERTICAL);
  main_sizer->Add(main_label,0,wxEXPAND|wxALIGN_LEFT|wxTOP|wxLEFT|wxRIGHT,3);
  main_sizer->Add(prog_bar,0,wxEXPAND|wxALIGN_LEFT|wxBOTTOM|wxLEFT|wxRIGHT,3);
  SetSizer(main_sizer);
  // recording sizes
  wxSize tsize = prog_bar->GetMinSize();
  bsize = tsize.GetHeight();
  wxFont qfont = main_label->GetFont();
  fpsize = qfont.GetPointSize();
  // finishing off
  SetToZero();
  Layout();
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting data methods
//------------------------------------------------------
// we increment the current part, and set the label
bool jfLabeledGauge::SetNew(int index_in, int tcx_in, wxString name) {
  const wxString fname = wxT("jfLabeledGauge::SetNew");
  curpart++;
  // assigning the values
  idx_curr = index_in;
  idx_total = tcx_in;
  pname = name;
  laction = saction;
  // setting the label
  main_label->SetLabel(BuildMsg());
  done = false;
  return true;
}
//------------------------------------------------------
// we change the label
void jfLabeledGauge::ChangeAction(wxString action) {
  laction = action;
  main_label->SetLabel(BuildMsg());
}
//------------------------------------------------------
// increment the done count, set the label, and update the progress par
bool jfLabeledGauge::SetDone(wxString action) {
  // assigning the values
  partsdone = curpart;
  laction = action;
  // setting the label
  main_label->SetLabel(BuildMsg());
  // updating the progress bar
  if (totalcount!=0) prog_bar->SetValue(partsdone);
  // done
  done = true;
  return true;
}
//-----------------------------------------------------------------
// skiping a few items to the last named one, which is done
bool jfLabeledGauge::SkipToDone(size_t skip_amount, wxString action, wxString name) {
  // checks for odd cases
  if (skip_amount==0) return false;
  assert((curpart+skip_amount)<=totalcount);
  // setting up tags and amounts
  pname = name;
  laction = action;
  curpart = curpart + skip_amount;
  partsdone = curpart;
  idx_total += skip_amount;
  // updating the gui stuff
  main_label->SetLabel(BuildMsg());
  if (totalcount!=0) prog_bar->SetValue(partsdone);
  // done
  done = true;
  return true;
}
//------------------------------------------------------
// changes the total number of items, updates gui elements appropriatly
bool jfLabeledGauge::SetCount(size_t newcount) {
  // a restriction... the count cannot be less than what we have already done, unless zero
  if ((newcount!=0) && (newcount<partsdone)) return false;
  // other than that, 2 main options
  // if newcount is 0, the count is undefined, we empty the progress bar
  if (newcount==0) prog_bar->SetValue(0);
  // otherwise, we change things
  else {
    prog_bar->SetRange(newcount);
    prog_bar->SetValue(partsdone);
  }
  // update the labels
  totalcount = newcount;
  main_label->SetLabel(BuildMsg());
  // done
  return true;
}
//------------------------------------------------------
// basically resets the labelled progress bar. however, one can optionally define a count
void jfLabeledGauge::SetToZero(size_t newcount) {
  // resetting internal values
  laction = wxEmptyString;
  pname = wxEmptyString;
  done = false;
  curpart = 0;
  idx_curr = -1;
  partsdone = 0;
  totalcount = newcount;
  if (totalcount==0) idx_total = -1;
  else idx_total = totalcount;
  // setting the gui elements
  main_label->SetLabel(BuildMsg());
  prog_bar->SetValue(0);
  if (newcount!=0) prog_bar->SetRange(newcount);
}
//-------------------------------------------------------------------------
void jfLabeledGauge::ChangeLabel(wxString inmsg) {
  main_label->SetLabel(inmsg);
}
//----------------------------------------------------------------------
void jfLabeledGauge::ChangeTypestring(wxString intype, wxString saction_in) {
  ltype = intype;
  saction = saction_in;
  main_label->SetLabel(BuildMsg());
}
//---------------------------------------------------------------------
bool jfLabeledGauge::ChangeBarHeight(size_t bheight) {
  if (bheight<10) return false;
  // size hints for the gauge
  minsize = wxSize(100,bheight);
  wxSize tsize = prog_bar->GetMinSize();
  prog_bar->SetMinSize(minsize);
  Layout();
  return true;
}
//------------------------------------------------------------
bool jfLabeledGauge::ChangeFontSize(size_t psize) {
  if (psize<7) return false;
  wxFont qfont = main_label->GetFont();
  qfont.SetPointSize(psize);
  main_label->SetFont(qfont);
  return true;
}
//-------------------------------------------------------------
bool jfLabeledGauge::ChangeSizes(size_t bheight, size_t psize) {
  if ((psize<7) && (psize!=0)) return false;
  bool qres = ChangeBarHeight(bheight);
  if ((qres) && (psize!=0)) qres = ChangeFontSize(psize);
  return qres;
}
//-------------------------------------------------------------
bool jfLabeledGauge::RevertSizes() {
  if (bsize==0) return false;
  return ChangeSizes(bsize,fpsize);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting info
//------------------------------------------------------
size_t jfLabeledGauge::GetIndex() const {
  return curpart;
}
//------------------------------------------------------
size_t jfLabeledGauge::GetCount() const {
  return totalcount;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// helper methods
wxString jfLabeledGauge::BuildMsg() const {
  wxString result;
  // option one, nothing done!
  if (curpart==0) {
    result = wxT("Nothing started yet.");
  }
  else {
    // start building things
    result = laction + wxT(" ") + ltype;
    if (idx_curr>=0) {
      result += wxT(" ") + IInt2Str(idx_curr) + wxT(" of ");
      if (idx_total<0) result += wxT("an unknown number");
      else result += IInt2Str(idx_total);
    }
    if (!pname.IsEmpty()) {
      result += wxT(" : ") + pname;
    }
    result += wxT(".");
  }
  // done
  return result;
}
/*****************************************************************************/
