/******************************************************************************
Name    :   progresspanel.h
Author  :   John Q Metro
Purpose :   The new progress panel, for displaying progress and button that control it.
Created :   November 12, 2013
Updated :   January 26, 2014
******************************************************************************/
#ifndef PROGRESSPANEL_H_INCLUDED
#define PROGRESSPANEL_H_INCLUDED
#endif // PROGRESSPANEL_H_INCLUDED
//------------------------------------
#ifndef CORE_THREADS_H_INCLUDED
  #include "../../connect/core_threads.h"
#endif // CORE_THREADS_H_INCLUDED
#ifndef DOWN_DATA_H_INCLUDED
  #include "../../connect/down_data.h"
#endif // DOWN_DATA_H_INCLUDED
#ifndef PPANEL_PARTS_H_INCLUDED
  #include "ppanel_parts.h"
#endif // PPANEL_PARTS_H_INCLUDED
/*****************************************************************************/
// status enumeration
enum jf_PPS {jfp_NOTSTARTED, jfp_STARTING, jfp_WORKING, jfp_PAUSING, jfp_PAUSED,
          jfp_STOPPING, jfp_HALTED, jfp_DONE, jfp_FAILED};

bool IsStatusActive(const jf_PPS& instat, bool istop = false);
//=======================================================================================
class jfProgressPanelBase : public wxPanel {
  public:
    // constructor
    jfProgressPanelBase(wxWindow* parent, wxString motif_name);
    // getting data
    jfNewPauser* GetPauser();
    // some special external methods
    bool CancelStart();
    bool OrderStop();
    bool InProgress() const;
    virtual bool Reset() = 0;
    // handlers for events generated from this panel
    void StartStopBtnHandler(wxCommandEvent& event);
    void PauseResumeBtnHandler(wxCommandEvent& event);
    // handlers for events that come from the getter
    void HandleRetry(wxCommandEvent& event);
    void HandleRequestPause(wxCommandEvent& event);
    // handlers for events that come from the pauser
    void HandlePause(wxCommandEvent& event);
    void HandleResume(wxCommandEvent& event);
    // handlers for thread events
    void HandleStart(jfThreadEvent& tevent);
    void HandleEnd(jfThreadEvent& tevent);
    void HandleTotalCount(jfThreadEvent& tevent);
    void HandleLabels(jfThreadEvent& tevent);
    void HandleNewItem(jfThreadEvent& tevent);
    void HandleAction(jfThreadEvent& tevent);
    virtual void HandleDone(jfThreadEvent& tevent) = 0;
    void HandleHalt(jfThreadEvent& tevent);
    void HandleReset(jfThreadEvent& tevent);
    // resizer methods
    virtual bool ReduceSize() = 0;
    virtual bool UnReduceSize() = 0;
  protected:
    // internal helper methods
    bool SetupWidgets();
    bool ChangeButtonHights(size_t nheight, wxButton* extrabtn = NULL);
    virtual void SetupSizers() = 0;
    bool BaseReset();
    bool StopAction(jf_PPS instatus);
    void SetStatus(jf_PPS instatus);
    virtual void PostFinished() = 0;
    virtual void PostFaliure(bool halted) = 0;
    virtual void PostStopping() = 0;
    virtual void PostStarted() = 0;
    // GUI Elements
    jfLabeledGauge* page_display;
    wxButton* startstop_btn;
    wxButton* pauseresume_btn;
    wxStaticText* stat_label;
    jfNewPauser* thepauser;
    // status info
    bool working;
    bool rpause;
    jf_PPS status;
    size_t pageindex;
    size_t dbheight;
    // declare event table
    DECLARE_EVENT_TABLE()
};
//============================================================================
// one source-catgory search results panel
class jfProgressPanelSingle : public jfProgressPanelBase {
  public:
    // the constructor
    jfProgressPanelSingle(wxWindow* parent, wxString motif_name);
    // reset method
    virtual bool Reset();
    // implemented handler
    virtual void HandleDone(jfThreadEvent& tevent);
    // resizer methods
    virtual bool ReduceSize();
    virtual bool UnReduceSize();
  protected:
    // internal implemented helper methods
    virtual void SetupSizers();
    virtual void PostFinished();
    virtual void PostFaliure(bool halted);
    virtual void PostStopping();
    virtual void PostStarted();
    // sizer
    wxStaticBoxSizer* topsizer;
    // declare event table
    DECLARE_EVENT_TABLE()
};
/*****************************************************************************/
